/**
 * @file manager.h
 * 
 * This file contains the declarations for functions for email manager module
 *
 *  Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */

#ifndef MANAGER_H_
#define MANAGER_H_

#include <libosso.h>
//#include <log-functions.h>
#include "dbus_utils_def.h"

#define   EMAIL_APPLICATION    "osso_email"
#define   EMAIL_APPLICATION_VERSION "0.1"
#define   TASKNAV_APP "tn_mail"
#define   UI_START_TIMEOUT  60000
#define   STATE_SAVE_BUFF_SIZE 1024

#define FLIGHT_MODE  "flight"
#define MODE_LEN 6

#define   ENPROCESS_NAME      "/usr/bin/enprocess"

enum operation_type {
	OSSO_EMAIL_HOME = 0,
	OSSO_EMAIL_SEND_RECV = 1,
	OSSO_EMAIL_FILE_MGR,
	OSSO_EMAIL_URL,
	OSSO_EMAIL_MAIL_TO,
	OSSO_TASKNAV_OPEN_MSG,
	OSSO_TASKNAV_OPEN_NEW_MSG,
	OSSO_TASKNAV_OPEN_INBOX,
	OSSO_TASKNAV_DELETE_MSG
};

#define FILE_MGR_ARG       "--files="
#define URL_ARG            "--url="
#define MAIL_TO_ARG        "--to="
#define OPEN_MSG_ARG       "--openmsg="
#define UI_WINDOW_REG  "--ui_window_reg="

/* method name to tell the email app to  send n' receive 
 * unsent/unread mails. Has no arguments 
 */
#define SEND_AND_RECEIVE_METHOD "send_and_receive"

/* Method name used to add an unread mail to TN */
#define OSSO_TASKNAV_ADD_MAIL                "add_unread_mail"

/* Method name used to remove an unread mail from TN */
#define OSSO_TASKNAV_REMOVE_MAIL             "remove_unread_mail"

/* Method name used to update last updated status */
#define OSSO_TASKNAV_UPDATE_STATUS           "update_status"

/* Method name used to dim/activ the send_recv button in Tasknav */
#define OSSO_TASKNAV_SEND_RECV_FOCUS "send_recv_button_focus"

/* Method name used to delete the mails related to account */
#define REMOVE_ACCOUNT_MAILS "remove_account_mails"

/* Method name used to rename the mails related to account */
#define RENAME_ACCOUNT_MAILS "rename_account_mails"

/* From TN to email application */

/* method name to tell mail app to creat a new
 * email message. Has no arguments */
#define NEW_MAIL_METHOD "create_new_mail"

/* method name to open inbox.  Has no arguments  */
#define OPEN_INBOX_METHOD "open_inbox"

/* method name to activate a mail.   */
#define ACTIVATE_MAIL_METHOD "open_email"

/* method name for getting the folder list */
#define GET_FOLDERS_LIST_METHOD   "get_folders_list"

/* method name for getting the email standard folder list */
#define GET_STANDARD_FOLDERS_METHOD   "get_email_standard_folders"

/* method name to send files from other application   */
#define SEND_FILES_METHOD    "send_files"

/* method name to send url  */
#define SEND_URL_METHOD      "send_url"

/* method name to mailto  */
#define SEND_MAILTO_METHOD   "send_mailto"

/* method name to statesave */
#define STATE_SAVE_METHOD    "state_save"

/* method name to search emails.  Has no arguments  */
#define SEARCH_EMAILS_METHOD "search_emails"

/* method name to open email hit request from Global search application */
#define OPEN_HIT_METHOD "open_hit"

/* method name to delete email hit from Global search application */
#define DELETE_HIT_METHOD "delete_hit"

/* method name to Top the UI Application */
#define TOP_APPLICATION "top_application"

#define BACKUP_START_EVENT "backup_start_handle"

enum osso_email_state {
	OSSO_STATE_UN_INIT = 1,	/* No Initialisation */
	OSSO_STATE_INIT_LIMITED,	/* Only engine is up and running */
	OSSO_STATE_UI_INIT_PROGRESS,	/* UI init happening */
	OSSO_STATE_INIT_COMPLETE,	/* Both engine and UI is UP */
};

enum osso_statesave_operation {
	OSSO_EMAIL_STATESAVE_WRITE = 0,
	OSSO_EMAIL_STATESAVE_READ
};

enum {
	SIGNAL_EXIT,
	SIGNAL_SHUTDOWN_IND,
	SIGNAL_SAVE_UNSAVED_DATA_IND,
	SIGNAL_MEMORY_LOW_IND,
	SIGNAL_SYSTEM_INACTIVITY_IND
};

#define     MAILTO            "mailto:"
#define     QUESTION          '?'
#define     EQUALTO           '='
#define     AMPERSAND         '&'
#define     FILE_COLEN        "file:"
#define     TO_STR            "to"
#define     CC_STR            "cc"
#define     BCC_STR           "bcc"
#define     SUBJECT_STR       "subject"
#define     BODY_STR          "body"
#define     ATTACH_STR        "attach"
#define     AMP               "amp;"


/*
typedef struct _BrowserUrl  BrowserUrl;
struct _BrowserUrl
{
  gchar  *protocol;
  gchar  *user;
  gchar  *authmech;
  gchar  *passwd;
  gchar  *host;
  gint    port;
  gchar  *path;
  GData *params;
  gchar  *query;
  gchar  *fragment;
};

*/
#endif				/* MGR_H_ */
