/**
  @file  accountMgmt.c 
 
  This file contains the implementaions of Account  management
   Copyright (c) 2005 Nokia. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
   USA
*/


/* Inclusion of Header file which contains all standerd paths.*/
#include "ui_common.h"

/* Include User Defined Header files */
#include"accountMgmt.h"
#include"accountMgmt_callback.h"
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "ui_account_store.h"
#include "cl.h"

extern MainViewInfo *email_main_view;	/* declared in ui_interface.c */



/*To store the account setup window pointers to move among the windows */
GSList *ui_accMgmt_listptr = NULL;


/* Need to be changed */
GtkListStore *folderListStore = NULL;


/**
This is the global structure from account management module.
*/
AccountMgtMngr *gActMgtMngr = NULL;

static void ui_accMgmt_create_advance_security_dialog(const AdvOptionViewInfo * advOptionViewInfo);
static void on_configure_servers_clicked(GtkWidget * configure_button, GtkWidget * output_server);
static GtkWidget *create_smtp_server_details_dialog(GtkWidget * dialog_smtp_server);
#if CL_USE
static void ui_get_iap_smtp_server_details(GtkWidget * iap_smtp_server_details_dialog);
#endif
static void enable_configure_servers(GtkWidget * iap_checkbutton, GtkWidget * iap_caption);


/**
This function is a hepler function. It is used to find whether a given 
Account Name is already exist in the Account List or not.

@param account_name , which holds the pointer to the new account Name
@return It returns TRUE if it already exist in the Account list
        FALSE otherwise.
*/

gboolean ui_accmgmt_search_accountname(const gchar * account_name)
{
	GSList *configuredaccount = NULL;

	/*Input param checking */
	CHECK_PARAM((account_name == NULL), "Invalid Input Parameter", return FALSE);

	configuredaccount = (GSList *) email_main_view->account_manager->configured_accounts;
	while (configuredaccount != NULL) {
		if (ui_compare_utf8_strings(configuredaccount->data, account_name) == 0) {
			return TRUE;
		}
		configuredaccount = configuredaccount->next;
	}
	return FALSE;
}

/**
This function is a helper function. It is used to get the default Account Name.

@param void.
@return It returns the default Account Name. Calling function needs to free the string.
*/

gchar *ui_get_new_accountname_display_string(void)
{
	gchar *display_str = NULL;
	GString *new_accountname_str = NULL;
	gint accountname_count = 1;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	new_accountname_str = g_string_new(NULL);
	g_string_printf(new_accountname_str, _("mcen_ia_emailsetup_defaultname"));

	/*Reapeat unless got an Account Name which is not exist in the device. */
	while (ui_accmgmt_search_accountname(new_accountname_str->str) == TRUE) {
		g_string_free(new_accountname_str, TRUE);
		new_accountname_str = g_string_new(NULL);
		g_string_printf(new_accountname_str, "%s%d",
				_("mcen_ia_emailsetup_defaultname"), accountname_count);
		accountname_count++;
	}

	display_str = g_string_free(new_accountname_str, FALSE);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return display_str;
}

/**
This function will be called by the Main UImodule to create new email account.

@param acc_creat_edit boolean variable represents Edit or Create an account.
@return GtkWidget pointer to the window which it will create. 
*/

GtkWidget *ui_accmgmt_createAccountsetupDialog(gboolean acc_creat_edit)
{
	GtkWidget *accsetup_dialog = NULL;
	GtkWidget *accsetup_vbox = NULL;
	GtkWidget *accsetup_label = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *finish_button = NULL;
	GtkWidget *back_button = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *cancel_button = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	ui_accmgmt_initialize_global_variable(TRUE);
	gActMgtMngr->edit_operation = acc_creat_edit;
	accsetup_dialog = gtk_dialog_new();

	gtk_window_set_resizable(GTK_WINDOW(accsetup_dialog), TRUE);
	gtk_window_set_default_size(GTK_WINDOW(accsetup_dialog), DIALOG_WID, WIDGET_HEIGHT);
	gtk_window_set_title(GTK_WINDOW(accsetup_dialog), _("mcen_ti_emailsetup_welcome"));

	gtk_dialog_set_has_separator(GTK_DIALOG(accsetup_dialog), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(accsetup_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(accsetup_dialog), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(accsetup_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(accsetup_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	accsetup_vbox = GTK_DIALOG(accsetup_dialog)->vbox;
	gtk_widget_show(accsetup_vbox);

	accsetup_label = gtk_label_new(_("mcen_ia_emailsetup_intro"));
	gtk_label_set_line_wrap((GtkLabel *) accsetup_label, TRUE);
	gtk_misc_set_alignment(GTK_MISC(accsetup_label), 0, 0);
	gtk_widget_show(accsetup_label);
	gtk_box_pack_start(GTK_BOX(accsetup_vbox), accsetup_label, FALSE, FALSE, WIDGET_PACKING);

	/* Appending Dialog pointer to the global list. */
	ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, accsetup_dialog);

	dialog_action_area1 = GTK_DIALOG(accsetup_dialog)->action_area;
	gtk_widget_show(dialog_action_area1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);
	finish_button = gtk_button_new_with_label(_("mcen_bd_emailsetup_welcome_finish"));
	gtk_widget_show(GTK_WIDGET(finish_button));
	gtk_dialog_add_action_widget(GTK_DIALOG(accsetup_dialog), finish_button, 0);
	gtk_widget_set_sensitive(finish_button, FALSE);

	back_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_welcome_back"));
	gtk_widget_show(back_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(accsetup_dialog), back_button, 0);
	gtk_widget_set_sensitive(back_button, FALSE);
	next_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_welcome_next"));
	gtk_widget_show(next_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(accsetup_dialog), next_button, 0);
	g_signal_connect(G_OBJECT(next_button), "clicked",
			 G_CALLBACK(ui_accmgmt_callback_onnext_cb), accsetup_dialog);
	cancel_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_welcome_cancel"));
	gtk_widget_show(cancel_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(accsetup_dialog), cancel_button, 0);
	g_signal_connect(G_OBJECT(finish_button), "insensitive_press",
			 G_CALLBACK(ui_accmgmt_insensitive_press_cb),
			 _("mcen_ib_enter_fill_in_all_fields"));
	g_signal_connect(G_OBJECT(back_button), "insensitive_press",
			 G_CALLBACK(ui_accmgmt_back_insensitive_press_cb),
			 _("mcen_ib_unable_to_goback"));
	g_signal_connect(G_OBJECT(cancel_button), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbackcancel1_cb), accsetup_dialog);
	g_signal_connect(G_OBJECT(accsetup_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);

	/* Checking whether help content exist or not. */
	if (ossohelp_exists(HELP_ID_WELCOMETOSETUP) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) accsetup_dialog);
		g_signal_connect(G_OBJECT(accsetup_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_WELCOMETOSETUP);
	}

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(accsetup_dialog, accsetup_dialog, "emailAccountSetUp");


	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
	return accsetup_dialog;
}


/**
This function will be called by Account Management module to create E-mail  account set up 1/4 window.

@param  acc_creat_edit boolean variable represents Edit or Create an account.
@return GtkWidget pointer contains window pointer which this function will create.  
*/

GtkWidget *ui_accmgmt_createemailaccountsetup1(gboolean acc_creat_edit)
{
	GtkWidget *dialog1 = NULL;
	GtkWidget *vbox_setup1 = NULL;
	GtkWidget *scrolledwindow1 = NULL;
	GtkWidget *viewport1 = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *entry_acctname = NULL;
	GtkWidget *combo_accttype = NULL;
	GtkComboBox *combobox_accttype = NULL;
	GtkWidget *label_note = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *button_finish = NULL;
	GtkWidget *back_button = NULL;
	GtkWidget *button_next = NULL;
	GtkWidget *cancel_button = NULL;
	gchar *display_acctname = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	accountFlag *account_flag = NULL;

	account_flag = g_new0(accountFlag, 1);
	account_flag->account_preedit = FALSE;
	account_flag->account_commit = FALSE;

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailEditInfo");

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	gActMgtMngr->edit_operation = acc_creat_edit;
	gActMgtMngr->ACCT_NAME_MODY = FALSE;

	/* Editing an existing account. */
	if (acc_creat_edit) {
		if (ui_accMgmt_editinfo == NULL) {
			ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n",
				   __FUNCTION__, __LINE__);
			return NULL;
		}
	}

	/* Create new Account settings structure */
	ui_accMgmt_accountinfo = g_new0(PrefsAccount, 1);

	ui_acctmgmt_initialize(&ui_accMgmt_accountinfo);

	/* While editing an existing account. */
	if (gActMgtMngr->edit_operation == TRUE) {
		ui_accmgmt_save_advancedvalues1(ui_accMgmt_accountinfo);
	}

	/* Dialog creation. */
	dialog1 = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog1), _("mcen_ti_emailsetup_title_1_4"));
	gtk_window_set_default_size(GTK_WINDOW(dialog1), DIALOG_WID, WIDGET_HEIGHT);
	gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(dialog1), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(dialog1), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(dialog1), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog1), FALSE);

	scrolledwindow1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow1);
	gtk_box_pack_start(GTK_BOX((GTK_DIALOG(dialog1)->vbox)), scrolledwindow1,
			   TRUE, TRUE, HILDON_MARGIN_DEFAULT);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow1),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow1), GTK_SHADOW_NONE);
	viewport1 = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport1);
	gtk_container_add(GTK_CONTAINER(scrolledwindow1), viewport1);
	gtk_viewport_set_shadow_type((GtkViewport *) viewport1, GTK_SHADOW_NONE);

	vbox_setup1 = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_container_add(GTK_CONTAINER(viewport1), vbox_setup1);

	/*Account name field */
	entry_acctname = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(entry_acctname), HILDON_GTK_INPUT_MODE_FULL);
	if (acc_creat_edit) {
		gtk_entry_set_text((GtkEntry *) entry_acctname,
				   ui_accMgmt_editinfo->user_settings.account_name);
	} else {
		display_acctname = ui_get_new_accountname_display_string();
		gtk_entry_set_text((GtkEntry *) entry_acctname, display_acctname);
		g_free(display_acctname);
	}
	gtk_widget_grab_focus(entry_acctname);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_li_emailsetup_account_name"),
			       entry_acctname, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, ENTRY_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox_setup1), caption_control, FALSE, FALSE, 0);

	/*Protocol  field (pop3/imap) */
	combo_accttype = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_accttype), _("mail_fi_emailtype_pop3"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_accttype), _("mail_fi_emailtype_imap"));
	combobox_accttype = GTK_COMBO_BOX(combo_accttype);
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_accttype), 0);

	/* Setting values to the widget while edit operation. */
	if (acc_creat_edit) {
		if (ui_accMgmt_editinfo->receive_settings.protocol == RECV_POP3) {
			gtk_combo_box_set_active(GTK_COMBO_BOX(combo_accttype), 0);
		} else {
			gtk_combo_box_set_active(GTK_COMBO_BOX(combo_accttype), 1);
		}
		gtk_widget_set_sensitive(combo_accttype, FALSE);
	}

	caption_control =
	    hildon_caption_new(size_group, _("mcen_li_emailsetup_type"),
			       combo_accttype, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox_setup1), caption_control, FALSE, FALSE, 0);

	label_note = gtk_label_new(_("mcen_ia_emailsetup_account_type"));

	gtk_misc_set_alignment(GTK_MISC(label_note), 0, 0.5);
	gtk_label_set_line_wrap((GtkLabel *) label_note, TRUE);

	gtk_box_pack_start(GTK_BOX(vbox_setup1), label_note, FALSE, FALSE, 0);

	dialog_action_area1 = GTK_DIALOG(dialog1)->action_area;
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);
	button_finish = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pageone_finish"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), button_finish, 0);
	if (!acc_creat_edit) {
		gtk_widget_set_sensitive(button_finish, FALSE);
		gtk_editable_select_region((GtkEditable *) entry_acctname, 0, DEFAULT_HEIGHT);
	}

	g_signal_connect(G_OBJECT(button_finish), "clicked",
			 G_CALLBACK(ui_accmgmt_callbackfinish1_cb), dialog1);

	g_signal_connect(G_OBJECT(button_finish), "insensitive_press",
			 G_CALLBACK(ui_accmgmt_insensitive_press_cb),
			 _("mcen_ib_enter_fill_in_all_fields"));
	back_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pageone_back"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), back_button, 0);
	gtk_widget_set_sensitive(back_button, FALSE);

	g_signal_connect(G_OBJECT(back_button), "insensitive_press",
			 G_CALLBACK(ui_accmgmt_back_insensitive_press_cb),
			 _("mcen_ib_unable_to_goback"));
	button_next = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pageone_next"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), button_next, 0);
	gtk_widget_grab_focus(entry_acctname);
	g_signal_connect(G_OBJECT(button_next), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbacknext1_cb), NULL);
	g_signal_connect(G_OBJECT(button_next), "insensitive_press",
			 G_CALLBACK(ui_accmgmt_next_insensitive_press_cb),
			 _("mcen_ib_enter_accountname"));

	cancel_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pageone_cancel"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), cancel_button, 0);

	g_signal_connect(G_OBJECT(entry_acctname), CHANGED,
			 G_CALLBACK(ui_accmgmt_acctentrychanged_cb), entry_acctname);
	g_signal_connect(G_OBJECT(GTK_ENTRY(entry_acctname)->im_context),
			 "preedit-changed",
			 G_CALLBACK(ui_accmgmt_account_name_preedit_changed_cb), NULL);
	g_signal_connect(G_OBJECT(GTK_ENTRY(entry_acctname)->im_context), "commit",
			 G_CALLBACK(ui_accmgmt_account_name_commit_cb), NULL);

	g_signal_connect(G_OBJECT(entry_acctname), CHANGED,
			 G_CALLBACK(ui_accmgmt_set_finish_button), NULL);
	g_signal_connect_after(G_OBJECT(entry_acctname), "insert-text",
			       G_CALLBACK(check_update_accname_entry), NULL);

	g_signal_connect(G_OBJECT(cancel_button), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbackcancel3_cb), NULL);
	g_object_set_data(G_OBJECT(dialog1), "EmailAccountInfo", ui_accMgmt_accountinfo);
	if (ossohelp_exists(HELP_ID_ACCOUNTDETAILS) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) dialog1);
		g_signal_connect(G_OBJECT(dialog1), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_ACCOUNTDETAILS);
	}
	g_signal_connect(G_OBJECT(dialog1), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);

	g_object_set_data(G_OBJECT(dialog1), "AccountFlag", account_flag);

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog1, dialog1, "Email-set up title");
	EMAIL_UI_HOOKUP_OBJECT(dialog1, entry_acctname, "accountNameEntry");
	EMAIL_UI_HOOKUP_OBJECT(dialog1, combo_accttype, "comboAccountType");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog1, dialog_action_area1, "dialog_action_area1");
	EMAIL_UI_HOOKUP_OBJECT(dialog1, button_next, "buttonNext");
	EMAIL_UI_HOOKUP_OBJECT(dialog1, button_finish, "buttonFinish");

	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
	return dialog1;
}

/**
This function will be called by Account Management moduleto create E-mail     
account set up 2/4 window.

@param  acc_creat_edit boolean variable represents Edit or Create an account.
@return GtkWidget pointer contains window pointer which this function will    
   create.
*/
GtkWidget *ui_accmgmt_createemailaccountsetup2(gboolean acc_creat_edit)
{
	GtkWidget *dialog_setup2 = NULL;
	GtkWidget *scrolledwindow1 = NULL;
	GtkWidget *viewport1 = NULL;
	GtkWidget *vbox_setup2 = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *name_entry = NULL;
	GtkWidget *username_entry = NULL;
	GtkWidget *passwd_entry = NULL;
	GtkWidget *email_entry = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *button_finish = NULL;
	GtkWidget *back_button = NULL;
	GtkWidget *button_next = NULL;
	GtkWidget *cancel_button = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	userSettingFlag *user_setting_flag = NULL;
	gchar *userentry_type = "USERNAME";
	gchar *email_type = "EMAIL";

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailEditInfo");

	user_setting_flag = g_new0(userSettingFlag, 1);
	user_setting_flag->username_flag = FALSE;
	user_setting_flag->email_flag = FALSE;
	user_setting_flag->username_commit = FALSE;
	user_setting_flag->email_commit = FALSE;

/*Store widget pointers and local variables,for use in callback functions */
	if (acc_creat_edit) {
		if (ui_accMgmt_editinfo == NULL) {
			ULOG_DEBUG("Null Pointer assigned in ui_accMgmt_editinfo");
			return NULL;
		}
	}

	dialog_setup2 = gtk_dialog_new();
	gtk_window_set_transient_for(GTK_WINDOW(dialog_setup2), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(dialog_setup2), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(dialog_setup2), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(dialog_setup2), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_default_size(GTK_WINDOW(dialog_setup2), DIALOG_WID, WIDGET_HEIGHT);
	gtk_window_set_title(GTK_WINDOW(dialog_setup2), _("mcen_ti_emailsetup_title_2_4"));
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog_setup2), FALSE);

	scrolledwindow1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow1);
	gtk_box_pack_start(GTK_BOX((GTK_DIALOG(dialog_setup2)->vbox)), scrolledwindow1,
			   TRUE, TRUE, HILDON_MARGIN_DEFAULT);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow1),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow1), GTK_SHADOW_NONE);
	viewport1 = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport1);
	gtk_container_add(GTK_CONTAINER(scrolledwindow1), viewport1);
	gtk_viewport_set_shadow_type((GtkViewport *) viewport1, GTK_SHADOW_NONE);

	vbox_setup2 = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_container_add(GTK_CONTAINER(viewport1), vbox_setup2);

	/*Your name field */
	name_entry = gtk_entry_new();
	/*Retain 'autocap' property for NAME field */
	/*g_object_set(name_entry, "autocap", FALSE, NULL); */

	if (acc_creat_edit) {
		gtk_entry_set_text((GtkEntry *) name_entry,
				   ui_accMgmt_editinfo->user_settings.alias_name);
	}
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_li_emailsetup_name"), name_entry,
			       NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, ENTRY_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox_setup2), caption_control, FALSE, FALSE, 0);

	/*user name field */
	username_entry = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(username_entry), HILDON_GTK_INPUT_MODE_FULL);

	if (acc_creat_edit) {	/* when edit button is pressed */
		gtk_entry_set_text((GtkEntry *) username_entry,
				   ui_accMgmt_editinfo->user_settings.userid);
		if (ui_accMgmt_editinfo->receive_settings.is_not_editable == TRUE) {
			gtk_widget_set_sensitive(username_entry, FALSE);
		}

	}
	caption_control =
	    hildon_caption_new(size_group, _("mcen_li_emailsetup_username"),
			       username_entry, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, ENTRY_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox_setup2), caption_control, FALSE, FALSE, 0);

	/*password field */
	passwd_entry = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(passwd_entry), HILDON_GTK_INPUT_MODE_FULL);
	gtk_entry_set_max_length(GTK_ENTRY(passwd_entry), MAX_PASSWD_ENTRY_LEN);
	gtk_entry_set_visibility((GtkEntry *) passwd_entry, FALSE);

	if (acc_creat_edit) {	/* when edit button is pressed */
		gtk_entry_set_text((GtkEntry *) passwd_entry,
				   ui_accMgmt_editinfo->user_settings.passwd);
	}
	gtk_widget_show(passwd_entry);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_li_emailsetup_password"),
			       passwd_entry, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, ENTRY_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox_setup2), caption_control, FALSE, FALSE, 0);

	/*Email Address field */
	email_entry = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(email_entry), HILDON_GTK_INPUT_MODE_FULL);
	if (acc_creat_edit) {	/* when edit button is pressed */
		gtk_entry_set_text((GtkEntry *) email_entry,
				   ui_accMgmt_editinfo->user_settings.email);
	}

	caption_control =
	    hildon_caption_new(size_group, _("mcen_li_emailsetup_email_address"),
			       email_entry, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, ENTRY_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox_setup2), caption_control, FALSE, FALSE, 0);
	dialog_action_area1 = GTK_DIALOG(dialog_setup2)->action_area;
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);
	button_finish = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagetwo_finish"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup2), button_finish, 0);
	if (!acc_creat_edit) {
		gtk_widget_set_sensitive(button_finish, FALSE);
	}
	back_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagetwo_back"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup2), back_button, 0);
	g_signal_connect(G_OBJECT(back_button), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbackback2_cb), dialog_setup2);
	button_next = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagetwo_next"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup2), button_next, 0);
	if (!acc_creat_edit) {	/* Creation of New Account */
		gtk_widget_set_sensitive(button_next, FALSE);
	}
	gtk_widget_grab_focus(name_entry);

	cancel_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagetwo_cancel"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup2), cancel_button, 0);

	/*Signal Handlers */
	g_signal_connect(G_OBJECT(GTK_ENTRY(username_entry)->im_context), "preedit-changed",
			 G_CALLBACK(ui_accmgmt_usrNameEmailChanged_cb), userentry_type);
	g_signal_connect(G_OBJECT(GTK_ENTRY(email_entry)->im_context), "preedit-changed",
			 G_CALLBACK(ui_accmgmt_usrNameEmailChanged_cb), email_type);
	g_signal_connect(G_OBJECT(GTK_ENTRY(username_entry)->im_context), "commit",
			 G_CALLBACK(ui_accmgmt_usrNameEmailCommit_cb), userentry_type);
	g_signal_connect(G_OBJECT(GTK_ENTRY(email_entry)->im_context), "commit",
			 G_CALLBACK(ui_accmgmt_usrNameEmailCommit_cb), email_type);
	g_signal_connect(G_OBJECT(button_finish), "clicked",
			 G_CALLBACK(ui_accmgmt_callbackfinish1_cb), dialog_setup2);
	g_signal_connect(G_OBJECT(button_finish), "insensitive_press",
			 G_CALLBACK(ui_accmgmt_setup2_insensitive_press_cb), (gpointer) TRUE);
	g_signal_connect(G_OBJECT(button_next), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbacknext2_cb), NULL);
	g_signal_connect(G_OBJECT(button_next), "insensitive_press",
			 G_CALLBACK(ui_accmgmt_setup2_insensitive_press_cb), FALSE);
	g_signal_connect(G_OBJECT(cancel_button), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbackcancel2_cb), dialog_setup2);
	g_signal_connect(G_OBJECT(username_entry), CHANGED,
			 G_CALLBACK(ui_accmgmt_user_name_email_changed_cb), username_entry);
	g_signal_connect(G_OBJECT(email_entry), CHANGED,
			 G_CALLBACK(ui_accmgmt_user_name_email_changed_cb), email_entry);
	g_signal_connect_after(G_OBJECT(email_entry), "insert-text",
			       G_CALLBACK(check_update_accname_entry), NULL);
	g_signal_connect_after(G_OBJECT(name_entry), "insert-text",
			       G_CALLBACK(check_update_accname_entry), NULL);
	g_signal_connect_after(G_OBJECT(username_entry), "insert-text",
			       G_CALLBACK(check_update_accname_entry), NULL);

	g_signal_connect(G_OBJECT(username_entry), CHANGED,
			 G_CALLBACK(ui_accmgmt_set_finish_button), NULL);
	g_signal_connect(G_OBJECT(email_entry), CHANGED,
			 G_CALLBACK(ui_accmgmt_set_finish_button), NULL);
	if (ossohelp_exists(HELP_ID_USERDETAILS) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) dialog_setup2);
		g_signal_connect(G_OBJECT(dialog_setup2), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_USERDETAILS);
	}
	g_signal_connect(G_OBJECT(dialog_setup2), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);

	g_object_set_data(G_OBJECT(dialog_setup2), "UserSettingFlag", user_setting_flag);
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_setup2, dialog_setup2, "emailAccountSetUpTitle");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup2, name_entry, "nameEntry");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup2, username_entry, "userNameEntry");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup2, passwd_entry, "passWdEntry");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup2, email_entry, "emailEntry");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup2, button_next, "buttonNext");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup2, button_finish, "buttonFinish");

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_setup2, dialog_action_area1, "dialog_action_area1");

	return dialog_setup2;
}

/**
This function will be called by Account Management module to create E-mail    
account set up 3/4 window.

@param  acc_creat_edit boolean variable represents Edit or Create an account.
@return GtkWidget pointer contains window pointer which this function will               create     
*/

GtkWidget *ui_accmgmt_createemailaccountsetup3(gboolean acc_creat_edit)
{
	/* Variable declarations */
	GtkWidget *dialog_setup3 = NULL;
	GtkWidget *scrolledwindow1 = NULL;
	GtkWidget *viewport1 = NULL;
	GtkWidget *vbox_setup3 = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *caption_control_smtpserver = NULL;
	GtkWidget *caption_control_iap = NULL;
	GtkWidget *entryInputServer = NULL;
	GtkWidget *entryOutputServer = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *finish_button = NULL;
	GtkWidget *back_button = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *cancel_button = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentDialog = NULL;
	GtkWidget *checkbutton_iap = NULL;
	GtkWidget *hseparator = NULL;
	gchar *server_name_label = NULL;
	gchar *ipserver = "IPSERVER", *outserver = "OUTSERVER";
	serverSettingFlag *server_setting_flag = NULL;
	GtkWidget *configure_button = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	server_setting_flag = g_new0(serverSettingFlag, 1);
	server_setting_flag->opserver_flag = FALSE;
	server_setting_flag->outputserver_commit = FALSE;
	server_setting_flag->ipserver_flag = FALSE;
	server_setting_flag->ipserver_commit = FALSE;

	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailEditInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	if (acc_creat_edit) {
		if (ui_accMgmt_editinfo == NULL) {
			ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n",
				   __FUNCTION__, __LINE__);
			return NULL;
		}
	}

	/* Dialog creation. */
	dialog_setup3 = gtk_dialog_new();

	gtk_window_set_transient_for(GTK_WINDOW(dialog_setup3), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(dialog_setup3), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(dialog_setup3), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(dialog_setup3), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_default_size(GTK_WINDOW(dialog_setup3), DIALOG_WID, WIDGET_HEIGHT);
	gtk_window_set_title(GTK_WINDOW(dialog_setup3), _("mcen_ti_emailsetup_title_3_4"));
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog_setup3), FALSE);

	scrolledwindow1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow1);
	gtk_box_pack_start(GTK_BOX((GTK_DIALOG(dialog_setup3)->vbox)), scrolledwindow1,
			   TRUE, TRUE, HILDON_MARGIN_DEFAULT);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow1),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow1), GTK_SHADOW_NONE);

	viewport1 = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport1);
	gtk_container_add(GTK_CONTAINER(scrolledwindow1), viewport1);
	gtk_viewport_set_shadow_type((GtkViewport *) viewport1, GTK_SHADOW_NONE);

	vbox_setup3 = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_container_add(GTK_CONTAINER(viewport1), vbox_setup3);

	/*Incoming server address field */
	entryInputServer = gtk_entry_new();
	if (acc_creat_edit == TRUE) {
		if (ui_accMgmt_accountinfo->receive_settings.is_not_editable == TRUE) {
			gtk_widget_set_sensitive(entryInputServer, FALSE);
		}
	}
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(entryInputServer), HILDON_GTK_INPUT_MODE_FULL);
	gtk_entry_set_max_length(GTK_ENTRY(entryInputServer), MAX_TEXT_ENTRY_LEN);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	/* Creation of input server address Label  depending on account type. */
	if (ui_accMgmt_accountinfo->receive_settings.protocol == RECV_POP3) {
		server_name_label = g_strdup_printf(_("mcen_li_emailsetup_servertype"),
						    _("mcen_li_emailsetup_servertype_pop"));
	} else {
		server_name_label = g_strdup_printf(_("mcen_li_emailsetup_servertype"),
						    _("mcen_li_emailsetup_servertype_imap"));
	}
	caption_control =
	    hildon_caption_new(size_group, server_name_label,
			       entryInputServer, NULL, HILDON_CAPTION_OPTIONAL);

	g_free(server_name_label);
	gtk_widget_set_size_request(caption_control, -1, ENTRY_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox_setup3), caption_control, FALSE, FALSE, 0);

	/*Out going server address field */
	entryOutputServer = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(entryOutputServer), HILDON_GTK_INPUT_MODE_FULL);
	gtk_entry_set_max_length(GTK_ENTRY(entryOutputServer), MAX_TEXT_ENTRY_LEN);
	caption_control_smtpserver =
	    hildon_caption_new(size_group, _("mcen_li_emailsetup_smtp"),
			       entryOutputServer, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox_setup3), caption_control_smtpserver, FALSE, FALSE, 0);
	gtk_widget_set_size_request(caption_control_smtpserver, -1, ENTRY_HEIGHT);

	/*Horizonhal separator */
	hseparator = gtk_hseparator_new();
	gtk_widget_show(hseparator);
	gtk_box_pack_start(GTK_BOX(vbox_setup3), hseparator, TRUE, TRUE, 0);

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	/* IAP smtp server configuration field */
	checkbutton_iap = gtk_check_button_new_with_mnemonic("");
	caption_control_iap =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_connection_smtp"),
			       checkbutton_iap, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox_setup3), caption_control_iap, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control_iap);

	/*configure button */
	configure_button =
	    gtk_button_new_with_mnemonic(_("mcen_bd_advsetup_optional_smtp_configure"));
	caption_control_iap =
	    hildon_caption_new(size_group, _("mcen_fi_advsetup_optional_smtp"), configure_button,
			       NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox_setup3), caption_control_iap, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control_iap);

	g_signal_connect(G_OBJECT(configure_button), "clicked",
			 G_CALLBACK(on_configure_servers_clicked), entryOutputServer);
	g_signal_connect(G_OBJECT(checkbutton_iap), "clicked",
			 G_CALLBACK(enable_configure_servers), caption_control_iap);


	/*set the iap value */
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton_iap),
				     ui_accMgmt_accountinfo->advanced_settings.use_iap_smtp_serv);
	enable_configure_servers(checkbutton_iap, caption_control_iap);
	/* Dialog Buttons Creation. */
	dialog_action_area1 = GTK_DIALOG(dialog_setup3)->action_area;
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);
	finish_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagethree_finish"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup3), finish_button, 0);

	gtk_widget_grab_focus(entryInputServer);

	back_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagethree_back"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup3), back_button, 0);
	g_signal_connect(G_OBJECT(back_button), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbackback3_cb), dialog_setup3);
	next_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagethree_next"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup3), next_button, 0);

	if (acc_creat_edit) {	/* While Edit an existing Account. */
		if (ui_accMgmt_editinfo->receive_settings.recv_server == NULL) {
			gtk_entry_set_text((GtkEntry *) entryInputServer, "");
		} else {
			gtk_entry_set_text((GtkEntry *) entryInputServer,
					   ui_accMgmt_editinfo->receive_settings.recv_server);
		}
		if (ui_accMgmt_editinfo->send_settings.smtp_server == NULL) {
			gtk_entry_set_text((GtkEntry *) entryOutputServer, "");
		} else {
			gtk_entry_set_text((GtkEntry *) entryOutputServer,
					   ui_accMgmt_editinfo->send_settings.smtp_server);
		}
		gtk_widget_set_sensitive(next_button, TRUE);
	} else {		/* While Creation of a new Account. */
		gtk_widget_set_sensitive(finish_button, FALSE);
		gtk_widget_set_sensitive(next_button, FALSE);
	}

	g_signal_connect(G_OBJECT(finish_button), "clicked",
			 G_CALLBACK(ui_accmgmt_callbackfinish1_cb), dialog_setup3);
	g_signal_connect(G_OBJECT(finish_button), "insensitive_press",
			 G_CALLBACK(ui_accmgmt_setup3_insensitive_press_cb), (gpointer) TRUE);
	g_signal_connect(G_OBJECT(next_button), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbacknext3_cb), dialog_setup3);
	g_signal_connect(G_OBJECT(next_button), "insensitive_press",
			 G_CALLBACK(ui_accmgmt_setup3_insensitive_press_cb), FALSE);

	cancel_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagethree_cancel"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup3), cancel_button, 0);
	g_signal_connect(G_OBJECT(cancel_button), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbackcancel4_cb), dialog_setup3);
	g_signal_connect(G_OBJECT(entryInputServer), CHANGED,
			 G_CALLBACK(ui_accmgmt_server_name_changed_cb), entryInputServer);

	g_signal_connect(G_OBJECT(entryOutputServer), CHANGED,
			 G_CALLBACK(ui_accmgmt_server_name_changed_cb), entryOutputServer);

	g_signal_connect(G_OBJECT(GTK_ENTRY(entryInputServer)->im_context), "preedit-changed",
			 G_CALLBACK(ui_accmgmt_server_name_preedit_changed_cb), ipserver);

	g_signal_connect(G_OBJECT(GTK_ENTRY(entryInputServer)->im_context), "commit",
			 G_CALLBACK(ui_accmgmt_server_name_commit_cb), ipserver);
	g_signal_connect(G_OBJECT(GTK_ENTRY(entryOutputServer)->im_context), "preedit-changed",
			 G_CALLBACK(ui_accmgmt_server_name_preedit_changed_cb), outserver);
	g_signal_connect(G_OBJECT(GTK_ENTRY(entryOutputServer)->im_context), "commit",
			 G_CALLBACK(ui_accmgmt_server_name_commit_cb), outserver);

	g_signal_connect(G_OBJECT(entryInputServer), CHANGED,
			 G_CALLBACK(ui_accmgmt_set_finish_button), NULL);
	g_signal_connect(G_OBJECT(entryOutputServer), CHANGED,
			 G_CALLBACK(ui_accmgmt_set_finish_button), NULL);
	if (ossohelp_exists(HELP_ID_SERVERDETAILS) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) dialog_setup3);
		g_signal_connect(G_OBJECT(dialog_setup3), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SERVERDETAILS);
	}
	g_signal_connect(G_OBJECT(dialog_setup3), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);

	g_object_set_data(G_OBJECT(dialog_setup3), "ServerSettingFlag", server_setting_flag);

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_setup3, dialog_setup3, "Email-account set up title");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup3, entryInputServer, "inputServerAddress");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup3, entryOutputServer, "outputServerAddress");
	EMAIL_UI_HOOKUP_OBJECT(parent_dialog, checkbutton_iap, "iapenable");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup3, caption_control, "inputServerCaptionControl");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_setup3, dialog_action_area1, "dialog_action_area1");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup3, next_button, "buttonNext");
	EMAIL_UI_HOOKUP_OBJECT(dialog_setup3, finish_button, "buttonFinish");

	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
	return dialog_setup3;
}

/**
This function will be called by Account Management module to create E-mail    
account set up 4/4 window.

@param  acc_creat_edit boolean variable represents Edit or Create an account.
@return GtkWidget pointer contains window pointer which this function will    
   create.
*/
GtkWidget *ui_accmgmt_createemailaccountsetup4(gboolean acc_creat_edit)
{
	GtkWidget *dialog_setup4 = NULL;
	GtkWidget *scrolledwindow1 = NULL;
	GtkWidget *viewport1 = NULL;
	GtkWidget *vbox1 = NULL;
	GtkWidget *label1 = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *button1 = NULL;
	GtkWidget *back_button = NULL;
	GtkWidget *button3 = NULL;
	GtkWidget *cancel_button = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentDialog = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailEditInfo");


	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	if (acc_creat_edit) {
		if (ui_accMgmt_editinfo == NULL) {
			ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n",
				   __FUNCTION__, __LINE__);
			return NULL;
		}
	}
	dialog_setup4 = gtk_dialog_new();

	gtk_window_set_transient_for(GTK_WINDOW(dialog_setup4), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(dialog_setup4), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(dialog_setup4), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(dialog_setup4), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_title(GTK_WINDOW(dialog_setup4), _("mcen_ti_emailsetup_title_4_4"));
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog_setup4), FALSE);
	gtk_window_set_default_size(GTK_WINDOW(dialog_setup4), DIALOG_WID, WIDGET_HEIGHT);

	scrolledwindow1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow1);
	gtk_box_pack_start(GTK_BOX((GTK_DIALOG(dialog_setup4)->vbox)), scrolledwindow1,
			   TRUE, TRUE, HILDON_MARGIN_DEFAULT);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow1),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow1), GTK_SHADOW_NONE);
	viewport1 = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport1);
	gtk_container_add(GTK_CONTAINER(scrolledwindow1), viewport1);
	gtk_viewport_set_shadow_type((GtkViewport *) viewport1, GTK_SHADOW_NONE);


	vbox1 = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(viewport1), vbox1);

	/*Setup complete label */
	label1 = gtk_label_new(_("mcen_ia_emailsetup_setup_complete"));
	gtk_misc_set_alignment(GTK_MISC(label1), 0, 0);
	gtk_label_set_line_wrap((GtkLabel *) label1, TRUE);

	gtk_box_pack_start(GTK_BOX(vbox1), label1, FALSE, FALSE, 0);

	dialog_action_area1 = GTK_DIALOG(dialog_setup4)->action_area;
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);
	button1 = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagefour_finish"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup4), button1, 0);
	g_signal_connect(G_OBJECT(button1), "clicked",
			 G_CALLBACK(ui_accmgmt_callbackfinish4_cb), dialog_setup4);

	back_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagefour_back"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup4), back_button, 0);
	g_signal_connect(G_OBJECT(back_button), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbackback4_cb), dialog_setup4);
	button3 = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_advanced"));

	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup4), button3, 0);
	g_signal_connect(G_OBJECT(button3), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbackadvance_cb), dialog_setup4);
	cancel_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_pagefour_cancel"));
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_setup4), cancel_button, 0);
	g_signal_connect(G_OBJECT(cancel_button), "clicked",
			 G_CALLBACK(ui_accmgmt_accountsetupcallbackcancel5_cb), dialog_setup4);
	if (ossohelp_exists(HELP_ID_SETUPCOMPLETE) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) dialog_setup4);
		g_signal_connect(G_OBJECT(dialog_setup4), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SETUPCOMPLETE);
	}
	g_signal_connect(G_OBJECT(dialog_setup4), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_setup4, dialog_setup4, "Email-account set up title");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_setup4, dialog_action_area1, "dialog_action_area1");

	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
	return dialog_setup4;
}

/**
This function is a helper function which creates view model for account list.

@param  accountname pointer to GSList contains all existing email account
	 names.
@return GtkWidget pointer contains window pointer which this function will
	create.
*/
GtkTreeModel *ui_accmgmt_create_accountview_model(const GSList * accountname)
{
	/* Variable declarations. */
	GtkTreeIter iter2;	/* Parent  iter  */
	GtkTreeSortable *sortable = NULL;
	GtkTreeModel *accountViewModel = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	CHECK_PARAM((accountname == NULL), "Invalid Input Parameter", return NULL);

	/*Create a new list store object and store it in the global variable */
	folderListStore = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
	accountViewModel = GTK_TREE_MODEL(folderListStore);

	while (accountname != NULL) {
		gtk_list_store_append(folderListStore, &iter2);
		if (ui_compare_utf8_strings
		    (accountname->data, email_main_view->account_manager->default_account) == 0) {
			gtk_list_store_set(folderListStore, &iter2, 0, accountname->data, 1,
					   _("mcen_ia_emailsetup_default"), 2, TRUE,
					   DEFAULT_HEIGHT);
		} else {
			gtk_list_store_set(folderListStore, &iter2, 0, accountname->data, 2, FALSE,
					   DEFAULT_HEIGHT);
		}
		accountname = accountname->next;
	}
	sortable = GTK_TREE_SORTABLE(accountViewModel);
	gtk_tree_sortable_set_sort_func(sortable, 0, sort_iter_compare_func,
					GINT_TO_POINTER(0), NULL);
	gtk_tree_sortable_set_sort_column_id(sortable, 0, GTK_SORT_ASCENDING);
	return accountViewModel;
}

/**
This function is a helper function which creates view model for account list.

@param  selection which is a pointer to GtkTreeSelection.
@return  void 
*/
/*TODO: Need to revamp the function */
void
ui_accmgmt_set_selected_accountview(GtkTreeSelection * selection, GtkTreeModel * accountViewModel)
{
	GtkTreeIter iter;
	gboolean default_account = FALSE;

	gtk_tree_model_get_iter_first(accountViewModel, &iter);
	do {
		gtk_tree_model_get(accountViewModel, &iter, 2, &default_account, -1);
		if (default_account == TRUE) {
			gtk_tree_selection_select_iter(selection, &iter);
			break;
		}
	}
	while (gtk_tree_model_iter_next(accountViewModel, &iter));
	return;
}

/**
This function is a helper function which creates view model for account list.

@param  accountname pointer to GSList contains all existing email account 
     names.
@return GtkWidget pointer contains window pointer which this function will    
     create.
*/
GtkWidget *ui_acccmgmt_create_view_model(const GSList * accountname)
{
	GtkTreeViewColumn *tree_view_column = NULL;
	GtkTreeModel *model = NULL;
	GtkCellRenderer *renderer = NULL;
	GtkWidget *view = NULL;
	GtkTreeSelection *selection = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	view = gtk_tree_view_new();

	tree_view_column = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(tree_view_column, renderer, TRUE);
	gtk_tree_view_column_add_attribute(tree_view_column, renderer, "text", 0);
	gtk_tree_view_column_set_max_width(tree_view_column, ACC_NAME_COL_WIDTH);
	g_object_set(renderer, ATTRIBUTE_ELLIPSIZE, PANGO_ELLIPSIZE_END,
		     ATTRIBUTE_ELLIPSIZE_SET, TRUE, NULL);

	tree_view_column = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), tree_view_column);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(tree_view_column, renderer, TRUE);
	gtk_tree_view_column_add_attribute(tree_view_column, renderer, "text", 1);
	model = ui_accmgmt_create_accountview_model(accountname);
	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
	ui_accmgmt_set_selected_accountview(selection, model);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return view;
}

/**
This function will be called by Main UI module.

@param  accountname pointer to GSList contains all existing email account 
        names.
@return GtkWidget pointer contains window pointer which this function will    
        create.
*/

GtkWidget *ui_accmgmt_createemailaccountsedit(const GSList * accountname)
{
	GtkWidget *dialog_editsetup = NULL;
	GtkWidget *vbox1 = NULL;
	GtkWidget *scrolledwindow1 = NULL;
	GtkWidget *treeview1 = NULL;
	GtkTreeSelection *selection = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *button1 = NULL;
	GtkWidget *back_button = NULL;
	GtkWidget *button3 = NULL;
	GtkWidget *cancel_button = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	if (accountname == NULL) {
		ULOG_DEBUG("Invalid Input Param");
		return NULL;
	}
	if (ui_accMgmt_editinfo != NULL) {
		ui_accmgmt_freeprefsaccount(ui_accMgmt_editinfo);
		ui_accMgmt_editinfo = NULL;
	}
	email_main_view->account_manager->is_mailbox_opened = TRUE;
	ui_accMgmt_editinfo = g_new0(PrefsAccount, 1);
	ui_accmgmt_initialize_global_variable(FALSE);
	dialog_editsetup = gtk_dialog_new();

	gtk_window_set_transient_for(GTK_WINDOW(dialog_editsetup), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(dialog_editsetup), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(dialog_editsetup), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(dialog_editsetup), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_default_size(GTK_WINDOW(dialog_editsetup), DIALOG_WID, WIDGET_HEIGHT);
	gtk_window_set_title(GTK_WINDOW(dialog_editsetup), _("mcen_ti_emailsetup_accounts"));
	gtk_window_set_modal(GTK_WINDOW(dialog_editsetup), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog_editsetup), FALSE);
	vbox1 = GTK_DIALOG(dialog_editsetup)->vbox;
	gtk_widget_show(vbox1);

	scrolledwindow1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow1);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolledwindow1, TRUE, TRUE, HILDON_MARGIN_DEFAULT);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow1),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow1), GTK_SHADOW_NONE);
	treeview1 = ui_acccmgmt_create_view_model(accountname);
	gtk_widget_show(treeview1);
	gtk_container_add(GTK_CONTAINER(scrolledwindow1), treeview1);
	gtk_container_set_border_width(GTK_CONTAINER(scrolledwindow1), HILDON_MARGIN_DEFAULT);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview1), FALSE);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview1));
	if (email_main_view->account_manager->default_account == NULL) {
		ULOG_DEBUG("default_account is NULL\n");
	}
	/* Appending dialog pointer to the global list. */
	ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, dialog_editsetup);

	dialog_action_area1 = GTK_DIALOG(dialog_editsetup)->action_area;
	gtk_widget_show(dialog_action_area1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);
	button1 = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_setasdef"));
	gtk_widget_show(button1);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_editsetup), button1, 0);
	g_signal_connect(G_OBJECT(button1), "clicked",
			 G_CALLBACK(ui_accmgmt_onsetdefault_cb), dialog_editsetup);
	back_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_edit"));
	gtk_widget_show(back_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_editsetup), back_button, 0);
	g_signal_connect(G_OBJECT(back_button), "clicked",
			 G_CALLBACK(ui_accmgmt_onedit_cb), dialog_editsetup);

	button3 = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_delete"));
	gtk_widget_show(button3);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_editsetup), button3, 0);
	g_signal_connect(G_OBJECT(button3), "clicked",
			 G_CALLBACK(accountsetupon_delete_cb), dialog_editsetup);
	cancel_button = gtk_button_new_with_mnemonic(_("mcen_bd_emailsetup_close"));
	gtk_widget_show(cancel_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(dialog_editsetup), cancel_button, 0);
	g_signal_connect(G_OBJECT(cancel_button), "clicked",
			 G_CALLBACK(ui_accmgmt_onclose_cb), dialog_editsetup);
	if (ossohelp_exists(HELP_ID_ACCOUNTSDIALOG) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) dialog_editsetup);
		g_signal_connect(G_OBJECT(dialog_editsetup), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_ACCOUNTSDIALOG);
	}
	g_signal_connect(G_OBJECT(dialog_editsetup), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);


	g_object_set_data(G_OBJECT(dialog_editsetup), "EmailEditInfo", ui_accMgmt_editinfo);
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_editsetup, dialog_editsetup, "emailaccountEdit");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_editsetup, treeview1, "accountListTreeView");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_editsetup, button1, "set_default_account_button");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_editsetup, back_button, "edit_account_button");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_editsetup, button3, "delete_account_button");
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(dialog_editsetup, cancel_button, "edit_account_close");

	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
	return dialog_editsetup;
}

/**
 This function is used to get the value for the incoming port 
 based on receiving protocols and secured port settings.
 
 @param recv_protocol is the gint for the protocol type.
 @param use_ssl is the gboolean representing if ssl is used.
 @param use_tls is the gboolean representing if tls is used.
 which contains all the widegts of the retrieve tab.
 
 @return gint.
*/
static gint ui_accmgmt_get_recv_port_no(gint recv_protocol, gboolean use_ssl, gboolean use_tls)
{
	gint set_port = 0;

	if (recv_protocol == RECV_IMAP) {
		if (use_ssl && !use_tls) {
			set_port = OUT_PORT;

		} else {
			set_port = DEFAULT_IMAP_PORT;
		}
	} else {
		if (use_ssl && !use_tls) {
			set_port = IN_PORT;

		} else {
			set_port = DEFAULT_POP3_PORT;
		}
	}
	return set_port;
}


/**
 This function is used to set the value for the incoming port.
 
 @param recv_protocol is the gint for the protocol type.
 @param port is the gint for the value to set to the spinbutton.
 @param use_ssl is the gboolean representing if ssl is used.
 @param use_tls is the gboolean representing if tls is used.
 @param RetrivevalTab is the pointer to the RetrivevalTab structure, 
 which contains all the widegts of the retrieve tab.
 
 @return void.
*/
static void
ui_accmgt_set_incoming_port(gint recv_protocol, gint port, gboolean use_ssl, gboolean use_tls,
			    const RetrivevalTab * retTab)
{
	gint set_port = 0;


	if (!gActMgtMngr->edit_operation) {
		if (!gActMgtMngr->port_changed) {
			set_port = port;
		} else {
			gActMgtMngr->port_changed = FALSE;
			set_port = ui_accmgmt_get_recv_port_no(recv_protocol, use_ssl, use_tls);
		}
	} else {
		set_port = port;
	}

	/*set the appropriate incoming/outgoing port */
	hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(retTab->spinbutton_inport), set_port);
	return;
}

/**
 This is a utility function to get the retrieval type based on the protocol.
 
  @param recv_protocol is the gint representing the protocol (pop3/imap4)
  @param recv_type is the gint representing the retrieval type give by engine
 
  @return retrival type .
*/
static gint ui_get_retrieval_type(gint recv_protocol, gint recv_type)
{
	gint retrieval_type = HEADERS;

	switch (recv_type) {
	case HEADERS:
		retrieval_type = RECV_HEADERS;
		break;
	case BODY:
		if (recv_protocol == RECV_IMAP)
			retrieval_type = RECV_MESSAGE;
		break;
	case BODY_ATTCH:
		if (recv_protocol == RECV_IMAP)
			retrieval_type = RECV_MESSAGE_ATT;
		else
			retrieval_type = RECV_MESSAGE;
		break;
	}
	return retrieval_type;
}

/**
 This function is used to set the contents of the retrieval tab.
 
 @param RetrivevalTab is the pointer to the RetrivevalTab structure, 
 which contains all the widgets of the retrieve tab.
 @param ui_accMgmt_accountinfo is the pointer to the PrefsAccount structure.
 the account being edited.
 
 @return void.
*/
static void
ui_accmgt_set_adv_retrieval_contents(const RetrivevalTab * retTab,
				     const PrefsAccount * ui_accMgmt_accountinfo)
{
	gint recv_protocol = 0;
	RecvType recv_type = HEADERS;
	gint prot = 0;
	gint secure_port = 0;
	gboolean leave_msg_status;
	gboolean use_ssl = FALSE;
	gboolean use_tls = FALSE;
	gint retrieve_mails = RETRIEVE_10;

	recv_protocol = ui_accMgmt_accountinfo->receive_settings.protocol;
	recv_type = ui_accMgmt_accountinfo->receive_settings.recv_type;
	leave_msg_status = ui_accMgmt_accountinfo->receive_settings.leave_msg;
	use_ssl = ui_accMgmt_accountinfo->advanced_settings.use_ssl;
	use_tls = ui_accMgmt_accountinfo->advanced_settings.use_tls;
	prot = (gint) ui_accMgmt_accountinfo->receive_settings.port;

	/*Setting the retrieval type of the message */
	gtk_combo_box_set_active(GTK_COMBO_BOX(retTab->combo_recvtype),
				 ui_get_retrieval_type(recv_protocol, recv_type));

	/*setting leave message on server */
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(retTab->leave_mesg_checkbutton),
				     leave_msg_status);

	/*Setting the no. of emails to be retrieved */
	if (recv_protocol == RECV_IMAP) {
		switch (ui_accMgmt_accountinfo->receive_settings.recent_mails) {
		case 10:
			retrieve_mails = RETRIEVE_10;
			break;
		case ALL:
			retrieve_mails = RETRIEVE_ALL;
			break;
		case 25:
			retrieve_mails = RETRIEVE_25;
			break;
		case 50:
			retrieve_mails = RETRIEVE_50;
			break;
		}
		gtk_combo_box_set_active(GTK_COMBO_BOX(retTab->combo_no_email), retrieve_mails);
	}

	/*Setting the password authetication field */
	if (ui_accMgmt_accountinfo->receive_settings.passwd_auth_type == RECV_NORMAL) {
		gtk_combo_box_set_active(GTK_COMBO_BOX(retTab->combo_passwd_auth),
					 PASSWD_AUTH_NORMAL);
	} else {
		gtk_combo_box_set_active(GTK_COMBO_BOX(retTab->combo_passwd_auth),
					 PASSWD_AUTH_SECURE);
	}
	if (recv_protocol == RECV_IMAP) {
		gtk_widget_set_sensitive(retTab->combo_passwd_auth, FALSE);
	}

	/*Setting incoming security port while editing */
	if (use_tls) {
		secure_port = SECURE_USE_TLS;
	} else if (use_ssl) {
		secure_port = SECURE_USE_SSL;
	} else {
		secure_port = SECURE_NONE;
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(retTab->combo_ssl), secure_port);

	/*setting incoming port */
	ui_accmgt_set_incoming_port(recv_protocol, prot, use_ssl, use_tls, retTab);

	return;
}

/**
 This function is used to set the contents of the composing tab.
 
 @param  is the pointer to the SendingTab structure, 
 which contains all the widgets of the composing tab.
 @param ui_accMgmt_accountinfo is the pointer to the PrefsAccount structure.
 the account being edited.
 
 @return void.
*/
static void
ui_accmgt_set_adv_sending_contents(const SendingTab * sendingTab,
				   const PrefsAccount * ui_accMgmt_accountinfo)
{
	gchar *defaultAccount = NULL;
	gint sending_option = SEND_IMMEDIATELY;
	gboolean ckbtn_def_acc_active = FALSE;
	gboolean ckbtn_def_acc_sensitive = FALSE;


	/*Setting the  format of the email (plain/html) */
	if (ui_accMgmt_accountinfo->advanced_settings.email_type == RECV_PLAINTXT) {
		gtk_combo_box_set_active(GTK_COMBO_BOX(sendingTab->combo_email_type), PLAIN_FORMAT);
	} else {
		gtk_combo_box_set_active(GTK_COMBO_BOX(sendingTab->combo_email_type), HTML_FORMAT);
	}


	/*Setting the sending type of mail(Immediately/On request/On next Connection) */
	if (ui_accMgmt_accountinfo->advanced_settings.email_sending == 1) {
		sending_option = SEND_IMMEDIATELY;
	} else if (ui_accMgmt_accountinfo->advanced_settings.email_sending == 2) {
		sending_option = SEND_NEXT_CONN;
	} else {
		sending_option = SEND_ONREQUEST;
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(sendingTab->combo_sendemail), sending_option);


	/*setting include original in reply */
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sendingTab->checkbutton_org_rep),
				     ui_accMgmt_accountinfo->advanced_settings.org_in_reply);

	/*setting Read reciept request */
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sendingTab->checkbutton_read_receipt),
				     ui_accMgmt_accountinfo->advanced_settings.request_read_report);

	/*Setting is_default check button */
	if (!gActMgtMngr->edit_operation) {
		if (!email_main_view->account_manager->configured_accounts) {
			ckbtn_def_acc_active = TRUE;
			ckbtn_def_acc_sensitive = FALSE;
		} else {
			ckbtn_def_acc_sensitive = TRUE;
		}
	} else {
		ckbtn_def_acc_active = ui_accMgmt_accountinfo->user_settings.is_default;
		defaultAccount = email_main_view->account_manager->default_account;
		if (ui_compare_utf8_strings
		    (ui_accMgmt_accountinfo->user_settings.account_name, defaultAccount) == 0) {
			ckbtn_def_acc_sensitive = FALSE;
		} else {
			ckbtn_def_acc_sensitive = TRUE;
		}
	}

	return;
}

/**
 This function is used to set the contents of the sending tab.
 
 @param  is the pointer to the SmtpTab structure, 
	 which contains all the widgets of the sending tab.
 @param ui_accMgmt_accountinfo is the pointer to the PrefsAccount structure.
        it contains the actual prefs account data.
 @param ui_accMgmt_editinfo is the pointer to the PrefsAccount structure of 
	 the account being edited.It contains the edited prefs account data.
 
 @return void.
*/
static void
ui_accmgt_set_adv_smtp_contents(const SMTPTab * smtpTab,
				const PrefsAccount * ui_accMgmt_accountinfo,
				const PrefsAccount * ui_accMgmt_editinfo)
{
	gint smtp_auth = 0;
	gint secure_outgoing_port = 0;

	/*Setting smtp authetication */
	if (ui_accMgmt_accountinfo->send_settings.smtp_auth_type == 1) {
		smtp_auth = SMTP_LOGIN;
	} else if (ui_accMgmt_accountinfo->send_settings.smtp_auth_type == 0) {
		smtp_auth = SMTP_NONE;
	} else {
		smtp_auth = SMTP_CRAM_MD5;
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(smtpTab->combo_smtpauth), smtp_auth);


	/*set the sensitivity of user name based on smtp auth type .If NONE make the
	 * user name insensitive.
	 */
	if (ui_accMgmt_accountinfo->send_settings.smtp_auth_type == 0) {
		gtk_widget_set_sensitive(smtpTab->entry_user_name, FALSE);
	}
	/*setting user name field */
	if (!gActMgtMngr->edit_operation) {
		gtk_entry_set_text(GTK_ENTRY(smtpTab->entry_user_name),
				   ui_accMgmt_accountinfo->user_settings.userid);
	} else {
		if (ui_compare_utf8_strings
		    (ui_accMgmt_accountinfo->user_settings.userid,
		     ui_accMgmt_editinfo->user_settings.userid) == 0) {
			gtk_entry_set_text(GTK_ENTRY(smtpTab->entry_user_name),
					   ui_accMgmt_accountinfo->send_settings.username);
		} else {
			gtk_entry_set_text(GTK_ENTRY(smtpTab->entry_user_name),
					   ui_accMgmt_accountinfo->user_settings.userid);
		}
	}

	/*set the sensitivity of  password based on smtp auth type .If NONE make the
	 * password field insensitive.
	 */
	if (ui_accMgmt_accountinfo->send_settings.smtp_auth_type == 0) {
		gtk_widget_set_sensitive(smtpTab->entry_passwd, FALSE);
	}
	/*setting pass word field */
	if (!gActMgtMngr->edit_operation) {
		gtk_entry_set_text(GTK_ENTRY(smtpTab->entry_passwd),
				   ui_accMgmt_accountinfo->user_settings.passwd);
	} else {
		gtk_entry_set_text(GTK_ENTRY(smtpTab->entry_passwd),
				   ui_accMgmt_accountinfo->send_settings.password);
	}

	/*Setting Secure(Outgoing) field */
	if (ui_accMgmt_accountinfo->advanced_settings.smtp_use_tls) {
		secure_outgoing_port = SECURE_USE_TLS;
	} else if (ui_accMgmt_accountinfo->advanced_settings.smtp_use_ssl) {
		secure_outgoing_port = SECURE_USE_SSL;
	} else {
		secure_outgoing_port = SECURE_NONE;
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(smtpTab->combo_smtp_ssl), secure_outgoing_port);

	/*Setting Outgoing email port */
	hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(smtpTab->spinbutton_outport),
				       ui_accMgmt_accountinfo->send_settings.smtp_port);

	return;
}

/**
 This function is used to set the contents of the security tab.
 
 @param  is the pointer to the SecurityTab structure, 
	 which contains all the widgets of the security tab.
 @param ui_accMgmt_accountinfo is the pointer to the PrefsAccount structure.
 
 @return void.
*/
static void
ui_accmgt_set_adv_security_contents(const SecurityTab * securityTab,
				    const PrefsAccount * ui_accMgmt_accountinfo)
{
	gint smime_type = -1;
	gboolean sign = FALSE;
	gboolean encrypt = FALSE;

	/*setting the SMIME type */
	smime_type = ui_accMgmt_accountinfo->advanced_settings.smime_security;
	switch (smime_type) {
	case SMIME_SIGN:
		sign = TRUE;
		break;
	case SMIME_ENCRYPT:
		encrypt = TRUE;
		break;
	case SMIME_SIGN_ENCRYPT:
		sign = TRUE;
		encrypt = TRUE;
		break;
	}

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(securityTab->checkbutton_encrypt), encrypt);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(securityTab->checkbutton_sign), sign);

	/*setting check buttons for showing the warnings for untrusted &
	   expired ids */
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(securityTab->
						       checkbutton_encp_sign),
				     ui_accMgmt_accountinfo->advanced_settings.
				     display_sign_encrypt);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(securityTab->
						       checkbutton_untrusted_id),
				     ui_accMgmt_accountinfo->advanced_settings.display_trust);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(securityTab->
						       checkbutton_expired_id),
				     ui_accMgmt_accountinfo->advanced_settings.display_validity);


}

/**
  This is a helper function. It is used to created the notepad in Advanced Dialog screen.

  @param advOptionViewInfo it holds the pointer to the AdvOptionViewInfo structure.
  @return void.
 */
void ui_accMgmt_create_advance_dialog(AdvOptionViewInfo * advOptionViewInfo)
{
	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	if (advOptionViewInfo == NULL) {
		ULOG_DEBUG_L("Input Param checking failure");
		return;
	}
	advOptionViewInfo->dialog_advancedsetup = gtk_dialog_new();
	gtk_window_set_resizable(GTK_WINDOW(advOptionViewInfo->dialog_advancedsetup), TRUE);

	gtk_window_set_default_size(GTK_WINDOW
				    (advOptionViewInfo->dialog_advancedsetup),
				    ADVANCED_DIALOG_WIDTH, ADVANCED_DIALOG_HEIGHT);
	gtk_window_set_title(GTK_WINDOW(advOptionViewInfo->dialog_advancedsetup),
			     _("mcen_ti_advsetup_options"));

	gtk_window_set_transient_for(GTK_WINDOW
				     (advOptionViewInfo->dialog_advancedsetup), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(advOptionViewInfo->dialog_advancedsetup), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(advOptionViewInfo->dialog_advancedsetup), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(advOptionViewInfo->dialog_advancedsetup),
			       STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_dialog_set_has_separator(GTK_DIALOG(advOptionViewInfo->dialog_advancedsetup), FALSE);
	advOptionViewInfo->vbox5 = GTK_DIALOG(advOptionViewInfo->dialog_advancedsetup)->vbox;
	gtk_widget_show(advOptionViewInfo->vbox5);

	advOptionViewInfo->notebook1 = gtk_notebook_new();
	gtk_widget_show(advOptionViewInfo->notebook1);
	gtk_box_pack_start(GTK_BOX(advOptionViewInfo->vbox5),
			   advOptionViewInfo->notebook1, TRUE, TRUE, 0);
	if (ossohelp_exists(HELP_ID_ADVANCEDOPTIONS) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) advOptionViewInfo->dialog_advancedsetup);
		g_signal_connect(G_OBJECT(advOptionViewInfo->dialog_advancedsetup),
				 "help", G_CALLBACK(ui_on_dialog_help_activate),
				 HELP_ID_ADVANCEDOPTIONS);
	}

	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
}


/**
  This is a helper function. It is used to created the notepad in Advanced Dialog screen.

  @param advOptionViewInfo it holds the pointer to the AdvOptionViewInfo structure.
  @return void.
 */

void ui_accMgmt_create_advance_retrievel_dialog(const AdvOptionViewInfo * advOptionViewInfo)
{
	/* Variable declarations. */
	RetrivevalTab *retTab = NULL;
	gint recv_protocol = 0;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	GtkAdjustment *retrieve_vadj = NULL;
	GtkWidget *vbox = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *viewport = NULL;
	GtkSizeGroup *size_group = NULL;


	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);


	/*Getting the reference of ui_accMgmt_accountinfo structure, 
	 * holds the data for account settings.*/
	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	if (advOptionViewInfo == NULL || ui_accMgmt_accountinfo == NULL) {
		ULOG_DEBUG_L("Failed at Input Parameter validation");
		return;
	}
	retTab = (RetrivevalTab *) advOptionViewInfo->retrivevalTab;
	recv_protocol = ui_accMgmt_accountinfo->receive_settings.protocol;

	/*scrollwindow/viewport creation */
	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_container_add(GTK_CONTAINER(advOptionViewInfo->notebook1), scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);
	vbox = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(viewport), vbox);
	gtk_container_set_border_width(GTK_CONTAINER(viewport), HILDON_MARGIN_DEFAULT);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);

	retrieve_vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolledwindow));
	gtk_container_set_focus_vadjustment(GTK_CONTAINER(vbox), retrieve_vadj);

	/*Recv type field */
	retTab->combo_recvtype = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_recvtype),
				  _("mcen_fi_advsetup_headersonly"));
	if (ui_accMgmt_accountinfo->receive_settings.protocol == RECV_IMAP) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_recvtype),
					  _("mcen_fi_advsetup_messages"));
	}
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_recvtype),
				  _("mcen_fi_advsetup_messages_attachments"));
	gtk_widget_grab_focus(retTab->combo_recvtype);
	gtk_combo_box_set_active(GTK_COMBO_BOX(retTab->combo_recvtype), 0);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_advsetup_retrieve"),
			       retTab->combo_recvtype, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*Leave message on server checkbox field */
	retTab->leave_mesg_checkbutton = gtk_check_button_new_with_mnemonic("");
	caption_control = hildon_caption_new(size_group,
					     _("mcen_fi_advsetup_leave_on_server"),
					     retTab->leave_mesg_checkbutton, NULL,
					     HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	if (recv_protocol == RECV_IMAP) {
		gtk_widget_hide(caption_control);
	} else {
		gtk_widget_show_all(caption_control);
	}

#if RETAINED_EMAIL
	/*Email retrieval size limit field */
	retTab->spinbutton_size_limit = hildon_number_editor_new(MIN_SIZELIMIT, MAX_SIZELIMIT);
	hildon_number_editor_set_value((HildonNumberEditor *) retTab->spinbutton_size_limit,
				       ui_accMgmt_accountinfo->receive_settings.size_limit);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_advsetup_sizelimit"),
			       retTab->spinbutton_size_limit, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);
#endif

	/*No. of emails to be retrieved field */
	retTab->combo_no_email = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_no_email),
				  _("mcen_fi_advsetup_retrieve_all"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_no_email),
				  _("mcen_fi_advsetup_retrieve_10"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_no_email),
				  _("mcen_fi_advsetup_retrieve_25"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_no_email),
				  _("mcen_fi_advsetup_retrieve_50"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(retTab->combo_no_email), 0);
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_retrieve_number"),
			       retTab->combo_no_email, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	if (recv_protocol == RECV_IMAP) {
		gtk_widget_show_all(caption_control);
	}

	/*Password authentication field */
	retTab->combo_passwd_auth = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_passwd_auth),
				  _("mcen_fi_advsetup_password_normal"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_passwd_auth),
				  _("mcen_fi_advsetup_password_secure"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(retTab->combo_passwd_auth), 0);
	if (recv_protocol == RECV_IMAP) {
		gtk_widget_set_sensitive(retTab->combo_passwd_auth, FALSE);
	}
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_advsetup_password_auth"),
			       retTab->combo_passwd_auth, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);

	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*Incoming (Security) field */
	retTab->combo_ssl = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_ssl),
				  _("mcen_fi_advsetup_other_security_none"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_ssl),
				  _("mcen_fi_advsetup_other_security_normal"));
	if (ui_accMgmt_accountinfo->receive_settings.protocol == RECV_POP3) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_ssl),
					  _("mcen_fi_advsetup_other_security_securepop3s"));
	} else {
		gtk_combo_box_append_text(GTK_COMBO_BOX(retTab->combo_ssl),
					  _("mcen_fi_advsetup_other_security_secureimap4s"));
	}
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_other_security"),
			       retTab->combo_ssl, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);


	/*Incoming Email port */
	retTab->spinbutton_inport = hildon_number_editor_new(LOWER_PORT, UPPER_PORT);
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_port_incoming"),
			       retTab->spinbutton_inport, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*set the contents of retrieval tab with the appropriate settings */
	ui_accmgt_set_adv_retrieval_contents(retTab, ui_accMgmt_accountinfo);

	g_signal_connect(G_OBJECT(retTab->combo_ssl), "changed",
			 G_CALLBACK(ui_acctmgmt_change_security), retTab);
	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(advOptionViewInfo->notebook1),
					gtk_notebook_get_nth_page(GTK_NOTEBOOK
								  (advOptionViewInfo->notebook1),
								  0),
					_("mcen_ti_advsetup_retrieval"));
	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
}

/**
 This function is a hepler function . It is used to create Sending Tab of advanced window.
 
 @param advOptionViewInfo which holds the pointer to the AdvOptionViewInfo structure.
 @return void.
*/
void ui_accMgmt_create_advance_sending_dialog(const AdvOptionViewInfo * advOptionViewInfo)
{
	SendingTab *sendingTab = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	GtkAdjustment *send_vadj = NULL;
	GtkWidget *vbox = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *viewport = NULL;
	GtkSizeGroup *size_group = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);


	/*Getting the reference of ui_accMgmt_accountinfo structure, it
	 * holds the data for account settings.*/
	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	if (advOptionViewInfo == NULL || ui_accMgmt_accountinfo == NULL) {
		ULOG_DEBUG_L("Failed at Input Parameter validation");
		return;
	}
	sendingTab = (SendingTab *) advOptionViewInfo->sendingTab;

	/*Scrollwindow & viewport creation */
	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_container_add(GTK_CONTAINER(advOptionViewInfo->notebook1), scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);
	vbox = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(viewport), vbox);
	gtk_container_set_border_width(GTK_CONTAINER(viewport), HILDON_MARGIN_DEFAULT);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);

	send_vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolledwindow));
	gtk_container_set_focus_vadjustment(GTK_CONTAINER(vbox), send_vadj);

	/*Default email format field */
	sendingTab->combo_email_type = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(sendingTab->combo_email_type),
				  _("mcen_fi_advsetup_def_html"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(sendingTab->combo_email_type),
				  _("mcen_fi_advsetup_def_plain"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(sendingTab->combo_email_type), 0);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_def_email"), sendingTab->combo_email_type,
			       NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*Send email option field(Immediately/On Request/On Next Connection) */
	sendingTab->combo_sendemail = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(sendingTab->combo_sendemail),
				  _("mcen_fi_advsetup_send_now"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(sendingTab->combo_sendemail),
				  _("mcen_fi_advsetup_send_connection"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(sendingTab->combo_sendemail),
				  _("mcen_fi_advsetup_send_request"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(sendingTab->combo_sendemail), 0);
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_send_e-mail"),
			       sendingTab->combo_sendemail, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*Include original in reply field */
	sendingTab->checkbutton_org_rep = gtk_check_button_new();
	gtk_widget_set_sensitive(sendingTab->checkbutton_org_rep, TRUE);
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_include_orig"),
			       sendingTab->checkbutton_org_rep, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*Read request reciept field */
	sendingTab->checkbutton_read_receipt = gtk_check_button_new_with_mnemonic("");
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_advsetup_receipt"),
			       sendingTab->checkbutton_read_receipt, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*set the contents of sending tab with the appropriate settings */
	ui_accmgt_set_adv_sending_contents(sendingTab, ui_accMgmt_accountinfo);


	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(advOptionViewInfo->notebook1),
					gtk_notebook_get_nth_page(GTK_NOTEBOOK
								  (advOptionViewInfo->notebook1),
								  2),
					_("mcen_fi_advsetup_composing"));
	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);

	return;
}

/**
  This function is a helper function which creates the SMTP Tab of Advanced Dialog.
  
  @param advOptionViewInfo which holds the pointer to the AdvOptionViewInfo structure.
  @return void.
 */
void ui_accMgmt_create_advance_smtp_dialog(const AdvOptionViewInfo * advOptionViewInfo)
{
	SMTPTab *smtpTab = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentDialog = NULL;
	GtkAdjustment *smtp_vadj = NULL;
	GtkWidget *vbox = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *viewport = NULL;
	GtkSizeGroup *size_group = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	parentDialog = GTK_WIDGET(g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailEditInfo");

	/*Getting the reference of ui_accMgmt_accountinfo structure,it 
	 * holds the data for account settings.*/
	parent_dialog = GTK_WIDGET(g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	if (!advOptionViewInfo || !ui_accMgmt_accountinfo) {
		ULOG_DEBUG_L("Failed at Input Parameter validation");
		return;
	}
	smtpTab = (SMTPTab *) (advOptionViewInfo->smtpTab);

	/*scrollwindow/viewport creation */
	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_container_add(GTK_CONTAINER(advOptionViewInfo->notebook1), scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);
	gtk_container_set_border_width(GTK_CONTAINER(viewport), HILDON_MARGIN_DEFAULT);
	vbox = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(viewport), vbox);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);

	smtp_vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolledwindow));
	gtk_container_set_focus_vadjustment(GTK_CONTAINER(vbox), smtp_vadj);

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	/*SMTP authetication field */
	smtpTab->combo_smtpauth = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(smtpTab->combo_smtpauth),
				  _("mcen_fi_advsetup_smtp_none"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(smtpTab->combo_smtpauth),
				  _("mcen_fi_advsetup_smtp_login"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(smtpTab->combo_smtpauth),
				  _("mcen_fi_advsetup_smtp_cram_md5"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(smtpTab->combo_smtpauth), 0);

	if (ui_accMgmt_accountinfo->send_settings.smtp_auth_type == 1) {
		gtk_combo_box_set_active(GTK_COMBO_BOX(smtpTab->combo_smtpauth), 1);
	} else if (ui_accMgmt_accountinfo->send_settings.smtp_auth_type == 0) {
		gtk_combo_box_set_active(GTK_COMBO_BOX(smtpTab->combo_smtpauth), 0);
	} else {
		gtk_combo_box_set_active(GTK_COMBO_BOX(smtpTab->combo_smtpauth), 2);
	}
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_smtp_smtpauth"),
			       smtpTab->combo_smtpauth, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*User name entry field */
	smtpTab->entry_user_name = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(smtpTab->entry_user_name),
					HILDON_GTK_INPUT_MODE_FULL);
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_smtp_username"), smtpTab->entry_user_name,
			       NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, ENTRY_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*Password field */
	smtpTab->entry_passwd = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(smtpTab->entry_passwd),
					HILDON_GTK_INPUT_MODE_FULL);
	gtk_entry_set_visibility(GTK_ENTRY(smtpTab->entry_passwd), FALSE);
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_smtp_password"), smtpTab->entry_passwd,
			       NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, ENTRY_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*Secure(Outgoing) field */
	smtpTab->combo_smtp_ssl = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(smtpTab->combo_smtp_ssl),
				  _("mcen_fi_advsetup_other_security_none"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(smtpTab->combo_smtp_ssl),
				  _("mcen_fi_advsetup_other_security_normal"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(smtpTab->combo_smtp_ssl),
				  _("mcen_fi_advsetup_other_security_ssl"));
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_other_security"),
			       smtpTab->combo_smtp_ssl, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*Outgoing email port */
	smtpTab->spinbutton_outport = hildon_number_editor_new(LOWER_PORT, UPPER_PORT);
	hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(smtpTab->
							    spinbutton_outport), TWENTYFIVE);
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_port_outgoing"),
			       smtpTab->spinbutton_outport, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*set the contents of sending tab with the appropriate settings */
	ui_accmgt_set_adv_smtp_contents(smtpTab, ui_accMgmt_accountinfo, ui_accMgmt_editinfo);

	g_signal_connect(G_OBJECT(smtpTab->combo_smtpauth), CHANGED,
			 G_CALLBACK(ui_accmgmt_onsmtpauthtype_cb), smtpTab);
	g_signal_connect(G_OBJECT(smtpTab->combo_smtp_ssl), "changed",
			 G_CALLBACK(ui_acctmgmt_change_smtp_security), smtpTab);

	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(advOptionViewInfo->notebook1),
					gtk_notebook_get_nth_page(GTK_NOTEBOOK
								  (advOptionViewInfo->notebook1),
								  1),
					_("mcen_ti_advsetup_sending"));
	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);

	return;
}


/**
  This function is a helper function. It is used to create the Other Tab in the Advanced Window.

  @param advOptionViewInfo which holds the pointer to the AdvOptionViewInfo structure.
  @return void.
*/

static void ui_accMgmt_create_advance_security_dialog(const AdvOptionViewInfo * advOptionViewInfo)
{

	SecurityTab *securityTab = NULL;
#if RETAINED_EMAIL
	GString *security_mesg = NULL;
#endif
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkAdjustment *security_vadj = NULL;
	GtkWidget *vbox = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *viewport = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *display_label = NULL;
#if RETAINED_EMAIL
	GtkWidget *security_label = NULL;
#else
	GtkWidget *separator = NULL;
#endif

	if (advOptionViewInfo == NULL) {
		CHECK_PARAM((advOptionViewInfo == NULL), "Invalid Input Parameter", return);
	}


	/*Getting the reference of ui_accMgmt_accountinfo structure,it 
	 * holds the data for account settings.*/
	parent_dialog = GTK_WIDGET(g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	securityTab = (SecurityTab *) advOptionViewInfo->securityTab;

	/*scrollwindow /viewport creation */
	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_container_add(GTK_CONTAINER(advOptionViewInfo->notebook1), scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);
	gtk_container_set_border_width(GTK_CONTAINER(viewport), HILDON_MARGIN_DEFAULT);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);
	vbox = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(viewport), vbox);
	security_vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolledwindow));
	gtk_container_set_focus_vadjustment(GTK_CONTAINER(vbox), security_vadj);

#if RETAINED_EMAIL
	/*security label */
	security_mesg = g_string_new(NULL);
	g_string_printf(security_mesg, _("mcen_fi_advsetup_security_info"),
			ui_accMgmt_accountinfo->user_settings.account_name);
	security_label = gtk_label_new(security_mesg->str);
	g_string_free(security_mesg, TRUE);
	gtk_widget_show(security_label);
	gtk_label_set_ellipsize(GTK_LABEL(security_label), PANGO_ELLIPSIZE_END);
	gtk_box_pack_start(GTK_BOX(vbox),
			   security_label, FALSE, FALSE, ACC_SECURITY_LABELS_PADDING);
#endif

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	/*Encrypt checkbutton */
	securityTab->checkbutton_encrypt = gtk_check_button_new_with_mnemonic("");
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_encrypt_mess"),
			       securityTab->checkbutton_encrypt, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*Sign checkbutton */
	securityTab->checkbutton_sign = gtk_check_button_new_with_mnemonic("");
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_sign_mess"),
			       securityTab->checkbutton_sign, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

#if CHANGED_EMAIL
	separator = gtk_hseparator_new();
	gtk_widget_show(separator);
	gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 0);
#endif

	/*Dispaly label */
	display_label = gtk_label_new(_("mcen_fi_advsetup_display_secdialog"));
	gtk_widget_show(display_label);
	gtk_box_pack_start(GTK_BOX(vbox), display_label, FALSE, FALSE, ACC_SECURITY_LABELS_PADDING);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	securityTab->checkbutton_encp_sign = gtk_check_button_new_with_mnemonic("");
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_encrypted_signed"),
			       securityTab->checkbutton_encp_sign, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*check button for showing warning for untrusted ids */
	securityTab->checkbutton_untrusted_id = gtk_check_button_new_with_mnemonic("");
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_untrusted"),
			       securityTab->checkbutton_untrusted_id, NULL,
			       HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*check button for showing warning for expired ids */
	securityTab->checkbutton_expired_id = gtk_check_button_new_with_mnemonic("");
	caption_control =
	    hildon_caption_new(size_group,
			       _("mcen_fi_advsetup_expired_invalid"),
			       securityTab->checkbutton_expired_id, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*set the contents of sending tab with the appropriate settings */
	ui_accmgt_set_adv_security_contents(securityTab, ui_accMgmt_accountinfo);
#if RETAINED_EMAIL
	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(advOptionViewInfo->notebook1),
					gtk_notebook_get_nth_page(GTK_NOTEBOOK
								  (advOptionViewInfo->notebook1),
								  3),
					_("mcen_fi_advsetup_security"));
#else
	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(advOptionViewInfo->notebook1),
					gtk_notebook_get_nth_page(GTK_NOTEBOOK
								  (advOptionViewInfo->notebook1),
								  2),
					_("mcen_fi_advsetup_security"));
#endif
}

/**
 This function is a helper function. It is used to add the buttons in the Advanced window.

 @param advOptionViewInfo which holds the pointer to the AdvOptionViewInfo structure.
 @return void.
 */
void ui_accMgmt_create_advance_buttons(const AdvOptionViewInfo * advOptionViewInfo)
{
	AdvOptionButtons *advOptButtons = NULL;
	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	if (advOptionViewInfo == NULL) {
		ULOG_DEBUG("Failed at Input Parameter validation");
		return;
	}
	advOptButtons = (AdvOptionButtons *) advOptionViewInfo->advOptionButtons;

	advOptButtons->dialog_action_area1 =
	    GTK_DIALOG(advOptionViewInfo->dialog_advancedsetup)->action_area;
	gtk_widget_show(advOptButtons->dialog_action_area1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX
				  (advOptButtons->dialog_action_area1), GTK_BUTTONBOX_END);
	advOptButtons->button1 = gtk_button_new_with_mnemonic(_("mcen_bd_advsetup_ok"));
	gtk_widget_show(advOptButtons->button1);
	gtk_dialog_add_action_widget(GTK_DIALOG
				     (advOptionViewInfo->dialog_advancedsetup),
				     advOptButtons->button1, 0);

	g_signal_connect(G_OBJECT(advOptButtons->button1), "clicked",
			 G_CALLBACK(ui_accmgmt_save_advance_setting_cb),
			 (gpointer) advOptionViewInfo);

	advOptButtons->button2 = gtk_button_new_with_mnemonic(_("mcen_bd_advsetup_cancel"));
	gtk_widget_show(advOptButtons->button2);
	gtk_dialog_add_action_widget(GTK_DIALOG
				     (advOptionViewInfo->dialog_advancedsetup),
				     advOptButtons->button2, 0);
	g_signal_connect(G_OBJECT(advOptButtons->button2), "clicked",
			 G_CALLBACK(ui_accMgmt_cancel_advance_setting_cb),
			 (gpointer) advOptionViewInfo);

	g_signal_connect(G_OBJECT(advOptionViewInfo->dialog_advancedsetup),
			 "key-release-event", G_CALLBACK(ui_on_escape_key_pressed),
			 advOptButtons->button2);
	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
	return;
}

/**
This function will be called by Account Management module to create E-mail
account set up Advanced options window.

@param  acc_creat_edit boolean variable represents Edit or Create an account.
@return GtkWidget pointer contains window pointer which this function will
create.
*/
AdvOptionViewInfo *ui_accmgmt_createadvancedsettingdialog(gboolean acc_creat_edit)
{
	AdvOptionViewInfo *advOptionViewInfo = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	advOptionViewInfo = g_new0(AdvOptionViewInfo, 1);
	advOptionViewInfo->retrivevalTab = g_new0(RetrivevalTab, 1);
	advOptionViewInfo->smtpTab = g_new0(SMTPTab, 1);
	advOptionViewInfo->sendingTab = g_new0(SendingTab, 1);
	advOptionViewInfo->securityTab = g_new0(SecurityTab, 1);
	advOptionViewInfo->advOptionButtons = g_new0(AdvOptionButtons, 1);

	ui_accMgmt_create_advance_dialog(advOptionViewInfo);
	ui_accMgmt_create_advance_retrievel_dialog(advOptionViewInfo);
	ui_accMgmt_create_advance_smtp_dialog(advOptionViewInfo);
#if RETAINED_EMAIL
	ui_accMgmt_create_advance_sending_dialog(advOptionViewInfo);
#endif
	ui_accMgmt_create_advance_security_dialog(advOptionViewInfo);
	ui_accMgmt_create_advance_buttons(advOptionViewInfo);

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return advOptionViewInfo;
}

/**
This is a helper function. It is used to sort the account list.

@param	model		Pointer to the storage model to be sorted
@param  iter_a          Pointer to GtkTreeIter of first row to be compared
@param  iter_b          Pointer to GtkTreeIter of second row to be compared
@param  userdata        Pointer to the user data (not used here) 
@return integer         +1 --> b comes before a
                        -1 --> a comes before b
                         0  --> Both rows are equal
*/
gint
sort_iter_compare_func(GtkTreeModel * model, GtkTreeIter * iter_a,
		       GtkTreeIter * iter_b, gpointer userdata)
{

	gchar *name1 = NULL;
	gchar *name2 = NULL;
	gchar *compstr1 = NULL;
	gchar *compstr2 = NULL;
	gchar *account1 = NULL;
	gchar *account2 = NULL;
	gchar *ptr1 = NULL;
	gchar *ptr2 = NULL;

	gboolean is_account_def1 = FALSE;
	gboolean is_account_def2 = FALSE;

	gint ret = 0;

	ULOG_DEBUG("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	/* Input param validation */
	if (NULL == model || NULL == iter_a || NULL == iter_b) {
		ULOG_DEBUG("[%s][%s]:the input parameter is NULL :\n", __FILE__, __FUNCTION__);
		return 0;
	}

	gtk_tree_model_get(model, iter_a, 0, &name1, 1, &is_account_def1, -1);
	gtk_tree_model_get(model, iter_b, 0, &name2, 1, &is_account_def2, -1);
	if (name1 == NULL || name2 == NULL) {
		if (name1 == NULL && name2 == NULL) {
			ULOG_DEBUG("[%s][%s]:leaves from here as both names are null:\n",
				   __FILE__, __FUNCTION__);
			return 0;
		}
		ret = (name1 == NULL) ? -1 : 1;
	} else {
		if (is_account_def1 == TRUE) {
			ptr1 = strstr(name1, "<default>");
			account1 = g_strndup(name1,
					     (g_utf8_strlen(name1, -1) - g_utf8_strlen(ptr1, -1)));
		} else {
			account1 = g_strdup(name1);
		}
		if (is_account_def2 == TRUE) {
			ptr2 = strstr(name2, "<default>");
			account2 = g_strndup(name2,
					     (g_utf8_strlen(name2, -1) - g_utf8_strlen(ptr2, -1)));
		} else {
			account2 = g_strdup(name2);
		}
		compstr1 = g_utf8_strdown(account1, -1);
		compstr2 = g_utf8_strdown(account2, -1);
		ret = g_utf8_collate(compstr1, compstr2);
		g_free(compstr1);
		compstr1 = NULL;
		g_free(compstr2);
		compstr2 = NULL;
		g_free(account1);
		account1 = NULL;
		g_free(account2);
		account2 = NULL;

	}
	g_free(name1);
	name1 = NULL;
	g_free(name2);
	name2 = NULL;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return ret;
}

/**
 This function is called from Account Management module. 
 It is used to initialize the global variables of Account Management module.
 It allocates memory for AccountMgtMngr structure.

 @param status 
 @return void
 */
void ui_accmgmt_initialize_global_variable(gboolean status)
{
	ULOG_DEBUG("[%s][%s] : enters here :\n", __FILE__, __FUNCTION__);

	/* Allocating memory and initializing for account mgnt global variables */
	gActMgtMngr = g_new0(AccountMgtMngr, 1);

	gActMgtMngr->setting_modified = FALSE;
	gActMgtMngr->ssl_flag = FALSE;
	gActMgtMngr->smtp_auth_active = FALSE;
	gActMgtMngr->advanced_setup_active = FALSE;
	gActMgtMngr->setup3_active = FALSE;
	gActMgtMngr->setup2_active = FALSE;
	gActMgtMngr->RECV_PROT_ADVSETUP = RECV_POP3;
	gActMgtMngr->ACCT_NAME_MODY = FALSE;
	if (status == TRUE) {
		/* Creation of new account */
		gActMgtMngr->edit_operation = FALSE;
	}
	/* Editing the existing account */
	gActMgtMngr->edit_operation = TRUE;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
}

/**
This function is used to create password protected dialog, which will be lunched
from engine if for a perticular account no password is there and some send
opertaion is performed.
@param user_name Pointer to user name of the account which doesn't have 
       password or invalid password.
@return Pointer to newly created widget.
*/
static
GtkWidget *ui_create_mailbox_password_dialog(const gchar * user_name, const gchar * server)
{
	GtkWidget *password_dialog = NULL;
	GtkWidget *dialog_vbox1 = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *user_name_entry = NULL;
	GtkWidget *password_entry = NULL;
	GtkWidget *dialog_action_area = NULL;
	GtkWidget *cancel_button = NULL;
	GtkWidget *ok_button = NULL;
	GtkWidget *label = NULL;
	GtkWidget *user_cptn_cntrl = NULL;
	GtkWidget *password_cptn_cntrl = NULL;
	gchar *label_text = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Input param validation */
	if (user_name == NULL || server == NULL) {
		ULOG_DEBUG("[%s][%s]:the input parameter is NULL :\n", __FILE__, __FUNCTION__);
		return NULL;
	}
	password_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(password_dialog), _("mail_ti_password_protected"));
	gtk_window_set_position(GTK_WINDOW(password_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(password_dialog), GTK_WINDOW(app));
	gtk_dialog_set_has_separator(GTK_DIALOG(password_dialog), FALSE);
	gtk_window_set_modal(GTK_WINDOW(password_dialog), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(password_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(password_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_resizable(GTK_WINDOW(password_dialog), FALSE);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(password_dialog), TRUE);

	dialog_vbox1 = GTK_DIALOG(password_dialog)->vbox;
	gtk_widget_show(dialog_vbox1);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	/* Creating label for password dialog */
	label_text = g_strdup_printf(_("mail_ia_password_info"), server);
	label = gtk_label_new(label_text);
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(dialog_vbox1), label, FALSE, FALSE, 0);
	gtk_widget_set_size_request(label, -1, 50);
	g_free(label_text);

	/* Creating user name entry */
	user_name_entry = gtk_entry_new();
	gtk_editable_set_editable(GTK_EDITABLE(user_name_entry), FALSE);
	gtk_widget_show(user_name_entry);

	/*Get the user name from current send and recieve account name */
	gtk_entry_set_text(GTK_ENTRY(user_name_entry), user_name);
	user_cptn_cntrl =
	    hildon_caption_new(size_group, _("mail_fi_user_name"),
			       user_name_entry, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox1), user_cptn_cntrl, TRUE, TRUE, 0);
	gtk_widget_show(user_cptn_cntrl);

	/* Creating password entry */
	password_entry = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(password_entry), HILDON_GTK_INPUT_MODE_FULL);
	gtk_entry_set_max_length(GTK_ENTRY(password_entry), MAX_PASSWD_ENTRY_LEN);
	gtk_entry_set_visibility((GtkEntry *) password_entry, FALSE);
	gtk_widget_show(password_entry);
	password_cptn_cntrl =
	    hildon_caption_new(size_group, _("mail_fi_password"),
			       password_entry, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox1), password_cptn_cntrl, TRUE, TRUE, 0);
	gtk_widget_show(password_cptn_cntrl);

	/* Getting the dialog action area for password dialog */
	dialog_action_area = GTK_DIALOG(password_dialog)->action_area;
	gtk_widget_show(dialog_action_area);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area), GTK_BUTTONBOX_END);
	/* Creating ok button */
	ok_button =
	    gtk_dialog_add_button(GTK_DIALOG(password_dialog),
				  _("mail_bd_password_ok"), GTK_RESPONSE_ACCEPT);

	/* Creating cancel button */
	cancel_button =
	    gtk_dialog_add_button(GTK_DIALOG(password_dialog),
				  _("mail_bd_password_cancel"), GTK_RESPONSE_REJECT);

	/* Signal handlers */
	g_signal_connect(G_OBJECT(password_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);
	gtk_widget_grab_focus(password_entry);

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(password_dialog, password_entry, "password_entry");

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return password_dialog;
}

/** 
This function is used to launch password protected dialog. This will
be called from engine when ever for a account has invalid/NULL password 
while receiving or sending mails.
@param acct_name Pointer to account name of the account.
@param smpt_flag Sending flag, it will be set while the operation is sending
                 other wise FALSE.
@param show_error_info If this flag is set, "Username or password incorrect"
                       note is shown on top of the dialog.
@return void.
*/
void ui_on_mailbox_protected_dialog(gchar * acct_name, gboolean smpt_flag, gboolean show_error_info)
{
	PrefsAccount *prefaccount_local = NULL;
	GtkWidget *password_dialog = NULL;
	GtkWidget *password_entry = NULL;
	gchar *password_str = NULL;
	gboolean dialog_exit = FALSE, other_conn = FALSE;
	gint result = -1;
	gchar *user_name = NULL;
	gchar *server = NULL;

	ULOG_DEBUG("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	/* Input param validation */
	CHECK_PARAM((acct_name == NULL), "Invalid Input Parameter", return);
	CHECK_PARAM((email_main_view == NULL), ":email_main_view is NULL", return);
	CHECK_PARAM((email_main_view->tags == NULL), ":email_main_view->tags is NULL", return);

	prefaccount_local = ui_get_account_detail(acct_name);
	CHECK_PARAM((prefaccount_local == NULL), ":Prefs account is NULL", return);

	if (smpt_flag == TRUE) {
		user_name = prefaccount_local->send_settings.username;
		server = prefaccount_local->send_settings.smtp_server;
	} else {
		user_name = prefaccount_local->user_settings.userid;
		server = prefaccount_local->receive_settings.recv_server;
	}

	if (email_main_view->tags->msg_download_in_progress == TRUE)
		other_conn = TRUE;
	osso_log(LOG_INFO, "the flag send to engine is %d\n", other_conn);

	password_dialog = ui_create_mailbox_password_dialog(user_name, server);

	if (password_dialog == NULL) {
		ULOG_CRIT("[%s][%s]:Password Dialog is NULL :\n", __FILE__, __FUNCTION__);
		ui_accmgmt_freeprefsaccount(prefaccount_local);
		return;
	}
	password_entry = EMAIL_UI_LOOKUP_OBJECT(password_dialog, "password_entry");
	if (password_entry == NULL) {
		osso_log(LOG_INFO, "Look up widgets are missing\n");
		ui_accmgmt_freeprefsaccount(prefaccount_local);
		return;
	}

	if (show_error_info) {
		gtk_infoprint(GTK_WINDOW(password_dialog), _("mcen_ib_username_pw_incorrect"));
	}

	while (dialog_exit != TRUE) {
		result = gtk_dialog_run(GTK_DIALOG(password_dialog));
		switch (result) {
		case GTK_RESPONSE_ACCEPT:
			/* Get the password entry */
			password_str = g_strdup(gtk_entry_get_text(GTK_ENTRY(password_entry)));
			password_str = g_strstrip(password_str);
			/* Fire the dbus call */
			ui_dbus_set_tmp_passwd(acct_name, password_str, FALSE, other_conn);
			g_free(password_str);
			password_str = NULL;
			dialog_exit = TRUE;
			break;

		case GTK_RESPONSE_REJECT:
			gtk_infoprint((GtkWindow *) app, _("mail_ib_login_cancelled"));
			ui_dbus_set_tmp_passwd(acct_name, password_str, TRUE, other_conn);
			dialog_exit = TRUE;
			break;

		default:
			break;
		}
	}
	/* Destroy the window */
	gtk_widget_destroy(password_dialog);
	ui_accmgmt_freeprefsaccount(prefaccount_local);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is callback function when trying to edit connection specific 
 the configured smtp server.
 @param edit_widget is the GtkWidget that registered the callback.
 @param dialog_smtp_server is the pointer to the smtp_server_dailog
 @return void.
*/
static void smtp_servers_edit_activate(GtkWidget * edit_widget, GtkWidget * dialog_smtp_server)
{
	GtkWidget *smtp_server_details_dialog = NULL;
	smtp_server_details_dialog = create_smtp_server_details_dialog(dialog_smtp_server);
	gtk_widget_show(smtp_server_details_dialog);
	return;

}

/**
 This is callback function when cancel is clicked on the configured smtp server dialog.
 @param cacel_widget is the GtkWidget that registered the callback.
 @param dialog_smtp_server is the pointer to the smtp_server_dailog
 @return void.
*/
static void smtp_servers_cancel_activate(GtkWidget * cancel_widget, GtkWidget * dialog_smtp_server)
{
	gtk_widget_destroy(dialog_smtp_server);
	return;
}

/**
 This is callback function when ok is clicked on edit server details dialog.
 @param okbutton is the GtkWidget that registered the callback.
 @param dialog_smtp_server_details is the pointer to the smtp_server_details dialog
 
 @return void.
*/
static void
smtp_servers_details_ok_activate(GtkWidget * okbutton, GtkWidget * dialog_smtp_server_details)
{
#if CL_USE
	ui_get_iap_smtp_server_details(dialog_smtp_server_details);
#endif
	gtk_widget_destroy(dialog_smtp_server_details);
	return;
}


/**
 This is callback function when cancel is clicked on the edit  smtp server dialog.
 @param cacel_widget is the GtkWidget that registered the callback.
 @param dialog_smtp_server_details is the pointer to the smtp_server_details_dailog
 
 @return void.
*/
static void
smtp_servers_details_cancel_activate(GtkWidget * cancel_widget,
				     GtkWidget * dialog_smtp_server_details)
{
	gtk_widget_destroy(dialog_smtp_server_details);
	return;
}

#if CL_USE
/**
 This function takes the values from edit servers details dialog &
 save the smtp server settings.
 @param iap_smtp_server_details_dialog is the pointer to the smtp_server_details_dailog
 
 @return void.
*/
static void ui_get_iap_smtp_server_details(GtkWidget * iap_smtp_server_details_dialog)
{
	GtkWidget *entry_servername = NULL;
	GtkWidget *combo_smtp_auth = NULL;
	GtkWidget *entry_username = NULL;
	GtkWidget *entry_password = NULL;
	GtkWidget *combo_security = NULL;
	GtkWidget *spinbutton_outport = NULL;
	GtkWidget *treeview = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection = NULL;
	GtkTreeIter iter;
	const gchar *servername = NULL;
	const gchar *username = NULL;
	const gchar *password = NULL;
	IapInfo *iapinfo = NULL;

	entry_servername =
	    EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "entry_servername");
	combo_smtp_auth = EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "combo_smtp_auth");
	entry_username = EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "entry_username");
	entry_password = EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "entry_password");
	combo_security = EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "combo_security");
	spinbutton_outport =
	    EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "spinbutton_outport");
	treeview = EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "treeview");

	iapinfo = g_new0(IapInfo, 1);

	/*get iapname */
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	gtk_tree_selection_get_selected(selection, &model, &iter);
	gtk_tree_model_get(model, &iter, 0, &(iapinfo->iap_name), -1);

	/*get servername */
	servername = gtk_entry_get_text(GTK_ENTRY(entry_servername));
	if (servername && strlen(servername) != 0)
		iapinfo->smtp_servname = g_strdup(servername);
	/*get username */
	username = gtk_entry_get_text(GTK_ENTRY(entry_username));
	if (username && strlen(username) != 0)
		iapinfo->smtp_username = g_strdup(username);
	/*get password */
	password = gtk_entry_get_text(GTK_ENTRY(entry_password));
	if (password && strlen(password) != 0)
		iapinfo->smtp_passwd = g_strdup(password);

	/*get smtp auth type */
	iapinfo->smtp_auth_type = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_smtp_auth));

	/*get smtp port */
	iapinfo->smtp_port =
	    hildon_number_editor_get_value((HildonNumberEditor *) spinbutton_outport);

	/*get secure port */
	switch (gtk_combo_box_get_active(GTK_COMBO_BOX(combo_security))) {
	case 2:
		iapinfo->smtp_use_ssl = TRUE;
		iapinfo->smtp_use_tls = FALSE;
		break;
	case 1:
		iapinfo->smtp_use_ssl = TRUE;
		iapinfo->smtp_use_tls = TRUE;
		break;
	default:
		iapinfo->smtp_use_ssl = FALSE;
		iapinfo->smtp_use_tls = FALSE;
		break;
	}

	if (!ui_update_iap_smtp_server_details(iapinfo))
		osso_log(LOG_ERR, "unable to update the server details");

	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 1, servername, -1);
	ui_free_iapinfo_structure(iapinfo, NULL);

	return;
}

/**
 This function sets the values in the edit smtp servers dialog specific to the iap.
 @param iap_smtp_server_details_dialog is the pointer to the smtp_server_details_dailog
 @param smtp_servname is the const gchar pointer to the server name.
 @param smtp_username is the const gchar pointer to the username.
 @param smtp_passwd is the const gchar pointer to the passwd.
 @param smtp_use_tls is the gboolean representing whether to use tls or not.
 @param smtp_use_ssl is the gboolean representing whether to use ssl or not.
 @param smtp_auth_type is the enum representing the smtp authentication type..
 @param smtp_port is the gushort representing smtp port number.
 
 @return void.
*/
static void
ui_set_iap_smtp_server_details(GtkWidget * iap_smtp_server_details_dialog,
			       const gchar * smtp_servname, const gchar * smtp_username,
			       const gchar * smtp_passwd, gboolean smtp_use_tls,
			       gboolean smtp_use_ssl, gint smtp_auth_type, gushort smtp_port)
{
	GtkWidget *entry_servername = NULL;
	GtkWidget *combo_smtp_auth = NULL;
	GtkWidget *entry_username = NULL;
	GtkWidget *entry_password = NULL;
	GtkWidget *combo_security = NULL;
	GtkWidget *spinbutton_outport = NULL;
	gint secure_port = 0;
	gint smtp_auth = 0;

	entry_servername =
	    EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "entry_servername");
	combo_smtp_auth = EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "combo_smtp_auth");
	entry_username = EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "entry_username");
	entry_password = EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "entry_password");
	combo_security = EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "combo_security");
	spinbutton_outport =
	    EMAIL_UI_LOOKUP_OBJECT(iap_smtp_server_details_dialog, "spinbutton_outport");

	/*set server name ,username, password */
	if (smtp_servname)
		gtk_entry_set_text(GTK_ENTRY(entry_servername), smtp_servname);
	if (smtp_username)
		gtk_entry_set_text(GTK_ENTRY(entry_username), smtp_username);
	if (smtp_passwd)
		gtk_entry_set_text(GTK_ENTRY(entry_password), smtp_passwd);

	/*Setting Secure(Outgoing) field */
	if (smtp_use_tls && smtp_use_ssl) {
		secure_port = SECURE_USE_TLS;
	} else if (smtp_use_ssl && !smtp_use_tls) {
		secure_port = SECURE_USE_SSL;
	} else {
		secure_port = SECURE_NONE;
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_security), secure_port);

	/*Setting smtp authetication */
	if (smtp_auth_type == 1) {
		smtp_auth = SMTP_LOGIN;
	} else if (smtp_auth_type == 0) {
		smtp_auth = SMTP_NONE;
	} else {
		smtp_auth = SMTP_CRAM_MD5;
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_smtp_auth), smtp_auth);


	/*set the sensitivity of user name based on smtp auth type .If NONE make the
	 * user name insensitive.  */
	if (smtp_auth_type == 0) {
		gtk_widget_set_sensitive(entry_username, FALSE);
		gtk_widget_set_sensitive(entry_password, FALSE);
	}

	/*set outgoing port */
	hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(spinbutton_outport), smtp_port);
}

/**
 This function sets the values in the edit smtp servers dialog specific to the iap.
 either configured smtp server or the default smtp servers are set if no smtp server is configured .
 @param iap_smtp_server_details_dialog is the pointer to the smtp_server_details_dailog
 @param treeview is the pointer to the GtkTreeView representing the list of iaps.

 @return void.
*/
static void ui_set_smtp_server_info(GtkWidget * iap_smtp_server_details_dialog,
				    GtkWidget * treeview)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeIter iter;
	GtkTreeModel *model = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentDialog = NULL;
	gchar *iap_name = NULL;
	gchar *server_name = NULL;
	const IapInfo *iap_info = NULL;
	gchar *pass_word = NULL;
	gchar *user_name = NULL;
	gchar *default_server = NULL;
	gchar *title = NULL;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	gtk_tree_selection_get_selected(selection, &model, &iter);
	gtk_tree_model_get(model, &iter, 0, &iap_name, 1, &server_name, 2, &default_server, -1);

	if (iap_name) {
		iap_info = ui_get_smtp_server_detail(iap_name);

		/*check if there is no  server configured use default server settings */
		if (!iap_info || (!iap_info->smtp_servname &&
				  !iap_info->smtp_username && !iap_info->smtp_passwd)) {
			parentDialog =
			    GTK_WIDGET(g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
			ui_accMgmt_editinfo =
			    g_object_get_data(G_OBJECT(parentDialog), "EmailEditInfo");

			/*Getting the reference of ui_accMgmt_accountinfo structure,it 
			 * holds the data for account settings.*/
			parent_dialog =
			    GTK_WIDGET(g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
			ui_accMgmt_accountinfo =
			    g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

			if (!ui_accMgmt_accountinfo) {
				ULOG_DEBUG_L("Failed at Input Parameter validation");
				return;
			}
			/*getting user name & password  */
			if (!gActMgtMngr->edit_operation) {
				user_name = ui_accMgmt_accountinfo->user_settings.userid;
				pass_word = ui_accMgmt_accountinfo->user_settings.passwd;
			} else {
				if (ui_compare_utf8_strings
				    (ui_accMgmt_accountinfo->user_settings.userid,
				     ui_accMgmt_editinfo->user_settings.userid) == 0) {
					user_name = ui_accMgmt_accountinfo->send_settings.username;
				} else {
					user_name = ui_accMgmt_accountinfo->user_settings.userid;
				}
				pass_word = ui_accMgmt_accountinfo->send_settings.password;
			}
			/*is default serevr itself is not configured the just show default string */
			if (!default_server)
				default_server = g_strdup(server_name);

			/*set the default server settings */
			ui_set_iap_smtp_server_details(iap_smtp_server_details_dialog,
						       default_server,
						       user_name,
						       pass_word,
						       ui_accMgmt_accountinfo->advanced_settings.
						       smtp_use_tls,
						       ui_accMgmt_accountinfo->advanced_settings.
						       smtp_use_ssl,
						       ui_accMgmt_accountinfo->send_settings.
						       smtp_auth_type,
						       ui_accMgmt_accountinfo->send_settings.
						       smtp_port);
		} else {
			ui_set_iap_smtp_server_details(iap_smtp_server_details_dialog,
						       iap_info->smtp_servname,
						       iap_info->smtp_username,
						       iap_info->smtp_passwd,
						       iap_info->smtp_use_tls,
						       iap_info->smtp_use_ssl,
						       iap_info->smtp_auth_type,
						       iap_info->smtp_port);
		}
		title = g_strdup_printf(_("mcen_ti_connection_connection_name"), default_server);
		gtk_window_set_default_size(GTK_WINDOW(iap_smtp_server_details_dialog),
					    ui_predict_dialog_width_based_on_title
					    (iap_smtp_server_details_dialog, title), -1);
		gtk_window_set_title(GTK_WINDOW(iap_smtp_server_details_dialog), title);
		g_free(title);
	}
	g_free(iap_name);
	g_free(server_name);
	g_free(default_server);

	return;


}
#endif

/**
 This function is used to show the info banner that the server name is empty
 when dimmed ok button is clicked.
 @param ok_button is the GtkWidget that registered the callback for insensitive press.
 @param parent_dialog is the pointer to iap server details dialog.
 
 @return void.
*/
static void check_server_user_name_empty(GtkWidget * ok_button, GtkWindow * parent_dialog)
{
	GtkWidget *server_entry = NULL;
	const gchar *server_name = NULL;

	server_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "entry_servername");
	server_name = gtk_entry_get_text(GTK_ENTRY(server_entry));

	if (!server_name || (strlen(server_name) == 0)) {
		gtk_infoprint(parent_dialog, _("mcen_ib_enter_ipname"));
		gtk_widget_grab_focus(server_entry);
	}

	return;
}

/**
 This function is used to set the port acccording to the security option 
 if tLS is selected 465 port is selected.
 else port no 25 is used for sending.
 @param combo_security is the GtkWidget that registerd the callback.
 @param parent_dialog is the GtkWidegt pointer to the dialog.
 
*/

static void on_securitytype_change(GtkWidget * combo_security, GtkWidget * parent_dialog)
{
	GtkWidget *smtp_port = 0;
	smtp_port = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "spinbutton_outport");

	/*SSL is selected set the port to 465 */
	if (gtk_combo_box_get_active(GTK_COMBO_BOX(combo_security)) == 2)
		hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(smtp_port), 465);
	else
		hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(smtp_port), 25);

}

/**
 This is a helper function to dim /undim the user name password fields
 based on smtp auth type.for authetication NONE both are dimmed.
 @param smtp_auth_entry is the combo box wiget that registered the callback.
 @param parent_dialog is the pointer to iap server details dialog.
 
 @return void.
*/
static void on_smtpauthtype_change(GtkWidget * combo_smtpauth, GtkWidget * parent_dialog)
{
	GtkWidget *password_entry = NULL;
	GtkWidget *username_entry = NULL;

	password_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "entry_password");
	username_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "entry_username");

	if (gtk_combo_box_get_active(GTK_COMBO_BOX(combo_smtpauth)) == 0) {
		gtk_widget_set_sensitive(username_entry, FALSE);
		gtk_widget_set_sensitive(password_entry, FALSE);
	} else {
		gtk_widget_set_sensitive(username_entry, TRUE);
		gtk_widget_set_sensitive(password_entry, TRUE);
	}

	return;
}


/**
 This is a helper function to dim /undim the ok button if server name is empty
 @param entry wiget that registered the callback.
 @param parent_dialog is the pointer to iap server details dialog.
 
 @return void.
*/
static void enable_save_settings(GtkWidget * entry, GtkWidget * parent_dialog)
{
	const gchar *server_name = NULL;
	GtkWidget *server_entry = NULL;
	GtkWidget *ok_button = NULL;

	server_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "entry_servername");
	ok_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "okbutton");

	server_name = gtk_entry_get_text(GTK_ENTRY(server_entry));

	if (server_name && strlen(server_name) != 0)
		gtk_widget_set_sensitive(ok_button, TRUE);
	else
		gtk_widget_set_sensitive(ok_button, FALSE);

	return;
}


/**
 This function is used to create the server details dialog for the selected iap
 @param smtp_servers_dialog is the pointer to iap-server dialog.
 
 @return void.
*/
static GtkWidget *create_smtp_server_details_dialog(GtkWidget * smtp_servers_dialog)
{
	GtkWidget *iap_smtp_server_details_dialog = NULL;
	GtkWidget *dialog_vbox = NULL;
	GtkWidget *entry_servername = NULL;
	GtkWidget *combo_smtp_auth = NULL;
	GtkWidget *entry_username = NULL;
	GtkWidget *entry_password = NULL;
	GtkWidget *combo_security = NULL;
	GtkWidget *spinbutton_outport = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *cancelbutton = NULL;
	GtkWidget *okbutton = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *treeview = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	gint recv_protocol = 0;

	/*Getting the reference of ui_accMgmt_accountinfo structure,it
	 *          * holds the data for account settings.*/
	parent_dialog = GTK_WIDGET(g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");
	recv_protocol = ui_accMgmt_accountinfo->receive_settings.protocol;



	iap_smtp_server_details_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(iap_smtp_server_details_dialog),
			     _("mcen_ti_connection_connection_name"));
	gtk_window_set_position(GTK_WINDOW(iap_smtp_server_details_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(iap_smtp_server_details_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(iap_smtp_server_details_dialog), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(iap_smtp_server_details_dialog), FALSE);

	dialog_vbox = GTK_DIALOG(iap_smtp_server_details_dialog)->vbox;
	gtk_widget_show(dialog_vbox);

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	/*server name */
	entry_servername = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(entry_servername), HILDON_GTK_INPUT_MODE_FULL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_connection_servername"),
			       entry_servername, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), caption_control, TRUE, TRUE, 0);
	gtk_widget_show_all(caption_control);

	/*smtp auth type */
	combo_smtp_auth = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_smtp_auth),
				  _("mcen_fi_connection_auth_none"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_smtp_auth),
				  _("mcen_fi_connection_auth_login"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_smtp_auth),
				  _("mcen_fi_connection_auth_crammd5"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_smtp_auth), 0);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_connection_auth"),
			       combo_smtp_auth, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), caption_control, TRUE, TRUE, 0);
	gtk_widget_show_all(caption_control);

	/*user name */
	entry_username = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(entry_username), HILDON_GTK_INPUT_MODE_FULL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_connection_username"),
			       entry_username, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), caption_control, TRUE, TRUE, 0);
	gtk_widget_show_all(caption_control);

	/*password */
	entry_password = gtk_entry_new();
	hildon_gtk_entry_set_input_mode(GTK_ENTRY(entry_password), HILDON_GTK_INPUT_MODE_FULL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_connection_password"),
			       entry_password, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), caption_control, TRUE, TRUE, 0);
	gtk_entry_set_visibility(GTK_ENTRY(entry_password), FALSE);
	gtk_widget_show_all(caption_control);

	/*security */
	combo_security = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_security),
				  _("mcen_fi_connection_security_none"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_security),
				  _("mcen_fi_connection_security_normal"));

	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_security),
				  _("mcen_fi_connection_security_ssl"));

	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_security), 0);

	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_connection_security"),
			       combo_security, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), caption_control, TRUE, TRUE, 0);
	gtk_widget_show_all(caption_control);

	/*outgoing port */
	spinbutton_outport = hildon_number_editor_new(LOWER_PORT, UPPER_PORT);
	hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(spinbutton_outport), 25);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_connection_outgoing_port"),
			       spinbutton_outport, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);



	dialog_action_area1 = GTK_DIALOG(iap_smtp_server_details_dialog)->action_area;
	gtk_widget_show(dialog_action_area1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_SPREAD);

	okbutton = gtk_button_new_with_mnemonic(_("mcen_bd_connection_ok"));
	gtk_widget_show(okbutton);
	gtk_dialog_add_action_widget(GTK_DIALOG(iap_smtp_server_details_dialog), okbutton,
				     GTK_RESPONSE_OK);

	cancelbutton = gtk_button_new_with_mnemonic(_("mcen_bd_connection_cancel"));
	gtk_widget_show(cancelbutton);
	gtk_dialog_add_action_widget(GTK_DIALOG(iap_smtp_server_details_dialog), cancelbutton,
				     GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS(okbutton, GTK_CAN_DEFAULT);
	GTK_WIDGET_SET_FLAGS(cancelbutton, GTK_CAN_DEFAULT);

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(iap_smtp_server_details_dialog,
				      iap_smtp_server_details_dialog,
				      "iap_smtp_server_details_dialog");
	EMAIL_UI_HOOKUP_OBJECT(iap_smtp_server_details_dialog, entry_servername,
			       "entry_servername");
	EMAIL_UI_HOOKUP_OBJECT(iap_smtp_server_details_dialog, combo_smtp_auth, "combo_smtp_auth");
	EMAIL_UI_HOOKUP_OBJECT(iap_smtp_server_details_dialog, entry_username, "entry_username");
	EMAIL_UI_HOOKUP_OBJECT(iap_smtp_server_details_dialog, entry_password, "entry_password");
	EMAIL_UI_HOOKUP_OBJECT(iap_smtp_server_details_dialog, combo_security, "combo_security");
	EMAIL_UI_HOOKUP_OBJECT(iap_smtp_server_details_dialog, spinbutton_outport,
			       "spinbutton_outport");
	EMAIL_UI_HOOKUP_OBJECT(iap_smtp_server_details_dialog, okbutton, "okbutton");

	treeview = EMAIL_UI_LOOKUP_OBJECT(smtp_servers_dialog, "treeview");
	EMAIL_UI_HOOKUP_OBJECT(iap_smtp_server_details_dialog, treeview, "treeview");
#if CL_USE
	/*set the smtp server details */
	ui_set_smtp_server_info(iap_smtp_server_details_dialog, treeview);
#endif

	/*server name is empty dont enable the ok  button */
	enable_save_settings(entry_servername, iap_smtp_server_details_dialog);

	g_signal_connect((gpointer) entry_servername, "changed",
			 G_CALLBACK(enable_save_settings), iap_smtp_server_details_dialog);
	g_signal_connect((gpointer) entry_username, "changed",
			 G_CALLBACK(enable_save_settings), iap_smtp_server_details_dialog);
	g_signal_connect((gpointer) okbutton, "insensitive_press",
			 G_CALLBACK(check_server_user_name_empty), iap_smtp_server_details_dialog);
	g_signal_connect(G_OBJECT(combo_smtp_auth), "changed",
			 G_CALLBACK(on_smtpauthtype_change), iap_smtp_server_details_dialog);
	g_signal_connect(G_OBJECT(combo_security), "changed",
			 G_CALLBACK(on_securitytype_change), iap_smtp_server_details_dialog);
	g_signal_connect((gpointer) okbutton, "clicked",
			 G_CALLBACK(smtp_servers_details_ok_activate),
			 iap_smtp_server_details_dialog);
	g_signal_connect((gpointer) cancelbutton, "clicked",
			 G_CALLBACK(smtp_servers_details_cancel_activate),
			 iap_smtp_server_details_dialog);

	return iap_smtp_server_details_dialog;
}

/**
 This function is used to create the dialog representing the list of iaps & 
 their corresponding smtp serevrs.

 @return GtkWidget pointer to the created dialog.
*/
static
GtkWidget *ui_create_smtp_severs_view()
{
	GtkTreeModel *model = NULL;
	GtkCellRenderer *renderer = NULL;
	GtkTreeViewColumn *col = NULL;
	GtkWidget *view = NULL;
	GtkListStore *listStore = NULL;

	view = gtk_tree_view_new();
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, _("mcen_ia_optionalsmtp_connection_name"));
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", 0);

	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, _("mcen_ia_optionalsmtp_servername"));
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", 1);

	listStore = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	model = GTK_TREE_MODEL(listStore);
	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection
				    (GTK_TREE_VIEW(view)), GTK_SELECTION_SINGLE);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), TRUE);

	return view;
}

#if CL_USE
/**
 This function is used to sort the list of iap_names.It is a callback function for
 g_slsist_sort.
 @param iap_info1 is the pointer to the IapInfo structure in one node of the gslist.
 @param iap_info2 is the pointer to the IapInfo structure in another node of the gslist.
 @return 0 - if the names are equal.
         < 0 - if iap_info1->iap_name is less then iap_info2->iap_name.
         > 0 - if iap_info1->iap_name is greater then iap_info2->iap_name.
*/
static
gint ui_sort_iap_list(const IapInfo * iap_info1, const IapInfo * iap_info2)
{
	gint ret_status = -1;

	/* Input param checking */
	if (!iap_info1 || !iap_info2)
		return -1;

	ret_status = ui_compare_utf8_strings(iap_info1->iap_name, iap_info2->iap_name);
	return ret_status;
}

/**
 This function is used to update the list of iaps & there servers in the view.
 @param servers_list is teh GSList of IapInfo stuctures.
 @param treeview is the GtkTreeView pointer.
 @param output_server is the pointer to the smtp server entry in 3/4 screen.

 @return void.
*/
static void ui_update_servers_view(const GSList * servers_list, GtkTreeView * treeview,
				   GtkWidget * output_server)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	GSList *sorted_list = NULL;
	IapInfo *server_info = NULL;
	gchar *server_name = NULL;

	model = gtk_tree_view_get_model(treeview);

	sorted_list = g_slist_sort((GSList *) servers_list, (GCompareFunc) ui_sort_iap_list);
	gtk_banner_show_animation(GTK_WINDOW(app), _("mcen_ib_optionalsmtp_updating"));
	ui_flush_gtk_events();

	for (; sorted_list != NULL && sorted_list->data != NULL; sorted_list = sorted_list->next) {
		server_info = sorted_list->data;
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, server_info->iap_name, -1);

		if (!server_info->smtp_servname && !server_info->smtp_username
		    && !server_info->smtp_passwd) {
			server_name =
			    g_strdup_printf(_("mcen_ia_optionalsmtp_default"),
					    gtk_entry_get_text(GTK_ENTRY(output_server)));
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, 1, server_name, 2,
					   gtk_entry_get_text(GTK_ENTRY(output_server)), -1);
			g_free(server_name);
		} else if (server_info->smtp_servname) {
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, 1,
					   server_info->smtp_servname, -1);
		}

	}
	gtk_banner_close(GTK_WINDOW(app));
	return;
}
#endif
/**
 This function is used to create the dialog for iaps & ther configured servers.

 @param servers_list is the pointer to the GSList containing the IapInfo structures.
 @return pointer to the created dialog.
*/
static GtkWidget *create_smtp_servers_dialog(const GSList * servers_list)
{
	GtkWidget *smtp_servers_dialog = NULL;
	GtkWidget *dialog_vbox = NULL;
	GtkWidget *label_smtpnote = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *treeview = NULL;
	GtkWidget *no_connections_label = NULL;
	GtkWidget *dialog_action_area = NULL;
	GtkWidget *cancelbutton = NULL;
	GtkWidget *editbutton = NULL;

	smtp_servers_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(smtp_servers_dialog), _("mcen_ti_optionalsmtp_servers"));
	gtk_window_set_transient_for(GTK_WINDOW(smtp_servers_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(smtp_servers_dialog), TRUE);
	gtk_window_set_default_size(GTK_WINDOW(smtp_servers_dialog), DIALOG_WID, WIDGET_HEIGHT);
	gtk_dialog_set_has_separator(GTK_DIALOG(smtp_servers_dialog), FALSE);

	dialog_vbox = GTK_DIALOG(smtp_servers_dialog)->vbox;
	gtk_widget_show(dialog_vbox);

	if (servers_list) {
		label_smtpnote = gtk_label_new(_("mcen_ia_optionalsmtp_note"));
		gtk_label_set_line_wrap(GTK_LABEL(label_smtpnote), TRUE);
		gtk_widget_show(label_smtpnote);
		gtk_box_pack_start(GTK_BOX(dialog_vbox), label_smtpnote, FALSE, FALSE, 0);

		scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
		gtk_widget_show(scrolledwindow);
		gtk_box_pack_start(GTK_BOX(dialog_vbox), scrolledwindow, TRUE, TRUE, 0);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
					       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

		treeview = ui_create_smtp_severs_view();
		gtk_widget_show_all(treeview);
		gtk_container_add(GTK_CONTAINER(scrolledwindow), treeview);
	} else {
		no_connections_label = gtk_label_new(_("mcen_ia_optionalsmtp_noconnection"));
		gtk_label_set_line_wrap(GTK_LABEL(no_connections_label), TRUE);
		gtk_widget_show(no_connections_label);
		gtk_box_pack_start(GTK_BOX(dialog_vbox), no_connections_label, FALSE, FALSE, 0);
	}

	dialog_action_area = GTK_DIALOG(smtp_servers_dialog)->action_area;
	gtk_widget_show(dialog_action_area);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area), GTK_BUTTONBOX_SPREAD);


	editbutton = gtk_button_new_with_mnemonic(_("mcen_bd_optionalsmtp_edit"));
	gtk_widget_show(editbutton);
	gtk_dialog_add_action_widget(GTK_DIALOG(smtp_servers_dialog), editbutton, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS(editbutton, GTK_CAN_DEFAULT);

	cancelbutton = gtk_button_new_with_mnemonic(_("mcen_bd_optionalsmtp_close"));
	gtk_widget_show(cancelbutton);
	gtk_dialog_add_action_widget(GTK_DIALOG(smtp_servers_dialog), cancelbutton,
				     GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS(cancelbutton, GTK_CAN_DEFAULT);

	if (!servers_list)
		gtk_widget_set_sensitive(editbutton, FALSE);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(smtp_servers_dialog, smtp_servers_dialog,
				      "smtp_servers_dialog");
	EMAIL_UI_HOOKUP_OBJECT(smtp_servers_dialog, label_smtpnote, "label_smtpnote");
	if (treeview)
		EMAIL_UI_HOOKUP_OBJECT(smtp_servers_dialog, treeview, "treeview");

	g_signal_connect((gpointer) editbutton, "clicked",
			 G_CALLBACK(smtp_servers_edit_activate), smtp_servers_dialog);
	g_signal_connect((gpointer) cancelbutton, "clicked",
			 G_CALLBACK(smtp_servers_cancel_activate), smtp_servers_dialog);



	return smtp_servers_dialog;
}

/**
 Function to free the IapInfo structure.
 @param smtp_server_info is the pointer to the IapInfo.
 @param data is the user data -not used.

 @return void.
*/
#if CL_USE
void ui_free_iapinfo_structure(IapInfo * smtp_server_info, gpointer data)
{
	g_free(smtp_server_info->iap_name);
	g_free(smtp_server_info->smtp_servname);
	g_free(smtp_server_info->smtp_username);
	g_free(smtp_server_info->smtp_passwd);
	g_free(smtp_server_info);
	return;

}
#endif

/**
 This function is used to enable the configure button if
 use connection specific smtp server is checked.
 @param iap_checkbutton is the GtkWidget pointer to the iap check button.
 @param configure_button is the GtkWidget pointer to the configure_button.
 
*/
static void enable_configure_servers(GtkWidget * iap_checkbutton, GtkWidget * caption_iap)
{

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(iap_checkbutton))) {
		gtk_widget_set_sensitive(caption_iap, TRUE);
	} else {
		gtk_widget_set_sensitive(caption_iap, FALSE);
	}

	return;
}

/**
 This is a callback function when configure button is clicked .It creates the
 iap-servers list dilaog.
 @param configure_button is GtkWidget the registered the callback.
 @param output_server is the pointer the GtkEntry of smtp server in 3/4 screen.
 
 @return void.
*/
static void on_configure_servers_clicked(GtkWidget * configure_button, GtkWidget * outgoing_server)
{
	GtkWidget *smtp_servers_dialog = NULL;
	GtkWidget *tree_view = NULL;
	const GSList *servers_list = NULL;

#if CL_USE
	/*initailse the connection specific smtp servers list */
	ui_init_smtp_conn_servers();

	/*Get the list of connection specific smtp serevrs */
	servers_list = ui_get_iapinfo_list();
#endif
	smtp_servers_dialog = create_smtp_servers_dialog(servers_list);
	gtk_widget_show(smtp_servers_dialog);

	/*Update the list */
	if (servers_list) {
		tree_view = EMAIL_UI_LOOKUP_OBJECT(smtp_servers_dialog, "treeview");
#if CL_USE
		ui_update_servers_view(servers_list, GTK_TREE_VIEW(tree_view), outgoing_server);
#endif
	}
	return;
}

/* End of file */
