/**
 * @file attachments.c
 *
 * This file contains all callback functions for the attachment handling.
 *
 *  Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

#include "ui_common.h"
#include "ui_utils.h"
#include "ui_defs.h"
#include "editcallback.h"
#include "callback.h"
#include "ui_al_callbacks.h"
#include "ui_interface.h"



/**
 Function to create the attachment listview.
 
 @param tree_view is the OUT param for treeview creation.
 @param liststore is the OUT param for liststore needed for attachments.
 @param table is the GtkSizeGroup for alingment
 
 @return void
 */
void ui_create_attachment_list_view(MsgFields * fields, GtkWidget * table, gboolean from_viewer)
{

	GtkTreeViewColumn *tree_view_column = NULL;
	GtkCellRenderer *renderer = NULL;
	GtkTreeModel *model = NULL;

/*attachment icon col*/
	fields->view = gtk_tree_view_new();
	gtk_widget_show(fields->view);
	tree_view_column = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(fields->view), tree_view_column);
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(tree_view_column, renderer, TRUE);
	gtk_tree_view_column_add_attribute(tree_view_column, renderer, "pixbuf", COL_ICON);
/*attachment name column*/
	tree_view_column = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(fields->view), tree_view_column);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_set_expand(tree_view_column, TRUE);
	gtk_tree_view_column_set_max_width(tree_view_column, TREE_COLUMN_MAX_WIDTH);
	gtk_tree_view_column_pack_start(tree_view_column, renderer, TRUE);
	gtk_tree_view_column_add_attribute(tree_view_column, renderer, "text", COL_ATT_NAME);
	g_object_set(G_OBJECT(renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
/*attachment size column*/
	tree_view_column = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(fields->view), tree_view_column);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_set_expand(tree_view_column, TRUE);
	gtk_tree_view_column_set_max_width(tree_view_column, TREE_COLUMN_MAX_WIDTH);
	gtk_tree_view_column_pack_start(tree_view_column, renderer, TRUE);
	gtk_tree_view_column_add_attribute(tree_view_column, renderer, "text", COL_ATT_SIZE);
	g_object_set(G_OBJECT(renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);

	fields->liststore =
	    gtk_list_store_new(NUM_COL, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
	model = GTK_TREE_MODEL(fields->liststore);
	gtk_tree_view_set_model(GTK_TREE_VIEW(fields->view), model);

	gtk_tree_selection_set_mode(gtk_tree_view_get_selection
				    (GTK_TREE_VIEW(fields->view)), GTK_SELECTION_MULTIPLE);

	fields->attachment_lable = gtk_label_new(_("mail_va_attachment"));
	if (from_viewer == FALSE) {
		fields->sw_attach = gtk_scrolled_window_new(NULL, NULL);
		gtk_container_add((GtkContainer *) fields->sw_attach, fields->view);
		gtk_table_attach((GtkTable *) table, fields->attachment_lable, 0, 2, 4,
				 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0,
				 0);
		gtk_table_attach_defaults((GtkTable *) table, fields->sw_attach, 2, 4, 4, 5);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(fields->sw_attach),
					       GTK_POLICY_NEVER, GTK_POLICY_NEVER);
		gtk_scrolled_window_set_shadow_type((GtkScrolledWindow *) fields->
						    sw_attach, GTK_SHADOW_ETCHED_IN);
		gtk_widget_hide(fields->sw_attach);
	} else {
		gtk_table_attach((GtkTable *) table, fields->attachment_lable, 0, 1, 4,
				 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0,
				 0);
		fields->sw_attach = hildon_scroll_area_new(fields->scrollwindow_body, fields->view);
		gtk_widget_hide(fields->sw_attach);
		gtk_table_attach((GtkTable *) table, fields->sw_attach, 1, 4, 4, 5, GTK_FILL,
				 GTK_FILL, 0, 0);
	}

	gtk_misc_set_alignment(GTK_MISC(fields->attachment_lable), 1, 0);
	gtk_widget_set_size_request(fields->view, -1, ATTA_FIELDS_SIZE);
	return;
}

/**
 This is a helper function to get the attachment name for the given selection.
 
 @param msgFields is the pointer to the MsgFields structure.
 
 @return GSList of  attachment names.
 */
GSList *ui_get_selected_attachment_name(const MsgFields * fields)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeView *view = NULL;
	GtkTreeIter iter;
	gchar *attachment_name = NULL;
	GtkListStore *liststore = NULL;
	GList *pathslist = NULL;
	GSList *selected_names = NULL;

	CHECK_PARAM(fields == NULL, "Input param check failed\n", return NULL);

	view = (GtkTreeView *) fields->view;
	liststore = fields->liststore;

	selection = gtk_tree_view_get_selection(view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);
	/*Get the iterators of the selections & the attachment names */
	while (pathslist != NULL) {
		if (gtk_tree_model_get_iter((GtkTreeModel *) liststore, &iter,
					    (GtkTreePath *) pathslist->data)) {
			gtk_tree_model_get((GtkTreeModel *) liststore, &iter, 1, &attachment_name,
					   -1);
			selected_names = g_slist_append(selected_names, attachment_name);
		}
		pathslist = pathslist->next;
	}

	pathslist = g_list_nth(pathslist, 0);
	if (pathslist != NULL) {
		g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
		g_list_free(pathslist);
	}
	return selected_names;
}

/**
 This is a helper function to get the attachment name for the given selection.
 
 @param msgFields is the pointer to the MsgFields structure.
 
 @return gint the num  of the attachment.
*/
gint ui_get_attachment_num(const MsgFields * fields, gint nth_path)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeView *view = NULL;
	gint *indices = NULL;
	GtkListStore *liststore = NULL;
	GList *pathslist = NULL;
	gint attachment_num = 0;
	GtkTreeIter iter;

	CHECK_PARAM(fields == NULL, "Input param check failed\n", return -1);

	view = (GtkTreeView *) fields->view;
	liststore = fields->liststore;

	selection = gtk_tree_view_get_selection(view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);

	if (pathslist != NULL && pathslist->data != NULL) {
		indices =
		    gtk_tree_path_get_indices((GtkTreePath *) g_list_nth_data(pathslist, nth_path));
		gtk_tree_model_get_iter((GtkTreeModel *) liststore, &iter,
					(GtkTreePath *) g_list_nth_data(pathslist, nth_path));
		gtk_tree_model_get((GtkTreeModel *) liststore, &iter, 3, &attachment_num, -1);

		if (indices != NULL) {
			if (fields->hbox_dateTime == NULL)	/*just to deffertiate between editor & viewer */
				return (indices[0]);
			else
				return (attachment_num);
		}
	}
	return RETURN_ERROR;
}

/**
 This is a helper function for removing the attachments from the listview.
 
 @param msgFields is the MsgFields structure pointer.
 @param remove_att is the remove attachemnt menu item.
 
 @return void.
 */
gboolean ui_remove_attachment_from_list(const MsgFields * fields, GtkWidget * remove_attachment)
{
	GtkTreeSelection *selection = NULL;
	GtkTreeView *view = NULL;
	GtkTreeIter iter;
	GtkListStore *liststore = NULL;
	gint num_of_rows = 0;
	gint attachment_num = 0;
	GList *pathslist = NULL, *node = NULL;
	gboolean mycard_removed = FALSE;

	CHECK_PARAM(fields == NULL, "Input param check failed\n", return FALSE);

	view = (GtkTreeView *) fields->view;
	liststore = fields->liststore;

	selection = gtk_tree_view_get_selection(view);
	pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);
	node = pathslist;
	/*Get the iterators of the selections & the attachment names */
	while (pathslist != NULL) {
		if (gtk_tree_model_get_iter((GtkTreeModel *) liststore, &iter,
					    (GtkTreePath *) pathslist->data)) {
			gtk_tree_model_get((GtkTreeModel *) liststore, &iter, 3, &attachment_num,
					   -1);
			if (attachment_num == 1)
				mycard_removed = TRUE;
			gtk_list_store_remove(liststore, &iter);
			/*  again getting the selection after removal of row
			 *  freeing the previous selctions */
			pathslist = g_list_nth(node, 0);
			if (pathslist != NULL) {
				g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
				g_list_free(pathslist);
				pathslist = NULL;
			}
			selection = gtk_tree_view_get_selection(view);
			pathslist = gtk_tree_selection_get_selected_rows(selection, NULL);
			node = pathslist;
		} else {
			ULOG_DEBUG("error in getting iter");
			break;
		}
	}
	num_of_rows = gtk_tree_model_iter_n_children((GtkTreeModel *) liststore, NULL);
	if (num_of_rows == 0) {
		gtk_widget_set_sensitive(remove_attachment, FALSE);
		show_attachment_field((MsgFields *) fields, FALSE);
	} else {
		if (num_of_rows < 3) {
			gtk_widget_set_size_request(fields->sw_attach, DEFAULT_WIDTH,
						    ATTA_FIELDS_SIZE * num_of_rows);
		} else {
			gtk_widget_set_size_request(fields->sw_attach, DEFAULT_WIDTH,
						    MAX_ATTA_FIELDS_SIZE);
		}
	}
	pathslist = node;
	if (pathslist != NULL) {
		g_list_foreach(pathslist, (GFunc) gtk_tree_path_free, NULL);
		g_list_free(pathslist);
	}
	return mycard_removed;
}

/**
 This is a callback function whe ever the attachment is selected.
 
 @param selection is the GtkTreeselection of the selected attachemnt.
 @param msgFields is the pointer to the MsgFields structure.
 
 @return void.
 */
void ui_attachment_selection(GtkTreeSelection * selection, const EditorInfo * editor_info)
{
	GtkWidget *remove_attachment_menu = NULL;
	CHECK_PARAM(editor_info == NULL, "Input param check failed\n", return);
	CHECK_PARAM(editor_info->menu == NULL, "Input param check failed\n", return);

	remove_attachment_menu = editor_info->menu->remove_attachment;
	if (remove_attachment_menu != NULL && GTK_IS_WIDGET(remove_attachment_menu)) {
		gtk_widget_set_sensitive(remove_attachment_menu, TRUE);
	}
	gtk_widget_set_sensitive(editor_info->menu->copy, FALSE);
	gtk_widget_set_sensitive(editor_info->menu->cut, FALSE);
	gtk_widget_set_sensitive(editor_info->menu->paste, FALSE);

	return;
}
