/**
 * @file email_mgr.c
 *
 * This file contains ui related  dbus module functions.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */

#define DBUS_API_SUBJECT_TO_CHANGE

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <unistd.h>
#include "libosso.h"
#include <log-functions.h>
#include "utils.h"
#include "defs.h"

#include "dbus_utils.h"
#include "dbus_utils_xml.h"
#include "dbus_utils_def.h"
#include "common.h"
#include "prefs_account.h"
#include "folder.h"
#include "procmsg.h"
#include "pop.h"
#include "smtp_errorcodes.h"
#include "statesave.h"
#include "email_mgr.h"
#include "callbacks.h"
#include "ui_send.h"
#include "al_callbacks.h"
#include "manager.h"
#include "email_gconf.h"

#if CL_USE
#include "cl.h"
static GSList *ret_iapinfo_list = NULL;
static void ui_dbus_get_iapinfo_list_callback(const gchar * recv_str);
static void ui_dbus_set_iapinfo_callback(const gchar * recv_str);
#endif

static gint ui_dbus_mmc_hlr(const gchar * recv_str);

static gint engine_init_status = 0;	/* Engeine initialisation status */
static gint engine_error_code = 0;	/* Latest error occured in engine */
static gint folder_return_code = 0;	/* folder operation return value */
static gint account_error_code = 0;	/* Account operation retrurn status */
static gint engine_acct_status = 0;	/* Account Status */
static GSList *msgheader_list = NULL;	/* folder populated msg header list */
static gchar *multipart_filename = NULL;	/* file name for multi part body */
static gchar *new_draft_msgid = NULL;	/* New msgid of saved draft mail */
static PrefsAccount *ret_prefs_account = NULL;	/* account info returned */
static MimeInfo *ret_mimeinfo = NULL;	/* returned mimeinfo for message */
static MsgInfo *ret_msg_info = NULL;	/* returned msginfo for message */
static gchar *ret_convertedtext = NULL;	/* filename of converted text */
static EmailStateInfo *ret_emailStateInfo = NULL;	/* ret state info */
static GSList *ret_account_list;	/*account list returned by engine */
static GSList *ret_folder_list;	/*account list returned by engine */
static GSList *ret_prefs_account_list;	/*prefs account list returned by engine */
static GSList *ret_section_list;	/*section number list returned by engine */

static DBusConnection *connection = NULL;
static DBusConnection *send_recv_connection = NULL;
static DBusServer *uidbus_server = NULL;
static gboolean engine_status = FALSE;	/* Engine Status */
static gboolean set_iap_status = FALSE;	/* Status of setting iap operation */


/* static functions */

static DBusHandlerResult
ui_message_handler(DBusConnection * connection, DBusMessage * message, void *user_data);
static gint
new_connection_callback(DBusServer * server, DBusConnection * new_connection, void *user_data);
static gint ui_dbus_connect_to_remote(const gchar * msg_to_send);

static gint ui_dbus_account_return_code(const gchar * recv_str);

static void ui_dbus_callback_message_handler(const gchar * message);

static gboolean ui_dbus_connect_to_remote_for_send_recv(const gchar * msg_to_send);

static gint ui_dbus_smtp_param_value_missing(const char *recv_str);

static gint ui_dbus_smtp_set_msg(const char *recv_str);

static gint ui_dbus_smtp_set_percentage(const char *recv_str);

static gint ui_dbus_smtp_update_folder_view(const char *recv_str);

static gint ui_dbus_smtp_update_outbox_mails_sent(const gchar * recv_str);

static osso_pack_return_t ui_dbus_smtp_send_complete(const gchar * recv_str);

static gint ui_dbus_pop3_param_value_missing(const char *recv_str);

static gint ui_dbus_pop3_set_msg(const char *recv_str);

static gint ui_dbus_pop3_set_percentage(const char *recv_str);

static gint ui_dbus_pop3_recv_single(const char *recv_str);

static gint ui_dbus_pop3_recv_complete(const char *recv_str);

static gint ui_dbus_pop3_recv_body_complete(const char *recv_str);

static gint ui_dbus_imap4_param_value_missing(const char *recv_str);

static gint ui_dbus_imap4_msg_headers(const char *recv_str);

static gint ui_dbus_imap4_set_msg(const char *recv_str);

static gint ui_dbus_acct_error_code(const char *recv_str);

static gchar *ui_dbus_return_send_get_msg_id(const gchar * recv_str);

static void ui_dbus_get_account_list_callback(const gchar * recv_str);

static void ui_dbus_folder_get_list_callback(const gchar * recv_str);

static void ui_dbus_get_account_settings_callback(const gchar * recv_str);

static MimeInfo *ui_dbus_recv_msg_body_callback(const gchar * recv_str);

static gchar *ui_dbus_recv_multipart_body_callback(const gchar * recv_str);

static MsgInfo *ui_dbus_recv_msg_attachment_callback(const gchar * recv_str);

static gint ui_dbus_folder_return_code(const gchar * message);

static gint ui_dbus_create_server_thread(void);

static gchar *ui_dbus_return_html_to_text_converter(const gchar * recv_str);

static gint ui_dbus_return_engine_error_code(const gchar * recv_str);

static gint ui_dbus_return_autosave_mail(const gchar * recv_str);

static gint ui_dbus_return_engine_init_status(const gchar * recv_str);

static osso_pack_return_t ui_dbus_tasknav_msg_operation(const gchar * recv_str);

static gint ui_dbus_tasknav_window_id(const gchar * recv_str);

static gint ui_dbus_open_inbox(const gchar * recv_str);

static gint ui_dbus_statesave_signal(const gchar * recv_str);

static gint ui_dbus_return_statesave_read(const gchar * recv_str);

static gint ui_dbus_return_account_check_status(const gchar * recv_str);

static osso_pack_return_t
ui_dbus_parse_set_msg_cb_args(const gchar * recv_str, gint * msgnumber, GArray ** ret_val,
			      gint * index);
static osso_pack_return_t ui_dbus_retrv_msg_status(const gchar * recv_str);

/**
  Function to remove attachments

  @param recv_str: Function ID and request information
  @return list of attachments removed
*/
static GSList *ui_dbus_remove_attach_callback(const gchar * recv_str);

/**
  Function to kill application signal

  @param recv_str Function ID
  @return OSSO_PACK_OK on success and failure otherwise
*/
static osso_pack_return_t ui_dbus_app_killer_signal(const gchar * recv_str);

static osso_pack_return_t ui_dbus_msg_attach_headers(const gchar * recv_str);

/**
  This Function is called when UI DBus module received list of messages to 
  add/delete/update as a result of folder operation 
  or progress information 
  @param recv_str contains list type and message list 
  @return OSSO_PACK_OK on successfully unpacking 
*/
static osso_pack_return_t ui_dbus_folder_list(const gchar * recv_str);

/**
  This Function is called when UI DBus module received folder error code 
  or progress information 
  @param recv_str contains error message or progress information 
  @return OSSO_PACK_OK on successfully unpacking 
*/
static osso_pack_return_t ui_dbus_folder_set_msg(const gchar * recv_str);

/**
  This function is callback used to send MsgInfo to UI.

  @param recv_str MsgInfo of mails
  @return on successful operation returns OSSO_PACK_OK. Otherwise 
  returns errorcode
*/
static osso_pack_return_t ui_dbus_recv_attachment(const gchar * recv_str);
/**
    This Function is called when UI DBus module received list of prefs account 
    information 
    @param recv_str contains funcid:GSList of prefs account 
    @return nothing
*/
static void ui_dbus_prefs_account_get_list_callback(const gchar * recv_str);

/**
  Function sets whether callback is received from engine or not

  @param Function ID
  @return on successful operation returns OSSO_PACK_OK. Otherwise
  returns errorcode
*/
static osso_pack_return_t ui_dbus_engine_status(const gchar * recv_str);

/**
  Function to delete list of messages

  @param list of msginfo of mails to be removed
  @return on successful operation returns OSSO_PACK_OK. Otherwise
  returns errorcode
*/
static osso_pack_return_t ui_dbus_del_list(const gchar * recv_str);

/**
    This Function is called when backup process starts 
    @param none 
    @return nothing
*/
static osso_pack_return_t ui_dbus_handle_backup(const gchar * recv_str);

static UI_DBusFuncs cbFromEngine[] = {
	/* Smtp Call back function Handling */
	{SMTP_PARAM_VALUE_MISSING, ui_dbus_smtp_param_value_missing},
	{SMTP_SET_MSG, ui_dbus_smtp_set_msg},
	{SMTP_SET_PERCENTAGE, ui_dbus_smtp_set_percentage},
	{SMTP_UPDATE_FOLDER_VIEW, ui_dbus_smtp_update_folder_view},
	{SMTP_UPDATE_OUTBOX_MAILS_SENT, ui_dbus_smtp_update_outbox_mails_sent},
	{SMTP_SEND_COMPLETE, ui_dbus_smtp_send_complete},

	/* Pop3 Call back handling functionality */
	{POP3_PARAM_VALUE_MISSING, ui_dbus_pop3_param_value_missing},
	{POP3_SET_MSG, ui_dbus_pop3_set_msg},
	{POP3_SET_PERCENTAGE, ui_dbus_pop3_set_percentage},
	{POP3_RECV_SINGLE, ui_dbus_pop3_recv_single},
	{POP3_RECV_COMPLETE, ui_dbus_pop3_recv_complete},
	{POP3_RECV_BODY_COMPLETE, ui_dbus_pop3_recv_body_complete},

	/* Imap4 Call back Handling Functionality */
	{IMAP4_PARAM_VALUE_MISSING, ui_dbus_imap4_param_value_missing},
	{IMAP4_MSG_HEADERS, ui_dbus_imap4_msg_headers},
	{IMAP4_SET_MSG, ui_dbus_imap4_set_msg},

	/* Other operations from Engine */
	{ACCT_ERROR_CODE, ui_dbus_acct_error_code},
	{AL_FOLDER_LST, ui_dbus_folder_list},
	{AL_FOLDER_MSG, ui_dbus_folder_set_msg},
	{RECV_ATTACHMENT_CALLBACK, ui_dbus_recv_attachment},
	{EDITOR_CREATE, ui_dbus_editor_create},
	{TASKNAV_MSG_OPERATION, ui_dbus_tasknav_msg_operation},
	{TASKNAV_WINDOW_ID, ui_dbus_tasknav_window_id},
	{TASKNAV_OPEN_INBOX, ui_dbus_open_inbox},
	{BACKUP_START_EVENT, ui_dbus_handle_backup},
	{RX_DEL_LST, ui_dbus_del_list},
	{AL_RETR_MSGS_STATUS, ui_dbus_retrv_msg_status},

	/* State Save operation */
	{STATESAVE_SIGNAL, ui_dbus_statesave_signal},
	/*  Mmc handling */
	{MMC_HLR, ui_dbus_mmc_hlr},
	{APP_KILLER_SIGNAL, ui_dbus_app_killer_signal},
	{ENGINE_STATUS_CALLBACK, ui_dbus_engine_status},
	{ATTACH_LIST_CB, ui_dbus_msg_attach_headers},
	{NULL, NULL}
};


static UI_DBusFuncs cbRequiredFuncList[] = {

#if CL_USE
	{GET_IAPINFO_LIST, NULL},
	{SET_IAPINFO_SETTINGS, NULL},
#endif

	{GET_ACCOUNT_SETTINGS, NULL},
	{SET_ACCOUNT_SETTINGS, NULL},
	{GET_ACCOUNT_LIST, NULL},
	{DELETE_ACCOUNT, NULL},
	{RENAME_ACCOUNT, NULL},
	{FOLDER_GET_LIST, NULL},
	{RECV_MSG_BODY_CB, NULL},
	{RECV_MSG_ATTACHMENT, NULL},
	{POPULATE_FOLDER, NULL},
	{GET_MSG_HEADER_LIST, NULL},
	{FOLDER_CREATE, NULL},
	{FOLDER_DELETE, NULL},
	{FOLDER_RENAME, NULL},
	{FOLDER_MOVE_MSG, NULL},
	{FOLDER_REMOVE_MSG, NULL},
	{FOLDER_GET_LIST, NULL},
	{FOLDER_GET_SIZE, NULL},
	{FOLDER_COPY_MSG, NULL},
	{FOLDER_REMOVE_ATTACHMENT, NULL},
	{FOLDER_REMOVEALL_ATTACHMENT, NULL},
	{GET_ENGINE_ERRNO, NULL},
	{GET_ENGINE_INIT_STATUS, NULL},
	{HTML_TO_TEXT_CONVERTER, NULL},
	{TASKNAV_WINDOW_OPERATION, NULL},
	{RECV_MULTIPART_BODY, NULL},
	{AUTOSAVE_MAIL, NULL},
	{STATESAVE_READ, NULL},
	{ACCOUNT_CHECK_STATUS, NULL},
	{PREFS_ACCOUNT_GET_LIST, NULL},
	{UPDATE_SEEN_FLAG, NULL},
	{SEND_GET_MSG_ID},
	{NULL, NULL}
};


/**
    This method is called to create dbus server thread in UI. 
    @param  none 
    @return  none
*/
void ui_dbus_init(void)
{
	ui_dbus_create_server_thread();
}

/**
   This Method is called when UI Dbus module receives a DBus message 
   from engine. Then it checks for connection and received message is 
   signal or contains data.  When it receives Disconnected signal, it 
   closes the UI. 

   @param    message     Message contains signal or data
   @param    user_data   user data
   @return   status of DBus Message handler 
*/

static DBusHandlerResult
ui_message_handler(DBusConnection * connection, DBusMessage * message, void *user_data)
{
	DBusMessageIter iter;
	DBusError error;
	const gchar *str = NULL;
	gchar *word = NULL;
	gchar word1[MEM_ALLOC_SIZE1] = { 0 };
	gint succ = 0;
	gint idx = 0;

	if ((connection == NULL) || (message == NULL)) {
		osso_log(LOG_INFO, "Invalid values for connection or message");
		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	}
	memset(word1, '\0', MEM_ALLOC_SIZE1);

	if (dbus_message_is_signal(message, dbus_message_get_interface(message), "Disconnected")) {
		osso_log(LOG_INFO, "Disconnected signal received for UI \n");
		/* Reopen connection */
		dbus_error_init(&error);

		connection = dbus_connection_open(DBUS_UNIX_SOCKET_CLNT_PATH, &error);
		if (connection == NULL) {
			osso_log(LOG_ERR, "could not open connection:\n");
			dbus_error_free(&error);
			close_application();
			return DBUS_HANDLER_RESULT_HANDLED;
		}
	}

	dbus_message_iter_init(message, &iter);
	dbus_message_iter_get_basic(&iter, &str);

	if (str == NULL) {
		osso_log(LOG_INFO, " str is NULL ...\n");
		return DBUS_HANDLER_RESULT_HANDLED;
	}
	word = strstr(str, seperator);
	if (word == NULL) {
		osso_log(LOG_INFO, "Got a NULL value in the string");
		return DBUS_HANDLER_RESULT_HANDLED;
	} else {
		strncpy(word1, str, word - str);
	}
	for (idx = 0; cbFromEngine[idx].hname != NULL; idx++) {
		if (g_ascii_strcasecmp(word1, cbFromEngine[idx].hname) == 0) {
			succ = cbFromEngine[idx].msghandler(str);
			break;
		}
	}
	return DBUS_HANDLER_RESULT_HANDLED;
}

 /* 
  * Smtp Callback handling
  */

/**
    This method is called when UI Dbus receives message to update the 
    status or missing smtp module parameter 

    @param    recv_str  contains functionid:param_number 
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_smtp_param_value_missing(const gchar * recv_str)
{
	gchar *message = NULL;
	ParamMissing param_number = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);
	param_number = atoi(message);
	smtp_param_value_missing_cb(param_number);
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to update the 
    status smtp module operation.

    @param    recv_str   contains functionid:xml data <status:info>
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/
static osso_pack_return_t ui_dbus_smtp_set_msg(const gchar * recv_str)
{
	gchar *message = NULL;
	gint msgnumber = 0;
	GArray *arguments = NULL;
	gint idx = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	ui_dbus_parse_set_msg_cb_args(message, &msgnumber, &arguments, &idx);

	smtp_set_msg_cb(msgnumber, idx, arguments);
	if (arguments != NULL) {
		g_array_free(arguments, TRUE);
	}
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to update the 
    percentage of mail sent by smtp module.

    @param    recv_str   contains functionid:percentage
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_smtp_set_percentage(const gchar * recv_str)
{
	gint percentage = 0;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);
	percentage = atoi(message);
	smtp_set_percentage_cb(percentage);
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to update the 
    number of mails sent from outbox folder. 

    @param    recv_str   contains functionid:xmldata <outboxmails count:
    mails sent count>
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_smtp_update_outbox_mails_sent(const gchar * recv_str)
{
	gchar *message = NULL;
	xmlNodePtr cur = NULL, node = NULL;
	xmlDocPtr doc = NULL;
	gint totalCount = 0, mailsSent = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	DBUS_XML_UNPACK_INIT(message, return OSSO_PACK_ERROR);

	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, OUTBOXMAILSCOUNT) == 0) {
				osso_email_xml_unpack_int(node, doc, XML_INT, &totalCount);
				continue;
			}
			if (g_ascii_strcasecmp(node->name, OUTBOXMAILSSENT) == 0) {
				osso_email_xml_unpack_int(node, doc, XML_INT, &mailsSent);
				continue;
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;

	smtp_update_outbox_mails_sent(totalCount, mailsSent);
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message status in the 
    outbox folder. Memory allocated for msgheader is to be freed by UI
    module.
    @param    recv_str   contains functionid:xmldata<msgheader:status>
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_smtp_update_folder_view(const gchar * recv_str)
{
	MsgHeader *msgheader = NULL;
	gint option = 0;
	gboolean status = FALSE;
	gchar *message = NULL;
	xmlNodePtr cur = NULL;
	xmlNodePtr node = NULL;
	xmlDocPtr doc = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	DBUS_XML_UNPACK_INIT(message, return OSSO_PACK_ERROR);
	msgheader = g_new0(MsgHeader, 1);
	if (msgheader == NULL) {
		osso_log(LOG_ERR, "Mem alloc Failed");
		DBUS_XML_UNPACK_UN_INIT;
		return OSSO_PACK_ERROR;
	}

	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, XML_MSG_HEADER) == 0) {
				osso_email_xml_unpack_msgheader(node, doc, XML_MSG_HEADER,
								msgheader);
				continue;
			}
			if (g_ascii_strcasecmp(node->name, XML_INT) == 0) {
				osso_email_xml_unpack_int(node, doc, XML_INT, &option);
				continue;
			}
			if (g_ascii_strcasecmp(node->name, XML_BOOL) == 0) {
				osso_email_xml_unpack_bool(node, doc, XML_BOOL, &status);
				continue;
			}

		}
	}
	DBUS_XML_UNPACK_UN_INIT;
	/* msgheader will be freed by UI module */
	smtp_update_folder_view_cb(msgheader, option, status);
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to inform the UI 
    about the completion of send mail.
    @param    recv_str   contains functionid:
    @return on successful operation returns OSSO_PACK_OK. 
*/

static osso_pack_return_t ui_dbus_smtp_send_complete(const gchar * recv_str)
{
	smtp_send_complete_cb();
	return OSSO_PACK_OK;
}

/*
 * Pop3 Callback Handling
 */

/**
    This method is called when UI Dbus receives message to update the 
    status or missing pop3 module parameter 

    @param    recv_str  contains functionid:param_number 
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_pop3_param_value_missing(const gchar * recv_str)
{
	ParamMissing param_number = 0;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	param_number = atoi(message);
	pop3_param_value_missing_cb(param_number);
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to update the 
    status information about the POP3  receiving mails 

    @param    recv_str  contains functionid:param_number 
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_pop3_set_msg(const gchar * recv_str)
{
	gchar *message = NULL;
	gint msgnumber = 0;
	GArray *arguments = NULL;
	gint idx = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	ui_dbus_parse_set_msg_cb_args(message, &msgnumber, &arguments, &idx);
	pop3_set_msg_cb(msgnumber, idx, arguments);
	if (arguments != NULL) {
		g_array_free(arguments, TRUE);
	}
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to update the 
    percentage of mail received by pop3 module.

    @param    recv_str   contains functionid:percentage
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_pop3_set_percentage(const gchar * recv_str)
{
	gint percentage = 0;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);
	percentage = atoi(message);
	pop3_set_percentage_cb(percentage);
	return OSSO_PACK_OK;
}


/**
    This method is called when UI Dbus receives message to update the 
    header information about the received mail in pop3 module. UI module
    will display the header information in inbox folder and also resposible
    for freeing the memory.
    @param    recv_str   contains functionid:xmldata<magheader>
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_pop3_recv_single(const gchar * recv_str)
{
	MsgHeader *msgheader = NULL;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	msgheader = g_new0(MsgHeader, 1);
	if (msgheader == NULL) {
		osso_log(LOG_ERR, "Mem alloc failed \n");
		return OSSO_PACK_ALLOC_FAIL;
	}
	if ((osso_email_utils_unpack_msgHeader(message, msgheader))
	    == OSSO_PACK_ERROR) {
		procmsg_msgheader_free(msgheader);
		return OSSO_PACK_ERROR;
	}
	/* msg header will be freed by UI module */
	pop3_recv_single_cb(msgheader);
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to inform the UI 
    about the completion of received mail.
    @param    recv_str   contains functionid:
    @return on successful operation returns OSSO_PACK_OK. 
*/

static osso_pack_return_t ui_dbus_pop3_recv_complete(const gchar * recv_str)
{
	pop3_recv_complete_cb();
	return OSSO_PACK_OK;
}

/*
 * Imap4 Callback Handling
 */


/**
    This method is called when UI Dbus receives message to update the 
    status or missing invalid input parameter in imap4 module. 
    @param    recv_str  contains functionid:param_number 
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_imap4_param_value_missing(const gchar * recv_str)
{
	ParamMissing param_number = 0;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);
	param_number = atoi(message);
	imap4_param_value_missing_cb(param_number);
	return OSSO_PACK_OK;
}


/**
    This method is called when UI Dbus receives message to update the 
    message header list information of the received mails. UI module 
    will display the header list in inbox folder and also resposible
    to free the list.
    @param    recv_str   contains functionid:xmldata<magheaderlist>
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_msg_attach_headers(const gchar * recv_str)
{
	gchar *message = NULL;
	GSList *list_imap4 = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	osso_email_utils_unpack_msginfo_in_gslist(message, &list_imap4);
	/* msg header list will be free by UI module */
	imap4_msg_attachment_headers_cb(list_imap4);

	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to update the 
    message header list information of the received mails. UI module 
    will display the header list in inbox folder and also resposible
    to free the list.
    @param    recv_str   contains functionid:xmldata<magheaderlist>
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_imap4_msg_headers(const gchar * recv_str)
{
	gchar *message = NULL;
	GSList *list_imap4 = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	osso_email_utils_unpack_msginfo_in_gslist(message, &list_imap4);
	/* msg header list will be free by UI module */
	if (list_imap4 != NULL) {
		imap4_msg_headers_cb(list_imap4);
	}
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to update the 
    information about mails received on the account by imap4 module. 

    @param    recv_str  contains functionid:param_number 
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static gint ui_dbus_imap4_set_msg(const gchar * recv_str)
{
	gchar *message = NULL;
	gint msgnumber = 0;
	GArray *arguments = NULL;
	gint idx = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return FALSE);

	ui_dbus_parse_set_msg_cb_args(message, &msgnumber, &arguments, &idx);
	imap4_set_msg_cb(msgnumber, idx, arguments);

	if (arguments != NULL) {
		g_array_free(arguments, TRUE);
	}
	return OSSO_PACK_OK;
}

/**
    This method is called when UI Dbus receives message to about the error
    occured during account operation.

    @param    recv_str  contains functionid:error number 
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
    returns errorcode 
*/

static osso_pack_return_t ui_dbus_acct_error_code(const gchar * recv_str)
{
	gchar *message = NULL;
	gint error_code = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return FALSE);

	error_code = atoi(message);
	if (error_code != 0) {
		acct_error_code_cb(error_code);
	}

	return OSSO_PACK_OK;
}

/**
    This method is used to free data in Dbus connection 
    @param  data to be freed
    @return nothing
*/

static void test_data_free(gchar * data)
{
	g_free(data);
}

/**
    Callback Function For Handling New connection from another process
    @param DBusServer server process instance
    @param DBusConnection for the server
    @param User data to be passed to the process
    @return status 0 on success otherwise Dbus Error code   
*/

static gint
new_connection_callback(DBusServer * server, DBusConnection * new_connection, void *user_data)
{
	gint succ = 0;

	dbus_connection_ref(new_connection);
	dbus_connection_setup_with_g_main(new_connection, NULL);

	succ = dbus_connection_add_filter(new_connection, ui_message_handler,
					  NULL, (DBusFreeFunction) test_data_free);
	if (!succ) {
		osso_log(LOG_ERR, "server Unable to add filter\n");
		return succ;
	}
	return succ;
}

/**
    This function is to initialize DBUS threads and start DBusServer 
    which will be listening for requests come from Engine module.
    @return TRUE on success or FALSE on failure	
*/

static gboolean ui_dbus_create_server_thread(void)
{
	DBusError error;

	dbus_error_init(&error);
	uidbus_server = dbus_server_listen(DBUS_UNIX_SOCKET_SERV_PATH, &error);
	if (uidbus_server == NULL) {
		osso_log(LOG_ERR, "Unable to start server %s\n", error.message);
		dbus_error_free(&error);
		return FALSE;
	}
	dbus_server_set_new_connection_function(uidbus_server, (DBusNewConnectionFunction)
						new_connection_callback, NULL, NULL);
	dbus_server_setup_with_g_main(uidbus_server, NULL);
	return TRUE;
}



/**
   This fuction is used to send packed to data to through Send Receive Dbus 
   connection 

   @param  pack_str  string to be sent to remote 
   @param  func  from which function this method called ( Debug purpose) 
   @param  free_pack_str to free pack_str set this value to TRUE  
   @return on successful sending data returns OSSO_PACK_OK. In case of
    error returns OSSO_PACK_ERROR 
*/

static osso_pack_return_t
ui_dbus_send_on_send_recv_connection(gchar * pack_str, const gchar * func, gboolean free_pack_str)
{
	osso_pack_return_t rv = OSSO_PACK_OK;

	if ((pack_str == NULL)) {
		osso_log(LOG_INFO, "pack str is NULL \n");
		return OSSO_PACK_ERROR;
	}

	if (ui_dbus_connect_to_remote_for_send_recv(pack_str) == FALSE) {
		if (func) {
			osso_log(LOG_ERR, "Unable to connect remote %s", func);
		} else {
			osso_log(LOG_ERR, "Unable to connect remote ");
		}
		rv = OSSO_PACK_ERROR;
	}
	if (free_pack_str == TRUE) {
		g_free(pack_str);
	}
	return rv;
}

/**
   This fuction is used to send packed to data to through Dbus connection 

   @param  pack_str  string to be sent to remote 
   @param  func  from which function this method called ( Debug purpose) 
   @param  free_pack_str to free pack_str set this value to TRUE  
   @return on successful sending data returns OSSO_PACK_OK. In case of
    error returns OSSO_PACK_ERROR 
*/

static osso_pack_return_t
ui_dbus_send_on_normal_connection(gchar * pack_str, const gchar * func, gboolean free_pack_str)
{
	osso_pack_return_t rv = OSSO_PACK_OK;

	if ((pack_str == NULL)) {
		osso_log(LOG_INFO, "pack str is NULL \n");
		return OSSO_PACK_ERROR;
	}

	if (ui_dbus_connect_to_remote(pack_str) == FALSE) {
		if (func) {
			osso_log(LOG_ERR, "Unable to connect remote %s", func);
		} else {
			osso_log(LOG_ERR, "Unable to connect remote ");
		}
		rv = OSSO_PACK_ERROR;
	}
	if (free_pack_str == TRUE) {
		g_free(pack_str);
	}
	return rv;
}


/*
 *  Sending Functions Handling
 */

/**
    This Function is called by UI module to perfom send opertion
    @param MsgInfo paramaters needed for sending mail like to,cc,bcc
    @param Message is the Message Body
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode 
*/

osso_pack_return_t ui_dbus_send_mail(const MsgInfo * msginfo, const gchar * message)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;
	if (msginfo == NULL) {
		osso_log(LOG_INFO, " Received NULL values for msginfo\n");
		return OSSO_PACK_INVALID;
	}
	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_msginfo(root, doc, XML_MSG_INFO, (MsgInfo *) msginfo);
	if (message != NULL) {
		osso_email_xml_pack_str_base64(root, doc, XML_MESSAGE, &message);
	}
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(SEND_MAIL, (gchar *) xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
}

/**
    This Function is called by UI module to send outbox mails 
    @param account for which mail has to be sent
    @param msgid The outbox mailid which is to be sent 
    @param keepconnalive  keep the connection alive  
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode 
*/

osso_pack_return_t
ui_dbus_send_outbox_mail(const gchar * account, const gchar * msgid, gboolean keepconnalive)
{

	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if (account == NULL) {
		osso_log(LOG_INFO, "Received NULL value for account\n");
		return OSSO_PACK_INVALID;
	}

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, XML_STRING, &account);
	osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
	osso_email_xml_pack_bool(root, doc, XML_BOOL, &keepconnalive);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);

	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(SEND_OUTBOX_MAIL, xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
}

/**
    This Function is called by UI module to cancel selected mails 
    @param msgid_list GSList of mails to be canceled
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode 
*/

osso_pack_return_t ui_dbus_cancel_send_mail(const GSList * msgid_list)
{
	gchar *xmldata = NULL;
	gchar *str = NULL;

	osso_email_utils_pack_string_to_gslist((GSList *) msgid_list, &xmldata);
	str = osso_email_utils_concat_strings(CANCEL_SEND_MAIL, xmldata);
	xmlFree(xmldata);
	return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
}


/**
    This Function is called by UI module to perfom send opertion
    @param MsgInfo paramaters needed for sending mail like to,cc,bcc
    @param Message is the Message Body
    @flag  when it 0 mail to be saved in draft folder and 1 for to send 
    @flag  msgid  contains the new message id of mail saved in draft folder,  
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode 
*/

osso_pack_return_t
ui_dbus_send_save_draft_mail(const MsgInfo * msginfo,
			     const gchar * message, gboolean flag, gchar * old_id)
{
	osso_pack_return_t rv = OSSO_PACK_OK;
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if (msginfo == NULL) {
		osso_log(LOG_INFO, "Received NULL values for msginfo ");
		return OSSO_PACK_ERROR;
	}

	DBUS_XML_PACK_INIT;

	osso_email_xml_pack_msginfo(root, doc, XML_MSG_INFO, (MsgInfo *) msginfo);
	if (message != NULL) {
		osso_email_xml_pack_str_base64(root, doc, XML_MESSAGE, &message);
	}
	osso_email_xml_pack_bool(root, doc, XML_BOOL, &flag);

	if (old_id != NULL) {
		osso_email_xml_pack_str_base64(root, doc, XML_MSGID, &old_id);
	}
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(SEND_SAVE_DRAFT_MAIL, xmlbuff);
	xmlFree(xmlbuff);
	rv = ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return rv;
}

/**
    This Function is called by UI module to send selected mails from 
    specified folder
    @param folderType Folder from which Mails to be sent
    @param msgid_list GSList of mails  to be sent 
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode 
*/

osso_pack_return_t ui_dbus_send_selected_mails(gint folderType, const GSList * msgid_list)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;
	if (msgid_list == NULL) {
		osso_log(LOG_INFO, "Received NULL values for msgid list ");
		return OSSO_PACK_ERROR;
	}

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_msgid_list(root, doc, XML_MSG_ID_LST, (GSList *) msgid_list);
	osso_email_xml_pack_int(root, doc, XML_INT, &folderType);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(SEND_SELECTED_MAILS, xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
}

/**
    This Function is called by UI module to send selected mails from 
    specified folder
    @param msgid_list GSList of mimeinfo.   
    @param msgid_list GSList of attachment tags.   
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode 
*/

osso_pack_return_t
ui_dbus_recv_msg_multiple_attachments(const GSList * mime_list,
				      const GSList * attach_tag_list, const gchar * msgid)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;
	if (mime_list == NULL || attach_tag_list == NULL) {
		osso_log(LOG_INFO, "Received NULL values for mimeinfo/attach_tag_list list ");
		return OSSO_PACK_ERROR;
	}

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_mimeinfo_list(root, doc, XML_MIME_INFO_LST, (GSList *) mime_list);
	osso_email_xml_pack_int_list(root, doc, XML_ATTACH_TAG_LST, (GSList *) attach_tag_list);
	osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(RECV_MULTIPLE_ATTACHMENTS, xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
}

  /* 
   * Receiving Functions Handling
   */

/**
    This Function is called by UI module to receive mails for the 
    selected account 
    @param account Name of the account to perform receive operation 
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode 
*/

osso_pack_return_t ui_dbus_recv_mail(const gchar * account)
{
	gchar *str = NULL;

	if (account == NULL) {
		osso_log(LOG_INFO, "Received NULL Value for account\n");
		return OSSO_PACK_INVALID;
	}
	str = osso_email_utils_concat_strings(RECV_MAIL, account);
	return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
}

/**
    This Function is called by UI module to receive message body of the 
    mail
    @param  msgid for which body to recieve
    @param  fully_downloaded Message is fully downloaded or not 
    @param email_id of sender
    @param email id associated with current account
    @return MinmeInfo of given msgid on success or NULL of failure
*/

MimeInfo *ui_dbus_recv_msg_body(const gchar * msgid,
				const gboolean cbflag,
				gboolean complete_download,
				gchar * sender_id, gchar * recip_id,
				gint smime_type, gboolean read_flag,
				guint cert_id, gchar * password, gboolean edit_flag,
				gboolean get_html_flag)
{
	MimeInfo *ret_mime_info = NULL;
	gchar *str = NULL;

	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if (msgid == NULL) {
		osso_log(LOG_INFO, "Received NULL Value for msgid\n");
		return NULL;
	}

	ret_mimeinfo = NULL;

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
	osso_email_xml_pack_bool(root, doc, XML_BOOL, &complete_download);
	osso_email_xml_pack_str(root, doc, "SENDER_EMAIL", &sender_id);
	osso_email_xml_pack_str(root, doc, "RECIP_EMAIL", &recip_id);
	osso_email_xml_pack_int(root, doc, XML_INT, &smime_type);
	osso_email_xml_pack_bool(root, doc, CB_FLAG, (gboolean *) & cbflag);
	osso_email_xml_pack_bool(root, doc, READ_FLAG, &read_flag);
	osso_email_xml_pack_bool(root, doc, EDIT_FLAG, &edit_flag);
	osso_email_xml_pack_bool(root, doc, HTML_FLAG, &get_html_flag);
	osso_email_xml_pack_int(root, doc, CERT_ID, &cert_id);
	osso_email_xml_pack_str(root, doc, PASS, &password);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	if (cbflag == TRUE) {
		str = osso_email_utils_concat_strings(RECV_MSG_BODY_CB, (gchar *) xmlbuff);
	} else {
		str = osso_email_utils_concat_strings(RECV_MSG_BODY, (gchar *) xmlbuff);
	}

	xmlFree(xmlbuff);
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	ret_mime_info = ret_mimeinfo;
	return ret_mime_info;
}

/**
   This method is called when DBus module receives response contains the  
   message attachment of the requested message id.
   @param  recv_str contains funcid:xmldata <msginfo>
   @return MsgInfo on success or NULL of failure
*/

static MsgInfo *ui_dbus_recv_msg_attachment_callback(const gchar * recv_str)
{
	gchar *message = NULL;
	MsgInfo *msg_info = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return NULL);

	msg_info = g_new0(MsgInfo, 1);
	if (msg_info == NULL) {
		osso_log(LOG_ERR, "Mem Alloc failed \n");
		return NULL;
	}
	osso_email_utils_unpack_msgInfo(message, msg_info);
	return msg_info;
}

/**
    This Function is called by UI module to cancel receive operation
    @param account_name to cancel the receive opeartion of selected account 
    @param send_on_normal_conn Whether to send the request on normal connection
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode 
*/

osso_pack_return_t
ui_dbus_cancel_recv_mail(const gchar * account_name, gboolean send_on_normal_conn)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_strings(CANCEL_RECV_MAIL, account_name);
	if (send_on_normal_conn) {
		return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	} else {
		return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
	}
}

/*
 * Folder and Message Handling Functions
 */

/**
    This Function is called by UI module to create new folder
    @param name Name of the folder to be created
    @param path Path in which folder to be created
    @param Type of folder whether local or remote
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint ui_dbus_folder_create(const gchar * name, const gchar * path, FolderType type)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((name == NULL), "Folder name is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((path == NULL), "Folder path is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, FOLDER_NAME, &name);
	osso_email_xml_pack_str(root, doc, PATH, &path);
	osso_email_xml_pack_int(root, doc, TYPE, &type);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_CREATE, xmlbuff);
	xmlFree(xmlbuff);
	folder_return_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return folder_return_code;
}

/**
    This Function is called by UI module to delete folder
    @param name Name of the folder to be deleted
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint ui_dbus_folder_delete(const gchar * name)
{
	gchar *str = NULL;

	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((name == NULL), "Folder name is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, FOLDER_NAME, &name);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_DELETE, xmlbuff);
	xmlFree(xmlbuff);
	folder_return_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return folder_return_code;
}

/**
    This Function is called by UI module to rename folder
    @param old_name original name of the folder
    @param new_name of the folder
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/
gint ui_dbus_folder_rename(const gchar * old_name, const gchar * new_name)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((old_name == NULL), "Invalid folder name(null)", return OSSO_PACK_INVALID);
	CHECK_STATUS((new_name == NULL), "Invalid folder name(null)", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, FOLDER_OLD_NAME, &old_name);
	osso_email_xml_pack_str(root, doc, FOLDER_NEW_NAME, &new_name);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_RENAME, xmlbuff);
	xmlFree(xmlbuff);
	folder_return_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return folder_return_code;
}

/**
    This Function is called by UI module to move message from one folder
    to another folder
    @param srcfolder source from which message to be moved
    @param destfolder folder where message to be moved
    @param msgid message id of the message to be moved
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint ui_dbus_folder_move_msg(const gchar * srcfolder, const gchar * destfolder, const gchar * msgid)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((srcfolder == NULL), "Source folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((destfolder == NULL), "Dest folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((msgid == NULL), "Msgid is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, SRC_FOLDER, &srcfolder);
	osso_email_xml_pack_str(root, doc, DES_FOLDER, &destfolder);
	osso_email_xml_pack_str(root, doc, MSGID, &msgid);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_MOVE_MSG, xmlbuff);
	xmlFree(xmlbuff);
	folder_return_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return folder_return_code;
}

/**
    This Function is called by UI module to move multiple messages from 
    one folder to another folder
    @param src_folder source from which message to be moved
    @param destfolder folder where message to be moved
    @param msglist list of message id's to be moved
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint
ui_dbus_folder_move_msgs(const gchar * srcfolder,
			 const gchar * destfolder, const GSList * msgid_list)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((srcfolder == NULL), "Source folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((destfolder == NULL), "Dest folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((msgid_list == NULL), "MsgidList is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, SRC_FOLDER, &srcfolder);
	osso_email_xml_pack_str(root, doc, DES_FOLDER, &destfolder);
	osso_email_xml_pack_msgid_list(root, doc, XML_MSG_ID_LST, (GSList *) msgid_list);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_MOVE_MSGS, xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
}

/**
    This Function is called by UI module to remove message from folder
    @param folder folder from which message to be removed
    @param msgid message id of the message to be removed
    @param flag for whether to be remove message locally or locally and 
     remore sever also
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint ui_dbus_folder_remove_msg(const gchar * folder, const gchar * msgid, gboolean flag)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((folder == NULL), "folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((msgid == NULL), "Msgid is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, FOLDER_NAME, &folder);
	osso_email_xml_pack_str(root, doc, MSGID, &msgid);
	osso_email_xml_pack_bool(root, doc, FLAG, &flag);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_REMOVE_MSG, xmlbuff);
	xmlFree(xmlbuff);
	folder_return_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return folder_return_code;
}

/**
    This Function is called by UI module to remove messages from folder
    @param folder folder from which message to be removed
    @param msglist list of message id's to be removed
    @param flag for whether to be remove message locally or locally and 
     remore sever also
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint ui_dbus_folder_remove_msgs(const gchar * folder, const GSList * msgid_list, gboolean flag)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((folder == NULL), "folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((msgid_list == NULL), "Msgid is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, FOLDER_NAME, &folder);
	osso_email_xml_pack_msgid_list(root, doc, XML_MSG_ID_LST, (GSList *) msgid_list);
	osso_email_xml_pack_bool(root, doc, FLAG, &flag);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_REMOVE_MSGS, xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
}

/**
    This Function is called by UI module to remove all messages from folder
    @param folder folder from which message to be removed
    @param flag for whether to be remove message locally or locally and 
     remore sever also.
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint ui_dbus_folder_remove_all_msg(const gchar * folder, gboolean flag)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((folder == NULL), "folder is null", return OSSO_PACK_INVALID);
	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, FOLDER_NAME, &folder);
	osso_email_xml_pack_bool(root, doc, XML_BOOL, &flag);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_REMOVE_ALL_MSGS, xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
}

/**
    This Function is called by UI module to get the list of folders 
    @param folder_list GSList of folder names given by engine module
     populated in this list
    @return on returns folder errorcode 
*/

gint ui_dbus_folder_get_list(GSList ** folder_list)
{
	gchar *str = NULL;

	if (folder_list == NULL) {
		osso_log(LOG_INFO, "Invalid folder list \n");
		return OSSO_PACK_INVALID;
	}
	str = osso_email_utils_concat_strings(FOLDER_GET_LIST, seperator);
	folder_return_code = 0;
	ret_folder_list = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	*folder_list = ret_folder_list;
	return folder_return_code;
}

/**
    This Function is called by UI module to get the size of the folder 
    @param folder name of the folder to get the size
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint ui_dbus_folder_get_size(const gchar * folder)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((folder == NULL), "folder is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, FOLDER_NAME, &folder);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_GET_SIZE, xmlbuff);
	xmlFree(xmlbuff);
	folder_return_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return folder_return_code;
}

/**
    This Function is called by UI module to get the list of the messages
    headers existing in the selected folder. 
    @param name folder name of the folder to get the message header list
    @param index of the messages
    @return on returns GSList of message headers 
*/

GSList *ui_dbus_populate_folder(const gchar * name, gint index)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((name == NULL), "folder name is null", return NULL);
	CHECK_STATUS((index < 0), "Index is Invalid", return NULL);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, FOLDER_NAME, &name);
	osso_email_xml_pack_int(root, doc, INDEX, &index);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(POPULATE_FOLDER, (gchar *) xmlbuff);
	xmlFree(xmlbuff);
	msgheader_list = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return msgheader_list;
}

/**
    This Function is called by UI module to copy message from one folder 
    to other folder.
    @param srcfolder folder from which message is exist
    @param destfolder folder from which message to be copied
    @param msgid message id on which folder copy operation performed
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint ui_dbus_folder_copy_msg(const gchar * srcfolder, const gchar * destfolder, const gchar * msgid)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((srcfolder == NULL), "Source folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((destfolder == NULL), "Dest folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((msgid == NULL), "Msgid is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, SRC_FOLDER, &srcfolder);
	osso_email_xml_pack_str(root, doc, DES_FOLDER, &destfolder);
	osso_email_xml_pack_str(root, doc, MSGID, &msgid);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_COPY_MSG, xmlbuff);
	xmlFree(xmlbuff);
	folder_return_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return folder_return_code;
}

/**
    This Function is called by UI module to copy messages from one folder 
    to other folder.
    @param srcfolder folder from which message is exist
    @param destfolder folder from which message to be copied
    @param msgid_list list of message ids to be copied
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

gint
ui_dbus_folder_copy_msgs(const gchar * srcfolder,
			 const gchar * destfolder, const GSList * msgid_list)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((srcfolder == NULL), "Source folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((destfolder == NULL), "Dest folder is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((msgid_list == NULL), "MsgidList is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, SRC_FOLDER, &srcfolder);
	osso_email_xml_pack_str(root, doc, DES_FOLDER, &destfolder);
	osso_email_xml_pack_msgid_list(root, doc, XML_MSG_ID_LST, (GSList *) msgid_list);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(FOLDER_COPY_MSGS, xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
}

/**
    This Function is called by UI module to remove attachments in the mail 
    @param attach_list list of attachment names 
    @return on returns folder errorcode or returns OSSO_PACK_INVALID
     for invalid input parameters 
*/

GSList *ui_dbus_folder_remove_attachment(GSList * attach_list)
{
	gchar *xmlbuff = NULL;
	gchar *str = NULL;
	GSList *msgid_list = NULL;
	GSList *cur = NULL;
	FolderAttachInfo *folder_attachinfo = NULL;


	CHECK_STATUS((attach_list == NULL), "folder_attach list is null", return NULL);
	folder_attachinfo = attach_list->data;

	msgid_list = g_slist_append(msgid_list, g_strdup(folder_attachinfo->msgid));

	for (cur = attach_list; cur != NULL; cur = cur->next) {
		folder_attachinfo = cur->data;
		msgid_list = g_slist_append(msgid_list,
					    g_strdup(folder_attachinfo->section_number));

	}
	osso_email_utils_pack_msgid_to_gslist(msgid_list, &xmlbuff);
	str = osso_email_utils_concat_strings(FOLDER_REMOVE_ATTACHMENT, xmlbuff);
	xmlFree(xmlbuff);

	slist_free_strings(msgid_list);
	g_slist_free(msgid_list);

	ret_section_list = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return ret_section_list;
}

/**
    This Function is called by UI module to get the errorcode of latest
    error occured in engine module. 
    @return latest errorcode occured in engine
*/

gint ui_dbus_get_engine_errno(void)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_strings(GET_ENGINE_ERRNO, NULL);
	engine_error_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return engine_error_code;
}

/**
    This Function is called by UI module to get the engine initialisation
    status. This is called at the time of UI initailisation.
    @return status of engine 
*/

gint ui_dbus_get_engine_init_status(void)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_strings(GET_ENGINE_INIT_STATUS, NULL);
	engine_init_status = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return engine_init_status;
}

 /*
  * Account Mangement Functions
  */

/**
    This Function is called by UI module to get the list of accounts exist 
    @return GSList of account list given by engine 
*/

GSList *ui_dbus_get_account_list(void)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_strings(GET_ACCOUNT_LIST, NULL);
	ret_account_list = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return ret_account_list;
}

gint ui_dbus_account_check_status(const gchar * account)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if (account == NULL) {
		osso_log(LOG_INFO, " Invalid account name \n");
	}

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, ACCOUNT_NAME, &account);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(ACCOUNT_CHECK_STATUS, xmlbuff);
	xmlFree(xmlbuff);
	engine_acct_status = -1;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return engine_acct_status;
}

/**
    This Function is called by UI module to get the account settings of
    specified account 
    @param account name of the account which settings are required 
    @param type  specifies which type of account settings are required
    @prefs_account contains required account information on success
    @return on returns OSSO_PACK_OK on success or returns error code 
*/

osso_pack_return_t
ui_dbus_get_account_settings(const gchar * account, SettingType type, PrefsAccount ** prefs_account)
{
	gchar *str = NULL;
	osso_pack_return_t rv = OSSO_PACK_OK;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((account == NULL), "Account Name is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((type == 0), "Type is invalid", return OSSO_PACK_INVALID);
	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, ACCOUNT_NAME, &account);
	osso_email_xml_pack_int(root, doc, TYPE, &type);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(GET_ACCOUNT_SETTINGS, xmlbuff);
	xmlFree(xmlbuff);
	ret_prefs_account = NULL;
	rv = ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);

	if (prefs_account != NULL) {
		*prefs_account = ret_prefs_account;
	}
	return rv;
}

/**
    This Function is called by UI module to set the account settings of
    specified account 
    @param type  specifies which type of account settings are required
    @prefs_account contains required account information 
    @return on returns OSSO_PACK_OK on success or returns error code 
*/

osso_pack_return_t
ui_dbus_set_account_settings(SettingType type, const PrefsAccount * prefs_account)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((type == 0), "Type is invalid", return OSSO_PACK_INVALID);
	CHECK_STATUS((prefs_account == NULL), "Prefs Account NULL", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_int(root, doc, TYPE, &type);
	osso_email_xml_pack_prefsaccount(root, doc, XML_PREFS_ACCOUNT,
					 (PrefsAccount *) prefs_account);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(SET_ACCOUNT_SETTINGS, xmlbuff);
	xmlFree(xmlbuff);
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return account_error_code;
}

/**
    This Function is called by UI module to delete the account 
    @param account_name to be deleted
    @return on returns errorcode returned by engine  for account delete 
    opeartion or returns OSSO_PACK_INVALID for invalid input parameteres 
*/

osso_pack_return_t ui_dbus_delete_account(const gchar * account)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((account == NULL), "Account Name is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, ACCOUNT_NAME, &account);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(DELETE_ACCOUNT, xmlbuff);
	xmlFree(xmlbuff);
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return account_error_code;
}

/**
    This Function is called by UI module to rename the account 
    @param old_name  old account name to be renamed
    @param new_name  new account name 
    @return on returns errorcode returned by engine  for account delete 
    opeartion or returns OSSO_PACK_INVALID for invalid input parameteres 
*/

gint ui_dbus_rename_account(const gchar * old_name, const gchar * new_name)
{
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;
	gchar *str = NULL;

	CHECK_STATUS((old_name == NULL), "old account is null", return OSSO_PACK_INVALID);
	CHECK_STATUS((new_name == NULL), "new account is null", return OSSO_PACK_INVALID);

	if (strcmp(old_name, new_name) == 0) {
		return OSSO_PACK_OK;
	}
	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, OLD_ACCOUNT_NAME, &old_name);
	osso_email_xml_pack_str(root, doc, NEW_ACCOUNT_NAME, &new_name);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(RENAME_ACCOUNT, xmlbuff);
	xmlFree(xmlbuff);
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return account_error_code;
}

/**
    This Function is called when UI DBus module recieved status for folder
    operation performed
    @param recv_str funcid:folderstatus
    @return status of the folder operation in engine module 
*/

static gint ui_dbus_folder_return_code(const gchar * recv_str)
{
	gint error_code = 0;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);
	error_code = atoi(message);
	return error_code;
}

/**
    This Function is called when UI DBus module recieved status for account 
    operation performed
    @param recv_str funcid:status
    @return status of the account delete/rename operation in engine module 
*/

static gint ui_dbus_account_return_code(const gchar * recv_str)
{
	gint error_code = 0;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);
	error_code = atoi(message);
	return error_code;
}

/**
    This method is called when UI DBus module receives response for operation
    requested by UI module.
    @param message contains callbackfuncid:data 
    @return nothing 
*/

static void ui_dbus_callback_message_handler(const gchar * message)
{
	gchar *word = NULL;
	gchar word1[MEM_ALLOC_SIZE1];

	if (message == NULL) {
		osso_log(LOG_INFO, "Invalid values received for message \n");
		return;
	}

	word = strstr(message, seperator);
	if (word == NULL) {
		osso_log(LOG_INFO, "Got a NULL value in the string\n");
		return;
	}
	memset(word1, '\0', MEM_ALLOC_SIZE1);
	strncpy(word1, message, word - message);
	if (g_ascii_strcasecmp(word1, GET_ACCOUNT_LIST_CALLBACK) == 0) {
		ui_dbus_get_account_list_callback(message);
	} else if (g_ascii_strcasecmp(word1, GET_ACCOUNT_SETTINGS_CALLBACK) == 0) {
		ui_dbus_get_account_settings_callback(message);
	} else if (g_ascii_strcasecmp(word1, RENAME_ACCOUNT_CALLBACK) == 0) {
		account_error_code = ui_dbus_account_return_code(message);
	} else if (g_ascii_strcasecmp(word1, DELETE_ACCOUNT_CALLBACK) == 0) {
		account_error_code = ui_dbus_account_return_code(message);
	} else if (g_ascii_strcasecmp(word1, SET_ACCOUNT_CALLBACK) == 0) {
		account_error_code = ui_dbus_account_return_code(message);
	} else if (g_ascii_strcasecmp(word1, FOLDER_GET_LIST_CALLBACK) == 0) {
		ui_dbus_folder_get_list_callback(message);
	} else if (g_ascii_strcasecmp(word1, RECV_MSG_BODY_CALLBACK) == 0) {
		ret_mimeinfo = ui_dbus_recv_msg_body_callback(message);
	} else if (g_ascii_strcasecmp(word1, POPULATE_FOLDER_CALLBACK) == 0) {
		msgheader_list = ui_dbus_populate_folder_callback(message);
	} else if (g_ascii_strcasecmp(word1, GET_MSG_HEADER_LIST_CALLBACK) == 0) {
		msgheader_list = ui_dbus_populate_folder_callback(message);
	} else if (g_ascii_strcasecmp(word1, RECV_MSG_ATTACHMENT_CALLBACK) == 0) {
		ret_msg_info = ui_dbus_recv_msg_attachment_callback(message);
	} else if (g_ascii_strcasecmp(word1, RECV_MULTIPART_BODY_CALLBACK) == 0) {
		multipart_filename = ui_dbus_recv_multipart_body_callback(message);
	} else if (g_ascii_strcasecmp(word1, REMOVE_ATTACHMENT_CALLBACK) == 0) {
		ret_section_list = ui_dbus_remove_attach_callback(message);

	} else if (g_ascii_strcasecmp(word1, REMOVEALL_ATTACHMENT_CALLBACK) == 0) {
		ret_section_list = ui_dbus_remove_attach_callback(message);
	} else if (g_ascii_strcasecmp(word1, FOLDER_OPERATION_CALLBACK) == 0) {
		folder_return_code = ui_dbus_folder_return_code(message);
	} else if (g_ascii_strcasecmp(word1, GET_ENGINE_ERRNO_CALLBACK) == 0) {
		engine_error_code = ui_dbus_return_engine_error_code(message);
	} else if (g_ascii_strcasecmp(word1, GET_ENGINE_INIT_STATUS_CALLBACK)
		   == 0) {
		engine_init_status = ui_dbus_return_engine_init_status(message);
	} else if (g_ascii_strcasecmp(word1, HTML_TO_TEXT_CONVERTER_CALLBACK)
		   == 0) {
		ret_convertedtext = ui_dbus_return_html_to_text_converter(message);
	} else if (g_ascii_strcasecmp(word1, SEND_GET_MSG_ID_CALLBACK) == 0) {
		new_draft_msgid = ui_dbus_return_send_get_msg_id(message);
	} else if (g_ascii_strcasecmp(word1, AUTOSAVE_MAIL_CALLBACK) == 0) {
		engine_error_code = ui_dbus_return_autosave_mail(message);
	} else if (g_ascii_strcasecmp(word1, STATESAVE_READ_CALLBACK) == 0) {
		engine_error_code = ui_dbus_return_statesave_read(message);
	} else if (g_ascii_strcasecmp(word1, ACCOUNT_CHECK_STATUS_CALLBACK) == 0) {
		engine_acct_status = ui_dbus_return_account_check_status(message);
	} else if (g_ascii_strcasecmp(word1, PREFS_ACCOUNT_GET_LIST_CALLBACK) == 0) {
		ui_dbus_prefs_account_get_list_callback(message);
	} else if (g_ascii_strcasecmp(word1, RECV_MULTIPART_ATTACHMENT_CALLBACK) == 0) {
		ret_msg_info = ui_dbus_recv_msg_attachment_callback(message);
	}
#if CL_USE
	else if (g_ascii_strcasecmp(word1, GET_IAPINFO_LIST_CALLBACK) == 0) {
		ui_dbus_get_iapinfo_list_callback(message);
	} else if (g_ascii_strcasecmp(word1, SET_IAPINFO_SETTINGS_CALLBACK) == 0) {
		ui_dbus_set_iapinfo_callback(message);
	}
#endif
	return;
}

/**
    This Function is called when UI DBus module recieved the information 
    about new message of saved draft mail
    @param recv_str funcid: Func ID
    @return msgid of saved draft mail 
*/

static gchar *ui_dbus_return_send_get_msg_id(const gchar * recv_str)
{
	gchar *message = NULL;
	gchar *ret_string = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return NULL);
	ret_string = g_strdup(message);
	return ret_string;
}

/**
    This Function is called when UI DBus module recieved the converted html 
    to text data  
    @param recv_str contains funcid:html converted string  
    @return Text representation of HTML information
*/

static gchar *ui_dbus_return_html_to_text_converter(const gchar * recv_str)
{
	gchar *message = NULL;
	gchar *ret_string = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return NULL);

	ret_string = g_strdup(message);
	return ret_string;
}


/**
    This Function is called when UI DBus module recieved engine errorcode  
    @param recv_str contains funcid:engine latest error code
    @return latest errorcode in engine
*/

static gint ui_dbus_return_engine_error_code(const gchar * recv_str)
{
	gchar *message = NULL;
	gint errcode = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return 0);

	errcode = atoi(message);
	return errcode;
}

/**
    This Function is called when UI DBus module request for the status
    of account( whether in use or not) 
    @param recv_str contains funcid:account status
    @return status of account 
*/

static gint ui_dbus_return_account_check_status(const gchar * recv_str)
{
	gchar *message = NULL;
	gint acct_status = 0;
	GET_DATA_FROM_RECV_STRING(recv_str, message, return 0);
	acct_status = atoi(message);
	return acct_status;
}


/**
    This Function is called when UI DBus module recieved engine status  
    @param recv_str contains funcid:engine latest error code
    @return engine status at the of initialsation 
*/

static gint ui_dbus_return_engine_init_status(const gchar * recv_str)
{
	gchar *message = NULL;
	gint engine_status = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return 0);

	engine_status = atoi(message);

	return engine_status;
}


/**
    This Function is called when UI DBus module recieved account 
    information for request account
    @param recv_str contains funcid:prefsaccount 
    @return nothing
*/

static void ui_dbus_get_account_settings_callback(const gchar * recv_str)
{
	PrefsAccount *prefsAccount = NULL;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return);

	prefsAccount = g_new0(PrefsAccount, 1);
	if (prefsAccount == NULL) {
		osso_log(LOG_ERR, "Memory Alloc failed \n");
		return;
	}
	osso_email_utils_unpack_prefsAccount(message, prefsAccount);
	ret_prefs_account = prefsAccount;

	return;
}

/**
    This Function is called when UI DBus module received account list
    @param recv_str contains funcid:GSList of account names
    @return nothing
*/

static void ui_dbus_get_account_list_callback(const gchar * recv_str)
{
	GSList *out_account_list = NULL;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return);

	osso_email_utils_unpack_string_in_gslist(message, &out_account_list);

	ret_account_list = out_account_list;
}

/**
    This Function is called when UI DBus module received message header
    list of specific folder
    @param recv_str contains funcid:GSList of message headers
    @return nothing
*/

GSList *ui_dbus_populate_folder_callback(const gchar * recv_str)
{
	gchar *message = NULL;
	GSList *populate_folder_list = NULL;


	GET_DATA_FROM_RECV_STRING(recv_str, message, return NULL);
	osso_email_utils_unpack_populate_folder_to_gslist(message, &populate_folder_list);

	return populate_folder_list;
}

/**
    This Function is called when UI DBus module received mime information 
    of the specific message
    @param   recv_str contains func_identifer:MimeInfo
    @return  MimeInfo of selected message 
*/

static MimeInfo *ui_dbus_recv_msg_body_callback(const gchar * recv_str)
{
	gchar *message = NULL;
	MimeInfo *mimeinfo = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return NULL);
	mimeinfo = osso_email_utils_unpack_mimeInfo(message);
	return mimeinfo;
}


/**
    This Function is called when UI DBus module received multipart body 
    inforamtion of the specific message
    @param   recv_str contains func_identifer:xmldata<filename>
    @return  filename of multipartbody
*/

static gchar *ui_dbus_recv_multipart_body_callback(const gchar * recv_str)
{
	gchar *message = NULL;
	gchar *filename = NULL;
	xmlNodePtr cur = NULL, node = NULL;
	xmlDocPtr doc = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return NULL);

	DBUS_XML_UNPACK_INIT(message, return NULL);
	node = cur->xmlChildrenNode;
	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, XML_STRING) == 0) {
				osso_email_xml_unpack_str(node, doc, XML_STRING, &filename);
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;
	return filename;
}

/**
    This Function is called when UI DBus module received list of folders 
    inforamtion 
    @param recv_str contains funcid:GSList of folders 
    @return nothing
*/
static void ui_dbus_folder_get_list_callback(const gchar * recv_str)
{
	GSList *out_folder_list = NULL;
	gint status = 0;
	xmlNodePtr cur = NULL, node = NULL;
	xmlDocPtr doc = NULL;
	gchar *message = NULL;
	/*
	 * Skip the func_identifier which is in the beginning the recv_str.
	 * Format of the recv_str is: func_identifer:GSList (Folder List)
	 */

	ret_folder_list = NULL;
	GET_DATA_FROM_RECV_STRING(recv_str, message, return);

	DBUS_XML_UNPACK_INIT(message, return);
	node = cur->xmlChildrenNode;
	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, STATUS) == 0) {
				osso_email_xml_unpack_int(node, doc, STATUS, &status);
				continue;
			}
			if (g_ascii_strcasecmp(node->name, XML_FOLDER_LST) == 0) {
				osso_email_xml_unpack_folder_list(node, doc, XML_FOLDER_LST,
								  &out_folder_list);
				continue;
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;
	folder_return_code = status;
	ret_folder_list = out_folder_list;
}


/**
     This method is called for conecting to another dbus server for 
     send recv operations
     @param msg_to_send message to be sent to engine DBUS module
     @return TRUE on success or FALSE on failure
*/

static gboolean ui_dbus_connect_to_remote_for_send_recv(const gchar * msg_to_send)
{
	DBusMessage *message = NULL;
	DBusError error;
	guint ret = 0;
	gint succ = 0;
	static gboolean connection_flag = FALSE;

	CHECK_STATUS((msg_to_send == NULL), "Invalid msg send to remote", return FALSE);
	dbus_error_init(&error);

	if (connection_flag == FALSE) {
		send_recv_connection =
		    dbus_connection_open(DBUS_UNIX_SOCKET_SEND_RECV_PATH, &error);
		if (send_recv_connection == NULL) {
			osso_log(LOG_ERR, "could not open connection:\n");
			dbus_error_free(&error);
			return FALSE;
		}
		connection_flag = TRUE;
	}

	if (send_recv_connection != NULL) {
		dbus_connection_setup_with_g_main(send_recv_connection, NULL);
		message = dbus_message_new_method_call(EMAIL_DBUS_SERVICE,
						       EMAIL_DBUS_PATH,
						       EMAIL_DBUS_INTERFACE, EMAIL_DBUS_METHOD);
		if (message == NULL) {
			osso_log(LOG_INFO, "Could not allocate DBUS message\n");
			return FALSE;
		}
		succ = dbus_message_append_args(message, DBUS_TYPE_STRING,
						&msg_to_send, DBUS_TYPE_INVALID);
		if (succ == FALSE) {
			osso_log(LOG_INFO, " Unable to append args to message \n");
			dbus_message_unref(message);
			return FALSE;
		}
		succ = dbus_connection_send(send_recv_connection, message, &ret);
		dbus_connection_flush(connection);

		dbus_message_unref(message);
		if (!succ) {
			osso_log(LOG_ERR, "Sending of message Failed\n");
			return FALSE;
		}
	}
	return TRUE;
}

/**
     This method is called for conecting to another dbus server for 
     sending dbus messages
     @param msg_to_send message to be sent to engine DBUS module
     @return TRUE on success or FALSE on failure
*/

static gboolean ui_dbus_connect_to_remote(const gchar * msg_to_send)
{
	DBusMessage *message = NULL, *reply = NULL;
	DBusError error;
	guint ret = 0;
	gint succ = 0, idx = 0;
	gboolean cbRequired = FALSE;
	gchar *reply_string = NULL;
	static gboolean connection_flag = FALSE;
	gchar **func_message = NULL;

	CHECK_STATUS((msg_to_send == NULL), "Invalid msg send to remote", return FALSE);
	func_message = g_strsplit(msg_to_send, seperator, -1);
	dbus_error_init(&error);

	if (connection_flag == FALSE) {
		connection = dbus_connection_open(DBUS_UNIX_SOCKET_CLNT_PATH, &error);
		if (connection == NULL) {
			osso_log(LOG_ERR, "could not open connection:\n");
			dbus_error_free(&error);
			g_strfreev(func_message);
			return FALSE;
		}
		connection_flag = TRUE;
	}

	if (connection != NULL) {
		dbus_connection_setup_with_g_main(connection, NULL);
		message = dbus_message_new_method_call(EMAIL_DBUS_SERVICE,
						       EMAIL_DBUS_PATH,
						       EMAIL_DBUS_INTERFACE, EMAIL_DBUS_METHOD);
		if (message == NULL) {
			osso_log(LOG_INFO, "Could not allocate DBUS message\n");
			g_strfreev(func_message);
			return FALSE;
		}
		succ = dbus_message_append_args(message, DBUS_TYPE_STRING,
						&msg_to_send, DBUS_TYPE_INVALID);
		if (succ == FALSE) {
			osso_log(LOG_INFO, " Unable to append args to message \n");
			dbus_message_unref(message);
			g_strfreev(func_message);
			return FALSE;
		}


		for (idx = 0; cbRequiredFuncList[idx].hname != NULL; idx++) {
			if (g_ascii_strncasecmp(func_message[0],
						cbRequiredFuncList[idx].hname,
						MAX_STR_LEN(func_message[0],
							    cbRequiredFuncList[idx].hname)
			    ) == 0) {
				cbRequired = TRUE;
				break;
			}
		}

		if (cbRequired == TRUE) {
			reply =
			    dbus_connection_send_with_reply_and_block(connection,
								      message,
								      DBUS_TIME_WAIT, &error);
			dbus_message_unref(message);
			if (dbus_error_is_set(&error)) {
				osso_log(LOG_INFO, "%s call back error :%s\n",
					 func_message[0], error.message);
				g_strfreev(func_message);
				dbus_error_free(&error);
				return FALSE;
			}
			g_strfreev(func_message);

			if (reply == NULL) {
				osso_log(LOG_INFO, "Unable to get reply from the remote"
					 "server %s\n", error.message);
				dbus_error_free(&error);
				return FALSE;
			} else {
				dbus_message_get_args(reply, &error, DBUS_TYPE_STRING,
						      &reply_string, DBUS_TYPE_INVALID);
				if (reply_string != NULL) {
					osso_log(LOG_INFO, "We got the reply\n");
					ui_dbus_callback_message_handler(reply_string);
					dbus_message_unref(reply);
					return TRUE;
				}
				dbus_message_unref(reply);
				return FALSE;
			}
		} else {
			g_strfreev(func_message);
			succ = dbus_connection_send(connection, message, &ret);
			dbus_connection_flush(connection);
			dbus_message_unref(message);
			if (!succ) {
				osso_log(LOG_ERR, "Sending of message Failed\n");
				return FALSE;
			}
		}
	}
	return TRUE;
}


/* Recv Functionality starts here */

/**
    This function is called when UI DBus module receives message body
    complete callback information from engine.
    @param message  funcid:xmldata<mimeinfo:msgid> 
    @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

osso_pack_return_t ui_dbus_pop3_recv_body_complete(const gchar * recv_str)
{
	gchar *message = NULL, *msgid = NULL;
	MimeInfo *mimeinfo = NULL;
	gboolean remote_copy = FALSE;

	xmlNodePtr cur = NULL, node = NULL;
	xmlDocPtr doc = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	DBUS_XML_UNPACK_INIT(message, return OSSO_PACK_ERROR);

	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp((gchar *) node->name, XML_MIME_INFO) == 0) {

				mimeinfo = g_new0(MimeInfo, 1);
				if (mimeinfo == NULL) {
					osso_log(LOG_ERR,
						 "Unable to allocate memory to mimeinfo \n");
					DBUS_XML_UNPACK_UN_INIT;
					return OSSO_PACK_ALLOC_FAIL;
				}

				osso_email_xml_unpack_mimeinfo(node, doc, XML_MIME_INFO, mimeinfo);
				continue;
			}
			if (g_ascii_strcasecmp((gchar *) node->name, XML_MSG_ID) == 0) {
				osso_email_xml_unpack_str(node, doc, XML_MSG_ID, &msgid);
				continue;
			}
			if (g_ascii_strcasecmp((gchar *) node->name, XML_BOOL) == 0) {
				osso_email_xml_unpack_bool(node, doc, XML_INT, &remote_copy);
				break;
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;

	if (msgid != NULL) {
		pop3_recv_body_complete_cb(mimeinfo, msgid, remote_copy);
	}
	g_free(msgid);

	return OSSO_PACK_OK;
}


/**
    This function is called when UI module wants get attachment from 
    specific message 
    @param msgid message id for which attachment to be downloaded
    @param mimeinfo Mime Information of message 
    @return Message Information for message
*/

MsgInfo *ui_dbus_recv_msg_attachment(const gchar * msgid, const MimeInfo * mimeinfo)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if ((msgid == NULL) || (mimeinfo == NULL)) {
		osso_log(LOG_INFO, "Received NULL for msgid or mimeinfo \n");
		return NULL;
	}
	DBUS_XML_PACK_INIT;

	osso_email_xml_pack_mimeinfo(root, doc, XML_MIME_INFO, (MimeInfo *) mimeinfo);
	osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);

	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(RECV_MSG_ATTACHMENT, xmlbuff);
	xmlFree(xmlbuff);
	ret_msg_info = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return ret_msg_info;
}

/**
    This function is called when UI module wants get file name of multipart
    of  specific message 
    @param mimeinfo Mime Information of message 
    @param msgid message id for which attachment to be downloaded
    @return filename of the multipart body received 
*/

gchar *ui_dbus_recv_multipart_body(const MimeInfo * mimeinfo,
				   const gchar * msgid, const gchar * default_font)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if ((msgid == NULL) || (mimeinfo == NULL) || default_font == NULL) {
		osso_log(LOG_INFO, "Received NULL for msgid or mimeinfo \n");
		return NULL;
	}

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_mimeinfo(root, doc, XML_MIME_INFO, (MimeInfo *) mimeinfo);
	osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
	osso_email_xml_pack_str(root, doc, XML_STRING, &default_font);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(RECV_MULTIPART_BODY, xmlbuff);
	xmlFree(xmlbuff);
	multipart_filename = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return multipart_filename;
}

/**
     This method is called by UI module for converting HTML to text foramt
     @param filename filename for HTML mail
     @param charset character set for message
     @param convertedtext file name  which contains converted text from HTML
     @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

osso_pack_return_t
ui_dbus_html_to_text_converter(const char *filename, const gchar * charset, gchar ** convertedtext)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if ((filename == NULL) || (convertedtext == NULL)) {
		osso_log(LOG_INFO, "Received NULL Values\n");
		return OSSO_PACK_INVALID;
	}

	DBUS_XML_PACK_INIT;

	osso_email_xml_pack_str(root, doc, XML_FILE_NAME, &filename);
	osso_email_xml_pack_str(root, doc, XML_CHARSET, &charset);
	osso_email_xml_pack_str(root, doc, XML_STRING, convertedtext);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);

	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(HTML_TO_TEXT_CONVERTER, xmlbuff);
	xmlFree(xmlbuff);
	ret_convertedtext = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);

	if (*convertedtext != NULL) {
		g_free(*convertedtext);
		*convertedtext = NULL;
	}
	*convertedtext = ret_convertedtext;
	return OSSO_PACK_OK;
}

/*\@}*/
/**
 * \defgroup  UI interaction to Email Manager 
 */
/*\@{*/

/*\@}*/


/**
   This Metod executed when Email Manager wants to inform Email UI application
   about the window id
   @param  id  Window id ( message id of window to be displayed)
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

static osso_pack_return_t ui_dbus_tasknav_window_id(const gchar * recv_str)
{
	gchar *message = NULL;
	gchar *windowid = NULL;
	xmlNodePtr cur = NULL;
	xmlNodePtr node = NULL;
	xmlDocPtr doc = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	DBUS_XML_UNPACK_INIT(message, return OSSO_PACK_ERROR);

	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp((gchar *) node->name, XML_STRING) == 0) {
				osso_email_xml_unpack_str(node, doc, XML_STRING, &windowid);
				break;
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;
	tasknav_window_id(windowid);
	return OSSO_PACK_OK;
}

/**
   This fuction is called when ui dbus receives a trigger to display
   Editor window. This request is given by Email Manager wants to
   invoke UI to open editior window(Send As Functionality)/
   @param  recv_str XML formatted string contains fnid:msginfo:message  
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

osso_pack_return_t ui_dbus_editor_create(const gchar * recv_str)
{
	MsgInfo *msginfo = NULL;
	gchar *msgbody = NULL;
	gchar *temp = NULL;
	xmlNodePtr cur = NULL;
	xmlNodePtr node = NULL;
	xmlDocPtr doc = NULL;

	CHECK_STATUS((recv_str == NULL), "Recv String is NULL", return OSSO_PACK_ERROR);

	/*
	 * Skip the func_identifier which is in the beginning of the recv_str.
	 * Format of the recv_str is : func_identifer:xmldata(msginfo, message)
	 */

	temp = strstr(recv_str, seperator);
	if ((temp == NULL) || (strlen(temp) == 0)) {
		osso_log(LOG_INFO, "Recv String contains wrong data");
		return OSSO_PACK_ERROR;
	}
	temp++;


	DBUS_XML_UNPACK_INIT(temp, return OSSO_PACK_ERROR);

	msginfo = g_new0(MsgInfo, 1);
	if (msginfo == NULL) {
		osso_log(LOG_ERR, "Memory Alloc Fail");
		DBUS_XML_UNPACK_UN_INIT;
		return OSSO_PACK_ALLOC_FAIL;
	}

	node = cur->xmlChildrenNode;
	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp((gchar *) node->name, XML_MSG_INFO) == 0) {
				osso_email_xml_unpack_msginfo(node, doc, XML_MSG_INFO, msginfo);
			}
			if (g_ascii_strcasecmp((gchar *) node->name, XML_MESSAGE) == 0) {
				osso_email_xml_unpack_str(node, doc, XML_MESSAGE, &msgbody);
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;

	if (msginfo != NULL) {
		editor_create_cb(msginfo, msgbody);
	}
	return OSSO_PACK_OK;
}

/**
   This fuction is called when ui dbus receives a request to open/delete 
   message from tasknavigator. If the message id is null for opening, it 
   opens a new mail. 
   @param  recv_str XML formatted string contains fnid:msgid:option 
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

static osso_pack_return_t ui_dbus_tasknav_msg_operation(const gchar * recv_str)
{
	gchar *msgid = NULL;
	gint option = 0;
	gchar *message = NULL;
	xmlNodePtr cur = NULL;
	xmlNodePtr node = NULL;
	xmlDocPtr doc = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	DBUS_XML_UNPACK_INIT(message, return OSSO_PACK_ERROR);

	node = cur->xmlChildrenNode;
	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, XML_MSG_ID) == 0) {
				osso_email_xml_unpack_str(node, doc, XML_MSG_ID, &msgid);
				continue;
			}
			if (g_ascii_strcasecmp(node->name, XML_INT) == 0) {
				osso_email_xml_unpack_int(node, doc, XML_INT, &option);
				continue;
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;
	tasknav_open_msg_cb(msgid);
	return OSSO_PACK_OK;
}

/**
   This fuction is called when ui dbus receives a trigger to display

   @param  recv_str XML formatted string contains fnid:type 
   @return on success returns TRUE or FAIL on fail
*/

static gboolean ui_dbus_open_inbox(const gchar * recv_str)
{
	tasknav_open_inbox();
	return TRUE;
}

/**
   This fuction is called to to backup autosave mail in ~/email/tmp folder 
   and restore it back. 

   @param  msgid message id of the mail to be saved
   @param  operation Backup(1) and restore(2) 
   @return status of auto save operation returned by engine or 
    OSSO_PACK_INVALID for null msgid
*/

gint ui_dbus_autosave_mail(const gchar * msgid, gint operation)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if (msgid == NULL) {
		osso_log(LOG_INFO, "Received NULL value for msgid\n");
		return OSSO_PACK_INVALID;
	}

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
	osso_email_xml_pack_int(root, doc, XML_INT, &operation);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(AUTOSAVE_MAIL, xmlbuff);
	xmlFree(xmlbuff);
	engine_error_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return engine_error_code;
}


/**
    This is callbvack function to handle the errorcode for autosave mail 
    @param recv_str contains the funcid:errorcode 
    @return error code  while performing the autosave mail 
*/
static gint ui_dbus_return_autosave_mail(const gchar * recv_str)
{
	gint error_code = 0;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);
	error_code = atoi(message);
	return error_code;
}


/**
    Function for getting the message header list for the given msgidlist
    @param msgid_list GSList of msgid's
    @return GSList of message header list for requested msgid list
*/

GSList *ui_dbus_get_msgheader_list(GSList * msgid_list)
{
	gchar *xmlbuff = NULL;
	gchar *str = NULL;

	CHECK_STATUS((msgid_list == NULL), "Null Msg header ", return NULL);

	osso_email_utils_pack_msgid_to_gslist(msgid_list, &xmlbuff);
	str = osso_email_utils_concat_strings(GET_MSG_HEADER_LIST, xmlbuff);
	xmlFree(xmlbuff);
	msgheader_list = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return msgheader_list;
}

/**
    Function to convert the given to base64 format string to normal string 
    @param  base64data string which is to converted in base64 format 
    @param  convdata converted data for given input string 
    @return nothing 
*/
void ui_dbus_unpack_str_base64(const guchar * base64data, gchar ** convdata)
{
	osso_email_utils_unpack_base64_to_str(base64data, convdata);
	return;
}


/**
   This function is called when UI open/close a message for editing purpose.
   The message id added or deleted to list maintained by engine.  
   So that when send/receive operation performed mails opened for 
   editing will not be sent. 

   @param  msgid message id of the mail opened by editor 
   @param  operation delete(0) and add(2) 
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

osso_pack_return_t ui_dbus_editor_open_mail(const gchar * msgid, gint operation)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if (msgid == NULL) {
		osso_log(LOG_INFO, "Received NULL value for msgid\n");
		return OSSO_PACK_INVALID;
	}

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
	osso_email_xml_pack_int(root, doc, XML_INT, &operation);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(EDITOR_OPEN_MAIL, xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
}




/**
 * \defgroup statesaving support from ui dbus 
 * 
 * @{
 *
 * This functions provide support for state saving operations from UI 
 */

/**
   This Method executed when UI wants to save its statesave information
   @param  stateInfo contains the state information to be saved.
           UI may want to reset previous state by sending this as NULL.
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/
osso_pack_return_t ui_dbus_statesave_write(const EmailStateInfo * stateInfo)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_emailStateInfo(root, doc, XML_EMAIL_STATE_INFO, (void *) stateInfo);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;
	str = osso_email_utils_concat_strings(STATESAVE_WRITE, xmlbuff);
	xmlFree(xmlbuff);
	return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
}

/**
   This Method executed when UI wants to save its persistent information
   @param  perInfo contains the Persistent information to be saved 
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/
osso_pack_return_t ui_dbus_persistentInfo_write(EmailPersistentInfo * perInfo)
{
	if (perInfo == NULL) {
		osso_log(LOG_INFO, "Received NULL value for pesistent info\n");
		return OSSO_PACK_INVALID;
	}
	return dbus_utils_persistentInfo_write_to_file(perInfo);
}

/**
   This Method executed when UI wants to save its persistent information
   @param  perInfo contains the Persistent information to be saved 
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/
osso_pack_return_t ui_dbus_persistentInfo_read(EmailPersistentInfo ** perInfo)
{
	EmailPersistentInfo *pinfo = NULL;
	osso_pack_return_t rv = OSSO_PACK_ERROR;

	if (perInfo == NULL) {
		osso_log(LOG_INFO, "Received NULL value \n");
		return OSSO_PACK_INVALID;
	}
	pinfo = g_new0(EmailPersistentInfo, 1);
	pinfo->mainview_pane_pos = -1;

	rv = dbus_utils_persistentInfo_read_from_file(pinfo);
	if (rv != OSSO_PACK_OK) {
		g_free(pinfo);
		pinfo = NULL;
	}
	*perInfo = pinfo;
	return rv;
}

/**
   This Method executed when UI wants to get its statesave information

   @param  stateInfo contains the state information returned by engine 
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

osso_pack_return_t ui_dbus_statesave_read(EmailStateInfo ** stateInfo)
{
	gchar *str = NULL;
	if (stateInfo == NULL) {
		return OSSO_PACK_ERROR;
	}
	ret_emailStateInfo = NULL;
	str = osso_email_utils_concat_str_int(STATESAVE_READ, TRUE);
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	*stateInfo = ret_emailStateInfo;
	return OSSO_PACK_OK;
}

/**
   This Method is called when UI Bus receives statesave information from engine. 
   @param recv_str contains the funcid:xmldata<statesave information>
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

static osso_pack_return_t ui_dbus_return_statesave_read(const gchar * recv_str)
{
	gchar *message = NULL;
	xmlNodePtr cur = NULL, node = NULL;
	xmlDocPtr doc = NULL;
	EmailStateInfo *stateInfo = NULL;

	ret_emailStateInfo = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	DBUS_XML_UNPACK_INIT(message, return OSSO_PACK_ERROR);
	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, XML_EMAIL_STATE_INFO) == 0) {
				stateInfo = g_new0(EmailStateInfo, 1);
				if (stateInfo == NULL) {
					osso_log(LOG_ERR, " Malloc failed \n");
					DBUS_XML_UNPACK_UN_INIT;
					return OSSO_PACK_ALLOC_FAIL;
				}
				osso_email_xml_unpack_emailStateInfo(node, doc, XML_INT, stateInfo);
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;

	ret_emailStateInfo = stateInfo;
	return OSSO_PACK_OK;
}

/**
   This Method executed from email manager to request the UI to save its
   statesave information. 

   @param  recv_str contains the statesave signal id information  
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

static gint ui_dbus_statesave_signal(const gchar * recv_str)
{
	osso_log(LOG_INFO, "Statesave signal(oom) received \n");
	oom_signal_recv();
	return OSSO_PACK_OK;
}


#if CL_USE

/**
   This Method is called by UI module to get the iap list
   @return  GSList of iap info list given by engine module
*/

GSList *ui_dbus_get_iapinfo_list(void)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_strings(GET_IAPINFO_LIST, NULL);
	ret_iapinfo_list = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return ret_iapinfo_list;
}

/**
   This Method is called when UI Bus receives iap list information 
   from engine. 
   @param recv_str contains funcid:xmldata<iapinfo list>
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

static void ui_dbus_get_iapinfo_list_callback(const gchar * recv_str)
{
	GSList *iapinfo_list = NULL;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return);
	osso_email_utils_unpack_iapinfo_list(message, &iapinfo_list);
	ret_iapinfo_list = iapinfo_list;
}

/**
   This Method is called by UI module to set the iap info 
*/

gboolean ui_dbus_set_iapinfo(IapInfo * iapinfo)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if (!iapinfo) {
		return FALSE;
	}
	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_iapinfo(root, doc, XML_IAPINFO, iapinfo);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(SET_IAPINFO_SETTINGS, xmlbuff);
	xmlFree(xmlbuff);
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return engine_error_code;	/*TBD */
}

/**
   This Method is called when UI Bus receives status iap set operation 
   from engine. 
   @param recv_str contains funcid:status
   @return none 
*/

static void ui_dbus_set_iapinfo_callback(const gchar * recv_str)
{
	gchar *message = NULL;
	GET_DATA_FROM_RECV_STRING(recv_str, message, return);
	set_iap_status = atoi(message);
}

#endif

  /*
   * mmc Callback Handling
   */

/**
   This Method is called by UI DBus module when DBus message for mmc
   status (mount/unmount) received.
   @param recv_str contains funcid:mmc status code
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

static gint ui_dbus_mmc_hlr(const gchar * recv_str)
{
	ParamMissing param_number = 0;
	gchar *message = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	osso_log(LOG_INFO, "mmc_hlr message : %s \n", message);
	param_number = atoi(message);
	mmc_hlr_cb(param_number);
	return OSSO_PACK_OK;
}

/**
   This Method is called to parse the smtp, imap4, pop3 set_msg_cb arguments
   and populate them in a GArray.
   @param msg xml packed data contains the arguments 
   @param msgnumber status/error code to be retrived from given input string 
   @param ret_val GArray of osso_rpc_t elements populated from input string 
   @param index length of GArray elements 
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

static osso_pack_return_t
ui_dbus_parse_set_msg_cb_args(const gchar * msg, gint * msgnumber, GArray ** ret_val, gint * index)
{
	GArray *arguments = NULL;
	osso_rpc_t element[MAX_SET_MSG_CB_ARGS] = { {0} };
	xmlNodePtr cur = NULL, node = NULL;
	xmlDocPtr doc = NULL;
	gint idx = 0;

	if ((msg == NULL) || (msgnumber == NULL) || (ret_val == NULL)
	    || (index == NULL)) {
		osso_log(LOG_INFO, "Invalid arguments \n");
		return OSSO_PACK_ERROR;
	}

	*index = 0;
	DBUS_XML_UNPACK_INIT(msg, return OSSO_PACK_ERROR);
	arguments = g_array_new(FALSE, FALSE, sizeof(osso_rpc_t));
	if (arguments == NULL) {
		osso_log(LOG_ERR, " Memory alloc failed \n");
		DBUS_XML_UNPACK_UN_INIT;
		return OSSO_PACK_ERROR;
	}
	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, STATUS) == 0) {
				osso_email_xml_unpack_int(node, doc, XML_INT, msgnumber);
				continue;
			}
			if (g_ascii_strcasecmp(node->name, XML_STRING) == 0) {
				osso_email_xml_unpack_str(node, doc, XML_STRING,
							  &element[idx].value.s);
				element[idx].type = DBUS_TYPE_STRING;
				g_array_append_val(arguments, element[idx]);
				idx++;
				continue;
			}
			if (g_ascii_strcasecmp((gchar *) node->name, XML_INT) == 0) {
				element[idx].type = DBUS_TYPE_INT32;
				osso_email_xml_unpack_int(node, doc, XML_INT,
							  &element[idx].value.i);
				g_array_append_val(arguments, element[idx]);
				idx++;
				continue;
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;

	*index = idx;
	*ret_val = arguments;
	return OSSO_PACK_OK;
}


/**
   This Method is called by UI DBus to update engine that UI is ready 
   @return on successful operation returns OSSO_PACK_OK. Otherwise 
     returns errorcode
*/

osso_pack_return_t ui_dbus_is_ready(void)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_str_int(UI_IS_READY, TRUE);
	osso_email_gconf_unset(EMAIL_GCONF_UI_CLOSED);
	return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
}

gint ui_dbus_prefs_account_get_list(GSList ** prefs_account_list)
{
	gchar *str = NULL;
	if (prefs_account_list == NULL) {
		osso_log(LOG_INFO, "Invalid prefs account list \n");
		return OSSO_PACK_INVALID;
	}
	str = osso_email_utils_concat_strings(PREFS_ACCOUNT_GET_LIST, seperator);
	account_error_code = 0;
	ret_prefs_account_list = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	*prefs_account_list = ret_prefs_account_list;
	return account_error_code;
}
static void ui_dbus_prefs_account_get_list_callback(const gchar * recv_str)
{
	GSList *out_prefs_account_list = NULL;
	gint status = 0;
	xmlNodePtr cur = NULL, node = NULL;
	xmlDocPtr doc = NULL;
	gchar *message = NULL;
	/*
	 * Skip the func_identifier which is in the beginning the recv_str.
	 * Format of the recv_str is: func_identifer:GSList (Folder List)
	 */

	ret_prefs_account_list = NULL;
	GET_DATA_FROM_RECV_STRING(recv_str, message, return);

	DBUS_XML_UNPACK_INIT(message, return);
	node = cur->xmlChildrenNode;
	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, STATUS) == 0) {
				osso_email_xml_unpack_int(node, doc, STATUS, &status);
				continue;
			}
			if (g_ascii_strcasecmp(node->name, XML_PREFS_ACCOUNT_LST) == 0) {
				osso_email_xml_unpack_prefsaccount_list(node, doc,
									XML_PREFS_ACCOUNT_LST,
									&out_prefs_account_list);
				continue;
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;
	account_error_code = status;
	ret_prefs_account_list = out_prefs_account_list;
}
MsgInfo *ui_dbus_recv_multipart_attachment(const gchar * msgid,
					   const MimeInfo * mimeinfo, gint attach_flag,
					   const gchar * default_font)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if ((msgid == NULL) || (mimeinfo == NULL)) {
		osso_log(LOG_INFO, "Received NULL for msgid or mimeinfo \n");
		return NULL;
	}

	DBUS_XML_PACK_INIT;

	osso_email_xml_pack_mimeinfo(root, doc, XML_MIME_INFO, (MimeInfo *) mimeinfo);
	osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
	osso_email_xml_pack_int(root, doc, ATTA_TAG, &attach_flag);
	osso_email_xml_pack_str(root, doc, XML_STRING, &default_font);

	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);

	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(RECV_MULTIPART_ATTACHMENT, xmlbuff);
	xmlFree(xmlbuff);
	ret_msg_info = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return ret_msg_info;
}
static GSList *ui_dbus_remove_attach_callback(const gchar * recv_str)
{
	gchar *message = NULL;
	GSList *section_list = NULL;
	xmlNodePtr cur = NULL, node = NULL;
	xmlDocPtr doc = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return NULL);
	DBUS_XML_UNPACK_INIT(message, return NULL);

	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, XML_MSG_ID_LST) == 0) {
				osso_email_xml_unpack_msgid_list(node, doc, XML_MSG_ID_LST,
								 &section_list);
				break;
			}
		}
	}

	DBUS_XML_UNPACK_UN_INIT;

	return section_list;
}

GSList *ui_dbus_folder_remove_all_attachment(GSList * msgid_list)
{
	gchar *xmlbuff = NULL;
	gchar *str = NULL;


	CHECK_STATUS((msgid_list == NULL), "folder_attach list is null", return NULL);

	osso_email_utils_pack_msgid_to_gslist(msgid_list, &xmlbuff);
	str = osso_email_utils_concat_strings(FOLDER_REMOVEALL_ATTACHMENT, xmlbuff);

	xmlFree(xmlbuff);
	ret_section_list = NULL;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return ret_section_list;
}
static osso_pack_return_t ui_dbus_folder_list(const gchar * recv_str)
{
	GSList *ret_list = NULL;
	gchar *message = NULL;
	gchar *name = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr cur = NULL;
	xmlNodePtr node = NULL;
	gint type = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	DBUS_XML_UNPACK_INIT(message, return OSSO_PACK_ERROR);

	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, FOLDER_NAME) == 0) {

				osso_email_xml_unpack_int(node, doc, FOLDER_NAME, &type);

			} else if (g_ascii_strcasecmp((gchar *) node->name,
						      XML_MSG_HEADER_LST) == 0) {

				osso_email_xml_unpack_msgheader_list(node, doc,
								     XML_MSG_HEADER_LST, &ret_list);
			}
		}
	}

	DBUS_XML_UNPACK_UN_INIT;

	/* call the relevent CB func */
	folder_msg_headers_cb(type, ret_list);
	/* free list & name */
	g_free(name);
	procmsg_msgheader_list_free(ret_list);

	return OSSO_PACK_OK;
}

static osso_pack_return_t ui_dbus_recv_attachment(const gchar * recv_str)
{
	MsgInfo *msginfo = NULL;
	gchar *temp = NULL;

	xmlNodePtr cur = NULL, node = NULL;
	xmlDocPtr doc = NULL;
	gint attach_flag = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, temp, return OSSO_PACK_ERROR);

	msginfo = g_new0(MsgInfo, 1);
	if (msginfo == NULL) {
		osso_log(LOG_ERR, "Mem Alloc Failed for msginfo struct");
		return OSSO_PACK_ALLOC_FAIL;
	}

	DBUS_XML_UNPACK_INIT(temp, return OSSO_PACK_ERROR);

	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, XML_MSG_INFO) == 0) {
				osso_email_xml_unpack_msginfo(node, doc, XML_MSG_INFO, msginfo);
			}
			if (g_ascii_strcasecmp(node->name, ATTA_TAG) == 0) {
				osso_email_xml_unpack_int(node, doc, ATTA_TAG, &attach_flag);
			}
		}
	}
	DBUS_XML_UNPACK_UN_INIT;
	ui_recv_msg_attachment_cb(msginfo);

	return TRUE;
}

static osso_pack_return_t ui_dbus_folder_set_msg(const gchar * recv_str)
{
	gchar *message = NULL;
	gint msgnumber = 0;
	GArray *arguments = NULL;
	gint idx = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	ui_dbus_parse_set_msg_cb_args(message, &msgnumber, &arguments, &idx);

	folder_set_msg_cb(msgnumber, idx, arguments);

	if (arguments != NULL) {
		g_array_free(arguments, TRUE);
	}
	return OSSO_PACK_OK;
}

osso_pack_return_t ui_dbus_folder_cancel(void)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_strings(FOLDER_CANCEL, NULL);
	return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
}

static osso_pack_return_t ui_dbus_is_quit(void)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_str_int(UI_IS_READY, FALSE);
	return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
}

osso_pack_return_t ui_dbus_disconnect_iap_conn(void)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_str_int(UI_DISCONNECT_IAP, TRUE);
	osso_email_gconf_set_int(EMAIL_GCONF_UI_CLOSED, 1);
	/* send on both the connections to ensure cancel quickly */
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, FALSE);
	return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
}

static osso_pack_return_t ui_dbus_app_killer_signal(const gchar * recv_str)
{
	gchar *message = NULL;
	ParamMissing param_number = 0;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);
	param_number = atoi(message);
	if (param_number == SIGNAL_SHUTDOWN_IND) {
		/* update stateinfo and send engine about completion and exit */
		oom_signal_recv();
		ui_dbus_is_quit();
		ui_exit(0);
	}
	return OSSO_PACK_OK;
}

osso_pack_return_t
ui_dbus_set_tmp_passwd(const gchar * account_name,
		       const gchar * passwd, gboolean cancel, gboolean other_conn)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	CHECK_STATUS((account_name == NULL), "Account name is null", return OSSO_PACK_INVALID);

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_str(root, doc, XML_STRING, &account_name);
	if (passwd != NULL) {
		osso_email_xml_pack_str(root, doc, MSGID, &passwd);
	}
	osso_email_xml_pack_bool(root, doc, XML_BOOL, &cancel);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(SET_TMP_PASSWD, xmlbuff);
	xmlFree(xmlbuff);
	if (other_conn) {
		return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
	} else {
		return ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	}
	//return ui_dbus_send_on_normal_connection ( str, __FUNCTION__, TRUE );
}

gint ui_dbus_rem_msg_attachments(const gchar * msgid, const GSList * file_pos_list)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if ((msgid == NULL) || (file_pos_list == NULL)) {
		osso_log(LOG_INFO, "Received NULL for msgid or mimeinfo \n");
		return 1;
	}

	DBUS_XML_PACK_INIT;

	osso_email_xml_pack_int_list(root, doc, XML_INT_LST, (GSList *) file_pos_list);

	osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);

	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(REM_MSG_ATTACHMENTS, xmlbuff);
	xmlFree(xmlbuff);
	folder_return_code = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	return folder_return_code;
}

gboolean ui_dbus_is_engine_ready(void)
{
	gchar *str = NULL;

	engine_status = FALSE;
	osso_log(LOG_DEBUG, "\nUITEST>>> UIDBUS REaDY FUNC \n");

	str = osso_email_utils_concat_strings(ENGINE_CHECK_STATUS, NULL);
	engine_acct_status = 0;
	ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);

	while (1) {
		osso_log(LOG_DEBUG, "\n UITEST>>> waiting \n");
		(void) g_main_context_iteration(NULL, FALSE);

		if (engine_status == TRUE) {
			break;
		}
	}

	return engine_status;
}
static osso_pack_return_t ui_dbus_engine_status(const gchar * recv_str)
{
	/* received callback from engine */
	engine_status = TRUE;
	return OSSO_PACK_OK;
}

osso_pack_return_t ui_dbus_send_recv_mail(const gchar * account)
{
	gchar *str = NULL;
	str = osso_email_utils_concat_strings(SEND_RECV_MAIL, account);
	return ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);
}

osso_pack_return_t
ui_dbus_send_get_msg_id(const MsgInfo * msginfo, gchar ** msgid, gboolean sending)
{
	osso_pack_return_t rv = OSSO_PACK_OK;
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;

	if (msginfo == NULL) {
		osso_log(LOG_INFO, "Received NULL values for msginfo ");
		return OSSO_PACK_ERROR;
	}

	DBUS_XML_PACK_INIT;
	osso_email_xml_pack_msginfo(root, doc, XML_MSG_INFO, (MsgInfo *) msginfo);
	osso_email_xml_pack_bool(root, doc, SEND_FLAG, &sending);
	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
	DBUS_XML_PACK_UN_INIT;

	str = osso_email_utils_concat_strings(SEND_GET_MSG_ID, xmlbuff);
	xmlFree(xmlbuff);
	rv = ui_dbus_send_on_normal_connection(str, __FUNCTION__, TRUE);
	*msgid = new_draft_msgid;
	return rv;
}
static osso_pack_return_t ui_dbus_del_list(const gchar * recv_str)
{
	GSList *msgid_list = NULL;
	gchar *message = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr cur = NULL;
	xmlNodePtr node = NULL;

	GET_DATA_FROM_RECV_STRING(recv_str, message, return OSSO_PACK_ERROR);

	DBUS_XML_UNPACK_INIT(message, return OSSO_PACK_ERROR);

	for (node = cur->xmlChildrenNode; node != NULL; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) {
			if (g_ascii_strcasecmp(node->name, XML_MSG_ID_LST) == 0) {
				osso_email_xml_unpack_msgid_list(node, doc, XML_MSG_ID_LST,
								 &msgid_list);
				break;
			}
		}
	}

	DBUS_XML_UNPACK_UN_INIT;

	/* call the relevent CB func,which also frees the memory */
	ui_remove_messages_cb(msgid_list);

	return OSSO_PACK_OK;
}

void ui_dbus_retrieve_msgs(GSList * retr_list, gboolean copy)
{
	gchar *str = NULL;
	xmlDocPtr doc = NULL;
	xmlNodePtr root = NULL;
	xmlChar *xmlbuff = NULL;
	int buffersize = 0;


	CHECK_STATUS((retr_list == NULL), "folder_attach list is null", return);

	DBUS_XML_PACK_INIT;

	osso_email_xml_pack_retr_list(root, doc, XML_RETR_LST, retr_list);
	osso_email_xml_pack_bool(root, doc, XML_BOOL, &copy);

	xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);

	str = osso_email_utils_concat_strings(AL_RETR_MSGS, xmlbuff);

	xmlFree(xmlbuff);

	DBUS_XML_PACK_UN_INIT;

	ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);

}
static osso_pack_return_t ui_dbus_handle_backup(const gchar * recv_str)
{
	return OSSO_PACK_OK;
}
static osso_pack_return_t
ui_dbus_retrv_msg_status(const gchar * recv_str)
{
	retrv_msgs_status();	
	return OSSO_PACK_OK;
	
}
void ui_dbus_update_seen_flag(const gchar * msgid)
{
        gchar *str = NULL;
        xmlDocPtr doc = NULL;
        xmlNodePtr root = NULL;
        xmlChar *xmlbuff = NULL;
        int buffersize = 0;

        if (msgid == NULL) {
                osso_log(LOG_INFO, "Received NULL for msgid \n");
                return;
        }

        DBUS_XML_PACK_INIT;

        osso_email_xml_pack_str(root, doc, XML_MSG_ID, &msgid);
        xmlDocDumpFormatMemory(doc, &xmlbuff, &buffersize, 1);
        str = osso_email_utils_concat_strings(UPDATE_SEEN_FLAG, xmlbuff);
        xmlFree(xmlbuff);

        DBUS_XML_PACK_UN_INIT;

        ui_dbus_send_on_send_recv_connection(str, __FUNCTION__, TRUE);

}
