/**
 * @file enablesignatures.c
 *
 * This file contains the functions to initialize the signature data structure 
 * and creating signature window. 
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */

/* Header file containing all standard paths.*/
#include "ui_common.h"

/*UI related user defined header files.*/
#include "enablesignatures_callbacks.h"
#include "intlsupport.h"
#include "ui_utils.h"
#include "utils.h"
#include "email_mgr.h"
#include "ui_defs.h"
#include "osso-log.h"
#include "editview.h"
#include "callback.h"
#include "ui_account_store.h"
#include "ui_interface.h"

extern MainViewInfo *email_main_view;

/**
This function is used to create the enablesignature window

@param void.
@return GtkWidget  pointer to the newly createcd widget.
*/
GtkWidget *create_enablesignature_window(GSList * accounts)
{
	GtkWidget *enablesignature_dialog = NULL;
	GtkWidget *dialog_action_area2 = NULL;
	GtkWidget *enablesignature_vbox = NULL;
	GtkWidget *enablesignature_label = NULL;
	GtkWidget *enablesignature_scrolledwindow = NULL;
	GtkWidget *enablesignature_treeview = NULL;
	GtkWidget *enablesignature_hbox = NULL;
	GtkWidget *enablesignature_ok_button = NULL;
	GtkWidget *enablesignature_edit_button = NULL;
	GtkWidget *enablesignature_cancel_button = NULL;
	GSList *enablesignature_data_list = NULL;

	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	enablesignature_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(enablesignature_dialog),
			     (_("mcen_ti_email_signatures_title")));
	gtk_window_set_transient_for(GTK_WINDOW(enablesignature_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(enablesignature_dialog), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(enablesignature_dialog), FALSE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(enablesignature_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(enablesignature_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);

	enablesignature_vbox = GTK_DIALOG(enablesignature_dialog)->vbox;
	gtk_widget_show(enablesignature_vbox);

	enablesignature_label = gtk_label_new(_("mcen_ia_email_signatures_dlg_label"));
	gtk_label_set_line_wrap((GtkLabel *) enablesignature_label, TRUE);
	gtk_widget_show(enablesignature_label);
	gtk_box_pack_start(GTK_BOX(enablesignature_vbox), enablesignature_label, FALSE, FALSE, 0);

	enablesignature_scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(enablesignature_scrolledwindow);
	gtk_box_pack_start(GTK_BOX(enablesignature_vbox),
			   enablesignature_scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (enablesignature_scrolledwindow),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	enablesignature_treeview = enablesignatures_create_view_email_accounts(accounts);
	if (enablesignature_treeview == NULL) {
		gtk_widget_destroy(enablesignature_dialog);
		return NULL;
	}
	gtk_widget_show(enablesignature_treeview);
	gtk_container_add(GTK_CONTAINER(enablesignature_scrolledwindow), enablesignature_treeview);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(enablesignature_treeview), FALSE);

	enablesignature_hbox = gtk_hbox_new(TRUE, 0);
	gtk_widget_show(enablesignature_hbox);
	gtk_box_pack_start(GTK_BOX(enablesignature_vbox), enablesignature_hbox, TRUE, TRUE, 0);

	dialog_action_area2 = GTK_DIALOG(enablesignature_dialog)->action_area;
	gtk_widget_show(dialog_action_area2);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area2), GTK_BUTTONBOX_END);

	enablesignature_ok_button = gtk_button_new_with_mnemonic(_("mcen_bd_email_signatures_ok"));
	gtk_widget_show(enablesignature_ok_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(enablesignature_dialog),
				     enablesignature_ok_button, 0);

	enablesignature_edit_button =
	    gtk_button_new_with_mnemonic(_("mcen_bd_email_signatures_edit"));
	gtk_widget_show(enablesignature_edit_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(enablesignature_dialog),
				     enablesignature_edit_button, 0);

	enablesignature_cancel_button =
	    gtk_button_new_with_mnemonic(_("mcen_bd_email_signatures_cancel"));
	gtk_widget_show(enablesignature_cancel_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(enablesignature_dialog),
				     enablesignature_cancel_button, 0);

	enablesignature_data_list =
	    g_slist_append(enablesignature_data_list, enablesignature_dialog);

	enablesignature_data_list =
	    g_slist_append(enablesignature_data_list, enablesignature_treeview);

	g_signal_connect(G_OBJECT(enablesignature_edit_button), "clicked",
			 G_CALLBACK(on_click_enablesignature_edit_button),
			 enablesignature_treeview);

	g_signal_connect(G_OBJECT(enablesignature_ok_button), "clicked",
			 G_CALLBACK(on_click_enablesignature_ok_button), enablesignature_data_list);

	g_signal_connect(G_OBJECT(enablesignature_cancel_button), "clicked",
			 G_CALLBACK(on_click_enablesignature_cancel_button),
			 enablesignature_dialog);

	if (ossohelp_exists(HELP_ID_SIGNATURES) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) enablesignature_dialog);
		g_signal_connect(G_OBJECT(enablesignature_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SIGNATURES);
	}

	g_signal_connect(G_OBJECT(enablesignature_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), enablesignature_cancel_button);

	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return enablesignature_dialog;
}

/* end of file */
