/**
 * @file font_settings.c
 *
 * This file contains the implementation for font settings module of OSSO EMAIL
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */



#include "ui_common.h"
#include "folder_mgmt.h"
#include "message_handler.h"
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "font_settings.h"
#include "ui_state_mgmt.h"
#include "htcallback.h"
#include "email_gconf.h"
#include "accountMgmt.h"
#include "accountMgmt_callback.h"
#include "ui_account_store.h"

/* Global variables*/
extern MainViewInfo *email_main_view;	/* Declared in ui_interface.c */

void set_font_values(GtkWidget * fontsettings_win)
{

	GtkWidget *combo_viewhtml = NULL;
	GtkWidget *combo_viewplain = NULL;
	GtkWidget *combo_edithtml = NULL;
	GtkWidget *combo_editplain = NULL;
	gint viewing_html_fontname_index = 0;
	gint viewing_plain_fontname_index = 0;
	gint editing_html_fontname_index = 0;
	gint editing_plain_fontname_index = 0;


	combo_viewhtml = EMAIL_UI_LOOKUP_OBJECT(fontsettings_win, "viewinghtml");
	combo_viewplain = EMAIL_UI_LOOKUP_OBJECT(fontsettings_win, "viewingtext");
	combo_edithtml = EMAIL_UI_LOOKUP_OBJECT(fontsettings_win, "editinghtml");
	combo_editplain = EMAIL_UI_LOOKUP_OBJECT(fontsettings_win, "editingtext");

	viewing_html_fontname_index = gtk_combo_box_get_active((GtkComboBox *) combo_viewhtml);
	viewing_plain_fontname_index = gtk_combo_box_get_active((GtkComboBox *) combo_viewplain);
	editing_plain_fontname_index = gtk_combo_box_get_active((GtkComboBox *) combo_editplain);
	editing_html_fontname_index = gtk_combo_box_get_active((GtkComboBox *) combo_edithtml);

	ui_set_default_settings(viewing_html_fontname_index,
				viewing_plain_fontname_index,
				editing_html_fontname_index, editing_plain_fontname_index);

	return;
}

GtkWidget *launch_font_setting_dialog(void)
{
	GtkWidget *font_setting_dialog = NULL;	/*Font Widget Dialog */
	GtkWidget *font_setting_notebook = NULL;	/*NotePad on dialog */
	GtkWidget *tab_page_viewing = NULL;	/*Viewing Page */
	GtkWidget *tab_label_viewing = NULL;	/*Tab label in viewing page */
	GtkWidget *tab_page_editing = NULL;	/*Editing Page */
	GtkWidget *tab_label_editing = NULL;	/*Tab label in editing page */
	GtkWidget *ok_button = NULL;
	GtkWidget *cancel_button = NULL;

	/*Widgets for viewing page */
	GtkSizeGroup *sizegroup_viewing = NULL;
	GtkWidget *caption_viewing_html = NULL;
	GtkWidget *caption_viewing_text = NULL;

	GtkWidget *combo_viewing_text = NULL;
	GtkWidget *combo_viewing_html = NULL;

	GtkSizeGroup *size_group_editing = NULL;
	GtkWidget *caption_editing_html = NULL;
	GtkWidget *caption_editing_text = NULL;

	GtkWidget *combo_editing_text = NULL;
	GtkWidget *combo_editing_html = NULL;
	gint i = 0;
	gint viewing_html_fontname_index = 0;
	gint viewing_plain_fontname_index = 0;
	gint editing_html_fontname_index = 0;
	gint editing_plain_fontname_index = 0;

	gint num_families = 0;
	num_families = sizeof(font_families) / sizeof(gchar *);


	viewing_html_fontname_index = ui_get_viewer_html_fontindex();
	viewing_plain_fontname_index = ui_get_viewer_plain_fontindex();
	editing_html_fontname_index = ui_get_editor_html_fontindex();
	editing_plain_fontname_index = ui_get_editor_plain_fontindex();

	/*Create Font Dialog */

	font_setting_dialog =
	    gtk_dialog_new_with_buttons(_("mcen_ti_font_settings"),
					GTK_WINDOW(app),
					GTK_DIALOG_DESTROY_WITH_PARENT, NULL, NULL);
	gtk_dialog_set_has_separator(GTK_DIALOG(font_setting_dialog), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(font_setting_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(font_setting_dialog), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(font_setting_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(font_setting_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	gtk_window_set_resizable(GTK_WINDOW(font_setting_dialog), TRUE);
	gtk_window_set_default_size(GTK_WINDOW(font_setting_dialog), FONT_SETTINGS_DIALOG_WIDTH,
				    -1);

	ok_button =
	    gtk_dialog_add_button(GTK_DIALOG(font_setting_dialog),
				  _("mcen_bd_font_settings_ok"), GTK_RESPONSE_OK);
	cancel_button =
	    gtk_dialog_add_button(GTK_DIALOG(font_setting_dialog),
				  _("mcen_bd_font_settings_cancel"), GTK_RESPONSE_CANCEL);


	/*Create Notebook */
	font_setting_notebook = gtk_notebook_new();

	/*Viewing Page */
	tab_page_viewing = gtk_vbox_new(FALSE, HILDON_MARGIN_DEFAULT);
	tab_label_viewing = gtk_label_new(_("mcen_fi_font_settings_viewing"));
	gtk_notebook_append_page(GTK_NOTEBOOK(font_setting_notebook),
				 tab_page_viewing, tab_label_viewing);
	/*Add Widgets on viewing page */
	sizegroup_viewing = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	combo_viewing_html = gtk_combo_box_new_text();
	for (i = 0; i < num_families; i++) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_viewing_html), (gchar *)
					  font_families[i]);

	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_viewing_html), viewing_html_fontname_index);

	combo_viewing_text = gtk_combo_box_new_text();
	for (i = 0; i < num_families; i++) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_viewing_text), (gchar *)
					  font_families[i]);
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_viewing_text), viewing_plain_fontname_index);
	caption_viewing_html =
	    hildon_caption_new(sizegroup_viewing,
			       _("mcen_fi_font_settings_default_html"),
			       combo_viewing_html, NULL, HILDON_CAPTION_OPTIONAL);

	caption_viewing_text =
	    hildon_caption_new(sizegroup_viewing,
			       _("mcen_fi_font_settings_default_plain"),
			       combo_viewing_text, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(tab_page_viewing), caption_viewing_html, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(tab_page_viewing), caption_viewing_text, TRUE, TRUE, 0);

	/*Editing Page */
	tab_page_editing = gtk_vbox_new(FALSE, HILDON_MARGIN_DEFAULT);
	tab_label_editing = gtk_label_new(_("mcen_fi_font_settings_editing"));
	gtk_notebook_append_page(GTK_NOTEBOOK(font_setting_notebook),
				 tab_page_editing, tab_label_editing);

	/*Add Widgets on Editing page */
	size_group_editing = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	/*Combo Html Font for Editing Tab */
	combo_editing_html = gtk_combo_box_new_text();
	for (i = 0; i < num_families; i++) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_editing_html), (gchar *)
					  font_families[i]);
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_editing_html), editing_html_fontname_index);

	/*Combo Text  Font for Editing Tab */
	combo_editing_text = gtk_combo_box_new_text();
	for (i = 0; i < num_families; i++) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_editing_text), (gchar *)
					  font_families[i]);
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo_editing_text), editing_plain_fontname_index);

	caption_editing_html =
	    hildon_caption_new(size_group_editing,
			       _("mcen_fi_font_settings_default_html"),
			       combo_editing_html, NULL, HILDON_CAPTION_OPTIONAL);

	caption_editing_text =
	    hildon_caption_new(size_group_editing,
			       _("mcen_fi_font_settings_default_plain"),
			       combo_editing_text, NULL, HILDON_CAPTION_OPTIONAL);

	gtk_box_pack_start(GTK_BOX(tab_page_editing), caption_editing_html, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(tab_page_editing), caption_editing_text, TRUE, TRUE, 0);


	/*Add notepad to dialog */
	gtk_box_pack_start(GTK_BOX((GTK_DIALOG(font_setting_dialog))->vbox),
			   font_setting_notebook, TRUE, TRUE, 1);
	gtk_widget_show_all(font_setting_dialog);
	EMAIL_UI_HOOKUP_OBJECT_NO_REF(font_setting_dialog, font_setting_dialog,
				      "fontsettingsDialog");
	EMAIL_UI_HOOKUP_OBJECT(font_setting_dialog, combo_viewing_html, "viewinghtml");
	EMAIL_UI_HOOKUP_OBJECT(font_setting_dialog, combo_viewing_text, "viewingtext");
	EMAIL_UI_HOOKUP_OBJECT(font_setting_dialog, combo_editing_html, "editinghtml");
	EMAIL_UI_HOOKUP_OBJECT(font_setting_dialog, combo_editing_text, "editingtext");

	if (ossohelp_exists(HELP_ID_FONTSETTINGS) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) font_setting_dialog);
		g_signal_connect(G_OBJECT(font_setting_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_FONTSETTINGS);
	}

	return font_setting_dialog;
}

void ui_on_font_setting_activate(GtkWidget * widget)
{
	GtkWidget *font_dialog = NULL;
	gint response = GTK_RESPONSE_OK;
	font_dialog = launch_font_setting_dialog();
	response = gtk_dialog_run((GtkDialog *) font_dialog);
	if (response == GTK_RESPONSE_OK) {
		set_font_values(font_dialog);
		gtk_widget_destroy(font_dialog);

	} else {
		gtk_widget_destroy(font_dialog);
	}
	return;
}

/**
 This function is used to get the index value from the 
 scheduling interval.
 
 @param interval is the gint representing the scheduling interval.
 @return gint the index to be set in the combo.
*/
static gint ui_index_from_interval(gint interval)
{
	gint interval_time[] = { 0, 5, 10, 15, 30, 60, 120 };
	gint index = 0;
	for (index = 0; index < sizeof(interval_time) / sizeof(gint); index++) {
		if (interval_time[index] == interval)
			break;
	}
	return index;
}

/**
 This function is used to get the scheduling interval value from the
 combo index.
 
 @param index is the gint representing the index of combo.
 @return gint the scheduling interval value in minutes.(ex 10/15/20/45/60)
*/
static gint ui_get_interval_from_index(gint index)
{
	gint interval_time[] = { 0, 5, 10, 15, 30, 60, 120 };
	if (index >= (sizeof(interval_time) / sizeof(gint))) {
		ULOG_DEBUG("the index is out of range");
		return 0;
	}
	return (interval_time[index]);
}

#if RETAINED_EMAIL
/**
 This function is used to get the gconf values of scheduling on &
  scheduling interval & set them in the options  dialog. 
 
 @param options_dialog is the GtkWidget pointer to the option dialog.

 @return void.
*/
static void ui_set_option_window_values(GtkWidget * interval_combo)
{
	gint interval = 0;


	/*Get the Scheduling interval value from Gconf */
	if (!osso_email_gconf_get_int(EMAIL_SCHEDULING_INTERVAL, &interval))
		osso_log(LOG_DEBUG, "Error getting the EMAIL_SCHEDULING_INTERVAL value");

	gtk_combo_box_set_active(GTK_COMBO_BOX(interval_combo), ui_index_from_interval(interval));
	return;
}

/**
 This function is used to set the gconf values of scheduling on &
  scheduling interval based on the options selected in the dialog. 
 
 @param options_dialog is the GtkWidget pointer to the option dialog.

 @return false--is schedule interval is not interger in range 1-99.
         true --else.
*/
static gboolean ui_get_option_window_values(GtkWidget * interval_combo)
{

	gint index = 0, interval = 0, prev_interval = 0;

	index = gtk_combo_box_get_active(GTK_COMBO_BOX(interval_combo));

	interval = ui_get_interval_from_index(index);

	/*Get the Scheduling value from Gconf */
	if (!osso_email_gconf_get_int(EMAIL_SCHEDULING_INTERVAL, &prev_interval))
		osso_log(LOG_DEBUG, "Error getting the EMAIL_SCHEDULING_INTERVAL value");

	/*show an apppropriate infobanner if scheduling interval is changed from  0 to any value. */
	if (prev_interval && !interval) {
		gtk_infoprint((GtkWindow *) app, _("mail_ib_autoretrieve_off"));
	} else if (!prev_interval && interval) {
		gtk_infoprint((GtkWindow *) app, _("mail_ib_autoretrieve_on"));
	}

	if (!osso_email_gconf_set_int(EMAIL_SCHEDULING_INTERVAL, interval))
		osso_log(LOG_DEBUG, "Error setting the interval value");

	return TRUE;
}
#endif

/**
 This is a callback handler called when cancel button is clicked
 
 @param widget is the GtkWidget  pointer to cancel button.
 @param options_dialog is the GtkWidget pointer to the options dialog.
 
 @return void.
*/
static void on_cancel_options_dialog(GtkWidget * widget, GtkWidget * options_dialog)
{
	if (!options_dialog) {
		ULOG_DEBUG_L("Input Param checking failure");
		return;
	}
	gtk_widget_destroy(options_dialog);
	return;
}

#if RETAINED_EMAIL
/**
 This is a callback handler called when ok button is clicked.
 
 @param widget is the GtkWidget  pointer to ok button.
 @param options_dialog is the GtkWidget pointer to the options dialog.
 
 @return void.
*/
static void on_ok_options_dialog(GtkWidget * widget, GtkWidget * options_dialog)
{
	GtkWidget *interval_combo = NULL;
	if (!options_dialog) {
		ULOG_DEBUG_L("Input Param checking failure");
		return;
	}

	interval_combo = EMAIL_UI_LOOKUP_OBJECT(options_dialog, "interval_value");

	if (ui_get_option_window_values(interval_combo))
		gtk_widget_destroy(options_dialog);
	return;
}

/**
 This function creates the option dialog.
 
 @return void.
*/
static GtkWidget *create_option_dialog(void)
{
	GtkWidget *options_dialog = NULL;
	GtkWidget *dialog_vbox = NULL;
	GtkWidget *label_auto_retrieve = NULL;
	GtkWidget *combo_interval = NULL;
	GtkWidget *hseparator = NULL;
	GtkWidget *caption_control = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *button_signature = NULL;
	GtkWidget *button_font = NULL;
	GtkWidget *dialog_action_area = NULL;
	GtkWidget *okbutton = NULL;
	GtkWidget *cancelbutton = NULL;

	options_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(options_dialog), _("mcen_ti_options"));
	gtk_dialog_set_has_separator(GTK_DIALOG(options_dialog), FALSE);

	gtk_window_set_transient_for(GTK_WINDOW(options_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(options_dialog), TRUE);

	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(options_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);

	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(options_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);


	dialog_vbox = GTK_DIALOG(options_dialog)->vbox;
	gtk_widget_show(dialog_vbox);

	/*auto retrieve label */
	label_auto_retrieve = gtk_label_new(_("mcen_ia_options_autoretrieve"));
	gtk_label_set_line_wrap(GTK_LABEL(label_auto_retrieve), TRUE);
	gtk_widget_show(label_auto_retrieve);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), label_auto_retrieve, FALSE, FALSE, 0);

	combo_interval = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_interval),
				  (const gchar *) (_("mcen_va_options_interval_disabled")));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_interval),
				  (const gchar *) (_("mcen_va_options_interval_5")));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_interval),
				  (const gchar *) (_("mcen_va_options_interval_10")));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_interval),
				  (const gchar *) (_("mcen_va_options_interval_15")));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_interval),
				  (const gchar *) (_("mcen_va_options_interval_30")));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_interval),
				  (const gchar *) (_("mcen_va_options_interval_1h")));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_interval),
				  (const gchar *) (_("mcen_va_options_interval_2h")));

	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_options_autoretrieve"),
			       combo_interval, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), caption_control, TRUE, TRUE, 0);

	gtk_widget_show_all(caption_control);


	hseparator = gtk_hseparator_new();
	gtk_widget_show(hseparator);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), hseparator, TRUE, TRUE, 0);

	button_signature = gtk_button_new_with_mnemonic(_("mcen_bd_options_signatures"));
	gtk_widget_show(button_signature);

	/*signature field */
	caption_control =
	    hildon_caption_new(size_group, _("mcen_ia_options_signatures"),
			       button_signature, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_show(caption_control);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), caption_control, FALSE, FALSE, 0);

	hseparator = gtk_hseparator_new();
	gtk_widget_show(hseparator);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), hseparator, TRUE, TRUE, 0);

	/*font settings field */
	button_font = gtk_button_new_with_mnemonic(_("mcen_bd_options_fonts"));
	gtk_widget_show(button_font);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_ia_options_fonts"),
			       button_font, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_show(caption_control);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), caption_control, FALSE, FALSE, 0);

	dialog_action_area = GTK_DIALOG(options_dialog)->action_area;
	gtk_widget_show(dialog_action_area);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area), GTK_BUTTONBOX_SPREAD);

	okbutton = gtk_button_new_with_mnemonic(_("mcen_bd_options_ok"));
	gtk_widget_show(okbutton);
	gtk_dialog_add_action_widget(GTK_DIALOG(options_dialog), okbutton, GTK_RESPONSE_OK);

	GTK_WIDGET_SET_FLAGS(okbutton, GTK_CAN_DEFAULT);

	cancelbutton = gtk_button_new_with_mnemonic(_("mcen_bd_options_cancel"));
	gtk_widget_show(cancelbutton);

	gtk_dialog_add_action_widget(GTK_DIALOG(options_dialog), cancelbutton, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS(cancelbutton, GTK_CAN_DEFAULT);
	gtk_widget_show(options_dialog);


	EMAIL_UI_HOOKUP_OBJECT_NO_REF(options_dialog, options_dialog, "options_dialog");
	EMAIL_UI_HOOKUP_OBJECT(options_dialog, combo_interval, "interval_value");

	g_signal_connect((gpointer) button_font, "clicked",
			 G_CALLBACK(ui_on_font_setting_activate), NULL);
	g_signal_connect((gpointer) button_signature, "clicked",
			 G_CALLBACK(ui_on_signatures1_activate), NULL);

	g_signal_connect((gpointer) okbutton, "clicked",
			 G_CALLBACK(on_ok_options_dialog), options_dialog);
	g_signal_connect((gpointer) cancelbutton, "clicked",
			 G_CALLBACK(on_cancel_options_dialog), options_dialog);
	g_signal_connect(G_OBJECT(options_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancelbutton);

	/*the scheduling value with gconf value */
	ui_set_option_window_values(combo_interval);
	return options_dialog;
}
#endif

#if CHANGED_EMAIL
/**
 Callback function when the update field of settings dialog is clicked.
 Toggles the update connection type and interval fields.

 @param update_ckbtn is pointer to the clicked object.
 @param settings_dialog is the settings dialog.
 @return void.
*/
static void reset_autoupdate_fields(GtkWidget * update_ckbtn, GtkWidget * settings_dialog)
{
	GtkWidget *con_type_combo = NULL;
	GtkWidget *interval_combo = NULL;
	gboolean autoupdate_checked = FALSE;

	if (!update_ckbtn || !settings_dialog) {
		ULOG_DEBUG_L("Input Param checking failure");
		return;
	}
	con_type_combo = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "update_connection");
	interval_combo = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "interval_value");

	autoupdate_checked = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(update_ckbtn));
	gtk_widget_set_sensitive(con_type_combo, autoupdate_checked);
	gtk_widget_set_sensitive(interval_combo, autoupdate_checked);
}

/**
 Callback function when the cleanup sent folder field of settings dialog is clicked.
 Toggles the update connection type and interval fields.

 @param cleanup_ckbtn is pointer to the clicked object.
 @param msg_expiry_combo whose sensitivity is to be set.
 @return void.
*/
static void reset_cleanup_frequency_field(GtkWidget * cleanup_ckbtn, GtkWidget * msg_expiry_combo)
{
	gboolean cleanup_checked = FALSE;

	if (!cleanup_ckbtn || !msg_expiry_combo) {
		ULOG_DEBUG_L("Input Param checking failure");
		return;
	}

	cleanup_checked = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cleanup_ckbtn));
	gtk_widget_set_sensitive(msg_expiry_combo, cleanup_checked);
}

/**
 This function is used to get the index value from the 
 sent folder cleanup duration.
 
 @param days is the gint representing the duration in days.
 @return gint the index to be set in the combo.
*/
static gint get_index_for_days(gint days)
{
	gint cleanup_days[] = { 1, 7, 14, 30, 90, 180 };
	gint index = 0;
	for (index = 0; index < sizeof(cleanup_days) / sizeof(gint); index++) {
		if (cleanup_days[index] == days)
			break;
	}
	return index;
}

/**
 This function is used to get the duration in days for sent item cleanup from the combo index.
 
 @param index is the gint representing the index of combo.
 @return gint the duration in days for sent item cleanup.
*/
static gint get_days_from_index(gint index)
{
	gint days[] = { 1, 7, 14, 30, 90, 180 };
	if (index >= (sizeof(days) / sizeof(gint))) {
		ULOG_DEBUG("the index is out of range");
		return 0;
	}
	return (days[index]);
}

/**
 Changes the various items of prefs_account for all accounts from the settings dialog.
 
 @param   read_receipt represents whether to request for a read report.
 @param   size_limit represents size limit of a message.
 @param   msg_format_type represents html/plain nature of msg
 @param   inc_original represents whether to include original msg info while replying a mail.
 
 @return void.
*/
static void set_accounts_with_settings_info(gboolean read_receipt, gint size_limit,
					    gint msg_format_type, gboolean inc_original)
{
	PrefsAccount *update_prefsAccount = NULL;
	GSList *account_list = NULL;
	gchar *account = NULL;
	SettingType type;

	/*Get the list of configured accounts. */
	account_list = email_main_view->account_manager->configured_accounts;
	if (!account_list)
		return;
	while (account_list != NULL) {
		account = (gchar *) account_list->data;
		update_prefsAccount = ui_get_account_detail(account);
		if (update_prefsAccount == NULL) {
			return;
		}
		update_prefsAccount->receive_settings.size_limit = size_limit;
		if (msg_format_type == 0)
			update_prefsAccount->advanced_settings.email_type = EMAIL_HTML;
		else
			update_prefsAccount->advanced_settings.email_type = EMAIL_TEXT;
		update_prefsAccount->advanced_settings.org_in_reply = inc_original;
		update_prefsAccount->advanced_settings.request_read_report = read_receipt;
		type = RECEIVE_SETTING | ADVANCED_SETTING;
		//type = USER_SETTING | SEND_SETTING | RECEIVE_SETTING | ADVANCED_SETTING;
		/*update the account settings for that particular account. */
		ui_dbus_set_account_settings(type, update_prefsAccount);
		ui_free_prefs_account_structure(update_prefsAccount);
		update_prefsAccount = NULL;
		account_list = g_slist_next(account_list);
	}
	ui_init_account_store();
}

/**
 This is a callback handler called when ok button in settings dialog is clicked.
 Sets the values from dialog into persistent setting and update items in all the  accounts .
 
 @param widget is the GtkWidget  pointer to ok button.
 @param settings_dialog is the GtkWidget pointer to the settings dialog.
 
 @return void.
*/
static void on_ok_settings_dialog(GtkWidget * widget, GtkWidget * settings_dialog)
{
	GtkWidget *update_checkbutton = NULL;
	GtkWidget *con_type_combo = NULL;
	GtkWidget *interval_combo = NULL;
	GtkWidget *size_limit_editor = NULL;
	GtkWidget *sound_checkbutton = NULL;
	GtkWidget *msg_type_combo = NULL;
	GtkWidget *inc_original_checkbutton = NULL;
	GtkWidget *read_rec_checkbutton = NULL;
	GtkWidget *clean_sent_checkbutton = NULL;
	GtkWidget *msg_expiry_combo = NULL;
	gint index = 0, interval = 0;
	gboolean via_wlan = TRUE;
	gint size_limit = 1000;
	gboolean play_sound = FALSE;
	gint msg_format_type = 0;
	gboolean inc_original = TRUE;
	gboolean read_receipt = FALSE;
	gint sent_folder_clean_option = -1, sent_folder_clean_time_stamp = -1;
	gint sent_folder_setting_value = -1;
	time_t cur_time;

	if (!settings_dialog) {
		ULOG_DEBUG_L("Input Param checking failure");
		return;
	}

	update_checkbutton = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "autoupdate");
	con_type_combo = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "update_connection");
	interval_combo = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "interval_value");
	size_limit_editor = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "msg_size_limit");
	sound_checkbutton = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "play_sound");
	msg_type_combo = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "msg_format_type");
	inc_original_checkbutton = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "inc_original");
	read_rec_checkbutton = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "read_receipt");
	clean_sent_checkbutton = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "clean_sent");
	msg_expiry_combo = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "clean_sent_interval");

	/* Get autoupdate related info. */
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(update_checkbutton))) {
		index = gtk_combo_box_get_active(GTK_COMBO_BOX(interval_combo));
		interval = ui_get_interval_from_index(index + 1);
		index = gtk_combo_box_get_active(GTK_COMBO_BOX(con_type_combo));
	}
	if (index > 0) {
		via_wlan = FALSE;
	}
	/* Set autoupdate related info into GConf. */
	if (!osso_email_gconf_set_int(EMAIL_SCHEDULING_INTERVAL, interval))
		osso_log(LOG_DEBUG, "Error setting the interval value");
	if (!osso_email_gconf_set_bool(EMAIL_SCHEDULING_VIA_WLAN, via_wlan))
		osso_log(LOG_DEBUG, "Error setting the connection type");

	/* Get mailing settings */
	size_limit = hildon_number_editor_get_value((HildonNumberEditor *) (size_limit_editor));
	play_sound = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sound_checkbutton));
	msg_format_type = gtk_combo_box_get_active(GTK_COMBO_BOX(msg_type_combo));
	inc_original = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(inc_original_checkbutton));
	read_receipt = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(read_rec_checkbutton));
	/* Set mailing settings into persistent settings */
	ui_set_email_settings_info(read_receipt, size_limit,
				   play_sound, msg_format_type, inc_original);

	/*Change mailing settings of all accounts. */
	set_accounts_with_settings_info(read_receipt, size_limit, msg_format_type, inc_original);

	/*get the sent folder cleanup values and set it into the persistent settings. */
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(clean_sent_checkbutton))) {
		index = gtk_combo_box_get_active(GTK_COMBO_BOX(msg_expiry_combo));
		sent_folder_setting_value = get_days_from_index(index);
		sent_folder_clean_option = DAYS;
	} else {
		sent_folder_clean_option = DO_NOT_EMPTY;
	}
	/*Calculate current time_stamp when cleaning operation is done */
	time(&cur_time);
	sent_folder_clean_time_stamp = cur_time;
	ui_set_sent_folder_cleaning_settings(sent_folder_clean_option,
					     sent_folder_clean_time_stamp,
					     sent_folder_setting_value);

	gtk_widget_destroy(settings_dialog);

	if (sent_folder_clean_option != DO_NOT_EMPTY) {
		ui_empty_sent_folder(TRUE);
	}

	return;
}

/**
 Gets the gconf values related to autoupdate and sets the fileds in the settings dialog. 
@param settings_dialog is the GtkWidget pointer to the settings dialog.
@return void.
*/
static void set_settings_updating_info(GtkWidget * settings_dialog)
{
	GtkWidget *update_checkbutton = NULL;
	GtkWidget *con_type_combo = NULL;
	GtkWidget *interval_combo = NULL;
	gint interval = 0;
	gboolean via_wlan = TRUE;

	if (settings_dialog == NULL) {
		ULOG_DEBUG_L("Failed at Input Parameter validation");
		return;
	}

	update_checkbutton = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "autoupdate");
	con_type_combo = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "update_connection");
	interval_combo = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "interval_value");

	/*Get the autoupdate values from Gconf */
	if (!osso_email_gconf_get_int(EMAIL_SCHEDULING_INTERVAL, &interval))
		osso_log(LOG_DEBUG, "Error getting the EMAIL_SCHEDULING_INTERVAL value");
	if (!osso_email_gconf_get_bool(EMAIL_SCHEDULING_VIA_WLAN, &via_wlan))
		osso_log(LOG_DEBUG, "Error getting the EMAIL_SCHEDULING_VIA_WLAN value");

	/*Sets the fields with fetched info. */
	if (interval > 0) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(update_checkbutton), TRUE);
		gtk_combo_box_set_active(GTK_COMBO_BOX(interval_combo),
					 ui_index_from_interval(interval) - 1);
		if (via_wlan) {
			gtk_combo_box_set_active(GTK_COMBO_BOX(con_type_combo), 0);
		} else {
			gtk_combo_box_set_active(GTK_COMBO_BOX(con_type_combo), 1);
		}
	} else {
		gtk_widget_set_sensitive(interval_combo, FALSE);
		gtk_widget_set_sensitive(con_type_combo, FALSE);
	}
}

/**
Gets the various mailing info from persistent settings and sets fields in settings dialog. 
@param settings_dialog is the GtkWidget pointer to the settings dialog.
@return void.
*/
static void set_settings_mailing_info(GtkWidget * settings_dialog)
{
	GtkWidget *size_limit_editor = NULL;
	GtkWidget *sound_checkbutton = NULL;
	GtkWidget *msg_type_combo = NULL;
	GtkWidget *inc_original_checkbutton = NULL;
	GtkWidget *read_rec_checkbutton = NULL;
	gint size_limit = 1000;
	gboolean play_sound = FALSE;
	gint msg_format_type = 0;
	gboolean inc_original = TRUE;
	gboolean read_receipt = FALSE;

	if (settings_dialog == NULL) {
		ULOG_DEBUG_L("Failed at Input Parameter validation");
		return;
	}

	size_limit_editor = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "msg_size_limit");
	sound_checkbutton = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "play_sound");
	msg_type_combo = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "msg_format_type");
	inc_original_checkbutton = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "inc_original");
	read_rec_checkbutton = EMAIL_UI_LOOKUP_OBJECT(settings_dialog, "read_receipt");

	ui_get_email_settings_info(&read_receipt, &size_limit,
				   &play_sound, &msg_format_type, &inc_original);
	/*Set the fields distributed in 1st & 2nd pages of dialog. */
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sound_checkbutton), play_sound);
	hildon_number_editor_set_value((HildonNumberEditor *) size_limit_editor, size_limit);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(inc_original_checkbutton), inc_original);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(read_rec_checkbutton), read_receipt);
	gtk_combo_box_set_active(GTK_COMBO_BOX(msg_type_combo), msg_format_type);
}

/**
Creates the page for first tab i.e updating tab for settings dialog.
@param notebook that will hold the created page.
@param settings_dialog that holds the netebook.
@return void.
*/
static void create_settings_updating_page(GtkWidget * notebook, GtkWidget * settings_dialog)
{
	GtkWidget *vbox = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *viewport = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *update_checkbutton = NULL;
	GtkWidget *con_type_combo = NULL;
	GtkWidget *interval_combo = NULL;
	GtkWidget *separator = NULL;
	GtkWidget *size_limit_editor = NULL;
	GtkWidget *sound_checkbutton = NULL;

	if (notebook == NULL || settings_dialog == NULL) {
		ULOG_DEBUG_L("Failed at Input Parameter validation");
		return;
	}

	/*scrollwindow/viewport creation */
	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_container_add(GTK_CONTAINER(notebook), scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);
	vbox = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(viewport), vbox);
	gtk_container_set_border_width(GTK_CONTAINER(viewport), HILDON_MARGIN_DEFAULT);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);

	/*Update automatic checkbox field */
	update_checkbutton = gtk_check_button_new_with_mnemonic("");
	caption_control = hildon_caption_new(size_group,
					     _("mcen_fi_options_autoupdate"),
					     update_checkbutton, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);
	gtk_widget_grab_focus(update_checkbutton);
	g_signal_connect(G_OBJECT(update_checkbutton), "clicked",
			 G_CALLBACK(reset_autoupdate_fields), settings_dialog);

	/*connection type for autoupdate. */
	con_type_combo = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(con_type_combo),
				  _("mcen_va_options_connectiontype_wlan"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(con_type_combo),
				  _("mcen_va_options_connectiontype_all"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(con_type_combo), 0);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_options_connectiontype"),
			       con_type_combo, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*interval for autoupdate. */
	interval_combo = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(interval_combo),
				  _("mcen_va_options_updateinterval_5min"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(interval_combo),
				  _("mcen_va_options_updateinterval_10min"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(interval_combo),
				  _("mcen_va_options_updateinterval_15min"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(interval_combo),
				  _("mcen_va_options_updateinterval_30min"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(interval_combo),
				  _("mcen_va_options_updateinterval_1h"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(interval_combo),
				  _("mcen_va_options_updateinterval_2h"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(interval_combo), 0);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_options_updateinterval"),
			       interval_combo, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	separator = gtk_hseparator_new();
	gtk_widget_show(separator);
	gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 0);

	/*Email retrieval size limit field */
	size_limit_editor = hildon_number_editor_new(MIN_SIZELIMIT, MAX_SIZELIMIT);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_advsetup_sizelimit"),
			       size_limit_editor, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	/*sound play for msg arrival checkbox field */
	sound_checkbutton = gtk_check_button_new_with_mnemonic("");
	caption_control = hildon_caption_new(size_group,
					     _("mcen_fi_options_playsound"),
					     sound_checkbutton, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),
					gtk_notebook_get_nth_page(GTK_NOTEBOOK
								  (notebook),
								  0),
					_("mcen_ti_options_updating"));

	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, update_checkbutton, "autoupdate");
	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, con_type_combo, "update_connection");
	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, interval_combo, "interval_value");
	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, size_limit_editor, "msg_size_limit");
	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, sound_checkbutton, "play_sound");
}

/**
Creates the page for 2nd tab i.e composing tab for settings dialog.
@param notebook that will hold the created page.
@param settings_dialog that holds the netebook.
@return void.
*/
static void create_settings_composing_page(GtkWidget * notebook, GtkWidget * settings_dialog)
{
	GtkWidget *vbox = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *viewport = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *msg_type_combo = NULL;
	GtkWidget *inc_original_checkbutton = NULL;
	GtkWidget *read_rec_checkbutton = NULL;

	if (notebook == NULL || settings_dialog == NULL) {
		ULOG_DEBUG_L("Failed at Input Parameter validation");
		return;
	}

	/*scrollwindow/viewport creation */
	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_container_add(GTK_CONTAINER(notebook), scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);
	vbox = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(viewport), vbox);
	gtk_container_set_border_width(GTK_CONTAINER(viewport), HILDON_MARGIN_DEFAULT);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);

	/*Message format type filed */
	msg_type_combo = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(msg_type_combo),
				  _("mcen_va_options_messageformat_html"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(msg_type_combo),
				  _("mcen_va_options_messageformat_plain"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(msg_type_combo), 0);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_options_messageformat"),
			       msg_type_combo, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);
	gtk_widget_grab_focus(msg_type_combo);

	/*include original in reply checkbox field */
	inc_original_checkbutton = gtk_check_button_new_with_mnemonic("");
	caption_control = hildon_caption_new(size_group,
					     _("Include original in reply"),
					     inc_original_checkbutton, NULL,
					     HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);


	/*request read receipt checkbox field */
	read_rec_checkbutton = gtk_check_button_new_with_mnemonic("");
	caption_control = hildon_caption_new(size_group,
					     _("Request read receipt"),
					     read_rec_checkbutton, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),
					gtk_notebook_get_nth_page(GTK_NOTEBOOK
								  (notebook),
								  1),
					_("mcen_ti_options_composing"));
	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, msg_type_combo, "msg_format_type");
	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, inc_original_checkbutton, "inc_original");
	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, read_rec_checkbutton, "read_receipt");

	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
}

/**
Creates the page for 3rd tab i.e cleanup tab for settings dialog.
@param notebook that will hold the created page.
@param settings_dialog that holds the netebook.
@return void.
*/
static void create_settings_cleanup_page(GtkWidget * notebook, GtkWidget * settings_dialog)
{
	GtkWidget *vbox = NULL;
	GtkWidget *caption_control = NULL;
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *viewport = NULL;
	GtkSizeGroup *size_group = NULL;
	GtkWidget *msg_expiry_combo = NULL;
	GtkWidget *clean_sent_checkbutton = NULL;
	gint sent_folder_clean_option = -1;
	gint sent_folder_setting_value = -1;
	gint index = 0;

	if (notebook == NULL || settings_dialog == NULL) {
		ULOG_DEBUG_L("Failed at Input Parameter validation");
		return;
	}

	/*scrollwindow/viewport creation */
	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_container_add(GTK_CONTAINER(notebook), scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_widget_show(viewport);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);
	vbox = gtk_vbox_new(FALSE, ACCOUNT_FIELDS_GAP);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(viewport), vbox);
	gtk_container_set_border_width(GTK_CONTAINER(viewport), HILDON_MARGIN_DEFAULT);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);

	/*Clean sent folder checkbox field */
	clean_sent_checkbutton = gtk_check_button_new_with_mnemonic("");
	caption_control = hildon_caption_new(size_group,
					     _("mcen_fi_options_cleanup"),
					     clean_sent_checkbutton, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);
	gtk_widget_grab_focus(clean_sent_checkbutton);

	/*Clean up frequency field */
	msg_expiry_combo = gtk_combo_box_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(msg_expiry_combo),
				  _("mcen_va_options_cleanup_older_1"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(msg_expiry_combo),
				  _("mcen_va_options_cleanup_older_7"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(msg_expiry_combo),
				  _("mcen_va_options_cleanup_older_14"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(msg_expiry_combo),
				  _("mcen_va_options_cleanup_older_30"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(msg_expiry_combo),
				  _("mcen_va_options_cleanup_older_90"));
	gtk_combo_box_append_text(GTK_COMBO_BOX(msg_expiry_combo),
				  _("mcen_va_options_cleanup_older_180"));
	gtk_combo_box_set_active(GTK_COMBO_BOX(msg_expiry_combo), 0);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	caption_control =
	    hildon_caption_new(size_group, _("mcen_fi_options_cleanup_older"),
			       msg_expiry_combo, NULL, HILDON_CAPTION_OPTIONAL);
	gtk_widget_set_size_request(caption_control, -1, COMBO_HEIGHT);
	gtk_box_pack_start(GTK_BOX(vbox), caption_control, FALSE, FALSE, 0);
	gtk_widget_show_all(caption_control);

	g_signal_connect(G_OBJECT(clean_sent_checkbutton), "clicked",
			 G_CALLBACK(reset_cleanup_frequency_field), msg_expiry_combo);
	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),
					gtk_notebook_get_nth_page(GTK_NOTEBOOK
								  (notebook),
								  2), _("mcen_ti_options_cleanup"));

	sent_folder_clean_option = ui_get_sent_folder_clean_option();
	sent_folder_setting_value = ui_get_sent_folder_clean_time_value();

	if (sent_folder_clean_option == DO_NOT_EMPTY) {
		gtk_toggle_button_set_active((GtkToggleButton *) clean_sent_checkbutton, FALSE);
		gtk_widget_set_sensitive(msg_expiry_combo, FALSE);
	} else {
		gtk_toggle_button_set_active((GtkToggleButton *) clean_sent_checkbutton, TRUE);
		index = get_index_for_days(sent_folder_setting_value);
		gtk_combo_box_set_active(GTK_COMBO_BOX(msg_expiry_combo), index);
	}

	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, clean_sent_checkbutton, "clean_sent");
	EMAIL_UI_HOOKUP_OBJECT(settings_dialog, msg_expiry_combo, "clean_sent_interval");
}

/**
Creates the settings dialog.

@return GtkWidget pointer to the window which it will create. 
*/

static GtkWidget *create_settings_dialog(void)
{
	GtkWidget *settings_dialog = NULL;
	GtkWidget *settings_vbox = NULL;
	GtkWidget *notebook = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *ok_button = NULL;
	GtkWidget *cancel_button = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	settings_dialog = gtk_dialog_new();
	gtk_window_set_resizable(GTK_WINDOW(settings_dialog), TRUE);
	gtk_window_set_default_size(GTK_WINDOW(settings_dialog), DIALOG_WID, WIDGET_HEIGHT);
	gtk_window_set_title(GTK_WINDOW(settings_dialog), _("mcen_ti_options"));

	gtk_dialog_set_has_separator(GTK_DIALOG(settings_dialog), FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(settings_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(settings_dialog), TRUE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(settings_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(settings_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);
	settings_vbox = GTK_DIALOG(settings_dialog)->vbox;
	gtk_widget_show(settings_vbox);

	notebook = gtk_notebook_new();
	gtk_widget_show(notebook);
	gtk_box_pack_start(GTK_BOX(settings_vbox), notebook, TRUE, TRUE, 0);

	/*Create pages of the notebook. */
	create_settings_updating_page(notebook, settings_dialog);
	create_settings_composing_page(notebook, settings_dialog);
	create_settings_cleanup_page(notebook, settings_dialog);

	/*Set the pages with appropriate values. */
	set_settings_updating_info(settings_dialog);
	set_settings_mailing_info(settings_dialog);

	dialog_action_area1 = GTK_DIALOG(settings_dialog)->action_area;
	gtk_widget_show(dialog_action_area1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);
	ok_button = gtk_button_new_with_label(_("mcen_bd_options_ok"));
	gtk_widget_show(GTK_WIDGET(ok_button));
	gtk_dialog_add_action_widget(GTK_DIALOG(settings_dialog), ok_button, 0);

	cancel_button = gtk_button_new_with_mnemonic(_("mcen_bd_options_cancel"));
	gtk_widget_show(cancel_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(settings_dialog), cancel_button, 0);
	g_signal_connect(G_OBJECT(ok_button), "clicked",
			 G_CALLBACK(on_ok_settings_dialog), settings_dialog);
	g_signal_connect(G_OBJECT(cancel_button), "clicked",
			 G_CALLBACK(on_cancel_options_dialog), settings_dialog);
	g_signal_connect(G_OBJECT(settings_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), cancel_button);

	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
	return settings_dialog;
}
#endif

/**
 This is the callback function for launching options dialog
 
 @param tools_options is the meu item that registered the callback.
 @return void.
*/
void ui_on_tools_options_activate(GtkWidget * tools_options)
{
	GtkWidget *option_dialog = NULL;

#if RETAINED_EMAIL
	option_dialog = create_option_dialog();
#else
	option_dialog = create_settings_dialog();
#endif
	gtk_widget_show(option_dialog);

	return;
}
