/**
 * @file  accountMgmt.h
 *
 * This file contains the declarations for account management module of
 * OSSO_EMAIL.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


#ifndef __ACCOUNTMGMT_H__
#define __ACCOUNTMGMT_H__

#include<gtk/gtk.h>

/* Inclusion of AL Header files */
#include "common.h"
#include "prefs_account.h"
#include "procmsg.h"
#include "procmime.h"
#include "folder.h"

/* Inclusion of UI_DBUS Header files*/
#include "email_mgr.h"
#include "dbus_utils.h"

/* Hash Defines for this file */
#define RECV_POP3 0
#define RECV_IMAP 1
#define RECVTYPE_HEADER 1
#define RECVTYPE_MESG   2
#define RECV_PLAINTXT   1
#define RECV_NORMAL     1
#define EMAILSENDING_IMMD 1
#define EMAILSENDING_NXTCON 2
#define DIALOG_WID 385
#define  WIDGET_HEIGHT 240
#define BUTTON_WIDTH 60
#define DEFAULT_HEIGHT -1
#define DEFAULT_LEN 200


#define MAX_NAME_ENTRY_LEN     64
#define MAX_TEXT_ENTRY_LEN     64
#define MAX_PASSWD_ENTRY_LEN   256
#define RESTRICTED_CHAR_LEN    3
#define ADVANCED_DIALOG_WIDTH  720
#define ADVANCED_DIALOG_HEIGHT 320
#define TEN 10
#define ALL 0
#define CAP_CTRL_SPACE 5
#define TWENTYFIVE 25
#define FIFTY 50
#define WIDGET_PACKING 17
#define WIDGET_LEN 20
#define WIDGET_WIDTH 86
#define ENTRY_HEIGHT 30
#define COMBO_HEIGHT 35
#define ZERO 0
#define ACCOUNT_FIELDS_GAP 9
#define ACCOUNT_LABELS_PADDING 11
#define ACC_SECURITY_LABELS_PADDING 5
#define ACC_NAME_COL_WIDTH  275

typedef struct _AdvOptionViewInfo AdvOptionViewInfo;
typedef struct _RetrivevalTab RetrivevalTab;
typedef struct _SMTPTab SMTPTab;
typedef struct _SendingTab SendingTab;
typedef struct _Security SecurityTab;
typedef struct _AdvOptionButtons AdvOptionButtons;
typedef struct _AccountMgtMngr AccountMgtMngr;
typedef struct _AccountMgrInfo AccountMgrInfo;

enum {
	RETRIEVE_ALL,
	RETRIEVE_10,
	RETRIEVE_25,
	RETRIEVE_50
};
enum {
	PASSWD_AUTH_NORMAL,
	PASSWD_AUTH_SECURE
};
enum {
	SECURE_NONE,
	SECURE_USE_TLS,
	SECURE_USE_SSL
};
enum {
	RECV_HEADERS,
	RECV_MESSAGE,
	RECV_MESSAGE_ATT
};
enum {
	SMTP_NONE,
	SMTP_LOGIN,
	SMTP_CRAM_MD5
};
enum {
	HTML_FORMAT,
	PLAIN_FORMAT
};
enum {
	SEND_IMMEDIATELY,
	SEND_NEXT_CONN,
	SEND_ONREQUEST
};




struct _RetrivevalTab {
	GtkWidget *combo_recvtype;
	GtkWidget *leave_mesg_checkbutton;
	GtkWidget *spinbutton_size_limit;
	GtkWidget *combo_passwd_auth;
	GtkWidget *combo_no_email;
	GtkWidget *entry_imapfolder;
	GtkWidget *combo_ssl;
	GtkWidget *spinbutton_inport;
};

struct _SendingTab {
	GtkWidget *combo_email_type;
	GtkWidget *combo_sendemail;
	GtkWidget *checkbutton_org_rep;
	GtkWidget *checkbutton_read_receipt;
};

struct _SMTPTab {
	GtkWidget *checkbutton_smtpauth;
	GtkWidget *combo_smtpauth;
	GtkWidget *entry_user_name;
	GtkWidget *entry_passwd;
	GtkWidget *combo_smtp_ssl;
	GtkWidget *spinbutton_outport;
	GtkWidget *checkbutton_ssl;
};

struct _Security {
	GtkWidget *checkbutton_encrypt;
	GtkWidget *checkbutton_sign;
	GtkWidget *checkbutton_encp_sign;
	GtkWidget *checkbutton_untrusted_id;
	GtkWidget *checkbutton_expired_id;
};

struct _AdvOptionButtons {
	GtkWidget *dialog_action_area1;
	GtkWidget *button1;
	GtkWidget *button2;
	GtkWidget *button3;
	GtkWidget *button4;
};

struct _AdvOptionViewInfo {
	GtkWidget *dialog_advancedsetup;
	GtkWidget *vbox5;
	GtkWidget *notebook1;
	RetrivevalTab *retrivevalTab;
	SMTPTab *smtpTab;
	SendingTab *sendingTab;
	SecurityTab *securityTab;
	AdvOptionButtons *advOptionButtons;
};

struct _AccountMgtMngr {
	gboolean setting_modified;
	gboolean ssl_flag;
	gboolean smtp_auth_active;
	gboolean advanced_setup_active;
	gboolean setup3_active;
	gboolean setup2_active;
	gboolean is_setup1_modified;
	gint RECV_PROT_ADVSETUP;
	gboolean ACCT_NAME_MODY;
	gboolean edit_operation;
	gboolean is_mailbox_operation;
	gint incoming_port;
	gboolean port_changed;
};
struct _AccountMgrInfo {
	PrefsAccount *ui_accMgmt_editinfo;
};
GtkWidget *ui_accmgmt_createAccountsetupDialog(gboolean acc_creat_edit);
GtkWidget *ui_accmgmt_createemailaccountsetup1(gboolean acc_creat_edit);
GtkWidget *ui_accmgmt_createemailaccountsetup2(gboolean acc_creat_edit);
GtkWidget *ui_accmgmt_createemailaccountsetup3(gboolean acc_creat_edit);
GtkWidget *ui_accmgmt_createemailaccountsetup4(gboolean acc_creat_edit);
GtkWidget *ui_accmgmt_createemailaccountsedit(const GSList * accountname);

AdvOptionViewInfo *ui_accmgmt_createadvancedsettingdialog(gboolean acc_creat_edit);

void ui_accmgmt_set_selected_accountview
    (GtkTreeSelection * selection, GtkTreeModel * accountViewModel);

void ui_accmgmt_initialize_global_variable(gboolean status);
gboolean ui_accmgmt_search_accountname(const gchar * account_name);
GtkTreeModel *ui_accmgmt_create_accountview_model(const GSList * accountname);
GtkWidget *ui_acccmgmt_create_view_model(const GSList * accountname);
void ui_accMgmt_create_advance_dialog(AdvOptionViewInfo * advOptionViewInfo);

void ui_accMgmt_create_advance_retrievel_dialog(const AdvOptionViewInfo * advOptionViewInfo);

void ui_accMgmt_create_advance_sending_dialog(const AdvOptionViewInfo * advOptionViewInfo);
void ui_accMgmt_create_advance_smtp_dialog(const AdvOptionViewInfo * advOptionViewInfo);
void ui_accMgmt_create_advance_others_dialog(const AdvOptionViewInfo * advOptionViewInfo);
void ui_accMgmt_create_advance_buttons(const AdvOptionViewInfo * advOptionViewInfo);
gint sort_iter_compare_func(GtkTreeModel * model, GtkTreeIter * iter_a,
			    GtkTreeIter * iter_b, /*@unused@ */ gpointer userdata);
gchar *ui_get_new_accountname_display_string(void);
void ui_on_mailbox_protected_dialog(gchar * acct_name, gboolean smpt_flag,
				    gboolean show_error_info);

/**
   This function is used to free the IapInfo structure.
 
    @param iap_info is the pointer to the IapInfo structure.
    @return void.
 
*/
#if CL_USE
/**
 Function to free the IapInfo structure.
 @param smtp_server_info is the pointer to the IapInfo.
 @param data is the user data -not used.
 @return void.
*/
void ui_free_iapinfo_structure(IapInfo *smtp_server_info,gpointer data);
#endif


#endif

/* End of file */
