/**
 * @file al_callbacks.h
 * 
 * This file contains the callback function declarations for 
 * UI_DBUS module of OSSO_EMAIL.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#ifndef AL_CALLBACKS_H_
#define AL_CALLBACKS_H_

#include "prefs_account.h"
#include "common.h"
#include "procmsg.h"
#include "procmime.h"

void smtp_param_value_missing_cb(ParamMissing param_number);
void smtp_set_msg_cb(gint msg_number, gint index, GArray * arguments);
void smtp_set_percentage_cb(gint percent);
void smtp_update_folder_view_cb(MsgHeader * msgheader, gint operation, gboolean status);
void smtp_update_outbox_mails_sent(const gint total, const gint sent);
void smtp_send_complete_cb(void);
void pop3_param_value_missing_cb(ParamMissing param_number);
void pop3_set_msg_cb(gint msg_number, gint index, GArray * arguments);
void pop3_set_percentage_cb(gint percent);
void pop3_recv_single_cb(MsgHeader * msgheader);
void pop3_recv_complete_cb(void);
void pop3_recv_body_complete_cb(MimeInfo * mimeinfo, gchar * msgid, gboolean remote_copy);
void imap4_param_value_missing_cb(ParamMissing param_number);
void imap4_set_msg_cb(gint msg_number, gint index, GArray * arguments);
void imap4_msg_headers_cb(GSList * msg_header_list);
void imap4_msg_attachment_headers_cb(GSList * msg_header_list);
void ui_remove_messages_cb(GSList * msg_id_list);
void acct_error_code_cb(gint error_code);
void mmc_hlr_cb(gint error_code);
void tasknav_open_msg_cb(gchar * msgid);
void editor_create_cb(MsgInfo * msginfo, gchar * message);
void tasknav_window_id(gchar * windowid);
void tasknav_open_inbox(void);
void oom_signal_recv(void);
void close_application(void);

/**
  This function is called to update the UI module about the result 
  of folder operations.
  @param new_list    List of message headers 
  @return none
*/
void folder_msg_headers_cb(gint type, GSList * new_list);

/**
  This function is called to update the UI module about the folder 
  operation status/error code and progress bar related information. 

  @param msg_number contains the status/error code
  @param index contains the length of GArray arguments array
  @param arguments GArray of osso_rpc_t elements  which contains the 
    additional information
  @return none 
*/
void folder_set_msg_cb(gint msg_number, gint index, GArray * arguments);
void retrv_msgs_status(void);
#endif
