/**
 * @file callback.h
 * 
 * This file contains the function declarations  & the structures
 * needed for  viewer.c
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

#ifndef __CALLBACK_H__
#define __CALLBACK_H__

#include "ui_common.h"
#include "viewer.h"
#include "hyperlink_handler.h"
#include "editview.h"

GtkWidget *ui_viewer_create(HeaderInfo * header);
void ui_free_viewerinfo(ViewerInfo * viewerInfo);
void ui_viewer_reply_cb(GtkWidget * win, const ViewerInfo * viewerInfo);
void ui_viewer_forward_cb(GtkWidget * win, ViewerInfo * viewerInfo);
void ui_viewer_replyAll_cb(GtkWidget * win, const ViewerInfo * viewerInfo);
void ui_viewer_close_cb(GtkWidget * win, ViewerInfo * viewerInfo);
void ui_viewer_save_attachment_cb(GtkWidget * win, ViewerInfo * viewerInfo);
void ui_viewer_ok_save_attachment_cb(GSList *, GSList *, GtkFileChooser * win);
void ui_viewer_cancel_save_attachment_cb(GSList *, GSList *, GtkWidget * win);
void ui_view_attachment_cb(GtkWidget * win, ViewerInfo * viewerInfo);
gboolean ui_viewer_check_attachments(const MimeInfo * mimeinfo, GtkListStore * view,
				     GtkWidget * treeview, gboolean email_attachment);
void ui_view_next_message_cb(GtkWidget * win, ViewerInfo * viewer_info);
void ui_view_prev_message_cb(GtkWidget * win, ViewerInfo * viewer_info);
void ui_viewer_delete_message_cb(GtkWidget * win, ViewerInfo * viewer_info);
#if CHANGED_EMAIL
void viewer_message_properties_clicked(GtkWidget *menu, ViewerInfo * viewer_info);
#endif
void ui_view_attachment_cb(GtkWidget * win, ViewerInfo * viewer_info);
void ui_toggle_viewer_toolbar_cb(GtkWidget *, ViewerInfo *);
void ui_msg_find_text_cb(GtkWidget * win, MsgFields *);
void ui_viewer_add_to_contacts_cb(GtkWidget * widget, MsgFields * msgfields);
void ui_viewer_move_msg_cb(GtkWidget * win, ViewerInfo * viewer_info);
void ui_on_close_all_windows_activate(GtkWidget *, gpointer);
gchar *ui_file_size(const gchar * fileName);
void ui_show_attachments_in_buffer(GtkListStore *, const gchar * attachmentname,
				   const gchar * attachmentpath, const gchar * attachmentSize,
				   gboolean mycard_attached);
AttachmentStatus ui_viewer_check_for_attachments(const MimeInfo * mimeinfo, const gchar * msgid,
						 GtkListStore * attachment_view, MsgInfo * msgInfo,
						 gint msgType, gboolean download_attachments,
						 GSList ** ReAttachList);
void ui_set_priority_icon(gint priority, GtkImage * priorityImage);
void ui_update_window_pointers(const gchar * msgid);
GSList *ui_attachment_selection_cb(GtkTreeSelection *, ViewerInfo * viewer_info);
gchar *ui_editor_attachment_selection_cb(GtkWidget * attachmentview,
					 GtkWidget * selecteditem, GtkWidget * remove_menu);
  /*Browser embedding related helper functions */
GtkWidget *ui_viewer_embed_new_browser_window(GtkContainer * container, MsgFields * fields,
					      const char *url, gchar * msgid);
void ui_viewer_zoom_browser_window(GtkWidget * socket, gint zoom_percent);
void ui_viewer_select_all_browser_window(GtkWidget * socket);
void ui_show_next_window(void);
void ui_create_attachment_field(GtkWidget **, GtkWidget **, GtkSizeGroup *);
void update_fullydownloaded_flag(const HeaderInfo *, const MimeInfo *);
gboolean check_attachments_downloaded(const MimeInfo * mime_info);
gchar *ui_viewer_html_parser(const gchar *);
void invoke_browser( const gchar * url);
/*State save related functions*/
void restore_viewer_saved_state(ViewerInfo *);
void ui_set_saved_msg_values(MsgFields * msgFields, const gchar * msgid);
void ui_get_curr_msg_settings(MsgFields * fields, const gchar * msgid);
void ui_viewer_get_state_values_cb(GtkWidget *, ViewerInfo *);
void ui_toggle_toolbar(gboolean toolbar_hidden, HildonAppView * appview, GtkWidget * app_toolbar);
void activate_menus(GtkWidget * field, GdkEventFocus *, ViewerInfo * viewerInfo);
void read_file(const gchar *, gchar **);
gchar *get_receiversid(const gchar * msgid);
void save_view_dimming_note(GtkWidget * insensitive_widget, ViewerInfo *);
void purge_dimming_note(GtkWidget * insensitive_widget, ViewerInfo *);
/**
This is callback function called when font settings for viewer is called.

 @param fontsettingsmenu is the widget that registered the callback.
 @param viewerinfo is the ViewerInfo structure pointer.
 @return void.
*/
void ui_viewer_fontsettings_cb(GtkWidget * fontsettingsmenu, ViewerInfo * viewer);
/**
This is callback function called when searchmailsmenu for viewer is clicked.

 @param searchmailsmenu is the widget that registered the callback.
 @param viewerinfo is the ViewerInfo structure pointer.
 @return void.
*/
void ui_viewer_searchemails_cb(GtkWidget * searchmailsmenu);

/**
This is callback function called when purgemenu for viewer is clicked.

 @param purgemenu is the widget that registered the callback.
 @param viewerinfo is the ViewerInfo structure pointer.
 @return void.
*/
void ui_viewer_purge_attachment_cb(GtkWidget * purgemenu, ViewerInfo * viewer);

/**
This function is used to open a editor for composing a new mail.
@param nemmailmenu is the menu that registered the callback.
@return void.
*/
void ui_open_newmail_cb(GtkWidget * newmail_menu);

/**
 This function is used change the viewer to fullscreen mode.
 
 @param win is the widget that registered the callback.
 @param viewer is the pointer to the ViewerInfo structure pointer.
 @return void.
 */
void ui_viewer_fullscreen(GtkWidget * win, ViewerInfo * viewer);

/**
 This is toggles the fullscreen mode for  viewer.

 @param win isthe check menu item that register the callback.
 @param viewer is the pointer to viewerInfo structure.
 @return;
 */
void ui_viewer_toggle_to_fullscreen(GtkWidget * wid, ViewerInfo * viewer);

/**
 This function is called by higher layers for  registering the callback
    to enable/disable the toolbar.

 @param win is a pointer to the widget that registered the callback.
 @param viewerInfo is a pointer to the editorInfo of the current viewer.
         It is not freed in this function
 @return void
*/
void ui_toggle_viewer_fullscreen_toolbar_cb(GtkWidget * win, ViewerInfo * viewerInfo);


/**
 This is toggles the fullscreen mode for editor.

 @param win isthe check menu item that register the callback.
 @param editor is the pointer to editorInfo structure.
 @return;
 */
void ui_editor_toggle_to_fullscreen(GtkWidget * wid, EditorInfo * editor);

/**
  This is called when purge attachment menu is called.

  @param wid is the menu that registerd the callback.
  @param viewer_info is the pointer to the ViewerInfo structure pointer.
  @return void.
  */
void ui_viewer_purge_attachment_cb(GtkWidget * purgemenu, ViewerInfo * viewer_info);

/**
 This function is used to change the attachment icons after purging is done.
 
 @param wid is the menu that registerd the callback.
 @param viewer_info is the pointer to the ViewerInfo structure pointer.
 @return void.
 */
void change_purged_icon(GtkWidget * wid, ViewerInfo * viewer);

/**
 This function is used to launch the cm store to get the properties of the certificate.
 
 @param wid is the menu that registerd the callback.
 @param viewer_info is the pointer to the ViewerInfo structure pointer.
 @return void.
 */
void ui_viewer_launch_cm_cb(GtkWidget * wid, ViewerInfo * viewer);
/**
 This function is used to launch the cm store to get the 
 recip properties of the certificate.
 
 @param wid is the menu that registerd the callback.
 @param viewer_info is the pointer to the ViewerInfo structure pointer.
 @return void.
*/
gboolean ui_viewer_recip_cert_properties(GtkWidget * wid, GdkEvent * event, ViewerInfo * viewer);
/**
 This function is used to launch the cm store to get the 
 signer properties of the certificate.
 
 @param wid is the menu that registerd the callback.
 @param viewer_info is the pointer to the ViewerInfo structure pointer.
 @return void.
*/
gboolean ui_viewer_signer_cert_properties(GtkWidget * wid, GdkEvent * event, ViewerInfo * viewer);
/*
zooming the viewer with hardware keys.
GDK_F7 for increasing the zoom and
GDK_F8 for decreasing the zoom.

@param appview is the viewer appview.
@param event is the event which registered the callback.
@param viewer_info is the pointer to the ViewerInfo structure pointer.

@return TRUE-to stop the propagation of event.
        FALSE-else.
*/
void zoom_view(gint zoomvalue, Zoom * zoominfo);
/*
zooming the viewer with hardware keys.
 decreasing the zoom

@param appview is the viewer appview.
@param zoominfo is the pointer to the Zoom structure pointer.

@return .

*/
void decrease_zoom_view(GtkWidget * appview, Zoom * zoominfo);
/*
zooming the viewer with hardware keys.
 increasing the zoom

@param appview is the viewer appview.
@param zoominfo is the pointer to the Zoom structure pointer.

@return .

*/
void increase_zoom_view(GtkWidget * appview, Zoom * zoominfo);

/*
setting the viewer zoom menu after zooming with hardware keys.

@param appview is the viewer appview.
@param viewer_info is the pointer to the ViewerInfo structure pointer.

@return .

*/
void set_viewer_zoom_menu(GtkWidget * appview, ViewerInfo * viewer_info);


/**
   This is the helper function to get the first plain pointer of the mimeinfo.
   @param mime_info is the MiemInfo straucture pointer .
   @return TRUE if plain part is found.
           FALSE-else
*/
gboolean check_for_plain_html(const MimeInfo * mime_info);

void ui_viewer_bg_invoke_embed_new_browser(void);
void hide_progressbar(ViewerInfo * toolbar);
void show_progressbar(ViewerInfo * toolbar);
void update_progress_bar_text(GtkWidget * viewer, ViewerInfo * viewer_info);
gchar *ui_viewer_get_body(const ViewerInfo * viewer_info);
void ui_viewer_get_msginfo_thr_cb(GtkWidget * viewer, ViewerInfo * viewer_info);
HeaderInfo *ui_initialise_headerinfo(const ViewerInfo * viewer_info, gboolean download_attachments);
/*Helper function to get the mimetype for vacrd attachment.
 * @param attachment is the gchar poiter to thename of the attachment.
 * @return gboolen TRUE-->if vcard.
 * 		   FALSE-->else.
*/
gboolean check_for_vcard_attachment(const gchar * attachment);
gchar *get_password_for_decryption(guint certid);
void focus_to_start(GtkTextBuffer * buffer, GtkWidget * view);
const gchar *ui_viewer_get_default_font(void);
void set_header_field_heigth(ViewerInfo * viewer_info);
/**
 Function to update the mimeinfo once entire message is downloaded with attachments.
 only called when a email attachment is being viewed.The mime_info points to the sub
 part .

 @param viewer_info is the pointer to the ViewerInfo structure pointer.
      No memory freeing for viewer_info is done.
 @param filename is the name of the attachment.
 @return void.
*/
void assing_new_mimeinfo(ViewerInfo * viewer_info, const MimeInfo * mime_info);
void
display_attachments(MimeInfo * mimeInfo, GtkListStore * liststore, gint * numAttachments,
		    gboolean first_partflag, gboolean email_attachment);
/**
   updates the toolbar pixels values on toggling from normal/fullscreen view
   @param viewer_info is the ViewInfo structure pointer
   @return void.
*/
void update_toolbar_view(ViewerInfo * viewer_info);
void add_to_bookmark( const gchar * url);
#endif
