/**
 * @file contact_utils.h
 *
 * This file contains the declaration for utility functions used by contacts
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */


#ifndef __CONTACT_UTILS_H__
#define __CONTACT_UTILS_H__


#include <libosso-abook/osso-abook.h>


typedef struct {
        gchar *givenname;
        gchar *surname;
        gchar *email;
        gchar *emailhome;
        gchar *emailbusiness;
        gchar *result_string;
} CheckNameFields;

/**
 * This function returns the resolved recipients for a given EContact.
 * If no e-mail address is defined, it launches 'Add e-mail address to contact'
 * dialog to obtain one. If multiple e-mail addresses are found, it launches
 * 'Select e-mail address' dialog to allow user to select one or more e-mail
 * addresses for that contact.
 *
 * @param  Contact of type #EContact
 * @return List of resolved recipient strings, to be freed by calling function.
 */

GSList *get_recipients_for_given_contact(EContact *contact);


/**
 * This function is used to get econtact from a given contact in string format.
 * @param vieweraddress The formatted Email-address of the recepient.
 *
 * @return EContact 
 */

EContact *get_e_contact_from_string(gchar *contact);

#endif /* __CONTACT_UTILS_H__ */


