/**
 * @file editcallback.h
 * 
 * This file contains the function declarations  & the structures
 * needed for  editor.c
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

#ifndef __EDIT_CALLBACK_H__
#define __EDIT_CALLBACK_H__


/*AL header files*/
#include "al_callbacks.h"
#include "folder.h"
#include "manager.h"

/*UI_DBUS header files*/
#include "email_mgr.h"

#include "editview.h"
/*main_ui related header files*/
#include "ui_al_callbacks.h"
#include "ui_interface.h"
#include "folder_mgmt.h"
#include "message_handler.h"
#include "list_view.h"

/*Includes all the standared header files*/
#include "ui_common.h"
/*editor viewer related header files.*/
#include "viewer.h"
#include "editor.h"

#define DEFAULT_FONT_POS 3
#define HIGHFONT 42
#define DEFAULTFONT 21
#define LOWFONT 10

void ui_msg_showCc_cb(GtkWidget * win, const EditorInfo *);
void ui_msg_showBcc_cb(GtkWidget * win, const EditorInfo *);
HildonAppView *ui_editor_create(HeaderInfo * HeaderInfo, const gchar * body, gint msgType);
void ui_editor_close_cb(GtkWidget * win, EditorInfo * editorInfo);
void ui_toggle_editor_toolbar_cb(GtkWidget *, EditorInfo *);
void ui_toggle_editor_fullscreen_toolbar_cb(GtkWidget *, EditorInfo *);
void ui_msg_copy_cb(GtkWidget * win, MsgFields * fields);
void ui_msg_cut_cb(GtkWidget * win, MsgFields * fields);
void ui_msg_paste_cb(GtkWidget * win, EditorInfo * editorinfo);
void on_zoom_activate(GtkWidget * zoom_menuitem, Zoom *);
void set_format_presel_tag(GtkWidget * zoom_menuitem, EditorInfo * editor_info);
void ui_msg_select_all_cb(GtkWidget * win,MsgFields *fields);
void ui_editor_sendingOption_cb(GtkWidget * win, EditorInfo * editorInfo);
void ui_editor_send_msg_cb(GtkWidget * win, EditorInfo * editorInfo);
void ui_editor_insert_attachment_cb(GtkWidget * win, EditorInfo * editorInfo);
void ui_editor_openAddrBook_cb(GtkWidget * win, GdkEvent * event, GtkTextBuffer * recpBuffer);
void ui_editor_attach_image_cb(GtkWidget * win, EditorInfo * editorInfo);
void ui_editor_attach_file_cb(GtkWidget * win, EditorInfo * editorInfo);
gboolean ui_make_selection_cb(GtkTextView * win, GdkEvent * event, ViewerInfo * viewer_info);
gboolean make_selection(GtkTextView * win, GdkEvent * event, ViewerInfo * viewer_info);
gboolean ui_editor_select_emailid(GtkTextView * view, GdkEvent * event, gpointer data);
gboolean skip_resolved_emailid(GtkWidget * view, GdkEventKey * event, gpointer data);
gboolean skip_emailid_on_key_release(GtkWidget * view, GdkEventKey * event, gpointer data);
void ui_editor_remove_attachment_cb(GtkWidget * win, EditorInfo * editorInfo);
void ui_editor_subjectChanged_cb(GtkWidget * subjectEntry, gint start,
				 gint end, gboolean * modifiedFalg);
void ui_editor_subject_insert_cb(GtkEditable * subjectEntry, gchar * text,
				 gint len, gint * position, gboolean * modifiedFalg);

void ui_editor_check_modified_on_delete(GtkTextBuffer * editor_buffer,
					GtkTextIter * startiter,
					GtkTextIter * enditer, gboolean * modifiedFalg);

void ui_editor_check_modified_on_insert(GtkTextBuffer * editor_buffer,
					GtkTextIter * iter, gchar * inserttext,
					gint len, gboolean * editormodified);

gchar *ui_editor_forward_check(gchar * substr);
gchar *ui_editor_reply_check(gchar * substr);
void ui_invoke_contacts_cb(GtkWidget * win, GtkTextView * view);
void ui_editor_delete_message_cb(GtkWidget * win, EditorInfo * editorInfo);
void ui_read_bodyFile(const gchar * fileName, GtkTextBuffer * buffer_body);
void ui_editor_cancel_save_msg(GSList * mailInfo);
gboolean ui_editor_ok_save_msg(EditorInfo * editorInfo);
void ui_close_all_windows_cb(void);

void ui_editor_free_lists(GSList * quitparam);

void ch_toolbar(GtkWidget * win, GdkEventFocus * event, EditorInfo * editorInfo);
gboolean ch_toolbar_insens(GtkWidget * win, GdkEventFocus * event, EditorInfo * editorInfo);
void update_attach_menu_status(GtkWidget * win, EditorInfo * editorinfo);
void ui_editor_save_all_in_drafts(GtkWidget * appview, EditorInfo * editorInfo);
void ui_mimeinfo_free(MimeInfo * mimeinfo);
MimeInfo *ui_mimeinfo_next(MimeInfo * mimeinfo);
gboolean ui_editor_do_senderCheck(const gchar * fromAddr, const gchar * toAddr);
void ui_freeHeaderInfo(HeaderInfo * headerInfo);
void ui_freeMem(MsgInfo * composeMsg, gchar * body);
void ui_freeHeaderInfo_mimeinfo(MimeInfo *);
void ui_editor_openAddrBook_menu_cb(GtkWidget * win, GtkTextBuffer * recpBuffer);
void ui_editor_select_recipients_cb(GtkWidget * win, MsgFields * fields);
void ui_free_editorinfo(EditorInfo *);
void ui_editor_create_prevlist(EditorInfo *, GSList **);
void ui_file_paths_free(GSList *);
void ui_get_refresh_folder(gint foldernum);
gboolean ui_get_original_in_reply_setting(const gchar * msgid);
gchar *ui_create_filepath_link(const gchar * attachedpath, gchar ** attachmentName,
			       gboolean move, gboolean encode_pass);
gint progress_info(GnomeVFSXferProgressInfo * info, gpointer * data);
void ui_editor_attach_vcard_cb(GtkWidget *, EditorInfo *);
gboolean ui_fill_and_append_attachinfo(GSList * selectionlist,
				       GtkListStore * liststore, MsgFields *, MsgInfo * comMsg);
void handle_ccp_menus(GtkWidget * editmenu, EditorInfo * editor);
/*state save related calls*/
void restore_editor_saved_state(EditorInfo *);
void ui_editor_get_state_values_cb(GtkWidget *, EditorInfo *);

/*security related call*/
void ui_editor_set_encryptmail_cb(GtkWidget *, EditorInfo *);
void show_cc_field(MsgFields *, gboolean);
void show_bcc_field(MsgFields *, gboolean);
void show_attachment_field(MsgFields *, gboolean);
void prepare_the_icon_string(const gchar * mime_string, gchar ** icon_string,
			     gboolean is_downloaded, gboolean is_purged);
void format_dimming_note(GtkWidget * insensitive_widget, EditorInfo * editor);
void set_attachment_view_size(GSList * attach_list, MsgFields * fields);
void ui_freelist(AttachInfo * attInfo, gpointer data);
/**
 this function is used to increase the address field size based on the text entered.
 it increases upto 5 lines then a sollbar appears.

 @param textbuffer is the GtkTextBuffer of the address field.
 @param iter is the GtkTextIter wher the text is inserted.
 @param text is the gchar pointer to the text inserted.
 @param len is the gint for the length os the text inserted.
 @param fieldinfo is the address field info.
 @return void.
 */
void
resize_addr_field(GtkTextBuffer * textbuffer, GtkTextIter * iter, gchar * text,
		  gint len, AddressFieldInfo * fieldinfo);


void ui_editor_fill_smime_type(EmailSpecificSettings * Esettings, gint smime,
			       MsgFields * fields, EditorMenu * menu);
void ui_editor_show_attachments(GtkListStore * liststore, const GSList * attachment_list);
void ui_paste_messages_cb(GtkWidget * wid, EditorInfo * editor);
void ui_editor_fontsettings_cb(GtkWidget * wid, EditorInfo * editor);
void set_default_font(EditorInfo * editorInfo, EmailSpecificSettings * Esettings);

/*
setting the editor menus after zooming with hardware keys.

@param appview is the pointer to the editor appvieW.
@param editor_info is the pointer to the editorInfo structure pointer.

@return void.

*/
void set_editor_zoom_menu(GtkWidget * appview, EditorInfo * editor_info);
gboolean
ui_editor_validate_addresses(const gchar * Field, gchar ** Recepients,
			     GtkTextBuffer * addressBuffer);
void ui_make_temp_dir(void);
gchar *get_short_filename(const gchar * path, gchar **);
void ui_editor_get_new_msgid_cb(GtkWidget * editor, EditorInfo * editor_info);
void recipient_field_empty_check(GtkWidget * win, EditorInfo * Editor_info);
void ui_editor_msginfo_thr_cb(GtkWidget * editor, EditorInfo * editor_info);
/**
  helper function to stop the editor from recieving any events when the process of
  saving/opening.
  @param editor_info is the structure pointer to the editorInfo structure pointer.
  @param enable is the gboolean ,whether to allow /diaallow handling of signal in editor.
  @return void.
*/
void stop_editor_from_recieving_events(EditorInfo * editor_info, gboolean enable);
void updatemodified(GtkTextBuffer * buffer, GtkTextTag * tag,
                               GtkTextIter * start_it, GtkTextIter * end_it, gboolean * modified);
#endif
