/**
 * @file editor.h
 *
 * This file contains the function declarations  & the structures
 * needed for viewer and editor modules of OSSO_EMAIL.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/


#ifndef __EDITOR_H__
#define __EDITOR_H__

#include "viewer.h"
#include "ui_utils.h"
#include <hildon-widgets/hildon-color-button.h>
typedef struct _EditorInfo EditorInfo;
typedef struct _EditorMenu EditorMenu;
typedef struct _EditorToolbar EditorToolbar;
typedef struct _HTMLText HTMLText;
typedef struct _SettingsInfo SettingsInfo;
typedef struct _HTMLOffsetInfo HTMLOffsetInfo;
typedef struct _HTMLTagInfo HTMLTagInfo;
typedef struct _GtkOffsetInfo GtkOffsetInfo;

#define SUBJECT_MAX_CHARS 256
#define SUBJECT_MAX_WIDTH 1000
#define ACCOUNT_SEPERATOR '_'
#define AT_THE_RATE '@'
#define DOMAIN_SEPERATOR '.'
#define SEMICOLON_SEPERATOR ";"
#define TIME_STRING_SPACE 100
#define DIALOG_WIDTH 400
#define DIALOG_HEIGHT 300
#define DEFAULT_ADDR_FIELD_SZ 35
#define MAX_ADDR_FIELD_SZ 148
#define MAX_VIEWER_ADDR_FIELD_SZ 53
#define DEFAULT_ADDR_FIELD_CUR_SZ 30
#define CURSOR_HEIGHT 22
#define SUBJECT_WIDTH_CHARS 20
#define MAX_ROWS 5
#define MAX_COLS 4
enum {
	PRIORITY_NORMAL,
	PRIORITY_LOW,
	PRIORITY_HIGH
};
enum {
	NULL_USERNAME,
	NULL_PASSWD
};


enum {
	PLAIN_MAIL,
	HTML_MAIL
};

enum {
	NONE,
	ENCRY,
	SIGN,
	SIGN_ENCRY
};

enum {
	IMMEDIATELY,
	NEXT_CONN,
	ON_REQUEST
};

struct _EditorMenu {
	GtkWidget *send;
	GtkWidget *delete_msg;
	GtkWidget *select_recipients;
	GtkWidget *check_names;
	GtkWidget *add_to_desk;

	GtkWidget *undo;
	GtkWidget *cut;
	GtkWidget *copy;
	GtkWidget *paste;
	GtkWidget *select_all;
	GtkWidget *find;
	GtkWidget *searchemails;
	GtkWidget *attach_file;
	GtkWidget *attach_image;
	GtkWidget *remove_attachment;

	GtkWidget *fullscreen;
	GtkWidget *toolbar_fullscreen;
	GtkWidget *toolbar_normalview;
	GtkWidget *show_cc;
	GtkWidget *show_bcc;
	GtkWidget *zoom_out_level;
	GtkWidget *zoom_out_max_level;
	GtkWidget *current_zoom_level;
	GtkWidget *zoom_in_level;
	GtkWidget *zoom_80;
	GtkWidget *zoom_120;
	GtkWidget *format_menuitem;
	GtkWidget *font;
	GtkWidget *bullets;
	GtkWidget *numbering;
	GtkWidget *left;
	GtkWidget *centred;
	GtkWidget *right;
	GtkWidget *none;
	GtkWidget *bgcolor;
	GtkWidget *sign;
	GtkWidget *encrypt;
	GtkWidget *sign_encrypt;
	GtkWidget *no_encrypt;
	GtkWidget *close_all_windows;
	GdkColor bgcolorvalue;
};

struct _EditorToolbar {
	GtkWidget *toolbar;
	GtkWidget *send;
	GtkWidget *attachment;
	GtkWidget *recipients;
	GtkWidget *fontname;
	GtkWidget *fontsize;
	GtkWidget *bold;
	GtkWidget *italic;
	GtkWidget *fgcolor;
	HildonColorButton *color_btn;
	GtkWidget *bullets;
};

typedef enum {
	NEW_MAIL = 0,
	REPLY,
	REPLY_ALL,
	FORWARD,
	REPLY_FROM_FOLDER,
	REPLY_ALL_FROM_FOLDER,
	FORWARD_FROM_FOLDER,
	DRAFT_MAIL,
	OUTBOX_MAIL,
	FAILED_MAIL,
	SEND_AS
} ComposeType;

typedef enum {
	PLAIN_EDITOR,
	HTML_EDITOR
} EditorType;


struct _HTMLOffsetInfo {
	gchar *start_htmltag;
	gchar *end_htmltag;
	gint start_offset;
	gint end_offset;
};

typedef enum {
	FAMIL = 1,
	BLD,
	ITALIC,
	FGCOLOR,
	BGCOLOR,
	UNDERLINE,
	SUBSCRIPT,
	SUPERSCRIPT,
	ALIGNLEFT,
	ALIGNCENTRE,
	ALIGNRIGHT,
	STRIKETHRU
} HTMLProperty;



struct _HTMLTagInfo {
	gchar *formatname;
	HTMLProperty property;
	GtkTextTag *tag;
};


typedef enum {
	BLOCKED,
	UNBLOCKED
} SignalState;


typedef enum {
	OSSO_JUSTIFY_LEFT = GTK_JUSTIFY_LEFT,
	OSSO_JUSTIFY_RIGHT = GTK_JUSTIFY_RIGHT,
	OSSO_JUSTIFY_CENTER = GTK_JUSTIFY_CENTER,
	OSSO_JUSTIFY_FILL = GTK_JUSTIFY_FILL,
	OSSO_JUSTIFY_NONE = GTK_JUSTIFY_FILL + 1
} OssoJustification;



struct _EditorInfo {
	GtkWidget *editor_appview;
	GtkWidget *editor_toolbar;
	GtkWidget *editor_fullscreen_toolbar;
	EditorMenu *menu;
	MsgFields *msgFields;
	EditorToolbar *toolbar;
	MsgInfo *msgInfo;
	EditorType emailType;
	MimeInfo *mimeInfo;
	Zoom *zoominfo;
	AddressFieldInfo *fieldinfo_to;
	AddressFieldInfo *fieldinfo_cc;
	AddressFieldInfo *fieldinfo_bcc;
	GSList *addressBuffers;
	gchar *msgid;
	gchar *old_msgid;
	PangoAttrList *attr_list;
	OssoJustification justification;
	ComposeType composeType;
	PrefsAccount *accountSettings;
	gint folderNum;
	gboolean editorModified;
	gboolean toolbarhidden_normalview;
	gboolean toolbarhidden_fullscreen;
	gboolean autosaved;
	gboolean headersonly;
	gboolean settings_changed;
	gint msgnum;
	GQueue *undo_queue;
	GtkWidget *undofield;
	GdkColor *bgcolor;
	GdkColor *prevcolor;
	GtkStyle *body_style;
	gboolean events_blocked;
	gboolean formating_preselected;
	GtkTextMark *insert_mark;
	gboolean ignore_strikethrough;
	gboolean ignore_underline;
};
struct _SettingsInfo {
	GtkWidget *settings_dialog;
	EditorInfo *editor_info;
};
void ui_free_editorinfo(EditorInfo *);



#endif
