/**
 * @file editview.h
 *
 * This file contains the function declarations  & the structures
 * needed for both editInterface.c & viewer.c files
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

#ifndef __EDIT_VIEW_H__
#define __EDIT_VIEW_H__

#include<gtk/gtk.h>
#include "editor.h"
#include "statesave.h"

typedef struct _HeaderInfo HeaderInfo;

typedef struct {
	MsgInfo *msgContents;
	gchar *msg_body;
} _msgstruct;

struct _HeaderInfo {
	gchar *msgid;
	gchar *filename;
	gchar *email_filename;
	gint date;
	gint sent_date;
	gint recieve_date;
	gint creation_date;
	gchar *from;
	gchar *to;
	gchar *cc;
	gchar *bcc;
	gchar *subject;
	gchar *reply_to;
	gint priority;
	RecvType msgType;
	gboolean fully_downloaded;
	gboolean body_downloaded;
	gboolean download_attachments;
	gboolean email_attachment;
	gint protocol;
	MimeInfo *mimeInfo;
	gint folderNum;
	gint rowNum;
	gboolean read_receipt;
	gboolean copy_to_own_addr;
	gboolean in_use;
	gboolean editable_status;
	gboolean original_in_reply;
	gboolean read_status;
	gboolean set_prev_settings;
	SMimeType smime_type;
	gchar *password;
	guint certid;
	MsgInfo *msgInfo;
	gint zoomlevel;
	gboolean headers_hidden;
};

void ui_freeHeaderInfo(HeaderInfo *);
GSList *ui_get_selected_attachment_name(const MsgFields *);
gint ui_get_attachment_num(const MsgFields *, gint);
gboolean ui_remove_attachment_from_list(const MsgFields * msgFields, GtkWidget * remove_Att);
void ui_attachment_selection(GtkTreeSelection *, const EditorInfo *);
void ui_create_attachment_list_view(MsgFields *, GtkWidget *, gboolean);
void ui_view_topped_cb(const HildonAppView * view, gpointer data);

/**
 This function shows the find toolbar.

 @param find_toolbar is the pointer to the find toolbar widget.
 @param msgFields is the pointer to the Msgfields structure pointer.

 @return void.
*/
void show_find_toolbar_cb(GtkWidget * find_toolbar, MsgFields * msgFields);

/**
 This function creates the find toolbar.

  @param msgFields is the pointer to the MsgFields structure.
  @param vbox_editor is the pointer to the vbox in which the widgets
        are packed.
  @param find_menu is the pointer to the find menu widget.

  @return void.
*/
void create_find_toolbar(MsgFields * msgFields, GtkWidget * vbox_editor, GtkWidget * find_menu);



#endif
