/**
 * @file email_contacts.h
 *
 * This file contains the declarations for email_contacts.c file 
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */


#ifndef __EMAIL_CONTACTS_H__
#define __EMAIL_CONTACTS_H__

#include <gtk/gtk.h>

/**
 * This public function starts the email recipients dialog and sets the contacts list to the given text buffer
 * @param recp_buffer of type #GtkTextBuffer 
 *
 * @return #void 
 */

void start_email_recipients_dialog(GtkTextBuffer *recp_buffer);

/**
 * This public function creates the 'Add to contacts' dialog.
 * User can choose to add the email_address to an existing contact, or as a new contact.
 * 
 * @param contact of type #gchar
 *    
 * @return void 
 */

void create_contact_dialog(const gchar *contact);

/**
 * This public function stops the book view and unrefs the book and book view.
 * @return void
 */

void shutdown_addressbook(void);

/**
 *  This is a helper function to add new contacts to the address book. Input is list
 *  of email addresses, which are then converted to EContact objects for adding to
 *  AddressBook.
 *  @param resolved_contacts of type #GSList
 *     
 *  @return void 
 */

void add_contacts_to_addressbook(GSList * resolved_contacts);

/**
 * This is a helper function to commit a EContact to Address_Book application.
 *
 * @param  contact  Contact of type #EContact
 * @return void
 */

void commit_contact (EContact *contact);

/**
 * This is helper function to add recently used EContacts to recent_group of the
 * Address Book application.
 *
 * @param  used_econtact_uids  List of UIDs of used EContacts
 * @return void
 */
void add_contacts_to_recent_list(GSList * used_econtact_uids);

#endif /* __EMAIL_CONTACTS_H__ */


