/**
 * @file email_mgr.h
 *
 * This file contains the declarations for functions for UI DBus module 
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


#ifndef EMAIL_MGR_H_
#define EMAIL_MGR_H_

#include <glib.h>
#include "common.h"
#include "utils.h"
#include "prefs_account.h"
#include "procmsg.h"
#include "folder.h"
#include "statesave.h"
#include "dbus_utils_xml.h"

#if CL_USE
#include "cl.h"
#endif

#define MAX_SET_MSG_CB_ARGS 7

typedef struct _UI_DBusFuncs UI_DBusFuncs;

struct _UI_DBusFuncs {
	gchar *hname;
	 gint(*msghandler) (const gchar * recv_str);
};


void ui_dbus_init(void);

osso_pack_return_t ui_dbus_is_ready(void);

gboolean ui_dbus_is_uidbus_server_ready(void);

gchar *get_mimeComponent(gchar * packed_string, int type);

osso_pack_return_t ui_dbus_send_mail(const MsgInfo * msginfo, const gchar * message);

osso_pack_return_t
ui_dbus_send_outbox_mail(const gchar * account, const gchar * msgid, gboolean keepconnalive);

osso_pack_return_t ui_dbus_cancel_send_mail(const GSList * msgid_list);

osso_pack_return_t
ui_dbus_send_save_draft_mail(const MsgInfo * msginfo,
			     const gchar * message, gboolean flag, gchar * old_id);

/**
  This function gets the message ID from engine before saving a mail
  in draft folder or outbox

  @param msginfo - MsgInfo structure
  @param msgid - This is a return paramater and will contain 
  		message ID
*/
osso_pack_return_t
ui_dbus_send_get_msg_id(const MsgInfo * msginfo, gchar ** msgid, gboolean send_flag);

osso_pack_return_t ui_dbus_send_draft_mails(const GSList * msgid_list);

osso_pack_return_t ui_dbus_send_selected_mails(gint folderType, const GSList * msgid_list);

osso_pack_return_t
ui_dbus_recv_msg_multiple_attachments(const GSList * mime_list,
				      const GSList * attach_tag_list, const gchar * msgid);

osso_pack_return_t ui_dbus_recv_mail(const gchar * account);

gchar *ui_dbus_recv_multipart_body(const MimeInfo * mimeinfo,
				   const gchar * msgid, const gchar * font_name);
MimeInfo *ui_dbus_recv_msg_body(const gchar * msgid,
				const gboolean cbflag,
				gboolean complete_download,
				gchar * signer_email_id,
				gchar * recip_email_id,
				gint smime_type,
				gboolean read_flag,
				guint cert_id,
				gchar * password, gboolean edit_flag, gboolean html_flag);

MsgInfo *ui_dbus_recv_msg_attachment(const gchar * msgid, const MimeInfo * mimeinfo);

osso_pack_return_t
ui_dbus_cancel_recv_mail(const gchar * account_name, gboolean send_on_normal_conn);

osso_pack_return_t ui_dbus_folder_create(const gchar * name, const gchar * path, FolderType type);

osso_pack_return_t ui_dbus_folder_delete(const gchar * name);

osso_pack_return_t ui_dbus_folder_rename(const gchar * old_name, const gchar * new_name);

osso_pack_return_t
ui_dbus_folder_remove_msg(const gchar * folder, const gchar * msgid, gboolean flag);

osso_pack_return_t
ui_dbus_folder_remove_msgs(const gchar * srcfolder, const GSList * msglist, gboolean flag);

osso_pack_return_t ui_dbus_folder_remove_all_msg(const gchar * folder, gboolean flag);

osso_pack_return_t
ui_dbus_folder_move_msg(const gchar * srcfolder, const gchar * destfolder, const gchar * msgid);

osso_pack_return_t
ui_dbus_folder_move_msgs(const gchar * srcfolder,
			 const gchar * destfolder, const GSList * msgid_list);

osso_pack_return_t ui_dbus_folder_get_list(GSList ** folder_list);

GSList *ui_dbus_populate_folder(const gchar * name, gint index);

GSList *ui_dbus_populate_folder_callback(const gchar * recv_string);

osso_pack_return_t
ui_dbus_folder_copy_msg(const gchar * srcfolder, const gchar * destfolder, const gchar * msgid);

osso_pack_return_t
ui_dbus_folder_copy_msgs(const gchar * srcfolder,
			 const gchar * destfolder, const GSList * msgid_list);

GSList *ui_dbus_folder_remove_attachment(GSList * folder_attach_list);

/**
  This function removes all attachments in the mail

  @param folder_attach_list list of attachments to be removed
  @param List of attachment not removed or NULL if all are removed
*/
GSList *ui_dbus_folder_remove_all_attachment(GSList * folder_attach_list);

/**
  This Function is called by UI module to cancel folder operation 
  @param none 
  @return on successful operation returns OSSO_PACK_OK. Otherwise 
   returns errorcode 
*/
osso_pack_return_t ui_dbus_folder_cancel(void);


GSList *ui_dbus_get_account_list(void);

osso_pack_return_t
ui_dbus_get_account_settings(const gchar * account,
			     SettingType type, PrefsAccount ** prefs_account);

osso_pack_return_t
ui_dbus_set_account_settings(SettingType type, const PrefsAccount * prefs_account);

osso_pack_return_t ui_dbus_delete_account(const gchar * account_name);

osso_pack_return_t ui_dbus_rename_account(const gchar * old_name, const gchar * new_name);

osso_pack_return_t ui_dbus_folder_get_size(const gchar * folder);

osso_pack_return_t ui_dbus_get_engine_errno(void);

osso_pack_return_t ui_dbus_get_engine_init_status(void);

gboolean
ui_dbus_html_to_text_converter(const char *filename, const gchar * charset, gchar ** convertedtext);

osso_pack_return_t
ui_dbus_ui_tasknav_window_operation(gint operation,
				    gint windowid, const gchar * arg1, const gchar * arg2);

osso_pack_return_t ui_dbus_autosave_mail(const gchar * msgid, gint operation);

GSList *ui_dbus_get_msgheader_list(GSList * msgid_list);

void ui_dbus_unpack_str_base64(const guchar * base64data, gchar ** convdata);

osso_pack_return_t ui_dbus_editor_open_mail(const gchar * msgid, gint operation);

osso_pack_return_t ui_dbus_editor_create(const gchar * recv_string);

osso_pack_return_t ui_dbus_statesave_write(const EmailStateInfo * stateInfo);

osso_pack_return_t ui_dbus_statesave_read(EmailStateInfo ** stateInfo);

osso_pack_return_t ui_dbus_statesave_test(gint opeartion);

osso_pack_return_t ui_dbus_persistentInfo_read(EmailPersistentInfo ** perInfo);

osso_pack_return_t ui_dbus_persistentInfo_write(EmailPersistentInfo * perInfo);

/**
  Function to check if callback has been sent or not

  @param account name of account
  @return 1 on TRUE else FALSE
*/
gint ui_dbus_account_check_status(const gchar * account);

/**
    This Function is called by UI module to get the list of accounts 
    @param account_list GSList of account structures given by engine module
     populated in this list
    @return returns account errorcode 
*/
osso_pack_return_t ui_dbus_prefs_account_get_list(GSList ** prefs_account_list);

/**
   This method is called when DBus module receives response contains the  
   message attachment of the requested message id.
   @param  recv_str contains funcid:xmldata <msginfo>
   @return MsgInfo on success or NULL of failure
*/
MsgInfo *ui_dbus_recv_msg_attachment(const gchar * msgid, const MimeInfo * mimeinfo);

/**
   This method is called when message has mhtml email attachment.
   @param  mimeinfo mimeinformation pointer to the beginning of 
   the email attachment 
   @param  msgid the message id of the mesasge for which this operation 
   has to be done 
   @param  attach_no the sequence number representing the attachemnt 
   @param  default_font is the default_font for viewing the attcahment. 
   @return MsgInfo on success or NULL of failure
*/
MsgInfo *ui_dbus_recv_multipart_attachment(const gchar * msgid,
					   const MimeInfo * mimeinfo, gint attach_no,
					   const gchar * default_font);


/**
   This method is called to set temporary passwd for send/receive operation 
   @param  account_name    name of the account to set the password for 
   @param  passwd    temporary password 
   @param  cancel    indicates whether user has cancelled the operation 
   @return OSSO_PACK_OK on succes or OSSO_PACK_INVALID on failure 
*/
osso_pack_return_t
ui_dbus_set_tmp_passwd(const gchar * account_name,
		       const gchar * passwd, gboolean cancel, gboolean other_conn);
/**
   This method is called to purge attachments from other directories 
   @param  msgid    msgid of the file,whose attachments are to be purged 
   @param  file_pos_list    list of file positions of the attachments,
   which have to be purged.All attachments belong to same message 
   @return OSSO_PACK_OK on succes or OSSO_PACK_INVALID on failure 
*/
gint ui_dbus_rem_msg_attachments(const gchar * msgid, const GSList * file_pos_list);

#if CL_USE
GSList *ui_dbus_get_iapinfo_list(void);
gboolean ui_dbus_set_iapinfo(IapInfo *iapinfo);
#endif

/**
  Function to check if engine is ready
  
  @return TRUE if ready else FALSE
*/
gboolean ui_dbus_is_engine_ready(void);
void ui_recv_msg_attachment_cb(MsgInfo * msginfo);
void ui_exit(gint);

/**
  Function to initiate send.recv process

  @param account - Name of account for which send/recv needs to be done
  @return on successful operation returns OSSO_PACK_OK. Otherwise
  returns errorcode
*/
osso_pack_return_t ui_dbus_send_recv_mail(const gchar * account);

/** 
 Function to inform engine to disconnect IAP connection

 @return on successful operation returns OSSO_PACK_OK. Otherwise
 returns errorcode
*/
osso_pack_return_t ui_dbus_disconnect_iap_conn(void);

/**
  This function informs engine to retrieve body for headers

  @retr_list list of messages for which headers are required
*/
void ui_dbus_retrieve_msgs(GSList * retr_list, gboolean copy);
/**
 Function to update the seen flag for a message on the server for a imap mail
 after its read
*/
void ui_dbus_update_seen_flag(const gchar *msgid);
#endif				/* EAIL_MGR_H_ */
