/**
 * @file folder_mgmt.c
 *
 * This is the header file for folder management module.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#ifndef __FOLDERMGMT_H__
#define __FOLDERMGMT_H__

#include "ui_state_mgmt.h"

#define MAXLEN 1000
#define FOL_SETT_MAX 999
#define FOL_SETT_MIN 1

typedef struct _PopupFolderMenu PopupFolderMenu;

/*Structures & Enumerations*/
enum {
	VIEW_FOLDER_LIST = 0,
	VIEW_MSG_LIST
};

enum {
	COL_FOLDER_NAME = 0,
	COL_MODEL_SEQ_NUM,
	COL_USER_CREATED,
	COL_SORT_BY,
	COL_SORT_ORDER,
	COL_FOLDER_ICON,
	COL_PARENT_NAME,
	COL_DISPLAY_NAME,
	COL_UNREAD_UNSENT_COUNT,
	COL_FOLDER_POPULATED,
	NUM_FOLDER_VIEW_COLS
};

enum {
	COL_MANAGE_MODEL_SEQ_NUM,
	COL_MANAGE_USER_CREATED,
	COL_MANAGE_FOLDER_ICON,
	COL_MANAGE_DISPLAY_NAME,
	COL_MANAGE_IS_SUBFOLDER,
	NUM_MANAGE_VIEW_COLS
};

enum {
	COL_MSG_ID = 0,
	COL_FROM_ADDR,
	COL_TO_ADDR,
	COL_CC_ADDR,
	COL_BCC_ADDR,
	COL_SENDER_RECIPIENT,
	COL_SUBJECT,
	COL_REPLY_TO,
	COL_DATE_STATUS,
	COL_DISPLAY_DATE,
	COL_SENT_DATE,
	COL_RECV_DATE,
	COL_CREATED_DATE,
	COL_BODY,
	COL_READ_STATUS,
	COL_MSG_ICON,
	COL_ATTACHMENT_PRESENT,
	COL_ATTACHMENT_ICON,
	COL_PRIORITY,
	COL_PRIORITY_ICON,
	COL_MARK_STATUS,
	COL_SIZE,
	COL_RECV_PROTOCOL,
	COL_MSG_TYPE,
	COL_FOLDER_TYPE,
	COL_FULLY_DOWNLOADED,
	COL_BODY_DOWNLOADED,
	COL_REMOTE_COPY,
	COL_IN_USE_STATUS,
	COL_ACTION_AFTER_RETRIEVAL,
	COL_READ_RECEIPT,
	COL_COPY_TO_ADDR,
	COL_EDITABLE_STATUS,
	COL_SMIME_TYPE,
	COL_MSG_ID_KEY,
	COL_SENDER_RECIPIENT_KEY,
	COL_SUBJECT_KEY,
	COL_NULL_SUBJECT,
	COL_PURGE_ALLOW,
	COL_CUT_STATUS,
	COL_MSGID_HASH,
	NUM_LIST_VIEW_COLS
};

typedef enum {
	SORT_BY_ADDRESS = 0,
	SORT_BY_DATE,
	SORT_BY_SUBJECT,
	SORT_BY_SIZE,
	SORT_BY_PRIORITY,
	SORT_BY_READ_STATUS,
	SORT_BY_ATTACHMENT
} sortingField;

typedef enum {
	ORDER_ASCENDING = 0,
	ORDER_DESCENDING
} sortingOrder;

typedef enum {
	ACTION_MOVE = 0,
	ACTION_CUT,
	ACTION_COPY,
	ACTION_PASTE,
	ACTION_DELETE
} msgHandlingAction;

typedef enum {
	DO_NOT_EMPTY = 0,
	DAYS,
	WEEKS,
	MONTHS
} folderCleanOption;

struct _PopupFolderMenu {
	gchar *parent_folder;
	gchar *folder_name;
	GtkWidget *menu_item;
};

/*Window creation Functions*/
GtkWidget *ui_create_manage_folders_dialog(GtkTreeView * main_folder_view);
void ui_create_sort_messages_dialog(GtkTreeView * main_folder_view);
GtkWidget *ui_create_folder_settings_dialog(void);

/*Callback function signatures*/
void ui_on_new_manage_folders_dialog(GtkWidget * widget, gpointer data);
void ui_on_delete_manage_folders_dialog(GtkWidget * widget, gpointer userdata);
void ui_on_rename_manage_folders_dialog(GtkWidget * widget, gpointer userdata);

/*Helper Functions*/
void ui_add_new_folder(GtkTreeStore * folder_list_store, gchar * folder_path,
		       gboolean user_created, gint unread_unsent_count);
gboolean ui_is_header_already_present(GtkTreeModel * model, GtkTreePath * path,
				      GtkTreeIter * iter, gpointer data);
gboolean ui_update_msg_fully_downloaded(GtkTreeModel * model,
					GtkTreePath * path, GtkTreeIter * iter, gpointer data);
gboolean ui_update_msg_body_downloaded(GtkTreeModel * model,
				       GtkTreePath * path, GtkTreeIter * iter, gpointer data);
gboolean ui_update_action_after_retrieval(GtkTreeModel * model,
					  GtkTreePath * path, GtkTreeIter * iter, gpointer data);
gboolean ui_get_action_after_retrieval(GtkTreeModel * model,
				       GtkTreePath * path, GtkTreeIter * iter, gpointer data);
gboolean ui_update_msg_attachment_icon(GtkTreeModel * model,
				       GtkTreePath * path, GtkTreeIter * iter, gpointer data);
gboolean ui_update_msg_dustbin_icon(GtkTreeModel * model, GtkTreePath * path,
				    GtkTreeIter * iter, gpointer data);
gchar *ui_get_folder_name(guint model_num);
void ui_empty_sent_folder(gboolean show_note);
GtkTreeModel *ui_get_sorted_folder_model(gint model_number);
GtkTreeModel *ui_get_unsorted_folder_model(gint model_number);

/* Folder pop-up fuctions */
gint ui_get_folder_row_position_from_view(gchar * folder_name, gchar * parentFolder);
void ui_on_folder_menu_activated(GtkWidget * widget, gpointer data);
void ui_select_folder_menu_item(PopupFolderMenu * folderMenu);
gboolean ui_count_unread_rows(GtkTreeModel * model, GtkTreePath * path,
			      GtkTreeIter * iter, gpointer data);
void ui_on_folder_menu_select_activated(GtkWidget * widget, gpointer data);
gint ui_get_folder_model_number(const gchar * folder_name, const gchar * parent_folder);
gint ui_on_rename_folder_menu(gchar * current_folder_name, gchar * newFolderName);
void ui_delete_folder_from_menu(const gchar * folder_name);
void ui_set_folder_model_to_msgview(gint model_num, gboolean set_sorted_model);
void ui_display_inbox_folder_view(void);
void ui_apply_folder_sorting(FolderSortInfo * sort_info);
void ui_select_row_from_view(gchar * row_identifier, gint view_type);
GSList *ui_prepare_folder_sort_settings(void);

/* Drag and Drop */
void drag_data_received_handler(GtkWidget * treeview,
				GdkDragContext * drag_context, gint x, gint y,
				GtkSelectionData * data, guint info,
				guint event_time, gpointer user_data);

void drag_begin_handler(GtkWidget * treeview, GdkDragContext * drag_context, gpointer user_data);

void drag_data_get_handler(GtkWidget * treeview,
			   GdkDragContext * drag_context,
			   GtkSelectionData * data, guint info, guint time, gpointer user_data);

void drag_end_handler(GtkWidget * treeview, GdkDragContext * drag_context, gpointer user_data);
gboolean drag_motion_handler(GtkWidget * view, GdkDragContext * drag_context, gint x, gint y,
			     guint time, gpointer user_data);

gchar *ui_folder_name_with_unread_mails(const gchar * folderpath);

gboolean
ui_get_folder_names_from_path(const gchar * path, gchar ** parent_folder, gchar ** sub_folder);
gboolean ui_get_folder_iter(const gchar * folder_name, GtkTreeIter * iter);
gchar *ui_get_parent_folder_name(const gchar * folder_name);

void ui_free_folder_menu_items_list(void);
void ui_parent_folder_menu_selected(GtkWidget * widget, gpointer data);
PopupFolderMenu *ui_get_folder_activated_structure(gchar * parent_name, gchar * child_name);
void ui_create_folder_properties_dialog(GtkWidget * win, gpointer data);
void ui_on_message_properties_clicked(GtkWidget * win, gpointer data);
const gchar *ui_get_folder_display_name(gint folder_num, const gchar * folder_name);
gboolean ui_reset_folder_msg_count(GtkTreeModel * model, GtkTreePath * path,
				   GtkTreeIter * iter, gpointer data);
gboolean ui_get_folder_sort_order(GtkTreeModel * model, GtkTreePath * path,
				  GtkTreeIter * iter, gpointer data);
gboolean ui_set_folder_sort_order(GtkTreeModel * model, GtkTreePath * path,
				  GtkTreeIter * iter, gpointer data);
void ui_on_folder_delete_menu_item_dimmed(GtkWidget * widget, gpointer userdata);
void ui_on_folder_rename_menu_item_dimmed(GtkWidget * widget, gpointer userdata);
gboolean ui_delete_all_msgs_from_folder(gint model_num, gchar * folder_name);
void ui_load_icon_pixbuf(GdkPixbuf ** icon, const gchar * icon_name, gint icon_size);
void ui_add_folder_name_with_unread_unsent_mails(gint model_num);
GdkPixbuf *ui_get_folder_icon(gint folder_num);
void ui_on_ok_sort_messages_dialog(gint sort_key, GtkSortType sort_order);
#endif

/* end of file */
