/**
 * @file html_parser.h
 *
 * This file contains the function declarations  & the structures
 * needed for  html_parser.c
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


#ifndef __GTK_TO_HTML_H__
#define __GTK_TO_HTML_H__

#include <gtk/gtk.h>
#include "editcallback.h"

#define META_LEN 5
#define META_TAG "<META>"
#define OSSO_EMAIL_HEADER "<HTML><HEAD><TITLE></TITLE><style type = \"text/css\"><!-- body { font-family:\"\"  } --> </style><META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=UTF-8\"></HEAD>"
#define OSSO_EMAIL_FOOTER "</BODY></HTML>"
#define TAG_HTML_START "<HTML>"
#define TAG_HTML_END "</HTML>"
#define TAG_HEAD_START "<HEAD>"
#define TAG_HEAD_END "</HEAD>"
#define TAG_TITLE_START "<TITLE>"
#define TAG_TITLE_END "</TITLE>"
#define TAG_BODY_START "<BODY>"
#define TAG_BODY_END "</BODY>"

#define HTML_TAG_BOLD "<B>"
#define HTML_TAG_ITALIC "<I>"
#define HTML_TAG_UNDERLINE "<U>"
#define HTML_TAG_STRIKE_THROUGH "<STRIKE>"
#define HTML_TAG_SUPERSCRIPT "<SUP>"
#define HTML_TAG_SUBSCRIPT "<SUB>"
#define HTML_TAG_LEFT_ALIGN "<DIV ALIGN=\"LEFT\">"
#define HTML_TAG_CENTER_ALIGN "<DIV ALIGN=\"CENTER\">"
#define HTML_TAG_RIGHT_ALIGN "<DIV ALIGN=\"RIGHT\">"
#define HTML_TAG_BULLET "<ul>"
#define HTML_TAG_NUMBER "<ol>"
#define HTML_TAG_LIST_ITEM "<li>"
#define HTML_TAG_FONT "<font face=\""
#define HTML_TAG_FONT_END "pt\">"
#define HTML_TAG_FG "<FONT COLOR=\"#"
#define HTML_TAG_BG "<BODY BGCOLOR=\"#"


#define HTML_CLOSE_TAG_BOLD "</B>"
#define HTML_CLOSE_TAG_ITALIC "</I>"
#define HTML_CLOSE_TAG_UNDERLINE "</U>"
#define HTML_CLOSE_TAG_STRIKE_THROUGH "</STRIKE>"
#define HTML_CLOSE_TAG_SUPERSCRIPT "</SUP>"
#define HTML_CLOSE_TAG_SUBSCRIPT "</SUB>"
#define HTML_CLOSE_TAG_LEFT_ALIGN "</DIV>"
#define HTML_CLOSE_TAG_CENTER_ALIGN "</DIV>"
#define HTML_CLOSE_TAG_RIGHT_ALIGN "</DIV>"
#define HTML_CLOSE_TAG_BULLET "</ul>"
#define HTML_CLOSE_TAG_NUMBER "</ol>"
#define HTML_CLOSE_TAG_LIST_ITEM "</li>"
#define HTML_CLOSE_TAG_LIST_LEN 5
#define HTML_CLOSE_TAG_FONT "</font>"
#define HTML_CLOSE_TAG_FG "</FONT>"
#define HTML_CLOSE_TAG_BG "</BODY>"

#define HTML_TAG_GT "&gt;"
#define HTML_TAG_LT "&lt;"
#define HTML_TAG_AMP "&amp;"
#define HTML_TAG_SPC "&nbsp;"
#define HTML_TAG_NL  "<BR>\n"
#define HTML_TAG_NL_LEN 4

#define HTML_LT "<"
#define HTML_GT ">"
#define HTML_SPC " "
#define HTML_AMP "&"
#define HTML_NL "\n"
#define ESCAPE "\""

#define HTML_CLOSE_LT "</"
#define HTML_CLOSE_GT "\">"
#define HTML_LT_CHAR '<'
#define HTML_GT_CHAR '>'
#define HTML_SPC_CHAR ' '
#define HTML_AMP_CHAR '&'

#define COLOR_VAL_LEN 3
#define COLOR_BUF 7
#define TERM_0 '\0'
#define CHAR_0 '0'

#define FORMAT_ST "strikethrough"
#define FORMAT_BLD "weight"
#define FORMAT_FG "foreground"
#define FORMAT_BG "background"
#define FORMAT_BG_COL "background:"
#define FORMAT_UL "underline"
#define FORMAT_IT "italic"
#define FORMAT_SUB "subscript"
#define FORMAT_SUP "superscript"
#define FORMAT_AL_L "alignleft"
#define FORMAT_AL_R "alignright"
#define FORMAT_AL_C "aligncentre"
#define STYLE  "style"
#define STYLE_LEN  5
#define FOREGROUND_PRE  "foreground_pre"
#define FOREGROUND_PRE_COL  "foreground_pre:"
#define FOREGROUND_COL  "foreground:"
#define FOREGROUND_GDK  "foreground-gdk"
#define LEFT  "left"
#define CENTER  "center"
#define RIGHT  "right"
#define ALIGN  "align"
#define BOLD  "Bold"
#define ITALICS  "Italic"
#define FAMILY "family"
#define FAMILY_COL "family:"
#define SIZE_COL "size:"
#define SIZE "size"
#define FONT_SIZE "style=\"font-size:"

#define IMAGE_TAG "IMG"
#define JPEG_IMAGE "jpeg"

#define TAG_ERROR -1
#define CID "cid:"
#define CID_LENGTH 4
#define TEMP_IMAGE_PATH "/tmp/email/tmp/image_"
#define IMAGE_ "image_"


/**
This function will be invoked to prepare html header. The header
consists of <HTML> tag <HEAD> tag <TITLE> start and end tag
@return htmlheader
*/
gchar *osso_email_editor_prepare_header(void);

/** 
This function will be invoked to prepare html footer. The footer 
consists of </BODY> tag </HEAD> tag </HTML> start and end tag 
@return htmlfooter 
*/
gchar *osso_email_editor_prepare_footer(void);

/** 
* This function is called to set diffrent color components 
 
* @param color is the color selected. 
* @param temp_buf_ptr1 is variable to update.  
 
* @return gboolean 
**/
gboolean set_color_val(GdkColor * color, gchar ** temp_buf_ptr1);

/** 
Prepares html tag for foreground color 
 
@param color GdkColor value to be converted into html tag 
 
@return html foreground tag 
*/
gchar *prepare_html_fg_colortag(GdkColor * color);

/** 
Prepares html tag for background color 
 
@param color GdkColor value to be converted into html tag 
 
@return html background tag 
*/
gchar *prepare_html_bg_colortag(GdkColor * color);

/** 
Prepare html font tag for the font description 
 
@param font PangoFontDescription to be converted into html tag 
@param zoom_level describes the zoom level of text
@param tag_start is start of the tag
 
@return html font tag 
*/
gchar *get_html_font_tag_for_gtk_font_tag(PangoFontDescription * font,
					  gint zoom_level, GtkTextIter tag_start);

/** 
This function creates the html tag corresponding to the gtk tag attributes. 
 
@param tag GtkTextTag for which corresponding html tag to be generated 
@param htmltag variable to hold starting html tag 
@param htmltagend variable to hold ending html tag 
@param zoom_level describes the zoom level of text
@param tag_start is start of the tag
@param editorinfo is pointer to EditorInfo object having editor related info.
 
@return TRUE if success 
*/
gboolean
get_html_tag_for_gtk_tag(GtkTextTag * tag,
			 gchar * htmltag, gchar * htmltagend,
			 gint zoom_level, GtkTextIter tag_start, EditorInfo * editorinfo);

/** 
replace GtkTextTag with corresponding html tag at the offset specified. 
 
@param buffer GtkTextBuffer of the text area in html editor 
@param offset position where the tag needs to be replaced 
@param tag GtkTextTag representing text property 
@param zoom_level current selected zoom level 
*/
void replace_tag_at_offset(EditorInfo * editorinf,
			   gint * offset, GtkTextTag * tag, gint zoom_level);

/** 
Check for availability of gtk tags at the given offset.  
If a gtk tag is found the same is erplaced with corresponing html tag. 
 
@param buffer GtkTextBuffer of the text area in html editor 
@param offset position where the tag needs to be replaced 
@param zoom_level current selected zoom level 
*/
gboolean check_gtk_tags_at_offset(EditorInfo * editorinfo, gint * offset, gint zoom_value);

/** 
Replaces the given string at the specified iterator offset. 
 
@param buffer GtkTextBuffer of the text area in html editor 
@param start_iter iterator at selection start 
@param end_iter iterator at selection end 
@param replacestr string to be replaced with 
*/
void text_buffer_replace_string(GtkTextBuffer * buffer,
				GtkTextIter * start_iter, GtkTextIter * end_iter,
				gchar * replacestr);

/** 
Converts specific gtk delimiters to corresponding html delimiters. 
 
@param buffer GtkTextBuffer of the text area in html editor 
*/
void replace_gtk_to_html_delimiters(GtkTextBuffer * buffer);

/** 
Converts the gtk formated text to html data. 
 
@param edotor EditorInfo representing editor window 
@param buffer GtkTextBuffer of the text area in html editor 
*/
void osso_email_format_to_html_text(EditorInfo * editor, gchar ** buftext);

/** 
Retrives the font details from the html font tag. 
 
@param tag EditorInfo representing editor window 
 
@return font details 
*/
gchar *getfontdetails_from_htmltags(gchar * tag);

/** 
Parse the color format into GdkColor value. 
 
@param formatname string representing color 
 
@return GdkColor value corresponfing to format name 
*/
GdkColor osso_email_color_parse(gchar * formatname);

/** 
Creates GtktextTag for the given html tag specified.  
This is valid for font and color properties. 
 
@param buffer GtkTextBuffer of the text area in html editor 
@param start_tag html tag  
 
@return GtkTextTag for the html tag specified 
*/
GtkTextTag *get_gtk_tag_for_html_tag(GtkTextBuffer * buffer, gchar * start_tag);

/** 
Return html TAG type for the given html tag. 
 
@param htmltag html tag  
 
@return TAG ID for the html tag 
*/
gint find_property_start_htmltag(gchar * htmltag);

/** 
Returns substring between specified delimiter strings. 
 
@param string source string to be parsed  
@param st_delim starting delimiter string 
@param end_delim ending delimiter 
@param include_delim specifies if the delimiters to be  
included in the return string 
 
@return text between delimiters 
*/
gchar *text_between_delimiters(const gchar * buffer,
			       gchar st_delim, gchar end_delim, gboolean include_delim);

/** 
Apply gtk tag for the corresponding to the html tag specified. 
 
@param editor EditorInfo representing html editor  
@param start_tag html tag  
@param start_tag_st_off starting html tag starting offset in the text buffer 
@param end_tag ending html tag  
@param end_tag_st_off ending html tag starting offset 
*/
void apply_gtk_tags(EditorInfo * editor,
		    gchar * start_tag, gint start_tag_st_off, gchar * end_tag, gint end_tag_st_off);

/** 
Replace <BR> with newline character. 
 
@param buffer GtkTextBuffer of the text area in html editor 
@param start_br_it starting iterator for <BR> tag 
@param end_br_it ending iterator for <BR> tag. 
*/
void replace_br_tag(GtkTextBuffer * buffer, GtkTextIter * start_br_it, GtkTextIter * end_br_it);

/** 
Checks for a substring and returns the offset. 
 
@param source source string 
@param searchstring sub stringto be searched for 
 
@return offset of substring occurance 
*/
gint get_string_occurance(const gchar * source, const gchar * searchstring);

/** 
prepares closing tag from starting html tag. 
 
@param starting_tag starting html tag 
 
@return closing html tag 
*/
gchar *get_closing_tag(const gchar * starting_tag);

/** 
Process html text and comverts to GTK data.  
First occuring html tag is replaced with corresponding GTK tag 
 
@param editor EditorInfo representing html editor  
 
@return TRUE if html tag is found 
*/
gboolean process_html_text(EditorInfo * editor);

/** 
Process html text and comverts to GTK data.  
 
@param editor EditorInfo representing html editor  
@param textdata html text data to be converted. 
*/
void osso_email_convert_html_to_gtk(EditorInfo * editor, gchar * textdata);


/** 
Remove html delimiters and replace them with corresponding text data. 
 
@param txt_bt_tags html text data 
 
@return html text data with delimiters replaced. 
*/
void remove_html_delimeters(GtkTextBuffer * buffer);
/*gchar * 
remove_html_delimeters(gchar *txt_bt_tags);*/

/** 
 replace the bullets and numbers with the list item tags in html 
 
 @param buffer text buffer representing the text 
 @param start_line starting line number in the buffer 
 @param end_line ending line number in the buffer 
 @param tagname name of the tag 
 */
void process_list_items(GtkTextBuffer * buffer, gint start_line,
			gint end_line, const gchar * tagname);


/** 
 Replace the bullets and numbers with the list item tags present in the buffer 
  
 @param buffer text buffer representing the text 
*/
void text_buffer_replace_list_items(GtkTextBuffer * buffer);


/** 
 Replace the bullets and numbers with the list item tags present in the buffer 
 
 @param buffer text buffer representing the text 
 @param tag tag for bullets or numbeting 
*/
void update_bullet_or_number_list(GtkTextBuffer * buffer, GtkTextTag * tag);

/** 
 Insert the bullets or numbers in the buffer depending upon the tag 
 
 @param buffer text buffer representing the text 
 @param tag tag for bullets or numbeting 
*/
void insert_bullet_or_number_list(GtkTextBuffer * buffer, GtkTextTag * tag,
				  EditorInfo * editorinfo);


/** 
 Insert the bullets and numbers with the list item tags present in the buffer 
 
 @param buffer text buffer representing the text 
*/
void text_buffer_insert_list_items(GtkTextBuffer * buffer, EditorInfo * editorinfo);


/** 
 Callback for searching a image inserted in text buffer 
 
 @param ch character at which the search is 
 @param user_data 
 
 @return TRUE id the character is found 
*/
gboolean predicate(gunichar ch, gpointer user_data);


/** 
 Prepares HTML tag for the image located 
  
 @param filepath File path representing the image 
  
 @return html tag data representing the image 
*/
gchar *get_html_tag_for_image_path(GdkPixbuf * pixbuf, const gchar * filepath, MsgInfo * msginfo);



/** 
 Process the text buffer for inserted images 
  
 @param buffer text buffer representing the text 
*/
void text_buffer_process_images(GtkTextBuffer * buffer, EditorInfo * editinfo);

gboolean backgroundcolor_is_white(EditorInfo * editor);
void insert_newlines_in_listitem(GtkTextBuffer * buffer);

#endif
