/**
 * @file hyperlink_handler.h
 * 
 * This file contains the function declarations  & the structures
 * needed for  dnl recognisation
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#ifndef __HYPERLINK_HANDLER_H__
#define __HYPERLINK_HANDLER_H__

#define READ_WRITE 0666
#define BUFF_SIZE 100

typedef struct {
	guint offset;
	guint len;
} url_match_t;

/**
 This function is used add the charset info file containing
 palin text.
 @param file is gchar pointer to the path of the plain file.
 @param mime_info is the pointer to the MimeInfo structure pointer 
  for determining the charset.
 @return gchar a temporary file with the charset info.
*/
gchar *add_charset_info_to_file(const gchar * file, const MimeInfo * mime_info);


/**
 This function is used to prepare html files for urls containing
http:/https: protocols & that are send using SEND AS email.
@param editor_msginfo is the MsgInfo that is send to engine while sending the mail.
@param engine_msginfo the msginfo engine sends containing the urls to be sent.
@
*/
gchar *prepare_files_for_urls(const gchar * url);

gchar *ui_viewer_html_parser(const gchar *);

/**
 Helper function to check if the filepath send , has some link.
 for this the if the protocol is file:// or obex:// we send the attachment.
   else we treat it as link.

 @param filepath is the const gchar pinter to the filepath.
 @return TRUE--> is it has a link
         FALSE--> else file paths starting with file:// or obex://.
*/
gboolean check_for_links(const gchar * filepath);
/**
This function scans the entire strings for any emailid /url
& puts the <href> tag so that browser recognises the string.
*/
//void hyperlinkify_plain_text(GString * txt);
#endif
