/**
 * @file message_handler.c
 *
 * This is header file for message handler module.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#ifndef __MESSAGEHANDLER_H__
#define __MESSAGEHANDLER_H__

#include "editview.h"

typedef struct _MsgIdInfo MsgIdInfo;

struct _MsgIdInfo {
	gchar *msg_id;
	gint protocol_type;
	gboolean in_use_status;
	gboolean is_smime;
};

typedef enum {
	MARK_STATUS = 0,
	READ_STATUS,
	IN_USE_STATUS,
	PURGE_STATUS,
	FULLY_DOWNLOADED_STATUS,
	BODY_DOWNLOADED_STATUS,
	CUT_STATUS,
	MARK_BOOL_STATUS,
	ATTACHMENT_STATUS,
	REMOTE_COPY_STATUS
} rowTag;

/*Window creation Functions*/
gboolean ui_create_move_to_folder_dialog(GtkTreeView * folderView,
					 GtkTreeView * listView, gboolean msg_send_flag,
                                         GtkTreeRowReference *row_ref);

/*Helper Functions*/
gboolean ui_manage_selected_items(gint origModelNum, gint destModelNum,
				  msgHandlingAction action, gboolean undo_operation,
                                  GtkTreeRowReference *row_ref );
gboolean ui_get_row_information(HeaderInfo * header);
gboolean ui_select_all_rows(GtkTreeModel * model, GtkTreePath * path,
			    GtkTreeIter * iter, gpointer data);
void ui_invoke_marking_mode(gboolean markingMode);
void ui_count_marked_rows(GtkTreeView *view,gint *count);
gboolean ui_has_marked_row(GtkTreeModel * model);
gboolean ui_update_row_tag_in_folder(gint folderNum, const gchar * msgid,
				     rowTag tag, gboolean flag);
GSList *
ui_get_selected_not_inuse_msgids(GtkTreeView *view,
                                                  gint operation_type,gint *inuse_msg_count);
void ui_get_selected_row_msg_ids(GtkTreeView *view, 
                                      GSList ** msg_id_list);
void ui_get_unselected_row_msg_ids(GtkTreeView *view,GtkTreeModel * model,
                                       GSList ** msg_id_list);
gboolean ui_get_outbox_msgids_with_wait_status(GtkTreeModel * model, GtkTreePath * path,
					       GtkTreeIter * iter, GSList ** msg_id_list);
void ui_get_purgeable_row_msg_ids(GtkTreeView *view,
                                                             GSList ** msg_id_list);
void ui_get_unretrieved_selected_msgs_count(GtkTreeView * view, gint * count);
gboolean ui_undim_all_rows(GtkTreeModel * model, GtkTreePath * path,
			   GtkTreeIter * iter, gpointer data);
void ui_set_msg_cb(gint msg_number);
void ui_folderview_selection_changed_cb(GtkTreeSelection * selection, gpointer data);
//void ui_populate_folder (gchar * folderName, guint folderNum);
gboolean ui_is_move_operation_allowed(gint destModelNum, GtkWindow * window);
gint ui_get_row_number_of_msg(const gchar * msgid, gint folderModelNum);
gchar *ui_get_msg_subject(const gchar * msgid, gint model_num);
void ui_show_msgs_in_use_notification(gint operation, gint num_of_msgs_in_use);
void ui_dim_selected_rows(GSList * msg_id_list, gint folder_num);
GSList *
ui_get_unretrieved_msg_id_list( GtkTreeView * view);
void ui_create_message_properties_dialog(HeaderInfo * headerinfo, gint msg_size);

#endif
