/**
 * @file security.h
 *
 * This file contains the function declarations  & the structures
 * needed for handling secure MIME messages  of OSSO_EMAIL.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/


#ifndef __SECURITY_H__
#define __SECURITY_H__


//AL header files
#include "common.h"
#include "prefs_account.h"
#include "procmsg.h"
#include "procmime.h"
#include "email_mgr.h"
#include "ui_utils.h"
#include "ui_common.h"
#include "editview.h"
#include "validations.h"
#include "ui_defs.h"
#include "viewer.h"

#include "ui_common.h"
#include <osso-applet-certman.h>
#include <gconf/gconf-client.h>
typedef struct _SecuritySetting SecuritySetting;

typedef enum {
	INVALID_RECIP,
	INVALID_SENDER,
	NOT_TRUSTED,
	NEW_SIGNER_CERT,
	NEW_SIGNER_CERT_SAVED,
	ENCRYPT
} IdStatus;
typedef enum {
	NO_CERT,
	INVALID_CERT
} CertStatus;



typedef enum {
	READ_MAIL,
	SHOW_PROPERTIES,
	EDIT_TRUST_SETTINGS,
	SAVE_CERT
} SecureMailAction;

struct _SecuritySetting {
	PrefsAccount *accSettings;
	IdStatus indicationStatus;
};


/*
 Validations to be done when a mail is sent with security options.
@param msginfo is the MsgInfo structure.
 This param is not freed here.
@return gboolean-TRUE-if the recipient has a certificate & user wants to proceed sending
                 FALSE-else.
 */
gboolean ui_editor_smime_validations(MsgInfo *);

/* Based on the information about the security the appropriate
 * message is poped

 @param msgid is the gchar pointer to the msgid
 @param mime_info is the pointer to the MimeInfo structure pointer for the current viewer.
 @param msgfields is the pointer to the MsgFields structure pointer for the current viewer.

 @return SecureMailAction is a enum rep whether the mail can be read or some other action such as
 opening the CM ui for editing trust settings.
 */
SecureMailAction check_security_status(gchar *, MimeInfo * mimeinfo, MsgFields * msgfields);

/**
 This function is used to set the security icon in  editor
 based on the security type of the mail
 @param SMimeType is the enum representing the security option of the amil.
 @param msgFields is the pointer to the MsgFields structure.
 This paramter is not freed in this function.
 @return void.
 */
void set_security_icon(SMimeType mime, MsgFields * fields);

/**
 This function is used to set the security icon in viewer
 based on the security type of the mail
 @param msgFields is the pointer to the MsgFields structure.
 This paramter is not freed in this function.
 @param msgheader is the pointer to the HeaderInfo.
 This paramter is not freed in this function.
 @return void.
*/
void set_viewer_security_icon(MsgFields * fields, MimeInfo * mime_info);
gchar *get_certificate_password(const guint cert_id);
#endif
