/**
 * @file ui_account_store.h 
 *
 * This file contains the declarations for prefs account structure 
 * initialization functions of OSSO_EMAIL.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


#ifndef __UI1_ACCTS_STORE_H__
#define __UI1_ACCTS_STORE_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include "cl.h"


/**
This function is used to initialize the account_detail_store structure. 

@param void. 
@return It returns TRUE if the initialization process succeds.
        FALSE otherwise.
*/
void ui_init_account_store(void);
/**
 This function is a utility function. It is used to get the information of all accounts.

 @param void.
 @return It returns the list of prefsAccount structure which 
         contains the information about individual accounts.
 */

GSList ui_get_all_account_details(void);
/**
 This function is a utility function. It is used to get the details of a specific account.

 @param account_name It holds the account name whose details need to be returned.
 @return It returns pointer to the PrefsAccount structure which holds the details about the account name.
         The PrefsAccount structure pointer needs to be freed by the calling function.
 */
PrefsAccount *ui_get_account_detail(const gchar * account_name);
/**
 * This is a utility function. It used to free the account_detail_store list.
 *
 * @param void.
 * @return void.
 */
void ui_free_account_detail_store(void);
/**
 * This is an utility function. It is used to set/update account setting.
 *
 * @param type , it denotes the type of account setting.
 * @param prefs_account , it holds the account setting structure.
 * return gboolen It returns TRUE if Set Account operation is succed.
 *                FALSE otherwise.
 */
gboolean ui_update_account_setting(SettingType type, const PrefsAccount * prefs_account);

/**
 This function is used to get the list of configured connection specific
 smtp servers .The returned list is stored in the global list.
 @return void.
*/
void ui_init_smtp_conn_servers(void);
#if CL_USE
/**
 This function gets the configured smtp server for specified iap.
 @param iapname is the gchar pointer to the iap name.
 @return IapInfo , pointer to the IapInfo structure containg the server details.
*/
const IapInfo *ui_get_smtp_server_detail(const gchar * iapname);

/**
   This function is used to update the modified smtp server details for the iap.
   @param smtp_server_info is pointer to the IapInfo structure.
   @return gboolean TRUE- on success
                       FALSE - else.
*/
gboolean ui_update_iap_smtp_server_details( const IapInfo * smtp_server_info);

/**
  Helper function to free the global iapinfo list conating IapInfo sturtures.
  @return void.
*/
void ui_free_iap_smtp_servers_list(void);

/**
   Function to return the list of configured Iap smtp servers.
    @return GSList of configured  Iap smtp servers.
*/
const GSList * ui_get_iapinfo_list(void);
#endif
#endif				/* __ACCTS_STORE_H__ */
