/**
 * @file ui_al.h
 *
 * This file contains the declarations for AL calls to be used by OSSO_EMAIL.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#ifndef __UIAL_H__
#define __UIAL_H__

#include "callbacks.h"

gboolean al_register_to_engine(SmtpCallback * smtp_callback,
			       Pop3Callback * pop3_callback, Imap4Callback * imap4_callback);


void al_send_mail(MsgInfo * msginfo, gchar * message);


void al_cancel_send_mail();

gint al_get_account_list();

gboolean al_get_account_settings(const gchar * account_name,
				 SettingType type, PrefsAccount ** prefs_account);

gboolean al_set_account_settings(SettingType type, PrefsAccount * prefs_account);

gboolean al_delete_account(const gchar * account_name);

void al_recv_mail(gchar * account_name);

MimeInfo *al_recv_msg_body(gchar * msgid);

AttachInfo *al_recv_msg_attachment(gchar * msgid, MimeInfo * mimeinfo);

void al_cancel_recv_mail();

gint al_folder_create(gchar * name, gchar * path, FolderType type);

gint al_folder_delete(gchar * name);

gint al_folder_rename(gchar * oldfolder, gchar * newfolder);

gint al_folder_move_msg(gchar * srcfolder, gchar * destfolder, gchar * msgid);

gint al_folder_move_msgs(gchar * srcfolder, gchar * destfolder, GSList * msglist);

gint al_folder_remove_msg(gchar * folder, gchar * msgid);

gint al_folder_remove_msgs(gchar * folder, GSList * msglist);

gint al_folder_remove_all_msg(gchar * folder);

gint al_folder_get_list(GSList ** folderlist);

GSList *al_populate_folder(gchar * name, gint index);

#endif
