/**
 * @file ui_al_callbacks.h
 *
 * This file contains the declarations for UI callbacks of OSSO_EMAIL, that 
 * are called from UI_DBUS layer.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#ifndef __UI_AL_CALLBACKS_H__
#define __UI_AL_CALLBACKS_H__

#include <stdlib.h>
#include <glib.h>
#include "common.h"
#include "prefs_account.h"
#include "procmsg.h"
#include "smtp_errorcodes.h"
#include "smtp.h"

typedef enum {
	CONNECTING = 0,
	SENDING,
	RECEIVING,
	MSG_OPENING
} progress_bar_mode;

void ui_param_value_missing_cb(ParamMissing param_number);

void ui_set_percentage_cb(gint percent);

void ui_open_editor_in_send_as_mail_to_mode_cb(MsgInfo * msg_info, gchar * msg_body);

void ui_update_folder_view_cb(MsgHeader * msgheader, gboolean add, gboolean free_mem);

void ui_recv_single_cb(MsgHeader * msgheader);

void ui_send_recv_complete_cb(void);

void ui_recv_body_complete_cb(MimeInfo * mimeinfo, gchar * msgid, gboolean remote_copy);

void ui_imap_msg_headers_cb(GSList * new_list);

void ui_set_msg_cb(gint msg_number);

void ui_set_error_msg_cb(gint msg_number, const gchar * error_info1, const gchar * error_info2);

void ui_update_progressbar_text(progress_bar_mode mode, gchar * msgid);

void ui_update_progressbar_numbers(gint currentMsgNumber, gint totalMsgNumber,
				   progress_bar_mode mode);

void ui_update_msg_downloaded_cb(gint msg_fully_downloaded,
				 gint msg_body_downloaded, gint msg_remote_copy, gchar * msg_id);

void ui_handle_sync_error_cb(gchar * account, gboolean show_note);

void ui_add_archive_folder(void);

void ui_remove_archive_folder(void);

void ui_folder_msg_headers_cb(gint type, GSList * new_list);

void ui_copy_unfetched_messages_if_required(void);

gboolean ui_start_folder_progress_note(gpointer data);

gboolean ui_stop_folder_progress_note(void);

void ui_update_folder_progress_note(gfloat percent);

void ui_show_note_for_failed_msgs(void);

void ui_remove_messages_cb(GSList * msg_id_list);

//Task Navigator related callback functions

void ui_tasknav_top_msg_cb(gchar * msgid);

void ui_tasknav_open_msg_cb(gchar * msgid);

void ui_tasknav_show_inbox_cb(void);

void ui_tasknav_oom_signal_received(void);

//Helper function to close UI application
void ui_close_application(void);

void ui_recv_msg_attachment_cb(MsgInfo * msg_info);
void ui_recv_msg_attachments(GSList * new_list);
#endif
