/**
 * @file ui_interface.c
 *
 * This file contains the declarations for main view module.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#define DBUS_API_SUBJECT_TO_CHANGE
#define _DBUS_POINTER_SHIFT(p)   ((void*) (((char*)p) + sizeof (void*)))
#define DBUS_G_CONNECTION_FROM_CONNECTION(x)     ((DBusGConnection*) _DBUS_POINTER_SHIFT(x))

#include<dbus/dbus.h>
#include<dbus/dbus-glib.h>
#include <hildon-widgets/hildon-app.h>
#include <hildon-widgets/hildon-appview.h>

#ifndef __UI_INTERFACE_H__
#define __UI_INTERFACE_H__

#include "font_settings.h"
#include "ui_al_callbacks.h"
typedef struct _MainViewToolbar MainViewToolbar;
typedef struct _MainViewMenu MainViewMenu;
typedef struct _MainViewWidgets MainViewWidgets;
typedef struct _MainViewInfo MainViewInfo;
typedef struct _MessageViewPopupMenu MessageViewPopupMenu;
typedef struct _FolderViewPopupMenu FolderViewPopupMenu;
typedef struct _MainViewTags MainViewTags;
typedef struct _FolderManager FolderManager;
typedef struct _MessageHandler MessageHandler;
typedef struct _AccountManager AccountManager;
typedef struct _MainViewUndoInfo MainViewUndoInfo;
typedef struct _MainViewIcons MainViewIcons;

typedef enum {
	NO_ACTION = 0,
	INVOKE_SIGNATURES_DIALOG,
	SEND_EMAIL,
	SEND_MSGS,
	RESEND_MSGS,
	VIEWER_RESEND_MSG,
	FORWARD_MSGS,
	NEW_EMAILS,
	MAILBOX_ERROR
} actionAfterAcctCreation;

typedef enum {
	MOVE_AFTER_RETRIEVAL = 0,
	OPEN_AFTER_RETRIEVAL,
	COPY_AFTER_RETRIEVAL,
	REPLY_AFTER_RETRIEVAL,
	REPLY_ALL_AFTER_RETRIEVAL,
	FORWARD_AFTER_RETRIEVAL,
	NO_ACTION_AFTER_RETRIEVAL
} actionAfterRetrieval;

typedef enum {
	OPER_UNCUT = 0,
	OPER_DELETE,
	OPER_MOVE,
	OPER_NONE
} UndoOperation;

typedef enum {
	NONE_OP,
	EDIT_OP,
	DELETE_OP
} OperationStatus;


struct _MainViewUndoInfo {
	UndoOperation operation;
	GSList *message_id_list;
	gint orig_folder;
	gint dest_folder;
};

struct _MainViewToolbar {
	GtkWidget *toolbar;
	GtkWidget *fullscreen_toolbar;
	GtkWidget *left_space;
	GtkWidget *new_msg;
	GtkWidget *resend;
	GtkWidget *cancel_sending;
	GtkWidget *send;
	GtkWidget *reply;
	GtkWidget *reply_to_all;
	GtkWidget *forward;
	GtkWidget *move_to;
	GtkWidget *delete_msg;
	GtkWidget *send_receive;
	GtkWidget *separator;
	GtkWidget *progress_item;
	GtkWidget *progressbar;
	GtkWidget *progress_bar_button;
	GtkWidget *toolbar_box;
	GtkWidget *new_msg_space;
	GtkWidget *resend_space;
	GtkWidget *cancel_sending_space;
	GtkWidget *send_space;
	GtkWidget *reply_space;
	GtkWidget *reply_to_all_space;
	GtkWidget *forward_space;
	GtkWidget *move_to_space;
	GtkWidget *delete_msg_space;
	GtkWidget *send_receive_space;
};

struct _MainViewMenu {
	GtkWidget *message;
	GtkWidget *message_menu;
	GtkWidget *message_new_email;
	GtkWidget *message_open;
	GtkWidget *message_separator1;
	GtkWidget *message_reply;
	GtkWidget *message_reply_to_all;
	GtkWidget *message_resend;
	GtkWidget *message_forward;
	GtkWidget *message_cancel_sending;
	GtkWidget *message_send;
	GtkWidget *message_separator2;
	GtkWidget *message_delete;
	GtkWidget *message_separator3;
	GtkWidget *message_details;
	GtkWidget *edit;
	GtkWidget *edit_menu;
	GtkWidget *edit_undo;
	GtkWidget *edit_separator1;
	GtkWidget *edit_cut;
	GtkWidget *edit_copy;
	GtkWidget *edit_paste;
	GtkWidget *edit_select;
	GtkWidget *edit_select_menu;
	GtkWidget *edit_select_mark_all;
	GtkWidget *edit_select_clear_all_marks;
	GtkWidget *edit_separator2;
	GtkWidget *edit_move_to;
	GtkWidget *folders;
	GtkWidget *folders_menu;
	GtkWidget *folders_new_folder;
	GtkWidget *folders_manage_folders;
	GtkWidget *folders_folder_settings;
	GtkWidget *folders_details;
	GtkWidget *folders_separator;
	GtkWidget *folder_inbox;
	GtkWidget *view;
	GtkWidget *view_menu;
	GtkWidget *view_sort;
	GtkWidget *view_separator;
	GtkWidget *view_show_toolbar;
	GtkWidget *view_show_toolbar_menu;
	GtkWidget *view_show_toolbar_normal_screen;
	GtkWidget *view_show_toolbar_full_screen;
	GtkWidget *view_folders;
	GtkWidget *view_full_screen;
	GtkWidget *email;
	GtkWidget *email_menu;
	GtkWidget *email_send_receive;
	GtkWidget *email_send_receive_menu;
	GtkWidget *email_send_recv_all;
	GtkWidget *email_retrieve_mail_contents;
	GtkWidget *email_purge_attachments;
	GtkWidget *tools;
	GtkWidget *tools_menu;
	GtkWidget *tools_addressbook;
#if CHANGED_EMAIL
	GtkWidget *tools_signatures;
        GtkWidget *tools_font;
#endif
	GtkWidget *tools_separator1;
	GtkWidget *tools_new_e_mail_account;
	GtkWidget *tools_account_settings;
	GtkWidget *tools_options;
	GtkWidget *tools_separator4;
	GtkWidget *tools_search_e_mails;
	GtkWidget *tools_separator5;
	GtkWidget *tools_help;
	GtkWidget *close;
	GtkWidget *close_menu;
	GtkWidget *close_window;
	GtkWidget *close_all_windows;
	GSList *popup_folder_menu_group;
};

struct _MessageViewPopupMenu {
	GtkWidget *pop_up_menu;
	GtkWidget *open;
	GtkWidget *reply;
	GtkWidget *reply_to_all;
	GtkWidget *forward;
	GtkWidget *copy;
	GtkWidget *paste;
	GtkWidget *move_to;
	GtkWidget *delete_msg;
	GtkWidget *resend;
	GtkWidget *cancel_sending;
	GtkWidget *send;
	GtkWidget *help;
};

struct _FolderViewPopupMenu {
	GtkWidget *pop_up_menu;
	GtkWidget *open;
	GtkWidget *new_folder;
	GtkWidget *paste;
	GtkWidget *rename_folder;
	GtkWidget *delete_msg;
	GtkWidget *search;
	GtkWidget *help;
};

struct _MainViewWidgets {
	GtkWidget *folderView;
	GtkWidget *messageListView;
	GtkWidget *folder_msg_pane;
	GtkWidget *folder_scrollwindow;
	GtkWidget *message_scrollwindow;
	GtkWidget *no_msgs_label;
	GtkWidget *opened_viewer;
};

struct _MainViewTags {
	gboolean show_tool_bar;
	gboolean show_fullscreen_tool_bar;
	gboolean marking_mode;
	gboolean progress_bar_available;
	gboolean contact_file_changed;
	gboolean get_only_unretrieved_mails;
	gboolean main_view_topped;
	gboolean main_view_hidden;
	gboolean oom_signal_received;
	gboolean application_closure;
	gboolean full_screen_mode;
	gboolean prev_oom_status;
	gboolean is_sub_folder;
	gboolean folderview_has_focus;
	gboolean messageview_has_focus;
	gboolean check_name_unresolved;
	gboolean is_msg_being_sent;
	gboolean manage_folder_dlg;
	gboolean msg_download_in_progress;
	gboolean state_save_disabled;
	gulong send_receive_sig_handler;
	guint window_count;
	gint total_msg_count;
	gint current_msg_count;
	gint archive_model_num;
	gint current_click_x_pos;
	gint current_click_y_pos;
	gint reply_offset;
	gint fwd_offset;
	gint normal_screen_width;
	gchar *reply_indicator;
	gchar *fwd_indicator;
	progress_bar_mode current_mode;
};

struct _FolderManager {
	gint model_seq_num;
	guint currently_displayed_model;
	gint currently_popup_model;
	gint folder_msg_count;
	guint folder_sort_order;
	gboolean is_folder_name_taken;
	gboolean is_folder_present;
	gchar *searched_folder_name;
	gchar *parent_folder_name;
	GtkTreeModel *folder_view_model;
	GSList *folder_menu_items;
	GtkWidget *folder_progress_bar;
	GtkWidget *folder_progress_note;
	GtkWidget *move_to_dialog;
};

struct _MessageHandler {
	gint currently_selected_row;
	actionAfterRetrieval set_action_after_retrieval;
	actionAfterRetrieval get_action_after_retrieval;
	gboolean is_message_present;
	gboolean store_all_headers;
	GSList *unfetched_msgs_to_copy;
	GSList *msgs_to_copy;
	gint dest_folder_to_copy;
	gint current_operation;
	gint num_of_failed_msgs;
	gint num_of_msgs_in_use;
	gint cut_folder_num;
	gchar *current_draft_subject;
	gchar *msg_to_be_selected_after_del_opr;
	gboolean delete_draft;
};

struct _AccountManager {
	GSList *configured_accounts;
	gchar *default_account;
	gchar *current_sendrec_account;
	actionAfterAcctCreation action_after_acct_creation;
	gboolean is_mailbox_opened;
	gint mailbox_error_code;
	gchar *error_info1;
	gchar *error_info2;
	OperationStatus operation_status;
	GtkWidget *edit_account_dialog;
	gboolean esc_sgnl_handled;
	gchar *alias_name;
};

struct _MainViewIcons {
	GtkIconTheme *icon_theme;

	GdkPixbuf *inbox_icon;
	GdkPixbuf *outbox_icon;
	GdkPixbuf *drafts_icon;
	GdkPixbuf *sent_folder_icon;
	GdkPixbuf *user_folder_icon;
	GdkPixbuf *mmc_archive_icon;
	GdkPixbuf *device_icon;

	GdkPixbuf *hdr_low_unread_icon;
	GdkPixbuf *hdr_high_unread_icon;
	GdkPixbuf *hdr_unread_icon;
	GdkPixbuf *hdr_low_icon;
	GdkPixbuf *hdr_high_icon;
	GdkPixbuf *hdr_icon;

	GdkPixbuf *hdr_signed_low_icon;
	GdkPixbuf *hdr_signed_high_icon;
	GdkPixbuf *hdr_signed_icon;
	GdkPixbuf *hdr_unread_signed_low_icon;
	GdkPixbuf *hdr_unread_signed_high_icon;
	GdkPixbuf *hdr_unread_signed_icon;

	GdkPixbuf *hdr_encrypted_icon;
	GdkPixbuf *hdr_encrypted_high_icon;
	GdkPixbuf *hdr_encrypted_low_icon;
	GdkPixbuf *hdr_unread_encrypted_icon;
	GdkPixbuf *hdr_unread_encrypted_high_icon;
	GdkPixbuf *hdr_unread_encrypted_low_icon;

	GdkPixbuf *mail_low_unread_icon;
	GdkPixbuf *mail_high_unread_icon;
	GdkPixbuf *mail_unread_icon;
	GdkPixbuf *mail_low_icon;
	GdkPixbuf *mail_high_icon;
	GdkPixbuf *mail_icon;

	GdkPixbuf *mail_signed_unread_low_icon;
	GdkPixbuf *mail_signed_unread_high_icon;
	GdkPixbuf *mail_signed_unread_icon;
	GdkPixbuf *mail_signed_read_low_icon;
	GdkPixbuf *mail_signed_read_high_icon;
	GdkPixbuf *mail_signed_read_icon;

	GdkPixbuf *mail_encrypted_unread_low_icon;
	GdkPixbuf *mail_encrypted_unread_high_icon;
	GdkPixbuf *mail_encrypted_unread_icon;
	GdkPixbuf *mail_encrypted_read_high_icon;
	GdkPixbuf *mail_encrypted_read_low_icon;
	GdkPixbuf *mail_encrypted_read_icon;

	GdkPixbuf *attachment_icon;
	GdkPixbuf *check_msg_icon;
	GdkPixbuf *dustbin_icon;
};

struct _MainViewInfo {
	MainViewWidgets *widgets;
	MainViewMenu *menu;
	MainViewToolbar *tools;
	MessageViewPopupMenu *message_popup;
	FolderViewPopupMenu *folder_popup;
	MainViewTags *tags;
	FolderManager *folder_manager;
	MessageHandler *message_handler;
	AccountManager *account_manager;
	MainViewUndoInfo *undo;
	MainViewIcons *icons;
	GSList *marking_mode_widgets;
	GSList *sorted_model_list;
	GSList *account_menu_list;
	GSList *open_windows;
	GSList *msginfo_list;
	MsgInfo *msginfo;
	MimeInfo *mimeinfo;
};

gboolean ui_initialize(gint startup_mode);
void ui_do_send_recv(void);
void get_ui_context(void);
void ui_exit(int signal);
void ui_cleanup_memory(void);
void ui_add_mail_account(const gchar * accName);
void ui_remove_mail_account(gchar * accName);
void ui_prepare_send_receive_menu(void);
void ui_populate_inbox(void);
void ui_populate_folder(const gchar * folderName, guint folderNum);
gboolean ui_populate_all_folders(GtkTreeModel * model, GtkTreePath * path,
				 GtkTreeIter * iter, gpointer data);
void ui_refresh_required_folders(void);
void ui_display_message_dialog(GtkWindow * parent, const gchar * message);
void ui_folderview_on_row_selected(GtkTreeView * treeView, GtkTreePath * path,
				   GtkTreeViewColumn * column, gpointer data);
void ui_folderview_selection_changed_cb(GtkTreeSelection * selection, gpointer data);
void ui_messageview_selection_changed_cb(GtkTreeSelection * selection, gpointer data);
void ui_update_folder_specific_widgets(guint modelNum);
void ui_on_signatures1_activate(GtkWidget * widget, gpointer data);
void ui_on_new_e_mail_account1_activate(GtkWidget * widget, gpointer data);
void ui_on_send_activate(GtkWidget * widget, gpointer data);
void ui_on_resend_activate(GtkWidget * widget, gpointer data);
gboolean ui_hide_folders(GtkWidget * folder_view, gboolean user_action);
void ui_toggle_toolbar_visibility(gboolean user_action, gboolean show_toolbar);

gboolean ui_on_popup_folder_button_pressed(GtkWidget * treeview,
					   GdkEventButton * event, gpointer userdata);
gboolean ui_on_popup_message_button_pressed(GtkWidget * treeview,
					    GdkEventButton * event, gpointer userdata);
HildonAppView *ui_create_main_window(void);
void ui_update_message_popmenu_specific_widgets(gint modelNum);

void ui_folder_menu_item_open_activate(GtkWidget * widget, gpointer data);
void ui_folder_menu_item_new_folder_activate(GtkWidget * widget, gpointer data);
void ui_folder_menu_item_paste_activate(GtkWidget * widget, gpointer data);
void ui_folder_menu_item_rename_folder_activate(GtkWidget * widget, gpointer data);
void ui_folder_menu_item_delete_activate(GtkWidget * widget, gpointer data);
void ui_open_message(gint row_num, gint folder_num, gboolean set_prev_settings);
void ui_set_folderview_title(const gchar * folder_name);
gboolean ui_top_opened_message(const gchar * msg_id);
void ui_flush_gtk_events(void);
void ui_on_new_email_activate(GtkWidget * widget, gpointer data);
void ui_on_forward1_activate(GtkWidget * widget, gpointer data);
void ui_messageview_on_row_selected(GtkTreeView * treeView, GtkTreePath * path,
				    GtkTreeViewColumn * column, gpointer data);
gboolean ui_check_clipboard_has_valid_emails(gchar ** orig_folder,
					     GSList ** msg_id_list, gboolean * cut_operation_done);
void ui_on_cancel_send_activate(GtkWidget * widget, gpointer data);
gboolean ui_unselect_outbox_msg_being_sent(void);
void ui_add_unread_count_to_folder(gint model_num);
void ui_on_search_emails(GtkWidget * widget, gpointer data);
void ui_on_cancel_clicked(GtkWidget * widget, gpointer data);
void ui_topmost_status_lost(GtkWidget * widget, gpointer data);
void ui_topmost_status_acquired(GtkWidget * widget, gpointer data);
void ui_send_outbox_mails_with_wait_status_cb(void);
void ui_on_folder_settings_activate(GtkWidget * widget, gpointer data);
/**
   This function shows the appropriate confirmation note for copying unfetched mails
   @param unfetched_msg_id_lis is the GSList of the unfetched msgid list.
   @return gboolean TRUE--> if user confirms the retrival
                    FALSE other wise.
*/
gboolean
copy_unretrieved_mails_confirmation (const GSList * unfetched_msg_id_list);
/**
   This function is used to retrieved the mail contents of unfetched mails
   on userconfirmation for copying.
   @param unfetched_msg_id_lis is the GSList of the unfetched msgid list.
   @param foldernum is the gint representing the model number from which the messages are copied.
*/
void
retrieve_mail_contents (const GSList * unfetched_msg_id_list, gint foldernum);
void ui_retrv_complete_cb(void);
#endif
