/**
 * @file ui_state_mgmt.h
 *
 * This is header file for state management module of OSSO_EMAIL.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#ifndef __UI_STATE_MGMT_H__
#define __UI_STATE_MGMT_H__


#include "statesave.h"

gboolean ui_get_previous_state_info(void);
gboolean ui_read_persistent_settings(void);
void ui_store_current_state_info(void);
void ui_store_persistent_settings(void);
void ui_apply_persistent_settings(void);
void ui_initialize_previous_state_settings(void);
void ui_restore_opened_msgs(const GSList * open_msgs_list);
//void ui_restore_folder_sort_settings (GSList * folder_sort_info_list);
void ui_store_marked_msgs(void);
void ui_restore_marked_msgs(GSList * marked_msgs, gboolean marking_on);
gboolean ui_openmsg_get_toolbar_hidden(const gchar * msgid);
void ui_openmsg_set_toolbar_hidden(const gchar * msgid, gboolean flag);
guint ui_openmsg_get_scroll_position(const gchar * msgid);
void ui_openmsg_set_scroll_position(const gchar * msgid, guint position);
guint ui_openmsg_get_zoom_value(const gchar * msgid);
void ui_openmsg_set_zoom_value(const gchar * msgid, guint zoom);
void ui_openmsg_get_cursor_position(const gchar * msgid, guint * x, guint * y);
void ui_openmsg_set_cursor_position(const gchar * msgid, guint x, guint y);
gboolean ui_get_cc_field_status(void);
void ui_set_cc_field_status(gboolean status);
gboolean ui_get_bcc_field_status(void);
void ui_set_bcc_field_status(gboolean status);
void ui_set_split_view_mode_status(gboolean status);
void ui_set_folder_pane_width (GtkWidget *hpaned);
gint ui_get_folder_pane_width (void);
void ui_set_toolbar_hidden_status(gboolean status);
void ui_set_fullscreen_toolbar_hidden_status(gboolean status);
void ui_update_folder_sort_settings(const gchar * folder_name,
				    const guint sort_key, const guint sort_order);
void ui_store_folder_selection(void);
void ui_store_message_selection(const gchar * current_msg_id);
void ui_remove_previous_state_settings(void);
void ui_free_email_state_info(void);
void ui_free_email_persistent_info(void);
void ui_restore_latest_msg_headers(void);
gboolean ui_get_oom_signal_status(void);
void ui_set_oom_signal_status(void);
void ui_unset_oom_signal_status(void);
void ui_set_viewer_toolbar_hidden_status(gboolean status);
gboolean ui_get_viewer_toolbar_hidden_status(void);
void ui_set_viewer_fullscreen_toolbar_hidden_status(gboolean status);
gboolean ui_get_viewer_fullscreen_toolbar_hidden_status(void);
void ui_set_editor_toolbar_hidden_status(gboolean status);
gboolean ui_get_editor_toolbar_hidden_status(void);
void ui_set_editor_fullscreen_toolbar_hidden_status(gboolean status);
gboolean ui_get_editor_fullscreen_toolbar_hidden_status(void);
gboolean ui_get_toolbar_hidden_status(void);
gboolean ui_get_fullscreen_toolbar_hidden_status(void);

void ui_set_sent_folder_cleaning_settings(gint clean_option,
					  gint clean_time_stamp, gint clean_time_value);
gint ui_get_sent_folder_clean_option(void);
gint ui_get_sent_folder_clean_time_stamp(void);
gint ui_get_sent_folder_clean_time_value(void);
void ui_set_default_settings(gint viewing_html_fontname_index,
			     gint viewing_plain_fontname_index,
			     gint editing_html_fontname_index,
			     gint editing_plain_fontname_index);
gint ui_get_viewer_html_fontindex(void);
gint ui_get_viewer_plain_fontindex(void);
gint ui_get_editor_html_fontindex(void);
gint ui_get_editor_plain_fontindex(void);
void ui_set_delete_email_info(gboolean delete_severmail_also);
gboolean ui_get_delete_email_info(void);
void ui_set_email_settings_info(gboolean read_receipt, gint size_limit,
		                gboolean play_sound, gint msg_format_type, 
				gboolean inc_original);
void ui_get_email_settings_info(gboolean *read_receipt, gint *size_limit,
		                gboolean *play_sound, gint *msg_format_type, 
				gboolean *inc_original);

gchar *ui_get_current_message_id(void);

#endif
