/**
 * @file ui_utils.h
 * 
 * This file contains the declarations for utility functions of OSSO_EMAIL.
 * 
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA.
 */

#ifndef __UI1_UTILS_H__
#define __UI1_UTILS_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <time.h>
#include <gtk/gtk.h>
#include <libgnomevfs/gnome-vfs.h>
#include "intlsupport.h"
#include "email_hildon.h"
#include "smtp_errorcodes.h"
#include "imap.h"
#include "defs.h"
#include "osso-helplib.h"

/*Macro definitions*/
#define MAX_STR_LEN(str1,str2) \
( \
 strlen(str1)>strlen(str2)?strlen(str1):strlen(str2) \
)

#define EMAIL_UI_HOOKUP_OBJECT(component, widget, name) \
          g_object_set_data_full (G_OBJECT(component), name, \
	    gtk_widget_ref(widget), (GDestroyNotify) gtk_widget_unref)

#define EMAIL_UI_HOOKUP_OBJECT_NO_REF(component, widget, name) \
          g_object_set_data (G_OBJECT(component), name, widget)

#define EMAIL_UI_LOOKUP_OBJECT(parent, name) \
          (GtkWidget *) g_object_get_data (G_OBJECT(parent), name)
#define DEFAULT_FONT "Sans"
gchar *font_families[3];

typedef struct _FontInfo FontInfo;

struct _FontInfo {
	gchar *size;
	gchar *name;
	gboolean bold;
	gboolean italic;
};

enum {
	INFO_BANNER,
	CONFIRM_DIALOG
};

typedef struct _Zoom {
	gboolean mailtype;
	gboolean zoom_off;
	gint previous_level;
	double zoom_factor;
	GtkTextView *msgbody;
	gchar *msgid;
	GtkWidget *mail_viewer;
	GtkWidget *zoom_menu50;
	GtkWidget *zoom_menu100;
	GtkWidget *zoom_menu150;
	GtkWidget *zoom_menu200;
	GtkWidget *zoom_menu80;
	GtkWidget *zoom_menu120;
} Zoom;

enum {
	LOCALE_SETTINGS = 0,
	IMAGE_SETTINGS
};

typedef struct {
	gboolean is_mandatory;
	gint field_value;	//Represent the bitwise value of the particular field.
	gint field_number;
	gboolean is_shown;
	GtkWidget *caption_control;
} CaptionControlInfo;

void debug_print(const gchar * format, ...);
gint get_index_font_size(gint fontsize);
gchar *setzoom(PangoFontDescription * fd, gint previous_level, gint current_level);
gboolean get_user_confirmation(const gchar * confirmationNote,
			       const gchar * first_button,
			       const gchar * second_button, gint confirmationType);

Zoom *set_initial_zoom_level(GtkTextView * textview_body,
			     GtkWidget * browser_socket, gchar *, gint previous_level);

gboolean ui_read_settings_from_file(gint settings_type);
gchar *ui_get_localized_string(gchar * key);
const gchar *ui_get_image_path(const gchar * image_name);
FontInfo *getfontinfo(PangoFontDescription * fd);
void ui_free_prefs_account_structure(PrefsAccount * ac_prefs);
gchar *ui_get_account_name(const gchar * msg_id);
gint ui_null_check_strings(const gchar * str1, const gchar * str2);
gboolean ui_get_msg_folder_names(const gchar * msg_id, gchar ** parent_folder,
				 gchar ** folder_name);
gchar *ui_get_config_dir(void);

void set_contact_field_variable(guint16 bit_value, gboolean value);
gboolean get_contact_field_variable(guint16 value);
CaptionControlInfo *ui_create_text_entry(gchar * label,
					 GtkSizeGroup * size_group,
					 gint maxlen, gboolean caption_status,
					 gint field_value, gint field_number,
					 gboolean unset_autocap);
void ui_get_contact_field_entry(GtkWidget * entry, gchar ** field);
guint ui_get_contact_id(void);
gboolean ui_add_contact_search_string_to_list(gchar * search_string);
GList *ui_get_contact_search_list(void);
void ui_free_contact_search_list(void);
GtkWidget *ui_get_contact_caption_control(void);
void ui_set_contact_caption_control(GtkWidget * caption_control);
void ui_set_current_contact_field_number(gint field_number);
gint ui_get_current_contact_field_number(void);
void ui_editor_do_check_names(GtkWidget * win, gpointer check_names_list);
void set_contact_field_status(guint16 field_value);
void set_contact_field_status_variable(guint16 bit_value, gboolean value);
gboolean get_contact_field_status_variable(guint16 value);
guint16 get_contact_field_staus(void);
void ui_on_dialog_help_activate(GtkWidget * win, gchar * help_id);
void ui_on_view_help_activate(GtkWidget * win, gchar * help_id);
gchar *ui_get_device_name(void);
GSList *ui_prepare_dest_msg_ids(const GSList * orig_msg_list, const gchar * dest_folder_path);
gchar *ui_parse_from_address(const gchar * from);
gint ui_compare_utf8_strings(const gchar * str1, const gchar * str2);
gboolean ui_on_escape_key_pressed(GtkWidget * widget, GdkEventKey * event, gpointer data);
void ui_undim_cut_messages(void);
gchar *update_css_file(gboolean html_mail);
gboolean has_prefix_insensitive(const gchar * str, const gchar * pfx);
GtkWidget *ui_get_top_level_window(void);
gint ui_predict_dialog_width_based_on_title(GtkWidget * dialog, const gchar * title);
void add_progress_fraction(void);
gchar *ui_get_tmp_path(void);
gchar *ui_get_tmp_attach_dir(void);
gchar *ui_get_tmp_attach_path(void);
gint make_dir(const gchar * dir);
gchar *ui_get_tmp_attach_name(void);
void show_dimming_note(GtkWidget * insensitive_widget, const gchar * infonote);
void
set_zoominfo_menus(Zoom * zoominfo, gpointer menu);
void ui_complex_dlg_destroy_notify(gpointer data);
gchar *ui_format_size_value(guint size_in_bytes);
/**
 This is a helper funtion to get the path of the message for the msgid & the folder number.

 @param msgid is the gchar pointer to the msgid.
 @param foldernum is the gint representing the folder number.
 @return gchar poiter to the message path.
*/
gchar *ui_get_message_file_path(const gchar * msgid, gint foldernum);

/**
   This is a utility function to get the default plain/html font for the viewer.
    @param html is a gboolean representing whether html/plain mail.
    @return font_name is a gchar representing the font_name
*/
const gchar *ui_get_default_font(gboolean html);
/**
This is helper function to set the data in conatct field entries.
It validates the whether its utf8 encoded string, if not converts to utf8.

@param entry This is Widget pointer from where the field value to be retireved.
@param data This is the gchar pointer where the field value to be stored.
@return Nothing.
*/
void ui_set_contact_field_entry(GtkWidget * entry, const gchar * data);

/**
This function is called to check recipient names when enter is pressed.

@param textview is pointer to the recipient field.
@param event is key press event.
@param check_names_list is list of recipient fields.
@return gboolean.
*/
gboolean ui_check_recp_name(GtkWidget * textview, GdkEventKey * event, GSList * check_names_list);

/**
 This is a utility function that takes a list of GtkTextBuffers, and checks the presence
 of 'econtact-uid' property for each of the resolved email addresses. If this property
 is set, this contact has to be set to recent_group of AB; otherwise this is a new contact.

 @param addr_buffers  List of GtkTextBuffers
 @param used_econtact_uids  Out param to store list of used EContact UIDs
 @param new_emailids  Out param to store list of new email IDs
 @return void
*/
void get_recipients_list_to_add_to_addbook(GSList * addr_buffers, GSList ** used_econtact_uids,
                                           GSList ** new_emailids);

/**
 * This function is used to fill the recipient list on clicking Ok button
 * @param recpBuffer is recipient field to be updated
 * @param recipientlist_email_recipient list of email ids
 * @param econtact_uid is the UID of the current EContact
 * @return gboolean
 * */
gboolean
ui_update_recipient_buffer(GtkTextBuffer * recpBuffer, GSList * recipientlist_email_recipient,
                           const gchar *econtact_uid);

/**
  helper function to handle the zoom fuctionality when editor is created.
 
  @param msgType is the gint representing the mode in which editor is opened.
  @param zoominfo is the pointer to the Zoom structuer.
  @param zoomlevel is the integer value representing the zoom.
  @return void.
*/
void set_inherited_zoom(gint msgType, Zoom *zoominfo, gint zoomlevel);

/**set the zoom of current message.
 
   @param zoominfo is a pointer to structure ZoomInfo.
         This parameter is not freed here.
   @param current_level isthe current zoom level.
 
   @return void.
*/
void ui_set_zoom(Zoom * zoominfo, gint current_level);
/**
 function to get the end iterator of the email id.
 
 @param start is the pointer to the GtkTextIter from where the search starts.
 @param end is the pointer to the GtkTextIter, where the email id ends. OUT param.
 @param addressBuffer is the pointer to the GtkTextBuffer in which the emailid is present.
*/
void
get_end_iter_for_emailid(GtkTextIter *start, GtkTextIter *end, GtkTextBuffer *addressBuffer);
#endif				/* __UTILS_H__ */
