/**
 * @file undo.h
 *
 * This file contains the function declarations  & the structures
 * needed for Undo functionality
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


#include "editor.h"

typedef enum {
	UNDO_INSERT,
	UNDO_DELETE,
	UNDO_FMT_CHANGE,
	UNDO_DEL_FMTD,
	UNDO_TAG_APLD,
	UNDO_TAG_REMD,
	UNDO_BG_MOD,
	UNDO_MSG_TYPE_CHANGE,
	UNDO_BULLET_CHANGE
} UndoAction;

typedef struct _UndoOpInfo {
	GtkWidget *undobuf;
	UndoAction action;
	gchar *text;
	gint start_pos;
	gint end_pos;
	gint cursor_pos;
	gint del_start;
	gint del_end;
	GdkColor *prevcolor;
	GSList *tag_list;
	GSList *img_list;
} UndoOpInfo;

typedef struct _imgInfo {
	gint img_offset;
	GdkPixbuf *pixbuf;
} imgInfo;

void undoins(GtkTextBuffer * undobuffer, GtkTextIter * iter,
	     gchar * undotext, gint len, EditorInfo * editorinfo);
void undo_image_insertion(GtkTextBuffer * buffer, gint no_of_images, EditorInfo * editorinfo);
void undo_paste(GtkTextBuffer * buffer, gint start_offset, EditorInfo * editorinfo);
void undodel(GtkTextBuffer * undobuffer, GtkTextIter * delstart,
	     GtkTextIter * delend, EditorInfo * editorinfo);
UndoOpInfo *update_undoinfo_on_format_change(EditorInfo * editorinfo, GtkWidget * formatitem);
UndoOpInfo *update_undo_on_msg_type_change(EditorInfo * editorinfo);

void undoinssub(GtkEditable * entry, gchar * undotext, gint len,
		gint * position, EditorInfo * editorinfo);

void undodelsub(GtkEditable * entry, gint start, gint end, EditorInfo * editorinfo);
void update_undoinfo_on_bg_change(EditorInfo * editorinfo,
				  GtkTextIter start_it, GtkTextIter end_it);
void undo_set_delete_range(UndoOpInfo * op, EditorInfo * editorinfo);

void undo_do_undo(GtkWidget * win, EditorInfo * editorInfo);

void undo_free_op(UndoOpInfo * op);

void undo_destroy(EditorInfo * editorinfo);
