/**
 * @file viewer.h
 *
 * This file contains the function declarations  & the structures
 * needed for viewer and editor modules of OSSO_EMAIL.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */

#ifndef __VIEWER_H__
#define __VIEWER_H__

#define TOOLBAR_MARGIN_BTW_BUTTONS 32
#define TOOLBAR_LEFT_MARGIN 32
#define TOOLBAR_RIGHT_MARGIN 32
#define FULL_SCREEN_TOOLBAR_MARGIN_BTW_BUTTONS 41
#define FULL_SCREEN_TOOLBAR_LEFT_MARGIN 41
#define FULL_SCREEN_TOOLBAR_RIGHT_MARGIN 40
#define TOOLBAR_BUTTON_WIDTH 48
#define DATE_TIME_SPACE 12
#define UTF8 "UTF-8"
#define BUFFER_SPACE 5
#define ARRIVAL_TIME_BUF 15
#define KILOBYTE 1024
#define BODY_BUFFER 1000
#define ATTACHMENT_SIZE_BUFFER 20
#define PATH_SEPARATOR ':'
#define FILE_PERMISSION 0x400
#define EMAIL_TYPE "email attachment"
#define ATTACHMENT "attachment"
#define ADDR_FIELDS_SIZE 23
#define DEFAULT_WIDTH -1
#define DEFAULT_HEIGHT -1
#define SUBJECT_FIELD_WIDTH 80
#define ZERO_PADDING 0
#define ZERO_SPACING 0
#define X_RIGHT 1
#define Y_BOTTOM 0
#define XALIGN 0.5
#define YALIGN 0.5
#define XSPACE 0
#define YSPACE 0
#define ZERO_ZOOM 0
#define NEXT 1
#define PREV 1
#define BODY_WIDTH 75
#define BODY_HEIGHT 111
#define DNL "dnl"
#define IMAGE "image"
#define VIDEO "video"
#define AUDIO "audio"
#define VCARD "text/x-vcard"
#define VCALENDAR "text/x-vcalendar"
#define SKETCH "sketch/png"
#define PDF "application/pdf"
#define PLAIN_TEXT "text/plain"
#define ICON_SIZE 20
#define ATTA_FIELDS_SIZE 30
#define MAX_ATTA_FIELDS_SIZE 90
#define NOT_DEFINED -1
#define RETURN_ERROR -1
/*Browser & mime related header files.*/
#include <osso-mime.h>
#include "ui_utils.h"
#include "ui_defs.h"
#include "mailviewer.h"
#include<osso-browser-interface.h>

typedef struct _ViewerMenu ViewerMenu;
typedef struct _ViewerToolbar ViewerToolbar;
typedef struct _MsgFields MsgFields;
typedef struct _MailInfo MailInfo;
typedef struct _SignalId SignalId;
typedef struct _ViewerInfo ViewerInfo;


enum {
	URL = 2,
	EMAIL_ID = 4
};

enum {
	NONE_PURGED,
	SOME_PURGED,
	ALL_PURGED,
	NOT_LOCAL,
	SOME_NOT_LOCAL
};

enum {
	ATTACH_NOTRETRIEVED,
	EMAIL_ATTACH,
	NORMAL_ATTACH
};

enum {
	COL_ICON,
	COL_ATT_NAME,
	COL_ATT_SIZE,
	COL_ATT_NUM,
	NUM_COL
};


struct _ViewerMenu {
	GtkWidget *message_menuitem;
	GtkWidget *reply;
	GtkWidget *resend;
	GtkWidget *reply_all;
	GtkWidget *forward;
	GtkWidget *newmail;
	GtkWidget *delete_msg;
#if CHANGED_EMAIL
	GtkWidget *detail_msg;
#endif
	GtkWidget *edit;
	GtkWidget *copy;
	GtkWidget *cut;
	GtkWidget *paste;
	GtkWidget *selectall;

	GtkWidget *attachments_menuitem;
	GtkWidget *view;
	GtkWidget *save_attachment;
	GtkWidget *purge_attachment;

	GtkWidget *toolbar_normalview;
	GtkWidget *toolbar_fullscreen;
	GtkWidget *fullscreen;
	GtkWidget *next_msg;
	GtkWidget *prev_msg;
	GtkWidget *zoom;
	GtkWidget *zoom_out_max_level;
	GtkWidget *zoom_out_level;
	GtkWidget *current_zoom_level;
	GtkWidget *zoom_in_level;
	GtkWidget *zoom_80;
	GtkWidget *zoom_120;

	GtkWidget *tools;
	GtkWidget *font_settings;
	GtkWidget *add_to_contacts;
	GtkWidget *find;
	GtkWidget *searchemails;
	GtkWidget *help;

	GtkWidget *properties;
	GtkWidget *close;
	GtkWidget *close_window;
	GtkWidget *close_all_windows;
};

struct _ViewerToolbar {
	GtkWidget *left_space;
	GtkWidget *viewer_toolbar;
	GtkWidget *bbox_toolbar;
	GtkWidget *reply;
	GtkWidget *reply_space;
	GtkWidget *replyAll;
	GtkWidget *replyAll_space;
	GtkWidget *resend;
	GtkWidget *resend_space;
	GtkWidget *forward;
	GtkWidget *forward_space;
	GtkWidget *saveAttachment;
	GtkWidget *saveAttachment_space;
	GtkWidget *moveTo;
	GtkWidget *moveTo_space;
	GtkWidget *delete_msg;
	GtkWidget *delete_msg_space;
	GtkWidget *prevMsg;
	GtkWidget *prevMsg_space;
	GtkWidget *nextMsg;
	GtkWidget *nextMsg_space;
	GtkWidget *progress_hbox;
	GtkWidget *progressbar;
	GtkWidget *progressbar_cancel;
	GtkWidget *separator;
	GtkWidget *right_space;
};

struct _MsgFields {
	GtkWidget *vbox;
	GtkWidget *table;
	GtkWidget *priority_image;
	GtkWidget *subject;
	GtkWidget *encrypt_image;

	GtkWidget *attachment_lable;
	GtkWidget *attachment_view;
	GtkWidget *view;
	GtkWidget *sw_attach;
	GtkListStore *liststore;

	GtkWidget *from;
	GtkWidget *from_lable;
	GtkWidget *from_hbox;

	GtkWidget *to_addrimage;
	GtkWidget *to_lable;
	GtkWidget *to;
	GtkWidget *sign_image;
	GtkWidget *to_hbox;
	GtkWidget *to_lable_hbox;

	GtkWidget *cc_addrimage;
	GtkWidget *cc_lable;
	GtkWidget *cc;
	GtkWidget *ccencrypt_image;
	GtkWidget *cc_hbox;
	GtkWidget *cc_lable_hbox;

	GtkWidget *subject_lable;
	GtkWidget *subject_lable_hbox;
	GtkWidget *subject_hbox;

	GtkWidget *bcc_addrimage;
	GtkWidget *bcc_lable;
	GtkWidget *bcc;
	GtkWidget *bccencrypt_image;
	GtkWidget *bcc_hbox;
	GtkWidget *bcc_lable_hbox;

	GtkWidget *date_editor;
	GtkWidget *time_editor;
	GtkWidget *hbox_dateTime;
	GtkWidget *date_lable;
	GtkWidget *scrollwindow_body;
	GtkWidget *scroll_area;
	GtkWidget *vbox_msg;
	GtkWidget *body;
	GtkWidget *mail_viewer;
	GtkWidget *find_toolbar_hbox;
	GtkWidget *find_toolbar;

	GtkWidget *hide_headers_button;
	GtkWidget *hide_headers_image;
	gboolean cc_available;
	gboolean from_available;
	gboolean attachments_available;
	gboolean mycard_attached;

};
struct _MailInfo {
	gint protocol;
	RecvType recv_type;
	gboolean fully_downloaded;
	gboolean body_downloaded;
	gint row_num;
	gint folder_num;
	gint priority;
	gint date;
	gint msgnum;
	gboolean htmlmail;
	gboolean email_attachment;
	gint smime_type;
	gint zoomvalue;
	gboolean cc_field;
	gboolean set_prev_settings;
	gboolean progress_on;
	gboolean headers_hidden;
	gboolean same_appview;
	guint timer_value;
};

struct _SignalId {
	guint close_signalid;
	guint increase_zoom_signalid;
	guint decrease_zoom_signalid;
	guint state_save_signalid;
	guint purge_icons_signalid;
	guint fullscreen_signalid;
	guint update_mimeinfo_signalid;
	guint update_msginfo_signalid;
	guint update_progressbar_signalid;
	guint resend_signalid;
	guint cancel_progressbar_signalid;
};

enum {
	TO,
	CC,
	BCC,
	FROM
};

typedef enum {
	ATTACHMENTS_PRESENT,
	NO_ATTACHMENTS,
	MEM_ALLOCATION_FAIL
} AttachmentStatus;

enum {
	NOT_MHTML,
	MHTML_ATTACHMENT,
	DECRYPT_ERROR
};
typedef struct _AddressFieldInfo {
	GtkWidget *field_sw;
	GtkWidget *field;
	GtkWidget *field_hbox;
	gboolean from_viewer;
} AddressFieldInfo;
enum {
	SAVING,
	VIEWING_EMAIL,
	VIEWING_ATTA,
	OPENING,
	FORWARDING,
	NOACTION
};
enum {
	EMAIL_ATTA,
	NON_EMAIL_ATTA,
	ERROR
};



struct _ViewerInfo {
	GtkWidget *viewer_appview;
	ViewerMenu *menu;
	ViewerToolbar *toolbar;
	MsgFields *msg_fields;
	MailInfo *mail_info;
	SignalId *signal_id;
	MimeInfo *mime_info;
	AddressFieldInfo *fieldinfo_to;
	AddressFieldInfo *fieldinfo_cc;
	AddressFieldInfo *fieldinfo_from;
	gchar *msgid;
	gchar *receiverid;
	gchar *senderid;
	gchar *password;
	guint certid;
	gchar *email_attachment_filename;
	gchar *email_attachment_body_path;
	gchar *reply_to;
	gboolean toolbarhidden_normalview;
	gboolean toolbarhidden_fullscreen;
	GtkWidget *viewer_toolbar;
	GtkWidget *viewer_fullscreen_toolbar;
	Zoom *zoominfo;
	gint mode;
	gchar *plain_filename;
	GSList *mimeinfo_list;
	GSList *seqno_list;
	GSList *selected_attachment_names;
	GSList *local_attachments;
	GSList *selected_attachment_paths;
	GSList *selected_attachment_names_save;
};

#endif
