/**
 * @file signatures.c
 *
 * This file contains the functions to create the signature editor window.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


/*Inclusion of header file which contains all standerd paths*/
#include "ui_common.h"

/*Inclusion of user defined header files.*/
#include "signatures_callbacks.h"
#include "enablesignatures_callbacks.h"
#include "utils.h"
#include "ui_account_store.h"
#include "accountMgmt_callback.h"
#include "ui_interface.h"

/* Global variables*/
extern MainViewInfo *email_main_view;	/* Declared in ui_interface.c */


/**
This function is used to create the signature window
@param  accountname Name of the account in string format whose signature is shown.
@param  enablesignature_account_view  pointer to the widget by which this
routine is called.
@returns  GtkWidget  pointer to the widget which is newly created.
**/
GtkWidget *create_signature_window(gchar * accountname, GtkWidget * enablesignature_account_view)
{
	GtkWidget *signature_dialog = NULL;
	GtkWidget *dialog_action_area1 = NULL;
	GtkWidget *signature_vbox = NULL;
	GtkWidget *signature_label = NULL;
	GtkWidget *signature_scrolledwindow = NULL;
	GtkWidget *signature_textview = NULL;
	GtkWidget *signature_okcancel_hbox = NULL;
	GtkWidget *signature_ok_button = NULL;
	GtkWidget *signature_cancel_button = NULL;
	gchar *sig_body = NULL;
	GtkTextBuffer *buffer_local = NULL;
	gchar *labelaccounts = NULL;
	GSList *signaturedata_list = NULL;
	gboolean *sign_change = NULL;
	gint str_len = 0;
	gchar *signaturetext = NULL;
	gchar *signature_file = NULL;

	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	if (NULL == accountname) {
		ULOG_WARN("[%s][%s]:the input accountname is NULL :\n", __FILE__, __FUNCTION__);
		return NULL;
	}
	/*Create Signature Editor window */
	signature_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(signature_dialog),
			     (_("mcen_ti_email_signatures_edit_title")));
	gtk_window_set_transient_for(GTK_WINDOW(signature_dialog), GTK_WINDOW(app));
	gtk_window_set_modal(GTK_WINDOW(signature_dialog), TRUE);
	gtk_dialog_set_has_separator(GTK_DIALOG(signature_dialog), FALSE);
	/*Set application name to the window */
	g_object_set_data_full(G_OBJECT(signature_dialog), WINDOW_NAME,
			       g_strdup(APP_NAME), (GDestroyNotify) g_free);
	/*Set STATE_SAVE_DISABLED property to the window */
	email_main_view->tags->state_save_disabled = TRUE;
	g_object_set_data_full(G_OBJECT(signature_dialog), STATE_SAVE_DISABLED,
			       g_strdup(APP_NAME), (GDestroyNotify) ui_complex_dlg_destroy_notify);

	signature_vbox = GTK_DIALOG(signature_dialog)->vbox;
	gtk_widget_show(signature_vbox);
	str_len = strlen(_("mcen_ia_email_signatures_edit_dlg_label")) + strlen(accountname) + 1;
	labelaccounts = g_new(gchar, str_len);
	g_snprintf(labelaccounts, str_len,
		   _("mcen_ia_email_signatures_edit_dlg_label"), accountname);
	signature_label = gtk_label_new(NULL);
	gtk_label_set_text(GTK_LABEL(signature_label), labelaccounts);
	gtk_widget_show(signature_label);
	gtk_box_pack_start(GTK_BOX(signature_vbox), signature_label, FALSE, FALSE, 0);
	g_free(labelaccounts);
	labelaccounts = NULL;
	/* Create Scrollbar window to display signature data */
	signature_scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(signature_scrolledwindow);
	gtk_box_pack_start(GTK_BOX(signature_vbox), signature_scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (signature_scrolledwindow),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type((GtkScrolledWindow *)
					    signature_scrolledwindow, GTK_SHADOW_ETCHED_IN);

	signature_textview = gtk_text_view_new();
	gtk_widget_show(signature_textview);
	buffer_local = gtk_text_view_get_buffer(GTK_TEXT_VIEW(signature_textview));

	sign_change = g_new0(gboolean, 1);
	*sign_change = FALSE;
	signature_file = get_signature_filepath(accountname);
	read_file(signature_file, &signaturetext);
	g_free(signature_file);

	if (signaturetext != NULL) {
		sig_body = (gchar *)
		    normalize_newlines_in_signature(signaturetext);
		gtk_text_buffer_set_text(buffer_local, sig_body, -1);
		gtk_text_view_set_buffer(GTK_TEXT_VIEW(signature_textview), buffer_local);
		g_free(sig_body);
		g_free(signaturetext);
	}

	gtk_widget_show(signature_textview);

	gtk_container_add(GTK_CONTAINER(signature_scrolledwindow), signature_textview);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(signature_textview), GTK_WRAP_WORD);

	signature_okcancel_hbox = gtk_hbox_new(TRUE, 0);
	gtk_widget_show(signature_okcancel_hbox);
	gtk_box_pack_start(GTK_BOX(signature_vbox), signature_okcancel_hbox, TRUE, TRUE, 0);

	dialog_action_area1 = GTK_DIALOG(signature_dialog)->action_area;
	gtk_widget_show(dialog_action_area1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1), GTK_BUTTONBOX_END);

	signature_ok_button = gtk_button_new_with_mnemonic(_("mcen_bd_email_signatures_edit_ok"));
	gtk_widget_show(signature_ok_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(signature_dialog), signature_ok_button, 0);

	signature_cancel_button =
	    gtk_button_new_with_mnemonic(_("mcen_bd_email_signatures_edit_cancel"));
	gtk_widget_show(signature_cancel_button);
	gtk_dialog_add_action_widget(GTK_DIALOG(signature_dialog), signature_cancel_button, 0);

	signaturedata_list = g_slist_append(signaturedata_list, signature_dialog);
	signaturedata_list = g_slist_append(signaturedata_list, signature_textview);
	signaturedata_list = g_slist_append(signaturedata_list, accountname);
	signaturedata_list = g_slist_append(signaturedata_list, enablesignature_account_view);
	g_slist_append(signaturedata_list, sign_change);

	/* Connect signals to callback functions */
	g_signal_connect(G_OBJECT(signature_ok_button), "clicked",
			 G_CALLBACK(on_click_signature_ok_button), signaturedata_list);

	g_signal_connect(G_OBJECT(signature_cancel_button), "clicked",
			 G_CALLBACK(on_click_signature_cancel_button), signature_dialog);
	g_signal_connect(G_OBJECT(buffer_local), "changed", G_CALLBACK(sign_changed), sign_change);
	g_signal_connect(G_OBJECT(signature_dialog), "delete_event",
			 G_CALLBACK(free_sign_bool), sign_changed);
	if (ossohelp_exists(HELP_ID_SIGNATUREEDITOR) == OSSO_OK) {
		gtk_dialog_help_enable((GtkDialog *) signature_dialog);
		g_signal_connect(G_OBJECT(signature_dialog), "help",
				 G_CALLBACK(ui_on_dialog_help_activate), HELP_ID_SIGNATUREEDITOR);
	}
	g_signal_connect(G_OBJECT(signature_dialog), "key-release-event",
			 G_CALLBACK(ui_on_escape_key_pressed), signature_cancel_button);

	EMAIL_UI_HOOKUP_OBJECT_NO_REF(signature_dialog, signature_dialog, "emailSignatureDialog");
	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return signature_dialog;
}

/* end of file */
