/**
 * @file signatures_callbacks.c
 *
 * This file contains the callback functions that are being used by 
 * the signature window.  
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


#include "ui_common.h"
#include "osso-log.h"

#include "signatures_callbacks.h"
#include "enablesignatures_callbacks.h"
#include "intlsupport.h"
#include "ui_utils.h"
#include "ui_account_store.h"


static gboolean get_set_sign_file(gchar * textbuf, gchar * account)
{
	FILE *signaturefile = NULL;
	gchar *signature_file_name = NULL;

	signature_file_name = get_signature_filepath(account);
	signaturefile = fopen(signature_file_name, WRITE_MODE);
	g_free(signature_file_name);
	if (NULL == signaturefile) {
		get_user_confirmation((_("emev_ni_internal_error")),
				      (_("mcen_bd_email_signatures_ok")), NULL, INFO_BANNER);
		return FALSE;
	}

	fprintf(signaturefile, "%s", textbuf);
	fclose(signaturefile);

	return TRUE;
}


/**
This function is used to update the data on clicking the ok button in the signature window
@param button -- okbutton which which is being clicked
@param signaturedata -- the data which contains the signature specific information
@return void
*/
void on_click_signature_ok_button(GtkWidget * button, const gpointer signaturedata)
{
	GSList *signaturedata_list = NULL;
	GtkWidget *signature_window = NULL;
	GtkWidget *signature_textview_local = NULL;
	GtkTextBuffer *buffer = NULL;
	gchar *textbuf = NULL;
	GtkTextIter startiter;
	GtkTextIter enditer;
	gchar *accountname_local = NULL;
	GtkTreeView *enablesignature_account_view_local = NULL;
	GtkTreeModel *model = NULL;
	GtkTreeIter iter_get;
	gchar *username_select = NULL;
	gboolean enabled_select = FALSE;
	gboolean enabled_2_select = FALSE;
	gboolean result = FALSE;
	gboolean *sign_changed = NULL;

	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);
	if (NULL == signaturedata) {
		ULOG_WARN("[%s][%s]:the input signaturedata is NULL :\n", __FILE__, __FUNCTION__);
		return;
	}

	signaturedata_list = (GSList *) signaturedata;
	signature_window = signaturedata_list->data;
	signaturedata_list = g_slist_next(signaturedata_list);
	if (signaturedata_list != NULL) {
		signature_textview_local = (GtkWidget *) signaturedata_list->data;
	} else {
		ULOG_DEBUG_L("[%s][%s]:the input signaturedata_list is NULL :\n",
			     __FILE__, __FUNCTION__);
		return;
	}


	signaturedata_list = g_slist_next(signaturedata_list);
	if (NULL != signaturedata_list) {
		accountname_local = (gchar *) signaturedata_list->data;
	} else {
		ULOG_DEBUG_L("[%s][%s]:the input signaturedata_list is NULL :\n",
			     __FILE__, __FUNCTION__);
		return;
	}

	signaturedata_list = g_slist_next(signaturedata_list);
	if (NULL != signaturedata_list) {
		enablesignature_account_view_local = GTK_TREE_VIEW(signaturedata_list->data);
	}
	signaturedata_list = g_slist_next(signaturedata_list);
	if (NULL != signaturedata_list) {
		sign_changed = (gboolean *) (signaturedata_list->data);
	}


	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(signature_textview_local));
	gtk_text_buffer_get_bounds(buffer, &startiter, &enditer);
	textbuf = gtk_text_buffer_get_text(buffer, &startiter, &enditer, FALSE);
	if (get_set_sign_file(textbuf, accountname_local) == FALSE) {
		return;
	}
	if (*sign_changed == TRUE) {
		gtk_infoprint(GTK_WINDOW(app), _("mcen_ib_email_signatures_edit_saved"));
	}

	if (NULL == textbuf) {
		return;
	}
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(enablesignature_account_view_local));
	if (NULL == model) {
		ULOG_DEBUG_L("[%s][%s]:Failed to get the model :\n", __FILE__, __FUNCTION__);
		g_free(textbuf);
		return;
	}

	result = gtk_tree_model_get_iter_first(model, &iter_get);

	while (TRUE == result) {
		gtk_tree_model_get(model, &iter_get, 0,
				   &enabled_select, 1, &username_select, 3, &enabled_2_select, -1);
		if (0 == strcmp(username_select, accountname_local)) {
			if (0 == strlen(textbuf)) {
				enabled_select = 0;
				gtk_list_store_set(GTK_LIST_STORE(model), &iter_get,
						   0, enabled_select, 1, username_select,
						   2, _("mcen_ia_email_signatures_notdefined"),
						   3, FALSE, -1);
			} else {
				gtk_list_store_set(GTK_LIST_STORE(model), &iter_get,
						   2, textbuf, 3, TRUE, -1);
			}
			g_free(username_select);
			username_select = NULL;
			break;
		}
		result = gtk_tree_model_iter_next(model, &iter_get);
		g_free(username_select);
		username_select = NULL;
	}

	g_free(accountname_local);
	g_free(textbuf);
	gtk_widget_destroy(GTK_WIDGET(signature_window));
	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
}

/**
This function is called whenever the signature editor is changed
@param win is the GtkTextBuffer
@param sign_changed is boolean variable to be updated.
@ return void
**/
void sign_changed(GtkWidget * win, gboolean * sign_changed)
{
	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);
	if ((NULL == win) || NULL == (sign_changed)) {
		ULOG_WARN("[%s][%s]:the input  is NULL :\n", __FILE__, __FUNCTION__);
		return;
	}
	*sign_changed = TRUE;

	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This function is used to free the memory
@param win is the window being destroyed
@param sign_changed is boolean variable
@ return void
**/
void free_sign_bool(GtkWidget * win, gboolean * sign_changed)
{
	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);

	if ((NULL == win) || NULL == (sign_changed)) {
		ULOG_WARN("[%s][%s]:the input  is NULL :\n", __FILE__, __FUNCTION__);
		return;
	}
	g_free(sign_changed);
	sign_changed = NULL;

	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is used to update the data on clicking the ok button in the signature window
@param button -- okbutton which which is being clicked
@param signaturedata -- the window which contains signature specific information
@ returns void
**/
void on_click_signature_cancel_button(GtkWidget * button, gpointer signaturedata)
{
	ULOG_DEBUG_L("[%s][%s]:enters here :\n", __FILE__, __FUNCTION__);
	if (NULL == signaturedata) {
		ULOG_WARN("[%s][%s]:the input signaturedata is NULL :\n", __FILE__, __FUNCTION__);
		return;
	}

	gtk_widget_destroy(GTK_WIDGET(signaturedata));
	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
}

/**
This function is used for newline updation 
@param sign_text is signature
@return gchar
**/
gchar *normalize_newlines_in_signature(const gchar * sign_text)
{
	const gchar *signature_ptr = NULL;
	gchar *signature = NULL;
	gchar *sign_output = NULL;
	ULOG_DEBUG_L("[%s][%s]:starts from here :\n", __FILE__, __FUNCTION__);

	if (NULL == sign_text) {
		ULOG_WARN("[%s][%s]:input parameter is null :\n", __FILE__, __FUNCTION__);
		return NULL;

	}

	sign_output = g_malloc0((gulong) strlen(sign_text) + 1);
	if (sign_output == NULL) {
		ULOG_CRIT("[%s][%d] : Memory allocation failure!\n", __FUNCTION__, __LINE__);
		return NULL;
	}
	signature = sign_output;

	for (signature_ptr = sign_text; *signature_ptr != '\0'; ++signature_ptr) {
		if (*signature_ptr == '\r') {
			if (*(signature_ptr + 1) != '\n') {
				*sign_output++ = '\n';
			}
		} else {
			*sign_output++ = *signature_ptr;
		}
	}

	*sign_output = '\0';

	ULOG_DEBUG_L("[%s][%s]:leaves from here :\n", __FILE__, __FUNCTION__);
	return signature;
}

/* end of file */
