/**
 * @file ui_account_store.c 
 *
 * This file contains the implementation for prefs account 
 * initialization functions of OSSO EMAIL.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


#include "email_hildon.h"
#include "ui_utils.h"
#include "ui_common.h"
#include "ui_defs.h"
#include "accountMgmt_callback.h"
#include "accountMgmt.h"
#include "email_mgr.h"
#include "ui_account_store.h"

/* Function Declaration */
static GSList *account_detail_store = NULL;
static GSList *iap_smtp_servers = NULL;

static PrefsAccount *ui_copy_prefsaccount(PrefsAccount * s_update_prefsAccount)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	PrefsAccount *prefsaccount_temp = NULL;
	if (s_update_prefsAccount == NULL) {
		ULOG_DEBUG("[%s][%s] : Input param check failed! :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}

	prefsaccount_temp = g_new0(PrefsAccount, 1);

	if (s_update_prefsAccount->user_settings.account_name != NULL) {
		prefsaccount_temp->user_settings.account_name =
		    g_strdup(s_update_prefsAccount->user_settings.account_name);
	} else {
		prefsaccount_temp->user_settings.account_name = NULL;
	}
	if (s_update_prefsAccount->user_settings.userid != NULL) {
		prefsaccount_temp->user_settings.userid =
		    g_strdup(s_update_prefsAccount->user_settings.userid);
	} else {
		prefsaccount_temp->user_settings.userid = NULL;
	}
	if (s_update_prefsAccount->user_settings.passwd != NULL) {
		prefsaccount_temp->user_settings.passwd =
		    g_strdup(s_update_prefsAccount->user_settings.passwd);
	} else {
		prefsaccount_temp->user_settings.passwd = NULL;
	}
	if (s_update_prefsAccount->send_settings.username != NULL) {
		prefsaccount_temp->send_settings.username =
		    g_strdup(s_update_prefsAccount->send_settings.username);
	} else {
		prefsaccount_temp->send_settings.username = NULL;
	}
	if (s_update_prefsAccount->send_settings.password != NULL) {
		prefsaccount_temp->send_settings.password =
		    g_strdup(s_update_prefsAccount->send_settings.password);
	} else {
		prefsaccount_temp->send_settings.password = NULL;
	}
	if (s_update_prefsAccount->user_settings.email != NULL) {
		prefsaccount_temp->user_settings.email =
		    g_strdup(s_update_prefsAccount->user_settings.email);
	} else {
		prefsaccount_temp->user_settings.email = NULL;
	}
	if (s_update_prefsAccount->user_settings.alias_name != NULL) {
		prefsaccount_temp->user_settings.alias_name =
		    g_strdup(s_update_prefsAccount->user_settings.alias_name);
	} else {
		prefsaccount_temp->user_settings.alias_name = NULL;
	}
	if (s_update_prefsAccount->receive_settings.recv_server != NULL) {
		prefsaccount_temp->receive_settings.recv_server =
		    g_strdup(s_update_prefsAccount->receive_settings.recv_server);
	} else {
		prefsaccount_temp->receive_settings.recv_server = NULL;
	}
	if (s_update_prefsAccount->send_settings.smtp_server != NULL) {
		prefsaccount_temp->send_settings.smtp_server =
		    g_strdup(s_update_prefsAccount->send_settings.smtp_server);
	} else {
		prefsaccount_temp->send_settings.smtp_server = NULL;
	}
	if (s_update_prefsAccount->advanced_settings.signature_file != NULL) {
		prefsaccount_temp->advanced_settings.signature_file =
		    g_strdup(s_update_prefsAccount->advanced_settings.signature_file);
	} else {
		prefsaccount_temp->advanced_settings.signature_file = NULL;
	}
#if CL_USE
	prefsaccount_temp->advanced_settings.use_iap_smtp_serv =
	    s_update_prefsAccount->advanced_settings.use_iap_smtp_serv;
	if (s_update_prefsAccount->advanced_settings.iap_name != NULL) {
		prefsaccount_temp->advanced_settings.iap_name =
		    g_strdup(s_update_prefsAccount->advanced_settings.iap_name);
	} else {
		prefsaccount_temp->advanced_settings.iap_name = NULL;
	}
#endif

	prefsaccount_temp->receive_settings.protocol =
	    s_update_prefsAccount->receive_settings.protocol;
	prefsaccount_temp->advanced_settings.email_type =
	    s_update_prefsAccount->advanced_settings.email_type;
	prefsaccount_temp->advanced_settings.email_sending =
	    s_update_prefsAccount->advanced_settings.email_sending;
	prefsaccount_temp->advanced_settings.request_read_report =
	    s_update_prefsAccount->advanced_settings.request_read_report;
	prefsaccount_temp->receive_settings.port = s_update_prefsAccount->receive_settings.port;
	prefsaccount_temp->user_settings.is_default =
	    s_update_prefsAccount->user_settings.is_default;
	prefsaccount_temp->receive_settings.recv_type =
	    s_update_prefsAccount->receive_settings.recv_type;
	prefsaccount_temp->receive_settings.size_limit =
	    s_update_prefsAccount->receive_settings.size_limit;
	prefsaccount_temp->receive_settings.passwd_auth_type =
	    s_update_prefsAccount->receive_settings.passwd_auth_type;
	prefsaccount_temp->receive_settings.recent_mails =
	    s_update_prefsAccount->receive_settings.recent_mails;
	prefsaccount_temp->advanced_settings.copy_to_own_address =
	    s_update_prefsAccount->advanced_settings.copy_to_own_address;
	prefsaccount_temp->advanced_settings.use_ssl =
	    s_update_prefsAccount->advanced_settings.use_ssl;
	prefsaccount_temp->advanced_settings.use_tls =
	    s_update_prefsAccount->advanced_settings.use_tls;
	prefsaccount_temp->advanced_settings.smtp_use_ssl =
	    s_update_prefsAccount->advanced_settings.smtp_use_ssl;
	prefsaccount_temp->advanced_settings.smtp_use_tls =
	    s_update_prefsAccount->advanced_settings.smtp_use_tls;
	prefsaccount_temp->receive_settings.enable_size_limit =
	    s_update_prefsAccount->receive_settings.enable_size_limit;
	prefsaccount_temp->send_settings.smtp_auth_type =
	    s_update_prefsAccount->send_settings.smtp_auth_type;
	prefsaccount_temp->advanced_settings.include_signature =
	    s_update_prefsAccount->advanced_settings.include_signature;
	prefsaccount_temp->advanced_settings.org_in_reply =
	    s_update_prefsAccount->advanced_settings.org_in_reply;
	prefsaccount_temp->send_settings.smtp_auth_type =
	    s_update_prefsAccount->send_settings.smtp_auth_type;
	prefsaccount_temp->send_settings.smtp_port = s_update_prefsAccount->send_settings.smtp_port;
	prefsaccount_temp->advanced_settings.smime_security =
	    s_update_prefsAccount->advanced_settings.smime_security;
	prefsaccount_temp->advanced_settings.display_sign_encrypt =
	    s_update_prefsAccount->advanced_settings.display_sign_encrypt;
	prefsaccount_temp->advanced_settings.display_trust =
	    s_update_prefsAccount->advanced_settings.display_trust;
	prefsaccount_temp->advanced_settings.display_validity =
	    s_update_prefsAccount->advanced_settings.display_validity;
	prefsaccount_temp->advanced_settings.signature_flag =
	    s_update_prefsAccount->advanced_settings.signature_flag;
	prefsaccount_temp->receive_settings.leave_msg =
	    s_update_prefsAccount->receive_settings.leave_msg;


	prefsaccount_temp->receive_settings.enable_size_limit = 1;


	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return prefsaccount_temp;
}

void ui_init_account_store(void)
{
	ui_free_account_detail_store();
	ui_dbus_prefs_account_get_list(&account_detail_store);
	return;
}

PrefsAccount *ui_get_account_detail(const gchar * account_name)
{
	GSList *temp_account_detail_store = NULL;
	PrefsAccount *temp_prefs_account = NULL;
	PrefsAccount *prefs_account = NULL;

	temp_account_detail_store = account_detail_store;
	while (temp_account_detail_store) {
		temp_prefs_account = (PrefsAccount *) (temp_account_detail_store->data);
		if (strcmp(temp_prefs_account->user_settings.account_name, account_name) == 0) {
			prefs_account = ui_copy_prefsaccount(temp_prefs_account);
			return prefs_account;
		}
		temp_account_detail_store = g_slist_next(temp_account_detail_store);
	}
	return NULL;
}

gboolean ui_update_account_setting(SettingType type, const PrefsAccount * prefs_account)
{
	if (ui_dbus_set_account_settings(type, prefs_account) == SETTING_SUCCESS) {
		ui_init_account_store();
		return TRUE;
	} else {
		return FALSE;
	}
}

void ui_free_account_detail_store(void)
{
	if (account_detail_store == NULL) {
		return;
	}

	g_slist_foreach(account_detail_store, (GFunc) ui_free_prefs_account_structure, NULL);
	g_slist_free(account_detail_store);
	account_detail_store = NULL;
}

/*Iap connect specific server related changes*/
/**
 This function is used to get the list of configured connection specific
 smtp servers .The returned list is stored in the global list.

 @return void.
*/
#if CL_USE
void ui_init_smtp_conn_servers(void)
{
	ui_free_iap_smtp_servers_list();
	iap_smtp_servers = ui_dbus_get_iapinfo_list();
	return;
}

/**
 This function gets the configured smtp server for specified iap.
 
 @param iapname is the gchar pointer to the iap name.
 @return IapInfo , pointer to the IapInfo structure containg the server details.
*/
const IapInfo *ui_get_smtp_server_detail(const gchar * iapname)
{
	GSList *temp_iap_smtp_servers = NULL;
	IapInfo *temp_smtp_server_info = NULL;

	temp_iap_smtp_servers = iap_smtp_servers;
	while (temp_iap_smtp_servers) {
		temp_smtp_server_info = (IapInfo *) (temp_iap_smtp_servers->data);
		if (ui_compare_utf8_strings(temp_smtp_server_info->iap_name, iapname) == 0) {
			return temp_smtp_server_info;
		}
		temp_iap_smtp_servers = temp_iap_smtp_servers->next;
	}
	return NULL;
}

/**
 This function is used to update the modified smtp server details for the iap.
 @param smtp_server_info is pointer to the IapInfo structure.
 @return gboolean TRUE- on success
                  FALSE - else.
*/
gboolean ui_update_iap_smtp_server_details(const IapInfo * smtp_server_info)
{
	if (ui_dbus_set_iapinfo(smtp_server_info) == SETTING_SUCCESS) {
		ui_init_smtp_conn_servers();
		return TRUE;
	} else {
		return FALSE;
	}
}

/**
 Helper function to free the global iapinfo list conating IapInfo sturtures.
 
 @return void.
*/
void ui_free_iap_smtp_servers_list(void)
{

	if (!iap_smtp_servers) {
		return;
	}
	g_slist_foreach(iap_smtp_servers, (GFunc) ui_free_iapinfo_structure, NULL);
	g_slist_free(iap_smtp_servers);
	iap_smtp_servers = NULL;
}

/**
 FUnction to return the list of configured Iap smtp servers.
 @return GSList of configured  Iap smtp servers.
*/
const GSList *ui_get_iapinfo_list(void)
{
	return iap_smtp_servers;
}
#endif
/* End of file */
