/**
 * @file viewer.c
 *
 * This file contains all function to the create viewer
 *
 *  Copyright (c) 2005 Nokia. All rights reserved
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
*/

/*Editor-Viewer header files*/
#include "editcallback.h"
#include "callback.h"
#include "html_parser.h"
/*smime related headerfile*/
#include "security.h"

/*account management related header files*/
#include "ui_account_store.h"
#include "engine_errno.h"
#include "osso-uri.h"
extern MainViewInfo *email_main_view;	/*Declared in ui_interface.c */
extern GSList *passwd_list;	/*list for caching the decryption passwords */

static guint close_signalid = 0;
static guint increase_zoom_signalid = 0;
static guint decrease_zoom_signalid = 0;
static guint state_save_signalid = 0;
static guint purge_icons_signalid = 0;
static guint fullscreen_signalid = 0;
static guint update_mimeinfo_signalid = 0;
static guint update_msginfo_signalid = 0;
static guint update_progressbar_signalid = 0;
static guint resend_signalid = 0;
static guint cancel_progressbar_signalid = 0;

static gboolean hide_unhide_in_focus = TRUE;	/*Flag to check Whether the focus is in hide headers button */
gboolean signals_created = FALSE;	/*Flag to keep the creation of new signals only once */
gboolean common_signals_created = FALSE;	/*Flag to keep the creation of new signals only once */
GtkWidget *currentviewer = NULL;

static gboolean update_progress(ViewerInfo * viewer_info);
static void update_body_load_progress(GtkWidget * mail_viewer, gfloat frac,
				      ViewerInfo * viewer_info);
static void ui_viewer_embed_body(ViewerInfo * viewer_info);
static void ui_set_smime_listview_icon(const MailInfo * mail_info);
static void assing_signalid_to_global_ids(ViewerInfo * viewer_info);
static void hide_headers(GtkWidget *, GdkEvent * event, ViewerInfo * viewer_info);
void assing_new_mimeinfo(ViewerInfo * viewer_info, const MimeInfo * mime_info);
static void hyperlink_clicked(GtkWidget * mail_viewer, const gchar * uri, OssoURIAction * action,
			      ViewerInfo * viewer_info);
static void copy_hyperlink_clicked(GtkWidget * mail_viewer, const gchar * uri);

/**Function to hide progressbar
 
 @param viewer_info is the ViewerInfo structure pointer
 @return void
*/
void hide_progressbar(ViewerInfo * viewer_info)
{
	ViewerToolbar *toolbar = NULL;
	toolbar = viewer_info->toolbar;
	if (toolbar->reply != NULL) {
		gtk_widget_set_sensitive(toolbar->reply, TRUE);
		gtk_widget_set_sensitive(toolbar->replyAll, TRUE);
		gtk_widget_set_sensitive(viewer_info->menu->reply, TRUE);
		gtk_widget_set_sensitive(viewer_info->menu->reply_all, TRUE);
	} else {
		gtk_widget_set_sensitive(toolbar->resend, TRUE);
		gtk_widget_set_sensitive(viewer_info->menu->resend, TRUE);
	}

	gtk_widget_set_sensitive(toolbar->forward, TRUE);
	gtk_widget_hide(toolbar->progress_hbox);
	gtk_widget_hide(toolbar->separator);
	gtk_widget_hide(toolbar->progressbar_cancel);
	gtk_widget_show(toolbar->nextMsg);
	gtk_widget_show(toolbar->nextMsg_space);
	gtk_widget_show(toolbar->prevMsg);
	gtk_widget_show(toolbar->prevMsg_space);
	gtk_widget_show(toolbar->moveTo);
	gtk_widget_show(toolbar->moveTo_space);
	gtk_widget_show(toolbar->delete_msg);
	gtk_widget_show(toolbar->delete_msg_space);

	/*bug fix for 19712,dimming view menu */
	if (viewer_info->mode != OPENING)
		gtk_widget_set_sensitive(viewer_info->menu->view, TRUE);
	if (viewer_info->mode == SAVING)
		gtk_widget_set_sensitive(viewer_info->menu->save_attachment, TRUE);
	gtk_widget_set_sensitive(viewer_info->menu->forward, TRUE);
	viewer_info->mail_info->progress_on = FALSE;

	/*removing periodical updation of progress bar once it is closed */
	if (viewer_info->mail_info->timer_value > 0)
		g_source_remove(viewer_info->mail_info->timer_value);
	viewer_info->mail_info->timer_value = 0;
	viewer_info->mode = NOACTION;
	return;
}

/*Function to show progressbar
 
 @param viewer_info is the ViewerInfo structure pointer
 @return void
 */
void show_progressbar(ViewerInfo * viewer_info)
{
	ViewerToolbar *toolbar = NULL;
	toolbar = viewer_info->toolbar;

/*when mail is opened from send folder reply toolitem is not represent.
 * hence this check*/
	if (toolbar->reply != NULL) {
		gtk_widget_set_sensitive(toolbar->reply, FALSE);
		gtk_widget_set_sensitive(toolbar->replyAll, FALSE);
		gtk_widget_set_sensitive(viewer_info->menu->reply, FALSE);
		gtk_widget_set_sensitive(viewer_info->menu->reply_all, FALSE);
	} else {
		gtk_widget_set_sensitive(toolbar->resend, FALSE);
		gtk_widget_set_sensitive(viewer_info->menu->resend, FALSE);
	}

	gtk_widget_set_sensitive(toolbar->forward, FALSE);
	gtk_widget_show_all(toolbar->progress_hbox);
	gtk_widget_show(toolbar->separator);
	gtk_widget_show(toolbar->progressbar_cancel);
	gtk_widget_hide(toolbar->nextMsg);
	gtk_widget_hide(toolbar->nextMsg_space);
	gtk_widget_hide(toolbar->prevMsg);
	gtk_widget_hide(toolbar->prevMsg_space);
	gtk_widget_hide(toolbar->moveTo);
	gtk_widget_hide(toolbar->moveTo_space);
	gtk_widget_hide(toolbar->delete_msg);
	gtk_widget_hide(toolbar->delete_msg_space);

	if (viewer_info->mail_info->body_downloaded == FALSE || viewer_info->mode != OPENING) {
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(toolbar->progressbar),
					  _("mcen_me_connecting"));
		ULOG_DEBUG_F("the text in progress bar is %s\n", _("mcen_me_connecting"));
	} else if (viewer_info->mode == OPENING) {
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(toolbar->progressbar),
					  _("mail_me_opening"));
		email_main_view->tags->current_mode = MSG_OPENING;
	}

	gtk_widget_set_sensitive(viewer_info->menu->view, FALSE);
	gtk_widget_set_sensitive(viewer_info->menu->save_attachment, FALSE);
	gtk_widget_set_sensitive(viewer_info->menu->forward, FALSE);
	viewer_info->mail_info->progress_on = TRUE;

	/*connecting a timeout call to update the progress bar */
	viewer_info->mail_info->timer_value =
	    g_timeout_add(1000, (GSourceFunc) update_progress, viewer_info);
	return;
}

/**Function to update progressbar text
 
 @param viewer is the appview pointer to the viewer. 
 @param viewer_info is the ViewerInfo structure pointer
 @return void
*/
void update_progress_bar_text(GtkWidget * viewer, ViewerInfo * viewer_info)
{

	ViewerToolbar *toolbar = viewer_info->toolbar;

	if (email_main_view->tags->current_mode == CONNECTING ||
	    email_main_view->tags->current_mode == MSG_OPENING) {
		if (viewer_info->mail_info->fully_downloaded == FALSE) {
			if (viewer_info->mail_info->body_downloaded == FALSE
			    || viewer_info->mode != OPENING) {
				gtk_progress_bar_set_text(GTK_PROGRESS_BAR(toolbar->progressbar),
							  _("mcen_me_connecting"));
				ULOG_DEBUG_F("the text in progress bar is %s\n",
					     _("mcen_me_connecting"));
			} else if (viewer_info->mode == OPENING
				   || email_main_view->tags->current_mode == MSG_OPENING) {
				gtk_progress_bar_set_text(GTK_PROGRESS_BAR(toolbar->progressbar),
							  _("mail_me_opening"));
				email_main_view->tags->current_mode = MSG_OPENING;
			}
		} else {
			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(toolbar->progressbar),
						  _("mail_me_opening"));
		}
	} else if (email_main_view->tags->current_mode == RECEIVING) {
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(viewer_info->toolbar->progressbar),
					      0.1);
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(viewer_info->toolbar->progressbar),
					  _("mail_me_receiving"));
		ULOG_DEBUG_F("the text in progress bar is %s\n", _("mcen_me_receiving"));
	} else if (email_main_view->tags->progress_bar_available == FALSE) {
		//hide_progressbar (viewer_info);
	}
	return;
}

/**Function cancel the current processing event.i.e recieving the mail. 
 
 @param viewer is the appview pointer to the viewer. 
 @param viewer_info is the ViewerInfo structure pointer
 @return void
*/
static void cancel_progressbar(GtkWidget * win, ViewerInfo * viewer_info)
{
	if (viewer_info->mail_info->progress_on == FALSE)
		return;

	ui_dbus_cancel_recv_mail(email_main_view->account_manager->current_sendrec_account, FALSE);
	hide_progressbar(viewer_info);

	if (viewer_info->mode == OPENING && HILDON_IS_APPVIEW(win) == FALSE)
		ui_viewer_close_cb(win, viewer_info);
	return;
}

/**Function to hide the from feild in case of mail opened from sent folder. 
 
 @param msgfields is the MsgFields structure pointer .
 @return void
*/
static void hide_from_field(MsgFields * msgfields)
{
	gtk_widget_hide(msgfields->from_lable);
	gtk_widget_hide(msgfields->from_hbox);
	gtk_widget_hide(msgfields->sign_image);
}

/**Function to dynamically set the heigth of the scrollwindow heigth
  max is 50% of total heigth. 
 
 @param viewer_info is the ViewerInfo structure pointer
 @return void
*/
void set_header_field_heigth(ViewerInfo * viewer_info)
{
	gint screen_width = 0, screen_height = 0, header_height = 0;

	GtkRequisition requisition = { 0, 0 };

	gtk_window_get_size(GTK_WINDOW(app), &screen_width, &screen_height);
	header_height = (screen_height / 2 - 3 * HILDON_MARGIN_DOUBLE);
	gtk_widget_size_request(viewer_info->msg_fields->table, &requisition);
	if (requisition.height >= header_height)
		gtk_widget_set_size_request(viewer_info->msg_fields->scrollwindow_body,
					    requisition.width, header_height);
	else
		gtk_widget_set_size_request(viewer_info->msg_fields->scrollwindow_body,
					    requisition.width + HILDON_MARGIN_DOUBLE,
					    requisition.height + HILDON_MARGIN_DOUBLE);
	gtk_widget_set_size_request(viewer_info->msg_fields->hide_headers_button, 30, 30);
	return;
}

/**
Hides the header fields if not required.

@param viewer_info is pointer to structure ViewerInfo having viewer related info.

@return void
*/
static void hide_header_widgets(ViewerInfo * viewer_info)
{
	CHECK_PARAM(viewer_info == NULL, "Input  param check failed", return);

	if (viewer_info->mail_info->folder_num == SENT_MODEL_NUM) {
		hide_from_field(viewer_info->msg_fields);
	}

	if (viewer_info->msg_fields->attachments_available == FALSE) {
		show_attachment_field(viewer_info->msg_fields, FALSE);
	}

	if (viewer_info->mime_info == NULL) {
		gtk_widget_hide(viewer_info->msg_fields->bccencrypt_image);
		if (viewer_info->msg_fields->ccencrypt_image)
			gtk_widget_hide(viewer_info->msg_fields->ccencrypt_image);
		if (viewer_info->msg_fields->encrypt_image != NULL)
			gtk_widget_hide(viewer_info->msg_fields->encrypt_image);
	}
	return;
}

/**
 This function is used update the viewer with the mimeinfo we recieve
 through callback from engine.

 @param viewer is the pointer to the viewer appview.
 @param viewer_info is the pointer to the ViewerInfo structure pointer.

 @return void.
*/

static void ui_set_body_thr_cb(GtkWidget * viewer, ViewerInfo * viewer_info)
{
	HeaderInfo *msgHeader = NULL;
	GtkTreeModel *model = NULL;
	gboolean fullydownloaded = FALSE;
	MsgFields *msgfields = NULL;
	gboolean attachmentstatus = FALSE;
	ViewerToolbar *toolbar = NULL;
	gchar *body = NULL;
	GtkWidget *forwardWindow = NULL;
	MimeInfo *mimeinfo = NULL;
	guint msgid_hash = 0;

	/*If for this msg already callback has come ignore
	 * them*/
	if (viewer_info->mode == NOACTION) {
		ui_mimeinfo_free(email_main_view->mimeinfo);
		email_main_view->mimeinfo = NULL;
		return;
	}

	msgfields = viewer_info->msg_fields;
	model = ui_get_unsorted_folder_model(viewer_info->mail_info->folder_num);

	/*update the mimeinfo recieved from the callback */
	if (viewer_info->mail_info->email_attachment == FALSE) {
		viewer_info->mime_info = email_main_view->mimeinfo;
	} else {
		assing_new_mimeinfo(viewer_info, email_main_view->mimeinfo);
	}
	viewer_info->mail_info->smime_type = viewer_info->mime_info->smime_type;
	ui_set_smime_listview_icon(viewer_info->mail_info);
	email_main_view->mimeinfo = NULL;

	fullydownloaded = check_attachments_downloaded(viewer_info->mime_info);

	if (viewer_info->mail_info->email_attachment == FALSE) {
		msgid_hash = g_str_hash(viewer_info->msgid);
		if (fullydownloaded == TRUE) {
			gtk_tree_model_foreach(model, ui_update_msg_fully_downloaded, &msgid_hash);
			viewer_info->mail_info->fully_downloaded = TRUE;
		}

		gtk_tree_model_foreach(model, ui_update_msg_body_downloaded, &msgid_hash);
		/*Set the message as READ  */
		ui_update_row_tag_in_folder(viewer_info->mail_info->folder_num, viewer_info->msgid,
					    READ_STATUS, FALSE);
	}
	viewer_info->mail_info->body_downloaded = TRUE;

/*opening the editor after retrieving the entire mail*/
	if (viewer_info->mode == FORWARDING) {
		hide_progressbar(viewer_info);
		msgHeader = ui_initialise_headerinfo(viewer_info, TRUE);
		body = ui_viewer_get_body(viewer_info);
		forwardWindow = GTK_WIDGET(ui_editor_create(msgHeader, body, FORWARD));
		if (forwardWindow != NULL) {
			gtk_widget_show(forwardWindow);
			hildon_app_set_appview(app, (HildonAppView *) forwardWindow);
		}
		ui_freeHeaderInfo(msgHeader);
		viewer_info->mode = NOACTION;
		g_free(body);
		return;
	}
/*opening the mail after retrieving the entire mail*/
	toolbar = viewer_info->toolbar;
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR
				  (viewer_info->toolbar->progressbar), _("mail_me_opening"));
	ULOG_DEBUG_F("progress bar text opening \n ");
	email_main_view->tags->current_mode = MSG_OPENING;
	attachmentstatus =
	    ui_viewer_check_attachments(viewer_info->mime_info,
					msgfields->liststore, msgfields->view,
					viewer_info->mail_info->email_attachment);


	if (viewer_info->mail_info->email_attachment == TRUE
	    && viewer_info->mail_info->smime_type != SMIME_NONE
	    && viewer_info->mail_info->smime_type != SMIME_INTERNAL) {
		mimeinfo = viewer_info->mime_info->main;
	} else {
		mimeinfo = viewer_info->mime_info;
	}


	if (attachmentstatus == TRUE) {
		show_attachment_field(msgfields, TRUE);
		viewer_info->msg_fields->attachments_available = TRUE;
	} else if (attachmentstatus == FALSE) {
		gtk_widget_set_sensitive(viewer_info->menu->view, FALSE);
		ULOG_DEBUG_F(":no attachments\n");
	}
	set_viewer_security_icon(msgfields, mimeinfo);
#if SMIME
	/*Checks whether the mail has any security options set */
	if (viewer_info->mime_info->smime_type != SMIME_NONE) {
		/*stoping the progress until usr clicks read mail */
		if (viewer_info->mail_info->timer_value > 0)
			g_source_remove(viewer_info->mail_info->timer_value);

		check_security_status(viewer_info->msgid, mimeinfo, msgfields);

		/*connecting a timeout call to update the progress bar */
		update_progress(viewer_info);
		viewer_info->mail_info->timer_value =
		    g_timeout_add(1000, (GSourceFunc) update_progress, viewer_info);
	}
#endif

/*Checks Whether a html /plain mail & embeds the content in browser or plain textview*/
	ui_viewer_embed_body(viewer_info);
	set_viewer_security_icon(msgfields, mimeinfo);
	ui_set_priority_icon(viewer_info->mail_info->priority,
			     (GtkImage *) msgfields->priority_image);

	hide_header_widgets(viewer_info);

	/*Retaining the persistence of hide/unhide headers across next/prev 
	 * messages*/
	if (viewer_info->mail_info->same_appview == TRUE)
		hide_headers(NULL, NULL, viewer_info);

	/*setting the heigth of the headers */
	set_header_field_heigth(viewer_info);

	/*setting the zoom */
	ui_set_zoom(viewer_info->zoominfo, viewer_info->mail_info->zoomvalue);
	viewer_info->mode = NOACTION;

	return;
}

/**
 This is a helper function to set the priority of the msg in subject
 
 @param date_editor is a hildon_date_editor  
 @param time_editor is a hildon_time_editor  
 @param recvdate is the date in secs 
 
 @returns void.
*/
static void
ui_viewer_set_date_time_for_msg(GtkLabel * date_editor, GtkLabel * time_editor, time_t recvdate)
{
	struct tm *time_ptr = NULL;
	gint retval = 0;
	gchar date[ARRIVAL_TIME_BUF] = { '\0' };	/*for readability date/time varibles are used.
							   one is sufficient */
	gchar time[ARRIVAL_TIME_BUF] = { '\0' };	/*for readability date/time varibles are used.
							   one is sufficient */

	ULOG_DEBUG_F(": starts  ");

	memset(&date, 0, ARRIVAL_TIME_BUF);
	memset(&time, 0, ARRIVAL_TIME_BUF);
	CHECK_PARAM(date_editor == NULL, "Input  param check failed", return);
	CHECK_PARAM(time_editor == NULL, "Input  param check failed", return);
	CHECK_PARAM(recvdate == 0, "Input  param check failed", return);

	time_ptr = localtime((const time_t *) &(recvdate));

	retval = strftime(date, ARRIVAL_TIME_BUF, "%x", time_ptr);
	if (retval == 0)
		return;
	gtk_label_set_text(date_editor, date);

	retval = strftime(time, ARRIVAL_TIME_BUF, "%X", time_ptr);
	if (retval == 0)
		return;
	gtk_label_set_text(time_editor, time);

	ULOG_DEBUG_F(" ends  ");
	return;
}

/**
 This is a helper function to set the priority icon of the msg in subject
 if the message has high or low priority. No icon is set if it is of 
 normal priority
 @param priority is a gint representing the priority of msg
 @param priorityImage is the image to be displayed.
 
 @return void.
*/
void ui_set_priority_icon(gint priority, GtkImage * priorityImage)
{
	ULOG_DEBUG_F(" starts \n ");

	CHECK_PARAM(priorityImage == NULL, "Input  param check failed", return);
	if (priority == HIGH) {
		gtk_image_set_from_icon_name(priorityImage,
					     "qgn_list_messaging_high", GTK_ICON_SIZE_BUTTON);
	} else if (priority == LOW) {
		gtk_image_set_from_icon_name(priorityImage,
					     "qgn_list_messaging_low", GTK_ICON_SIZE_BUTTON);
	}
	/*for normal priority we dont show any icon */
	if (priority == LOW || priority == HIGH) {
		gtk_widget_show((GtkWidget *) priorityImage);
	} else {
		gtk_widget_hide((GtkWidget *) priorityImage);
	}

	ULOG_DEBUG_F(" ends \n ");
	return;
}


/**
 enable the save/view attachment menu  if the focus is on attachment field.
 No memory freeing for viewer_info is done.

 @param view is the pointer to the attachment widget of the viewer.
 @param event is the pointer to the gdkevent structure.
 @param viewer_info is the ViewerInfo pointer of the current message.

 @return void.
*/
static void ui_attachment_focused(GtkWidget * view, GdkEventFocus * event, ViewerInfo * viewer_info)
{
	GSList *selected_names = NULL;
	GtkTreeSelection *attachment_selection = NULL;
	GtkTreeIter iter;
	MsgFields *msgfields = NULL;
	msgfields = viewer_info->msg_fields;

	selected_names = ui_get_selected_attachment_name(viewer_info->msg_fields);
	if (selected_names == NULL) {
		attachment_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgfields->view));
		gtk_tree_model_get_iter_first(gtk_tree_view_get_model
					      ((GtkTreeView *) msgfields->view), &iter);
		gtk_tree_selection_select_iter(attachment_selection, &iter);
		selected_names = ui_get_selected_attachment_name(viewer_info->msg_fields);
	}
	ui_attachment_selection_cb(gtk_tree_view_get_selection((GtkTreeView *) view), viewer_info);
	g_slist_foreach(selected_names, (GFunc) g_free, NULL);
	g_slist_free(selected_names);
	return;
}

/**
 This function is for viewing a attcahment on tapping on the attachemnt.
 
 @param view is the GtkTreeView pointer.
 @param psth is the GtkTreePath pointer for the selected attachment.
 @param iter is the GtkTreeViewColumn pointer.
 @param viewer_info is the pointer to the viewerInfo structure pointer.
 
 @return void.
 
*/
static void
ui_open_attachment_cb(GtkTreeView * view, GtkTreePath * path,
		      GtkTreeViewColumn * iter, ViewerInfo * viewer_info)
{

	/*check if the attachemnt is not already in the process of retrieval */
	if (viewer_info->mail_info->progress_on == FALSE)
		ui_view_attachment_cb((GtkWidget *) view, viewer_info);
	return;
}



/*Function to resend the mail
 
 @param widget is the GTkwidget that registered the acllback.
 @param viewer_info is the ViewerInfo structure pointer.

 @return void.
**/
static void ui_viewer_resend_cb(GtkWidget * widget, ViewerInfo * viewer_info)
{
	gboolean user_confirmation = FALSE;
	GSList *msgid_list = NULL;

/*Check if there are no accounts configured
     And launch new account configuration dialog if required */
	if (email_main_view->account_manager->configured_accounts == NULL) {
		user_confirmation =
		    get_user_confirmation(_("mcen_nc_no_email_acnts_defined"),
					  _("mcen_bd_define"), NULL, CONFIRM_DIALOG);

		/*Return if a new account has not been successfully added */
		if (user_confirmation) {
			email_main_view->account_manager->action_after_acct_creation =
			    VIEWER_RESEND_MSG;
			currentviewer = viewer_info->viewer_appview;
			ui_on_new_e_mail_account1_activate(GTK_WIDGET(app), NULL);
		} else {
			return;
		}
	}
	msgid_list = g_slist_append(msgid_list, g_strdup(viewer_info->msgid));
	ui_dbus_send_selected_mails(OPR_RESEND_MAILS, msgid_list);
	g_slist_foreach(msgid_list, (GFunc) g_free, NULL);
	g_slist_free(msgid_list);
}

/**
 This function is used to get the parent mimeInfo of a
 emailtype attachment since the mimeInfo of this mail starts from the sub part.
 The parent mimeInfo is assinged to the ViewerInfo structure .
 No memory freeing for viewerInfo & mimeInfo is done.
 
 @param viewerInfo is the ViewerInfo structure pointer
 @param mimeInfo is the MiemInfo structure pointer to the current viewer.
 
 @return void. 
*/
static void get_root_mimeinfo(ViewerInfo * viewerInfo, MimeInfo * mimeInfo)
{
	while (mimeInfo != NULL) {
		if (mimeInfo->parent == NULL && mimeInfo->main == NULL) {
			viewerInfo->mime_info = mimeInfo;
			return;
		}
		get_root_mimeinfo(viewerInfo, mimeInfo->main);
		get_root_mimeinfo(viewerInfo, mimeInfo->parent);
		return;
	}
	return;
}

/**
 This function frees all the memory allocted for  ViewerInfo structure.
 
 @param viewerInfo is the ViewerInfo structure pointer
 @return void.
*/
void ui_free_viewerinfo(ViewerInfo * viewerInfo)
{
	GtkWidget *menu = NULL;
	GtkWidget *opened_viewer = NULL;
	GtkClipboard *clipboard;

	CHECK_PARAM(viewerInfo == NULL, "input param failed", return);
	CHECK_PARAM(email_main_view == NULL, "input param failed", return);

	menu = (GtkWidget *) hildon_appview_get_menu((HildonAppView *) viewerInfo->viewer_appview);
	opened_viewer = email_main_view->widgets->opened_viewer;
	/*Store the clipboard contents */
	clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
	if (clipboard) {
		gtk_clipboard_set_can_store(clipboard, NULL, 0);
		gtk_clipboard_store(clipboard);
	}

	if (viewerInfo->mail_info->progress_on)
		ui_dbus_cancel_recv_mail(email_main_view->account_manager->current_sendrec_account,
					 FALSE);


	if (viewerInfo->viewer_appview != NULL && GTK_IS_WIDGET(viewerInfo->viewer_appview)) {
		if (opened_viewer == NULL) {
			gtk_widget_destroy(menu);
			/*This call unregisters the view */
			hildon_app_unregister_view(app, viewerInfo->viewer_appview);
			gtk_widget_destroy(viewerInfo->viewer_appview);

			/*Reinitailising the signalid values once the appview is closed */
			close_signalid = 0;
			increase_zoom_signalid = 0;
			decrease_zoom_signalid = 0;
			state_save_signalid = 0;
			purge_icons_signalid = 0;
			update_mimeinfo_signalid = 0;
			update_msginfo_signalid = 0;
			update_progressbar_signalid = 0;
			resend_signalid = 0;
			cancel_progressbar_signalid = 0;
		} else {
			gtk_container_foreach((GtkContainer *) menu,
					      (GtkCallback) (gtk_widget_destroy), NULL);
			gtk_container_foreach((GtkContainer *)
					      HILDON_APPVIEW(opened_viewer),
					      (GtkCallback) (gtk_widget_destroy), NULL);
			gtk_container_foreach((GtkContainer *)
					      HILDON_APPVIEW(opened_viewer)->vbox,
					      (GtkCallback) (gtk_widget_destroy), NULL);
			/*assings the signal is to the global ids which will be disconnected
			 * for next/prev messages*/
			assing_signalid_to_global_ids(viewerInfo);
		}

	}
	g_free(viewerInfo->menu);
	g_free(viewerInfo->toolbar);
	g_free(viewerInfo->msg_fields);
	g_free(viewerInfo->msgid);
	if (viewerInfo->mime_info != NULL) {
		if (viewerInfo->mail_info->email_attachment == TRUE) {
			get_root_mimeinfo(viewerInfo, viewerInfo->mime_info);
		}
		/*unlinking the body file recieved for mimeInfo */
		if (viewerInfo->mime_info->filename != NULL) {
			gnome_vfs_unlink(viewerInfo->mime_info->filename);
		}
		ui_freeHeaderInfo_mimeinfo(viewerInfo->mime_info);
		viewerInfo->mime_info = NULL;
	}


	/*removing periodical updation of progress bar once it is closed */
	if (viewerInfo->mail_info->timer_value > 0)
		g_source_remove(viewerInfo->mail_info->timer_value);
	viewerInfo->mail_info->timer_value = 0;


	g_free(viewerInfo->mail_info);
	g_free(viewerInfo->signal_id);

	if (viewerInfo->zoominfo != NULL) {
		g_free(viewerInfo->zoominfo->msgid);
		g_free(viewerInfo->zoominfo);
	}
	if (viewerInfo->plain_filename != NULL) {
		gnome_vfs_unlink(viewerInfo->plain_filename);
		g_free(viewerInfo->plain_filename);
	}
	/*The path given by engine ,for attached email */
	if (viewerInfo->email_attachment_body_path != NULL) {
		gnome_vfs_unlink(viewerInfo->email_attachment_body_path);
		g_free(viewerInfo->email_attachment_body_path);
	}
	g_free(viewerInfo->email_attachment_filename);
	g_free(viewerInfo->senderid);
	g_free(viewerInfo->receiverid);
	g_free(viewerInfo->password);
	g_free(viewerInfo->reply_to);
	g_free(viewerInfo->fieldinfo_to);
	g_free(viewerInfo->fieldinfo_cc);
	g_free(viewerInfo->fieldinfo_from);
	g_free(viewerInfo);
	viewerInfo = NULL;
	return;
}

/**
 This function call is used to get the receivers mail id based on the msgid .
 
 @param msgid is the gchar representing the msgid of the mail.
 @return gchar which is the receivers mail id.caller has to free this param
*/
gchar *get_receiversid(const gchar * msgid)
{
	gchar *temp_acc = NULL;
	gchar *current_acc = NULL;
	PrefsAccount *accountSettings = NULL;
	gchar *receiverid = NULL;

	osso_log(LOG_DEBUG, "[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (msgid == NULL) {
		return NULL;
	}
	temp_acc = strchr(msgid, MSGID_SEPARATOR);
	if (temp_acc != NULL) {
		current_acc = g_strndup(msgid, strlen(msgid) - strlen(temp_acc));
		if (current_acc != NULL && g_ascii_strcasecmp(current_acc, DEFAULT) != 0) {
			accountSettings = ui_get_account_detail(current_acc);
			if (accountSettings != NULL) {
				receiverid = g_strdup(accountSettings->user_settings.email);
				ui_free_prefs_account_structure(accountSettings);
			}
		}
		g_free(current_acc);
	}
	osso_log(LOG_DEBUG, "[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return receiverid;
}

/**
 Allocating memory for the ViewerInfo structure.
 
 @return the newly created ViewerInfo pointer
*/
static ViewerInfo *ui_create_viewer_info()
{
	ViewerInfo *viewerInfo = NULL;
	viewerInfo = g_new0(ViewerInfo, 1);
	viewerInfo->menu = g_new0(ViewerMenu, 1);
	viewerInfo->toolbar = g_new0(ViewerToolbar, 1);
	viewerInfo->msg_fields = g_new0(MsgFields, 1);
	viewerInfo->mail_info = g_new0(MailInfo, 1);
	viewerInfo->signal_id = g_new0(SignalId, 1);
	viewerInfo->toolbarhidden_normalview = ui_get_viewer_toolbar_hidden_status();
	viewerInfo->toolbarhidden_fullscreen = ui_get_viewer_fullscreen_toolbar_hidden_status();

	/*various list for handling nonlocal attachemnts */
	viewerInfo->selected_attachment_paths = NULL;
	viewerInfo->selected_attachment_names_save = NULL;
	viewerInfo->mimeinfo_list = NULL;
	viewerInfo->seqno_list = NULL;
	viewerInfo->local_attachments = NULL;

	return viewerInfo;
}

/**
 This is a helper function to set the smime icon for a headers only mail.

 @param mail_info is the const stucture pointer to the MailInfo structure 
 containing the rownum/foldernum/priority/recv_type/smime_type of the mail.
 @return void.
*/
static void ui_set_smime_listview_icon(const MailInfo * mail_info)
{

	GtkTreeModel *model = NULL;
	GtkTreeModel *child_model = NULL;
	GtkTreePath *path = NULL;
	GtkTreeIter iter;
	GtkTreeIter child_iter;
	guint folderCount = 0;
	GdkPixbuf *msgicon = NULL;

	ULOG_DEBUG_L("starts here");

	CHECK_PARAM((email_main_view->sorted_model_list == NULL), "Input param check failed",
		    return);

	if (mail_info->smime_type == SMIME_NONE || mail_info->smime_type == SMIME_INTERNAL)
		return;

	/*Check if such a folder exists at all! */
	folderCount = g_slist_length(email_main_view->sorted_model_list);
	if (mail_info->folder_num > folderCount) {
		ULOG_DEBUG_F("Folder does not exist!\n");
		return;
	}
	msgicon =
	    ui_get_message_display_icon(mail_info->recv_type, mail_info->priority,
					FALSE, mail_info->smime_type);
	model = ui_get_sorted_folder_model(mail_info->folder_num);

	path = gtk_tree_path_new_from_indices(mail_info->row_num, -1);
	if (gtk_tree_model_get_iter(model, &iter, path)) {
		gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT
							       (model), &child_iter, &iter);
		child_model = gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(model));
		gtk_list_store_set(GTK_LIST_STORE(child_model), &child_iter,
				   COL_MSG_ICON, msgicon, COL_SMIME_TYPE, mail_info->smime_type,
				   -1);

	}
	gtk_tree_path_free(path);
	ULOG_DEBUG_L("ends here");
	return;
}

/**
 This function is used to call the ui_dbus_recv_msg_body
 based on the smime type & whether the body downloaded status.
 
 @param viewer_info is the viewerInfo structure pointer.
   No memory freeing for viewer_info  is done.
 @param msgHeader is the MsgHeader structure pointer.
   No memory freeing for  msgHeader is done.
 @return gboolean FALSE-on error
                  TRUE-else.
*/
static gboolean call_recv_msg_body(HeaderInfo * msgHeader, ViewerInfo * viewer_info)
{
	guint certid = 0;
	gint cert_state = VALID;
	gchar *password = NULL;
	SMimeType smime_type = SMIME_NONE;
	smime_type = msgHeader->smime_type;

	if (msgHeader->smime_type == SMIME_INTERNAL) {
		smime_type = SMIME_NONE;
	}
	if (msgHeader->smime_type == SMIME_INTERNAL ||
	    msgHeader->smime_type == SMIME_NONE || msgHeader->smime_type == SMIME_SIGN
	    ||
	    ((msgHeader->smime_type == SMIME_ENCRYPT || msgHeader->smime_type == SMIME_SIGN_ENCRYPT)
	     && msgHeader->body_downloaded == FALSE)) {

		viewer_info->mime_info =
		    ui_dbus_recv_msg_body((msgHeader->msgid), FALSE, FALSE,
					  viewer_info->senderid,
					  viewer_info->receiverid,
					  smime_type, (msgHeader->read_status), certid, NULL, FALSE,
					  FALSE);
	}
	/*getting the password for decryting a encrypted mail */
	else {
		certid = get_private_certificate(viewer_info->receiverid, &cert_state);
		if (certid == 0) {
			get_user_confirmation(_("mcen_in_smime_cannot_decrypt"), NULL, NULL,
					      INFO_BANNER);
			return FALSE;
		}

		password = get_certificate_password(certid);
		if (password == NULL)
			return FALSE;
		viewer_info->password = g_strdup(password);
		viewer_info->certid = certid;
		viewer_info->mime_info =
		    ui_dbus_recv_msg_body((msgHeader->msgid), FALSE, FALSE,
					  viewer_info->senderid,
					  viewer_info->receiverid,
					  msgHeader->smime_type,
					  (msgHeader->read_status), certid, password, FALSE, FALSE);
		g_free(password);
	}

	return TRUE;

}

/**
 Filling the ViewerInfo structure with appropriate msgheader data.
 the data includes the 
 message protocol,receive type,msgid,mimeinfo,fullydownloaded status flag,
 the folder number of the message opened.the row number of the message opened.
 
 @param viewer_info is the viewerInfo structure pointer.
   No memory freeing for viewer_info  is done.
 @param msgHeader is the MsgHeader structure pointer.
   No memory freeing for  msgHeader is done.
 @return gboolean FALSE-if engine returns error.
                 TRUE-else.
*/
static gboolean ui_fill_viewerinfo(ViewerInfo * viewer_info, HeaderInfo * msgHeader)
{
	CHECK_PARAM(msgHeader == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(viewer_info == NULL, "Input param check failed\n", return FALSE);

	if (msgHeader->msgid != NULL) {
		viewer_info->msgid = g_strdup(msgHeader->msgid);
	}
	viewer_info->receiverid = get_receiversid(msgHeader->msgid);
	viewer_info->senderid = ui_parse_from_address(msgHeader->from);
	if (msgHeader->reply_to != NULL)
		viewer_info->reply_to = g_strdup(msgHeader->reply_to);
	/*When the viewer is opened from callback i.e only headers Recvtype.
	   we get the mimeInfo.else we make dbus call to get the mimeinfo.
	 */
	viewer_info->mode = OPENING;
	if (msgHeader->mimeInfo != NULL) {
		viewer_info->mime_info = msgHeader->mimeInfo;
		if (msgHeader->password != NULL)
			viewer_info->password = g_strdup(msgHeader->password);
		viewer_info->certid = msgHeader->certid;
	}

	if (msgHeader->zoomlevel != 0) {
		viewer_info->mail_info->zoomvalue = msgHeader->zoomlevel;
	} else {
		viewer_info->mail_info->zoomvalue = DEFAULT_ZOOM;
	}
	viewer_info->email_attachment_filename = g_strdup(msgHeader->filename);
	viewer_info->email_attachment_body_path = g_strdup(msgHeader->email_filename);
	viewer_info->mail_info->email_attachment = msgHeader->email_attachment;

	viewer_info->mail_info->protocol = msgHeader->protocol;
	viewer_info->mail_info->recv_type = msgHeader->msgType;
	viewer_info->mail_info->fully_downloaded = msgHeader->fully_downloaded;
	viewer_info->mail_info->body_downloaded = msgHeader->body_downloaded;
	viewer_info->mail_info->row_num = msgHeader->rowNum;
	viewer_info->mail_info->folder_num = msgHeader->folderNum;
	viewer_info->mail_info->date = msgHeader->sent_date;
	viewer_info->mail_info->priority = msgHeader->priority;
	viewer_info->mail_info->smime_type = msgHeader->smime_type;
	viewer_info->mail_info->set_prev_settings = msgHeader->set_prev_settings;
	viewer_info->mail_info->headers_hidden = msgHeader->headers_hidden;

	return TRUE;
}

/**
 This function sets the add_to_contacts menu enabled/disabled based on the
 whether the focus is in address fields or not.

 @param wid is the GtkWidget that registered the callback.
 @param viewer is the ViewerInfo staructure pointer.
 @return void.
*/
static void manage_tool_items(GtkWidget * wid, ViewerInfo * viewer)
{
	GtkWidget *view = NULL;
	GtkTextBuffer *buffer = NULL;
	GtkTextIter start, end;
	GtkTextIter match_start, match_end;

	view = gtk_window_get_focus(GTK_WINDOW(app));
	if (GTK_IS_TEXT_VIEW(view) == FALSE)
		return;

	buffer = gtk_text_view_get_buffer((GtkTextView *) view);
	gtk_widget_set_sensitive(viewer->menu->add_to_contacts, FALSE);
	if (gtk_text_buffer_get_selection_bounds(buffer, &start, &end) == TRUE) {
		if (!gtk_text_iter_forward_search(&start, ",", GTK_TEXT_SEARCH_VISIBLE_ONLY,
						  &match_start, &match_end, &end)) {
			gtk_widget_set_sensitive(viewer->menu->add_to_contacts, TRUE);
		} else {
			gtk_widget_set_sensitive(viewer->menu->add_to_contacts, FALSE);
		}
	}
}

/**
 This function sets the fullscreen menu checked/unchecked based on the
 whether the view is in fullscreen.

 @param wid is the GtkWidget that registered the callback.
 @param viewer is the ViewerInfo staructure pointer.
 @return void.
*/
static void ui_viewer_view_menu_activate_cb(GtkWidget * wid, ViewerInfo * viewer_info)
{

	gboolean full_screen_state = TRUE;

	full_screen_state = email_main_view->tags->full_screen_mode;

	g_signal_handlers_block_by_func(G_OBJECT(viewer_info->menu->fullscreen), (gpointer)
					ui_viewer_toggle_to_fullscreen, viewer_info);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				       (viewer_info->menu->fullscreen), full_screen_state);
	g_signal_handlers_unblock_by_func(G_OBJECT(viewer_info->menu->fullscreen), (gpointer)
					  ui_viewer_toggle_to_fullscreen, viewer_info);
	return;
}

/**
 This function sets the copy menu enabled/disabled based on the
 whether the focus is in attachment fields or not.

 @param wid is the GtkWidget that registered the callback.
 @param viewer is the ViewerInfo staructure pointer.
 @return void.
*/
static void ccp_menus_for_viewer(GtkWidget * edit_menu, ViewerInfo * viewer_info)
{
	GtkTextBuffer *textbuffer_from = NULL;
	GtkTextBuffer *textbuffer_to = NULL;
	GtkTextBuffer *textbuffer_cc = NULL;
	GtkTextBuffer *textbuffer_subject = NULL;

	textbuffer_from = gtk_text_view_get_buffer(GTK_TEXT_VIEW(viewer_info->msg_fields->from));
	textbuffer_cc = gtk_text_view_get_buffer(GTK_TEXT_VIEW(viewer_info->msg_fields->cc));
	textbuffer_to = gtk_text_view_get_buffer(GTK_TEXT_VIEW(viewer_info->msg_fields->to));
	textbuffer_subject =
	    gtk_text_view_get_buffer(GTK_TEXT_VIEW(viewer_info->msg_fields->subject));

	/*check if there is any selection */
	if (gtk_text_buffer_get_selection_bounds(textbuffer_subject, NULL, NULL) ||
	    gtk_text_buffer_get_selection_bounds(textbuffer_from, NULL, NULL) ||
	    gtk_text_buffer_get_selection_bounds(textbuffer_to, NULL, NULL) ||
	    gtk_text_buffer_get_selection_bounds(textbuffer_cc, NULL, NULL) ||
	    mail_viewer_has_selection(MAIL_VIEWER(viewer_info->msg_fields->mail_viewer))) {
		/*enable the cut/copy menus. */
		gtk_widget_set_sensitive(viewer_info->menu->copy, TRUE);
	} else {
		gtk_widget_set_sensitive(viewer_info->menu->copy, FALSE);
	}
	return;
}

/**
 Menu creation function
 
 @param viewer_info is the ViewerInfo structure pointer.
   No memory freeing for viewer_info is done.

 @return void.
*/
static void ui_viewer_create_menubar(ViewerInfo * viewer_info)
{
	GtkWidget *menubar = NULL;
	GtkWidget *message_menu = NULL;
	GtkWidget *message_menuitem = NULL;
	GtkWidget *msg_separator = NULL;

	GtkWidget *edit_menuitem = NULL;
	GtkWidget *edit_menu = NULL;

	GtkWidget *attachments_menuitem = NULL;
	GtkWidget *attachments_menu = NULL;

	GtkWidget *view_menuitem = NULL;
	GtkWidget *view_menu = NULL;
	GtkWidget *zoom = NULL;
	GtkWidget *zoom_menu = NULL;

	GtkWidget *show_toolbar_menu = NULL;
	GtkWidget *show_toolbar = NULL;
#if RETAINED_EMAIL
	GtkWidget *security_menu = NULL;
	GtkWidget *security = NULL;
#endif
	GtkWidget *separator = NULL;
	GSList *zoomlevels = NULL;
	GtkWidget *tools_menuitem = NULL;
	GtkWidget *tools_menu = NULL;

	GtkWidget *close_menu = NULL;
	GtkWidget *close = NULL;

	ViewerMenu *menu = NULL;
	gchar zoomstr[BUFFER_SIZE];
	CHECK_PARAM(viewer_info == NULL, "Input  param check failed", return);
	CHECK_PARAM(viewer_info->menu == NULL, "Input  param check failed", return);

	menu = viewer_info->menu;
	menubar =
	    (GtkWidget *) hildon_appview_get_menu((HildonAppView *) viewer_info->viewer_appview);

	message_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_message"));
	gtk_widget_show(message_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), message_menuitem);

	message_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(message_menuitem), message_menu);
	if (viewer_info->mail_info->folder_num == SENT_MODEL_NUM) {
		/*Reply menu */
		menu->resend = gtk_menu_item_new_with_mnemonic(_("mcen_me_sent_resend"));
		gtk_widget_show(menu->resend);
		gtk_container_add(GTK_CONTAINER(message_menu), menu->resend);
		g_signal_connect(G_OBJECT(menu->resend), "activate",
				 G_CALLBACK(ui_viewer_resend_cb), viewer_info);
		g_signal_connect(G_OBJECT(menu->resend), "insensitive_press",
				 G_CALLBACK(show_dimming_note),
				 _("mail_ib_notavailable_downloading"));
	} else {
		/*Reply menu */
		menu->reply = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_reply"));
		gtk_widget_show(menu->reply);
		gtk_container_add(GTK_CONTAINER(message_menu), menu->reply);
		g_signal_connect(G_OBJECT(menu->reply), "activate",
				 G_CALLBACK(ui_viewer_reply_cb), viewer_info);
		g_signal_connect(G_OBJECT(menu->reply), "insensitive_press",
				 G_CALLBACK(show_dimming_note),
				 _("mail_ib_notavailable_downloading"));
		/*Reply all menu */
		menu->reply_all = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_replytoall"));
		gtk_widget_show(menu->reply_all);
		gtk_container_add(GTK_CONTAINER(message_menu), menu->reply_all);
		g_signal_connect(G_OBJECT(menu->reply_all), "activate",
				 G_CALLBACK(ui_viewer_replyAll_cb), viewer_info);
		g_signal_connect(G_OBJECT(menu->reply_all), "insensitive_press",
				 G_CALLBACK(show_dimming_note),
				 _("mail_ib_notavailable_downloading"));
	}
	/*forward menu */
	menu->forward = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_forward"));
	gtk_widget_show(menu->forward);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->forward);
	g_signal_connect(G_OBJECT(menu->forward), "activate",
			 G_CALLBACK(ui_viewer_forward_cb), viewer_info);
	g_signal_connect(G_OBJECT(menu->forward), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mail_ib_notavailable_downloading"));

	msg_separator = gtk_menu_item_new();
	gtk_widget_show(msg_separator);
	gtk_container_add(GTK_CONTAINER(message_menu), msg_separator);
	gtk_widget_set_sensitive(msg_separator, FALSE);

	/*New mail menu */
	menu->newmail = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_newemail"));
	gtk_widget_show(menu->newmail);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->newmail);
	g_signal_connect(G_OBJECT(menu->newmail), "activate",
			 G_CALLBACK(ui_on_new_email_activate), NULL);
	msg_separator = gtk_menu_item_new();
	gtk_widget_show(msg_separator);
	gtk_container_add(GTK_CONTAINER(message_menu), msg_separator);
	gtk_widget_set_sensitive(msg_separator, FALSE);

	/*Delete menu */
	menu->delete_msg = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_delete_message"));
	gtk_widget_show(menu->delete_msg);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->delete_msg);
	g_signal_connect((gpointer) menu->delete_msg, "activate",
			 G_CALLBACK(ui_viewer_delete_message_cb), viewer_info);
#if CHANGED_EMAIL
	msg_separator = gtk_menu_item_new();
	gtk_widget_show(msg_separator);
	gtk_container_add(GTK_CONTAINER(message_menu), msg_separator);
	gtk_widget_set_sensitive(msg_separator, FALSE);

	/*Detail menu. */
	menu->detail_msg = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_details"));
	gtk_widget_show(menu->detail_msg);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->detail_msg);
	g_signal_connect((gpointer) menu->detail_msg, "activate",
			 G_CALLBACK(viewer_message_properties_clicked), viewer_info);
#endif

#if RETAINED_EMAIL
	/*Edit menu */
	edit_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_edit"));
	gtk_widget_show(edit_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), edit_menuitem);
	g_signal_connect(G_OBJECT(edit_menuitem), "activate",
			 G_CALLBACK(ccp_menus_for_viewer), viewer_info);

	edit_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit_menuitem), edit_menu);

	/*cut menu */
	menu->cut = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_cut"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->cut), FALSE);
	gtk_widget_show(menu->cut);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->cut);
	g_signal_connect(G_OBJECT(menu->cut), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_unable_to_cut"));

	/*copy menu */
	menu->copy = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_copy"));
	gtk_widget_show(menu->copy);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->copy);
	g_signal_connect(G_OBJECT(menu->copy), "activate",
			 G_CALLBACK(ui_msg_copy_cb), viewer_info->msg_fields);
	g_signal_connect(G_OBJECT(menu->copy), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_unable_to_copy"));

	/*paste menu */
	menu->paste = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_paste"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->paste), FALSE);
	gtk_widget_show(menu->paste);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->paste);
	g_signal_connect(G_OBJECT(menu->paste), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_unable_to_paste"));

	/*selectall menu */
	menu->selectall = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_selectall"));
	gtk_widget_show(menu->selectall);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->selectall);
	g_signal_connect(G_OBJECT(menu->selectall), "activate",
			 G_CALLBACK(ui_msg_select_all_cb), viewer_info->msg_fields);
	/*attachment menu */
	attachments_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_attachments"));
	gtk_widget_show(attachments_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), attachments_menuitem);

	attachments_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(attachments_menuitem), attachments_menu);
	/*attachment view menu */
	menu->view = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_view"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->view), FALSE);
	gtk_widget_show(menu->view);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->view);
	g_signal_connect((gpointer) menu->view, "activate",
			 G_CALLBACK(ui_view_attachment_cb), viewer_info);
	g_signal_connect(G_OBJECT(menu->view), "insensitive_press",
			 G_CALLBACK(save_view_dimming_note), viewer_info);

	/*attachment save menu */
	menu->save_attachment =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_save_attachment"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->save_attachment), FALSE);
	gtk_widget_show(menu->save_attachment);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->save_attachment);
	g_signal_connect(G_OBJECT(menu->save_attachment), "activate",
			 G_CALLBACK(ui_viewer_save_attachment_cb), viewer_info);
	/*changes for showing infobanners --> user dat param is changed */
	g_signal_connect(G_OBJECT(menu->save_attachment), "insensitive_press",
			 G_CALLBACK(save_view_dimming_note), viewer_info);

	/*attachment purge menu */
	menu->purge_attachment =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_purge_attachments"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->purge_attachment), FALSE);
	gtk_widget_show(menu->purge_attachment);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->purge_attachment);
	g_signal_connect(G_OBJECT(menu->purge_attachment), "activate",
			 G_CALLBACK(ui_viewer_purge_attachment_cb), viewer_info);
	g_signal_connect(G_OBJECT(menu->purge_attachment), "insensitive_press",
			 G_CALLBACK(purge_dimming_note), viewer_info);

	/*view menu */
	view_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_view"));
	gtk_widget_show(view_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), view_menuitem);

	view_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(view_menuitem), view_menu);

	g_signal_connect(G_OBJECT(view_menuitem), "activate",
			 G_CALLBACK(ui_viewer_view_menu_activate_cb), viewer_info);
	/*Zoom menu */
	zoom = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_zoom"));
	gtk_widget_show(zoom);
	gtk_container_add(GTK_CONTAINER(view_menu), zoom);

	zoom_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(zoom), zoom_menu);

	/*50% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_50"));
	menu->zoom_in_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_in_level));
	gtk_widget_show(menu->zoom_in_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_in_level);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*80% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_80"));
	menu->zoom_80 = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_80));
	gtk_widget_show(menu->zoom_80);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_80);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*100% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_100"));
	menu->current_zoom_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->current_zoom_level));
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu->current_zoom_level), TRUE);
	gtk_widget_show(menu->current_zoom_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->current_zoom_level);
	memset(zoomstr, 0, BUFFER_SIZE);


	/*120% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_120"));
	menu->zoom_120 = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_120));
	gtk_widget_show(menu->zoom_120);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_120);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*150% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_150"));
	menu->zoom_out_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_out_level));
	gtk_widget_show(menu->zoom_out_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_out_level);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*200% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_200"));
	menu->zoom_out_max_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_out_max_level));
	gtk_widget_show(menu->zoom_out_max_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_out_max_level);
	memset(zoomstr, 0, BUFFER_SIZE);


	/*show toolbar menu */
	show_toolbar = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_toolbar"));
	gtk_widget_show(show_toolbar);
	gtk_container_add(GTK_CONTAINER(view_menu), show_toolbar);

	show_toolbar_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(show_toolbar), show_toolbar_menu);

	/*normal view menu */
	menu->toolbar_normalview =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_viewer_normalview"));
	gtk_widget_show(menu->toolbar_normalview);
	gtk_container_add(GTK_CONTAINER(show_toolbar_menu), menu->toolbar_normalview);
	g_signal_connect(G_OBJECT(menu->toolbar_normalview), "activate",
			 G_CALLBACK(ui_toggle_viewer_toolbar_cb), viewer_info);

	/*full screen view menu */
	menu->toolbar_fullscreen =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_viewer_optimizedview"));
	gtk_widget_show(menu->toolbar_fullscreen);
	gtk_container_add(GTK_CONTAINER(show_toolbar_menu), menu->toolbar_fullscreen);
	g_signal_connect(G_OBJECT(menu->toolbar_fullscreen), "activate",
			 G_CALLBACK(ui_toggle_viewer_fullscreen_toolbar_cb), viewer_info);

	/*full screen menu */
	menu->fullscreen = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_viewer_fullscreen"));
	gtk_widget_show(menu->fullscreen);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->fullscreen);
	g_signal_connect(G_OBJECT(menu->fullscreen), "activate",
			 G_CALLBACK(ui_viewer_toggle_to_fullscreen), viewer_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(view_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*next msg menu */
	menu->next_msg = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_nextmessage"));
	gtk_widget_show(menu->next_msg);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->next_msg);

	g_signal_connect((gpointer) menu->next_msg, "activate",
			 G_CALLBACK(ui_view_next_message_cb), viewer_info);
	if (!viewer_info->mail_info->email_attachment)
		g_signal_connect(G_OBJECT(menu->next_msg), "insensitive_press",
				 G_CALLBACK(ui_viewer_close_cb), viewer_info);
	else
		g_signal_connect(G_OBJECT(menu->next_msg), "insensitive_press",
				 G_CALLBACK(show_dimming_note), _("mcen_ib_nomore_messages"));

	/*previous msg menu */
	menu->prev_msg = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_previousmessage"));
	gtk_widget_show(menu->prev_msg);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->prev_msg);

	g_signal_connect((gpointer) menu->prev_msg, "activate",
			 G_CALLBACK(ui_view_next_message_cb), viewer_info);
	if (!viewer_info->mail_info->email_attachment)
		g_signal_connect(G_OBJECT(menu->prev_msg), "insensitive_press",
				 G_CALLBACK(ui_viewer_close_cb), viewer_info);
	else
		g_signal_connect(G_OBJECT(menu->prev_msg), "insensitive_press",
				 G_CALLBACK(show_dimming_note), _("mcen_ib_nomore_messages"));

	tools_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_tools"));
	gtk_widget_show(tools_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), tools_menuitem);
	g_signal_connect(G_OBJECT(tools_menuitem), "activate",
			 G_CALLBACK(manage_tool_items), viewer_info);

	tools_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(tools_menuitem), tools_menu);

	/*add to contacts menu */
	menu->add_to_contacts = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_addtocontacts"));
	gtk_widget_show(menu->add_to_contacts);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->add_to_contacts);
	g_signal_connect((gpointer) menu->add_to_contacts, "activate",
			 G_CALLBACK(ui_viewer_add_to_contacts_cb), viewer_info->msg_fields);
	g_signal_connect(G_OBJECT(menu->add_to_contacts), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_select_only_one_recipient"));

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*font settings menu */
	menu->font_settings = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_fontsettings"));
	gtk_widget_show(menu->font_settings);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->font_settings);
	g_signal_connect((gpointer) menu->font_settings, "activate",
			 G_CALLBACK(ui_viewer_fontsettings_cb), viewer_info->msg_fields);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*find menu */
	menu->find = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_viewer_find"));
	gtk_widget_show(menu->find);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->find);
	g_signal_connect(G_OBJECT(menu->find), "button_press_event",
			 G_CALLBACK(hide_headers), viewer_info);
	g_signal_connect(G_OBJECT(menu->find), "activate",
			 G_CALLBACK(show_find_toolbar_cb), viewer_info->msg_fields);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);
	/*search emails menu */
	menu->searchemails = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_searchemails"));
	gtk_widget_show(menu->searchemails);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->searchemails);
	g_signal_connect((gpointer) menu->searchemails, "activate",
			 G_CALLBACK(ui_viewer_searchemails_cb), NULL);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*help menu */
	menu->help = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_help"));
	gtk_widget_show(menu->help);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->help);
	g_signal_connect(menu->help, "activate", G_CALLBACK(ui_on_view_help_activate),
			 HELP_ID_VIEWER);

	/*security menu */
	security = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_security"));
	gtk_widget_show(security);
	gtk_container_add(GTK_CONTAINER(menubar), security);

	security_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(security), security_menu);

	/*security properties menu */
	menu->properties = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_sec_properties"));
	if (viewer_info->mail_info->smime_type == SMIME_NONE
	    || viewer_info->mail_info->smime_type == SMIME_INTERNAL) {
		gtk_widget_set_sensitive(menu->properties, FALSE);
	}
	gtk_widget_show(menu->properties);
	gtk_container_add(GTK_CONTAINER(security_menu), menu->properties);
	g_signal_connect((gpointer) menu->properties, "activate",
			 G_CALLBACK(ui_viewer_launch_cm_cb), viewer_info);
	g_signal_connect(G_OBJECT(menu->properties), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mail_ib_no_security_properties"));

#else
	/*Edit menu */
	edit_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_edit"));
	gtk_widget_show(edit_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), edit_menuitem);
	g_signal_connect(G_OBJECT(edit_menuitem), "activate",
			 G_CALLBACK(ccp_menus_for_viewer), viewer_info);

	edit_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit_menuitem), edit_menu);

	/*cut menu */
	menu->cut = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_cut"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->cut), FALSE);
	gtk_widget_show(menu->cut);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->cut);
	g_signal_connect(G_OBJECT(menu->cut), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_unable_to_cut"));

	/*copy menu */
	menu->copy = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_copy"));
	gtk_widget_show(menu->copy);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->copy);
	g_signal_connect(G_OBJECT(menu->copy), "activate",
			 G_CALLBACK(ui_msg_copy_cb), viewer_info->msg_fields);
	g_signal_connect(G_OBJECT(menu->copy), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_unable_to_copy"));

	/*paste menu */
	menu->paste = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_paste"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->paste), FALSE);
	gtk_widget_show(menu->paste);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->paste);
	g_signal_connect(G_OBJECT(menu->paste), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_unable_to_paste"));

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(edit_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*selectall menu */
	menu->selectall = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_selectall"));
	gtk_widget_show(menu->selectall);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->selectall);
	g_signal_connect(G_OBJECT(menu->selectall), "activate",
			 G_CALLBACK(ui_msg_select_all_cb), viewer_info->msg_fields);

	/*view menu */
	view_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_view"));
	gtk_widget_show(view_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), view_menuitem);

	view_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(view_menuitem), view_menu);

	g_signal_connect(G_OBJECT(view_menuitem), "activate",
			 G_CALLBACK(ui_viewer_view_menu_activate_cb), viewer_info);
	/*Zoom menu */
	zoom = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_zoom"));
	gtk_widget_show(zoom);
	gtk_container_add(GTK_CONTAINER(view_menu), zoom);

	zoom_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(zoom), zoom_menu);

	/*50% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_50"));
	menu->zoom_in_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_in_level));
	gtk_widget_show(menu->zoom_in_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_in_level);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*80% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_80"));
	menu->zoom_80 = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_80));
	gtk_widget_show(menu->zoom_80);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_80);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*100% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_100"));
	menu->current_zoom_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->current_zoom_level));
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu->current_zoom_level), TRUE);
	gtk_widget_show(menu->current_zoom_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->current_zoom_level);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*120% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_120"));
	menu->zoom_120 = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_120));
	gtk_widget_show(menu->zoom_120);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_120);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*150% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_150"));
	menu->zoom_out_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_out_level));
	gtk_widget_show(menu->zoom_out_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_out_level);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*200% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_200"));
	menu->zoom_out_max_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_out_max_level));
	gtk_widget_show(menu->zoom_out_max_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_out_max_level);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(view_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*full screen menu */
	menu->fullscreen = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_viewer_fullscreen"));
	gtk_widget_show(menu->fullscreen);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->fullscreen);
	g_signal_connect(G_OBJECT(menu->fullscreen), "activate",
			 G_CALLBACK(ui_viewer_toggle_to_fullscreen), viewer_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(view_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*show toolbar menu */
	show_toolbar = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_toolbar"));
	gtk_widget_show(show_toolbar);
	gtk_container_add(GTK_CONTAINER(view_menu), show_toolbar);

	show_toolbar_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(show_toolbar), show_toolbar_menu);

	/*normal view menu */
	menu->toolbar_normalview =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_viewer_normalview"));
	gtk_widget_show(menu->toolbar_normalview);
	gtk_container_add(GTK_CONTAINER(show_toolbar_menu), menu->toolbar_normalview);
	g_signal_connect(G_OBJECT(menu->toolbar_normalview), "activate",
			 G_CALLBACK(ui_toggle_viewer_toolbar_cb), viewer_info);

	/*full screen view menu */
	menu->toolbar_fullscreen =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_viewer_optimizedview"));
	gtk_widget_show(menu->toolbar_fullscreen);
	gtk_container_add(GTK_CONTAINER(show_toolbar_menu), menu->toolbar_fullscreen);
	g_signal_connect(G_OBJECT(menu->toolbar_fullscreen), "activate",
			 G_CALLBACK(ui_toggle_viewer_fullscreen_toolbar_cb), viewer_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(view_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*next msg menu */
	menu->next_msg = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_nextmessage"));
	gtk_widget_show(menu->next_msg);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->next_msg);

	g_signal_connect((gpointer) menu->next_msg, "activate",
			 G_CALLBACK(ui_view_next_message_cb), viewer_info);
	g_signal_connect(G_OBJECT(menu->next_msg), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_nomore_messages"));

	/*previous msg menu */
	menu->prev_msg = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_previousmessage"));
	gtk_widget_show(menu->prev_msg);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->prev_msg);

	g_signal_connect((gpointer) menu->prev_msg, "activate",
			 G_CALLBACK(ui_view_next_message_cb), viewer_info);
	g_signal_connect(G_OBJECT(menu->prev_msg), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_nomore_messages"));

	/*attachment menu */
	attachments_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_attachments"));
	gtk_widget_show(attachments_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), attachments_menuitem);

	attachments_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(attachments_menuitem), attachments_menu);
	/*attachment view menu */
	menu->view = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_view"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->view), FALSE);
	gtk_widget_show(menu->view);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->view);
	g_signal_connect((gpointer) menu->view, "activate",
			 G_CALLBACK(ui_view_attachment_cb), viewer_info);
	g_signal_connect(G_OBJECT(menu->view), "insensitive_press",
			 G_CALLBACK(save_view_dimming_note), viewer_info);

	/*attachment save menu */
	menu->save_attachment =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_save_attachment"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->save_attachment), FALSE);
	gtk_widget_show(menu->save_attachment);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->save_attachment);
	g_signal_connect(G_OBJECT(menu->save_attachment), "activate",
			 G_CALLBACK(ui_viewer_save_attachment_cb), viewer_info);
	/*changes for showing infobanners --> user dat param is changed */
	g_signal_connect(G_OBJECT(menu->save_attachment), "insensitive_press",
			 G_CALLBACK(save_view_dimming_note), viewer_info);

	/*attachment purge menu */
	menu->purge_attachment =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_purge_attachments"));
	gtk_widget_set_sensitive(GTK_WIDGET(menu->purge_attachment), FALSE);
	gtk_widget_show(menu->purge_attachment);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->purge_attachment);
	g_signal_connect(G_OBJECT(menu->purge_attachment), "activate",
			 G_CALLBACK(ui_viewer_purge_attachment_cb), viewer_info);
	g_signal_connect(G_OBJECT(menu->purge_attachment), "insensitive_press",
			 G_CALLBACK(purge_dimming_note), viewer_info);
	/*tools menu items */
	tools_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_tools"));
	gtk_widget_show(tools_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), tools_menuitem);
	g_signal_connect(G_OBJECT(tools_menuitem), "activate",
			 G_CALLBACK(manage_tool_items), viewer_info);

	tools_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(tools_menuitem), tools_menu);

	/*add to contacts menu */
	menu->add_to_contacts = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_addtocontacts"));
	gtk_widget_show(menu->add_to_contacts);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->add_to_contacts);
	g_signal_connect((gpointer) menu->add_to_contacts, "activate",
			 G_CALLBACK(ui_viewer_add_to_contacts_cb), viewer_info->msg_fields);
	g_signal_connect(G_OBJECT(menu->add_to_contacts), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_select_only_one_recipient"));

	/*security properties menu */
	menu->properties = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_sec_properties"));
	if (viewer_info->mail_info->smime_type == SMIME_NONE
	    || viewer_info->mail_info->smime_type == SMIME_INTERNAL) {
		gtk_widget_set_sensitive(menu->properties, FALSE);
	}
	gtk_widget_show(menu->properties);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->properties);
	g_signal_connect((gpointer) menu->properties, "activate",
			 G_CALLBACK(ui_viewer_launch_cm_cb), viewer_info);
	g_signal_connect(G_OBJECT(menu->properties), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mail_ib_no_security_properties"));

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*find menu */
	menu->find = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_viewer_find"));
	gtk_widget_show(menu->find);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->find);
	g_signal_connect(G_OBJECT(menu->find), "button_press_event",
			 G_CALLBACK(hide_headers), viewer_info);
	g_signal_connect(G_OBJECT(menu->find), "activate",
			 G_CALLBACK(show_find_toolbar_cb), viewer_info->msg_fields);

	/*search emails menu */
	menu->searchemails = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_searchemails"));
	gtk_widget_show(menu->searchemails);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->searchemails);
	g_signal_connect((gpointer) menu->searchemails, "activate",
			 G_CALLBACK(ui_viewer_searchemails_cb), NULL);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*help menu */
	menu->help = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_help"));
	gtk_widget_show(menu->help);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->help);
	g_signal_connect(menu->help, "activate", G_CALLBACK(ui_on_view_help_activate),
			 HELP_ID_VIEWER);
#endif
	/*Close menu item */
	close = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_close"));
	gtk_widget_show(close);
	gtk_container_add(GTK_CONTAINER(menubar), close);

	close_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(close), close_menu);

	/*Close menu */
	menu->close_window = gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_close_window"));
	gtk_widget_show(menu->close_window);
	gtk_container_add(GTK_CONTAINER(close_menu), menu->close_window);

	g_signal_connect(G_OBJECT(menu->close_window), "activate",
			 G_CALLBACK(ui_viewer_close_cb), viewer_info);
	/*Close all menu */
	menu->close_all_windows =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_viewer_close_windows"));
	gtk_widget_show(menu->close_all_windows);
	gtk_container_add(GTK_CONTAINER(close_menu), menu->close_all_windows);
	g_signal_connect(G_OBJECT(menu->close_all_windows), "activate",
			 G_CALLBACK(ui_on_close_all_windows_activate), NULL);

	gtk_widget_show_all(menubar);
	return;
}

/**
 This function is used to show/hide the to fields.
 
 @param fields is the MsgFields structure pointer of the current editor.
 @param show is the gboolean representing the on/off status of cc
 
 @return void.
*/
static void show_to_field(const MsgFields * fields, gboolean show)
{
	if (show == TRUE) {
		gtk_widget_show(fields->to_lable);
		gtk_widget_show(fields->to_hbox);
	} else {
		gtk_widget_hide(fields->to_lable);
		gtk_widget_hide(fields->to_hbox);
	}
	return;
}

/**
 This function is used to show/hide the subject fields.
 
 @param fields is the MsgFields structure pointer of the current editor.
 @param show is the gboolean representing the on/off status of cc
 
 @return void.
*/
static void show_subject_field(const MsgFields * fields, gboolean show)
{
	if (show == TRUE) {
		if (fields->priority_image != NULL) {
			gtk_widget_show(fields->priority_image);
		}
		gtk_widget_show(fields->subject_lable);
		gtk_widget_show(fields->subject_hbox);
		gtk_widget_show(fields->subject_lable_hbox);
	} else {
		if (fields->priority_image != NULL) {
			gtk_widget_hide(fields->priority_image);
		}
		gtk_widget_hide(fields->subject_lable);
		gtk_widget_hide(fields->subject_hbox);
		gtk_widget_hide(fields->subject_lable_hbox);
	}
	return;
}

/*This function is used to get the default theme images path
 *@return void
 */
static gchar *get_theme_image_path()
{
	char *image_name = NULL;
	gchar **paths = NULL;
	gchar **str = NULL;
	gchar **str1 = NULL;
	gchar *contents = NULL;
	GError *error = NULL;

	paths = gtk_rc_get_default_files();	/*paths should not be freed */
	if (paths != NULL) {
		g_file_get_contents(*paths, &contents, NULL, &error);
		if (contents == NULL) {
			g_error_free(error);
			return g_strdup(HIDE_HEADERS_ICON_PATH);
		}
		if (contents != NULL) {
			str = g_strsplit(contents, "\"", -1);
			if (str != NULL && str[1] != NULL) {
				str1 = g_strsplit(str[1], "gtk-2.0/", -1);
				ULOG_DEBUG_F("the theme  path is %s\n", str1[0]);
				image_name = g_strconcat(str1[0], THEME_IMAGES, NULL);
			}
		}
	}
	g_free(contents);
	g_strfreev(str);
	g_strfreev(str1);
	return image_name;
}

/**
 This function is used to hide /unhide the headers of the viewer.

 @param hide_button  is the GtkWidget that registered the acllback.
 @param viewer_info is teh poinetr to the ViewerInfo structure pointer.
 @return void.
*/
static void
change_hide_headers_icon(GtkWidget * hide_button, GdkEvent * event, ViewerInfo * viewer_info)
{
	GtkIconInfo *icon_info = NULL;
	gchar *image_path = NULL;

	image_path = get_theme_image_path();
	gtk_icon_theme_append_search_path(email_main_view->icons->icon_theme, image_path);
	if (hide_button != NULL) {
		if (viewer_info->mail_info->headers_hidden == TRUE)
			icon_info =
			    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
						       "qgn_plat_header_show_pressed",
						       HILDON_ICON_SIZE_26, GTK_ICON_LOOKUP_NO_SVG);
		else
			icon_info =
			    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
						       "qgn_plat_header_hide_pressed",
						       HILDON_ICON_SIZE_26, GTK_ICON_LOOKUP_NO_SVG);
	} else {
		/*Reverting back the image pressed state */
		if (viewer_info->mail_info->headers_hidden == TRUE)
			icon_info =
			    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
						       "qgn_plat_header_show",
						       HILDON_ICON_SIZE_26, GTK_ICON_LOOKUP_NO_SVG);
		else
			icon_info =
			    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
						       "qgn_plat_header_hide",
						       HILDON_ICON_SIZE_26, GTK_ICON_LOOKUP_NO_SVG);
	}
	if (icon_info) {
		gtk_image_set_from_file((GtkImage *) viewer_info->msg_fields->
					hide_headers_image, gtk_icon_info_get_filename(icon_info));
		gtk_icon_info_free(icon_info);
	}
}

/**
 This function is used to hide /unhide the headers of the viewer.

 @param hide_button  is the GtkWidget that registered the acllback.
 @param viewer_info is teh poinetr to the ViewerInfo structure pointer.
 @return void.
*/
static void hide_headers(GtkWidget * hide_button, GdkEvent * event, ViewerInfo * viewer_info)
{
	MimeInfo *mimeinfo = NULL;
	MsgFields *msgfields = NULL;
	msgfields = viewer_info->msg_fields;
	GtkIconInfo *icon_info = NULL;
	gchar *image_path = NULL;

	/*If this is initiated during find then always hide headers if
	 * not already hidden*/
	if (hide_button == viewer_info->menu->find) {
		if (viewer_info->mail_info->headers_hidden == TRUE)
			return;
	}
	/*button_release happened out side the widget just
	 * revert back the icon without hide/unhide the header*/
	else if (hide_unhide_in_focus == FALSE && event != NULL) {
		change_hide_headers_icon(NULL, NULL, viewer_info);
		return;
	}
	image_path = get_theme_image_path();

	/*getting the proper mimeinfo for the message */
	if (viewer_info->mail_info->email_attachment == TRUE
	    && viewer_info->mail_info->smime_type != SMIME_NONE
	    && viewer_info->mail_info->smime_type != SMIME_INTERNAL)
		mimeinfo = viewer_info->mime_info->main;
	else
		mimeinfo = viewer_info->mime_info;

	if (viewer_info->mail_info->headers_hidden == TRUE) {
		/*show the headers */
		if (viewer_info->msg_fields->attachments_available == TRUE) {
			show_attachment_field(msgfields, TRUE);
		}
		if (msgfields->cc_available == TRUE) {
			show_cc_field(msgfields, TRUE);
		}
		show_to_field(msgfields, TRUE);

		show_subject_field(msgfields, TRUE);
		gtk_widget_show(msgfields->hbox_dateTime);
		gtk_widget_show(msgfields->date_lable);
		show_subject_field(msgfields, TRUE);

		if (mimeinfo != NULL)
			set_viewer_security_icon(viewer_info->msg_fields, mimeinfo);

		ui_set_priority_icon(viewer_info->mail_info->priority,
				     (GtkImage *) msgfields->priority_image);
		/*hide/unhide icon */
		gtk_icon_theme_append_search_path(email_main_view->icons->icon_theme, image_path);
		icon_info =
		    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
					       "qgn_plat_header_hide", HILDON_ICON_SIZE_26,
					       GTK_ICON_LOOKUP_NO_SVG);
		if (icon_info) {
			gtk_image_set_from_file((GtkImage *) viewer_info->msg_fields->
						hide_headers_image,
						gtk_icon_info_get_filename(icon_info));
			gtk_icon_info_free(icon_info);
		}
		gtk_scrolled_window_set_policy((GtkScrolledWindow *) viewer_info->
					       msg_fields->scrollwindow_body,
					       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	} else {
		/*hide the headers if they are alredy visible */
		if (viewer_info->mail_info->folder_num != SENT_MODEL_NUM) {
			show_to_field(msgfields, FALSE);
			/*Security image in to field */
			gtk_widget_hide(msgfields->bccencrypt_image);
		}
		show_cc_field(msgfields, FALSE);
		show_subject_field(msgfields, FALSE);
		show_attachment_field(msgfields, FALSE);

		/*showing date/time field */
		gtk_widget_hide(msgfields->hbox_dateTime);
		gtk_widget_hide(msgfields->date_lable);

		/*Security image in cc field */
		if (msgfields->ccencrypt_image)
			gtk_widget_hide(msgfields->ccencrypt_image);
		/*Security image in subject field */
		if (msgfields->encrypt_image != NULL)
			gtk_widget_hide(msgfields->encrypt_image);

		/*hide/unhide icon */
		gtk_icon_theme_append_search_path(email_main_view->icons->icon_theme, image_path);
		icon_info =
		    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
					       "qgn_plat_header_show", HILDON_ICON_SIZE_26,
					       GTK_ICON_LOOKUP_NO_SVG);
		if (icon_info) {
			gtk_image_set_from_file((GtkImage *) viewer_info->msg_fields->
						hide_headers_image,
						gtk_icon_info_get_filename(icon_info));
			gtk_icon_info_free(icon_info);
		}
		gtk_scrolled_window_set_policy((GtkScrolledWindow *) viewer_info->
					       msg_fields->scrollwindow_body,
					       GTK_POLICY_NEVER, GTK_POLICY_NEVER);


	}
	g_free(image_path);
	set_header_field_heigth(viewer_info);
	viewer_info->mail_info->headers_hidden = !viewer_info->mail_info->headers_hidden;
	return;
}

/**
This function updates the focus_in flag when ever the focus is in
the hide headers button area or when it leaves that area.
@param hide_headers is the GdkEventBox that registered the
 enter_notify/leave_notify events.
@param event is the GdkEventCrossing structure pointer.
@return TRUE -->to stop the propogation of event.
        FALSE --> else.
*/
static gboolean update_focus_in_flag(GtkWidget * hide_headers, GdkEventCrossing * event)
{
	if (event == NULL)
		return FALSE;

	if (event->type == GDK_ENTER_NOTIFY)
		hide_unhide_in_focus = TRUE;
	else
		hide_unhide_in_focus = FALSE;
	return FALSE;
}

/**
 This function is used to create the address fields of viewer.
 the same function is used to create the from/to/cc fields.so the address pointers
 are passed.
 
 @param address_field is the GtkWidget for the address(from/to/cc)field.
 @param table is the container widget for the address field.
 @param field is a emun specifing the diff address fields.
 @param viewer_info is the ViewerInfo structure pointer.
   No memory freeing for viewerInfo is done.

 @return void.
*/
static void
ui_viewer_create_address_field(GtkWidget ** address_field,
			       GtkWidget * table, gint field,
			       ViewerInfo * viewerInfo, const gchar * image_path)
{
	GtkWidget *scrolledwindow = NULL;
	GtkWidget *address_hbox = NULL;
	GtkWidget *image_event_box = NULL;
	GtkTextBuffer *address_buffer = NULL;
	AddressFieldInfo *fieldinfo = NULL;
	GtkWidget *alignment = NULL;
	GtkWidget *label = NULL;
	GtkIconInfo *icon_info = NULL;
	address_hbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(address_hbox);

	image_event_box = gtk_event_box_new();
	gtk_widget_show(image_event_box);
	*address_field = gtk_text_view_new();
	gtk_widget_show(*address_field);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(*address_field), FALSE);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(*address_field), FALSE);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(*address_field), GTK_WRAP_WORD_CHAR);
	gtk_text_view_set_accepts_tab(GTK_TEXT_VIEW(*address_field), FALSE);

	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
				       GTK_POLICY_NEVER, GTK_POLICY_NEVER);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), *address_field);
	gtk_box_pack_start((GtkBox *) address_hbox, scrolledwindow, TRUE, TRUE, 0);
	gtk_box_pack_start((GtkBox *) address_hbox, image_event_box, FALSE, FALSE, 0);
	address_buffer = gtk_text_view_get_buffer((GtkTextView *) * address_field);
	if (field == TO) {
		viewerInfo->fieldinfo_to = g_new0(AddressFieldInfo, 1);
		fieldinfo = viewerInfo->fieldinfo_to;
	} else if (field == CC) {
		viewerInfo->fieldinfo_cc = g_new0(AddressFieldInfo, 1);
		fieldinfo = viewerInfo->fieldinfo_cc;
	} else if (field == FROM) {
		viewerInfo->fieldinfo_from = g_new0(AddressFieldInfo, 1);
		fieldinfo = viewerInfo->fieldinfo_from;
	}

	fieldinfo->field = *address_field;
	fieldinfo->field_sw = scrolledwindow;
	fieldinfo->field_hbox = address_hbox;
	fieldinfo->from_viewer = TRUE;

	if (field == FROM) {
		viewerInfo->msg_fields->from_hbox = address_hbox;
		/*sign image */
		viewerInfo->msg_fields->sign_image =
		    gtk_image_new_from_icon_name("qgn_viewer_messagin_signed",
						 GTK_ICON_SIZE_BUTTON);
		gtk_container_add(GTK_CONTAINER(image_event_box),
				  viewerInfo->msg_fields->sign_image);
		if (viewerInfo->mail_info->folder_num != SENT_MODEL_NUM) {
			alignment = gtk_alignment_new(0, 0, 0, 0);
			/*hide headers image */
			gtk_icon_theme_append_search_path(email_main_view->icons->icon_theme,
							  image_path);
			viewerInfo->msg_fields->hide_headers_button = gtk_event_box_new();
			icon_info =
			    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
						       "qgn_plat_header_hide",
						       GTK_ICON_SIZE_BUTTON,
						       GTK_ICON_LOOKUP_NO_SVG);
			if (icon_info) {
				viewerInfo->msg_fields->hide_headers_image =
				    gtk_image_new_from_file(gtk_icon_info_get_filename(icon_info));
				gtk_icon_info_free(icon_info);
				gtk_container_add((GtkContainer *) viewerInfo->msg_fields->
						  hide_headers_button,
						  viewerInfo->msg_fields->hide_headers_image);

				gtk_widget_show(viewerInfo->msg_fields->hide_headers_image);
			}
			gtk_container_add((GtkContainer *) alignment,
					  viewerInfo->msg_fields->hide_headers_button);
			gtk_box_pack_start((GtkBox *) address_hbox, alignment, FALSE, FALSE, 0);
			gtk_widget_show(alignment);
			gtk_widget_show_all(viewerInfo->msg_fields->hide_headers_button);
			gtk_widget_set_size_request(viewerInfo->msg_fields->hide_headers_button, 30,
						    30);
		}
		/*header label */
		viewerInfo->msg_fields->from_lable = gtk_label_new(_("mail_va_from"));
		gtk_widget_show(viewerInfo->msg_fields->from_lable);
		label = viewerInfo->msg_fields->from_lable;
		gtk_misc_set_alignment(GTK_MISC(label), X_RIGHT, Y_BOTTOM);
		gtk_table_attach((GtkTable *) table, label, 0, 1, 0, 1,
				 (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0,
				 0);
		gtk_table_attach_defaults((GtkTable *) table, address_hbox, 1, 4, 0, 1);
		gtk_widget_hide(viewerInfo->msg_fields->sign_image);
		g_signal_connect_after(G_OBJECT(image_event_box),
				       "button_press_event",
				       G_CALLBACK(ui_viewer_signer_cert_properties),
				       (gpointer) viewerInfo);
	} else if (field == TO) {
		/*sign/encrypted image */
		viewerInfo->msg_fields->bccencrypt_image =
		    gtk_image_new_from_icon_name("qgn_viewer_messagin_encrypted",
						 GTK_ICON_SIZE_BUTTON);
		gtk_container_add(GTK_CONTAINER(image_event_box),
				  viewerInfo->msg_fields->bccencrypt_image);

		if (viewerInfo->mail_info->folder_num == SENT_MODEL_NUM) {
			gtk_icon_theme_append_search_path(email_main_view->icons->icon_theme,
							  image_path);
			/*hide headers image */
			viewerInfo->msg_fields->hide_headers_button = gtk_event_box_new();
			alignment = gtk_alignment_new(0, 0, 0, 0);
			icon_info =
			    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
						       "qgn_plat_header_hide",
						       HILDON_ICON_SIZE_26, GTK_ICON_LOOKUP_NO_SVG);
			if (icon_info) {
				viewerInfo->msg_fields->hide_headers_image =
				    gtk_image_new_from_file(gtk_icon_info_get_filename(icon_info));
				gtk_icon_info_free(icon_info);
				gtk_container_add((GtkContainer *) viewerInfo->msg_fields->
						  hide_headers_button,
						  viewerInfo->msg_fields->hide_headers_image);
				gtk_widget_show(viewerInfo->msg_fields->hide_headers_image);
			}
			gtk_container_add((GtkContainer *) alignment,
					  viewerInfo->msg_fields->hide_headers_button);
			gtk_box_pack_start((GtkBox *) address_hbox, alignment, FALSE, FALSE, 0);
			gtk_widget_show(alignment);
			gtk_widget_show_all(viewerInfo->msg_fields->hide_headers_button);
			gtk_widget_set_size_request(viewerInfo->msg_fields->hide_headers_button, 30,
						    30);
		}
		viewerInfo->msg_fields->to_hbox = address_hbox;

		/*header label */
		viewerInfo->msg_fields->to_lable = gtk_label_new(_("mail_va_to"));
		gtk_misc_set_alignment(GTK_MISC(viewerInfo->msg_fields->to_lable), X_RIGHT,
				       Y_BOTTOM);
		gtk_table_attach((GtkTable *) table, viewerInfo->msg_fields->to_lable, 0, 1, 1, 2,
				 (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0,
				 0);
		gtk_table_attach_defaults((GtkTable *) table, address_hbox, 1, 4, 1, 2);
		gtk_widget_hide(viewerInfo->msg_fields->bccencrypt_image);
		if (viewerInfo->mail_info->folder_num != SENT_MODEL_NUM) {
			g_signal_connect_after(G_OBJECT(image_event_box),
					       "button_press_event",
					       G_CALLBACK
					       (ui_viewer_recip_cert_properties),
					       (gpointer) viewerInfo);
		} else {
			g_signal_connect_after(G_OBJECT(image_event_box),
					       "button_press_event",
					       G_CALLBACK
					       (ui_viewer_signer_cert_properties),
					       (gpointer) viewerInfo);
		}
	} else if (field == CC) {
		viewerInfo->msg_fields->cc_hbox = address_hbox;
		viewerInfo->msg_fields->ccencrypt_image =
		    gtk_image_new_from_icon_name("qgn_viewer_messagin_encrypted",
						 GTK_ICON_SIZE_BUTTON);
		gtk_container_add(GTK_CONTAINER(image_event_box),
				  viewerInfo->msg_fields->ccencrypt_image);
		viewerInfo->msg_fields->cc_lable = gtk_label_new(_("mail_va_cc"));
		gtk_misc_set_alignment(GTK_MISC(viewerInfo->msg_fields->cc_lable), X_RIGHT,
				       Y_BOTTOM);
		gtk_table_attach((GtkTable *) table, viewerInfo->msg_fields->cc_lable, 0, 1, 2, 3,
				 (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0,
				 0);
		gtk_table_attach_defaults((GtkTable *) table, address_hbox, 1, 4, 2, 3);
		show_cc_field(viewerInfo->msg_fields, FALSE);
		gtk_widget_hide(viewerInfo->msg_fields->ccencrypt_image);
		gtk_widget_hide(viewerInfo->msg_fields->cc_hbox);
		g_signal_connect_after(G_OBJECT(image_event_box),
				       "button_press_event",
				       G_CALLBACK(ui_viewer_recip_cert_properties),
				       (gpointer) viewerInfo);
	}
	gtk_widget_set_size_request(address_hbox, BODY_WIDTH, ADDR_FIELDS_SIZE);

	g_signal_connect_after(G_OBJECT(address_buffer), "insert_text",
			       G_CALLBACK(resize_addr_field), (gpointer) fieldinfo);
	/*signal connectors for the hide/unhide button */
	if (field == FROM || (field == TO && viewerInfo->mail_info->folder_num == SENT_MODEL_NUM)) {
		g_signal_connect(G_OBJECT
				 (viewerInfo->msg_fields->hide_headers_button),
				 "enter-notify-event", G_CALLBACK(update_focus_in_flag),
				 viewerInfo);
		g_signal_connect(G_OBJECT(viewerInfo->msg_fields->hide_headers_button),
				 "leave-notify-event", G_CALLBACK(update_focus_in_flag),
				 viewerInfo);
		g_signal_connect(G_OBJECT(viewerInfo->msg_fields->hide_headers_button),
				 "button_press_event", G_CALLBACK(change_hide_headers_icon),
				 viewerInfo);

		g_signal_connect(G_OBJECT
				 (viewerInfo->msg_fields->hide_headers_button),
				 "button_release_event", G_CALLBACK(hide_headers), viewerInfo);
	}
	return;
}

/**
 This fuction returns the clicked hyperlink text.
 
 @param buffer is the GtktextBuffer of the subject field.
 @param iter is the GtkTextIter wher it is clicked.
 @return the hyperlink text clicked.
*/
static gchar *get_link_text(GtkTextBuffer * buffer, GtkTextIter * iter)
{
	GtkTextIter start, end, bk_start, bk_end;
	gboolean found_start = TRUE;
	gboolean found_end = TRUE;

	found_end =
	    gtk_text_iter_forward_search(iter, SPACE, GTK_TEXT_SEARCH_VISIBLE_ONLY, &start, &end,
					 NULL);
	found_start =
	    gtk_text_iter_backward_search(iter, SPACE, GTK_TEXT_SEARCH_VISIBLE_ONLY, &bk_start,
					  &bk_end, NULL);
	if (found_start == FALSE) {
		gtk_text_buffer_get_start_iter(buffer, &bk_start);
	}
	if (found_end == FALSE) {
		gtk_text_buffer_get_end_iter(buffer, &end);
	}

	return (gtk_text_buffer_get_text(buffer, &bk_start, &end, FALSE));
}

/**
 This function is used to launch the editor in send as mode
 for the emailid clicked.

 @param uri is the gchat pointer to the emailid.
 @return void.
*/
static void launch_editor(const gchar * uri)
{
	MsgInfo *msgInfo = NULL;
	MsgHeader *msgheader = NULL;
	HeaderInfo *header = NULL;
	GtkWidget *editor = NULL;
	gchar *emailid = NULL;

	if (!uri) {
		return;
	}
	header = g_new0(HeaderInfo, 1);
	msgInfo = g_new0(MsgInfo, 1);
	msgheader = g_new0(MsgHeader, 1);
	header->msgInfo = msgInfo;
	msgInfo->header = msgheader;
	if (g_ascii_strncasecmp(uri, "mailto:", strlen("mailto:")) == 0) {
		if ((emailid = strrchr(uri, GNOME_VFS_URI_PATH_CHR)) != NULL) {
			msgheader->to = g_strdup(emailid + 1);
		} else if ((emailid = strrchr(uri, SUBJECT_SEPERATOR_CHAR)) != NULL) {
			msgheader->to = g_strdup(emailid + 1);
		}
	} else {
		msgheader->to = g_strdup(uri);
	}
	editor = (GtkWidget *) ui_editor_create(header, NULL, SEND_AS);
	ui_freeHeaderInfo(header);
	if (!editor) {
		osso_log(LOG_INFO, "editor creation in sendas mode failed");
		return;
	}

	gtk_widget_show(editor);
	hildon_app_set_appview(app, (HildonAppView *) editor);
}

/**
 When the link is clicked in the subject field
 corresponding browser or email editor is opened.
 
 @param text_view is the GtkWidget representing the subject.
 @param iter is the GtkTextIter of the subject textview.
 @return void.
 */
static void open_application_for_link(GtkWidget * text_view, GtkTextIter * iter)
{
	gchar *hyperlink = NULL;
	GtkTextBuffer *buffer = NULL;

	buffer = gtk_text_view_get_buffer((GtkTextView *) text_view);
	hyperlink = get_link_text(buffer, iter);
	hyperlink_clicked(NULL, hyperlink, NULL, NULL);
	g_free(hyperlink);
	return;
}

/** Links can be activated by pressing Enter.
 
 @param text_view is the GtkWidget representing the subject.
 @param event  is GdkEventKey representing key value pressed.
 @ return TRUE-to stop the event from propagating.
         FALSE-else.
*/
static gboolean key_press_event(GtkWidget * text_view, GdkEventKey * event)
{
	GtkTextIter iter;
	GtkTextBuffer *buffer = NULL;

	switch (event->keyval) {
	case GDK_Return:
	case GDK_KP_Enter:
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));
		gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
		open_application_for_link(text_view, &iter);
		break;

	default:
		break;
	}
	return FALSE;
}

/** Links can also be activated by clicking.
 
 @param text_view is the GtkWidget representing the subject.
 @param ev  is GdkEventKey representing key value pressed.
 @return TRUE-to stop the event from propagating.
         FALSE-else.
*/
static gboolean event_after(GtkWidget * text_view, GdkEvent * ev)
{
	GtkTextIter start, end, iter;
	GtkTextBuffer *buffer = NULL;
	GdkEventButton *event = NULL;
	gint x = 0, y = 0;

	if (ev->type != GDK_BUTTON_RELEASE) {
		return FALSE;
	}

	event = (GdkEventButton *) ev;

	if (event->button != 1) {
		return FALSE;
	}

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));
	/* we shouldn't follow a link if the user has selected something */
	gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
	if (gtk_text_iter_get_offset(&start) != gtk_text_iter_get_offset(&end)) {
		return FALSE;
	}
	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(text_view),
					      GTK_TEXT_WINDOW_WIDGET, event->x, event->y, &x, &y);
	gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(text_view), &iter, x, y);
	open_application_for_link(text_view, &iter);
	return FALSE;
}

/**
  Change the cursor depening upon the link or plain text
  
 @param text_view is the GtkWidget representing the subject.
 @param x is the gint representing the x coordinate of the cursor. 
 @param y is the gint representing the y coordinate of the cursor. 
 @return void. 
*/
static void set_cursor_if_appropriate(GtkTextView * text_view, gint x, gint y)
{
	GSList *tags = NULL, *tagp = NULL;
	GtkTextTag *tag = NULL;
	GtkTextBuffer *buffer = NULL;
	GtkTextIter iter;
	gint type = 0;
	GdkCursor *hand_cursor = NULL;
	GdkCursor *regular_cursor = NULL;
	gboolean hovering_over_link = FALSE;

	buffer = gtk_text_view_get_buffer(text_view);
	gtk_text_view_get_iter_at_location(text_view, &iter, x, y);

	hand_cursor = gdk_cursor_new(GDK_HAND2);
	regular_cursor = gdk_cursor_new(GDK_XTERM);

	tags = gtk_text_iter_get_tags(&iter);
	for (tagp = tags; tagp != NULL; tagp = tagp->next) {
		tag = tagp->data;
		type = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(tag), DNL));
		if (type == URL || type == EMAIL_ID) {
			hovering_over_link = TRUE;
			break;
		}
	}
	if (hovering_over_link == TRUE) {
		gdk_window_set_cursor(gtk_text_view_get_window
				      (text_view, GTK_TEXT_WINDOW_TEXT), hand_cursor);
	} else {
		gdk_window_set_cursor(gtk_text_view_get_window
				      (text_view, GTK_TEXT_WINDOW_TEXT), regular_cursor);
	}
	/*Node of this list contains integers.No memory freeing is required. */
	g_slist_free(tags);
	gdk_cursor_unref(hand_cursor);
	gdk_cursor_unref(regular_cursor);
	return;
}

/**
 To get the coordinates of the cursor as it is moved.
 
 @param subject_view is the GtkWidget representing the subject.
 @param event  is GdkEventKey representing key value pressed.
 @return TRUE-to stop the event from propagating.
         FALSE-else.
*/
static gboolean motion_notify_event(GtkWidget * subject_view, GdkEventMotion * event)
{
	gint x = 0, y = 0;

	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(subject_view),
					      GTK_TEXT_WINDOW_WIDGET, event->x, event->y, &x, &y);
	set_cursor_if_appropriate(GTK_TEXT_VIEW(subject_view), x, y);
	gdk_window_get_pointer(subject_view->window, NULL, NULL, NULL);
	return FALSE;
}

/*
 Creates the subject field for the viewer.
 
 @param msgfields is the pointer to the MsgField structure.
 @param table is the GtkWidget  pointer .
 @return void. 
 */
static void
ui_viewer_create_subject_field(MsgFields * msgfields, GtkWidget * table,
			       ViewerInfo * viewer_info, GtkSizeGroup * size_group)
{
	GtkWidget *label = NULL;
	GtkWidget *hbox_sub = NULL;
	GtkWidget *hbox = NULL;
	GtkWidget *image_event_box = NULL;

	/*hbox for subject label & priority icon */
	hbox_sub = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox_sub);
	msgfields->subject_lable_hbox = hbox_sub;

	/*priority icon */
	msgfields->priority_image =
	    gtk_image_new_from_icon_name("qgn_list_messaging_high", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(msgfields->priority_image);

	/*subject label */
	label = gtk_label_new(_("mail_va_subject"));
	gtk_widget_show(label);
	gtk_misc_set_alignment(GTK_MISC(label), X_RIGHT, Y_BOTTOM);
	gtk_size_group_add_widget(size_group, label);
	msgfields->subject_lable = label;

	gtk_box_pack_start(GTK_BOX(hbox_sub), msgfields->priority_image, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox_sub), label, TRUE, TRUE, 0);


	/*hbox for subject field & security icon if applicable */
	hbox = gtk_hbox_new(FALSE, 0);
	msgfields->subject_hbox = hbox;
	gtk_widget_show(hbox);


	msgfields->subject = gtk_text_view_new();
	gtk_text_view_set_editable(GTK_TEXT_VIEW(msgfields->subject), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), msgfields->subject, TRUE, TRUE, 0);

	gtk_table_attach((GtkTable *) table, hbox_sub, 0, 1, 3, 4,
			 (GtkAttachOptions) (GTK_FILL), 0, 0, 0);

	if (viewer_info->mail_info->folder_num == SENT_MODEL_NUM) {
		image_event_box = gtk_event_box_new();
		gtk_widget_show(image_event_box);
		msgfields->encrypt_image =
		    gtk_image_new_from_icon_name("qgn_viewer_messagin_signed",
						 GTK_ICON_SIZE_BUTTON);
		gtk_container_add(GTK_CONTAINER(image_event_box), msgfields->encrypt_image);
		gtk_box_pack_start(GTK_BOX(hbox), image_event_box, FALSE, FALSE, 0);
		g_signal_connect_after(G_OBJECT(image_event_box), "button_press_event",
				       G_CALLBACK(ui_viewer_recip_cert_properties),
				       (gpointer) viewer_info);
	}

	gtk_table_attach_defaults((GtkTable *) table, hbox, 1, 4, 3, 4);

	gtk_widget_set_size_request(msgfields->subject, BODY_WIDTH, ADDR_FIELDS_SIZE);

	g_signal_connect(G_OBJECT(msgfields->subject), "key-press-event",
			 G_CALLBACK(key_press_event), NULL);
	g_signal_connect(G_OBJECT(msgfields->subject), "event-after", G_CALLBACK(event_after),
			 NULL);
	g_signal_connect(G_OBJECT(msgfields->subject), "motion-notify-event",
			 G_CALLBACK(motion_notify_event), NULL);
	gtk_widget_set_size_request(hbox, BODY_WIDTH, ADDR_FIELDS_SIZE);
	gtk_widget_show(msgfields->subject);
	return;
}

/*
  Creation of date & time fields 
 
 @param msgfields is the pointer to the MsgField structure.
 @param table is the GtkWidget  pointer .
 @return void. 
*/
static void create_date_time_field(MsgFields * msgfields, GtkWidget * table)
{
	GtkWidget *label = NULL;
	GtkWidget *hbox_dateTime = NULL;

	label = gtk_label_new(_("mail_va_date"));
	gtk_misc_set_alignment(GTK_MISC(label), X_RIGHT, Y_BOTTOM);
	gtk_widget_show(label);
	gtk_table_attach((GtkTable *) table, label, 0, 1, 5, 6,
			 (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);

	hbox_dateTime = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
	msgfields->date_editor = gtk_label_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox_dateTime), msgfields->date_editor,
			   FALSE, FALSE, HILDON_MARGIN_DEFAULT);
	gtk_table_attach_defaults((GtkTable *) table, hbox_dateTime, 1, 4, 5, 6);
	msgfields->date_lable = label;
	msgfields->hbox_dateTime = hbox_dateTime;

	label = gtk_label_new(_("mail_va_time"));
	gtk_misc_set_alignment(GTK_MISC(label), X_RIGHT, Y_BOTTOM);
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox_dateTime), label, FALSE, FALSE, ZERO_PADDING);

	msgfields->time_editor = gtk_label_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox_dateTime), msgfields->time_editor, FALSE, FALSE,
			   ZERO_PADDING);
	gtk_widget_show_all(hbox_dateTime);

	return;
}

/**
 This function creates the layout for the viewer.
 the  from,to,cc, subject,attachment,body fields are created.
 
 @param viewer_info is the ViewerInfo structure pointer.
 No memory freeing of viewer_info is done.
 @return void.
*/
static void ui_viewer_create_layout(ViewerInfo * viewer_info, HeaderInfo * msgheader)
{
	GtkWidget *vbox_msg_content = NULL;
	MsgFields *msgfields = NULL;
	GtkWidget *table = NULL;
	GtkSizeGroup *size_group = NULL;
	gint screen_width = 0, screen_height = 0;
	GtkWidget *split_view_separator = NULL;
	gchar *image_path = NULL;

	CHECK_PARAM(viewer_info == NULL, "Input  param check failed", return);
	CHECK_PARAM(viewer_info->msg_fields == NULL, "Input  param check failed", return);
	msgfields = viewer_info->msg_fields;

	/*vbox with 3 cells */
	msgfields->vbox_msg = gtk_vbox_new(FALSE, HILDON_MARGIN_DEFAULT);
	gtk_container_add(GTK_CONTAINER(viewer_info->viewer_appview), msgfields->vbox_msg);
	gtk_widget_show(msgfields->vbox_msg);

	/*scrollwindow in the 1st cell for headers */
	msgfields->scrollwindow_body = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start(GTK_BOX(msgfields->vbox_msg),
			   msgfields->scrollwindow_body, FALSE, FALSE, ZERO_PADDING);
	gtk_widget_show(HILDON_APPVIEW(viewer_info->viewer_appview)->vbox);
	gtk_widget_show(msgfields->scrollwindow_body);
	/*gtk_container_set_border_width(GTK_CONTAINER(msgfields->scrollwindow_body),
	   HILDON_MARGIN_DEFAULT); */

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (msgfields->scrollwindow_body),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	/*Setting the default height of the srollwindow. */
	gtk_window_get_size(GTK_WINDOW(app), &screen_width, &screen_height);
	gtk_widget_set_size_request(viewer_info->msg_fields->scrollwindow_body,
				    screen_width, screen_height / 2);

/*vbox inside the scroolwindow-->can be removed*/
	vbox_msg_content = gtk_vbox_new(FALSE, HILDON_MARGIN_DOUBLE);
	gtk_widget_show(vbox_msg_content);
	gtk_scrolled_window_add_with_viewport((GtkScrolledWindow *) msgfields->
					      scrollwindow_body, vbox_msg_content);
	gtk_viewport_set_shadow_type((GtkViewport
				      *) ((GtkBin *) (msgfields->
						      scrollwindow_body))->child, GTK_SHADOW_NONE);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	/*All the header fields are inserted in a table of 6x4-to,cc,bcc,subject
	 *      ,attachments,time   are the 6 rows */
	table = gtk_table_new(MAX_ROWS + 1, MAX_COLS, FALSE);
	msgfields->table = table;
	gtk_table_set_col_spacing((GtkTable *) table, 0, HILDON_MARGIN_DOUBLE);

	gtk_widget_show(table);
	gtk_box_pack_start(GTK_BOX(vbox_msg_content), table, FALSE, FALSE, ZERO_PADDING);
	image_path = get_theme_image_path();
  /**Creation of address field from*/
	ui_viewer_create_address_field(&(msgfields->from), table, FROM, viewer_info, image_path);
	gtk_size_group_add_widget(size_group, viewer_info->msg_fields->from_lable);

	g_signal_connect(G_OBJECT(msgfields->from), "button_press_event",
			 G_CALLBACK(ui_make_selection_cb), viewer_info);

  /**Creation of address field to*/
	ui_viewer_create_address_field(&(msgfields->to), table, TO, viewer_info, image_path);
	gtk_size_group_add_widget(size_group, viewer_info->msg_fields->to_lable);
	g_signal_connect(G_OBJECT(msgfields->to), "button_press_event",
			 G_CALLBACK(ui_make_selection_cb), viewer_info);

  /**Creation of address field cc*/
	if (msgheader->cc) {
		ui_viewer_create_address_field(&(msgfields->cc), table, CC, viewer_info,
					       image_path);
		gtk_size_group_add_widget(size_group, viewer_info->msg_fields->cc_lable);
		g_signal_connect(G_OBJECT(msgfields->cc), "button_press_event",
				 G_CALLBACK(ui_make_selection_cb), viewer_info);
		g_signal_connect(G_OBJECT(msgfields->cc), "button_release_event",
				 G_CALLBACK(make_selection), viewer_info);
	}
	g_free(image_path);

	/*creation of subject field */
	ui_viewer_create_subject_field(msgfields, table, viewer_info, size_group);
	ui_set_priority_icon(viewer_info->mail_info->priority,
			     (GtkImage *) msgfields->priority_image);

	/*Creation of attachemnt field */
	ui_create_attachment_list_view(msgfields, table, TRUE);
	gtk_size_group_add_widget(size_group, viewer_info->msg_fields->attachment_lable);
	g_signal_connect(G_OBJECT
			 (gtk_tree_view_get_selection
			  ((GtkTreeView *) msgfields->view)), "changed",
			 G_CALLBACK(ui_attachment_selection_cb), viewer_info);
	g_signal_connect(G_OBJECT(msgfields->view), "row_activated",
			 G_CALLBACK(ui_open_attachment_cb), viewer_info);
	g_signal_connect(G_OBJECT(msgfields->view), "focus-in-event",
			 G_CALLBACK(ui_attachment_focused), viewer_info);

	/*Creation of date & time fields */
	create_date_time_field(msgfields, table);

	/*separator in the second cell */
	split_view_separator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(msgfields->vbox_msg), split_view_separator, FALSE, FALSE, 0);
	gtk_widget_show(split_view_separator);

	/*hiding of fields which are unnecessary */
	gtk_widget_hide(msgfields->sign_image);
	if (viewer_info->mail_info->folder_num != SENT_MODEL_NUM) {
		msgfields->from_available = TRUE;
	}
	gtk_widget_show(viewer_info->viewer_appview);
	return;
}

/**
 this function is used to set the persistence settings of the viewer toolbar

 @param viewer_info is the ViewerInfo structure pointer.
 @return void.
*/
static void set_viewer_toolbar_state(ViewerInfo * viewer_info)
{
	/*setting the toolbar as per the persistent settings */
	if (ui_get_viewer_toolbar_hidden_status() == TRUE) {
		ui_toggle_toolbar(FALSE,
				  (HildonAppView *) viewer_info->viewer_appview,
				  (viewer_info->viewer_toolbar));
		g_signal_handlers_block_by_func(viewer_info->menu->
						toolbar_normalview,
						ui_toggle_viewer_toolbar_cb, viewer_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (viewer_info->menu->toolbar_normalview), FALSE);
		g_signal_handlers_unblock_by_func(viewer_info->menu->
						  toolbar_normalview,
						  ui_toggle_viewer_toolbar_cb, viewer_info);
	} else {
		g_signal_handlers_block_by_func(viewer_info->menu->
						toolbar_normalview,
						ui_toggle_viewer_toolbar_cb, viewer_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (viewer_info->menu->toolbar_normalview), TRUE);
		g_signal_handlers_unblock_by_func(viewer_info->menu->
						  toolbar_normalview,
						  ui_toggle_viewer_toolbar_cb, viewer_info);
	}
	if (ui_get_viewer_fullscreen_toolbar_hidden_status() == TRUE) {
		ui_toggle_toolbar(FALSE,
				  (HildonAppView *) viewer_info->viewer_appview,
				  (viewer_info->viewer_fullscreen_toolbar));
		g_signal_handlers_block_by_func(viewer_info->menu->
						toolbar_fullscreen,
						ui_toggle_viewer_fullscreen_toolbar_cb,
						viewer_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (viewer_info->menu->toolbar_fullscreen), FALSE);
		g_signal_handlers_unblock_by_func(viewer_info->menu->toolbar_fullscreen,
						  ui_toggle_viewer_fullscreen_toolbar_cb,
						  viewer_info);
	} else {
		g_signal_handlers_block_by_func(viewer_info->menu->
						toolbar_fullscreen,
						ui_toggle_viewer_fullscreen_toolbar_cb,
						viewer_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (viewer_info->menu->toolbar_fullscreen), TRUE);
		g_signal_handlers_unblock_by_func(viewer_info->menu->toolbar_fullscreen,
						  ui_toggle_viewer_fullscreen_toolbar_cb,
						  viewer_info);
	}
	return;
}

/**
 This function creates the toolbar
 
 @param viewer_info is the ViewerInfo structure pointer.
 No memory freeing of viewer_info is done.
 
 @return void.
*/
static void ui_viewer_create_toolbar(ViewerInfo * viewer_info, gboolean body_downloaded)
{
	GtkWidget *viewer_toolbar = NULL;
	GtkAlignment *leftalignment = NULL;
	GtkWidget *image_reply = NULL;
	GtkWidget *image_replyall = NULL;
	GtkWidget *image_resend = NULL;
	GtkWidget *image_forward = NULL;
	GtkWidget *image_saveattach = NULL;
	GtkWidget *image_moveto = NULL;
	GtkWidget *image_delete = NULL;
	GtkWidget *image_prevmsg = NULL;
	GtkWidget *image_nextmsg = NULL;
	GtkWidget *image_animation = NULL;
	GtkWidget *image_prog_stop = NULL;
	GtkToolItem *tool_item = NULL;
	GtkIconInfo *prog_anim_info = NULL;
	ViewerToolbar *toolbar = NULL;
	CHECK_PARAM(viewer_info == NULL, "Input  param check failed", return);
	CHECK_PARAM(viewer_info->toolbar == NULL, "Input  param check failed", return);
	toolbar = viewer_info->toolbar;

	viewer_toolbar = gtk_toolbar_new();
	gtk_toolbar_set_style((GtkToolbar *) viewer_toolbar, GTK_ORIENTATION_HORIZONTAL);

	/*space for left margin */
	toolbar->left_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(toolbar->left_space, TOOLBAR_LEFT_MARGIN,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->left_space), -1);

	if (viewer_info->mail_info->folder_num != SENT_MODEL_NUM) {
		/*reply tool item */
		image_reply =
		    gtk_image_new_from_icon_name("qgn_toolb_messagin_reply", GTK_ICON_SIZE_BUTTON);
		toolbar->reply = (GtkWidget *) gtk_tool_button_new(image_reply, NULL);
		gtk_widget_show(toolbar->reply);
		gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->reply), -1);
		gtk_widget_set_size_request(toolbar->reply, TOOLBAR_BUTTON_WIDTH,
					    TOOLBAR_BUTTON_HEIGHT);
		g_signal_connect(G_OBJECT(toolbar->reply), "clicked",
				 G_CALLBACK(ui_viewer_reply_cb), viewer_info);
		g_signal_connect(G_OBJECT(gtk_bin_get_child((GtkBin *) toolbar->reply)),
				 "insensitive_press", G_CALLBACK(show_dimming_note),
				 _("mail_ib_notavailable_downloading"));
		/*reply space */
		toolbar->reply_space = GTK_WIDGET(gtk_tool_item_new());
		gtk_widget_set_size_request(toolbar->reply_space, TOOLBAR_MARGIN_BTW_BUTTONS,
					    TOOLBAR_BUTTON_HEIGHT);
		gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->reply_space),
				   -1);


		/*reply all tool item */
		image_replyall =
		    gtk_image_new_from_icon_name("qgn_toolb_messagin_replytoall",
						 GTK_ICON_SIZE_BUTTON);
		toolbar->replyAll = (GtkWidget *) gtk_tool_button_new(image_replyall, NULL);
		gtk_widget_show(toolbar->replyAll);
		gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->replyAll),
				   -1);
		gtk_widget_set_size_request(toolbar->replyAll, TOOLBAR_BUTTON_WIDTH,
					    TOOLBAR_BUTTON_HEIGHT);

		g_signal_connect(G_OBJECT(toolbar->replyAll), "clicked",
				 G_CALLBACK(ui_viewer_replyAll_cb), viewer_info);
		g_signal_connect(G_OBJECT(gtk_bin_get_child((GtkBin *) toolbar->replyAll)),
				 "insensitive_press", G_CALLBACK(show_dimming_note),
				 _("mail_ib_notavailable_downloading"));

		/*reply_all space */
		toolbar->replyAll_space = GTK_WIDGET(gtk_tool_item_new());
		gtk_widget_set_size_request(toolbar->replyAll_space, TOOLBAR_MARGIN_BTW_BUTTONS,
					    TOOLBAR_BUTTON_HEIGHT);
		gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar),
				   GTK_TOOL_ITEM(toolbar->replyAll_space), -1);
	} else {
		/*resend tool item */
		image_resend =
		    gtk_image_new_from_icon_name("qgn_toolb_messagin_resend", GTK_ICON_SIZE_BUTTON);
		toolbar->resend = (GtkWidget *) gtk_tool_button_new(image_resend, NULL);
		gtk_widget_show(toolbar->resend);
		gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->resend), -1);
		gtk_widget_set_size_request(toolbar->resend, TOOLBAR_BUTTON_WIDTH,
					    TOOLBAR_BUTTON_HEIGHT);

		g_signal_connect(G_OBJECT(toolbar->resend), "clicked",
				 G_CALLBACK(ui_viewer_resend_cb), viewer_info);
		g_signal_connect(G_OBJECT(gtk_bin_get_child((GtkBin *) toolbar->resend)),
				 "insensitive_press", G_CALLBACK(show_dimming_note),
				 _("mail_ib_notavailable_downloading"));
		/*resend space */
		toolbar->resend_space = GTK_WIDGET(gtk_tool_item_new());
		gtk_widget_set_size_request(toolbar->resend_space, TOOLBAR_MARGIN_BTW_BUTTONS,
					    TOOLBAR_BUTTON_HEIGHT);
		gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar),
				   GTK_TOOL_ITEM(toolbar->resend_space), -1);
	}

	/*forward tool item */
	image_forward =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_forward", GTK_ICON_SIZE_BUTTON);
	toolbar->forward = (GtkWidget *) gtk_tool_button_new(image_forward, NULL);
	gtk_widget_show(toolbar->forward);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->forward), -1);
	gtk_widget_set_size_request(toolbar->forward, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);

	/*forward space */
	toolbar->forward_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(toolbar->forward_space, TOOLBAR_MARGIN_BTW_BUTTONS,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->forward_space), -1);
	g_signal_connect(G_OBJECT(toolbar->forward), "clicked",
			 G_CALLBACK(ui_viewer_forward_cb), viewer_info);
	g_signal_connect(G_OBJECT(gtk_bin_get_child((GtkBin *) toolbar->forward)),
			 "insensitive_press", G_CALLBACK(show_dimming_note),
			 _("mail_ib_notavailable_downloading"));

	/*save attachment tool item */
	image_saveattach =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_saveattachment", GTK_ICON_SIZE_BUTTON);
	toolbar->saveAttachment = (GtkWidget *) gtk_tool_button_new(image_saveattach, NULL);
	gtk_widget_set_sensitive(toolbar->saveAttachment, FALSE);
	gtk_widget_show(toolbar->saveAttachment);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->saveAttachment), -1);
	gtk_widget_set_size_request(toolbar->saveAttachment, TOOLBAR_MARGIN_BTW_BUTTONS,
				    TOOLBAR_BUTTON_HEIGHT);

	g_signal_connect(G_OBJECT(toolbar->saveAttachment), "clicked",
			 G_CALLBACK(ui_viewer_save_attachment_cb), viewer_info);
	g_signal_connect(G_OBJECT
			 (gtk_bin_get_child
			  ((GtkBin *) toolbar->saveAttachment)),
			 "insensitive_press", G_CALLBACK(save_view_dimming_note), viewer_info);
	/*save attachment space */
	toolbar->saveAttachment_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(toolbar->saveAttachment_space, TOOLBAR_MARGIN_BTW_BUTTONS,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar),
			   GTK_TOOL_ITEM(toolbar->saveAttachment_space), -1);

	/*move to tool item */
	image_moveto =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_moveto", GTK_ICON_SIZE_BUTTON);
	toolbar->moveTo = (GtkWidget *) gtk_tool_button_new(image_moveto, NULL);
	gtk_widget_show(toolbar->moveTo);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->moveTo), -1);
	gtk_widget_set_size_request(toolbar->moveTo, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	g_signal_connect(G_OBJECT(toolbar->moveTo), "clicked",
			 G_CALLBACK(ui_viewer_move_msg_cb), viewer_info);

	/*move to space */
	toolbar->moveTo_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(toolbar->moveTo_space, TOOLBAR_MARGIN_BTW_BUTTONS,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->moveTo_space), -1);

	/*delete tool item */
	image_delete =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_delete", GTK_ICON_SIZE_BUTTON);
	toolbar->delete_msg = (GtkWidget *) gtk_tool_button_new(image_delete, NULL);
	gtk_widget_show(toolbar->delete_msg);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->delete_msg), -1);
	gtk_widget_set_size_request(toolbar->delete_msg, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	g_signal_connect(G_OBJECT(toolbar->delete_msg), "clicked",
			 G_CALLBACK(ui_viewer_delete_message_cb), viewer_info);
	/*delete space */
	toolbar->delete_msg_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(toolbar->delete_msg_space, TOOLBAR_MARGIN_BTW_BUTTONS,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->delete_msg_space),
			   -1);

	/*next message tool item */
	image_nextmsg =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_next", GTK_ICON_SIZE_BUTTON);
	toolbar->nextMsg = (GtkWidget *) gtk_tool_button_new(image_nextmsg, NULL);
	gtk_widget_show(toolbar->nextMsg);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->nextMsg), -1);
	gtk_widget_set_size_request(toolbar->nextMsg, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	g_signal_connect((gpointer) toolbar->nextMsg, "clicked",
			 G_CALLBACK(ui_view_next_message_cb), viewer_info);
	/*fix for bug #25165 display of no more messages caused the bug
	   instead  viewer is closed */
	if (!viewer_info->mail_info->email_attachment)
		g_signal_connect(G_OBJECT
				 (gtk_bin_get_child((GtkBin *) toolbar->nextMsg)),
				 "insensitive_press", G_CALLBACK(ui_viewer_close_cb), viewer_info);
	else
		g_signal_connect(G_OBJECT
				 (gtk_bin_get_child((GtkBin *) toolbar->nextMsg)),
				 "insensitive_press", G_CALLBACK(show_dimming_note),
				 _("mcen_ib_nomore_messages"));
	/*next  msg space */
	toolbar->nextMsg_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(toolbar->nextMsg_space, TOOLBAR_MARGIN_BTW_BUTTONS,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->nextMsg_space), -1);

	/*previous message tool item */
	image_prevmsg =
	    gtk_image_new_from_icon_name("qgn_toolb_messagin_previous", GTK_ICON_SIZE_BUTTON);
	toolbar->prevMsg = (GtkWidget *) gtk_tool_button_new(image_prevmsg, NULL);
	gtk_widget_show(toolbar->prevMsg);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->prevMsg), -1);
	gtk_widget_set_size_request(toolbar->prevMsg, TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	g_signal_connect((gpointer) toolbar->prevMsg, "clicked",
			 G_CALLBACK(ui_view_next_message_cb), viewer_info);
	/*fix for bug #25165 display of no more messages caused the bug
	   instead  viewer is closed */
	if (!viewer_info->mail_info->email_attachment)
		g_signal_connect(G_OBJECT
				 (gtk_bin_get_child((GtkBin *) toolbar->prevMsg)),
				 "insensitive_press", G_CALLBACK(ui_viewer_close_cb), viewer_info);
	else
		g_signal_connect(G_OBJECT
				 (gtk_bin_get_child((GtkBin *) toolbar->prevMsg)),
				 "insensitive_press", G_CALLBACK(show_dimming_note),
				 _("mcen_ib_nomore_messages"));
	/*prev  msg space */
	toolbar->prevMsg_space = GTK_WIDGET(gtk_tool_item_new());
	gtk_widget_set_size_request(toolbar->prevMsg_space, TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->prevMsg_space), -1);

	/*download progress bar */

	toolbar->progress_hbox = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
	/*progress separator */
	toolbar->separator = GTK_WIDGET(gtk_separator_tool_item_new());
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->separator), -1);
	gtk_widget_show(toolbar->separator);


	/*cancel progress button */
	image_prog_stop = gtk_image_new_from_icon_name("qgn_toolb_gene_stop", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image_prog_stop);
	toolbar->progressbar_cancel = GTK_WIDGET(gtk_tool_button_new(image_prog_stop, NULL));
	gtk_widget_set_size_request(toolbar->progressbar_cancel,
				    TOOLBAR_BUTTON_WIDTH, TOOLBAR_BUTTON_HEIGHT);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), GTK_TOOL_ITEM(toolbar->progressbar_cancel),
			   -1);
	g_signal_connect(toolbar->progressbar_cancel, "clicked", G_CALLBACK(cancel_progressbar),
			 viewer_info);

	/*progress bar */
	toolbar->progressbar = gtk_progress_bar_new();
	leftalignment = (GtkAlignment *) gtk_alignment_new(XALIGN, XALIGN, 1, XSPACE);
	gtk_widget_show((GtkWidget *) leftalignment);
	gtk_box_pack_start(GTK_BOX(toolbar->progress_hbox), (GtkWidget *) leftalignment, TRUE, TRUE,
			   0);
	gtk_container_add(GTK_CONTAINER(leftalignment), toolbar->progressbar);
	/*progress animation */
	prog_anim_info =
	    gtk_icon_theme_lookup_icon(email_main_view->icons->icon_theme,
				       "qgn_indi_pball_a", HILDON_ICON_SIZE_26,
				       GTK_ICON_LOOKUP_NO_SVG);
	if (prog_anim_info) {
		image_animation =
		    gtk_image_new_from_file(gtk_icon_info_get_filename(prog_anim_info));
		gtk_icon_info_free(prog_anim_info);
		gtk_box_pack_start(GTK_BOX(toolbar->progress_hbox), image_animation, FALSE, FALSE,
				   0);
	}

	tool_item = gtk_tool_item_new();
	gtk_tool_item_set_expand(GTK_TOOL_ITEM(tool_item), TRUE);
	gtk_container_add((GtkContainer *) tool_item, toolbar->progress_hbox);
	gtk_widget_show_all((GtkWidget *) tool_item);
	gtk_toolbar_insert(GTK_TOOLBAR(viewer_toolbar), tool_item, -1);


	gtk_box_pack_start(GTK_BOX(HILDON_APPVIEW
				   (viewer_info->viewer_appview)->vbox),
			   viewer_toolbar, FALSE, FALSE, ZERO_PADDING);

	gtk_widget_show_all(viewer_toolbar);
	gtk_widget_show(HILDON_APPVIEW(viewer_info->viewer_appview)->vbox);

	viewer_info->viewer_toolbar = viewer_toolbar;
	viewer_info->viewer_fullscreen_toolbar = viewer_toolbar;

	if (viewer_info->mail_info->email_attachment != TRUE && viewer_info->mime_info == NULL) {
		show_progressbar(viewer_info);
	} else {
		hide_progressbar(viewer_info);
	}
	if (viewer_info->mime_info != NULL) {
		if (viewer_info->mail_info->folder_num == SENT_MODEL_NUM || body_downloaded == TRUE) {
			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(toolbar->progressbar),
						  _("mail_me_opening"));
			ULOG_DEBUG_F("the progress bar text is opening");
		}
	}

	set_viewer_toolbar_state(viewer_info);
	update_toolbar_view(viewer_info);
	return;
}

/**
 This is the helper function to get the first plain pointer of the mimeinfo.
 @param mime_info is the MiemInfo straucture pointer .
 @return TRUE if plain part is found.
         FALSE-else
*/
gboolean check_for_plain_html(const MimeInfo * mime_info)
{
	gboolean plainpart = FALSE;
	MimeInfo *part_info = NULL;

	part_info = (MimeInfo *) mime_info;
	while (part_info != NULL) {
		if (part_info->mime_type == MIME_TEXT) {
			plainpart = TRUE;
			break;
		}
		part_info = part_info->children;
	}
	return plainpart;
}

/**
 This function is used to set the body of the mail in the browser/normal textview.
 
 @param viewer_info is the ViewerInfo structure pointer.
 No memory freeing of viewer_info is done.
 @param body_file is the file name of a body mail.
 
 @return void
*/
static void ui_viewer_set_body_content(ViewerInfo * viewer_info, const gchar * body_file)
{
	MailInfo *mailinfo = NULL;
	MsgFields *msgfields = NULL;

	CHECK_PARAM(viewer_info == NULL, "Input  param check failed", return);
	CHECK_PARAM(viewer_info->msg_fields == NULL, "Input  param check failed", return);
	CHECK_PARAM(viewer_info->mail_info == NULL, "Input  param check failed", return);

	mailinfo = viewer_info->mail_info;
	msgfields = viewer_info->msg_fields;

	viewer_info->plain_filename = g_strdup(body_file);
	/*Creating mail viewer widget */
	msgfields->mail_viewer = mail_viewer_new();
	gtk_widget_show(msgfields->mail_viewer);
	msgfields->scroll_area = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start(GTK_BOX(msgfields->vbox_msg),
			   msgfields->scroll_area, TRUE, TRUE, ZERO_PADDING);
	gtk_widget_show(msgfields->scroll_area);
	gtk_container_set_border_width(GTK_CONTAINER(msgfields->scroll_area),
				       HILDON_MARGIN_DEFAULT);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(msgfields->scroll_area),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_container_add((GtkContainer *) msgfields->scroll_area, msgfields->mail_viewer);
	viewer_info->zoominfo->mail_viewer = msgfields->mail_viewer;

	g_signal_connect(G_OBJECT(msgfields->mail_viewer), "loaded_part",
			 G_CALLBACK(update_body_load_progress), viewer_info);
	g_signal_connect(G_OBJECT(msgfields->mail_viewer), "link_clicked",
			 G_CALLBACK(hyperlink_clicked), viewer_info);

	GTK_WIDGET_SET_FLAGS(msgfields->mail_viewer, GTK_CAN_FOCUS);
	if (!mail_viewer_set_content_from_file(MAIL_VIEWER(msgfields->mail_viewer), body_file)) {
		/*close the progress bar if no contents are set */
		if (mailinfo->email_attachment == FALSE)
			hide_progressbar(viewer_info);
	}
	gtk_window_set_focus((GtkWindow *) app, msgfields->mail_viewer);
	return;
}

/**
 This function checks content type recursively until it is multipart/alternative
        or multipart/related parsing only 'children' nodes .
	     
 @param mimeInfo is the MimeInfo structure pointer.
	 No memory freeing for mimeInfo is done .
 @param html_filebody is the OUT param representing the file body.
 @param file name is the name of the body file.
 @param msgid is the gchar representing the unique id of the mail.
 
 @returns the html body file in html_file_body OUT param &
         TRUE-if html mail
	 FALSE-else
*/
static gboolean
ui_viewer_get_html_body_file(const MimeInfo * mimeInfo,
			     gchar ** html_file_body,
			     const gchar * filename, const gchar * msgid,
			     const gchar * font_name, gboolean email_attachment)
{
	MimeInfo *partinfo = NULL;
	gboolean htmlmail = FALSE;

	for (partinfo = (MimeInfo *) mimeInfo; partinfo != NULL; partinfo = partinfo->children) {
	 /**Checking the content type to be multipart alternative.if so get the 
	   corresponding html body file*/
		if (g_ascii_strncasecmp
		    (partinfo->content_type, MULTIPART_ALTERNATIVE,
		     MAX_STR_LEN(partinfo->content_type,
				 MULTIPART_ALTERNATIVE)) == 0
		    || g_ascii_strncasecmp(partinfo->content_type,
					   MULTIPART_RELATED,
					   MAX_STR_LEN(partinfo->content_type,
						       MULTIPART_RELATED)) == 0) {
			if (email_attachment == FALSE) {
				/*Get html body content location from engine */
				*html_file_body =
				    ui_dbus_recv_multipart_body(partinfo, msgid, font_name);
				if (*html_file_body == NULL) {
					*html_file_body =
					    add_charset_info_to_file(filename, mimeInfo);
				}
			}
			htmlmail = TRUE;
			break;
		}
	}
	/*If the content type is not multipart alternative,check if it is multipart html 
	   if so assing the mimeinfo->filename to the html body file name */
	if (htmlmail == FALSE) {
		for (partinfo = (MimeInfo *) mimeInfo; partinfo != NULL;
		     partinfo = partinfo->children) {
			if (g_ascii_strncasecmp
			    (partinfo->content_type, MULTIPART_TEXT_HTML,
			     MAX_STR_LEN(partinfo->content_type, MULTIPART_TEXT_HTML)) == 0) {
				if (email_attachment == FALSE)
					*html_file_body =
					    add_charset_info_to_file(filename, mimeInfo);
				htmlmail = TRUE;
			}
		}
	}
	return htmlmail;
}

/**
 Checks whether the mail is  html/plain.
 Gets the corresponding filename that contains the body of the mail.
 
 @param viewer_info is the ViewerInfo stucture pointer.
  No memory freeing of viewer_info is done .
 @param msgHeader is the HeaderInfo structure pointer.
  No memory freeing of msgHeader is done .

 @return void.
*/
static void ui_viewer_embed_body(ViewerInfo * viewer_info)
{
	MimeInfo *mimeInfo = NULL;
	gchar *body_file = NULL;
	MailInfo *mailinfo = NULL;
	const gchar *font_name = NULL;

	if (viewer_info == NULL) {
		return;
	}
	mailinfo = viewer_info->mail_info;
	mimeInfo = viewer_info->mime_info;

	gint engine_errno = 0;
	if (mimeInfo != NULL && mimeInfo->filename == NULL) {
		engine_errno = ui_dbus_get_engine_errno();
		if (engine_errno == ENGINE_ERROR_DEVICE_MEMORY_FULL) {
			if (mailinfo->email_attachment == FALSE) {
				/*error occured while creating the body file */
				hide_progressbar(viewer_info);
			}
			return;
		}
	}

	if (mailinfo->email_attachment == TRUE) {
		body_file = g_strdup(viewer_info->email_attachment_filename);
	}

	if (mimeInfo != NULL) {
		/*gets the default font */
		font_name = ui_get_default_font(TRUE);

		if (mimeInfo->mime_type == MIME_MULTIPART) {
			/*check content type recursively until it is multipart/alternative
			   or multipart/related parsing only 'children' nodes */
			mailinfo->htmlmail =
			    ui_viewer_get_html_body_file(mimeInfo, &body_file,
							 mimeInfo->filename, viewer_info->msgid,
							 font_name, mailinfo->email_attachment);
		} else if (mimeInfo->mime_type == MIME_TEXT_HTML) {
			if (!mailinfo->email_attachment) {
				/*Get html body content location from engine */
				body_file =
				    ui_dbus_recv_multipart_body(mimeInfo, viewer_info->msgid,
								font_name);
				if (body_file == NULL) {
					body_file =
					    add_charset_info_to_file(mimeInfo->filename, mimeInfo);
				}
			}
			mailinfo->htmlmail = TRUE;
		} else {
			mailinfo->htmlmail = FALSE;
		}

		/*For plain mail add the charset info */
		if (!mailinfo->htmlmail) {
			g_free(body_file);
			if (!mailinfo->email_attachment)
				body_file =
				    add_charset_info_to_file(mimeInfo->filename,
							     viewer_info->mime_info);
			else
				body_file =
				    add_charset_info_to_file(viewer_info->email_attachment_filename,
							     viewer_info->mime_info);
		}
	      /**Sets the mail body */
		ui_viewer_set_body_content(viewer_info, body_file);
	}
	g_free(body_file);
	return;
}

/**
 This function is used to set the subject of the viewer.
 It also recognises the dnl,if present it is present it is underlined. 
 
 @param subject_widget is the GtkWidget representing the subject field. 
 @param subject is the char represpenting the subject.
 @return void. 
         
*/
static void ui_viewer_set_subject(GtkWidget * subject_widget, const gchar * subject)
{
	GtkTextBuffer *bufferSub = NULL;
	GtkTextIter start, end;
	gchar *emailsub = NULL;
	gchar **parseStr = NULL, **splitstr = NULL, **tmpStr = NULL;
	gint i = 0;
	GtkTextTag *tagurl = NULL, *tagmailid = NULL;

	bufferSub = gtk_text_view_get_buffer(GTK_TEXT_VIEW(subject_widget));
	tagurl =
	    gtk_text_buffer_create_tag(bufferSub, NULL, "foreground", "blue",
				       "underline", PANGO_UNDERLINE_SINGLE, NULL);
	g_object_set_data(G_OBJECT(tagurl), DNL, GINT_TO_POINTER(URL));
	tagmailid =
	    gtk_text_buffer_create_tag(bufferSub, NULL, "foreground", "blue",
				       "underline", PANGO_UNDERLINE_SINGLE, NULL);
	g_object_set_data(G_OBJECT(tagmailid), DNL, GINT_TO_POINTER(EMAIL_ID));
	if (subject != NULL && strlen(subject) != 0) {
		/*Parse for the hyperlinks subject is freed inside the call */
		emailsub = ui_viewer_html_parser((subject));
		parseStr = g_strsplit(emailsub, HREF, -1);
		g_free(emailsub);
		gtk_text_buffer_get_bounds(bufferSub, &start, &end);
		gtk_text_buffer_get_start_iter(bufferSub, &start);
		while (parseStr[i] != NULL) {
			splitstr = g_strsplit(parseStr[i], HREF_SEP, TWO);
			if (splitstr[0] != NULL && splitstr[1] != NULL) {
				tmpStr = g_strsplit(splitstr[1], HREF_END, TWO);
				if (strchr(tmpStr[0], AT_THE_RATE) != NULL) {
					gtk_text_buffer_insert_with_tags(bufferSub, &start,
									 tmpStr[0], -1, tagmailid,
									 NULL);
				} else {
					gtk_text_buffer_insert_with_tags(bufferSub, &start,
									 tmpStr[0], -1, tagurl,
									 NULL);
				}
				gtk_text_buffer_get_end_iter(bufferSub, &start);
				if (tmpStr[1] != NULL) {
					gtk_text_buffer_insert(bufferSub, &start, tmpStr[1], -1);
					gtk_text_buffer_get_end_iter(bufferSub, &start);
				}
				g_strfreev(splitstr);
			} else {
				gtk_text_buffer_insert(bufferSub, &start, splitstr[0], -1);
				g_strfreev(splitstr);
			}
			i++;
		}
		g_strfreev(parseStr);
	} else {
		gtk_text_buffer_set_text(bufferSub, _("mail_va_no_subject"), -1);
	}
	focus_to_start(bufferSub, subject_widget);
	return;
}

/**
 This is a helper function to set the cursor back to the start position after
 the set text is done in various editor/viewer fields.
 */
void focus_to_start(GtkTextBuffer * buffer, GtkWidget * view)
{
	GtkTextIter iter;
	GtkTextMark *mark = NULL;
	gtk_widget_grab_focus(view);
	gtk_text_buffer_get_start_iter(buffer, &iter);
	gtk_text_buffer_place_cursor(buffer, &iter);
	mark = gtk_text_buffer_get_mark(buffer, "insert");
	gtk_text_view_scroll_mark_onscreen((GtkTextView *) view, mark);
	return;
}



/**
 This function fills the viewer with the msgHeader data give by the main ui.
 
 @param viewer_info is the ViewerInfo stucture pointer.
  No memory freeing of viewerInfo is done .
 @param msgHeader is the HeaderInfo structure pointer.
  No memory freeing of msgHeader is done .
 @return gboolean TRUE if mail body contents are read
                  FALSE,else-condition arises only for secure mails. 
*/
static gboolean ui_viewer_set_contents(ViewerInfo * viewer_info, HeaderInfo * msgHeader)
{
	GtkTextBuffer *bufferFrom = NULL;
	GtkTextBuffer *bufferTo = NULL;
	GtkTextBuffer *bufferCc = NULL;
	gboolean attachmentstatus = FALSE;
	MsgFields *msgfields = NULL;
	MimeInfo *mimeinfo = NULL;
	/*Action to be taken for a secure mail durning a warning. */
	SecureMailAction mailAction = READ_MAIL;

	CHECK_PARAM(viewer_info == NULL, "Input param check failed\n", return FALSE);
	CHECK_PARAM(viewer_info->msg_fields == NULL, "Input param check failed \n", return FALSE);
	CHECK_PARAM(msgHeader == NULL, "Input param check failed\n", return FALSE);

	msgfields = viewer_info->msg_fields;

	if (viewer_info->mail_info->email_attachment == TRUE
	    && viewer_info->mail_info->smime_type != SMIME_NONE
	    && viewer_info->mail_info->smime_type != SMIME_INTERNAL) {
		mimeinfo = viewer_info->mime_info->main;
	} else {
		mimeinfo = viewer_info->mime_info;
	}


	/*hide all the unnecessary widgets, initially */
	if (viewer_info->mail_info->same_appview == TRUE) {
		hide_headers(NULL, NULL, viewer_info);
		viewer_info->mail_info->headers_hidden = !viewer_info->mail_info->headers_hidden;

	}
	hide_header_widgets(viewer_info);
	gtk_widget_hide(viewer_info->msg_fields->priority_image);

	/*Setting the from address */
	bufferFrom = gtk_text_view_get_buffer(GTK_TEXT_VIEW(msgfields->from));
	if (msgHeader->from != NULL && strlen(msgHeader->from) != 0) {
		gtk_text_buffer_set_text(bufferFrom, msgHeader->from, -1);
		focus_to_start(bufferFrom, msgfields->from);
		gtk_widget_show(msgfields->from);
	}

	if (msgHeader->folderNum == SENT_MODEL_NUM) {
		hide_from_field(msgfields);
	}
	/*Setting the to address */
	bufferTo = gtk_text_view_get_buffer(GTK_TEXT_VIEW(msgfields->to));
	if (msgHeader->to != NULL && strlen(msgHeader->to) != 0) {
		gtk_text_buffer_set_text(bufferTo, msgHeader->to, -1);
		gtk_widget_show(msgfields->to);
		gtk_widget_show(msgfields->to_lable);
		focus_to_start(bufferTo, msgfields->to);
	}

	/*Setting the cc address */
	if ((msgHeader->cc) != NULL) {
		bufferCc = gtk_text_view_get_buffer(GTK_TEXT_VIEW(msgfields->cc));
		gtk_text_buffer_set_text(bufferCc, msgHeader->cc, -1);
		show_cc_field(msgfields, TRUE);
		msgfields->cc_available = TRUE;
		focus_to_start(bufferCc, msgfields->cc);
	}

	/*Setting the subject */
	ui_viewer_set_subject(msgfields->subject, msgHeader->subject);
	/*sets the time & date of the msg received in the viewer */
	ui_viewer_set_date_time_for_msg((GtkLabel *) msgfields->date_editor,
					(GtkLabel *) msgfields->time_editor, msgHeader->sent_date);

	ui_set_priority_icon(viewer_info->mail_info->priority,
			     (GtkImage *) msgfields->priority_image);
	if (viewer_info->mime_info == NULL) {
		/*Retaining the persistence of hide/unhide headers across next/prev 
		 * messages*/
		if (viewer_info->mail_info->same_appview == TRUE) {
			hide_headers(NULL, NULL, viewer_info);
			viewer_info->mail_info->headers_hidden =
			    !viewer_info->mail_info->headers_hidden;
		}
		return TRUE;
	}

	attachmentstatus =
	    ui_viewer_check_attachments(viewer_info->mime_info,
					msgfields->liststore, msgfields->view,
					viewer_info->mail_info->email_attachment);
	if (attachmentstatus == TRUE) {
		show_attachment_field(msgfields, TRUE);
		viewer_info->msg_fields->attachments_available = TRUE;
	} else if (attachmentstatus == FALSE) {
		gtk_widget_set_sensitive(viewer_info->menu->view, FALSE);
		ULOG_DEBUG(":no attachments\n");
	}
	set_viewer_security_icon(msgfields, mimeinfo);
#if SMIME
	/*Checks whether the mail has any security options set */
	if (mimeinfo->smime_type != SMIME_NONE) {
		/*stoping the progress until usr clicks read mail */
		if (viewer_info->mail_info->timer_value > 0)
			g_source_remove(viewer_info->mail_info->timer_value);

		mailAction = check_security_status(viewer_info->msgid, mimeinfo, msgfields);

		/*connecting a timeout call to update the progress bar */
		update_progress(viewer_info);
		viewer_info->mail_info->timer_value =
		    g_timeout_add(1000, (GSourceFunc) update_progress, viewer_info);
	}
#endif

	/*Checks Whether a html /plain mail & embeds the content in browser or plain textview */
	ui_viewer_embed_body(viewer_info);


	set_viewer_security_icon(msgfields, mimeinfo);
	ui_set_priority_icon(viewer_info->mail_info->priority,
			     (GtkImage *) msgfields->priority_image);

	if (msgHeader->folderNum == SENT_MODEL_NUM) {
		hide_from_field(msgfields);
	}
	if (attachmentstatus == FALSE) {
		show_attachment_field(msgfields, FALSE);
	}
	if (msgHeader->cc == NULL) {
		show_cc_field(msgfields, FALSE);
	}
	if (mailAction != READ_MAIL) {
		return FALSE;
	}

	/*Retaining the persistence of hide/unhide headers across next/prev 
	 * messages*/
	if (viewer_info->mail_info->same_appview == TRUE)
		hide_headers(NULL, NULL, viewer_info);

	/*setting the heigth of the headers */
	set_header_field_heigth(viewer_info);

	return TRUE;
}

/**
 This function is used to update the viewer info into the global open windows
 structure.
 
 @param msgid is the unique msgid of the mail.
 @param viewerInfo is the pointer to the ViewerInfo structure of the current viewer.
  No memory freeing of viewerInfo is done .
 @return void.
*/
static void ui_viewer_register_in_openwindow(const gchar * msgid, ViewerInfo * viewerInfo)
{
	OpenedWindowData *open_window = NULL;

	CHECK_PARAM(msgid == NULL, "Input  param check failed\n", return);
	CHECK_PARAM(viewerInfo == NULL, "Input  param check failed\n", return);

	email_main_view->tags->window_count++;
	open_window = g_new0(OpenedWindowData, 1);
	/*Fix for bug #19576,same msgid for child & parent viewer was causing the bug.
	 * msgid is made unique for email attachments*/
	if (viewerInfo->mail_info->email_attachment == FALSE) {
		open_window->msgid = g_strdup(msgid);
		open_window->oldmsgid = g_strdup(msgid);
	} else {
		open_window->msgid = g_strdup_printf("%d", email_main_view->tags->window_count);
		open_window->oldmsgid = g_strdup(msgid);
	}

	open_window->window_ptr = (gpointer) viewerInfo->viewer_appview;
	open_window->msgNum = email_main_view->tags->window_count;
	open_window->iseditor = FALSE;
	email_main_view->open_windows =
	    g_slist_insert(email_main_view->open_windows, open_window, 0);
	open_window->zoom_percentage = viewerInfo->zoominfo->previous_level;
	viewerInfo->mail_info->msgnum = email_main_view->tags->window_count;
	return;
}

/**Dimming the next prev tool buttons if next/prev mesage not available

 @param viewerInfo is the pointer to the ViewerInfo structure of the current viewer.
  No memory freeing of viewerInfo is done .

 @return void.
*/
static void check_next_prev_dimming(const ViewerInfo * viewer_info)
{
	HeaderInfo *viewInfo = NULL;

	if (viewer_info == NULL) {
		return;
	}
	if (viewer_info->mail_info == NULL) {
		return;
	}
	if (viewer_info->mail_info->set_prev_settings == TRUE) {
		return;
	}
	viewInfo = g_new0(HeaderInfo, 1);
	viewInfo->rowNum = (viewer_info->mail_info->row_num + NEXT);
	viewInfo->folderNum = viewer_info->mail_info->folder_num;
	if (ui_get_row_information(viewInfo) == FALSE) {
		gtk_widget_set_sensitive(viewer_info->toolbar->nextMsg, FALSE);
		gtk_widget_set_sensitive(viewer_info->menu->next_msg, FALSE);
	}
	ui_freeHeaderInfo(viewInfo);
	viewInfo = NULL;
	viewInfo = g_new0(HeaderInfo, 1);
	viewInfo->rowNum = (viewer_info->mail_info->row_num - PREV);
	viewInfo->folderNum = viewer_info->mail_info->folder_num;
	if (ui_get_row_information(viewInfo) == FALSE) {
		gtk_widget_set_sensitive(viewer_info->toolbar->prevMsg, FALSE);
		gtk_widget_set_sensitive(viewer_info->menu->prev_msg, FALSE);

	}
	ui_freeHeaderInfo(viewInfo);
	return;
}

/**
 Function to check if entire mail with attachments is downloaded

 @param mimeInfo is a MimeInfo structure pointer for the current mail.
     No memory freeing of mime_info is done .
 @return TRUE-if downloaded
       FALSE-else.
*/
gboolean check_attachments_downloaded(const MimeInfo * mime_info)
{
	MimeInfo *tempMimeInfo = NULL;
	gboolean downloaded = TRUE;
	for (tempMimeInfo = (MimeInfo *) mime_info; tempMimeInfo != NULL;
	     tempMimeInfo = ui_mimeinfo_next(tempMimeInfo)) {
		if (((tempMimeInfo->content_disposition != NULL)
		     && (g_ascii_strcasecmp(tempMimeInfo->content_disposition, ATTACHMENT) == 0))
		    || (tempMimeInfo->mime_type == MIME_MESSAGE_RFC822)) {
			if (tempMimeInfo->is_downloaded == FALSE) {
				downloaded = FALSE;
				break;
			}
		}
	}
	return downloaded;
}

/**
 Function to update the body /fully downloaded flag when a message is opened
 
 @param Header is a pointer to the HeaderInfo structure.
  No memory freeing for Header is done .
 @param mimeInfo is a MimeInfo structure pointer for the current mail.
  No memory freeing for mimeInfo is done .

 @return void 
*/
void update_fullydownloaded_flag(const HeaderInfo * msgHeader, const MimeInfo * mimeInfo)
{
	GtkTreeModel *model = NULL;
	gboolean fullydownloaded = FALSE;
	guint msgid_hash = 0;

	model = ui_get_unsorted_folder_model(msgHeader->folderNum);

	if (msgHeader == NULL && mimeInfo == NULL) {
		ULOG_DEBUG("No msgid to update-its a NEW MAIL!!\n");
		return;
	}
	if (mimeInfo == NULL) {
		return;
	}

	fullydownloaded = check_attachments_downloaded(mimeInfo);
	msgid_hash = g_str_hash(msgHeader->msgid);

	if (fullydownloaded == TRUE && msgHeader->email_attachment == FALSE) {
		gtk_tree_model_foreach(model, ui_update_msg_fully_downloaded, &msgid_hash);
	}
	if (msgHeader != NULL && msgHeader->email_attachment == FALSE) {
		if ((msgHeader->editable_status == FALSE)
		    || msgHeader->original_in_reply == TRUE) {
			gtk_tree_model_foreach(model, ui_update_msg_body_downloaded, &msgid_hash);
		}
	}
	return;
}

/**
  This is callback function that gets called whenever any viewer window
  gets topped by Task Navigator.

 @param  view  Pointer to the view that got topped
 @param  data  User data (NULL)

 @return void
*/
void ui_view_topped_cb(const HildonAppView * view, gpointer data)
{
	gpointer view_data = NULL;
	gchar *msg_id = NULL;
	ULOG_DEBUG_L("starts here\n");

	view_data = g_object_get_data(G_OBJECT(view), VIEW_MSG_ID);
	msg_id = g_strdup((gchar *) view_data);
	ULOG_DEBUG_L("Topped view msg_id : %s\n", msg_id);
	ui_tasknav_top_msg_cb(msg_id);

	ULOG_DEBUG_L("ends here\n");
	return;
}


/**
 fills the signal ids into the signalid structure.
 @param viewer_info is the ViewerInfo structure pointer.
 @return void
 */
static void fill_signalid_structure(ViewerInfo * viewer_info)
{
	SignalId *signal_id = NULL;

	signal_id = viewer_info->signal_id;
	signal_id->close_signalid = close_signalid;
	signal_id->update_mimeinfo_signalid = update_mimeinfo_signalid;
	signal_id->update_msginfo_signalid = update_msginfo_signalid;
	signal_id->update_progressbar_signalid = update_progressbar_signalid;
	signal_id->increase_zoom_signalid = increase_zoom_signalid;
	signal_id->decrease_zoom_signalid = decrease_zoom_signalid;
	signal_id->fullscreen_signalid = fullscreen_signalid;
	signal_id->state_save_signalid = state_save_signalid;
	signal_id->purge_icons_signalid = purge_icons_signalid;
	signal_id->resend_signalid = resend_signalid;
	signal_id->cancel_progressbar_signalid = cancel_progressbar_signalid;
	return;
}

/**
  assings the signal  to the global ids which will be disconnected
   for next/prev messages
 @param viewer_info is the ViewerInfo structure pointer.
 @return void
 */
static void assing_signalid_to_global_ids(ViewerInfo * viewer_info)
{

	SignalId *signal_id = NULL;

	signal_id = viewer_info->signal_id;
	close_signalid = signal_id->close_signalid;
	update_mimeinfo_signalid = signal_id->update_mimeinfo_signalid;
	update_msginfo_signalid = signal_id->update_msginfo_signalid;
	update_progressbar_signalid = signal_id->update_progressbar_signalid;
	increase_zoom_signalid = signal_id->increase_zoom_signalid;
	decrease_zoom_signalid = signal_id->decrease_zoom_signalid;
	fullscreen_signalid = signal_id->fullscreen_signalid;
	state_save_signalid = signal_id->state_save_signalid;
	purge_icons_signalid = signal_id->purge_icons_signalid;
	resend_signalid = signal_id->resend_signalid;
	cancel_progressbar_signalid = signal_id->cancel_progressbar_signalid;
	return;
}

/**
 Creation of new signal handlers for the viewer.
 such as close/increase_zoom/decrease_zoom/get_state_values/change_purge_icons.
 
 @param viewer_info is the ViewerInfo structure pointer.
 @return void
 
*/
static void create_new_signals(ViewerInfo * viewer_info, gboolean same_appview)
{

	GtkWidget *viewer = NULL;

	viewer = viewer_info->viewer_appview;
	hildon_app_set_appview(app, (HildonAppView *) viewer);
	if (signals_created == FALSE) {
		if (common_signals_created == FALSE) {
			/*signal for closing */
			g_signal_new("close", G_TYPE_FROM_INSTANCE(viewer),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);


			/*signal for increasing_zoom */
			g_signal_new("increase_zoom", G_TYPE_FROM_INSTANCE(viewer),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);

			/*signal for decreasing_zoom */
			g_signal_new("decrease_zoom", G_TYPE_FROM_INSTANCE(viewer),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);

			/*signal for toggling fullscreen */
			g_signal_new("toggle_fullscreen", G_TYPE_FROM_INSTANCE(viewer),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);

			/*signal for storing the state values */
			g_signal_new("get_state_values", G_TYPE_FROM_INSTANCE(viewer),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);

			/*updating the msginfo got thr callback */
			g_signal_new("update_msginfo", G_TYPE_FROM_INSTANCE(viewer),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
			common_signals_created = TRUE;
		}
		g_signal_new("cancel_progress", G_TYPE_FROM_INSTANCE(viewer),
			     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
			     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);

		/*signal for changing the purge icons from mainui */
		g_signal_new("change_purge_icons", G_TYPE_FROM_INSTANCE(viewer),
			     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
			     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
		/*updating the mime_info got thr callback */
		g_signal_new("update_mimeinfo", G_TYPE_FROM_INSTANCE(viewer),
			     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
			     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
		/*updating the text of the progress bar */
		g_signal_new("update_progressbar_text",
			     G_TYPE_FROM_INSTANCE(viewer),
			     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
			     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
		/*resend the message after creating the account */
		g_signal_new("resend",
			     G_TYPE_FROM_INSTANCE(viewer),
			     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
			     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
		signals_created = TRUE;
	} else if (same_appview == TRUE) {

		/*diconnect the already connected handlers. */
		g_signal_handler_disconnect(viewer, close_signalid);
		g_signal_handler_disconnect(viewer, update_mimeinfo_signalid);
		g_signal_handler_disconnect(viewer, update_msginfo_signalid);
		g_signal_handler_disconnect(viewer, update_progressbar_signalid);
		g_signal_handler_disconnect(viewer, increase_zoom_signalid);
		g_signal_handler_disconnect(viewer, decrease_zoom_signalid);
		g_signal_handler_disconnect(viewer, fullscreen_signalid);
		g_signal_handler_disconnect(viewer, state_save_signalid);
		g_signal_handler_disconnect(viewer, purge_icons_signalid);
		g_signal_handler_disconnect(viewer, resend_signalid);
		g_signal_handler_disconnect(viewer, cancel_progressbar_signalid);
	}
	close_signalid = 0;
	increase_zoom_signalid = 0;
	decrease_zoom_signalid = 0;
	fullscreen_signalid = 0;
	state_save_signalid = 0;
	purge_icons_signalid = 0;
	update_mimeinfo_signalid = 0;
	update_msginfo_signalid = 0;
	update_progressbar_signalid = 0;
	cancel_progressbar_signalid = 0;

	close_signalid =
	    g_signal_connect(G_OBJECT(viewer), "close", G_CALLBACK(ui_viewer_close_cb),
			     viewer_info);

	increase_zoom_signalid =
	    g_signal_connect(G_OBJECT(viewer), "increase_zoom",
			     G_CALLBACK(increase_zoom_view), viewer_info->zoominfo);


	decrease_zoom_signalid =
	    g_signal_connect(G_OBJECT(viewer), "decrease_zoom",
			     G_CALLBACK(decrease_zoom_view), viewer_info->zoominfo);


	fullscreen_signalid =
	    g_signal_connect(G_OBJECT(viewer), "toggle_fullscreen",
			     G_CALLBACK(ui_viewer_toggle_to_fullscreen), viewer_info);

	state_save_signalid =
	    g_signal_connect(G_OBJECT(viewer), "get_state_values",
			     G_CALLBACK(ui_viewer_get_state_values_cb), viewer_info);

	purge_icons_signalid =
	    g_signal_connect(G_OBJECT(viewer), "change_purge_icons",
			     G_CALLBACK(change_purged_icon), viewer_info);

	update_mimeinfo_signalid =
	    g_signal_connect(G_OBJECT(viewer), "update_mimeinfo",
			     G_CALLBACK(ui_set_body_thr_cb), viewer_info);
	update_msginfo_signalid =
	    g_signal_connect(G_OBJECT(viewer), "update_msginfo",
			     G_CALLBACK(ui_viewer_get_msginfo_thr_cb), viewer_info);
	update_progressbar_signalid =
	    g_signal_connect(G_OBJECT(viewer), "update_progressbar_text",
			     G_CALLBACK(update_progress_bar_text), viewer_info);
	resend_signalid =
	    g_signal_connect(G_OBJECT(viewer), "resend", G_CALLBACK(ui_viewer_resend_cb),
			     viewer_info);
	cancel_progressbar_signalid =
	    g_signal_connect(G_OBJECT(viewer), "cancel_progress", G_CALLBACK(cancel_progressbar),
			     viewer_info);
  /**fills the signal ids into the signalid structure*/
	fill_signalid_structure(viewer_info);
	return;
}

/**
  This function is called by main list view to create the UI for the viewer window
  for e-mail.

 @param Header is a pointer to the HeaderInfo structure.
   No memory freeing for msgHeader is done .
 @return pointer to the viewer window widget created.
*/
GtkWidget *ui_viewer_create(HeaderInfo * msgHeader)
{
	HildonAppView *viewer = NULL;
	gboolean success = FALSE;
	ViewerInfo *viewer_info = NULL;

	ULOG_DEBUG_F(" Entered");
	if (msgHeader == NULL) {
		osso_log(LOG_ERR, "Input param failed \n");
		return NULL;
	}

 /**Allocating memory for the viewer_info structure & its sub fields.*/
	viewer_info = ui_create_viewer_info();
	if (viewer_info == NULL) {
		return NULL;
	}
	/*Filling the  appropriate  fields of the viewer_info from the incoming msgHeader */
	success = ui_fill_viewerinfo(viewer_info, msgHeader);
	if (success == FALSE) {
		return NULL;
	}


	/*durning next previous we should open in the same viewer.so when ever a viewer is opened
	 * a check for already any open viewer exists*/
	if (email_main_view->widgets->opened_viewer == NULL) {
		/*creation of the viewer appview */
		viewer = HILDON_APPVIEW(hildon_appview_new(NULL));

		/*Set full screen short cut key */
		hildon_appview_set_fullscreen_key_allowed(viewer, TRUE);

		gtk_widget_show((GtkWidget *) viewer);
		hildon_app_set_appview(app, HILDON_APPVIEW(viewer));
	} else {
		viewer = g_object_ref(email_main_view->widgets->opened_viewer);
		email_main_view->widgets->opened_viewer = NULL;
		viewer_info->mail_info->same_appview = TRUE;
	}
	if (msgHeader->subject != NULL) {
		hildon_appview_set_title(viewer, msgHeader->subject);
	}
	viewer_info->viewer_appview = (GtkWidget *) viewer;


  /**Create the menubar for the viewer*/
	ui_viewer_create_menubar(viewer_info);

  /**Creating the internal laypout for the viewer*/
	ui_viewer_create_layout(viewer_info, msgHeader);

  /**Initialising the zoom structure & Creating the internal laypout for the viewer*/
	viewer_info->zoominfo =
	    set_initial_zoom_level((GtkTextView *) viewer_info->msg_fields->body,
				   viewer_info->msg_fields->mail_viewer,
				   viewer_info->msgid, DEFAULT_ZOOM);
	/*part of intenal bug fix.Previouly zoom value was take from label text of menu.
	 * This was causing bugs with other languages.fix is done to take the value from 
	 * the menu being clicked*/
	set_zoominfo_menus(viewer_info->zoominfo, viewer_info->menu);
	create_new_signals(viewer_info, viewer_info->mail_info->same_appview);

	/*create find tool bar */
	create_find_toolbar(viewer_info->msg_fields, viewer->vbox, viewer_info->menu->find);

	/*Creation of the toolbar */
	ui_viewer_create_toolbar(viewer_info, msgHeader->body_downloaded);

	gtk_widget_set_sensitive(viewer_info->menu->copy, TRUE);
	gtk_widget_set_sensitive(viewer_info->menu->find, TRUE);


	g_signal_connect(G_OBJECT(viewer_info->menu->zoom_out_level),
			 "activate", G_CALLBACK(on_zoom_activate), viewer_info->zoominfo);
	g_signal_connect(G_OBJECT(viewer_info->menu->zoom_out_max_level),
			 "activate", G_CALLBACK(on_zoom_activate), viewer_info->zoominfo);
	g_signal_connect(G_OBJECT(viewer_info->menu->current_zoom_level),
			 "activate", G_CALLBACK(on_zoom_activate), viewer_info->zoominfo);
	g_signal_connect(G_OBJECT(viewer_info->menu->zoom_in_level),
			 "activate", G_CALLBACK(on_zoom_activate), viewer_info->zoominfo);
	g_signal_connect(G_OBJECT(viewer_info->menu->zoom_80),
			 "activate", G_CALLBACK(on_zoom_activate), viewer_info->zoominfo);
	g_signal_connect(G_OBJECT(viewer_info->menu->zoom_120),
			 "activate", G_CALLBACK(on_zoom_activate), viewer_info->zoominfo);
	/*Store window data in global list */
	ui_viewer_register_in_openwindow(msgHeader->msgid, viewer_info);
	email_main_view->tags->main_view_topped = FALSE;

	/*Fix for bug #19576,same msgid for child & parent viewer was causing the bug.
	 * msgid is made unique for email attachments*/
	if (viewer_info->mail_info->email_attachment == FALSE)
		g_object_set_data_full(G_OBJECT(viewer), VIEW_MSG_ID,
				       g_strdup(msgHeader->msgid), (GDestroyNotify) g_free);
	else
		g_object_set_data_full(G_OBJECT(viewer), VIEW_MSG_ID,
				       g_strdup_printf("%d", email_main_view->tags->window_count),
				       (GDestroyNotify) g_free);
	/*Connect switched_to signal for appview */
	g_signal_connect(G_OBJECT(viewer), "switched_to", G_CALLBACK(ui_view_topped_cb), NULL);

	/*set the fullscreen mode */
	hildon_appview_set_fullscreen((HildonAppView *) viewer,
				      email_main_view->tags->full_screen_mode);

	/*Set the message as IN_USE */
	if (viewer_info->mail_info->email_attachment == FALSE)
		ui_update_row_tag_in_folder(msgHeader->folderNum, msgHeader->msgid, IN_USE_STATUS,
					    TRUE);


	success = ui_viewer_set_contents(viewer_info, msgHeader);
	if (success == FALSE) {
		ui_free_viewerinfo(viewer_info);
		hildon_app_set_appview(app, HILDON_APPVIEW(appview));
		ui_update_row_tag_in_folder(msgHeader->folderNum, msgHeader->msgid, IN_USE_STATUS,
					    FALSE);
		return NULL;
	}

	/** Setting the initial zoom level*/
	set_inherited_zoom(-1, viewer_info->zoominfo, msgHeader->zoomlevel);

	/*restore the state saved */
	if (msgHeader->set_prev_settings == TRUE) {
		restore_viewer_saved_state(viewer_info);
	}

	check_next_prev_dimming(viewer_info);

	/*call the recv message body */
	if (!viewer_info->mime_info) {
		if (call_recv_msg_body(msgHeader, viewer_info) == FALSE) {
			hildon_app_set_appview((HildonApp *) app, (HildonAppView *) appview);
			ui_free_viewerinfo(viewer_info);
			ui_update_row_tag_in_folder(msgHeader->folderNum, msgHeader->msgid,
						    IN_USE_STATUS, FALSE);
			return NULL;
		}
	}

	set_header_field_heigth(viewer_info);
	/*hiding the widgets */
	hide_header_widgets(viewer_info);

	ULOG_DEBUG_F("ends");
	return ((GtkWidget *) viewer);
}

/**
 This is a callback function which updates the
 progressbar.
 @param mail_viewer is a pointer to the MailViewer widget.
 @param frac is the gfloat representing the fraction to update in progressbar.
 @param viewer_info is the ViewerInfo structure pointer.
 @return void.
*/
static void update_body_load_progress(GtkWidget * mail_viewer, gfloat frac,
				      ViewerInfo * viewer_info)
{
	if (viewer_info->mail_info->progress_on == FALSE)
		return;

	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(viewer_info->toolbar->progressbar), frac);
	if (frac == 1)
		hide_progressbar(viewer_info);
	return;
}

/**
 This is a time out function which updates the
 progressbar.
 @param viewer_info is the ViewerInfo structure pointer.
 @return TRUE-enable the periodic updation.
         FALSe-disable the time out source.
*/
static gboolean update_progress(ViewerInfo * viewer_info)
{
	gdouble value = 0;
	if (viewer_info->mail_info->progress_on == FALSE)
		return FALSE;

	value = gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(viewer_info->toolbar->progressbar));
	value = value + 0.1;
	/*if end of progress bar is reached ,again start from beginning */
	if (value > 1.0)
		value = 0.1;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(viewer_info->toolbar->progressbar), value);
	return TRUE;
}


/**
 This is a callback function when a link is clicked in viewer
 
 @param mail_viewer is a pointer to the MailViewer widget.
 @gchar uri is the gchar pointer to the hyperlink.
 @param viewer_info is the ViewerInfo structure pointer.
 @return void.
*/
static void hyperlink_clicked(GtkWidget * mail_viewer, const gchar * uri, OssoURIAction * action,
			      ViewerInfo * viewer_info)
{
	GSList *actions_list = NULL;
	OssoURIAction *default_action = NULL;

	if (!uri)
		return;

	/*if csm is activated */
	if (action) {
		if (g_ascii_strncasecmp(osso_uri_action_get_name(action), "weba_ap_web_browser",
			                                    strlen("weba_ap_web_browser")) == 0) {
                      /*lauch add to bookmarks */
			add_to_bookmark(uri);
			return;
               }
		osso_uri_open(uri, action, NULL);
		return;
	}

	/*when link is clicked */
	/*for email id the default action is add to contacts but launch the editor */
	if ((g_ascii_strncasecmp(uri, "mailto:", strlen("mailto:")) == 0) ||
	    ui_accmgmt_email_isvalid(uri)) {
		launch_editor(uri);
	} else {		/*get default action & call open_uri */
		gchar *scheme = osso_uri_get_scheme_from_uri(uri, NULL);

		if (scheme)
			actions_list = osso_uri_get_actions(scheme, NULL);
		g_free(scheme);
		if (!actions_list)
			return;

		default_action = (OssoURIAction *) actions_list->data;
		osso_uri_open(uri, default_action, NULL);
		osso_uri_free_actions(actions_list);
	}

	return;
}

/**
 This is a callback function when a copy is clicked in viewer csm

 @param mail_viewer is a pointer to the MailViewer widget.
 @gchar uri is the gchar pointer to the hyperlink.
 @return void.
*/
static void copy_hyperlink_clicked(GtkWidget * mail_viewer, const gchar * uri)
{
	GtkClipboard *clipboard = gtk_clipboard_get(GDK_NONE);
	if (!uri)
		return;

	/*if uri is mailid & has a mailto: remove it b4 pasting */
	if (strchr(uri, AT_THE_RATE) && g_ascii_strncasecmp(uri, "mailto:", strlen("mailto:")) == 0)
		gtk_clipboard_set_text(clipboard, uri + strlen("mailto:"),
				       strlen(uri) - strlen("mailto:"));
	else
		gtk_clipboard_set_text(clipboard, uri, strlen(uri));

	gtk_confirmation_banner_with_icon_name(GTK_WINDOW(app),
					       _("mail_ib_copied_to_clipboard"),
					       "qgn_note_infoprint");
	return;
}
