/**
 * @file accountMgmt_callback.c 
 *
 * This file contains the implementaions of  Account Management.
 * It contains all callback functions related to Account Management.
 *
 * Copyright (c) 2005 Nokia. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */


/* Inclusion of Header file which contains all standerd paths.*/
#include "ui_common.h"

/* Inclusion of AccountManagement Header Files */
#include"accountMgmt_callback.h"
#include"accountMgmt.h"
#include "ui_interface.h"
#include "ui_account_store.h"


/* Inclusion of UI_DBUS header files */
#include "email_mgr.h"

/* Inclusion of Signature Header files*/
#include "enablesignatures_callbacks.h"
#include "cl.h"

/* External Variables */

extern MainViewInfo *email_main_view;	/* Declared in ui_interface.c */
extern GSList *ui_accMgmt_listptr;	/* Declared in accountMgmt.c */
extern GSList *g_update_account_list;	/* Declared in accountMgmt.c */
extern GtkWidget *currenteditor;	/* Declared in editInterface.c */
extern GtkWidget *currentviewer;	/* Declared in viewer.c */
extern GtkListStore *folderListStore;	/* Declared in accountMgmt.c */
extern AccountMgtMngr *gActMgtMngr;	/* Declared in accountMgmt.c */


static void ui_accmgmt_save_serveraddress(void);
static gboolean ui_accmgmt_setup2_validation(void);
static void ui_accmgmt_savesetup3(void);
static void ui_accmgmt_save_setup2(void);
static void ui_accmgmt_update_liststore(void);
static void freeglobalmemory(void);
static gboolean ui_accmgmt_emptycheck_onsetup1(gchar * account_name);
static gboolean ui_accMgmt_userinfo_isvalid(gchar * userinfo, gboolean acct_flag);
static void ui_accmgmt_copy_account_setting(SaveSetup saveSetup);
static gboolean is_setup2_settings_modified(GtkWidget * parentWindow);
static gboolean is_setup3_settings_modified(GtkWidget * parent_widget);
static void ui_accmgmt_save_advancedsetting(AdvOptionViewInfo * advOptionViewInfo);
gboolean is_setup3_settings_incomplete(GtkWidget * parent_dialog);
static void ui_accmgmt_save_advancedsettings_checkbuttons(AdvOptionViewInfo * advOptionViewInfo);
void ui_accmgmt_default_advancedvalues(void);
void ui_accmgmt_save_advancedvalues(void);
static GtkWidget *ui_accmgmt_dialogmessagedelete(GtkWidget * window);
#if CL_USE
static gboolean search_iap_name_iaplist(const gchar * iap_name);
#endif


/**
This is a callback function called upon text insertion in account name entry field of 1/4 account
setup dialog.
It checks for character count in the entry field and if count exceeds the maximum value, the inserted text is deleted with invoke of an infobanner.

@param widget pointer to GtkEditable on which it is called.
@param text is the inserted text.
@param len is length of text.
@param position after the text entry.
@param data is unused.

@return void
*/
void check_update_accname_entry(GtkEditable * editable,
				gchar * text, gint len, gint * position, gpointer data)
{
	const gchar *entry_text = NULL;

	entry_text = gtk_entry_get_text(GTK_ENTRY(editable));
	if (strlen(entry_text) > MAX_TEXT_ENTRY_LEN) {
		gtk_infoprint(GTK_WINDOW(app), _("mcen_ib_maxchar_reached"));
		/*delete the latest inserted text */
		gtk_editable_delete_text(editable, *position - len, *position);
		/*move the cursor to earlier position */
		*position = *position - len;
	}
}

/**
This function is a callback function which will be called by Account Management
module. It changs the sensitivity status of the 'Next' button in setup1 dialog. 

@param widget pointer to GtkEditable on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_acctentrychanged_cb(GtkEditable * editable, gpointer user_data)
{

	GtkWidget *entry1 = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *finish_button = NULL;
	GtkWidget *parent_dialog = NULL;
	const gchar *accname = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	/*Check the input params */
	if ((editable == NULL) || (user_data == NULL)) {
		return;
	}

	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 1)->data);
	next_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonNext");
	finish_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonFinish");
	entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "accountNameEntry");

	gtk_widget_grab_focus((GtkWidget *) user_data);
	accname = gtk_entry_get_text((GtkEntry *) user_data);
	gtk_widget_grab_focus((GtkWidget *) user_data);

	if ((accname == NULL) || (g_utf8_strlen(accname, -1) == 0)) {
		gtk_widget_set_sensitive(next_button, FALSE);
		gtk_widget_set_sensitive(finish_button, FALSE);
		gtk_widget_grab_focus((GtkWidget *) user_data);
	} else {
		gtk_widget_set_sensitive(next_button, TRUE);
		if (gActMgtMngr->edit_operation == TRUE) {
			gtk_widget_set_sensitive(finish_button, TRUE);
		}
	}
}

/**
This function is a callback function which will be called by Account Management
module. It changes the sensitivity status of the 'Finish' buttons. 

@param editable pointer to GtkEditable on which it is called.
@param user_data gpointer variable contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_set_finish_button(GtkEditable * editable, gpointer user_data)
{
	/* Variables Declaration */
	GtkWidget *setup1_dialog = NULL;
	GtkWidget *setup2_dialog = NULL;
	GtkWidget *setup3_dialog = NULL;
	GtkWidget *account_entry = NULL;
	GtkWidget *username_entry = NULL;
	GtkWidget *email_entry = NULL;
	GtkWidget *recvserver_entry = NULL;
	GtkWidget *smtpserver_entry = NULL;
	GtkWidget *setup1_finish = NULL;
	GtkWidget *setup2_finish = NULL;
	GtkWidget *setup3_finish = NULL;
	gchar *account_name = NULL;
	gchar *user_name = NULL;
	gchar *emailid = NULL;
	gchar *ipserver = NULL;
	gchar *opserver = NULL;

	/*Input Param Checking */

	CHECK_PARAM((editable == NULL), "Invalid input Parameter", return);

	/*If the respective dialogs are already created. */
	if ((g_slist_nth(ui_accMgmt_listptr, 1) != NULL)
	    && (g_slist_nth(ui_accMgmt_listptr, 2) != NULL)
	    && (g_slist_nth(ui_accMgmt_listptr, 3) != NULL)) {

		setup1_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 1)->data);
		setup2_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);
		setup3_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 3)->data);

		/*Getting widget pointers */
		account_entry = EMAIL_UI_LOOKUP_OBJECT(setup1_dialog, "accountNameEntry");
		username_entry = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "userNameEntry");
		email_entry = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "emailEntry");
		recvserver_entry = EMAIL_UI_LOOKUP_OBJECT(setup3_dialog, "inputServerAddress");
		smtpserver_entry = EMAIL_UI_LOOKUP_OBJECT(setup3_dialog, "outputServerAddress");
		setup1_finish = EMAIL_UI_LOOKUP_OBJECT(setup1_dialog, "buttonFinish");
		setup2_finish = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "buttonFinish");
		setup3_finish = EMAIL_UI_LOOKUP_OBJECT(setup3_dialog, "buttonFinish");

		account_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(account_entry)));
		user_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(username_entry)));
		emailid = g_strdup(gtk_entry_get_text(GTK_ENTRY(email_entry)));
		ipserver = g_strdup(gtk_entry_get_text(GTK_ENTRY(recvserver_entry)));
		opserver = g_strdup(gtk_entry_get_text(GTK_ENTRY(smtpserver_entry)));

		/*If any mandatory fields is empty , make the finish button dimmed. */
		if (account_name == NULL || user_name == NULL || emailid == NULL
		    || ipserver == NULL || opserver == NULL
		    || g_utf8_strlen(account_name, -1) == 0
		    || g_utf8_strlen(user_name, -1) == 0
		    || g_utf8_strlen(emailid, -1) == 0
		    || g_utf8_strlen(ipserver, -1) == 0 || g_utf8_strlen(opserver, -1) == 0) {

			gtk_widget_set_sensitive(setup1_finish, FALSE);
			gtk_widget_set_sensitive(setup2_finish, FALSE);
			gtk_widget_set_sensitive(setup3_finish, FALSE);
		}

		/*If all the mandatory fields are filled , make the finish buttons sensitive. */
		else {
			gtk_widget_set_sensitive(setup1_finish, TRUE);
			gtk_widget_set_sensitive(setup2_finish, TRUE);
			gtk_widget_set_sensitive(setup3_finish, TRUE);
		}
	} else if (g_slist_nth(ui_accMgmt_listptr, 2) != NULL) {
		setup1_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 1)->data);
		setup2_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);
		setup1_finish = EMAIL_UI_LOOKUP_OBJECT(setup1_dialog, "buttonFinish");
		setup2_finish = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "buttonFinish");

		username_entry = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "userNameEntry");
		email_entry = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "emailEntry");
		user_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(username_entry)));
		emailid = g_strdup(gtk_entry_get_text(GTK_ENTRY(email_entry)));

		if (g_utf8_strlen(user_name, -1) == 0 || g_utf8_strlen(emailid, -1) == 0) {
			gtk_widget_set_sensitive(setup1_finish, FALSE);
			gtk_widget_set_sensitive(setup2_finish, FALSE);

		} else {
			if (gActMgtMngr->edit_operation == TRUE) {
				gtk_widget_set_sensitive(setup1_finish, TRUE);
				gtk_widget_set_sensitive(setup2_finish, TRUE);
			}
		}

	}

}

/**
  This is a callback function. This function is used to make chnage 
  the sensitivity of the Finish button in Advance window.

  @param editable which holds the pointer to the widget.
  @param user_data which holds the pointer to the parent dialog
  @return void
  */
void ui_accmgmt_user_name_email_changed_cb(GtkEditable * editable, gpointer user_data)
{
	/*Variable declarations. */
	GtkWidget *entry_username = NULL;
	GtkWidget *entry_email = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *finish_button = NULL;
	GtkWidget *parent_dialog = NULL;
	GtkWidget *passwd_entry = NULL;
	gchar *user_name = NULL, *email = NULL;

	/*Input Param Checking */
	CHECK_PARAM((editable == NULL), "Invalid input Parameter", return);
	CHECK_PARAM((user_data == NULL), "Invalid input Parameter", return);

	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);
	next_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonNext");
	finish_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonFinish");
	entry_username = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "userNameEntry");
	entry_email = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "emailEntry");
	passwd_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "passWdEntry");

	user_name = g_strdup(gtk_entry_get_text((GtkEntry *) entry_username));
	email = g_strdup(gtk_entry_get_text((GtkEntry *) entry_email));

	/*Checking for empty UserName and EmailAddress. */
	if ((g_utf8_strlen(user_name, -1) == 0) || (g_utf8_strlen(email, -1) == 0)) {
		gtk_widget_set_sensitive(next_button, FALSE);
		gtk_widget_set_sensitive(finish_button, FALSE);
	} else {
		gtk_widget_set_sensitive(next_button, TRUE);
		if (gActMgtMngr->edit_operation == TRUE) {
			gtk_widget_set_sensitive(finish_button, TRUE);
		}
	}
	g_free(user_name);
	g_free(email);
}

/**
 This is a callback function. This function is used to make chnage 
 the sensitivity of the  buttons in Account Setup-2 window.
        
 @param imcontext it hold sthe pointer to the GtkIMContext.
 @param arg1 which holds the pointer to the string.
 @param user_data which holds the pointer to the parent dialog
 @return void
 */
void ui_accmgmt_usrNameEmailCommit_cb(GtkIMContext * imcontext, gchar * arg1, gpointer user_data)
{
	GtkEntry *entry_username = NULL;
	GtkEntry *entry_email = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *finish_button = NULL;
	GtkWidget *parent_dialog = NULL;
	gchar *user_name = NULL, *email = NULL;
	userSettingFlag *user_setting_flag = NULL;


	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);
	next_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonNext");
	finish_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonFinish");
	entry_username = (GtkEntry *) EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "userNameEntry");
	entry_email = (GtkEntry *) EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "emailEntry");

	user_setting_flag = g_object_get_data(G_OBJECT(parent_dialog), "UserSettingFlag");

	if (strcmp(user_data, "USERNAME") == 0) {
		gtk_im_context_get_preedit_string((GtkIMContext *) (entry_username->
								    im_context),
						  &user_name, NULL, NULL);
		if (g_utf8_strlen(user_name, -1) != 0) {
			user_setting_flag->username_commit = TRUE;
		} else {
			user_setting_flag->username_commit = FALSE;
		}
	} else if (strcmp(user_data, "EMAIL") == 0) {
		gtk_im_context_get_preedit_string((GtkIMContext *) (entry_email->
								    im_context), &email,
						  NULL, NULL);
		if (g_utf8_strlen(email, -1) != 0) {
			user_setting_flag->email_commit = TRUE;
		} else {
			user_setting_flag->email_commit = FALSE;
		}
	}
}

/**
 This is a callback function. This function is used to make change 
 the sensitivity of the buttons in Account Setup2 dialog.
 
 @param editable which holds the pointer to the widget.
 @param user_data  which holds the pointer to the parent dialog
 @return void
*/

void ui_accmgmt_usrNameEmailChanged_cb(GtkEditable * editable, gchar * user_data)
{
	/* Variable declarations. */
	GtkEntry *entry_username = NULL;
	GtkEntry *entry_email = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *finish_button = NULL;
	GtkWidget *parent_dialog = NULL;
	GtkWidget *passwd_entry = NULL;
	gchar *user_name = NULL, *email = NULL;
	const gchar *user_id = NULL, *email_id = NULL;
	userSettingFlag *user_setting_flag = NULL;

	/* Input Param Checking */
	CHECK_PARAM((editable == NULL), "Invalid input Parameter", return);
	CHECK_PARAM((user_data == NULL), "Invalid input Parameter", return);

	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);
	next_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonNext");
	finish_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonFinish");
	entry_username = (GtkEntry *) EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "userNameEntry");
	entry_email = (GtkEntry *) EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "emailEntry");
	passwd_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "passWdEntry");

	user_setting_flag = g_object_get_data(G_OBJECT(parent_dialog), "UserSettingFlag");

	/* If the signal is generated in UserName entry */

	if (strcmp(user_data, "USERNAME") == 0) {
		gtk_im_context_get_preedit_string((GtkIMContext *) (entry_username->
								    im_context),
						  &user_name, NULL, NULL);
		if (g_utf8_strlen(user_name, -1) != 0) {
			user_setting_flag->username_flag = TRUE;
		} else {
			user_setting_flag->username_flag = FALSE;
		}

	}

	/* If the signal is generated in Email Entry */
	else if (strcmp(user_data, "EMAIL") == 0) {
		gtk_im_context_get_preedit_string((GtkIMContext *) (entry_email->
								    im_context), &email,
						  NULL, NULL);
		if (g_utf8_strlen(email, -1) != 0) {
			user_setting_flag->email_flag = TRUE;
		} else {
			user_setting_flag->email_flag = FALSE;
		}
	}
	user_id = gtk_entry_get_text(GTK_ENTRY(entry_username));
	email_id = gtk_entry_get_text(GTK_ENTRY(entry_email));
	if (g_utf8_strlen(user_id, -1) != 0) {
		user_setting_flag->username_flag = TRUE;
	}
	if (g_utf8_strlen(email_id, -1) != 0) {
		user_setting_flag->email_flag = TRUE;
	}
	if (((user_setting_flag->email_flag == TRUE)
	     || (user_setting_flag->email_commit == TRUE))
	    && ((user_setting_flag->username_flag == TRUE)
		|| (user_setting_flag->username_commit == TRUE))) {
		gtk_widget_set_sensitive(next_button, TRUE);
		if (gActMgtMngr->edit_operation == TRUE) {
			gtk_widget_set_sensitive(finish_button, TRUE);
		}
	} else {
		gtk_widget_set_sensitive(next_button, FALSE);
		gtk_widget_set_sensitive(finish_button, FALSE);
	}
}

/**
 This is a callback function. This function is used to make chnage 
 the sensitivity of the Finish button in Advance window.
 
 @param editable which holds the pointer to the widget.
 @param parent_dialog  which holds the pointer to the parent dialog
 @return void
*/

void ui_accmgmt_account_name_preedit_changed_cb(GtkEditable * editable, gpointer data)
{
	GtkEntry *entry1 = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *finish_button = NULL;
	GtkWidget *parent_dialog = NULL;
	gchar *accname = NULL;
	const gchar *account_name = NULL;
	accountFlag *account_flag = NULL;

	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 1)->data);
	next_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonNext");
	finish_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonFinish");
	entry1 = (GtkEntry *) EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "accountNameEntry");
	account_flag = g_object_get_data(G_OBJECT(parent_dialog), "AccountFlag");

	gtk_im_context_get_preedit_string((GtkIMContext *) (entry1->im_context),
					  &accname, NULL, NULL);
	account_name = gtk_entry_get_text(GTK_ENTRY(entry1));
	if (g_utf8_strlen(accname, -1) != 0) {
		account_flag->account_preedit = TRUE;
	} else {
		account_flag->account_preedit = FALSE;
	}
	if (g_utf8_strlen(account_name, -1) != 0) {
		account_flag->account_preedit = TRUE;
	}
	if (account_flag->account_preedit == TRUE || account_flag->account_commit == TRUE) {
		gtk_widget_set_sensitive(next_button, TRUE);

	} else {
		gtk_widget_set_sensitive(next_button, FALSE);
	}
	g_free(accname);

}

/**
 This is a callback function. This function is used to make chnage 
 the sensitivity of the  buttons in Account Setup-1 window.
    
 @param imcontext it hold sthe pointer to the GtkIMContext.
 @param arg1 which holds the pointer to the string.
 @param user_data which holds the pointer to the parent dialog
 @return void
 */

void ui_accmgmt_account_name_commit_cb(GtkIMContext * imcontext, gchar * arg1, gpointer user_data)
{
	GtkEntry *account_entry = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *finish_button = NULL;
	GtkWidget *parent_dialog = NULL;
	gchar *account_name = NULL;
	accountFlag *account_flag = NULL;

	parent_dialog = GTK_WIDGET((g_slist_nth(ui_accMgmt_listptr, 1)->data));
	next_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonNext");
	finish_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonFinish");
	account_entry = GTK_ENTRY(EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "accountNameEntry"));
	account_flag = g_object_get_data(G_OBJECT(parent_dialog), "AccountFlag");

	gtk_im_context_get_preedit_string((GtkIMContext *) (account_entry->
							    im_context), &account_name, NULL, NULL);
	if (g_utf8_strlen(account_name, -1) != 0) {
		account_flag->account_commit = TRUE;
	} else {
		account_flag->account_commit = FALSE;
	}
	g_free(account_name);

}

/**
  This is a callback function. This function is used to make chnage 
  the sensitivity of the Finish button in Advance window.

  @param editable which holds the pointer to the widget.
  @param parent_dialog  which holds the pointer to the parent dialog
  @return void
*/
void ui_accmgmt_server_name_preedit_changed_cb(GtkEditable * editable, gchar * user_data)
{
	/* Variable declarations. */
	GtkEntry *inputserver_entry = NULL;
	GtkEntry *outputserver_entry = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *finish_button = NULL;
	gchar *input_server = NULL;
	const gchar *ip_server = NULL;
	const gchar *op_server = NULL;
	gchar *output_server = NULL;
	GtkWidget *parent_dialog = NULL;
	serverSettingFlag *server_setting_flag = NULL;


	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 3)->data);
	next_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonNext");
	finish_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonFinish");
	inputserver_entry =
	    (GtkEntry *) EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "inputServerAddress");
	outputserver_entry =
	    (GtkEntry *) EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "outputServerAddress");
	server_setting_flag = g_object_get_data(G_OBJECT(parent_dialog), "ServerSettingFlag");

	if (strcmp(user_data, "IPSERVER") == 0) {
		gtk_im_context_get_preedit_string((GtkIMContext *) (inputserver_entry->
								    im_context),
						  &input_server, NULL, NULL);
		if (g_utf8_strlen(input_server, -1) != 0) {
			server_setting_flag->ipserver_flag = TRUE;
		} else {
			server_setting_flag->ipserver_flag = FALSE;
		}
	} else if (strcmp(user_data, "OUTSERVER") == 0) {
		gtk_im_context_get_preedit_string((GtkIMContext *) (outputserver_entry->
								    im_context),
						  &output_server, NULL, NULL);

		if (g_utf8_strlen(output_server, -1) != 0) {
			server_setting_flag->opserver_flag = TRUE;
		} else {
			server_setting_flag->opserver_flag = FALSE;
		}
	}
	ip_server = gtk_entry_get_text(GTK_ENTRY(inputserver_entry));
	op_server = gtk_entry_get_text(GTK_ENTRY(outputserver_entry));
	if (g_utf8_strlen(ip_server, -1) != 0) {
		server_setting_flag->ipserver_flag = TRUE;
	}
	if (g_utf8_strlen(op_server, -1) != 0) {
		server_setting_flag->opserver_flag = TRUE;
	}
	if (((server_setting_flag->ipserver_flag == TRUE)
	     || (server_setting_flag->ipserver_commit == TRUE))
	    && ((server_setting_flag->opserver_flag == TRUE)
		|| (server_setting_flag->outputserver_commit == TRUE))) {
		gtk_widget_set_sensitive(next_button, TRUE);
		gtk_widget_set_sensitive(finish_button, TRUE);
	} else {
		gtk_widget_set_sensitive(next_button, FALSE);
		gtk_widget_set_sensitive(finish_button, FALSE);
	}
}

/**
 This is a callback function. This function is used to make chnage 
 the sensitivity of the Finish button in Advance window.
 
 @param editable which holds the pointer to the widget.
 @param user_entry which holds the pointer to the GtkEntry pointer
 @return void
 */
void ui_accmgmt_server_name_changed_cb(GtkEditable * editable, GtkWidget * server_entry)
{
	/* Variable declarations. */
	GtkWidget *inputserver_entry = NULL;
	GtkWidget *outputserver_entry = NULL;
	GtkWidget *next_button = NULL;
	GtkWidget *finish_button = NULL;
	gchar *input_server = NULL;
	gchar *output_server = NULL;
	GtkWidget *parent_dialog = NULL;


	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 3)->data);
	next_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonNext");
	finish_button = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "buttonFinish");
	inputserver_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "inputServerAddress");
	outputserver_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "outputServerAddress");

	input_server = g_strdup(gtk_entry_get_text(GTK_ENTRY(inputserver_entry)));
	output_server = g_strdup(gtk_entry_get_text(GTK_ENTRY(outputserver_entry)));
	gtk_widget_grab_focus((GtkWidget *) server_entry);

	/*Checking for empty input server address. */
	if ((input_server == NULL) || (g_utf8_strlen(input_server, -1) == 0)) {
		gtk_widget_set_sensitive(next_button, FALSE);
		gtk_widget_set_sensitive(finish_button, FALSE);
		return;
	} else {
		gtk_widget_set_sensitive(next_button, TRUE);
		gtk_widget_set_sensitive(finish_button, TRUE);
		g_free(input_server);
	}

	/* Checking for empty output server address. */
	if ((output_server == NULL) || (g_utf8_strlen(output_server, -1) == 0)) {
		gtk_widget_set_sensitive(next_button, FALSE);
		gtk_widget_set_sensitive(finish_button, FALSE);
		return;
	} else {
		gtk_widget_set_sensitive(next_button, TRUE);
		gtk_widget_set_sensitive(finish_button, TRUE);
		g_free(output_server);
	}
}

/**
 This is a callback function. This function is used to make chnage 
 the sensitivity of the  buttons in Account Setup-3 window.
 
 @param imcontext it hold sthe pointer to the GtkIMContext.
 @param arg1 which holds the pointer to the string.
 @param user_data which holds the pointer to the parent dialog
 @return void
 */
void ui_accmgmt_server_name_commit_cb(GtkIMContext * imcontext, gchar * arg1, gpointer user_data)
{
	GtkEntry *ipserver_entry = NULL;
	GtkEntry *opserver_entry = NULL;
	GtkWidget *parent_dialog = NULL;
	gchar *output_server = NULL;
	gchar *ipserver = NULL;
	serverSettingFlag *server_setting_flag = NULL;


	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 3)->data);
	server_setting_flag = g_object_get_data(G_OBJECT(parent_dialog), "ServerSettingFlag");
	ipserver_entry = (GtkEntry *) EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "inputServerAddress");
	opserver_entry = (GtkEntry *) EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "outputServerAddress");

	if (strcmp(user_data, "IPSERVER") == 0) {
		gtk_im_context_get_preedit_string((GtkIMContext *) (ipserver_entry->
								    im_context),
						  &ipserver, NULL, NULL);
		if (g_utf8_strlen(ipserver, -1) != 0) {
			server_setting_flag->ipserver_commit = TRUE;
		} else {
			server_setting_flag->ipserver_commit = FALSE;

		}
	} else if (strcmp(user_data, "OUTSERVER") == 0) {
		gtk_im_context_get_preedit_string((GtkIMContext *) (opserver_entry->
								    im_context),
						  &output_server, NULL, NULL);
		if (g_utf8_strlen(output_server, -1) != 0) {
			server_setting_flag->outputserver_commit = TRUE;
		} else {
			server_setting_flag->outputserver_commit = FALSE;
		}
	}

}

/**
    This is a callback function. This function is used to make chnage 
    the sensitivity of the Finish button in Advance window.

    @param editable which holds the pointer to the widget.
    @param user_data which holds the pointer to the parent dialog
    @return void
 */

void ui_accmgmt_smtpNamePwdChanged_cb(GtkEditable * editable, gpointer user_data)
{
	/*Variable declarations. */
	GtkWidget *entry_username = NULL;
	GtkWidget *entry_passwd = NULL;
	GtkWidget *finish_button = NULL;
	SMTPTab *smtpTab = NULL;
	AdvOptionButtons *advOptionButtons = NULL;
	const gchar *user_name = NULL, *passwd = NULL;
	AdvOptionViewInfo *advOptionViewInfo = NULL;

	/*Input param checking. */
	if ((editable == NULL) || (user_data == NULL)) {
		return;
	}
	advOptionViewInfo = (AdvOptionViewInfo *) user_data;
	smtpTab = (SMTPTab *) advOptionViewInfo->smtpTab;
	advOptionButtons = (AdvOptionButtons *) advOptionViewInfo->advOptionButtons;

	finish_button = advOptionButtons->button1;

	entry_username = smtpTab->entry_user_name;
	entry_passwd = smtpTab->entry_passwd;

	user_name = gtk_entry_get_text((GtkEntry *) entry_username);
	passwd = gtk_entry_get_text((GtkEntry *) entry_passwd);

	if ((g_utf8_strlen(user_name, -1) == 0) || (g_utf8_strlen(passwd, -1) == 0)) {
		gtk_widget_set_sensitive(finish_button, FALSE);
	} else {
		gtk_widget_set_sensitive(finish_button, TRUE);
	}
}

/**
   This function is a callback function which will be called by Account Management
   module. It is used to copy the server address from
   ui_accMgmt_accountinfo to  ui_accMgmt_editinfo structure.

   @return void
*/
static void ui_accmgmt_save_serveraddress(void)
{
	/*Variable declarations. */
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentDialog = NULL;
	GtkWidget *parentWindow = NULL;
	const gchar *recv_server = NULL;
	const gchar *smtp_server = NULL;
	GtkWidget *win3_entry3 = NULL;
	GtkWidget *win3_entry2 = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailEditInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 3)->data);
	win3_entry3 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "inputServerAddress");
	win3_entry2 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "outputServerAddress");

	/*Input param checking. */
	if ((win3_entry3 == NULL) || (win3_entry2 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	recv_server = gtk_entry_get_text((GtkEntry *) win3_entry3);
	smtp_server = gtk_entry_get_text((GtkEntry *) win3_entry2);

	g_free(ui_accMgmt_accountinfo->receive_settings.recv_server);
	ui_accMgmt_accountinfo->receive_settings.recv_server = NULL;
	ui_accMgmt_accountinfo->receive_settings.recv_server = g_strdup(recv_server);

	g_free(ui_accMgmt_accountinfo->send_settings.smtp_server);
	ui_accMgmt_accountinfo->send_settings.smtp_server = NULL;
	ui_accMgmt_accountinfo->send_settings.smtp_server = g_strdup(smtp_server);


}

/**
This function is  called by Account Management module.
This function is used to save  default values of Advanced option.

@param void .
@return void
*/
void ui_accmgmt_default_advancedvalues(void)
{
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	/* If the SSL checkbutton is not checked in. */
	ui_accMgmt_accountinfo->receive_settings.recv_type = HEADERS;
	ui_accMgmt_accountinfo->advanced_settings.use_ssl = FALSE;
	ui_accMgmt_accountinfo->advanced_settings.use_tls = FALSE;
	ui_accMgmt_accountinfo->advanced_settings.smtp_use_ssl = FALSE;
	ui_accMgmt_accountinfo->advanced_settings.smtp_use_tls = FALSE;
	ui_accMgmt_accountinfo->receive_settings.leave_msg = TRUE;
	if (ui_accMgmt_accountinfo->receive_settings.protocol == PROTO_POP3) {
		ui_accMgmt_accountinfo->receive_settings.port = DEFAULT_POP3_PORT;
	} else {
		ui_accMgmt_accountinfo->receive_settings.port = DEFAULT_IMAP_PORT;
	}

	if (ui_accMgmt_accountinfo->user_settings.userid != NULL) {
		ui_accMgmt_accountinfo->send_settings.username =
		    g_strdup(ui_accMgmt_accountinfo->user_settings.userid);
	}
	if (ui_accMgmt_accountinfo->user_settings.passwd != NULL) {
		ui_accMgmt_accountinfo->send_settings.password =
		    g_strdup(ui_accMgmt_accountinfo->user_settings.passwd);
	}
	if (email_main_view->account_manager->configured_accounts == NULL) {
		ui_accMgmt_accountinfo->user_settings.is_default = TRUE;
	} else {
		ui_accMgmt_accountinfo->user_settings.is_default = FALSE;
	}
	ui_accMgmt_accountinfo->receive_settings.passwd_auth_type = AUTH_LOGIN;
	ui_accMgmt_accountinfo->advanced_settings.email_type = EMAIL_HTML;
	ui_accMgmt_accountinfo->advanced_settings.email_sending = SEND_IMMEDIATE;
	ui_accMgmt_accountinfo->advanced_settings.request_read_report = FALSE;
	ui_accMgmt_accountinfo->advanced_settings.org_in_reply = TRUE;
	ui_accMgmt_accountinfo->receive_settings.enable_size_limit = ONE;
	ui_accMgmt_accountinfo->receive_settings.size_limit = DEFAULT_SIZELIMIT;

	ui_accMgmt_accountinfo->send_settings.smtp_port = DEFAULT_SMTP_PORT;
	ui_accMgmt_accountinfo->advanced_settings.include_signature = 0;
	ui_accMgmt_accountinfo->send_settings.smtp_auth_type = 0;
	ui_accMgmt_accountinfo->advanced_settings.display_sign_encrypt = 0;
#if RETAINED_EMAIL
	ui_accMgmt_accountinfo->advanced_settings.display_trust = 1;
	ui_accMgmt_accountinfo->advanced_settings.display_validity = 1;
#else
	ui_accMgmt_accountinfo->advanced_settings.display_trust = 0;
	ui_accMgmt_accountinfo->advanced_settings.display_validity = 0;
#endif
}

/**
This function is  called by Account Management module.
This function is used to save  existing  values of ny account of Advances Option window.

@param ui_accMgmt_accountinfo , it holds the  PrefsAccount structure pointer .
@return void
*/
void ui_accmgmt_save_advancedvalues1(PrefsAccount * ui_accMgmt_accountinfo)
{
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentDialog = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailEditInfo");

	/* Copying existing account info to present account info */
	ui_accMgmt_accountinfo->receive_settings.port = ui_accMgmt_editinfo->receive_settings.port;
	ui_accMgmt_accountinfo->user_settings.is_default =
	    ui_accMgmt_editinfo->user_settings.is_default;
	ui_accMgmt_accountinfo->receive_settings.recv_type =
	    ui_accMgmt_editinfo->receive_settings.recv_type;
	ui_accMgmt_accountinfo->receive_settings.passwd_auth_type =
	    ui_accMgmt_editinfo->receive_settings.passwd_auth_type;
	ui_accMgmt_accountinfo->advanced_settings.email_type =
	    ui_accMgmt_editinfo->advanced_settings.email_type;
	ui_accMgmt_accountinfo->advanced_settings.email_sending =
	    ui_accMgmt_editinfo->advanced_settings.email_sending;
	ui_accMgmt_accountinfo->advanced_settings.org_in_reply =
	    ui_accMgmt_editinfo->advanced_settings.org_in_reply;
	ui_accMgmt_accountinfo->advanced_settings.request_read_report =
	    ui_accMgmt_editinfo->advanced_settings.request_read_report;
	ui_accMgmt_accountinfo->receive_settings.size_limit =
	    ui_accMgmt_editinfo->receive_settings.size_limit;
	ui_accMgmt_accountinfo->advanced_settings.include_signature =
	    ui_accMgmt_editinfo->advanced_settings.include_signature;
	ui_accMgmt_accountinfo->send_settings.username =
	    g_strdup(ui_accMgmt_editinfo->send_settings.username);
	ui_accMgmt_accountinfo->send_settings.password =
	    g_strdup(ui_accMgmt_editinfo->send_settings.password);
	ui_accMgmt_accountinfo->send_settings.smtp_auth_type =
	    ui_accMgmt_editinfo->send_settings.smtp_auth_type;
	ui_accMgmt_accountinfo->advanced_settings.display_sign_encrypt =
	    ui_accMgmt_editinfo->advanced_settings.display_sign_encrypt;
	ui_accMgmt_accountinfo->advanced_settings.display_trust =
	    ui_accMgmt_editinfo->advanced_settings.display_trust;
	ui_accMgmt_accountinfo->advanced_settings.display_validity =
	    ui_accMgmt_editinfo->advanced_settings.display_validity;
	ui_accMgmt_accountinfo->send_settings.smtp_port =
	    ui_accMgmt_editinfo->send_settings.smtp_port;
	ui_accMgmt_accountinfo->receive_settings.recent_mails =
	    ui_accMgmt_editinfo->receive_settings.recent_mails;
	ui_accMgmt_accountinfo->advanced_settings.smime_security =
	    ui_accMgmt_editinfo->advanced_settings.smime_security;
	ui_accMgmt_accountinfo->advanced_settings.use_ssl =
	    ui_accMgmt_editinfo->advanced_settings.use_ssl;
	ui_accMgmt_accountinfo->advanced_settings.use_tls =
	    ui_accMgmt_editinfo->advanced_settings.use_tls;
	ui_accMgmt_accountinfo->advanced_settings.smtp_use_ssl =
	    ui_accMgmt_editinfo->advanced_settings.smtp_use_ssl;
	ui_accMgmt_accountinfo->advanced_settings.smtp_use_tls =
	    ui_accMgmt_editinfo->advanced_settings.smtp_use_tls;
	ui_accMgmt_accountinfo->advanced_settings.signature_flag =
	    ui_accMgmt_editinfo->advanced_settings.signature_flag;
	ui_accMgmt_accountinfo->receive_settings.is_not_editable =
	    ui_accMgmt_editinfo->receive_settings.is_not_editable;
#if CL_USE
	ui_accMgmt_accountinfo->advanced_settings.use_iap_smtp_serv =
	    ui_accMgmt_editinfo->advanced_settings.use_iap_smtp_serv;
	ui_accMgmt_accountinfo->advanced_settings.iap_name =
	    g_strdup(ui_accMgmt_editinfo->advanced_settings.iap_name);
#endif
	return;
}

/**
This function is  called by Account Management module.
This function is used to save  existing  values of ny account of Advances Option window.

@param void .
@return void
*/
void ui_accmgmt_save_advancedvalues(void)
{
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentDialog = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailEditInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	ui_accMgmt_accountinfo->receive_settings.port = ui_accMgmt_editinfo->receive_settings.port;
	ui_accMgmt_accountinfo->user_settings.is_default =
	    ui_accMgmt_editinfo->user_settings.is_default;
	ui_accMgmt_accountinfo->receive_settings.recv_type =
	    ui_accMgmt_editinfo->receive_settings.recv_type;
	ui_accMgmt_accountinfo->receive_settings.passwd_auth_type =
	    ui_accMgmt_editinfo->receive_settings.passwd_auth_type;
	ui_accMgmt_accountinfo->advanced_settings.email_type =
	    ui_accMgmt_editinfo->advanced_settings.email_type;
	ui_accMgmt_accountinfo->advanced_settings.email_sending =
	    ui_accMgmt_editinfo->advanced_settings.email_sending;
	ui_accMgmt_accountinfo->advanced_settings.request_read_report =
	    ui_accMgmt_editinfo->advanced_settings.request_read_report;
	ui_accMgmt_accountinfo->advanced_settings.org_in_reply =
	    ui_accMgmt_editinfo->advanced_settings.org_in_reply;
	ui_accMgmt_accountinfo->receive_settings.size_limit =
	    ui_accMgmt_editinfo->receive_settings.size_limit;
	ui_accMgmt_accountinfo->advanced_settings.include_signature =
	    ui_accMgmt_editinfo->advanced_settings.include_signature;
	ui_accMgmt_accountinfo->send_settings.username =
	    g_strdup(ui_accMgmt_editinfo->send_settings.username);
	ui_accMgmt_accountinfo->send_settings.password =
	    g_strdup(ui_accMgmt_editinfo->send_settings.password);
	ui_accMgmt_accountinfo->send_settings.smtp_auth_type =
	    ui_accMgmt_editinfo->send_settings.smtp_auth_type;
	ui_accMgmt_accountinfo->advanced_settings.display_sign_encrypt =
	    ui_accMgmt_editinfo->advanced_settings.display_sign_encrypt;
	ui_accMgmt_accountinfo->advanced_settings.display_trust =
	    ui_accMgmt_editinfo->advanced_settings.display_trust;
	ui_accMgmt_accountinfo->advanced_settings.display_validity =
	    ui_accMgmt_editinfo->advanced_settings.display_validity;
	ui_accMgmt_accountinfo->send_settings.smtp_port =
	    ui_accMgmt_editinfo->send_settings.smtp_port;
	ui_accMgmt_accountinfo->receive_settings.leave_msg =
	    ui_accMgmt_editinfo->receive_settings.leave_msg;
	ui_accMgmt_accountinfo->receive_settings.recent_mails =
	    ui_accMgmt_editinfo->receive_settings.recent_mails;
	ui_accMgmt_accountinfo->advanced_settings.smime_security =
	    ui_accMgmt_editinfo->advanced_settings.smime_security;
	ui_accMgmt_accountinfo->advanced_settings.use_ssl =
	    ui_accMgmt_editinfo->advanced_settings.use_ssl;
	ui_accMgmt_accountinfo->advanced_settings.use_tls =
	    ui_accMgmt_editinfo->advanced_settings.use_tls;
	ui_accMgmt_accountinfo->advanced_settings.smtp_use_ssl =
	    ui_accMgmt_editinfo->advanced_settings.smtp_use_ssl;
	ui_accMgmt_accountinfo->advanced_settings.smtp_use_tls =
	    ui_accMgmt_editinfo->advanced_settings.smtp_use_tls;
	return;
}

/**
 This is an utility function. This function copy the existing account setting to 
 the newly created structure depending on the changes done by user.

 @param saveSetup ,It holds the Setup dialog number  which is not changed by the user.
 @return void.
 */

static void ui_accmgmt_copy_account_setting(SaveSetup saveSetup)
{
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentDialog = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailEditInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	if ((saveSetup == SAVE_SETUP1) || (saveSetup == SAVE_SETUP_ALL)) {
		ui_accMgmt_accountinfo->user_settings.account_name =
		    g_strdup(ui_accMgmt_editinfo->user_settings.account_name);
		ui_accMgmt_accountinfo->receive_settings.protocol =
		    ui_accMgmt_editinfo->receive_settings.protocol;
	}
	if ((saveSetup == SAVE_SETUP2) || (saveSetup == SAVE_SETUP_ALL)) {
		ui_accMgmt_accountinfo->user_settings.alias_name =
		    g_strdup(ui_accMgmt_editinfo->user_settings.alias_name);
		ui_accMgmt_accountinfo->user_settings.userid =
		    g_strdup(ui_accMgmt_editinfo->user_settings.userid);
		ui_accMgmt_accountinfo->user_settings.passwd =
		    g_strdup(ui_accMgmt_editinfo->user_settings.passwd);
		ui_accMgmt_accountinfo->user_settings.email =
		    g_strdup(ui_accMgmt_editinfo->user_settings.email);
	}
	if ((saveSetup == SAVE_SETUP3) || (saveSetup == SAVE_SETUP_ALL)) {
		ui_accMgmt_accountinfo->receive_settings.recv_server =
		    g_strdup(ui_accMgmt_editinfo->receive_settings.recv_server);
		ui_accMgmt_accountinfo->send_settings.smtp_server =
		    g_strdup(ui_accMgmt_editinfo->send_settings.smtp_server);
	}
}


/**
This function is  called in  Account Management module.
This function is used to check validation of account name.

@param void .
@return gboolean. This function returns FALSE when validation is failed, TRUE otherwise
*/
static gboolean ui_accmgmt_setup2_validation(void)
{
	/*Variable declarations. */
	GtkWidget *win2_entry1 = NULL;
	GtkWidget *win2_entry2 = NULL;
	GtkWidget *win2_entry4 = NULL;
	gchar *name = NULL, *temp_emailid = NULL, *id_user = NULL;
	GtkWidget *parent_dialog = NULL;
	gchar *restricted_chars_mesg = NULL;

	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);

	win2_entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "nameEntry");
	win2_entry2 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "userNameEntry");
	win2_entry4 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "emailEntry");
	name = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry1));

	/*Check whether 'Name' field include '<' or '>' . */
	if ((strchr(name, LESSER_THAN) != NULL)
	    || (strchr(name, GREATER_THAN) != NULL)) {
		restricted_chars_mesg =
		    g_strdup_printf(_("mcen_ib_illegal_chars_error"), RESTRICTED_NAME_CHARS);
		gtk_infoprint((GtkWindow *) parent_dialog, restricted_chars_mesg);
		gtk_widget_grab_focus(win2_entry1);
		gtk_editable_select_region((GtkEditable *) win2_entry1, 0, -1);
		g_free(restricted_chars_mesg);
		g_free(name);
		return FALSE;
	}
	id_user = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry2));

	/*Checking for blank user name. */
	if (((gtk_entry_get_text((GtkEntry *) win2_entry2)) == NULL)
	    || (g_utf8_strlen(id_user, -1) == 0)) {
		gtk_infoprint((GtkWindow *) parent_dialog, _("mcen_ib_enter_username"));
		gtk_widget_grab_focus(win2_entry2);
		gtk_editable_select_region((GtkEditable *) win2_entry2, 0, -1);
		g_free(name);
		g_free(id_user);
		return FALSE;
	}
	temp_emailid = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry4));

	/*Checking for blank Email Id. */
	if (((gtk_entry_get_text((GtkEntry *) win2_entry4)) == NULL)
	    || (g_utf8_strlen(temp_emailid, -1) == 0)) {
		gtk_infoprint((GtkWindow *) parent_dialog, _("mcen_ib_required_email"));
		gtk_editable_select_region((GtkEditable *) win2_entry4, 0, -1);
		gtk_widget_grab_focus(win2_entry4);
		g_free(name);
		g_free(id_user);
		g_free(temp_emailid);
		return FALSE;
	}
	g_free(name);
	g_free(id_user);
	g_free(temp_emailid);
	return TRUE;
}

/**
This function is a callback function which will be called by Account Management module.

@param widget pointer to GtkWidget on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_accountsetupcallbacknext2_cb(GtkWidget * widget, GSList * data)
{
	/*Variable declarations. */
	GtkWidget *window3 = NULL;
	GtkWidget *win2_entry1 = NULL;
	GtkWidget *win2_entry2 = NULL;
	GtkWidget *win2_entry3 = NULL;
	GtkWidget *win2_entry4 = NULL;
	GtkWidget *label1 = NULL;
	gchar *id_user = NULL, *name = NULL, *temp_passwd = NULL, *temp_emailid = NULL;
	GtkWidget *win = NULL;
	gboolean setup2_flag = FALSE;
	GtkWidget *parent_dialog = NULL;
	GtkWidget *parent_window = NULL;
	GtkWidget *edit_account_dialog = NULL;
	gchar *server_name_label = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentDialog = NULL;

	edit_account_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 0)->data);
	parentDialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 1)->data);

	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	if (widget == NULL) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}

	win = (GtkWidget *) parent_dialog;
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(edit_account_dialog), "EmailEditInfo");
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailAccountInfo");
	win2_entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "nameEntry");
	win2_entry2 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "userNameEntry");
	win2_entry3 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "passWdEntry");
	win2_entry4 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "emailEntry");

	/*Param Checking. */
	if ((win2_entry1 == NULL) || (win2_entry2 == NULL) || (win2_entry3 == NULL)
	    || (win2_entry4 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}

	/* Setting  the flag depending on  the setup2 setting has modified or not */
	if (gActMgtMngr->edit_operation) {
		gActMgtMngr->setup2_active = is_setup2_settings_modified(win);
	}

	id_user = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry2));
	name = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry1));
	g_strstrip(name);
	temp_emailid = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry4));
	if (!ui_accmgmt_setup2_validation()) {
		g_free(temp_emailid);
		g_free(id_user);
		g_free(name);
		return;
	}
	setup2_flag = ui_accmgmt_emptycheck_onsetup2((GtkWidget *) win);
	if (!setup2_flag) {
		g_free(name);
		g_free(id_user);
		g_free(temp_emailid);
		return;
	}
	if (ui_accMgmt_accountinfo->user_settings.alias_name != NULL) {
		g_free(ui_accMgmt_accountinfo->user_settings.alias_name);
		ui_accMgmt_accountinfo->user_settings.alias_name = NULL;
	}
	ui_accMgmt_accountinfo->user_settings.alias_name = g_strdup(name);

	if (gActMgtMngr->edit_operation) {
		if (ui_compare_utf8_strings(id_user, ui_accMgmt_editinfo->user_settings.userid) !=
		    0) {
			gActMgtMngr->ACCT_NAME_MODY = TRUE;
		}
	}
	if (ui_accMgmt_accountinfo->user_settings.userid != NULL) {
		g_free(ui_accMgmt_accountinfo->user_settings.userid);
		ui_accMgmt_accountinfo->user_settings.userid = NULL;
	}
	ui_accMgmt_accountinfo->user_settings.userid = g_strdup(id_user);
	temp_passwd = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry3));
	g_free(ui_accMgmt_accountinfo->user_settings.passwd);
	ui_accMgmt_accountinfo->user_settings.passwd = g_strdup(temp_passwd);
	g_free(ui_accMgmt_accountinfo->user_settings.email);
	ui_accMgmt_accountinfo->user_settings.email = g_strdup(temp_emailid);
	gtk_widget_hide((GtkWidget *) win);

	/* If Mailbox Setup 3 Dialog is not already created. */
	if ((g_slist_nth((GSList *) ui_accMgmt_listptr, 3)) == NULL) {
		window3 =
		    (GtkWidget *) ui_accmgmt_createemailaccountsetup3(gActMgtMngr->edit_operation);
		if (window3 == NULL) {
			ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__,
				   __LINE__);
			return;
		}
		g_object_set_data(G_OBJECT(window3), "EmailAccountInfo", ui_accMgmt_accountinfo);

		gtk_widget_show_all(window3);
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window3);
	} else {
		parent_window = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 3)->data);
		label1 = EMAIL_UI_LOOKUP_OBJECT(parent_window, "inputServerCaptionControl");
		if (ui_accMgmt_accountinfo->receive_settings.protocol == RECV_POP3) {
			server_name_label =
			    g_strdup_printf(_("mcen_li_emailsetup_servertype"),
					    _("mail_fi_emailtype_pop3"));
			hildon_caption_set_label((HildonCaption *) label1, server_name_label);
		} else {
			server_name_label =
			    g_strdup_printf(_("mcen_li_emailsetup_servertype"),
					    _("mail_fi_emailtype_imap"));
			hildon_caption_set_label((HildonCaption *) label1, server_name_label);
		}
		gtk_widget_show_all(g_slist_nth((GSList *) ui_accMgmt_listptr, 3)->data);
	}
	g_free(id_user);
	g_free(temp_passwd);
	g_free(name);
	g_free(temp_emailid);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This function is a callback function which will be called by Account Management
module. It will call the craetion of advanced window function.

@param widget pointer to GtkWidget on which it is called.
@param user_data  gpointer type contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_accountsetupcallbackadvance_cb(GtkWidget * widget, gpointer * user_data)
{
	AdvOptionViewInfo *advOptionViewInfo = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parentAdvanced = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Check the input params */
	if ((widget == NULL) || (user_data == NULL)) {
		ULOG_DEBUG("Invalid input param");
		return;
	}
	parentAdvanced = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo =
	    g_object_get_data(G_OBJECT((GtkWidget *) (parentAdvanced)), "EmailAccountInfo");
	gtk_widget_hide((GtkWidget *) (user_data));

	advOptionViewInfo =
	    (AdvOptionViewInfo *) ui_accmgmt_createadvancedsettingdialog(gActMgtMngr->
									 edit_operation);
	gtk_widget_show(advOptionViewInfo->dialog_advancedsetup);
	ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, advOptionViewInfo);
	gActMgtMngr->RECV_PROT_ADVSETUP = ui_accMgmt_accountinfo->receive_settings.protocol;
	gActMgtMngr->incoming_port = ui_accMgmt_accountinfo->receive_settings.port;

	/* Setting the Flag as Advance Setup Dialog is opened */
	gActMgtMngr->advanced_setup_active = TRUE;

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
  This function is a callback function. It is used to change the sensitivity of 
  the smtp username and password widgets.

  @param widget holds the pointer to the widget
  @param data  holds the pointer to the  parent dialog
  @return void.
*/
void ui_accMgmt_onsecureport_cb(GtkWidget * widget, gpointer data)
{
	/*Variable declarations. */
	GtkWidget *ssl_checkbutton = NULL;
	GtkWidget *inport_spinbutton = NULL;
	GtkWidget *outport_spinbutton = NULL;
	GtkWidget *parent_dialog = NULL;
	AdvOptionViewInfo *advOptionViewInfo = NULL;
	RetrivevalTab *retTab = NULL;
	SMTPTab *smtpTab = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parentDialog = NULL;

	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailAccountInfo");

	/* Input param checking. */
	if (widget == NULL) {
		ULOG_DEBUG("Null pointer assigned in widget:Function-ui_accmgmt_onadvanceport");

		return;
	}
	advOptionViewInfo = (AdvOptionViewInfo *) data;
	parent_dialog = (GtkWidget *) advOptionViewInfo->dialog_advancedsetup;
	retTab = (RetrivevalTab *) advOptionViewInfo->retrivevalTab;
	smtpTab = (SMTPTab *) advOptionViewInfo->smtpTab;
	ssl_checkbutton = smtpTab->checkbutton_ssl;

	inport_spinbutton = retTab->spinbutton_inport;
	outport_spinbutton = smtpTab->spinbutton_outport;

	gActMgtMngr->ssl_flag = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ssl_checkbutton));

	/*If SSL checkbutton is checked in. */
	if (gActMgtMngr->ssl_flag == TRUE) {
		if (ui_accMgmt_accountinfo->receive_settings.protocol == PROTO_POP3) {
			hildon_number_editor_set_value((HildonNumberEditor *)
						       inport_spinbutton, IN_PORT);
		} else {
			hildon_number_editor_set_value((HildonNumberEditor *)
						       inport_spinbutton, OUT_PORT);
		}
	} else {
		if (ui_accMgmt_accountinfo->receive_settings.protocol == PROTO_POP3) {
			hildon_number_editor_set_value((HildonNumberEditor *)
						       inport_spinbutton, DEFAULT_POP3_PORT);
		} else {
			hildon_number_editor_set_value((HildonNumberEditor *)
						       inport_spinbutton, DEFAULT_IMAP_PORT);
		}
	}
}

/**
This function is a callback function which will be called by Account Management
module. It will change the sensitivity of the smtp username password entry in Advance SMTP dialog.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void 
*/
void ui_accmgmt_onadvanceport_cb(GtkWidget * widget, gpointer data)
{
	GtkWidget *checkbutton_smtpauth = NULL;
	GtkWidget *combo_smtpauth = NULL;
	SMTPTab *smtpTab = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Input param checking. */
	if (widget == NULL) {
		ULOG_DEBUG("Null pointer assigned in widget:Function-ui_accmgmt_onadvanceport");

		return;
	}
	smtpTab = (SMTPTab *) data;
	combo_smtpauth = smtpTab->combo_smtpauth;
	checkbutton_smtpauth = smtpTab->checkbutton_smtpauth;
	if (checkbutton_smtpauth == NULL) {
		return;
	}
	gActMgtMngr->smtp_auth_active =
	    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_smtpauth));

	/*If SMTP AUTH check button is checked in. */
	if (gActMgtMngr->smtp_auth_active == TRUE) {
		gtk_combo_box_remove_text(GTK_COMBO_BOX(combo_smtpauth), 1);
		gtk_combo_box_remove_text(GTK_COMBO_BOX(combo_smtpauth), 0);
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_smtpauth),
					  _("mcen_fi_advsetup_smtp_cram_md5"));
		gtk_combo_box_set_active(GTK_COMBO_BOX(combo_smtpauth), 0);
	} else {
		gtk_combo_box_remove_text(GTK_COMBO_BOX(combo_smtpauth), 0);
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_smtpauth),
					  _("mcen_fi_advsetup_smtp_none"));
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo_smtpauth),
					  _("mcen_fi_advsetup_smtp_login"));
		gtk_combo_box_set_active(GTK_COMBO_BOX(combo_smtpauth), 0);
	}
	gtk_widget_show(combo_smtpauth);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_accountsetupcallbacknext1_cb(GtkWidget * widget, gpointer data)
{
	/* Variable Declarations. */
	GtkWidget *window2 = NULL;
	GtkWidget *parent_dialog = NULL;
	GtkWidget *entry1 = NULL;
	GtkWidget *combo_entry2 = NULL;
	gchar *acct_name = NULL;
	gint acct_type = 0;
	GSList *configuredaccount = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parentDialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;
	gchar *display_account = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailAccountInfo");

	/* Input Parama Checking. */
	CHECK_PARAM((widget == NULL), "Invalid input Parameter", return);


	parent_dialog = gtk_widget_get_ancestor(GTK_WIDGET(widget), GTK_TYPE_DIALOG);
	entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "accountNameEntry");
	combo_entry2 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "comboAccountType");

	/* Param Checking. */
	CHECK_PARAM((entry1 == NULL), "NULL pointer assigned ", return);
	CHECK_PARAM((combo_entry2 == NULL), "NULL pointer assigned ", return);

	display_account = ui_get_new_accountname_display_string();

	/*display_account will be freed in ui_accmgmt_emptycheck_onsetup1(). */
	if (!ui_accmgmt_emptycheck_onsetup1(display_account)) {
		return;
	}
	acct_name = g_strdup(gtk_entry_get_text((GtkEntry *) entry1));
	acct_type = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_entry2));
	g_free(ui_accMgmt_accountinfo->user_settings.account_name);
	ui_accMgmt_accountinfo->user_settings.account_name = NULL;
	configuredaccount = email_main_view->account_manager->configured_accounts;

	if (!gActMgtMngr->edit_operation) {	/*create new Account */
		/* Checking whether the given account name is already exist or not? */
		while (configuredaccount != NULL) {
			if (ui_compare_utf8_strings(configuredaccount->data, acct_name) == 0) {
				gtk_infoprint((GtkWindow *) parent_dialog,
					      _("mail_ib_account_name_already_existing"));
				g_free(acct_name);
				return;
			}
			configuredaccount = configuredaccount->next;
		}
	} else {		/* Editing new Account */

		while (configuredaccount != NULL) {
			if (ui_compare_utf8_strings
			    (configuredaccount->data,
			     ui_accMgmt_editinfo->user_settings.account_name) != 0
			    && ui_compare_utf8_strings(configuredaccount->data, acct_name) == 0) {
				gtk_infoprint((GtkWindow *) parent_dialog,
					      _("mail_ib_account_name_already_existing"));
				g_free(acct_name);
				return;
			}
			configuredaccount = configuredaccount->next;
		}
		if (ui_compare_utf8_strings
		    (acct_name, ui_accMgmt_editinfo->user_settings.account_name) != 0) {
			gActMgtMngr->ACCT_NAME_MODY = TRUE;
		}
	}
	ui_accMgmt_accountinfo->user_settings.account_name = g_strdup(acct_name);
	if ((ui_accMgmt_accountinfo->receive_settings.protocol != -1) &&
	    (acct_type != ui_accMgmt_accountinfo->receive_settings.protocol)) {
		gActMgtMngr->port_changed = TRUE;
	}
	ui_accMgmt_accountinfo->receive_settings.protocol = acct_type;
	gtk_widget_hide((GtkWidget *) parent_dialog);

	if (gActMgtMngr->edit_operation == FALSE && gActMgtMngr->advanced_setup_active == FALSE) {
		ui_accmgmt_default_advancedvalues();
	}

	/* Setup2 window will be created */
	if ((ui_accMgmt_listptr->next->next) == NULL) {
		window2 =
		    (GtkWidget *) ui_accmgmt_createemailaccountsetup2(gActMgtMngr->edit_operation);
		if (window2 != NULL) {
			gtk_widget_show_all(window2);
		} else {
			return;	/* If Setup2 Dialog Creation is failed. */
		}
		/* Append the newly created dialog pointer to the global list. */
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window2);
	} else {
		gtk_widget_show_all(ui_accMgmt_listptr->next->next->data);
	}
	g_free(acct_name);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
  This function is a helper function. It is used to save the the contents of the 
  setup3 dialog in Account Management.

  @param void
  @return void.
 */

static void ui_accmgmt_savesetup3(void)
{
	GtkWidget *win3_entry3 = NULL;
	GtkWidget *win3_entry2 = NULL;
	gchar *recv_server = NULL;
	gchar *smtpserver = NULL;
	GtkWidget *parent_window = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	parent_window = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 3)->data);

	win3_entry3 = EMAIL_UI_LOOKUP_OBJECT(parent_window, "inputServerAddress");
	win3_entry2 = EMAIL_UI_LOOKUP_OBJECT(parent_window, "outputServerAddress");

	if ((win3_entry3 == NULL) || (win3_entry2 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	recv_server = g_strdup(gtk_entry_get_text((GtkEntry *) win3_entry3));
	g_strstrip(recv_server);	/*Remove blank spaces and TAB. */
	if (gActMgtMngr->edit_operation) {
		if (ui_compare_utf8_strings
		    (recv_server, ui_accMgmt_editinfo->receive_settings.recv_server) != 0) {
			gActMgtMngr->ACCT_NAME_MODY = TRUE;
		}
	}
	g_free(ui_accMgmt_accountinfo->receive_settings.recv_server);
	ui_accMgmt_accountinfo->receive_settings.recv_server = NULL;
	ui_accMgmt_accountinfo->receive_settings.recv_server = g_strdup(recv_server);
	smtpserver = g_strdup(gtk_entry_get_text((GtkEntry *) win3_entry2));
	g_strstrip(smtpserver);
	g_free(ui_accMgmt_accountinfo->send_settings.smtp_server);
	ui_accMgmt_accountinfo->send_settings.smtp_server = NULL;
	ui_accMgmt_accountinfo->send_settings.smtp_server = g_strdup(smtpserver);
	g_free(smtpserver);
	g_free(recv_server);
}

/**
This function is a callback function which will be called by Account Management
module.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_accountsetupcallbacknext3_cb(GtkWidget * widget, gpointer data)
{
	GtkWidget *window3 = NULL;
	GtkWidget *win3_entry3 = NULL;
	GtkWidget *win3_entry2 = NULL;
	gint str_len = 0;
	gchar *recv_server = NULL;
	gint str_len1 = 0;
	gchar *smtpserver = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Input params check */
	if (widget == NULL || data == NULL) {
		ULOG_DEBUG
		    ("Null pointer assigned in widget:Function-ui_accmgmt_accountsetupcallbacknext3");
		return;
	}
	win3_entry3 = EMAIL_UI_LOOKUP_OBJECT(data, "inputServerAddress");
	win3_entry2 = EMAIL_UI_LOOKUP_OBJECT(data, "outputServerAddress");
	if ((win3_entry3 == NULL) || (win3_entry2 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	str_len = (gint) strlen(((gchar *) gtk_entry_get_text((GtkEntry *) win3_entry3)));
	str_len1 = (gint) strlen(((gchar *) gtk_entry_get_text((GtkEntry *) win3_entry2)));
	if (((gtk_entry_get_text((GtkEntry *) win3_entry3)) == NULL)
	    || (str_len == 0)) {
		gtk_infoprint(data, _("mcen_ib_enter_ipname"));
		gtk_widget_grab_focus((GtkWidget *) win3_entry3);
		return;
	}
	if (((gtk_entry_get_text((GtkEntry *) win3_entry2)) == NULL)
	    || (str_len1 == 0)) {
		gtk_infoprint(data, _("mcen_ib_enter_ipname"));
		gtk_widget_grab_focus(win3_entry2);
		return;
	}

	/*Setting the flag depending on the modification in the setup3 settings. */
	if (gActMgtMngr->edit_operation) {
		gActMgtMngr->setup3_active = is_setup3_settings_modified((GtkWidget *) data);
	}

	ui_accmgmt_savesetup3();
	gtk_widget_hide((GtkWidget *) data);

	/*If Mailbox Setup 4 dialog is not already created. */
	if ((g_slist_nth((GSList *) ui_accMgmt_listptr, 4)) == NULL) {
		window3 =
		    (GtkWidget *) ui_accmgmt_createemailaccountsetup4(gActMgtMngr->edit_operation);
		if (window3 != NULL) {
			g_object_set_data(G_OBJECT(window3), "EmailAccountInfo",
					  ui_accMgmt_accountinfo);
			gtk_widget_show_all(window3);
		} else {
			return;
		}
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window3);
	} else {
		gtk_widget_show_all(g_slist_nth((GSList *) ui_accMgmt_listptr, 4)->data);
	}
	g_free(smtpserver);
	g_free(recv_server);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module. It will call the creation of setup1 dialog function.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_callback_onnext_cb(GtkWidget * widget, gpointer data)
{
	GtkWidget *window1 = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Input param checking. */
	CHECK_PARAM((widget == NULL), "Invalid Input Parameter ", return);
	CHECK_PARAM((data == NULL), "Invalid Input Parameter ", return);

	gtk_widget_hide((GtkWidget *) (data));

	/* If Mailbox Setup 1 Dialog is not already created. */
	if ((ui_accMgmt_listptr->next) == NULL) {

		window1 =
		    (GtkWidget *) ui_accmgmt_createemailaccountsetup1(gActMgtMngr->edit_operation);
		if (window1 != NULL) {
			gtk_widget_show_all(window1);
		}
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window1);
	} else {

		gtk_widget_show_all(ui_accMgmt_listptr->next->data);
	}
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module.It will validate the account name.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/
static gboolean ui_accmgmt_accountname_validation()
{

	GtkWidget *entry1 = NULL;
	gchar *acctname = NULL;
	gchar *accountname = NULL;
	gchar *restricted_char_mesg = NULL;
	GtkWidget *parent_dialog_setup1 = NULL;

	parent_dialog_setup1 = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 1)->data);
	entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog_setup1, "accountNameEntry");
	if ((gtk_entry_get_text((GtkEntry *) entry1)) == NULL) {
		gtk_infoprint(g_slist_nth(ui_accMgmt_listptr, 1)->data,
			      _("mcen_ib_enter_fill_in_all_fields"));
		gtk_editable_select_region((GtkEditable *) entry1, 0, -1);
		gtk_widget_grab_focus(entry1);
		return FALSE;
	}
	acctname = g_strdup(gtk_entry_get_text((GtkEntry *) entry1));
	if (g_utf8_strlen(acctname, -1) == 0) {
		gtk_infoprint(g_slist_nth(ui_accMgmt_listptr, 1)->data,
			      _("mcen_ib_enter_fill_in_all_fields"));
		gtk_editable_select_region((GtkEditable *) entry1, 0, -1);
		gtk_widget_grab_focus(entry1);
		g_free(acctname);
		return FALSE;
	}
	if (ui_accMgmt_userinfo_isvalid(acctname, TRUE) == FALSE) {
		restricted_char_mesg =
		    g_strdup_printf(_("mcen_ib_illegal_chars_error"), RESTRICTED_CHARS);
		gtk_infoprint(g_slist_nth(ui_accMgmt_listptr, 1)->data, restricted_char_mesg);
		g_free(acctname);
		g_free(restricted_char_mesg);
		gtk_editable_select_region((GtkEditable *) entry1, 0, -1);
		gtk_widget_grab_focus(entry1);
		return FALSE;
	}
	accountname = acctname;
	g_free(accountname);
	return TRUE;
}

/**
 This function is a helper function. This is used to check the NULL checking of 
 certain Account Managemnet parameters.

 @param parent_dialog which holds the pointer to the parent dialog.
 @return It returns TRUE if some paramets has NULL value
 	 otherwise FALSE.
*/
static gboolean ui_accmgt_check_user_info(GtkWidget * parent_dialog)
{
	/* Variable declarations */
	GtkWidget *win2_entry1 = NULL;
	GtkWidget *win2_entry2 = NULL;
	GtkWidget *win2_entry3 = NULL;
	GtkWidget *win2_entry4 = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parentWindow = NULL;
	GtkWidget *setup2_dialog = NULL;
	const gchar *passwd = NULL, *username = NULL, *name = NULL, *emailid = NULL;
	gint email_checking = -1;
	gchar *restricted_chars_mesg = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailAccountInfo");

	setup2_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);
	win2_entry1 = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "nameEntry");
	win2_entry2 = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "userNameEntry");
	win2_entry3 = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "passWdEntry");
	win2_entry4 = EMAIL_UI_LOOKUP_OBJECT(setup2_dialog, "emailEntry");

	if ((win2_entry1 == NULL) || (win2_entry2 == NULL) || (win2_entry3 == NULL)
	    || (win2_entry4 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return TRUE;
	}

	username = gtk_entry_get_text((GtkEntry *) win2_entry2);
	emailid = gtk_entry_get_text((GtkEntry *) win2_entry4);

	/* Checking for blank Emailid and Username fields. */

	if (g_utf8_strlen(emailid, -1) == 0 || g_utf8_strlen(username, -1) == 0) {
		gtk_widget_hide(parent_dialog);
		gtk_widget_show(g_slist_nth(ui_accMgmt_listptr, 2)->data);
		gtk_infoprint(g_slist_nth(ui_accMgmt_listptr, 2)->data,
			      _("mcen_ib_enter_fill_in_all_fields"));
		if (g_utf8_strlen(emailid, -1) == 0) {
			gtk_widget_grab_focus(win2_entry4);
		} else if (g_utf8_strlen(username, -1) == 0) {
			gtk_widget_grab_focus(win2_entry2);
		}
		return TRUE;
	}
	/* Checking for email-id validation. */
	email_checking = ui_accmgmt_email_isvalid(emailid);
	if (email_checking != 1) {
		gtk_widget_grab_focus(win2_entry4);
		gtk_editable_select_region((GtkEditable *) win2_entry4, 0, -1);
		gtk_infoprint((GtkWindow *) g_slist_nth(ui_accMgmt_listptr, 2)->data,
			      _("mcen_ib_not_valid_email"));
		gtk_editable_select_region((GtkEditable *) win2_entry4, 0, -1);

		return TRUE;
	}

	/* Validation for Name field */
	name = gtk_entry_get_text((GtkEntry *) win2_entry1);
	if ((strchr(name, '<') != NULL) || (strchr(name, '>') != NULL)) {
		gtk_widget_hide(parent_dialog);
		gtk_widget_show(g_slist_nth(ui_accMgmt_listptr, 2)->data);

		restricted_chars_mesg =
		    g_strdup_printf(_("mcen_ib_illegal_chars_error"), RESTRICTED_NAME_CHARS);
		gtk_infoprint((GtkWindow *) g_slist_nth(ui_accMgmt_listptr, 2)->data,
			      restricted_chars_mesg);
		g_free(restricted_chars_mesg);
		return TRUE;
	}

	passwd = gtk_entry_get_text((GtkEntry *) win2_entry3);

	/* Saving User Settings in ui_accMgmt_accountinfo structure */
	if (ui_accMgmt_accountinfo->user_settings.alias_name != NULL) {
		g_free(ui_accMgmt_accountinfo->user_settings.alias_name);
		ui_accMgmt_accountinfo->user_settings.alias_name = NULL;
	}
	ui_accMgmt_accountinfo->user_settings.alias_name = g_strdup(name);

	if (ui_accMgmt_accountinfo->user_settings.userid != NULL) {
		g_free(ui_accMgmt_accountinfo->user_settings.userid);
		ui_accMgmt_accountinfo->user_settings.userid = NULL;
	}
	ui_accMgmt_accountinfo->user_settings.userid = g_strdup(username);

	g_free(ui_accMgmt_accountinfo->user_settings.passwd);
	ui_accMgmt_accountinfo->user_settings.passwd = NULL;
	ui_accMgmt_accountinfo->user_settings.passwd = g_strdup(passwd);

	g_free(ui_accMgmt_accountinfo->user_settings.email);
	ui_accMgmt_accountinfo->user_settings.email = NULL;
	ui_accMgmt_accountinfo->user_settings.email = g_strdup(emailid);
	return FALSE;
}

/**
This is helper function which will be called from each cancel function of
account management module. This function will invoke outbox cancel sending
function to set the selected messages to suspended mode.
@param void
@return void.
*/
static void ui_on_outbox_cancel_send()
{

	ULOG_DEBUG("[%s][%s] :starts here :\n", __FILE__, __FUNCTION__);

	/*Trigger action after account creation failed if required */
	if (email_main_view->account_manager->action_after_acct_creation == SEND_MSGS) {
		ui_on_cancel_send_activate(NULL, NULL);
	}
	email_main_view->account_manager->action_after_acct_creation = NO_ACTION;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/*
This is helper function which will be called from each finish function of 
account management module. This function will invoke the corresponding module
by which it is called to process the regular operation.

@param void
@return void.
*/

static void ui_trigger_after_account_creation(void)
{
	ULOG_DEBUG("[%s][%s] :starts here :\n", __FILE__, __FUNCTION__);

	/*Trigger action after account creation if required */
	switch (email_main_view->account_manager->action_after_acct_creation) {
	case INVOKE_SIGNATURES_DIALOG:
		ui_on_signatures1_activate((GtkWidget *) app, NULL);
		break;
	case SEND_EMAIL:
		g_signal_emit_by_name(currenteditor, "send");
		break;
	case SEND_MSGS:
		ui_on_send_activate(NULL, email_main_view->widgets->messageListView);
		break;
	case RESEND_MSGS:
		ui_on_resend_activate(NULL, email_main_view->widgets->messageListView);
		break;
	case VIEWER_RESEND_MSG:
		g_signal_emit_by_name(currentviewer, "resend");
		break;
	case FORWARD_MSGS:
		ui_on_forward1_activate(NULL, email_main_view->widgets->messageListView);
		break;
	case NEW_EMAILS:
		ui_on_new_email_activate(NULL, NULL);
		break;
	case MAILBOX_ERROR:
		ui_set_error_msg_cb(email_main_view->account_manager->mailbox_error_code,
				    email_main_view->account_manager->error_info1,
				    email_main_view->account_manager->error_info2);
		break;
	case NO_ACTION:
		break;
	}
	email_main_view->account_manager->action_after_acct_creation = NO_ACTION;

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

static void ui_set_advance_incoming_port()
{
	GtkWidget *parentDialog = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;

	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailAccountInfo");

	if (gActMgtMngr->port_changed == TRUE) {
		if (ui_accMgmt_accountinfo->receive_settings.protocol == RECV_IMAP) {
			if (ui_accMgmt_accountinfo->advanced_settings.use_ssl == TRUE &&
			    ui_accMgmt_accountinfo->advanced_settings.use_tls == FALSE) {
				ui_accMgmt_accountinfo->receive_settings.port = OUT_PORT;
			} else {
				ui_accMgmt_accountinfo->receive_settings.port = DEFAULT_IMAP_PORT;
			}
		} else {
			if (ui_accMgmt_accountinfo->advanced_settings.use_ssl == TRUE &&
			    ui_accMgmt_accountinfo->advanced_settings.use_tls == FALSE) {
				ui_accMgmt_accountinfo->receive_settings.port = IN_PORT;
			} else {
				ui_accMgmt_accountinfo->receive_settings.port = DEFAULT_POP3_PORT;
			}
		}
		gActMgtMngr->port_changed = FALSE;
	}
	return;
}


/**
This function is a callback function which will be called by Account Management
module. It calls freeglobalmemory() to free all global variables and 
destroy all widgets after the finish operation. 

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_callbackfinish1_cb(GtkWidget * window, gpointer data)
{
	/* Variables  Declarations */
	GtkWidget *combo_entry1 = NULL;
	GtkWidget *entry1 = NULL;
	gchar *acct_name = NULL;
	gchar *acc_name_old = NULL;
	GSList *configuredaccount = NULL;
	gint acct_type = 0;
	SettingType type;
	GtkWidget *parent_dialog = NULL;
	gboolean retval = FALSE;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parentWindow = NULL;
	GtkWidget *parentDialog = NULL;
	GtkWidget *checkbutton_iap = NULL;


	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	email_main_view->account_manager->is_mailbox_opened = FALSE;

	if (gActMgtMngr->edit_operation) {
		parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
		ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");
	}
	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailAccountInfo");

	/* Input Param Checking */
	CHECK_PARAM((window == NULL), "Invalid Input Parameter ", return);
	CHECK_PARAM((data == NULL), "Invalid Input Parameter ", return);

	combo_entry1 = EMAIL_UI_LOOKUP_OBJECT(parentDialog, "comboAccountType");
	entry1 = EMAIL_UI_LOOKUP_OBJECT(parentDialog, "accountNameEntry");
	checkbutton_iap = EMAIL_UI_LOOKUP_OBJECT(parentDialog, "iapenable");

	/*Param Checking. */
	CHECK_PARAM((entry1 == NULL), "NULL pointer assigned ", return);
	CHECK_PARAM((combo_entry1 == NULL), "NULL pointer assigned ", return);

	acct_name = g_strdup(gtk_entry_get_text((GtkEntry *) entry1));
	acct_type = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_entry1));

	ui_accMgmt_accountinfo->advanced_settings.use_iap_smtp_serv =
	    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_iap));

	if (gActMgtMngr->edit_operation) {	/* If Editing an existing account. */
		ui_accMgmt_accountinfo->receive_settings.protocol =
		    ui_accMgmt_editinfo->receive_settings.protocol;
		acc_name_old = g_strdup(ui_accMgmt_editinfo->user_settings.account_name);

		/* Whether Account Name has changed or not. */
		if (ui_compare_utf8_strings
		    (acct_name, ui_accMgmt_editinfo->user_settings.account_name) != 0) {
			gActMgtMngr->ACCT_NAME_MODY = TRUE;
		}
		if (ui_accMgmt_accountinfo->user_settings.account_name != NULL) {
			g_free(ui_accMgmt_accountinfo->user_settings.account_name);
			ui_accMgmt_accountinfo->user_settings.account_name = NULL;
		}
		ui_accMgmt_accountinfo->user_settings.account_name =
		    g_strdup(ui_accMgmt_editinfo->user_settings.account_name);

	} else {		/* If Creating a new account. */
		if (ui_accMgmt_accountinfo->user_settings.account_name != NULL) {
			g_free(ui_accMgmt_accountinfo->user_settings.account_name);
			ui_accMgmt_accountinfo->user_settings.account_name = NULL;
		}

		/*Setting default  SMTP username & password */
		ui_accMgmt_accountinfo->user_settings.account_name = g_strdup(acct_name);
		if (ui_accMgmt_accountinfo->send_settings.username == NULL
		    || ((ui_accMgmt_accountinfo->send_settings.username != NULL)
			&& (strlen(ui_accMgmt_accountinfo->send_settings.username) == 0))) {
			ui_accMgmt_accountinfo->send_settings.username =
			    g_strdup(ui_accMgmt_accountinfo->user_settings.userid);
		}
		if (ui_accMgmt_accountinfo->send_settings.password == NULL) {
			ui_accMgmt_accountinfo->send_settings.password =
			    g_strdup(ui_accMgmt_accountinfo->user_settings.passwd);
		}

	}

	/* If there is no Default Account . */
	if (email_main_view->account_manager->default_account == NULL) {
		ui_accMgmt_accountinfo->user_settings.is_default = TRUE;
	}

	/*Cheacking the validity of account name. */
	if (!ui_accmgmt_accountname_validation()) {
		g_free(acct_name);
		g_free(acc_name_old);
		return;
	}
	configuredaccount = email_main_view->account_manager->configured_accounts;

	/* Cheacking for existing account name */
	if (gActMgtMngr->edit_operation) {
		while (configuredaccount != NULL) {
			if (ui_compare_utf8_strings
			    (configuredaccount->data,
			     ui_accMgmt_editinfo->user_settings.account_name) != 0
			    && (ui_compare_utf8_strings(configuredaccount->data, acct_name) == 0)) {
				gtk_infoprint((GtkWindow *) data,
					      _("mail_ib_account_name_already_existing"));
				return;
			}
			configuredaccount = configuredaccount->next;
		}
		type = USER_SETTING | SEND_SETTING | RECEIVE_SETTING | ADVANCED_SETTING;
	} else {
		while (configuredaccount != NULL) {
			if (ui_compare_utf8_strings(configuredaccount->data, acct_name) == 0) {
				gtk_infoprint((GtkWindow *) data,
					      _("mail_ib_account_name_already_existing"));
				return;
			}
			configuredaccount = configuredaccount->next;
		}

		if ((ui_accMgmt_accountinfo->receive_settings.protocol != -1) &&
		    (acct_type != ui_accMgmt_accountinfo->receive_settings.protocol)) {
			gActMgtMngr->port_changed = TRUE;
		}
		ui_accMgmt_accountinfo->receive_settings.protocol = acct_type;

		ui_set_advance_incoming_port();

		type = USER_SETTING | SEND_SETTING | RECEIVE_SETTING | ADVANCED_SETTING;
	}

	/* If Mailbox Setup 2 dialog is already created. */
	if (g_slist_nth(ui_accMgmt_listptr, 2) != NULL) {
		parent_dialog = (GtkWidget *) data;
		retval = ui_accmgt_check_user_info(parent_dialog);
		if (retval == TRUE) {
			return;
		}
	} else {
		ui_accmgmt_save_advancedvalues();
		ui_accmgmt_copy_account_setting(SAVE_SETUP3);
		ui_accmgmt_copy_account_setting(SAVE_SETUP2);
	}

	if (g_slist_nth(ui_accMgmt_listptr, 3) != NULL) {
		ui_accmgmt_save_serveraddress();
	} else {
		ui_accmgmt_save_advancedvalues();
		ui_accmgmt_copy_account_setting(SAVE_SETUP3);
	}


	/* Dbus call to save account settings */
	if (ui_dbus_set_account_settings(type, ui_accMgmt_accountinfo) == SETTING_SUCCESS) {
		ULOG_DEBUG("Account addition succesfull!\n");
		if (gActMgtMngr->edit_operation) {
			if (gActMgtMngr->ACCT_NAME_MODY == TRUE) {
				rename_signature_filepath(acct_name, acc_name_old);
				ui_dbus_rename_account(acc_name_old, acct_name);
				/* Remove old account name from UI list. */
				ui_remove_mail_account(acc_name_old);
				/* Add new account name into UI list */
				ui_add_mail_account(acct_name);

			}
		} else {
			ui_add_mail_account(acct_name);
		}
		if (ui_accMgmt_accountinfo->user_settings.is_default == TRUE) {
			g_free(email_main_view->account_manager->default_account);
			email_main_view->account_manager->default_account = NULL;
			email_main_view->account_manager->default_account = g_strdup(acct_name);
		}
		gtk_confirmation_banner_with_icon_name((GtkWindow *) app,
						       _
						       ("mcen_ib_advsetup_settings_saved"),
						       "qgn_note_infoprint");
		freeglobalmemory();
		ui_init_account_store();
		ui_trigger_after_account_creation();
	} else {
		freeglobalmemory();
		gtk_confirmation_banner_with_icon_name((GtkWindow *) app,
						       _("mail_ib_setting_failed"),
						       "qgn_note_infoprint");
		ULOG_DEBUG("Account addition failed!\n");
	}
	g_free(acct_name);
	g_free(acc_name_old);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is  called in  Account Management module.This function is used for
validation checking of  user name , password and email address of an account.

@param message , it holds the pointer to the field name of account such as Name, User Name etc.
                 It is not Freed in this function.
@return gboolean , This function returns FALSE when validation is failed, TRUE 
otherwise.
*/
static gboolean ui_accmgmt_emptycheck_setup2(gchar * message)
{
	GtkWidget *win2_entry1 = NULL;
	GtkWidget *win2_entry2 = NULL;
	GtkWidget *win2_entry3 = NULL;
	GtkWidget *win2_entry4 = NULL;
	gchar *user_id = NULL, *email = NULL;
	GtkWidget *parent_dialog = NULL;
	GtkWidget *parentWindow = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 1)->data);
	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);
	win2_entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "nameEntry");
	win2_entry2 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "userNameEntry");
	win2_entry3 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "passWdEntry");
	win2_entry4 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "emailEntry");

	user_id = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry2));
	email = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry4));
	/*If User Name or Email address is NULL */
	if ((g_utf8_strlen(user_id, -1) == 0) || (g_utf8_strlen(email, -1) == 0)) {
		if (email == NULL || g_utf8_strlen(email, -1) == 0) {
			gtk_widget_grab_focus(win2_entry4);
		}
		if (user_id == NULL || g_utf8_strlen(user_id, -1) == 0) {
			/*Fix for bug #24677 grabing the focus to username_entry caused the bug.
			   The focus is grabbed to name_entry */
			gtk_widget_grab_focus(win2_entry1);
		}
		gtk_widget_hide(parentWindow);
		gtk_widget_show(parent_dialog);
		gtk_infoprint((GtkWindow *) parent_dialog, message);
		g_free(user_id);
		g_free(email);
		return FALSE;
	}
	g_free(user_id);
	g_free(email);
	return TRUE;
}

/**
This function is  called in  Account Management module.This function is used to 
save   alias name ,user name , password and email address of an account.

@return void 
*/
static void ui_accmgmt_save_setup2(void)
{
	/* Variable declarations. */
	GtkWidget *win2_entry1 = NULL;
	GtkWidget *win2_entry2 = NULL;
	GtkWidget *win2_entry3 = NULL;
	GtkWidget *win2_entry4 = NULL;
	gchar *name = NULL, *user_id = NULL, *email = NULL, *pass_wd = NULL;
	gchar *alias_name = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parentdialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	parentdialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parentdialog), "EmailAccountInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);
	win2_entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "nameEntry");
	win2_entry2 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "userNameEntry");
	win2_entry3 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "passWdEntry");
	win2_entry4 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "emailEntry");

	name = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry1));
	user_id = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry2));
	email = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry4));
	pass_wd = g_strdup(gtk_entry_get_text((GtkEntry *) win2_entry3));
	alias_name = name;
	g_strstrip(name);
	g_free(ui_accMgmt_editinfo->user_settings.alias_name);
	ui_accMgmt_editinfo->user_settings.alias_name = NULL;
	ui_accMgmt_editinfo->user_settings.alias_name = g_strdup(name);
	g_free(ui_accMgmt_editinfo->user_settings.userid);
	ui_accMgmt_editinfo->user_settings.userid = NULL;
	ui_accMgmt_editinfo->user_settings.userid = g_strdup(user_id);
	g_free(ui_accMgmt_editinfo->user_settings.passwd);
	ui_accMgmt_editinfo->user_settings.passwd = g_strdup(pass_wd);
	g_free(ui_accMgmt_editinfo->user_settings.email);
	ui_accMgmt_editinfo->user_settings.email = g_strdup(email);

	/* This is added for Error codes implimentation in ui END */
	g_free(alias_name);
	g_free(user_id);
	g_free(pass_wd);
	g_free(email);
}

/**
 * This is an utility function. It is used to find out whether Setup 3 of account setting 
 * has completed or not. If some fields are kept blank foucs is set to that particular field.
 
 * @param parent_dialog , it holds the parent dialog pointer which called this function.
 * return gboolean, it returns TRUE if setting is incompleted
  		    FALSE otherwise.
 */
gboolean is_setup3_settings_incomplete(GtkWidget * parent_dialog)
{
	GtkWidget *parent_window = NULL;
	GtkWidget *smtpserver_entry = NULL;
	GtkWidget *recvserver_entry = NULL;
	const gchar *recvserver = NULL;
	const gchar *smtpserver = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	/* Param Checking. */
	if (parent_dialog == NULL) {
		ULOG_DEBUG("Input  param check failed");
		return FALSE;
	}
	parent_window = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 3)->data);

	recvserver_entry = EMAIL_UI_LOOKUP_OBJECT(parent_window, "inputServerAddress");
	smtpserver_entry = EMAIL_UI_LOOKUP_OBJECT(parent_window, "outputServerAddress");

	smtpserver = gtk_entry_get_text((GtkEntry *) smtpserver_entry);
	recvserver = gtk_entry_get_text((GtkEntry *) recvserver_entry);

	if (smtpserver == NULL || g_utf8_strlen(smtpserver, -1) == 0) {
		gtk_widget_grab_focus(smtpserver_entry);
	} else if (recvserver == NULL || g_utf8_strlen(recvserver, -1) == 0) {
		gtk_widget_grab_focus(recvserver_entry);
	} else {
		return FALSE;
	}
	gtk_widget_hide(parent_dialog);
	gtk_widget_show(parent_window);
	gtk_infoprint((GtkWindow *) parent_window, _("mcen_ib_enter_fill_in_all_fields"));
	return TRUE;
}



/**
This function is a callback function which will be called by Account Management
module. It calls freeglobalmemory() to free all global variables and destroy all 
widgets after the finish operation.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_callbackfinish4_cb(GtkWidget * window, gpointer data)
{
	/* Variable declarations. */
	GtkWidget *win = NULL;
	SettingType type;
	gchar *acc_name_new = NULL;
	gchar *acc_name_old = NULL;
	gchar *acct_name = NULL;
	gchar *account_name = NULL;
	PrefsAccount *prefaccount_local = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;
	GtkWidget *checkbutton_iap = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");


	/* Check the input params */
	if ((window == NULL) || (data == NULL)) {
		ULOG_DEBUG("Input  param check failed");
		return;
	}

	email_main_view->account_manager->is_mailbox_opened = FALSE;
	acc_name_new = g_strdup(ui_accMgmt_accountinfo->user_settings.account_name);
	win = (GtkWidget *) (data);

	/*get the use smtp connect specific server */
	checkbutton_iap = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "iapenable");
	ui_accMgmt_accountinfo->advanced_settings.use_iap_smtp_serv =
	    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_iap));

	/* If no default account present */
	if ((email_main_view->account_manager->default_account == NULL)
	    || (strlen(email_main_view->account_manager->default_account) == 0)) {
		ui_accMgmt_accountinfo->user_settings.is_default = TRUE;
	} else {
		if (ui_accMgmt_accountinfo->user_settings.is_default == TRUE) {
			acct_name = g_strdup(ui_accMgmt_accountinfo->user_settings.account_name);
			account_name = g_strdup(email_main_view->account_manager->default_account);
			if (ui_compare_utf8_strings(acct_name, account_name) != 0) {
				type =
				    USER_SETTING | SEND_SETTING | RECEIVE_SETTING |
				    ADVANCED_SETTING;
				prefaccount_local = ui_get_account_detail(account_name);
				if (prefaccount_local == NULL) {
					return;
				}
				prefaccount_local->user_settings.is_default = FALSE;
				if (ui_dbus_set_account_settings(type, prefaccount_local) ==
				    SETTING_SUCCESS);
				{
					ui_init_account_store();
					ULOG_DEBUG("Account addition succesfull!\n");
				}
				ui_accmgmt_freeprefsaccount(prefaccount_local);
				g_free(email_main_view->account_manager->default_account);
				email_main_view->account_manager->default_account = NULL;
				email_main_view->account_manager->default_account =
				    g_strdup(ui_accMgmt_accountinfo->user_settings.account_name);

			}
		}
	}

	ui_accMgmt_accountinfo->receive_settings.enable_size_limit = 1;
	if (!gActMgtMngr->edit_operation) {
		type = USER_SETTING | SEND_SETTING | RECEIVE_SETTING | ADVANCED_SETTING;
		if (ui_accMgmt_accountinfo->send_settings.username == NULL) {
			ui_accMgmt_accountinfo->send_settings.username =
			    g_strdup(ui_accMgmt_accountinfo->user_settings.userid);
		}
		if (ui_accMgmt_accountinfo->send_settings.password == NULL) {
			ui_accMgmt_accountinfo->send_settings.password =
			    g_strdup(ui_accMgmt_accountinfo->user_settings.passwd);
		}
		ui_set_advance_incoming_port();

	} else {
		ui_accMgmt_accountinfo->receive_settings.protocol =
		    ui_accMgmt_editinfo->receive_settings.protocol;
		acc_name_old = g_strdup(ui_accMgmt_editinfo->user_settings.account_name);
		g_free(ui_accMgmt_accountinfo->user_settings.account_name);
		ui_accMgmt_accountinfo->user_settings.account_name = NULL;
		ui_accMgmt_accountinfo->user_settings.account_name = g_strdup(acc_name_old);
		type = USER_SETTING | SEND_SETTING | RECEIVE_SETTING | ADVANCED_SETTING;
	}
	if (ui_dbus_set_account_settings(type, ui_accMgmt_accountinfo) == SETTING_SUCCESS) {
		ULOG_DEBUG("Account addition succesfull!\n");
		/* Add new account name into UI list */
		if (!gActMgtMngr->edit_operation) {
			ui_add_mail_account(ui_accMgmt_accountinfo->user_settings.account_name);
		}
		if (gActMgtMngr->ACCT_NAME_MODY == TRUE) {
			rename_signature_filepath(acc_name_new, acc_name_old);
			ui_dbus_rename_account(acc_name_old, acc_name_new);

			/* Remove old account name from UI list. */
			ui_remove_mail_account(acc_name_old);
			/* Add new account name into UI list. */
			ui_add_mail_account(acc_name_new);

		}
		if (ui_accMgmt_accountinfo->user_settings.is_default == TRUE) {
			g_free(email_main_view->account_manager->default_account);
			email_main_view->account_manager->default_account = g_strdup(acc_name_new);
		}
		gtk_confirmation_banner_with_icon_name((GtkWindow *) app,
						       _
						       ("mcen_ib_advsetup_settings_saved"),
						       "qgn_note_infoprint");

		freeglobalmemory();
		ui_init_account_store();
		/* Trigger action after account creation if required */
		ui_trigger_after_account_creation();
	} else {
		ULOG_DEBUG("Account addition failed!\n");
		freeglobalmemory();
		gtk_confirmation_banner_with_icon_name((GtkWindow *) app,
						       _("mail_ib_setting_failed"),
						       "qgn_note_infoprint");
	}
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	g_free(acc_name_new);
	g_free(acc_name_old);
	return;
}

/**
This function is  called by Account Management module. 
This is used to save checkbutton and spinbutton   related entries in advanced option window.

@param advOptionViewInfo which holds the pointer to the AdvOptionViewInfo structure .
@return void
*/
static void ui_accmgmt_save_advancedsetting(AdvOptionViewInfo * advOptionViewInfo)
{
	/* Variable Declaration */
	GtkWidget *parent_window = NULL;
	GtkWidget *combo_recv_type = NULL;
	GtkWidget *combo_passwd_auth = NULL;
	GtkWidget *advcombo_entry9 = NULL;
#if RETAINED_EMAIL
	GtkWidget *combo_email_type = NULL;
	GtkWidget *advcombo_entry4 = NULL;
#endif
	GtkWidget *smtpauthcombo_entry = NULL;
	gint email_type = 0, snd_email = 0;
	gint smtpauth = 0, no_email = 0, passwdauth = 0;
	gint retrive = 0;
#if RETAINED_EMAIL
	SendingTab *sendingTab = NULL;
#endif
	RetrivevalTab *retrivevalTab = NULL;
	SMTPTab *smtpTab = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;
	GtkWidget *checkbutton_smtp_auth = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");
	parent_window = (GtkWidget *) advOptionViewInfo->dialog_advancedsetup;
#if RETAINED_EMAIL
	sendingTab = (SendingTab *) advOptionViewInfo->sendingTab;
#endif
	retrivevalTab = (RetrivevalTab *) advOptionViewInfo->retrivevalTab;
	smtpTab = (SMTPTab *) (advOptionViewInfo->smtpTab);

#if RETAINED_EMAIL
	advcombo_entry4 = sendingTab->combo_sendemail;
	combo_email_type = sendingTab->combo_email_type;
#endif

	combo_recv_type = retrivevalTab->combo_recvtype;
	combo_passwd_auth = retrivevalTab->combo_passwd_auth;
	advcombo_entry9 = retrivevalTab->combo_no_email;

	checkbutton_smtp_auth = smtpTab->checkbutton_smtpauth;

	smtpauthcombo_entry = smtpTab->combo_smtpauth;
	smtpauth = gtk_combo_box_get_active(GTK_COMBO_BOX(smtpauthcombo_entry));
	ui_accMgmt_accountinfo->send_settings.smtp_auth_type = smtpauth;

#if RETAINED_EMAIL
	email_type = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_email_type));
	snd_email = gtk_combo_box_get_active(GTK_COMBO_BOX(advcombo_entry4));
#else
	/*get the settings of settings dialog. */
	ui_get_email_settings_info(NULL, NULL, NULL, &email_type, NULL);
	snd_email = 0;
#endif
	if (email_type == 0) {
		ui_accMgmt_accountinfo->advanced_settings.email_type = EMAIL_HTML;
	} else {
		ui_accMgmt_accountinfo->advanced_settings.email_type = EMAIL_TEXT;
	}
	if (snd_email == 0) {
		ui_accMgmt_accountinfo->advanced_settings.email_sending = SEND_IMMEDIATE;
	} else if (snd_email == 1) {
		ui_accMgmt_accountinfo->advanced_settings.email_sending = SEND_NEXT_CONNECTION;
	} else {
		ui_accMgmt_accountinfo->advanced_settings.email_sending = SEND_ON_REQUEST;
	}

	if (ui_accMgmt_accountinfo->receive_settings.protocol == PROTO_IMAP4) {
		no_email = gtk_combo_box_get_active(GTK_COMBO_BOX(advcombo_entry9));
		switch (no_email) {
		case 0:
			ui_accMgmt_accountinfo->receive_settings.recent_mails = 0;
			break;
		case 1:
			ui_accMgmt_accountinfo->receive_settings.recent_mails = 10;
			break;
		case 2:
			ui_accMgmt_accountinfo->receive_settings.recent_mails = 25;
			break;
		default:
			ui_accMgmt_accountinfo->receive_settings.recent_mails = 50;
			break;
		}
	}
	passwdauth = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_passwd_auth));

	if (passwdauth == 0) {
		ui_accMgmt_accountinfo->receive_settings.passwd_auth_type = AUTH_LOGIN;
	} else {
		ui_accMgmt_accountinfo->receive_settings.passwd_auth_type = AUTH_APOP;
	}

	retrive = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_recv_type));
	if (ui_accMgmt_accountinfo->receive_settings.protocol == RECV_IMAP) {
		switch (retrive) {
		case 0:
			ui_accMgmt_accountinfo->receive_settings.recv_type = HEADERS;
			break;
		case 1:
			ui_accMgmt_accountinfo->receive_settings.recv_type = BODY;
			break;
		default:
			ui_accMgmt_accountinfo->receive_settings.recv_type = BODY_ATTCH;
			break;
		}
	} else {
		switch (retrive) {
		case 0:
			ui_accMgmt_accountinfo->receive_settings.recv_type = HEADERS;
			break;
		default:
			ui_accMgmt_accountinfo->receive_settings.recv_type = BODY_ATTCH;
			break;
		}
	}

	if (gActMgtMngr->edit_operation) {
		gActMgtMngr->setting_modified = FALSE;
#if RETAINED_EMAIL
		if (ui_accMgmt_accountinfo->advanced_settings.email_type !=
		    ui_accMgmt_editinfo->advanced_settings.email_type) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.email_sending !=
		    ui_accMgmt_editinfo->advanced_settings.email_sending) {
			gActMgtMngr->setting_modified = TRUE;
		}
#endif
		if (ui_accMgmt_accountinfo->receive_settings.recent_mails !=
		    ui_accMgmt_editinfo->receive_settings.recent_mails) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if ((ui_accMgmt_accountinfo->receive_settings.passwd_auth_type !=
		     ui_accMgmt_editinfo->receive_settings.passwd_auth_type)
		    || (ui_accMgmt_accountinfo->receive_settings.recv_type !=
			ui_accMgmt_editinfo->receive_settings.recv_type)) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->send_settings.smtp_auth_type !=
		    ui_accMgmt_editinfo->send_settings.smtp_auth_type) {
			gActMgtMngr->setting_modified = TRUE;
		}
	}
}

/**
This function is  called by Account Management module.
This is used to save combo-box related entries in advanced option window.

@param advOptionViewInfo which holds the pointer to AdvOptionViewInfo structure.
@return void
*/

static void ui_accmgmt_save_advancedsettings_checkbuttons(AdvOptionViewInfo * advOptionViewInfo)
{
	/* Variable declarations. */
#if RETAINED_EMAIL
	GtkWidget *checkbutton_org_reply = NULL;
	GtkWidget *checkbutton_read_receipt = NULL;
#endif
	GtkWidget *checkbutton_ssl = NULL;
	GtkWidget *spinbutton_inport = NULL;
	GtkWidget *spinbutton_outport = NULL;
#if RETAINED_EMAIL
	GtkWidget *spinbutton_size_limit = NULL;
#endif
	GtkWidget *entry_user_name = NULL;
	GtkWidget *entry_passwd = NULL;
	GtkWidget *leave_msg_checkbutton = NULL;

	/* Related to Security Tab. */
	GtkWidget *checkbutton_encrypt = NULL;
	GtkWidget *checkbutton_sign = NULL;
	GtkWidget *checkbutton_encp_sign = NULL;
	GtkWidget *checkbutton_untrusted_id = NULL;
	GtkWidget *checkbutton_expired_id = NULL;

	GtkWidget *parent_window = NULL;
#if RETAINED_EMAIL
	SendingTab *sendingTab = NULL;
#endif
	RetrivevalTab *retrivevalTab = NULL;
	SMTPTab *smtpTab = NULL;
	SecurityTab *securityTab = NULL;

	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;

	gboolean inc_org_reply;
	gboolean req_read;
	gboolean leave_msg_status = FALSE;
	gint active_security_type = 0;
	gint smtp_security_type = 0;
	gint inport = -1;
	gint recv_sizelimit = -1;
	gint outport = -1;
	gchar *username = NULL;
	gchar *pass_wd = NULL;
	gboolean encrypt_status;
	gboolean sign_status;
	gboolean encrypt_sign_status;
	gboolean untrusted_id_status;
	gboolean expired_id_status;


	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	parent_window = advOptionViewInfo->dialog_advancedsetup;
#if RETAINED_EMAIL
	sendingTab = (SendingTab *) advOptionViewInfo->sendingTab;
#endif
	retrivevalTab = (RetrivevalTab *) advOptionViewInfo->retrivevalTab;
	smtpTab = (SMTPTab *) advOptionViewInfo->smtpTab;
	securityTab = (SecurityTab *) advOptionViewInfo->securityTab;

#if RETAINED_EMAIL
	checkbutton_org_reply = sendingTab->checkbutton_org_rep;
	checkbutton_read_receipt = sendingTab->checkbutton_read_receipt;
#endif
	checkbutton_ssl = smtpTab->checkbutton_ssl;
	spinbutton_inport = retrivevalTab->spinbutton_inport;
	spinbutton_outport = smtpTab->spinbutton_outport;
#if RETAINED_EMAIL
	spinbutton_size_limit = retrivevalTab->spinbutton_size_limit;
#endif
	entry_user_name = smtpTab->entry_user_name;
	entry_passwd = smtpTab->entry_passwd;

	checkbutton_encrypt = securityTab->checkbutton_encrypt;
	checkbutton_sign = securityTab->checkbutton_sign;
	checkbutton_encp_sign = securityTab->checkbutton_encp_sign;
	checkbutton_untrusted_id = securityTab->checkbutton_untrusted_id;
	checkbutton_expired_id = securityTab->checkbutton_expired_id;
	leave_msg_checkbutton = retrivevalTab->leave_mesg_checkbutton;

	leave_msg_status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(leave_msg_checkbutton));
	ui_accMgmt_accountinfo->receive_settings.leave_msg = leave_msg_status;
	encrypt_status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_encrypt));
	sign_status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_sign));
	encrypt_sign_status =
	    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_encp_sign));
	untrusted_id_status =
	    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_untrusted_id));
	expired_id_status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_expired_id));

	/* Stroring data for smime_security */
	if (encrypt_status && sign_status) {
		ui_accMgmt_accountinfo->advanced_settings.smime_security = SMIME_SIGN_ENCRYPT;
	} else if (encrypt_status) {
		ui_accMgmt_accountinfo->advanced_settings.smime_security = SMIME_ENCRYPT;
	} else if (sign_status) {
		ui_accMgmt_accountinfo->advanced_settings.smime_security = SMIME_SIGN;
	} else {
		ui_accMgmt_accountinfo->advanced_settings.smime_security = SMIME_NONE;
	}

	ui_accMgmt_accountinfo->advanced_settings.display_sign_encrypt = encrypt_sign_status;
	ui_accMgmt_accountinfo->advanced_settings.display_trust = untrusted_id_status;
	ui_accMgmt_accountinfo->advanced_settings.display_validity = expired_id_status;

	username = (gchar *) gtk_entry_get_text(GTK_ENTRY(entry_user_name));
	pass_wd = (gchar *) gtk_entry_get_text(GTK_ENTRY(entry_passwd));
	if (ui_accMgmt_accountinfo->send_settings.username != NULL) {
		g_free(ui_accMgmt_accountinfo->send_settings.username);
		ui_accMgmt_accountinfo->send_settings.username = NULL;
	}
	ui_accMgmt_accountinfo->send_settings.username = g_strdup(username);
	if (ui_accMgmt_accountinfo->send_settings.password != NULL) {
		g_free(ui_accMgmt_accountinfo->send_settings.password);
		ui_accMgmt_accountinfo->send_settings.password = NULL;
	}
	ui_accMgmt_accountinfo->send_settings.password = g_strdup(pass_wd);

#if RETAINED_EMAIL
	inc_org_reply = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_org_reply));
	req_read = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_read_receipt));
	recv_sizelimit =
	    hildon_number_editor_get_value((HildonNumberEditor *) spinbutton_size_limit);
#else
	/*get the settings of settings dialog. */
	ui_get_email_settings_info(&req_read, &recv_sizelimit, NULL, NULL, &inc_org_reply);
#endif
	ui_accMgmt_accountinfo->advanced_settings.org_in_reply = inc_org_reply;
	ui_accMgmt_accountinfo->advanced_settings.request_read_report = req_read;

	active_security_type = gtk_combo_box_get_active(GTK_COMBO_BOX(retrivevalTab->combo_ssl));

	if (active_security_type == 2) {
		ui_accMgmt_accountinfo->advanced_settings.use_ssl = TRUE;
		ui_accMgmt_accountinfo->advanced_settings.use_tls = FALSE;
	} else if (active_security_type == 1) {
		ui_accMgmt_accountinfo->advanced_settings.use_ssl = TRUE;
		ui_accMgmt_accountinfo->advanced_settings.use_tls = TRUE;
	} else {
		ui_accMgmt_accountinfo->advanced_settings.use_ssl = FALSE;
		ui_accMgmt_accountinfo->advanced_settings.use_tls = FALSE;
	}

	smtp_security_type = gtk_combo_box_get_active(GTK_COMBO_BOX(smtpTab->combo_smtp_ssl));
	if (smtp_security_type == 2) {
		ui_accMgmt_accountinfo->advanced_settings.smtp_use_ssl = TRUE;
		ui_accMgmt_accountinfo->advanced_settings.smtp_use_tls = FALSE;
	} else if (smtp_security_type == 1) {
		ui_accMgmt_accountinfo->advanced_settings.smtp_use_ssl = TRUE;
		ui_accMgmt_accountinfo->advanced_settings.smtp_use_tls = TRUE;
	} else {
		ui_accMgmt_accountinfo->advanced_settings.smtp_use_ssl = FALSE;
		ui_accMgmt_accountinfo->advanced_settings.smtp_use_tls = FALSE;
	}
	inport = hildon_number_editor_get_value((HildonNumberEditor *) spinbutton_inport);
	outport = hildon_number_editor_get_value((HildonNumberEditor *) spinbutton_outport);

	ui_accMgmt_accountinfo->receive_settings.port = (gushort) inport;

	ui_accMgmt_accountinfo->receive_settings.size_limit = recv_sizelimit;
	ui_accMgmt_accountinfo->receive_settings.enable_size_limit = ONE;
	ui_accMgmt_accountinfo->send_settings.smtp_port = (gushort) outport;

	if (gActMgtMngr->edit_operation) {
		if (ui_accMgmt_accountinfo->advanced_settings.smime_security !=
		    ui_accMgmt_editinfo->advanced_settings.smime_security) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.display_sign_encrypt !=
		    ui_accMgmt_editinfo->advanced_settings.display_sign_encrypt) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.display_trust !=
		    ui_accMgmt_editinfo->advanced_settings.display_trust) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.display_validity !=
		    ui_accMgmt_editinfo->advanced_settings.display_validity) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->receive_settings.size_limit !=
		    ui_accMgmt_editinfo->receive_settings.size_limit) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->send_settings.smtp_port !=
		    ui_accMgmt_editinfo->send_settings.smtp_port) {
			gActMgtMngr->setting_modified = TRUE;
		}

		if (ui_accMgmt_accountinfo->receive_settings.leave_msg !=
		    ui_accMgmt_editinfo->receive_settings.leave_msg) {
			gActMgtMngr->setting_modified = TRUE;
		}

		if (ui_accMgmt_accountinfo->receive_settings.port !=
		    ui_accMgmt_editinfo->receive_settings.port) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.use_ssl !=
		    ui_accMgmt_editinfo->advanced_settings.use_ssl) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.use_tls !=
		    ui_accMgmt_editinfo->advanced_settings.use_tls) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.smtp_use_tls !=
		    ui_accMgmt_editinfo->advanced_settings.smtp_use_tls) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.smtp_use_ssl !=
		    ui_accMgmt_editinfo->advanced_settings.smtp_use_ssl) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.request_read_report !=
		    ui_accMgmt_editinfo->advanced_settings.request_read_report) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->advanced_settings.org_in_reply !=
		    ui_accMgmt_editinfo->advanced_settings.org_in_reply) {
			gActMgtMngr->setting_modified = TRUE;
		}
		if (ui_accMgmt_accountinfo->send_settings.password != NULL) {
			if (ui_accMgmt_editinfo->send_settings.password != NULL) {
				if (ui_compare_utf8_strings
				    (ui_accMgmt_accountinfo->send_settings.password,
				     ui_accMgmt_editinfo->send_settings.password) != 0) {
					gActMgtMngr->setting_modified = TRUE;
				}
			}
		}
		if (ui_accMgmt_accountinfo->send_settings.username != NULL) {
			if (ui_accMgmt_editinfo->send_settings.username != NULL) {
				if (ui_compare_utf8_strings
				    (ui_accMgmt_accountinfo->send_settings.username,
				     ui_accMgmt_editinfo->send_settings.username) != 0) {
					gActMgtMngr->setting_modified = TRUE;
				}
			}
		}
		if (ui_accMgmt_accountinfo->user_settings.is_default !=
		    ui_accMgmt_editinfo->user_settings.is_default) {
			gActMgtMngr->setting_modified = TRUE;
		}
	}

}


/**
This function is a callback function which will be called by Account Management
module. It calls freeglobalmemory() to free all global variables and 
destroy all widgets after the finish operation.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_accountsetupcallbackcancel1_cb(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */

	CHECK_PARAM(widget == NULL, "Input  param check failed", return);
	CHECK_PARAM(data == NULL, "Input  param check failed", return);

	email_main_view->account_manager->is_mailbox_opened = FALSE;
	ui_accmgmt_dialogmessagecancel2((GtkWidget *) data);

	if (email_main_view->account_manager->mailbox_error_code == MAILBOX_ERROR) {
		ui_set_error_msg_cb(email_main_view->account_manager->mailbox_error_code,
				    email_main_view->account_manager->error_info1,
				    email_main_view->account_manager->error_info2);
	}
	ui_on_outbox_cancel_send();

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 This is an utility function. This function is used to determine whether the existing 
 account setup for Setup2 dialog has changed or not.

 @param parentWindow ,it holds the pointer to the Setup2 dialog.
 @return gboolean, it return TRUE when setting is modified
                   otherwise FALSE.
 */

static gboolean is_setup2_settings_modified(GtkWidget * parentWindow)
{
	/*Variable declarations. */
	GtkWidget *win2_entry1 = NULL;
	GtkWidget *win2_entry2 = NULL;
	GtkWidget *win2_entry3 = NULL;
	GtkWidget *win2_entry4 = NULL;
	const gchar *name = NULL;
	const gchar *user_name = NULL;
	const gchar *password = NULL;
	const gchar *emailid = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parent_window = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parent_window = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parent_window), "EmailEditInfo");

	win2_entry1 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "nameEntry");
	win2_entry2 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "userNameEntry");
	win2_entry3 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "passWdEntry");
	win2_entry4 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "emailEntry");

	/* Cheak Params */
	if ((win2_entry1 == NULL) || (win2_entry2 == NULL) || (win2_entry3 == NULL)
	    || (win2_entry4 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return FALSE;
	}

	name = gtk_entry_get_text((GtkEntry *) win2_entry1);
	user_name = gtk_entry_get_text((GtkEntry *) win2_entry2);
	password = gtk_entry_get_text((GtkEntry *) win2_entry3);
	emailid = gtk_entry_get_text((GtkEntry *) win2_entry4);
	if (!ui_accMgmt_editinfo) {
		if ((name && (strlen(name) != 0)) || (user_name && (strlen(user_name) != 0)) ||
		    (password && (strlen(password) != 0)) || (emailid && (strlen(emailid) != 0)))
			return TRUE;

		return FALSE;
	}

	if ((ui_accMgmt_editinfo->user_settings.alias_name == NULL) ||
	    (ui_accMgmt_editinfo->user_settings.alias_name != NULL &&
	     strlen(ui_accMgmt_editinfo->user_settings.alias_name) == 0)) {
		if ((name == NULL) || (name != NULL && strlen(name) == 0)) {
			return FALSE;
		} else {
			return TRUE;
		}
	}
	if ((ui_accMgmt_editinfo->user_settings.passwd == NULL) ||
	    (ui_accMgmt_editinfo->user_settings.passwd != NULL &&
	     strlen(ui_accMgmt_editinfo->user_settings.passwd) == 0)) {
		if ((password == NULL) || ((password != NULL) && strlen(password) == 0)) {
			return FALSE;
		} else {
			return TRUE;
		}
	}
	if (ui_compare_utf8_strings(name, ui_accMgmt_editinfo->user_settings.alias_name) == 0
	    && ui_compare_utf8_strings(user_name, ui_accMgmt_editinfo->user_settings.userid) == 0
	    && ui_compare_utf8_strings(password, ui_accMgmt_editinfo->user_settings.passwd) == 0
	    && ui_compare_utf8_strings(emailid, ui_accMgmt_editinfo->user_settings.email) == 0) {
		return FALSE;
	} else {
		return TRUE;
	}

}

/**
This function is a callback function which will be called by Account Management
module.It calls freeglobalmemory() to free all global variables and 
destroy all widgets after the finish operation.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_accountsetupcallbackcancel2_cb(GtkWidget * widget, gpointer data)
{
	GtkWidget *parentWindow = NULL;
	GtkWidget *win2_entry1 = NULL;
	GtkWidget *win2_entry2 = NULL;
	GtkWidget *win2_entry3 = NULL;
	GtkWidget *win2_entry4 = NULL;
	const gchar *name = NULL;
	const gchar *user_name = NULL;
	const gchar *password = NULL;
	const gchar *emailid = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */

	CHECK_PARAM(widget == NULL, "Input  param check failed", return);
	CHECK_PARAM(data == NULL, "Input  param check failed", return);

	email_main_view->account_manager->is_mailbox_opened = FALSE;

	parentWindow = (GtkWidget *) data;
	win2_entry1 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "nameEntry");
	win2_entry2 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "userNameEntry");
	win2_entry3 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "passWdEntry");
	win2_entry4 = EMAIL_UI_LOOKUP_OBJECT(parentWindow, "emailEntry");

	if ((win2_entry1 == NULL) || (win2_entry2 == NULL) || (win2_entry3 == NULL)
	    || (win2_entry4 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}

	name = gtk_entry_get_text((GtkEntry *) win2_entry1);
	user_name = gtk_entry_get_text((GtkEntry *) win2_entry2);
	password = gtk_entry_get_text((GtkEntry *) win2_entry3);
	emailid = gtk_entry_get_text((GtkEntry *) win2_entry4);

	/*If Account Edit Operation */
	if (gActMgtMngr->edit_operation) {
		if ((is_setup2_settings_modified(parentWindow) == FALSE)
		    && (gActMgtMngr->is_setup1_modified == FALSE)
		    && (gActMgtMngr->setup3_active == FALSE)
		    && (gActMgtMngr->setting_modified == FALSE)) {
			ui_accmgmt_dialogmessagecancel1((GtkWidget *) parentWindow);
			return;
		}
	} else {		/*Creation of New Account */

		if (gActMgtMngr->is_setup1_modified == FALSE) {
			if ((g_utf8_strlen(name, -1) == 0)
			    && (g_utf8_strlen(user_name, -1) == 0)
			    && (g_utf8_strlen(password, -1) == 0)
			    && (g_utf8_strlen(emailid, -1) == 0)) {

				ui_accmgmt_dialogmessagecancel1((GtkWidget *) parentWindow);
				return;
			}
		}
	}
	/* New Account Creation and some (Account) dialog setups has modified */
	ui_accmgmt_dialogmessagecancel((GtkWidget *) parentWindow);
	if (email_main_view->account_manager->mailbox_error_code == MAILBOX_ERROR) {
		ui_set_error_msg_cb(email_main_view->account_manager->mailbox_error_code,
				    email_main_view->account_manager->error_info1,
				    email_main_view->account_manager->error_info2);
	}


	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module.It calls freeglobalmemory() to free all global variables and 
destroy all widgets after the finish operation.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_accountsetupcallbackcancel3_cb(GtkWidget * widget, gpointer data)
{
	/* Variable declarations. */
	GtkWidget *entry1 = NULL;
	GtkWidget *combo_entry1 = NULL;
	gint accttype = 0;
	gchar *acctname = NULL;
	gint recv_protocol = -1;
	GtkWidget *parent_dialog = NULL;
	gchar *account_name = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;
	GtkWidget *parentWindow2 = NULL;
	gboolean modified_flag = FALSE;
	GSList *node = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Check the input params */

	CHECK_PARAM(widget == NULL, "Input  param check failed", return);

	email_main_view->account_manager->is_mailbox_opened = FALSE;
	parent_dialog = ui_accMgmt_listptr->next->data;
	entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "accountNameEntry");
	combo_entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "comboAccountType");

	if ((entry1 == NULL) || (combo_entry1 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	acctname = g_strdup(gtk_entry_get_text((GtkEntry *) entry1));
	accttype = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_entry1));

	if (gActMgtMngr->edit_operation) {	/* Edit on existing account. */
		recv_protocol = ui_accMgmt_editinfo->receive_settings.protocol;
		if (ui_accMgmt_editinfo->user_settings.account_name == NULL) {
			ULOG_DEBUG("Account name is assigned NULL pointer");
			return;
		}
		if ((recv_protocol == accttype)
		    && ui_compare_utf8_strings(acctname,
					       ui_accMgmt_editinfo->user_settings.account_name) == 0
		    && (gActMgtMngr->setup2_active == FALSE)
		    && (gActMgtMngr->setup3_active == FALSE)
		    && (gActMgtMngr->setting_modified == FALSE)) {
			ui_accmgmt_dialogmessagecancel1((GtkWidget *) parent_dialog);
			g_free(acctname);
			return;
		}
	} else {
		account_name = ui_get_new_accountname_display_string();
		if ((node = g_slist_nth((GSList *) ui_accMgmt_listptr, 2))) {
			parentWindow2 = (GtkWidget *) (node->data);
			modified_flag = is_setup2_settings_modified(parentWindow2);
		}
		if ((accttype == POP3_ACCT)
		    && (ui_compare_utf8_strings(acctname, account_name) == 0) && !modified_flag) {
			ui_accmgmt_dialogmessagecancel1((GtkWidget *) parent_dialog);
			g_free(acctname);
			g_free(account_name);
			return;
		}
		g_free(account_name);

	}

	g_free(acctname);
	ui_accmgmt_dialogmessagecancel((GtkWidget *) parent_dialog);
	if (email_main_view->account_manager->mailbox_error_code == MAILBOX_ERROR) {
		ui_set_error_msg_cb(email_main_view->account_manager->mailbox_error_code,
				    email_main_view->account_manager->error_info1,
				    email_main_view->account_manager->error_info2);
	}
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
 This is an utility function. This function is used to determine whether the existing 
 account setup for Setup2 dialog has changed or not.

 @param parentWindow ,it holds the pointer to the Setup2 dialog.
 @return gboolean, it return TRUE when setting is modified
                        otherwise FALSE.
 */
static gboolean is_setup3_settings_modified(GtkWidget * parent_widget)
{
	const gchar *recvserver = NULL;
	const gchar *sendserver = NULL;
	GtkWidget *entry1 = NULL;
	GtkWidget *entry2 = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_widget, "inputServerAddress");
	entry2 = EMAIL_UI_LOOKUP_OBJECT(parent_widget, "outputServerAddress");
	if ((entry1 == NULL) || (entry2 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return FALSE;
	}

	recvserver = gtk_entry_get_text((GtkEntry *) entry1);
	sendserver = gtk_entry_get_text((GtkEntry *) entry2);
	if (ui_compare_utf8_strings(sendserver, ui_accMgmt_editinfo->send_settings.smtp_server) == 0
	    && ui_compare_utf8_strings(recvserver,
				       ui_accMgmt_editinfo->receive_settings.recv_server) == 0) {
		return FALSE;
	} else {
		return TRUE;
	}
}

/**
This function is a callback function which will be called by Account Management
module.It calls freeglobalmemory() to free all global variables and 
destroy all widgets after the finish operation.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_accountsetupcallbackcancel4_cb(GtkWidget * widget, gpointer data)
{
	const gchar *recvserver = NULL;
	const gchar *sendserver = NULL;
	GtkWidget *entry1 = NULL;
	GtkWidget *entry2 = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Check Input param. */
	CHECK_PARAM(widget == NULL, "Input  param check failed", return);
	CHECK_PARAM(data == NULL, "Input  param check failed", return);

	email_main_view->account_manager->is_mailbox_opened = FALSE;
	entry1 = EMAIL_UI_LOOKUP_OBJECT(data, "inputServerAddress");
	entry2 = EMAIL_UI_LOOKUP_OBJECT(data, "outputServerAddress");
	if ((entry1 == NULL) || (entry2 == NULL)) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	recvserver = gtk_entry_get_text((GtkEntry *) entry1);
	sendserver = gtk_entry_get_text((GtkEntry *) entry2);
	if (!gActMgtMngr->edit_operation) {
		ui_accmgmt_dialogmessagecancel((GtkWidget *) data);
		return;
	} else {
		if ((ui_accMgmt_editinfo->send_settings.smtp_server == NULL)
		    || (ui_accMgmt_editinfo->receive_settings.recv_server == NULL)) {
			gtk_widget_destroy((GtkWidget *) data);
			return;
		}
		if ((is_setup3_settings_modified((GtkWidget *) data) == FALSE)
		    && (gActMgtMngr->is_setup1_modified == FALSE)
		    && (gActMgtMngr->setup2_active == FALSE)
		    && (gActMgtMngr->setting_modified == FALSE)) {
			ui_accmgmt_dialogmessagecancel1((GtkWidget *) data);
			return;
		}
	}
	ui_accmgmt_dialogmessagecancel((GtkWidget *) data);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module.It calls freeglobalmemory() to free all global variables and 
destroy all widgets after the finish operation.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_accountsetupcallbackcancel5_cb(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM(widget == NULL, "Input  param check failed", return);
	CHECK_PARAM(data == NULL, "Input  param check failed", return);

	if (!gActMgtMngr->edit_operation) {
		ui_accmgmt_dialogmessagecancel((GtkWidget *) (data));
		return;
	} else {
		if ((gActMgtMngr->is_setup1_modified == FALSE)
		    && (gActMgtMngr->setup2_active == FALSE)
		    && (gActMgtMngr->setup3_active == FALSE)
		    && (gActMgtMngr->setting_modified == FALSE)) {
			ui_accmgmt_dialogmessagecancel1((GtkWidget *) data);
			return;
		}
	}
	ui_accmgmt_dialogmessagecancel((GtkWidget *) (data));
	if (email_main_view->account_manager->mailbox_error_code == MAILBOX_ERROR) {
		ui_set_error_msg_cb(email_main_view->account_manager->mailbox_error_code,
				    email_main_view->account_manager->error_info1,
				    email_main_view->account_manager->error_info2);
	}
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module.It calls freeglobalmemory() to free all global variables and 
destroy all widgets after the finish operation.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_advancecancel_cb(GtkWidget * widget, gpointer data)
{
	AdvOptionViewInfo *advStruct = NULL;
	GtkWidget *parent_window = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Check the input params */
	CHECK_PARAM(widget == NULL, "Input  param check failed", return);
	CHECK_PARAM(data == NULL, "Input  param check failed", return);

	advStruct = (AdvOptionViewInfo *) data;
	parent_window = (GtkWidget *) advStruct->dialog_advancedsetup;
	if (gActMgtMngr->edit_operation) {
		ui_accmgmt_save_advancedsetting(advStruct);
		ui_accmgmt_save_advancedsettings_checkbuttons(advStruct);
		if ((gActMgtMngr->setting_modified == TRUE)
		    || (gActMgtMngr->is_setup1_modified == TRUE)
		    || (gActMgtMngr->setup2_active == TRUE)
		    || (gActMgtMngr->setup3_active == TRUE)) {
			gActMgtMngr->setting_modified = FALSE;
			ui_accmgmt_dialogmessagecancel(parent_window);
			return;
		} else {
			ui_accmgmt_dialogmessagecancel1(parent_window);
		}
	} else {
		ui_accmgmt_dialogmessagecancel(parent_window);
		return;
	}
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module. This function is used to show the setup1 dialog.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_accountsetupcallbackback1_cb(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/*Input Param Checking. */
	CHECK_PARAM(widget == NULL, "Input  param check failed", return);
	CHECK_PARAM(data == NULL, "Input  param check failed", return);

	/*Hide Setup1 Dialog. */
	gtk_widget_hide((GtkWidget *) data);
	gtk_widget_show((GtkWidget *) ui_accMgmt_listptr->data);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module. This function is used to show the setup2 dialog.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_accountsetupcallbackback2_cb(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Input Param Checking. */
	CHECK_PARAM(widget == NULL, "Input  param check failed", return);
	CHECK_PARAM(data == NULL, "Input  param check failed", return);

	if (gActMgtMngr->edit_operation) {

		/* Setting  the flag depending on  the setup2 setting has modified or not */
		gActMgtMngr->setup2_active = is_setup2_settings_modified((GtkWidget *) data);
		ui_accmgmt_save_setup2();
		ULOG_DEBUG(" The value of flag setup2_active is %d\n", gActMgtMngr->setup2_active);
	}
	/*Hide Setup2 Dialog. */
	gtk_widget_hide((GtkWidget *) data);
	gtk_widget_show_all(ui_accMgmt_listptr->next->data);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}



/**
This function is a callback function which will be called by Account Management
module.This function is used to show the setup3 dialog.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/

void ui_accmgmt_accountsetupcallbackback3_cb(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (widget == NULL) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);

		return;
	}
	if (data == NULL) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	if (gActMgtMngr->edit_operation) {
		gActMgtMngr->setup3_active = is_setup3_settings_modified((GtkWidget *) data);
	}
	ui_accmgmt_savesetup3();
	gtk_widget_hide((GtkWidget *) data);
	gtk_widget_show_all(g_slist_nth((GSList *) ui_accMgmt_listptr, 2)->data);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module. This function is used to show the setup4 dialog.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_accountsetupcallbackback4_cb(GtkWidget * widget, gpointer data)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if ((widget == NULL) || (data == NULL)) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);

		return;
	}

	gtk_widget_hide((GtkWidget *) data);
	gtk_widget_show_all(g_slist_nth((GSList *) ui_accMgmt_listptr, 3)->data);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module. This function is used to show the advanced  dialog.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget pointers.
@return void
*/
void ui_accmgmt_onadvanceback_cb(GtkWidget * widget, gpointer data)
{
	GtkWidget *win = NULL;
	AdvOptionViewInfo *advStruct = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (widget == NULL) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	advStruct = (AdvOptionViewInfo *) data;
	if (g_slist_nth(ui_accMgmt_listptr, 5) != NULL) {
		ui_accmgmt_save_advancedsetting(advStruct);
		ui_accmgmt_save_advancedsettings_checkbuttons(advStruct);
	}
	win = (GtkWidget *) advStruct->dialog_advancedsetup;
	gtk_widget_hide(win);
	gtk_widget_show_all(g_slist_nth((GSList *) ui_accMgmt_listptr, 4)->data);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module. It calls freeglobalmemory() to free all global variables and 
destroy all widgets after the finish operation.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains NULL.
@return void
*/

void ui_accmgmt_onclose_cb(GtkWidget * widget, gpointer data)
{
	GSList *tempList = NULL;
	GtkWidget *dialog = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (widget == NULL) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	if (data == NULL) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);

		return;
	}

	/*Freeing the Singly List's contents */
	tempList = ui_accMgmt_listptr;
	while (tempList != NULL) {
		if (tempList->data != NULL) {
			dialog = GTK_WIDGET(tempList->data);
			gtk_widget_destroy(dialog);
		}
		tempList = g_slist_next(tempList);
	}
	if (ui_accMgmt_listptr != NULL) {
		g_slist_free(ui_accMgmt_listptr);
		ui_accMgmt_listptr = NULL;
	}
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function is a callback function which will be called by Account Management
module. It will call the creation of setup1 dialog function.

@param widget pointer to GtkWidget  on which it is called.
@param data gpointer variable contains GtkWidget.
@return void
*/
void ui_accmgmt_onedit_cb(GtkWidget * widget, gpointer data)
{
	GtkWidget *window1 = NULL;
	GtkTreeIter iter;
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection = NULL;
	GtkWidget *accountListView = NULL;
	GtkWidget *set_def_button = NULL;
	GtkWidget *edit_button = NULL;
	GtkWidget *delete_button = NULL;
	GtkWidget *close_button = NULL;
	PrefsAccount *prefaccount_local = NULL;
	SettingType type;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	PrefsAccount *temp_account_details = NULL;
	GtkWidget *parentWindow = NULL;
	gint account_status = -2;
	gboolean user_confirmation;
	gchar *acctname = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	if ((widget == NULL) || (data == NULL)) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	accountListView = EMAIL_UI_LOOKUP_OBJECT((GtkWidget *) data, "accountListTreeView");
	set_def_button = EMAIL_UI_LOOKUP_OBJECT((GtkWidget *) data, "set_default_account_button");
	edit_button = EMAIL_UI_LOOKUP_OBJECT((GtkWidget *) data, "edit_account_button");
	delete_button = EMAIL_UI_LOOKUP_OBJECT((GtkWidget *) data, "delete_account_button");
	close_button = EMAIL_UI_LOOKUP_OBJECT((GtkWidget *) data, "edit_account_close");
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(accountListView));

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, 0, &acctname, -1);
	} else {
		ULOG_DEBUG("[%s][%s] : No account is selected :\n", __FILE__, __FUNCTION__);
		return;
	}

	type = USER_SETTING | SEND_SETTING | RECEIVE_SETTING | ADVANCED_SETTING;
	ui_free_prefs_account_structure(prefaccount_local);
	prefaccount_local = NULL;

	/* Check whether any Send/Receive operation is going on or not. */
	if (email_main_view->account_manager->current_sendrec_account != NULL
	    && strlen(email_main_view->account_manager->current_sendrec_account) != 0) {
		account_status =
		    ui_compare_utf8_strings(acctname,
					    email_main_view->account_manager->
					    current_sendrec_account);
	}

	if (account_status == 0) {
		email_main_view->account_manager->operation_status = EDIT_OP;
		email_main_view->account_manager->edit_account_dialog = NULL;

		user_confirmation =
		    get_user_confirmation(_("emev_nc_disconnect_account"),
					  _("mcen_bd_email_signatures_edit_ok"), NULL,
					  CONFIRM_DIALOG);
		/* If user selects to disconnect the Send/receive opearation */
		if (user_confirmation == TRUE) {
			if (email_main_view->account_manager->operation_status == EDIT_OP) {
				/* Dbus call to cancel Sending operation. */
				ui_dbus_cancel_send_mail(NULL);
				/* Dbus call to cancel Receiving operation. */
				ui_dbus_cancel_recv_mail(acctname, TRUE);

				/* Setting the kind of operation while Send/Receive is going on. */
				email_main_view->account_manager->edit_account_dialog =
				    parentWindow;
				g_slist_free(ui_accMgmt_listptr);
				ui_accMgmt_listptr = NULL;
				ui_free_prefs_account_structure(ui_accMgmt_editinfo);
				gtk_widget_set_sensitive(set_def_button, FALSE);
				gtk_widget_set_sensitive(edit_button, FALSE);
				gtk_widget_set_sensitive(delete_button, FALSE);
				gtk_widget_set_sensitive(close_button, FALSE);
				return;
			}
		} else {
			g_free(acctname);
			return;
		}
	}
	/*Get  setting for "acctname" Account.  */
	prefaccount_local = ui_get_account_detail(acctname);
	if (prefaccount_local == NULL) {
		ULOG_CRIT("Connection setup failed [%s] at [%d]\n", __FUNCTION__, __LINE__);

		g_free(acctname);
		return;
	}
	if (prefaccount_local->receive_settings.is_not_editable == FALSE) {

		temp_account_details = g_new0(PrefsAccount, 1);
		ui_dbus_get_account_settings(acctname, type, &temp_account_details);
		if (temp_account_details != NULL) {
			ui_accMgmt_editinfo->receive_settings.is_not_editable =
			    temp_account_details->receive_settings.is_not_editable;
			prefaccount_local->receive_settings.is_not_editable =
			    temp_account_details->receive_settings.is_not_editable;
		}
		ui_free_prefs_account_structure(temp_account_details);
		temp_account_details = NULL;
	}
	if (ui_accmgmt_copyprefsaccount(prefaccount_local) == TRUE) {
		ui_free_prefs_account_structure(prefaccount_local);
		prefaccount_local = NULL;
	} else {
		ui_free_prefs_account_structure(prefaccount_local);
		prefaccount_local = NULL;
		g_free(acctname);
		return;
	}
	gtk_widget_hide((GtkWidget *) data);
	window1 = (GtkWidget *) ui_accmgmt_createemailaccountsetup1(TRUE);
	if (window1 != NULL) {
		gtk_widget_show_all(window1);
	}
	ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window1);
	if (gActMgtMngr->edit_operation == FALSE) {

		/*Saving the default values of Advanced setting to ui_accMgmt_accountinfo structure. */
		ui_accmgmt_default_advancedvalues();
	} else {
		/*Saving the existing values of Advanced setting to ui_accMgmt_accountinfo structure. */
		ui_accmgmt_save_advancedvalues();
	}
	g_free(acctname);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}


/**
This function is a helper function. It is called from Account Management module.

@param acct_name , which holds the Account Name.
		It is not Freed in this function.
@param win_flag, which holds the flag value
@return void
*/
void ui_accmgmt_display_dialog_for_feilds(const gchar * acct_name, guint win_flag)
{
	PrefsAccount *prefaccount_local = NULL;
	SettingType type;
	GtkWidget *window = NULL, *window1 = NULL, *window2 = NULL, *window3 =
	    NULL, *window4 = NULL;
	AdvOptionViewInfo *advOptionViewInfo = NULL;
	gchar *account_name = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;
	GtkWidget *user_name_entry = NULL;
	PrefsAccount *temp_account_details = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);


	account_name = g_strdup(acct_name);

	ULOG_DEBUG("[%s][%s] : Enters here :\n", __FILE__, __FUNCTION__);
	if (acct_name == NULL) {
		ULOG_DEBUG("Input param checking failed\n");
		return;
	}
	ui_accmgmt_initialize_global_variable(FALSE);
	type = USER_SETTING | SEND_SETTING | RECEIVE_SETTING | ADVANCED_SETTING;

	/* Freeing ui_accMgmt_editinfo structure pointer. */
	ui_free_prefs_account_structure(ui_accMgmt_editinfo);
	ui_accMgmt_editinfo = NULL;

	prefaccount_local = ui_get_account_detail(account_name);

	/*param checking. */
	if (prefaccount_local == NULL) {
		ULOG_DEBUG("[%s][%s] :Dbus call Faliure :\n", __FILE__, __FUNCTION__);
		return;
	}

	if (prefaccount_local->receive_settings.is_not_editable == FALSE) {

		temp_account_details = g_new0(PrefsAccount, 1);
		ui_dbus_get_account_settings(acct_name, type, &temp_account_details);
		prefaccount_local->receive_settings.is_not_editable =
		    temp_account_details->receive_settings.is_not_editable;
		ui_free_prefs_account_structure(temp_account_details);
		temp_account_details = NULL;
	}
	/* Creating the Edit Account dialog */
	window =
	    (GtkWidget *) ui_accmgmt_createemailaccountsedit(email_main_view->
							     account_manager->configured_accounts);

	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);

	email_main_view->account_manager->is_mailbox_opened = FALSE;

	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	ui_accMgmt_editinfo->receive_settings.is_not_editable =
	    prefaccount_local->receive_settings.is_not_editable;

	ui_accmgmt_copyprefsaccount(prefaccount_local);
	ui_free_prefs_account_structure(prefaccount_local);
	prefaccount_local = NULL;

	window1 = (GtkWidget *) ui_accmgmt_createemailaccountsetup1(TRUE);
	if (window1 == NULL) {
		ULOG_DEBUG("[%s][%s] Memory allocation failed:\n", __FILE__, __FUNCTION__);
		return;
	} else {
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window1);
	}
	parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");

	/* Saving account infomation in ui_accMgmt_accountinfo structure. */
	ui_accmgmt_copy_account_setting(SAVE_SETUP_ALL);
	ui_accmgmt_save_advancedvalues();

	window2 = (GtkWidget *) ui_accmgmt_createemailaccountsetup2(TRUE);
	if (window2 == NULL) {
		ULOG_DEBUG("[%s][%s] : Memory allocation failed:\n", __FILE__, __FUNCTION__);
		return;
	} else {
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window2);
		user_name_entry = EMAIL_UI_LOOKUP_OBJECT(window2, "userNameEntry");
	}
	window3 = (GtkWidget *) ui_accmgmt_createemailaccountsetup3(TRUE);
	if (window3 == NULL) {
		ULOG_DEBUG("[%s][%s] : Memory allocation failed:\n", __FILE__, __FUNCTION__);
		return;
	} else {
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window3);
	}
	window4 = (GtkWidget *) ui_accmgmt_createemailaccountsetup4(TRUE);
	if (window4 == NULL) {
		ULOG_DEBUG("[%s][%s] :Memory allocation failed:\n", __FILE__, __FUNCTION__);
		return;
	} else {
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window4);
	}
	advOptionViewInfo = (AdvOptionViewInfo *) ui_accmgmt_createadvancedsettingdialog(TRUE);
	if (advOptionViewInfo->dialog_advancedsetup == NULL) {
		ULOG_DEBUG("[%s][%s] :Failed at adv window:\n", __FILE__, __FUNCTION__);
		return;
	} else {
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, advOptionViewInfo);
	}
	switch (win_flag) {
	case ACCOUNT_DIALOG1:
		gtk_widget_show_all(window1);
		break;
	case ACCOUNT_DIALOG2:
		gtk_widget_show_all(window2);
		gtk_widget_grab_focus(user_name_entry);
		gtk_editable_select_region((GtkEditable *) user_name_entry, 0, -1);
		break;
	case ACCOUNT_DIALOG3:
		gtk_widget_show_all(window3);
		break;
	case ACCOUNT_DIALOG4:
		gtk_widget_show_all(window4);
		break;
	case RETRIEVAL_DIALOG:
		gtk_notebook_set_current_page((GtkNotebook *) advOptionViewInfo->notebook1, 0);
		gtk_widget_show(advOptionViewInfo->dialog_advancedsetup);
		break;
	case SENDING_DIALOG:
		gtk_notebook_set_current_page((GtkNotebook *) advOptionViewInfo->notebook1, 1);
		gtk_widget_show(advOptionViewInfo->dialog_advancedsetup);
		break;
	case SMTPSERVER_DIALOG:
		gtk_notebook_set_current_page((GtkNotebook *) advOptionViewInfo->notebook1, 1);
		gtk_widget_show(advOptionViewInfo->dialog_advancedsetup);
		break;
	case OTHERS_DIALOG:
		gtk_notebook_set_current_page((GtkNotebook *) advOptionViewInfo->notebook1, 3);
		gtk_widget_show(advOptionViewInfo->dialog_advancedsetup);
		break;
	default:
		ULOG_DEBUG("[%s][%s] : Dialog Error:\n", __FILE__, __FUNCTION__);
		break;
	}
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function will be called by Account Management module. 
This function is used to create message dialog to provide user information and warnings.

@param widget pointer to GtkWidget  contains pointer to its parent window.
@param message pointer to the text which is going to be shown in the note.
@return void
*/
void ui_accmgmt_dialogmessagenext(GtkWidget * window, gchar * message)
{
	GtkWidget *dialog = NULL;
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if ((window == NULL) || (message == NULL)) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}

	dialog =
	    gtk_message_dialog_new((GtkWindow *) window,
				   GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO,
				   GTK_BUTTONS_OK, message);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function  will be called by Account Management module. 
This function is used to create the  Cancel Dialog.

@param widget pointer to GtkWidget  contains pointer to its parent window.
@return void
*/
void ui_accmgmt_dialogmessagecancel(GtkWidget * window)
{
	/*Variable declarations. */
	HildonNote *note = NULL;
	gint response;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Input param checking. */
	if (window == NULL) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}

	note =
	    HILDON_NOTE(hildon_note_new_confirmation_add_buttons
			(GTK_WINDOW(window), _("imum_nc_wizard_confirm_lose_changes"),
			 _("imum_bd_lose_changes_ok"), GTK_RESPONSE_ACCEPT,
			 _("imum_bd_lose_changes_cancel"), GTK_RESPONSE_CANCEL, NULL));
	response = gtk_dialog_run(GTK_DIALOG(note));

	if (response == GTK_RESPONSE_ACCEPT) {
		freeglobalmemory();
		gtk_widget_destroy(GTK_WIDGET(note));
		if (email_main_view->account_manager->action_after_acct_creation == MAILBOX_ERROR) {
			email_main_view->account_manager->action_after_acct_creation = NO_ACTION;
			ui_set_error_msg_cb(email_main_view->account_manager->
					    mailbox_error_code,
					    email_main_view->account_manager->error_info1,
					    email_main_view->account_manager->error_info2);
		}
		ui_on_outbox_cancel_send();
	} else if (response == GTK_RESPONSE_DELETE_EVENT) {
		gtk_widget_destroy(GTK_WIDGET(note));
		email_main_view->account_manager->esc_sgnl_handled = TRUE;
		ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
		return;

	} else {
		gtk_widget_destroy(GTK_WIDGET(note));
		ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
		return;
	}
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function  will be called by Account Management module. 
This function is used to create and show the dialog box corresponding to Cancel operation.

@param widget pointer to GtkWidget  contains pointer to its parent window.
@return void
*/
void ui_accmgmt_dialogmessagecancel1(GtkWidget * window)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (window == NULL) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	freeglobalmemory();
	if (email_main_view->account_manager->action_after_acct_creation == MAILBOX_ERROR) {
		email_main_view->account_manager->action_after_acct_creation = NO_ACTION;
		ui_set_error_msg_cb(email_main_view->account_manager->mailbox_error_code,
				    email_main_view->account_manager->error_info1,
				    email_main_view->account_manager->error_info2);
	}
	ui_on_outbox_cancel_send();

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function  will be called by Account Management module. This function is used to create and
show the  dialog box corresponding to Cancel operation.

@param widget pointer to GtkWidget  contains pointer to its parent window.
@return void
*/
void ui_accmgmt_dialogmessagecancel2(GtkWidget * window)
{
	/* Variable declarations. */
	GtkWidget *setupDialogs = NULL;
	GSList *tempList = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (window == NULL) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	tempList = ui_accMgmt_listptr;

	/* Freeing List Contents */
	while (tempList) {
		if (tempList->data != NULL) {
			setupDialogs = tempList->data;
			gtk_widget_destroy(setupDialogs);
		}
		tempList = g_slist_next(tempList);
	}
	g_slist_free(ui_accMgmt_listptr);
	ui_accMgmt_listptr = NULL;
	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function  will be called by Account Management module. 
It will create the Delete Dialog.

@param widget pointer to GtkWidget  contains pointer to its parent window.
@return gboolean
*/
static GtkWidget *ui_accmgmt_dialogmessagedelete(GtkWidget * window)
{
	/* Variable declarations. */
	gchar *delete_message = NULL;
	gchar *device_name = NULL;
	HildonNote *note = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	/* Input Param Checking */
	CHECK_PARAM((window == NULL), "Invalid Input Parameter", return FALSE);

	/*Memory allocation for Delete Message. */
	/*Calling ui_get_device_name () to get the device name. */
	device_name = ui_get_device_name();
	delete_message = g_strdup_printf(_("emev_nc_delete_mailbox"), device_name);
	g_free(device_name);

	note = HILDON_NOTE(hildon_note_new_confirmation(GTK_WINDOW(app), delete_message));
	hildon_note_set_button_text(note, _("emev_bd_delete_mailbox_delete"));


	g_free(delete_message);

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return ((GtkWidget *) note);
}

/**
This function is a callback function which will be called by Account Management
module to delete an existing email account.

@param widget pointer to GtkWidget  contains pointer  to widget on which it is  
              called.
@param data pointer to GSList ,contains pointers to GtkWidgets.
@return void
*/
void accountsetupon_delete_cb(GtkWidget * win, GtkWidget * data)
{

	/* Variable declarations. */
	GtkTreeIter iter;
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection = NULL;
	GtkWidget *accountListView = NULL;
	GSList *accountList = NULL;
	gboolean is_acct_def = FALSE;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;
	GtkTreePath *path = NULL;
	gchar *accountname = NULL;
	gchar *defaultaccount = NULL;
	GtkWidget *note = NULL;
	gint confirmation_status = -1;
	gint account_status = -1;
	PrefsAccount *prefaccount_local = NULL;
	gint *indices = NULL;
	gint row_index = -1;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");


	SettingType type = USER_SETTING | SEND_SETTING | RECEIVE_SETTING | ADVANCED_SETTING;
	accountList = email_main_view->account_manager->configured_accounts;

	if ((win == NULL) || (data == NULL)) {
		ULOG_DEBUG("Input  param check failed [%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	accountListView = EMAIL_UI_LOOKUP_OBJECT((GtkWidget *) data, "accountListTreeView");
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(accountListView));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, 0, &accountname, 2, &is_acct_def, -1);
	} else {
		ULOG_DEBUG("[%s][%s]:Tree Selection failed:\n", __FILE__, __FUNCTION__);
		return;
	}
	path = gtk_tree_model_get_path(model, &iter);
	/*indices is used to select the next account afetr deleting an account */
	indices = gtk_tree_path_get_indices(path);
	row_index = indices[0];
	gtk_tree_path_free(path);
	path = NULL;

	if (email_main_view->account_manager->operation_status != DELETE_OP) {
		note = ui_accmgmt_dialogmessagedelete((GtkWidget *) (data));
		confirmation_status = gtk_dialog_run(GTK_DIALOG(note));
	} else {
		confirmation_status = GTK_RESPONSE_OK;
		email_main_view->account_manager->operation_status = NONE_OP;
	}

	if (confirmation_status == GTK_RESPONSE_OK) {
		if (is_acct_def == TRUE) {
			if (ui_compare_utf8_strings(accountname,
						    email_main_view->account_manager->
						    configured_accounts->data) == 0) {
				if (email_main_view->account_manager->configured_accounts->next !=
				    NULL) {
					defaultaccount =
					    g_strdup(email_main_view->account_manager->
						     configured_accounts->next->data);
				} else {
					defaultaccount = NULL;
				}
			} else {
				defaultaccount =
				    g_strdup(email_main_view->account_manager->
					     configured_accounts->data);
			}

			/*Get the account detail, the PrefsAccount structure  of defaultaccount */
			if (defaultaccount == NULL
			    || (defaultaccount != NULL && strlen(defaultaccount) == 0)) {
				g_free(email_main_view->account_manager->default_account);
				email_main_view->account_manager->default_account = NULL;
			} else {
				prefaccount_local = ui_get_account_detail(defaultaccount);
				if (prefaccount_local == NULL) {
					ULOG_DEBUG(" Get Account Setting Failed\n");
					g_free(accountname);
					g_free(defaultaccount);
					return;
				}
				prefaccount_local->user_settings.is_default = TRUE;
				if (ui_dbus_set_account_settings(type, prefaccount_local) ==
				    SETTING_SUCCESS) {
					ui_init_account_store();
					ULOG_DEBUG("Account addition succesfull!\n");
				}
				ui_free_prefs_account_structure(prefaccount_local);
				prefaccount_local = NULL;
				g_free(email_main_view->account_manager->default_account);
				email_main_view->account_manager->default_account = NULL;
				email_main_view->account_manager->default_account =
				    g_strdup(defaultaccount);
				g_free(defaultaccount);
			}
		}
		if (email_main_view->account_manager->current_sendrec_account != NULL
		    && strlen(email_main_view->account_manager->current_sendrec_account) != 0) {
			account_status = ui_compare_utf8_strings(accountname,
								 email_main_view->account_manager->
								 current_sendrec_account);
		}

		/* If Send/Receive operation is going on the present account. */
		if (account_status == 0) {
			/* Dbus call to cancel Sending operation. */
			ui_dbus_cancel_send_mail(NULL);
			/* Dbus call to cancel Receiving operation. */
			ui_dbus_cancel_recv_mail(accountname, TRUE);

			/* Setting the kind of operation while Send/Receive is going on. */
			email_main_view->account_manager->operation_status = DELETE_OP;
			email_main_view->account_manager->edit_account_dialog = parentWindow;
			gtk_widget_destroy(note);
			g_free(accountname);
			return;
		}

		/*Call the AL function for account deletion */
		if (ui_dbus_delete_account(accountname)) {
			ui_init_account_store();
			ULOG_DEBUG("\n\nAccount deletion succesfull!%s\n", accountname);
			/* Delete the account in the main UI */
			ui_remove_mail_account(accountname);
			/* Delete the account from the signatures */
			delete_signature_file(accountname);

			if (email_main_view->account_manager->configured_accounts != NULL) {
				ui_accmgmt_update_liststore();
			}

			model = gtk_tree_view_get_model(GTK_TREE_VIEW(accountListView));
			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(accountListView));
			if (gtk_tree_model_iter_n_children(model, NULL) > row_index + 1)
				path = gtk_tree_path_new_from_indices(row_index, -1);
			else
				path = gtk_tree_path_new_from_indices(row_index - 1, -1);
			if ((path != NULL) && (gtk_tree_model_get_iter(model, &iter, path))) {
				gtk_tree_selection_select_iter(selection, &iter);
			}
			gtk_tree_path_free(path);
		} else {
			ULOG_DEBUG("Account deletion failed!\n");
		}

	} else if (confirmation_status == GTK_RESPONSE_DELETE_EVENT) {
		ULOG_DEBUG("Deletion canceled");
		email_main_view->account_manager->esc_sgnl_handled = TRUE;
	} else {
		ULOG_DEBUG("Deletion canceled");
	}
	if (email_main_view->account_manager->configured_accounts == NULL) {
		freeglobalmemory();
	}
	gtk_widget_destroy(note);

	g_free(accountname);

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This function is a helper function which will be called while setting one account default or deleting 
 any existing account.

 @return void
*/
static void ui_accmgmt_update_liststore(void)
{
	/* Variable declarations. */
	GSList *accountname = NULL;
	GtkTreeIter iter2;

	accountname = email_main_view->account_manager->configured_accounts;
	gtk_list_store_clear(folderListStore);

	/* Traverse the whole list of accounts stored in device. */
	while (accountname != NULL) {
		gtk_list_store_append(folderListStore, &iter2);
		if (ui_compare_utf8_strings
		    (accountname->data, email_main_view->account_manager->default_account) == 0) {
			gtk_list_store_set(folderListStore, &iter2, 0, accountname->data, 1,
					   _("mcen_ia_emailsetup_default"), 2, TRUE, -1);
		} else {
			gtk_list_store_set(folderListStore, &iter2, 0, accountname->data, 2,
					   FALSE, -1);
		}
		accountname = accountname->next;
	}
}

/**
This function is a callback function which will be called by Account Management
module to delete an existing email account.

@param win pointer to GtkWidget  contains pointer  to widget on which it is 
              called.
@param data pointer to gchar ,contains pointers to accountname.
@return void
*/

void ui_accmgmt_onsetdefault_cb(GtkWidget * win, gpointer data)
{
	/* Variable declarations. */
	gchar *accountname = NULL;
	GtkTreeIter iter;
	GtkTreeModel *model = NULL;
	GtkTreeModel *accountViewModel = NULL;
	GtkTreeSelection *selection = NULL;
	GtkWidget *accountListView = NULL;
	gboolean is_account_def;
	PrefsAccount *prefaccount_local = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;
	SettingType type;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	accountViewModel = GTK_TREE_MODEL(folderListStore);
	accountListView = EMAIL_UI_LOOKUP_OBJECT((GtkWidget *) data, "accountListTreeView");
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(accountListView));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, 0, &accountname, 2, &is_account_def, -1);
	} else {
		ULOG_DEBUG("[%s][%s] : No account is selected :\n", __FILE__, __FUNCTION__);
		return;
	}
	if (is_account_def == TRUE) {
		g_free(accountname);
		return;
	}
	if ((win == NULL)
	    || (email_main_view->account_manager->default_account == NULL)) {
		ULOG_DEBUG("Default account is Null\n");
		g_free(accountname);
		return;
	}

	prefaccount_local = ui_get_account_detail(accountname);
	if (prefaccount_local == NULL) {
		ULOG_DEBUG(" Get Account Setting Failed  [%s] at [%d]\n", __FUNCTION__, __LINE__);
		g_free(accountname);
		return;
	}

	type = USER_SETTING | SEND_SETTING | RECEIVE_SETTING | ADVANCED_SETTING;
	prefaccount_local->user_settings.is_default = TRUE;
	if (ui_dbus_set_account_settings(type, prefaccount_local) == SETTING_SUCCESS) {
		ULOG_DEBUG(" Get Account Setting Failed  [%s] at [%d]\n", __FUNCTION__, __LINE__);

		ui_init_account_store();
	}
	ui_free_prefs_account_structure(prefaccount_local);

	prefaccount_local =
	    ui_get_account_detail(email_main_view->account_manager->default_account);
	if (prefaccount_local == NULL) {
		ULOG_DEBUG(" Get Account Setting Failed  [%s] at [%d]\n", __FUNCTION__, __LINE__);
		g_free(accountname);
		return;
	}
	prefaccount_local->user_settings.is_default = FALSE;
	if (ui_dbus_set_account_settings(type, prefaccount_local) == SETTING_SUCCESS) {
		ui_init_account_store();
		ULOG_DEBUG("Account addition succesfull!\n");
	}
	ui_free_prefs_account_structure(prefaccount_local);

	/*resetting the default account */
	if (email_main_view->account_manager->default_account != NULL)
		g_free(email_main_view->account_manager->default_account);
	email_main_view->account_manager->default_account = g_strdup(accountname);
	ui_accmgmt_update_liststore();
	ui_accmgmt_set_selected_accountview(selection, accountViewModel);
	g_free(accountname);

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
This function  will be called by Account Management module to free 
the memory allocated to Prefsaccount structure pointer.

@param  pstruct1 pointer to Prefsaccount structure  .
@return  void  
*/

void ui_accmgmt_freeprefsaccount(PrefsAccount * ipptr)
{
	/* Input param checking. */
	if (ipptr == NULL) {
		ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__, __LINE__);
		return;
	}
	g_free(ipptr->user_settings.account_name);
	ipptr->user_settings.account_name = NULL;
	g_free(ipptr->user_settings.userid);
	ipptr->user_settings.userid = NULL;
	g_free(ipptr->user_settings.passwd);
	ipptr->user_settings.passwd = NULL;
	g_free(ipptr->user_settings.email);
	ipptr->user_settings.email = NULL;
	g_free(ipptr->user_settings.alias_name);
	ipptr->user_settings.alias_name = NULL;
	g_free(ipptr->receive_settings.recv_server);
	ipptr->receive_settings.recv_server = NULL;
	g_free(ipptr->send_settings.smtp_server);
	ipptr->send_settings.smtp_server = NULL;
	g_free(ipptr->send_settings.smtp_server);
	ipptr->send_settings.smtp_server = NULL;
	g_free(ipptr->send_settings.smtp_server);
	ipptr->send_settings.smtp_server = NULL;
#if CL_USE
	g_free(ipptr->advanced_settings.iap_name);
#endif
	g_free(ipptr);

	return;
}

/**
This function  will be called by Account Management module to 
initialize ui_accMgmt_accountinfo structure of type   PrefsAccount .

@param  void .
@return  void.
*/
void ui_acctmgmt_initialize(PrefsAccount ** ui_accMgmt_accountinfo)
{
	(*ui_accMgmt_accountinfo)->user_settings.account_name = NULL;
	(*ui_accMgmt_accountinfo)->user_settings.userid = NULL;
	(*ui_accMgmt_accountinfo)->user_settings.passwd = NULL;
	(*ui_accMgmt_accountinfo)->user_settings.email = NULL;
	(*ui_accMgmt_accountinfo)->user_settings.alias_name = NULL;
	(*ui_accMgmt_accountinfo)->receive_settings.recv_server = NULL;
	(*ui_accMgmt_accountinfo)->send_settings.smtp_server = NULL;
	(*ui_accMgmt_accountinfo)->advanced_settings.imap_dir = NULL;
	(*ui_accMgmt_accountinfo)->receive_settings.protocol = -1;
}

/**
This function  will be called by Account Management module to free the memory al
located to Prefsaccount structure pointer.

@param   address  pointer to char which contains email address  .
@return  int which is 1 if it is a valid email address else 0.  
*/
int ui_accmgmt_email_isvalid(const gchar * address)
{
	int count = 0, ctr = 0, inc = 0, dot = 0;
	const gchar *tempaddress = NULL, *domain = NULL;
	static gchar *rfc822_specials = "()<>@,;:\\\"[]";

	/* Input param checking. */
	if (address == NULL)
		return 0;

	while ((*(address + inc) != '@') && (*(address + inc))) {
		inc++;
	}
	while (*(address + inc) != '\0') {
		if (*(address + inc) == '.') {
			dot++;
		}
		inc++;
	}
	inc--;
	if (*(address + inc) == '.')
		return 0;
	if (dot == 0)
		return 0;

	/* first we validate the name portion (name@domain) */
	for (tempaddress = address; *tempaddress; tempaddress++) {
		if (*tempaddress == '\"'
		    && (tempaddress == address || *(tempaddress - 1) == '.'
			|| *(tempaddress - 1) == '\"')) {
			while (*++tempaddress != '\0') {
				if (*tempaddress == '\"') {
					printf("invalid");
					break;
				}
				if (*tempaddress == '\\' && (*++tempaddress == ' '))
					continue;
				if (*tempaddress <= ' ' || (int) *tempaddress >= 127) {
					printf("Valid");
					return 0;
				}
			}
			if (!*tempaddress++)
				return 0;
			if (*tempaddress == '@')
				break;
			if (*tempaddress != '.')
				return 0;
			continue;
		}
		if (*tempaddress == '@')
			break;
		if (*tempaddress == ' ')
			return 0;
		if (strchr(rfc822_specials, *tempaddress))
			return 0;
	}
	if (tempaddress == address || *(tempaddress - 1) == '.')
		return 0;
	/* next we validate the domain portion (name@domain) */
	if (!*(domain = ++tempaddress))
		return 0;
	do {
		if (*tempaddress == '.') {	/*ASCII Charaters Checking has removed */
			if (tempaddress == domain || *(tempaddress - 1) == '.') {
				return 0;
			}
			count++;
		}
		if (*tempaddress == ' ')	/* ASCII Characters Checking has removed. */
			return 0;
		if (strchr(rfc822_specials, *tempaddress)) {
			return 0;
		}
		ctr++;
		/* Domain Validation is removed */
	}
	while (*++tempaddress != '\0');
	return (count >= 1);
}

/**
This function is  called from  Account Management module and Main Ui module . 
This function is used to  free the memory allocated for global  variables used in account managemnet module .
It frees ui_accMgmt_accountinfo & ui_accMgmt_editinfo global structure pointers.

@param void.
@return void.
*/
static void freeglobalmemory(void)
{
	/* Variable declarations. */
	GSList *tempList = NULL;
	GtkWidget *setupDialogs = NULL;
	AdvOptionViewInfo *tempAdvInfo = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parent_dialog = NULL;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	if (g_slist_nth((GSList *) ui_accMgmt_listptr, 1) != NULL) {
		parent_dialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
		ui_accMgmt_accountinfo =
		    g_object_get_data(G_OBJECT(parent_dialog), "EmailAccountInfo");
	}

	/* Free PrefsAccount structure pointer ui_accMgmt_accountinfo. */
	ui_accmgmt_freeprefsaccount(ui_accMgmt_accountinfo);
	if (gActMgtMngr->edit_operation) {
		ui_accmgmt_freeprefsaccount(ui_accMgmt_editinfo);
	}
	ui_accMgmt_accountinfo = NULL;
	ui_accMgmt_editinfo = NULL;

	/*Freeing List Contents */
	tempList = ui_accMgmt_listptr;
	while (tempList) {
		if (tempList->data != NULL) {
			if (GTK_IS_WIDGET(tempList->data) == TRUE) {
				setupDialogs = GTK_WIDGET(tempList->data);
				gtk_widget_destroy(setupDialogs);
			} else {
				tempAdvInfo = (AdvOptionViewInfo *) (tempList->data);
				if (tempAdvInfo->dialog_advancedsetup != NULL) {
					gtk_widget_destroy(tempAdvInfo->dialog_advancedsetup);
				}
			}

		}
		tempList = g_slist_next(tempList);
	}

	g_slist_free(ui_accMgmt_listptr);
	ui_accMgmt_listptr = NULL;
}

/**
This function is called in Account Management module.This function is used for 
validation checking of user name , password and email address of an account.

@param  window, pointer to GtkWidget which contains pointer to parent window 
on which this function is called .
@return gboolean , Returns FALSE  when validation is failed, 
		   TRUE   otherwise.  
*/

gboolean ui_accmgmt_emptycheck_onsetup2(GtkWidget * window)
{
	/* Variable declarations. */
	GtkWidget *entry1 = NULL;
	GtkWidget *entry2 = NULL;
	GtkWidget *entry3 = NULL;
	GtkWidget *entry4 = NULL;
	const gchar *emailid = NULL, *id_user = NULL, *temp_userid = NULL;
	gint email_checking = -1;
	GtkWidget *parent_dialog = NULL;

	parent_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 2)->data);
	entry1 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "nameEntry");
	entry2 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "userNameEntry");
	entry3 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "passWdEntry");
	entry4 = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "emailEntry");
	temp_userid = gtk_entry_get_text((GtkEntry *) entry2);
	id_user = temp_userid;

	emailid = gtk_entry_get_text((GtkEntry *) entry4);
	email_checking = ui_accmgmt_email_isvalid(emailid);

	/* If the given emailid is not valid one. */
	if (email_checking != 1) {
		gtk_widget_grab_focus(entry4);
		gtk_editable_select_region((GtkEditable *) entry4, 0, -1);
		gtk_infoprint((GtkWindow *) window, _("mcen_ib_not_valid_email"));
		return FALSE;
	}
	return TRUE;
}

/**
 
This function is  called in  Account Management module.
This function is used to check validation of account name. 
This function will free the memory allocated to the incoming string parameter.
@param void .
@return gboolean, This function returns FALSE when validation is failed, 
		   otherwise TRUE
*/
static gboolean ui_accmgmt_emptycheck_onsetup1(gchar * account_name)
{
	/* Variable declarations. */
	gchar *acctname = NULL, *acct_name = NULL;
	GtkWidget *accountNameEntry = NULL;
	GtkWidget *parent_dialog_setup1 = NULL;
	GtkWidget *emailtyp_combo = NULL;
	gint acct_type = -1;
	gActMgtMngr->is_setup1_modified = FALSE;
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;
	gchar *restricted_char_mesg = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	/* Input param checking. */
	CHECK_PARAM((account_name == NULL), "Invalid input Parameter", return FALSE);

	parent_dialog_setup1 = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 1)->data);
	accountNameEntry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog_setup1, "accountNameEntry");
	emailtyp_combo = EMAIL_UI_LOOKUP_OBJECT(parent_dialog_setup1, "comboAccountType");

	acctname = g_strdup(gtk_entry_get_text((GtkEntry *) accountNameEntry));
	acct_type = gtk_combo_box_get_active(GTK_COMBO_BOX(emailtyp_combo));

	if (gActMgtMngr->edit_operation == FALSE) {
		/* Check whether default setup1 setting has changed */
		if ((ui_compare_utf8_strings(acctname, account_name) != 0)
		    || (acct_type != POP3_ACCT)) {
			gActMgtMngr->is_setup1_modified = TRUE;
		}
	} else {
		if (ui_compare_utf8_strings
		    (acctname, ui_accMgmt_editinfo->user_settings.account_name) != 0) {
			gActMgtMngr->is_setup1_modified = TRUE;
		}
	}
	/* Checking for empty account name. */
	if (((gtk_entry_get_text((GtkEntry *) accountNameEntry)) == NULL)
	    || (g_utf8_strlen(acctname, -1) == 0)) {
		gtk_infoprint(ui_accMgmt_listptr->next->data, _("mcen_ib_enter_accountname"));
		g_free(acctname);
		g_free(account_name);
		account_name = NULL;
		return FALSE;
	}
	acct_name = acctname;

	/*If Account Name contains some invalid characters. */
	if (ui_accMgmt_userinfo_isvalid(acct_name, TRUE) == FALSE) {
		restricted_char_mesg =
		    g_strdup_printf(_("mcen_ib_illegal_chars_error"), RESTRICTED_CHARS);
		gtk_infoprint(g_slist_nth(ui_accMgmt_listptr, 1)->data, restricted_char_mesg);
		g_free(restricted_char_mesg);
		g_free(account_name);
		return FALSE;
	}
	g_free(account_name);
	g_free(acct_name);
	return TRUE;
}

/**
This function is  called in  Account Management module.This function take care  
about the sensitivity of username and password field in SMTP Server dialog.  

@param  editable which is a pointer to the widget on which this function is called.
@param  data which is pointer to gpointer which contains NULL.
@return void 
*/

void ui_accmgmt_onsmtpauthtype_cb(GtkEditable * editable, gpointer data)
{
	/* Variable declarations. */
	GtkWidget *combo_smtpauth = NULL;
	gint smtp_authtype = 0;
	GtkWidget *entry_user_name = NULL;
	GtkWidget *entry_passwd = NULL;
	GtkWidget *checkbutton_ssl = NULL;
	SMTPTab *smtpTab = NULL;

	if (editable == NULL) {
		return;
	}
	smtpTab = (SMTPTab *) data;

	checkbutton_ssl = smtpTab->checkbutton_smtpauth;
	combo_smtpauth = smtpTab->combo_smtpauth;
	entry_user_name = smtpTab->entry_user_name;
	entry_passwd = smtpTab->entry_passwd;
	smtp_authtype = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_smtpauth));

	/* If SMTP AUTH checkbutton is not checked in. */
	if (smtp_authtype == 0) {
		gtk_widget_set_sensitive(entry_user_name, FALSE);
		gtk_widget_set_sensitive(entry_passwd, FALSE);
	} else {
		gtk_widget_set_sensitive(entry_user_name, TRUE);
		gtk_widget_set_sensitive(entry_passwd, TRUE);
	}
}

/**
This function is  called in  Account Management module.This function is used to
check validation of user information such as userid, password, accountname etc.

@param userinfo which is a point to gchar, contains user information .
		It is not freed in this function.
@param acct_flag . acct_flag is set to TRUE when userinfo is account_name ,
                   FALSE otherwise.
@return gboolean, This function returns FALSE when validation is failed, TRUE
                  otherwise
*/
static gboolean ui_accMgmt_userinfo_isvalid(gchar * userinfo, gboolean acct_flag)
{
	/* Variable declarations. */
	const gchar *restricted_chars = "<>^:?/*\"|\\";
	const gchar *temp_str = NULL;
	ULOG_DEBUG("[%s][%s] : Starts here :\n", __FILE__, __FUNCTION__);

	/* Input param checking. */
	CHECK_PARAM((userinfo == NULL), "Invalid input Parameter", return FALSE);

	userinfo = g_strdelimit(userinfo, restricted_chars, '^');
	temp_str = strchr(userinfo, '^');
	if (temp_str != NULL) {
		return FALSE;
	}
	ULOG_DEBUG(" This is a valid Account");

	ULOG_DEBUG("[%s][%s] : Ends here :\n", __FILE__, __FUNCTION__);

	return TRUE;
}

/**
This function is  called in  Account Management module.This function is used to
check validation of receiving settings information etc.

@param account_info which is a point to Prefsaccount structure, contains information about an account .
@return gboolean, This function returns FALSE when validation is failed, TRUE
                  otherwise
*/
static gboolean ui_accMgmt_validate_receive_settings(PrefsAccount * account_info)
{
	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if ((account_info->receive_settings.protocol < PROTO_POP3)
	    || (account_info->receive_settings.protocol > PROTO_INVALID)
	    || (account_info->receive_settings.passwd_auth_type < AUTH_LOGIN)
	    || (account_info->receive_settings.passwd_auth_type > AUTH_APOP)
	    || (account_info->receive_settings.recv_type < HEADERS)
	    || (account_info->receive_settings.recv_type > BODY_ATTCH)
	    || (account_info->receive_settings.size_limit < MIN_SIZELIMIT)
	    || (account_info->receive_settings.size_limit > MAX_SIZELIMIT)) {
		ULOG_DEBUG("\nPROTOCOL %d\n", account_info->receive_settings.protocol);
		ULOG_DEBUG("\nRECEIVE TYPE %d\n", account_info->receive_settings.recv_type);
		ULOG_DEBUG("\nSIZELIMIT %d\n", account_info->receive_settings.size_limit);
		ULOG_DEBUG("\n PASSWORD AUTHENTICATION %d\n",
			   account_info->receive_settings.passwd_auth_type);
		return FALSE;
	}

	if (account_info->receive_settings.recent_mails == ALL
	    || account_info->receive_settings.recent_mails == TEN
	    || account_info->receive_settings.recent_mails == TWENTYFIVE
	    || account_info->receive_settings.recent_mails == FIFTY) {
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return TRUE;
	} else {
		ULOG_DEBUG("\n Invalid values in  RECENT_MAILS \n");
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
}

/**
This function is  called in  Account Management module.This function is used to
check validation of advance setting information etc.

@param account_info which is a point to Prefsaccount structure, contains  information about an account .
@return gboolean, This function returns FALSE when validation is failed, TRUE
                  otherwise
*/
static gboolean ui_accMgmt_validate_advance_setting(PrefsAccount * account_info)
{
	gchar *recv_server = NULL, *send_server = NULL;
	gboolean retval = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	recv_server = g_strdup(account_info->receive_settings.recv_server);
	send_server = g_strdup(account_info->send_settings.smtp_server);
	if ((g_utf8_strlen(recv_server, -1) == 0)
	    || (g_utf8_strlen(send_server, -1) == 0)) {
		ULOG_DEBUG("\n Invalid Server address in %s Account\n",
			   account_info->user_settings.account_name);
		g_free(send_server);
		g_free(recv_server);
		return FALSE;
	}
	g_free(send_server);
	g_free(recv_server);

	if ((account_info->advanced_settings.email_type < 0)
	    || (account_info->advanced_settings.email_type > 2)
	    || (account_info->send_settings.smtp_auth_type < 0)
	    || (account_info->send_settings.smtp_auth_type > 2)
	    || (account_info->advanced_settings.email_sending < 0)
	    || (account_info->advanced_settings.email_sending > 3)) {
		ULOG_DEBUG("\nAccount Name %s\n", account_info->user_settings.account_name);
		ULOG_DEBUG("\nEmail-TYPE %d\n", account_info->advanced_settings.email_type);
		ULOG_DEBUG("\nSMTP_AUTH TYPE %d\n", account_info->send_settings.smtp_auth_type);
		ULOG_DEBUG("\nEmailSending %d\n", account_info->advanced_settings.email_sending);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}
	retval = ui_accMgmt_validate_receive_settings(account_info);
	return retval;
}

/**
This function is  called in  Account Management module.This function is used to
check validation of user information such as userid, password, accountname etc.

@param account_info which is a point to Prefsaccount structure, contains  information about an account .
			It is not freed in this function.
@return gboolean, This function returns FALSE when validation is failed, TRUE
                  otherwise
*/
gboolean ui_accMgmt_validate_account(PrefsAccount * account_info)
{
	gchar *accountname = NULL, *user_id = NULL;
	gchar *emailid = NULL;
	gboolean retval = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	accountname = g_strdup(account_info->user_settings.account_name);
	if ((g_utf8_strlen(accountname, -1)) > 64) {
		g_free(accountname);
		return FALSE;
	}
	g_free(accountname);
	user_id = g_strdup(account_info->user_settings.userid);
	if ((g_utf8_strlen(user_id, -1) > MAX_NAME_ENTRY_LEN)
	    || (g_utf8_strlen(user_id, -1) == 0)) {
		ULOG_DEBUG("\n Invalid User Name  in %s Account\n",
			   account_info->user_settings.account_name);
		g_free(user_id);
		return FALSE;
	}
	g_free(user_id);
	emailid = g_strdup(account_info->user_settings.email);
	if (emailid == NULL || g_utf8_strlen(emailid, -1) == 0) {
		ULOG_DEBUG("\n Invalid Email address in %s Account\n",
			   account_info->user_settings.account_name);
		g_free(emailid);
		return FALSE;
	}
	g_free(emailid);
	retval = ui_accMgmt_validate_advance_setting(account_info);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return retval;
}

/*
This is helper function to initialize prefsaccount.

@return nothing.
*/
static void ui_accmgmt_initialize_prefsaccount()
{
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	ui_accMgmt_editinfo->user_settings.account_name = NULL;
	ui_accMgmt_editinfo->user_settings.userid = NULL;
	ui_accMgmt_editinfo->user_settings.passwd = NULL;
	ui_accMgmt_editinfo->user_settings.email = NULL;
	ui_accMgmt_editinfo->user_settings.alias_name = NULL;
	ui_accMgmt_editinfo->receive_settings.recv_server = NULL;
	ui_accMgmt_editinfo->send_settings.smtp_server = NULL;
	ui_accMgmt_editinfo->advanced_settings.imap_dir = NULL;
#if CL_USE
	ui_accMgmt_editinfo->advanced_settings.iap_name = NULL;
#endif

	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/*
This is helper function, when ever prefsaccount is retrieved from engine 
it get copied in to the local prefsaccount variable.
It allocate memory for the prefsAccount structure.

@param s_update_prefsAccount This is the data structure from engine.
@return gboolean return TRUE on success 
                        FALSE on failure.   
*/

gboolean ui_accmgmt_copyprefsaccount(PrefsAccount * s_update_prefsAccount)
{
	/* Variable Declarations. */
	PrefsAccount *ui_accMgmt_editinfo = NULL;
	GtkWidget *parentWindow = NULL;
#if CL_USE
	gboolean iap_status = FALSE;
#endif

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);
	parentWindow = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 0)->data);

	/* Getting the pointer of 
	   ui_accMgmt_editinfo, defined in accountMgmt.c file. */
	ui_accMgmt_editinfo = g_object_get_data(G_OBJECT(parentWindow), "EmailEditInfo");

	/* Input Param Checking. */
	if (s_update_prefsAccount == NULL) {
		ULOG_DEBUG("[%s][%s] : Input param check failed! :\n", __FILE__, __FUNCTION__);
		return FALSE;
	}

	/* Initialize String Variables of PrefsAccount structure. */
	ui_accmgmt_initialize_prefsaccount();

	/* Copying data elements from s_update_prefsAccount
	   to ui_accMgmt_editinfo structure. */
	if (s_update_prefsAccount->user_settings.account_name != NULL) {
		ui_accMgmt_editinfo->user_settings.account_name =
		    g_strdup(s_update_prefsAccount->user_settings.account_name);
	} else {
		ui_accMgmt_editinfo->user_settings.account_name = NULL;
	}
	if (s_update_prefsAccount->user_settings.userid != NULL) {
		ui_accMgmt_editinfo->user_settings.userid =
		    g_strdup(s_update_prefsAccount->user_settings.userid);
	} else {
		ui_accMgmt_editinfo->user_settings.userid = NULL;
	}
	if (s_update_prefsAccount->user_settings.passwd != NULL) {
		ui_accMgmt_editinfo->user_settings.passwd =
		    g_strdup(s_update_prefsAccount->user_settings.passwd);
	} else {
		ui_accMgmt_editinfo->user_settings.passwd = NULL;
	}
	if (s_update_prefsAccount->send_settings.username != NULL) {
		ui_accMgmt_editinfo->send_settings.username =
		    g_strdup(s_update_prefsAccount->send_settings.username);
	} else {
		ui_accMgmt_editinfo->send_settings.username = NULL;
	}
	if (s_update_prefsAccount->send_settings.password != NULL) {
		ui_accMgmt_editinfo->send_settings.password =
		    g_strdup(s_update_prefsAccount->send_settings.password);
	} else {
		ui_accMgmt_editinfo->send_settings.password = NULL;
	}
	if (s_update_prefsAccount->user_settings.email != NULL) {
		ui_accMgmt_editinfo->user_settings.email =
		    g_strdup(s_update_prefsAccount->user_settings.email);
	} else {
		ui_accMgmt_editinfo->user_settings.email = NULL;
	}
	if (s_update_prefsAccount->user_settings.alias_name != NULL) {
		ui_accMgmt_editinfo->user_settings.alias_name =
		    g_strdup(s_update_prefsAccount->user_settings.alias_name);
	} else {
		ui_accMgmt_editinfo->user_settings.alias_name = NULL;
	}
	if (s_update_prefsAccount->receive_settings.recv_server != NULL) {
		ui_accMgmt_editinfo->receive_settings.recv_server =
		    g_strdup(s_update_prefsAccount->receive_settings.recv_server);
	} else {
		ui_accMgmt_editinfo->receive_settings.recv_server = NULL;
	}
	if (s_update_prefsAccount->send_settings.smtp_server != NULL) {
		ui_accMgmt_editinfo->send_settings.smtp_server =
		    g_strdup(s_update_prefsAccount->send_settings.smtp_server);
	} else {
		ui_accMgmt_editinfo->send_settings.smtp_server = NULL;
	}
#if CL_USE
	ui_accMgmt_editinfo->advanced_settings.use_iap_smtp_serv =
	    s_update_prefsAccount->advanced_settings.use_iap_smtp_serv;
	if (s_update_prefsAccount->advanced_settings.iap_name != NULL) {
		iap_status =
		    search_iap_name_iaplist(s_update_prefsAccount->advanced_settings.iap_name);
		if (iap_status == TRUE) {
			ui_accMgmt_editinfo->advanced_settings.iap_name =
			    g_strdup(s_update_prefsAccount->advanced_settings.iap_name);
		} else {
			ui_accMgmt_editinfo->advanced_settings.iap_name = NULL;
		}
	} else {
		ui_accMgmt_editinfo->advanced_settings.iap_name = NULL;
	}
#endif

	ui_accMgmt_editinfo->receive_settings.protocol =
	    s_update_prefsAccount->receive_settings.protocol;
	ui_accMgmt_editinfo->advanced_settings.email_type =
	    s_update_prefsAccount->advanced_settings.email_type;
	ui_accMgmt_editinfo->advanced_settings.email_sending =
	    s_update_prefsAccount->advanced_settings.email_sending;
	ui_accMgmt_editinfo->advanced_settings.request_read_report =
	    s_update_prefsAccount->advanced_settings.request_read_report;
	ui_accMgmt_editinfo->advanced_settings.org_in_reply =
	    s_update_prefsAccount->advanced_settings.org_in_reply;
	ui_accMgmt_editinfo->receive_settings.port = s_update_prefsAccount->receive_settings.port;
	ui_accMgmt_editinfo->user_settings.is_default =
	    s_update_prefsAccount->user_settings.is_default;
	ui_accMgmt_editinfo->receive_settings.recv_type =
	    s_update_prefsAccount->receive_settings.recv_type;
	ui_accMgmt_editinfo->receive_settings.size_limit =
	    s_update_prefsAccount->receive_settings.size_limit;
	ui_accMgmt_editinfo->receive_settings.passwd_auth_type =
	    s_update_prefsAccount->receive_settings.passwd_auth_type;
	ui_accMgmt_editinfo->receive_settings.recent_mails =
	    s_update_prefsAccount->receive_settings.recent_mails;
	ui_accMgmt_editinfo->advanced_settings.use_ssl =
	    s_update_prefsAccount->advanced_settings.use_ssl;
	ui_accMgmt_editinfo->advanced_settings.use_tls =
	    s_update_prefsAccount->advanced_settings.use_tls;
	ui_accMgmt_editinfo->advanced_settings.smtp_use_ssl =
	    s_update_prefsAccount->advanced_settings.smtp_use_ssl;
	ui_accMgmt_editinfo->advanced_settings.smtp_use_tls =
	    s_update_prefsAccount->advanced_settings.smtp_use_tls;
	ui_accMgmt_editinfo->receive_settings.enable_size_limit =
	    s_update_prefsAccount->receive_settings.enable_size_limit;
	ui_accMgmt_editinfo->send_settings.smtp_auth_type =
	    s_update_prefsAccount->send_settings.smtp_auth_type;
	ui_accMgmt_editinfo->advanced_settings.include_signature =
	    s_update_prefsAccount->advanced_settings.include_signature;
	ui_accMgmt_editinfo->send_settings.smtp_auth_type =
	    s_update_prefsAccount->send_settings.smtp_auth_type;
	ui_accMgmt_editinfo->send_settings.smtp_port =
	    s_update_prefsAccount->send_settings.smtp_port;
	ui_accMgmt_editinfo->receive_settings.leave_msg =
	    s_update_prefsAccount->receive_settings.leave_msg;

	/* Adding Security Settings. */
	ui_accMgmt_editinfo->advanced_settings.smime_security =
	    s_update_prefsAccount->advanced_settings.smime_security;
	ui_accMgmt_editinfo->advanced_settings.display_sign_encrypt =
	    s_update_prefsAccount->advanced_settings.display_sign_encrypt;
	ui_accMgmt_editinfo->advanced_settings.display_trust =
	    s_update_prefsAccount->advanced_settings.display_trust;
	ui_accMgmt_editinfo->advanced_settings.display_validity =
	    s_update_prefsAccount->advanced_settings.display_validity;

	ui_accMgmt_editinfo->advanced_settings.signature_flag =
	    s_update_prefsAccount->advanced_settings.signature_flag;
	ui_accMgmt_editinfo->receive_settings.is_not_editable =
	    s_update_prefsAccount->receive_settings.is_not_editable;
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return TRUE;
}

/**
 This function is a callback function. It will be called when user click on insensitive 'Next'
 button in setup1 dialog.It will show a infoprint with the given message.
 
 @param button It is pointer to the button on which user click.
 @param message It points to the message which is going to be shown to the user.
 @return void.
 */
void ui_accmgmt_next_insensitive_press_cb(GtkWidget * button, gchar * message)
{

	GtkWidget *parent_dialog = NULL;
	GtkWidget *account_entry = NULL;

	/* Input param checking. */
	CHECK_PARAM((button == NULL), "Invalid input Parameter", return);
	CHECK_PARAM((message == NULL), "Invalid input Parameter", return);

	parent_dialog = gtk_widget_get_ancestor(GTK_WIDGET(button), GTK_TYPE_DIALOG);
	account_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "accountNameEntry");
	gtk_widget_grab_focus(account_entry);
	gtk_infoprint((GtkWindow *) parent_dialog, message);

}

/**
 * This function is a callback function. It will be called when user click on insensitive  Back button in setup1 dialog.
 * It will show a infoprint with the given message.
 * @param button It is pointer to the button on which user click.
 * @param message It points to the message which is going to be shown to the user.
 * @return void.
 */
void ui_accmgmt_back_insensitive_press_cb(GtkWidget * button, gchar * message)
{
	/* Variable declarations. */
	GtkWidget *parent_dialog = NULL;

	parent_dialog = gtk_widget_get_ancestor(GTK_WIDGET(button), GTK_TYPE_DIALOG);
	gtk_infoprint((GtkWindow *) parent_dialog, message);
}

/**
 * This function is a callback function. It will be called when user click on a insensitive button in setup1 dialog.
 * It will show a infoprint with the given message.
 * @param button It is pointer to the button on which user click.
 * @param message It points to the message which is going to be shown to the user.
 * @return void.
 */
void ui_accmgmt_insensitive_press_cb(GtkWidget * button, gchar * message)
{
	/* Variable declarations. */
	GtkWidget *parent_dialog = NULL;
	GtkWidget *account_entry = NULL;
	GtkWidget *window3 = NULL;
	GtkWidget *window2 = NULL;
	GtkWidget *setup1_dialog = NULL;
	const gchar *account_name = NULL;

	/* Input param checking. */
	CHECK_PARAM((button == NULL), "Invalid input Parameter", return);
	CHECK_PARAM((message == NULL), "Invalid input Parameter", return);

	parent_dialog = gtk_widget_get_ancestor(GTK_WIDGET(button), GTK_TYPE_DIALOG);


	if (g_slist_nth(ui_accMgmt_listptr, 1) == NULL) {
		window2 = (GtkWidget *) ui_accmgmt_createemailaccountsetup1(FALSE);
		if (window2 != NULL) {
			gtk_widget_show_all(window2);
		}
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window2);
	}
	setup1_dialog = (GtkWidget *) (g_slist_nth(ui_accMgmt_listptr, 1)->data);
	account_entry = EMAIL_UI_LOOKUP_OBJECT(setup1_dialog, "accountNameEntry");
	account_name = gtk_entry_get_text(GTK_ENTRY(account_entry));

	/* Checking whether account name is blank. */
	if ((account_name == NULL)
	    || ((account_name != NULL) && (g_utf8_strlen(account_name, -1) == 0))) {
		gtk_infoprint((GtkWindow *) parent_dialog, _("mcen_ib_enter_fill_in_all_fields"));
		gtk_widget_grab_focus(account_entry);
		return;
	}
	/* If Setup2 Dialog has  created. */
	if (g_slist_nth(ui_accMgmt_listptr, 2) != NULL) {
		if (!ui_accmgmt_emptycheck_setup2(message)) {
			return;
		}
	} else {		/* If Setup3 Dialog has already created. */

		window2 =
		    (GtkWidget *) ui_accmgmt_createemailaccountsetup2(gActMgtMngr->edit_operation);
		if (window2 == NULL) {
			ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__,
				   __LINE__);
			return;
		}
		gtk_widget_hide(parent_dialog);
		gtk_infoprint((GtkWindow *) window2, _("mcen_ib_enter_fill_in_all_fields"));
		gtk_widget_show_all(window2);
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window2);
		return;
	}
	/* If Setup3 Dialog has  created. */
	if (g_slist_nth(ui_accMgmt_listptr, 3) != NULL) {
		if (is_setup3_settings_incomplete(parent_dialog)) {
			return;
		}
	} else {		/* If setup3 dialog has not created already. */
		window3 =
		    (GtkWidget *) ui_accmgmt_createemailaccountsetup3(gActMgtMngr->edit_operation);
		if (window3 == NULL) {
			ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n", __FUNCTION__,
				   __LINE__);
			return;
		}
		gtk_widget_hide(parent_dialog);
		gtk_infoprint((GtkWindow *) window3, _("mcen_ib_enter_fill_in_all_fields"));
		gtk_widget_show_all(window3);
		ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window3);
	}

}


/**
 * This function is a callback function. It will be called when user click on a insensitive button .
 * It will  show a infoprint with the given message.
 * @param button It is pointer to the button on which user click.
 * @param is_finish_operaion It is boolean flag which is TRUE if user select  'Finish' button, 
 * else FALSE.
 * @return void.
 */
void ui_accmgmt_setup2_insensitive_press_cb(GtkWidget * button, gboolean is_finish_operaion)
{
	/* Variable Declarations. */

	GtkWidget *username_entry = NULL;
	GtkWidget *email_entry = NULL;
	GtkWidget *parent_dialog = NULL;
	GtkWidget *window3 = NULL;
	const gchar *user_name = NULL;
	const gchar *email = NULL;

	/* Input Param Checking. */
	CHECK_PARAM((button == NULL), "Invalid input Parameter", return);

	parent_dialog = gtk_widget_get_ancestor(GTK_WIDGET(button), GTK_TYPE_DIALOG);

	username_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "userNameEntry");
	email_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "emailEntry");
	user_name = gtk_entry_get_text(GTK_ENTRY(username_entry));

	/* Checking for NULL or Empty User Name. */
	if ((user_name == NULL)
	    || ((user_name != NULL) && (g_utf8_strlen(user_name, -1) == 0))) {
		if (is_finish_operaion == TRUE) {
			gtk_infoprint((GtkWindow *) parent_dialog,
				      _("mcen_ib_enter_fill_in_all_fields"));
		} else {
			gtk_infoprint((GtkWindow *) parent_dialog, _("mcen_ib_enter_username"));
		}
		gtk_widget_grab_focus(username_entry);
		return;
	}

	/* Checking NULL or Empty Email Address. */
	email = gtk_entry_get_text(GTK_ENTRY(email_entry));
	if ((email == NULL) || ((email != NULL) && (g_utf8_strlen(email, -1) == 0))) {
		if (is_finish_operaion == TRUE) {
			gtk_infoprint((GtkWindow *) parent_dialog,
				      _("mcen_ib_enter_fill_in_all_fields"));
		} else {
			gtk_infoprint((GtkWindow *) parent_dialog, _("mcen_ib_required_email"));
		}
		gtk_widget_grab_focus(email_entry);
		return;
	}
	if (is_finish_operaion == TRUE) {
		if (g_slist_nth(ui_accMgmt_listptr, 3) != NULL) {
			if (is_setup3_settings_incomplete(parent_dialog)) {
				return;
			}
		} else {
			/*Creating  Setup3 Dialog */
			window3 =
			    (GtkWidget *) ui_accmgmt_createemailaccountsetup3(gActMgtMngr->
									      edit_operation);
			if (window3 == NULL) {
				ULOG_DEBUG("NULL pointer assigned in:[%s] at [%d]\n",
					   __FUNCTION__, __LINE__);
				return;
			}
			gtk_widget_hide(parent_dialog);
			gtk_widget_show_all(window3);
			gtk_infoprint((GtkWindow *) window3, _("mcen_ib_enter_fill_in_all_fields"));
			ui_accMgmt_listptr = g_slist_append(ui_accMgmt_listptr, window3);
		}
	}
}


/**
 * This function is a callback function. It will be called when user click on a insensitive button .
 * It will  show a infoprint with the given message.
 * @param button It is pointer to the button on which user click.
 * @param is_finish_operaion It is boolean flag which is TRUE if user select  'Finish' button, else FALSE.
 * @return void.
*/
void ui_accmgmt_setup3_insensitive_press_cb(GtkWidget * button, gboolean is_finish_operaion)
{
	/* Variable Declarations. */
	GtkWidget *ipserver_entry = NULL;
	GtkWidget *opserver_entry = NULL;
	GtkWidget *parent_dialog = NULL;
	const gchar *inputserver = NULL;
	const gchar *outputserver = NULL;

	/* Check input param. */
	CHECK_PARAM((button == NULL), "Invalid input Parameter", return);

	parent_dialog = gtk_widget_get_ancestor(GTK_WIDGET(button), GTK_TYPE_DIALOG);

	ipserver_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "inputServerAddress");
	opserver_entry = EMAIL_UI_LOOKUP_OBJECT(parent_dialog, "outputServerAddress");
	inputserver = gtk_entry_get_text(GTK_ENTRY(ipserver_entry));

	if ((inputserver == NULL)
	    || ((inputserver != NULL) && (g_utf8_strlen(inputserver, -1) == 0))) {
		/* If 'Finish' button is clicked */
		if (is_finish_operaion == TRUE) {
			gtk_infoprint((GtkWindow *) parent_dialog, _("mcen_ib_enter_ipname"));
		} else {
			gtk_infoprint((GtkWindow *) parent_dialog, _("mcen_ib_enter_ipname"));
		}
		gtk_widget_grab_focus(ipserver_entry);
		return;
	}
	outputserver = gtk_entry_get_text(GTK_ENTRY(opserver_entry));
	if ((outputserver == NULL)
	    || ((outputserver != NULL) && (g_utf8_strlen(outputserver, -1) == 0))) {
		if (is_finish_operaion == TRUE) {
			gtk_infoprint((GtkWindow *) parent_dialog, _("mcen_ib_enter_ipname"));
		} else {
			gtk_infoprint((GtkWindow *) parent_dialog, _("mcen_ib_enter_ipname"));
		}
		gtk_widget_grab_focus(opserver_entry);
		return;
	}
}

/**
 * This is a callback function. This function has saved the changes in Advanced Dialog and
 * close it. And go back to setup4 dialog .
 *
 * @param widget pointer to GtkWidget  on which it is called.
 * @param data gpointer variable contains GtkWidget pointers.
 * @return void
 */
void ui_accmgmt_save_advance_setting_cb(GtkWidget * widget, gpointer data)
{
	/* Variable declaration. */
	AdvOptionViewInfo *advstruct = NULL;
	GtkWidget *advanced_dialog = NULL;

	/* Check Input Param. */
	CHECK_PARAM((widget == NULL), "Invalid input Parameter", return);
	CHECK_PARAM((data == NULL), "Invalid input Parameter", return);

	advstruct = (AdvOptionViewInfo *) data;

	/* Saving  advanced account settings  data */
	ui_accmgmt_save_advancedsetting(advstruct);
	ui_accmgmt_save_advancedsettings_checkbuttons(advstruct);

	advanced_dialog = (GtkWidget *) advstruct->dialog_advancedsetup;
	ui_accMgmt_listptr = g_slist_remove(ui_accMgmt_listptr, advstruct);
	gtk_widget_destroy(advanced_dialog);
	gtk_widget_show_all(g_slist_nth((GSList *) ui_accMgmt_listptr, 4)->data);
}

/**
 * This function is a callback function which will be called by Account Management
 * module. This function will discard all the changes done in Advanced Dialog and close this dialog
 * and go back to the setup 4 dialog. 
 *
 * @param widget pointer to GtkWidget  on which it is called.
 * @param data gpointer variable contains GtkWidget pointers.
 * @return void
 * */

void ui_accMgmt_cancel_advance_setting_cb(GtkWidget * widget, gpointer data)
{
	/* Variable declarations */
	AdvOptionViewInfo *advstruct = NULL;
	GtkWidget *advanced_dialog = NULL;

	/* Check Input Param. */
	CHECK_PARAM((widget == NULL), "Invalid input Parameter", return);
	CHECK_PARAM((data == NULL), "Invalid input Parameter", return);

	advstruct = (AdvOptionViewInfo *) data;
	advanced_dialog = (GtkWidget *) advstruct->dialog_advancedsetup;
	ui_accMgmt_listptr = g_slist_remove(ui_accMgmt_listptr, advstruct);

	/* Destroy the Advanced Dialog. */
	gtk_widget_destroy(advanced_dialog);

	/* Show Setup-4 dialog. */
	gtk_widget_show_all(g_slist_nth((GSList *) ui_accMgmt_listptr, 4)->data);

}

/**
This is callback function which will be called for help in advanced options.
Depending on the tab is shown in advanced window help is invoked.
@param win Widget pointer on which it is called.
@param data user data contains widget pointer.
@return nothing.
*/
void on_ui_accounts_tab_help_activate(GtkWidget * win, gpointer data)
{
	GtkWidget *notebook = NULL;
	gint cur_book_num = -1;

	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	if (win == NULL || data == NULL) {
		osso_log(LOG_DEBUG, "[%s][%s] : invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	notebook = (GtkWidget *) data;
	cur_book_num = gtk_notebook_get_current_page((GtkNotebook *) notebook);

	switch (cur_book_num) {
	case 0:
		ui_on_dialog_help_activate(win, HELP_ID_RETRIEVALTAB);
		break;
	case 1:
		ui_on_dialog_help_activate(win, HELP_ID_SENDINGTAB);
		break;
	case 2:
		ui_on_dialog_help_activate(win, HELP_ID_SMTPSENDINGTAB);
		break;
	case 3:
		ui_on_dialog_help_activate(win, HELP_ID_OTHERTAB);
		break;
	case 4:
		ui_on_dialog_help_activate(win, HELP_ID_SECURITYTAB);
		break;
	default:
		break;
	}
	osso_log(LOG_DEBUG, "[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

#if CL_USE
/**
 This is an utility function. This function is used to find a specific access point name 
 in the list of engine supplied IAP name.

 @param iap_name , it holds the name of the iap
 @return gboolean: TRUE if iap_name is found in a node of list of iap_infos.
*/
static gboolean search_iap_name_iaplist(const gchar * iap_name)
{
	GSList *iapinfo_list = NULL;
	GSList *temp_iapinfo_list = NULL;
	IapInfo *iap_info = NULL;
	gboolean success = FALSE;

	CHECK_PARAM((iap_name == NULL), "Invalid Input Parameter", return FALSE);

	/* Getting the iap list from the engine */
	iapinfo_list = ui_dbus_get_iapinfo_list();
	temp_iapinfo_list = iapinfo_list;

	while (iapinfo_list != NULL && iapinfo_list->data != NULL) {
		iap_info = (IapInfo *) iapinfo_list->data;
		if (ui_compare_utf8_strings(iap_info->iap_name, iap_name) == 0) {
			success = TRUE;
			break;
		}
		iapinfo_list = g_slist_next(iapinfo_list);
	}
	/*Freeing the iapinfolist */
	g_slist_foreach(temp_iapinfo_list, (GFunc) ui_free_iapinfo_structure, NULL);
	g_slist_free(temp_iapinfo_list);
	return success;
}
#endif

/** This is an utility function. This function will resume
 * Editing of account operation.
 */
void ui_resume_edit_accout_oper(void)
{
	if (email_main_view->account_manager->edit_account_dialog != NULL) {
		ui_accmgmt_display_dialog_for_feilds
		    (email_main_view->account_manager->current_sendrec_account, 0);
		gtk_widget_destroy(email_main_view->account_manager->edit_account_dialog);
	}

	email_main_view->account_manager->operation_status = NONE_OP;
	g_free(email_main_view->account_manager->current_sendrec_account);
	email_main_view->account_manager->current_sendrec_account = NULL;
	return;
}

/**This is an utility function. This function is used to resume delete account opearation.
 */
void ui_resume_delete_account_oper(void)
{
	g_free(email_main_view->account_manager->current_sendrec_account);
	email_main_view->account_manager->current_sendrec_account = NULL;
	gtk_widget_show(email_main_view->account_manager->edit_account_dialog);
	accountsetupon_delete_cb(email_main_view->account_manager->edit_account_dialog,
				 email_main_view->account_manager->edit_account_dialog);
	return;
}

void ui_acctmgmt_change_smtp_security(GtkWidget * smtp_security_combo, gpointer user_data)
{
	/* Variable declarations */
	SMTPTab *smtpTab = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parentDialog = NULL;
	gint active_security_type = 0;

	/* Input param checking */
	CHECK_PARAM((smtp_security_combo == NULL), "Invalid input param", return);
	CHECK_PARAM((user_data == NULL), "Invalid input param", return);

	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailAccountInfo");
	smtpTab = (SMTPTab *) user_data;
	active_security_type = gtk_combo_box_get_active(GTK_COMBO_BOX(smtpTab->combo_smtp_ssl));
	if (active_security_type == 2) {
		hildon_number_editor_set_value((HildonNumberEditor *)
					       smtpTab->spinbutton_outport, 465);
	} else {
		hildon_number_editor_set_value((HildonNumberEditor *)
					       smtpTab->spinbutton_outport, 25);
	}
}

/** 
 * This is a callback function. It is used to update incoming port number  while changes 
 * in security setting.
 */

void ui_acctmgmt_change_security(GtkWidget * security_combo, gpointer user_data)
{

	/* Variable declarations */
	RetrivevalTab *retTab = NULL;
	PrefsAccount *ui_accMgmt_accountinfo = NULL;
	GtkWidget *parentDialog = NULL;
	gint active_security_type = 0;
	gint incoming_port = 0;

	ULOG_DEBUG("[%s][%s] :Starts Here \n ", __FILE__, __FUNCTION__);

	/* Input param checking */
	CHECK_PARAM((security_combo == NULL), "Invalid input param", return);
	CHECK_PARAM((user_data == NULL), "Invalid input param", return);

	parentDialog = (GtkWidget *) (g_slist_nth((GSList *) ui_accMgmt_listptr, 1)->data);
	ui_accMgmt_accountinfo = g_object_get_data(G_OBJECT(parentDialog), "EmailAccountInfo");
	retTab = (RetrivevalTab *) user_data;
	active_security_type = gtk_combo_box_get_active(GTK_COMBO_BOX(retTab->combo_ssl));

	/* Saving the incoming port number to replace secure port number later */
	if (active_security_type == 2) {
		incoming_port = hildon_number_editor_get_value
		    ((HildonNumberEditor *) (retTab->spinbutton_inport));
		gActMgtMngr->incoming_port = incoming_port;

	}

	/* If user select secure port */
	if (active_security_type == 2) {
		if (ui_accMgmt_accountinfo->receive_settings.protocol == PROTO_POP3) {
			hildon_number_editor_set_value((HildonNumberEditor *)
						       retTab->spinbutton_inport, IN_PORT);
		} else {
			hildon_number_editor_set_value((HildonNumberEditor *)
						       retTab->spinbutton_inport, OUT_PORT);
		}
	}
	/* If user select TLS security  or None */
	else {
		hildon_number_editor_set_value((HildonNumberEditor *)
					       retTab->spinbutton_inport,
					       gActMgtMngr->incoming_port);
		gtk_widget_set_sensitive(retTab->spinbutton_inport, TRUE);
	}

	ULOG_DEBUG("[%s][%s] :Ends Here \n ", __FILE__, __FUNCTION__);
	return;
}


/* End of file */
