/**
 * @file editor.c
 *
 * This file contains all function to the create email editor. 
 * 
 *  Copyright (c) 2005 Nokia. All rights reserved
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 *  USA
 */

#include "utils.h"
/*Editor-Viewer header files*/
#include "editcallback.h"
#include "callback.h"

/*account maganagement header files.*/
#include "ui_account_store.h"

/*html header files*/
#include "htcallback.h"
#include "html_parser.h"
#include "htmltotextconv.h"

#include "enablesignatures_callbacks.h"
#include "undo.h"
#include "security.h"
#define SETTINGS_KEY_FONT_SCALING_FACTOR "/apps/osso/fontconfig/font_scaling_factor"
#define TITLE_LEN 20

extern MainViewInfo *email_main_view;	/*declared in ui_interface.c */
extern GSList *openWindows;	/*declared in ui_interface.c
				   for maintaining the list of open windows. */
gboolean editor_signals_created = FALSE;
extern gboolean common_signals_created;


/**
 this function is used to increase the address field size based on the text entered.
 it increases upto 5 lines then a sollbar appears.

 @param textbuffer is the GtkTextBuffer of the address field.
 @param iter is the GtkTextIter wher the text is inserted.
 @param text is the gchar pointer to the text inserted.
 @param len is the gint for the length os the text inserted.
 @param fieldinfo is the address field info.
 
 @return void.
*/
void
resize_addr_field(GtkTextBuffer * textbuffer, GtkTextIter * iter, gchar * text,
		  gint len, AddressFieldInfo * fieldinfo)
{
	GdkRectangle endrectangle = { 0, 0 };
	gint max_height = DEFAULT_ADDR_FIELD_CUR_SZ;
	GtkTextIter end;
	gint resizefactor = 0;
	gint factor = 1;

	ULOG_DEBUG_L("starts here");

	ui_flush_gtk_events();

	gtk_text_buffer_get_end_iter(textbuffer, &end);
	gtk_text_view_get_iter_location((GtkTextView *) fieldinfo->field, &end, &endrectangle);
	osso_log(LOG_INFO, "the end_height = %d end_y = %d \n", endrectangle.height,
		 endrectangle.y);
	if (fieldinfo->from_viewer == FALSE) {
		max_height = MAX_ADDR_FIELD_SZ;
		factor = endrectangle.y / endrectangle.height + 1;
		/*if resize factor is 1, thus avoiding resize for every character entry*/
		if(factor == 1 || endrectangle.height < CURSOR_HEIGHT)
			return;
	}

	else {
		if (endrectangle.y == 0) {
			gtk_widget_set_size_request(fieldinfo->field_hbox, BODY_WIDTH, max_height);
			return;
		}
		resizefactor =
		    DEFAULT_ADDR_FIELD_CUR_SZ * (endrectangle.y / endrectangle.height + 1);
		gtk_widget_set_size_request(fieldinfo->field_hbox, BODY_WIDTH, resizefactor);
		return;
	}

	if (endrectangle.y < max_height) {
		resizefactor = DEFAULT_ADDR_FIELD_CUR_SZ * factor;
		if (resizefactor > max_height) {
			gtk_widget_set_size_request(fieldinfo->field_hbox, BODY_WIDTH, max_height);
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
						       (fieldinfo->field_sw),
						       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
		} else
			gtk_widget_set_size_request(fieldinfo->field_hbox, BODY_WIDTH,
						    resizefactor);
	} else {
		gtk_widget_set_size_request(fieldinfo->field_hbox, BODY_WIDTH, max_height);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(fieldinfo->field_sw),
					       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	}
	ULOG_DEBUG_L("ends here");
}

/**
 this function is used to decrease the address field size based on the text deleted.

 @param textbuffer is the GtkTextBuffer of the address field.
 @param iter1 is the start GtkTextIter wher the text is deleted.
 @param iter2 is the end GtkTextIter wher the text is deleted.
 @param fieldinfo is the address field info.
 @return void.
*/
static void
decrease_address_field(GtkTextBuffer * textbuffer, GtkTextIter * iter1,
		       GtkTextIter * iter2, AddressFieldInfo * fieldinfo)
{
	GdkRectangle endrectangle = { 0, 0 };
	GtkTextIter end, start;
	gint resizefactor = 0;
	gboolean notext = FALSE;

	ULOG_DEBUG_L("starts here");
	gtk_text_buffer_get_bounds(textbuffer, &start, &end);
	if (gtk_text_iter_compare(&start, iter1) == 0 && gtk_text_iter_compare(&end, iter2) == 0) {
		notext = TRUE;
	}
	gtk_text_view_get_iter_location((GtkTextView *) fieldinfo->field, &end, &endrectangle);
	endrectangle.y += 22;
	if (endrectangle.y >= DEFAULT_ADDR_FIELD_CUR_SZ && endrectangle.y < MAX_ADDR_FIELD_SZ) {
		resizefactor = DEFAULT_ADDR_FIELD_CUR_SZ * (endrectangle.y / 22);

		if (resizefactor >= MAX_ADDR_FIELD_SZ)
			resizefactor = MAX_ADDR_FIELD_SZ;

		gtk_widget_set_size_request(fieldinfo->field_hbox, DEFAULT_WIDTH, resizefactor);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(fieldinfo->field_sw),
					       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
		if (endrectangle.y <= MAX_ADDR_FIELD_SZ) {
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
						       (fieldinfo->field_sw),
						       GTK_POLICY_NEVER, GTK_POLICY_NEVER);
		}
	} else if (endrectangle.y <= DEFAULT_ADDR_FIELD_CUR_SZ || notext == TRUE) {
		gtk_widget_set_size_request(fieldinfo->field_hbox, DEFAULT_WIDTH,
					    DEFAULT_ADDR_FIELD_SZ);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(fieldinfo->field_sw),
					       GTK_POLICY_NEVER, GTK_POLICY_NEVER);
	}
	ULOG_DEBUG_L("ends here");
	return;
}


/**
 Creation of temporary directory for attachments

 @return void.
*/
void ui_make_temp_dir()
{
	gchar *attachment_dir = NULL;
	GnomeVFSResult result = 0;
	GnomeVFSURI *directory = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	attachment_dir = ui_get_tmp_attach_dir();
	directory = gnome_vfs_uri_new(attachment_dir);
	g_free(attachment_dir);
	result = gnome_vfs_make_directory_for_uri(directory, S_IRWXU | S_IRWXO | S_IRWXG);
	if (result == GNOME_VFS_OK || result == GNOME_VFS_ERROR_FILE_EXISTS) {
		ULOG_DEBUG_F("Temporary directory created\n");
	}
	gnome_vfs_uri_unref(directory);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This fuction is used to navigate to different fields(to/cc/bcc/subject/body)
 in the editor.
 using the Up/Down arrow keys.
 
 @param curr_field is the current field to the editor.
 @param event is the key press event.
 @param prev_field is the prev field of the editor
 
 @return TRUE-to stop the event from propagating.
         else FALSE.
*/
static gboolean
ui_navigate_to_prev(GtkWidget * curr_field, GdkEventKey * event, GtkWidget * prev_field)
{

	GtkTextBuffer *curr_buffer = NULL;
	gint curr_line = -1;
	GtkTextIter curr_pos;
	GtkTextMark *insert_mark = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (event->keyval == GDK_Up && GTK_IS_ENTRY(curr_field) == TRUE) {
		ULOG_DEBUG("navigating to the next feild\n");
		gtk_widget_grab_focus(prev_field);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return TRUE;
	}

	if (GTK_IS_TEXT_VIEW(curr_field) == TRUE) {
		curr_buffer = gtk_text_view_get_buffer((GtkTextView *) curr_field);
		insert_mark = gtk_text_buffer_get_insert(curr_buffer);
		gtk_text_buffer_get_iter_at_mark(curr_buffer, &curr_pos, insert_mark);
		curr_line = gtk_text_iter_get_line(&curr_pos);
		ULOG_DEBUG("current line is %d\n", curr_line);
		if (event->keyval == GDK_Up) {
			if (curr_line == 0 && GTK_WIDGET_VISIBLE(prev_field) == TRUE) {
				ULOG_DEBUG("navigating to the next feild\n");
				gtk_widget_grab_focus(prev_field);
				return TRUE;
			}
		}

	}
	ULOG_DEBUG("[%s][%s] : ends	 here :\n", __FILE__, __FUNCTION__);
	return FALSE;
}

/**
 This fuction is used to navigate to different fields(to/cc/bcc/subject/body)
 in the editor.
 using the Up/Down arrow keys.
 
 @param curr_field is the current field to the editor.
 @param event is the key press event.
 @param next_field is the prev field of the editor
 
 @return TRUE-to stop the event from propagating.
       else FALSE.
*/
static gboolean
ui_navigate_to_next(GtkWidget * curr_field, GdkEventKey * event, GtkWidget * next_field)
{
	GtkTextBuffer *curr_buffer = NULL;
	gint line_count = 0, curr_line = 0;
	GtkTextIter curr_pos;
	GtkTextMark *insert_mark = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (event->keyval == GDK_Down && GTK_IS_ENTRY(curr_field) == TRUE) {
		ULOG_DEBUG("navigating to the next feild\n");
		curr_buffer = gtk_text_view_get_buffer((GtkTextView *) next_field);
		gtk_text_buffer_get_start_iter(curr_buffer, &curr_pos);
		gtk_widget_grab_focus(next_field);
		gtk_text_buffer_place_cursor(curr_buffer, &curr_pos);
		ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
		return TRUE;
	}
	if (GTK_IS_TEXT_VIEW(curr_field) == TRUE) {
		curr_buffer = gtk_text_view_get_buffer((GtkTextView *) curr_field);
		insert_mark = gtk_text_buffer_get_insert(curr_buffer);
		line_count = gtk_text_buffer_get_line_count(curr_buffer);
		gtk_text_buffer_get_iter_at_mark(curr_buffer, &curr_pos, insert_mark);
		curr_line = gtk_text_iter_get_line(&curr_pos);
		curr_line = curr_line + 2;
		if (event->keyval == GDK_Down && curr_line > line_count) {
			ULOG_DEBUG("navigating to the next feild\n");
			gtk_widget_grab_focus(next_field);
			return TRUE;
		}
	}
	ULOG_DEBUG("[%s][%s] : ends	 here :\n", __FILE__, __FUNCTION__);

	return FALSE;
}

/**
 This fuction is used to navigate to cc field either from(to field)
 in the editor.
 using the Down arrow keys.
 
 @param curr_field is the current field to the editor.
 @param event is the key press event.
 @param next_field is the prev field of the editor
 
 @return TRUE-to stop the event from propagating.
 else FALSE.
 */
static gboolean navigate_to_cc(GtkWidget * curr_field, GdkEventKey * event, GtkWidget * next_field)
{
	gboolean action = FALSE;

	CHECK_PARAM(curr_field == NULL, "Input param failed\n", return FALSE);
	CHECK_PARAM(next_field == NULL, "Input param failed\n", return FALSE);

	if (ui_get_cc_field_status() == FALSE) {
		return FALSE;
	}
	if (event->keyval == GDK_Down && GTK_IS_ENTRY(curr_field) == FALSE) {
		action = ui_navigate_to_next(curr_field, event, next_field);
	}
	return action;
}

/**
 This fuction is used to navigate to bcc field either from(to field)
 in the editor.
 using the Down arrow keys.
 
 @param curr_field is the current field to the editor.
 @param event is the key press event.
 @param next_field is the prev field of the editor
 
 @return TRUE-to stop the event from propagating.
  else FALSE.
*/
static gboolean navigate_to_bcc(GtkWidget * curr_field, GdkEventKey * event, GtkWidget * next_field)
{
	gboolean action = FALSE;

	CHECK_PARAM(curr_field == NULL, "Input param failed\n", return FALSE);
	CHECK_PARAM(next_field == NULL, "Input param failed\n", return FALSE);

	if (ui_get_bcc_field_status() == FALSE) {
		return FALSE;
	}
	if (event->keyval == GDK_Down && GTK_IS_ENTRY(curr_field) == FALSE) {
		action = ui_navigate_to_next(curr_field, event, next_field);
	}
	return action;

}

/**
 This fuction is used to navigate to cc field either from(to/bcc/subject)
 in the editor.
 using the Up/Down arrow keys.
 
 @param curr_field is the current field to the editor.
 @param event is the key press event.
 @param next_field is the prev field of the editor
 
 @return TRUE-to stop the event from propagating.
   else FALSE.
*/
static gboolean
ui_navigate_to_cc(GtkWidget * curr_field, GdkEventKey * event, GtkWidget * next_field)
{
	gboolean action = FALSE;

	CHECK_PARAM(curr_field == NULL, "Input param failed\n", return FALSE);
	CHECK_PARAM(next_field == NULL, "Input param failed\n", return FALSE);

	if (ui_get_cc_field_status() == FALSE) {
		return FALSE;
	}
	if (event->keyval == GDK_Down && GTK_IS_ENTRY(curr_field) == FALSE) {
		action = ui_navigate_to_next(curr_field, event, next_field);
	} else if (event->keyval == GDK_Up) {
		action = ui_navigate_to_prev(curr_field, event, next_field);
	}
	return action;

}

/**
 This fuction is used to navigate to bcc field either from(to/bcc/subject)
 in the editor.
 using the Up/Down arrow keys.
 
 @param curr_field is the current field to the editor.
 @param event is the key press event.
 @param next_field is the prev field of the editor
 
 @return TRUE-to stop the event from propagating.
   else FALSE.
*/
static gboolean
ui_navigate_to_bcc(GtkWidget * curr_field, GdkEventKey * event, GtkWidget * next_field)
{
	gboolean action = FALSE;

	CHECK_PARAM(curr_field == NULL, "Input param failed\n", return FALSE);
	CHECK_PARAM(next_field == NULL, "Input param failed\n", return FALSE);

	if (ui_get_bcc_field_status() == FALSE) {
		return FALSE;
	}
	if (event->keyval == GDK_Down && GTK_IS_ENTRY(curr_field) == FALSE) {
		action = ui_navigate_to_next(curr_field, event, next_field);
	} else if (event->keyval == GDK_Up) {
		action = ui_navigate_to_prev(curr_field, event, next_field);
	}
	return action;

}

/**
 This fuction is used to handle backspace for a resolved recipient. 
 in the editor.
 
 @param addressview is the address field to the editor.
 @param event is the key press event.
 @param data is user data.
 
 @return TRUE-to stop the event from propagating.
   else FALSE.
*/
static gboolean backspace_handling(GtkTextView * addressview, GdkEventKey * event, gpointer data)
{
	GtkTextBuffer *addrbuffer = NULL;
	GtkTextIter curr_pos, start, end;
	GtkTextMark *insert_mark = NULL;
	GtkTextTag *contact_tag = NULL;
	gint type = 0;
	GSList *tags = NULL, *tagp = NULL;
	gboolean bkspc = FALSE, delete = FALSE;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	if (event->keyval == GDK_BackSpace) {
		bkspc = TRUE;
	} else if (event->keyval == GDK_Delete) {
		delete = TRUE;
	} else {
		return FALSE;
	}

	addrbuffer = gtk_text_view_get_buffer((GtkTextView *) addressview);
	insert_mark = gtk_text_buffer_get_insert(addrbuffer);
	gtk_text_buffer_get_iter_at_mark(addrbuffer, &curr_pos, insert_mark);
	if (bkspc == TRUE) {
		gtk_text_iter_backward_char(&curr_pos);
	} else if (delete == TRUE) {
		gtk_text_iter_forward_char(&curr_pos);
	}
	tags = gtk_text_iter_get_tags(&curr_pos);
	for (tagp = tags; tagp != NULL; tagp = tagp->next) {
		contact_tag = tagp->data;
		type = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(contact_tag), "emailid"));
		if (type != EMAIL_ID) {
			continue;
		}
		if (gtk_text_iter_has_tag(&curr_pos, contact_tag) == TRUE) {
			start = curr_pos;
			end = curr_pos;
			gtk_text_iter_forward_to_tag_toggle(&end, NULL);
			gtk_text_iter_backward_to_tag_toggle(&start, NULL);
			gtk_text_buffer_delete(addrbuffer, &start, &end);
		}
	}
	return FALSE;
}

/**
 This is a helper function to update the tasknavigator when editor is  invoked.
 
 @param open_window is the structure poiter to the open windows.
	 This paramter is not freed in this function.
 @param headerInfo is the HeaderInfo structure pointer that contains msgid & subject.
	 This paramter is not freed in this function.
 @param editorInfo is the editorInfo structure pointer for the current editor.
	 This paramter is not freed in this function.
	 
 @return void.
*/
static void
ui_editor_update_tasknavigator(OpenedWindowData * open_window,
			       const HeaderInfo * headerInfo, EditorInfo * editorInfo)
{
	gchar *title = _("mail_va_no_subject");

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(open_window == NULL, "Input parameter check failed\n", return);
	CHECK_PARAM(editorInfo == NULL, "Input parameter check failed\n", return);
	/*Update Task Navigator */
	if (open_window->msgid == NULL) {
		editorInfo->msgid = g_strdup_printf("%d", email_main_view->tags->window_count);
		open_window->msgid = g_strdup(editorInfo->msgid);
	} else {
		editorInfo->msgid = g_strdup(open_window->msgid);
	}

	if (headerInfo != NULL && headerInfo->subject != NULL) {
		title = headerInfo->subject;
	}
	g_object_set_data_full(G_OBJECT(editorInfo->editor_appview), VIEW_MSG_ID,
			       g_strdup(open_window->msgid), (GDestroyNotify) g_free);
	ULOG_DEBUG("[%s][%s] : ends	 here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is a callback function used to upadate the modified flag
 when ever some formating to the amil is done.
 
 @param buffer the textbuffer on which the tag is being applied.
 @param tag is the texttag being applied.
 @param start_it & end_it specify the start & end iter position of the text modified.
 @param modified is the gboolean that get updated.
 
 @return void.
*/
void
updatemodified(GtkTextBuffer * buffer, GtkTextTag * tag,
	       GtkTextIter * start_it, GtkTextIter * end_it, gboolean * modified)
{
	GtkTextIter start, end;
	gchar *text = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);
	CHECK_PARAM(modified == NULL, "Input param failed\n", return)
	    gtk_text_buffer_get_bounds(buffer, &start, &end);
	text = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
	g_strstrip(text);
	if (text != NULL && strlen(text) > 0) {
		*modified = TRUE;
	}
	g_free(text);
	ULOG_DEBUG("[%s][%s] : ends here :\n", __FILE__, __FUNCTION__);
	return;
}

/**
 This is a callback function used to set the title of editor
 corresponding to the subject.
 
 @param subject_entry is a gtkwidget for subject.
 @param editorview is the pointer to the editor appview.
 
 @return void.
*/
static void ui_editor_change_title(GtkWidget * subject_entry, GtkWidget * editorview)
{
	const gchar *title = NULL;
	static gchar title_buf[4 * TITLE_LEN + 1];	/*utf8 chars are upto 4 bytes */

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	memset(title_buf, 0, 4 * TITLE_LEN + 1);
	title = gtk_entry_get_text((GtkEntry *) subject_entry);
	if (title != NULL) {
		g_utf8_strncpy(title_buf, title, TITLE_LEN);
		hildon_appview_set_title((HildonAppView *) editorview, title_buf);
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This is a helper function to set the focus when the editor is opened.
 
 @param editor_info is the EditorInfo structure pointer.
 This paramter is not freed in this function.
 
 @return void.
*/
static void ui_editor_set_focus(const EditorInfo * editor_info)
{
	MsgFields *msgFields = NULL;
	GtkTextBuffer *bodybuffer = NULL;
	GtkTextIter iter;
	gint mode = NEW_MAIL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);
	CHECK_PARAM(editor_info->msgFields == NULL, "Input param failed\n", return);

	mode = (editor_info->composeType);
	msgFields = editor_info->msgFields;

	if (mode == REPLY || mode == REPLY_ALL
	    || mode == REPLY_FROM_FOLDER || mode == REPLY_ALL_FROM_FOLDER) {
		bodybuffer = gtk_text_view_get_buffer((GtkTextView *) msgFields->body);
		gtk_text_buffer_get_start_iter(bodybuffer, &iter);
		gtk_widget_grab_focus(msgFields->body);
		gtk_text_buffer_place_cursor(bodybuffer, &iter);
	} else {
		gtk_widget_grab_focus(msgFields->to);
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This function checks for any duplicate address in the cc
 
 @param cc is the cc addresses  string.
 
 @return gchar* which does not have any duplicates.
*/
static gchar *check_duplicates(const gchar * cc)
{
	gchar **splitaddress = NULL;
	gchar *addresses = NULL;
	gchar *firststr = NULL, *secondstr = NULL;
	GString *uniqueAddress = NULL;
	gint i = 0, j = 0;
	gboolean duppresent = FALSE;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (cc == NULL) {
		return NULL;
	}
	uniqueAddress = g_string_new(NULL);
	splitaddress = g_strsplit(cc, ADDR_SEPERATOR_CHAR, -1);
	for (i = 0; splitaddress[i] != NULL; i++) {
		duppresent = FALSE;
		for (j = i + 1; splitaddress[j] != NULL; j++) {
			g_strstrip(splitaddress[i]);
			g_strstrip(splitaddress[j]);
			firststr = ui_parse_from_address(splitaddress[i]);
			secondstr = ui_parse_from_address(splitaddress[j]);
			if (g_ascii_strcasecmp(firststr, secondstr) == 0) {
				duppresent = TRUE;
			}
			g_free(firststr);
			g_free(secondstr);
			secondstr = NULL;
			firststr = NULL;
		}
		if (duppresent == FALSE) {
			g_string_append(uniqueAddress, splitaddress[i]);
			if (splitaddress[i + 1] != NULL) {
				g_string_append(uniqueAddress, ADDR_SEPERATOR_CHAR);
			}
		}
	}
	g_strfreev(splitaddress);
	addresses = g_string_free(uniqueAddress, FALSE);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return (addresses);
}

/*
 This function is used to check if the current account address is present in the to address list
 and removes that address if present.
 
 @param uniqueTo is the gchar representing the to address string separated with ",".
 @param currId is the gchar for the current account email address.
 
 @return a newly allocated gchar without the current email address of the account.
*/
static gchar *check_current_accountaddr(const gchar * uniqueTo, const gchar * currId)
{
	gchar *toaddr = NULL;
	gchar **parseString = NULL;
	GString *to = NULL;
	gint i = 0;
	if (uniqueTo == NULL || currId == NULL) {
		return NULL;
	}

	to = g_string_new(NULL);
	parseString = g_strsplit(uniqueTo, ADDR_SEPERATOR_CHAR, -1);
	while (parseString[i] != NULL) {
		toaddr = ui_parse_from_address(parseString[i]);
		if (g_ascii_strcasecmp(toaddr, currId) != 0) {
			g_string_append(to, parseString[i]);
			if (parseString[i + 1] != NULL) {
				g_string_append(to, ADDR_SEPERATOR_CHAR);
			}
		}
		g_free(toaddr);
		toaddr = NULL;
		i++;
	}
	toaddr = g_string_free(to, FALSE);
	return toaddr;
}

/**
 This is a helper function that fills the to field.
 
 @param mode is a gint that specifies the msgtype.
 @param buffer_to is the GtkTextBuffer for to field.
 @param from is the gchar* from string.
 @param to is the gchar* from string.
 
 @return void.
 */

static void
ui_editor_fill_to_address(gint mode, GtkTextBuffer * buffer_to,
			  const gchar * from, const gchar * to,
			  const gchar * reply_to, const gchar * currentAcc)
{
	gboolean senderExits = FALSE;
	gchar *toAddress = NULL;
	gchar *uniqueTo = NULL;
	gchar *uniqueAddr = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (mode == REPLY || mode == REPLY_FROM_FOLDER) {
		if (reply_to != NULL)
			toAddress = g_strconcat(reply_to, NULL);
		else
			toAddress = g_strconcat(from, NULL);
	} else if (mode == REPLY_ALL || mode == REPLY_ALL_FROM_FOLDER) {
		uniqueTo = check_duplicates(to);
		if (currentAcc != NULL && g_ascii_strcasecmp(currentAcc, DEFAULT) != 0) {
			uniqueAddr = check_current_accountaddr(uniqueTo, currentAcc);
		} else {
			uniqueAddr = g_strdup(uniqueTo);
		}
		g_free(uniqueTo);
		uniqueTo = NULL;
		senderExits = ui_editor_do_senderCheck(from, uniqueAddr);
		if (senderExits == FALSE) {
			if (reply_to != NULL)
				toAddress =
				    g_strconcat(reply_to, ADDR_SEPERATOR_CHAR, uniqueAddr, NULL);
			else
				toAddress =
				    g_strconcat(from, ADDR_SEPERATOR_CHAR, uniqueAddr, NULL);
		} else {
			toAddress = g_strconcat(uniqueAddr, NULL);
		}
		g_free(uniqueAddr);
	} else if (mode == DRAFT_MAIL || mode == OUTBOX_MAIL || mode == SEND_AS) {
		toAddress = g_strconcat(to, NULL);
	}

	if (toAddress != NULL && g_str_has_suffix(toAddress, ADDR_SEPERATOR_CHAR)
	    == TRUE) {
		/*removes the last separator charector */
		toAddress = g_strndup(toAddress, strlen(toAddress) - 1);
	}
	if (toAddress != NULL) {
		gtk_text_buffer_set_text(buffer_to, toAddress, -1);
		g_free(toAddress);
	}
	gtk_text_buffer_set_modified(buffer_to, FALSE);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}


/**
 This is a helper function that fills the cc  field.
 
 @param mode is a gint that specifies the msgtype.
 @param buffer_cc is the GtkTextBuffer for cc field.
 @param from is the gchar* from string.
 @param cc is the gchar* cc string.
 @param editor_info is the structure pointer to the
 	EditorInfo containg the e cc field widgets.
 	which is show based on the showcc flag.This param is not freed here.
 
 @return void.
*/
static void
ui_editor_fill_cc_address(gint mode, GtkTextBuffer * buffer_cc,
			  const gchar * from, const gchar * cc,
			  EditorInfo * editor_info, const gchar * currAccountid)
{
	gchar *uniqueCc = NULL;
	gchar *ccAddress = NULL;
	gchar *parsestr = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	if (cc && strlen(cc) == 0)
		return;

	if (mode == DRAFT_MAIL || mode == OUTBOX_MAIL || mode == SEND_AS) {
		uniqueCc = g_strdup(cc);

	}

	else if (mode == DRAFT_MAIL || mode == OUTBOX_MAIL || mode == REPLY_ALL
		 || mode == REPLY_ALL_FROM_FOLDER || mode == SEND_AS) {
		if (cc != NULL) {
			ccAddress = g_strdup(cc);
			/*Check for duplicate mailids. */
			uniqueCc = check_duplicates(ccAddress);
			/*removes the senders address from the cc list */
			parsestr = check_current_accountaddr(uniqueCc, from);

			if (currAccountid != NULL
			    && g_ascii_strcasecmp(currAccountid, DEFAULT) != 0) {
				/*removes the current accountid address from the cc list */
				uniqueCc = check_current_accountaddr(parsestr, currAccountid);
			} else {
				uniqueCc = g_strdup(parsestr);
			}
			if (uniqueCc != NULL
			    && g_str_has_suffix(uniqueCc, ADDR_SEPERATOR_CHAR) == TRUE) {
				uniqueCc = g_strndup(uniqueCc, strlen(uniqueCc) - 1);
			}
		}
	}
	if (uniqueCc != NULL) {
		gtk_text_buffer_set_text(buffer_cc, uniqueCc, -1);
		show_cc_field(editor_info->msgFields, TRUE);
		g_signal_handlers_block_by_func(editor_info->menu->show_cc, ui_msg_showCc_cb,
						editor_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(editor_info->menu->show_cc),
					       TRUE);
		g_signal_handlers_unblock_by_func(editor_info->menu->show_cc, ui_msg_showCc_cb,
						  editor_info);
		g_free(uniqueCc);
		g_free(ccAddress);
		g_free(parsestr);
	}


	gtk_text_buffer_set_modified(buffer_cc, FALSE);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This is a helper function that fills the bcc  field.
 
 @param mailType is a gint that specifies the msgtype.
 @param buffer_bcc is the GtkTextBuffer for bcc field.
 @param bcc is the gchar* bcc string.
 @param fields is the structure pointer to the MsgFields containg the e cc field widgets.
 	which is show based on the showcc flag.This is not freed here.
 
 @return void.
*/

static void
ui_editor_fill_bcc_address(gint mailType, GtkTextBuffer * buffer_bcc,
			   const gchar * bcc, EditorInfo * editor_info)
{
	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	if (mailType == OUTBOX_MAIL || mailType == DRAFT_MAIL || mailType == SEND_AS) {
		if (bcc != NULL) {
			gtk_text_buffer_set_text(buffer_bcc, bcc, -1);
			g_signal_handlers_block_by_func(editor_info->menu->show_bcc,
							ui_msg_showBcc_cb, editor_info);
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
						       (editor_info->menu->show_bcc), TRUE);
			g_signal_handlers_unblock_by_func(editor_info->menu->show_bcc,
							  ui_msg_showBcc_cb, editor_info);
			show_bcc_field(editor_info->msgFields, TRUE);
		}
	}

	gtk_text_buffer_set_modified(buffer_bcc, FALSE);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This is a helper function to tag the emailids when replying without 
 doing any checknames.
 @param check_names_list is the GSList of addressBuffers, pointers to GtkTextBuffer.
*/
static void editor_tag_reply_ids(GSList * check_names_list)
{
	GtkTextBuffer *addressBuffer = NULL;
	GSList *check_names_list_local = NULL, *node = NULL;
	GtkTextIter ratestart, rateend, start, end;
	GtkTextTag *tagcontact = NULL;


	if (check_names_list == NULL) {
		osso_log(LOG_DEBUG, "[%s][%s] : invalid input :\n", __FILE__, __FUNCTION__);
		return;
	}
	osso_log(LOG_DEBUG, "[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	check_names_list_local = (GSList *) check_names_list;

	while (check_names_list_local != NULL) {
		node = check_names_list_local;
		addressBuffer = (GtkTextBuffer *) node->data;
		tagcontact =
		    gtk_text_buffer_create_tag(addressBuffer, NULL, "underline",
					       PANGO_UNDERLINE_SINGLE, "editable", TRUE, NULL);
		g_object_set_data(G_OBJECT(tagcontact), "emailid", GINT_TO_POINTER(EMAIL_ID));

		/*search in the loop for the emailids and tag them */
		gtk_text_buffer_get_start_iter(addressBuffer, &start);
		while (gtk_text_iter_is_end(&start) == FALSE) {
			if (!gtk_text_iter_forward_search(&start, "@",
							  GTK_TEXT_SEARCH_VISIBLE_ONLY,
							  &ratestart, &rateend, NULL))
				break;

			/*Get the iter where the email id ends */
			get_end_iter_for_emailid(&ratestart, &end, addressBuffer);
			gtk_text_buffer_apply_tag(addressBuffer, tagcontact, &start, &end);
			start = end;
			gtk_text_iter_forward_char(&start);
		}
		check_names_list_local = check_names_list_local->next;
	}
}


/**
 This is a helper function that checks for additional RE: in subject.
 
 @param mode is a gint that specifies the msgtype.
 @param subject is the gchar * subject string .
 @param subjectString is a OUT parameter containg the modified subject string.
 
 @return void.
*/
static void
ui_editor_subject_in_reply_mode(gint mode, const gchar * subject, gchar ** subjectString)
{
	gchar *temp_subject = NULL;
	gchar *modified_subject = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (mode == REPLY || mode == REPLY_ALL || mode == REPLY_FROM_FOLDER
	    || mode == REPLY_ALL_FROM_FOLDER) {
		if (subject != NULL) {
			temp_subject = g_strdup(subject);
			/* In this function a newly allocated subject string is returned.
			   with temp_subject being freed. */
			modified_subject = ui_editor_reply_check(temp_subject);
			*subjectString =
			    g_strconcat(_("mail_va_re"), SUBJECT_SEPERATOR, SPACE, modified_subject,
					NULL);
			g_free(modified_subject);
		} else {
			*subjectString = g_strconcat(_("mail_va_re"), SUBJECT_SEPERATOR, NULL);
		}
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This is a helper function that checks for additional FW: in subject.
 
 @param msgType is a gint that specifies the msgtype.
 @param subject is the gchar * subject string .
 @param subjectString is a OUT parameter subject string.
 
 @return void.
*/
static void
ui_editor_subject_in_forward_mode(gint msgType, const gchar * subject, gchar ** subjectString)
{
	gchar *fwstring = NULL;
	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	if (msgType == FORWARD || msgType == FORWARD_FROM_FOLDER) {
		if (subject != NULL) {
			fwstring = g_strdup(subject);
			/* In this function a newly allocated subject string is returned.
			   with fwstring being freed. */
			fwstring = ui_editor_forward_check(fwstring);
			*subjectString =
			    g_strconcat(_("mail_va_fw"), SUBJECT_SEPERATOR, SPACE, fwstring, NULL);
			g_free(fwstring);
			fwstring = NULL;
		} else {
			*subjectString = g_strconcat(_("mail_va_fw"), SUBJECT_SEPERATOR, NULL);
		}
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This is a helper function that sets the subject of the editor.
 
 @param mode is the mode in which the editor is opened.
 @param headerInfo is the  pointer headerInfo structure.
 @param entry_subject is the GtkWidget for subject.
 @param subjectBuf is the gchar pointer to the modified subject.
 @param editorview is the editor appview for which the title has to be set.
 
 @return void
 */
static void
ui_editor_set_subject(gint mode, HeaderInfo * headerInfo,
		      GtkWidget * entry_subject, const gchar * subjectBuf, GtkWidget * editorview)
{
	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	if (mode != NEW_MAIL) {
		CHECK_PARAM(headerInfo == NULL, "input param failed", return);
		if (mode == DRAFT_MAIL || mode == OUTBOX_MAIL) {
			subjectBuf = headerInfo->subject;
		}
		if (subjectBuf != NULL) {
			gtk_entry_set_text(GTK_ENTRY(entry_subject), subjectBuf);
			hildon_appview_set_title((HildonAppView *) editorview, subjectBuf);
		}
	} else {
		hildon_appview_set_title((HildonAppView *) editorview, _("mail_va_new_email"));
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This function parses the given file path for %20 characters
 	replaces them with spaces. Also creates temporary file paths

 @param filepath is the gchar pointer to the filepath of the attachment.
	 This is not fred here.
 @return TRUE -->on success
         FALSE --> on error creating the tmp file due no memory space. 

 @return gchar which is a nely allocated filepath with no %20 charectors.
*/
static gboolean parse_the_link(const gchar * filepath, AttachInfo * attach_info)
{
	gchar *selectedpath = NULL;
	gchar *attachmentname = NULL;
	gchar *temppath = NULL;

	if (filepath == NULL) {
		ULOG_DEBUG("unable to attach the file");
		return FALSE;
	}
	/*this function gets the display name  with proper encoding */
	attachmentname = g_filename_display_basename(filepath);

	/* the file is copied into temporary location */
	temppath = ui_create_filepath_link(filepath, &attachmentname, FALSE, TRUE);
	if (temppath == NULL) {
		/*File operation failed */
		g_free(attachmentname);
		return FALSE;
	}
	attach_info->attachmentname = g_strdup(attachmentname);
	attach_info->filepath = g_strdup(temppath);

	g_free(attachmentname);
	g_free(selectedpath);
	g_free(temppath);
	return TRUE;
}

/**
 This is a helper function to fill the msginfo parameter in the headerinfo structure
	 in sendas mode
	 
 @param editor_msginfo is pointer to the  MsgInfo structure of the editor.
	 This paramter is not freed in this function.
 @param msgInfo is pointer to the MsgInfo structure .IN parameter of editor_create()
	 This paramter is not freed in this function.
	 
 @return TRUE -->on success
         FALSE --> on error creating the tmp file due no memory space.
*/
static gboolean ui_editor_attachments_in_sendas_mode(MsgInfo * editor_msginfo, MsgInfo * msgInfo)
{
	GSList *attachment_list = NULL;
	AttachInfo *attachmentInfo = NULL;
	AttachInfo *attach_info = NULL;
	gchar *selectedpath = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (editor_msginfo == NULL || msgInfo == NULL) {
		ULOG_DEBUG_F(" no msginfo parameter\nnot in send as mode");
		return FALSE;
	}
	if (msgInfo->attach_list == NULL) {
		return FALSE;
	}

	gtk_banner_show_animation((GtkWindow *) app, _("mail_me_opening"));
	for (attachment_list = msgInfo->attach_list;
	     attachment_list != NULL; attachment_list = attachment_list->next) {
		attach_info = (AttachInfo *) attachment_list->data;
		attachmentInfo = g_new0(AttachInfo, 1);
		/*check if there r any urls send as attachment */
		if (attach_info->filepath != NULL && check_for_links(attach_info->filepath)) {

			/*It creates a new html file for the given url */
			attachmentInfo->filepath = prepare_files_for_urls(attach_info->filepath);
			/*if unable to create the file for audio */
			if (attachmentInfo->filepath == NULL) {
				ui_freelist(attachmentInfo, NULL);
				continue;
			}
			attachmentInfo->type = DIRECT;
			attachmentInfo->attachmentname =
			    get_short_filename(attachmentInfo->filepath, &selectedpath);
			g_free(selectedpath);
			editor_msginfo->attach_list =
			    g_slist_append(editor_msginfo->attach_list, attachmentInfo);

		} else if (attach_info->filepath != NULL) {
			attachmentInfo->type = attach_info->type;
			attachmentInfo->msgid = NULL;

			/*escapes the filepath & copies file to temp folder .fills the attachmentname & filepath
			 * of the attachmentInfo structure.*/
			if (parse_the_link(attach_info->filepath, attachmentInfo) == FALSE) {
				/*File operation failed */
				ui_freelist(attachmentInfo, NULL);
				gtk_banner_close((GtkWindow *) app);
				return FALSE;
			}
			editor_msginfo->attach_list =
			    g_slist_append(editor_msginfo->attach_list, attachmentInfo);
		} else {
			/*free the attachmentInfo structure if filpath is NULL */
			ui_freelist(attachmentInfo, NULL);
		}
		ui_flush_gtk_events();
	}
	gtk_banner_close((GtkWindow *) app);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return TRUE;
}


/**
 This is a helper function to show the attachmnets in the attachment buffer.

 @param attach_view is the attachment view.
 @param attachment_list is the list of attachinfo structures.
 	This paramter is not freed in this function.
 
 @return void
*/
void ui_editor_show_attachments(GtkListStore * liststore, const GSList * attachment_list)
{
	AttachInfo *attach_info = NULL;
	gchar *attachment_name = NULL;
	gchar *attachmentSize = NULL;
	GSList *temp_list = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	temp_list = (GSList *) attachment_list;

	while (temp_list != NULL && temp_list->data != NULL) {
		attach_info = temp_list->data;
		attachment_name = attach_info->attachmentname;
		/*Calculates the attachment size & returns a formated string which
		 * has to be freed*/
		attachmentSize = (gchar *) ui_file_size(attach_info->filepath);
		ui_show_attachments_in_buffer(liststore, attachment_name, attach_info->filepath,
					      attachmentSize, FALSE);
		temp_list = temp_list->next;
		g_free(attachmentSize);
		attachmentSize = NULL;
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This is a helper function that checks for signature for a account
 
 @return gchar string of the signature.the caller function frees this.
*/
static gchar *ui_editor_check_for_siganuters(gchar * account)
{
	PrefsAccount *update_prefsAccount = NULL;
	gchar *sign = NULL;
	gchar *signature_file = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	update_prefsAccount = ui_get_account_detail(account);
	if (update_prefsAccount == NULL) {
		ULOG_CRIT("[%s][%s]:al_get_account_settings failed :\n", __FILE__, __FUNCTION__);
		return NULL;
	}
	if (update_prefsAccount->advanced_settings.include_signature == FALSE) {
		ui_free_prefs_account_structure(update_prefsAccount);
		return NULL;
	}

	signature_file = get_signature_filepath(account);
	read_file(signature_file, &sign);
	g_free(signature_file);
	ui_free_prefs_account_structure(update_prefsAccount);

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return sign;
}

/**
 This is a helper function to embed the siganture in the editor.
 
 @param sign  is a gchar for  signature string.
 @param mode is a gint representing  the mode in which the msg is opened.
 @param buffer_body is the textbuffer of the body.
 
 @return void.
*/
static void ui_editor_embed_siganture(const gchar * sign, gint mode, GtkTextBuffer * buffer_body)
{
	GtkTextIter iter;
	gchar *signature = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	if (sign == NULL) {
		return;
	}
	if (mode != DRAFT_MAIL && mode != OUTBOX_MAIL) {
		signature = g_strconcat(CARRIAGE_RETURN, sign, CARRIAGE_RETURN, NULL);
		if (mode == SEND_AS) {
			gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer_body), &iter);
		} else {
			gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(buffer_body), &iter);
		}

		gtk_text_buffer_insert(GTK_TEXT_BUFFER(buffer_body), &iter, signature, -1);
		g_free(signature);
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This is a helper function to insert the orininal string while not new mail.
 
 @param editor_info is a EditorInfo structure pointer.
 	This paramter is not freed in this function.
 @param buffer_body is a text buffer for body.
 
 @return void.
*/
static void ui_editor_insert_orgstring(EditorInfo * editor_info, GtkTextBuffer * buffer_body)
{
	GtkTextIter iter;
	gchar *orgstr = NULL;
	gint mode = NEW_MAIL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);

	mode = editor_info->composeType;
	orgstr = g_strconcat(CARRIAGE_RETURN, _("mcen_ia_editor_original_message"),
			     CARRIAGE_RETURN, NULL);

	if ((mode == REPLY_ALL_FROM_FOLDER || mode == REPLY_FROM_FOLDER
	     || mode == FORWARD_FROM_FOLDER || mode == REPLY || mode == REPLY_ALL
	     || mode == FORWARD)) {
		/*Checking whether the editor is directly opened from the Inbox/send.
		   if the editor is html then the body is send as attachment.
		   No need for inserting the string ORIGINAL_MSg */
		if (((mode == FORWARD_FROM_FOLDER || mode == FORWARD)
		     && editor_info->emailType == HTML_EDITOR)) {
			ULOG_DEBUG("In this mode mail is send as attcahment.");
		} else {
			gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer_body), &iter);
			gtk_text_buffer_insert(GTK_TEXT_BUFFER(buffer_body), &iter, orgstr, -1);
		}
	}

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	g_free(orgstr);
	return;
}

/**
This function sets the editor to be html/plain based on the
	mimetype of the mail.

@param editor_info is a EditorInfo structure pointer which contains the mimeinfo
 This paramter is not freed in this function.
 
@return void.
*/
static void ui_editor_set_mail_type(EditorInfo * editor_info)
{
	MimeInfo *partinfo = NULL, *temp_partinfo = NULL;
	MimeInfo *mimeInfo = NULL;
	gboolean html_mail = FALSE;

	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);
	CHECK_PARAM(editor_info->mimeInfo == NULL, "Input param failed\n", return);
	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	mimeInfo = editor_info->mimeInfo;

	partinfo = mimeInfo;

	if (partinfo->mime_type == MIME_MULTIPART) {
		for (temp_partinfo = (MimeInfo *) partinfo; temp_partinfo != NULL;
		     temp_partinfo = temp_partinfo->children) {
			if (g_ascii_strncasecmp
			    (temp_partinfo->content_type, MULTIPART_ALTERNATIVE,
			     MAX_STR_LEN(temp_partinfo->content_type,
					 MULTIPART_ALTERNATIVE)) == 0
			    || g_ascii_strncasecmp(temp_partinfo->content_type,
						   MULTIPART_RELATED,
						   MAX_STR_LEN(temp_partinfo->content_type,
							       MULTIPART_RELATED)) == 0
			    || g_ascii_strncasecmp(partinfo->content_type,
						   MULTIPART_TEXT_HTML,
						   MAX_STR_LEN(partinfo->content_type,
							       MULTIPART_TEXT_HTML)) == 0) {
				html_mail = TRUE;
				break;
			}
		}
		if (html_mail == TRUE) {
			editor_info->emailType = HTML_EDITOR;
			editor_info->msgInfo->settings->type = EMAIL_HTML;
		} else {
			editor_info->emailType = PLAIN_EDITOR;
			editor_info->msgInfo->settings->type = EMAIL_TEXT;
		}
	} else if (partinfo->mime_type == MIME_TEXT_HTML) {
		editor_info->emailType = HTML_EDITOR;
		editor_info->msgInfo->settings->type = EMAIL_HTML;
	} else {
		editor_info->emailType = PLAIN_EDITOR;
		editor_info->msgInfo->settings->type = EMAIL_TEXT;
	}


	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This is a helper function that embeds the attachment names in the editor
 in reply mode.
 
 @param editor_info is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
 
 @param attachments is a GSList containing the attachment names.
 
 @return void.
*/
static void
ui_editor_insert_attachments_in_reply(EditorInfo * editor_info, const GSList * attachments)
{
	gchar *attachmentName = NULL;
	gchar *FormatedName = NULL;
	MsgFields *msgFields = NULL;
	gint mode = NEW_MAIL;
	GtkTextIter iter;
	GtkTextBuffer *buffer_body = NULL;
	GSList *attachmentList = NULL;

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);

	mode = editor_info->composeType;
	msgFields = editor_info->msgFields;
	buffer_body = gtk_text_view_get_buffer((GtkTextView *) msgFields->body);
	attachmentList = (GSList *) attachments;

	if (mode == REPLY || mode == REPLY_ALL
	    || mode == REPLY_FROM_FOLDER || mode == REPLY_ALL_FROM_FOLDER) {
		gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer_body), &iter);
		gtk_text_buffer_insert(buffer_body, &iter, CARRIAGE_RETURN, -1);
		if (attachmentList != NULL) {
			gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer_body), &iter);
			gtk_text_buffer_insert(buffer_body, &iter,
					       _("mcen_ia_editor_attach_filename"), -1);
		}

		while (attachmentList != NULL && attachmentList->data != NULL) {
			attachmentName = attachmentList->data;
			FormatedName = g_strconcat(SPACE, HTML_LT, attachmentName, HTML_GT, NULL);
			gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer_body), &iter);
			gtk_text_buffer_insert(buffer_body, &iter, FormatedName, -1);
			g_free(FormatedName);
			FormatedName = NULL;
			attachmentName = NULL;
			attachmentList = attachmentList->next;
		}
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}


/*function is used to get the body contents of a plain email attachment.
 * This is used when reply/replyall/forward from a email attachment.

 @param editor_info is the EditorInfo structure paramter.
 @param msgHeader is the HeaderInfo structure paramter.
 @param mode is the gint represeting the mode in which editor is opened.
 @return gboolean TRUE--> if we get the body file
                  FALSE--> else.
*/
static gboolean get_body_file(EditorInfo * editor_info, HeaderInfo * msgHeader, gint mode)
{
	MsgInfo *msginfo = NULL;
	AttachInfo *attach_info = NULL;
	gboolean got_body_file = FALSE;

	ui_editor_set_mail_type(editor_info);
	if (editor_info->emailType == PLAIN_EDITOR || mode == REPLY || mode == REPLY_ALL) {
		msginfo =
		    ui_dbus_recv_msg_attachment(editor_info->msgid, editor_info->mimeInfo->main);
		if (msginfo == NULL)
			return FALSE;
		if (msginfo->attach_list != NULL) {
			attach_info = (AttachInfo *) g_slist_nth_data(msginfo->attach_list, 0);
			if (attach_info != NULL && attach_info->filepath != NULL) {
				g_free(editor_info->mimeInfo->main->filename);
				editor_info->mimeInfo->main->filename = NULL;
				editor_info->mimeInfo->main->filename =
				    g_strdup(attach_info->filepath);
				got_body_file = TRUE;
			}
		}
		ui_freeMem(msginfo, NULL);
	} else
		return TRUE;

	return got_body_file;
}

/**
 This function is used to get the mimeInfo structure pointer of the editor.
	 it also sets the email type of the ditor i.e plain/html.

 @param editor_info is the EditorInfo structure pointer of the current editor.
 @parma msgHeader is the HeaderInfo structure pointer.
 @return gboolean TRUE-if engine returns the mimeinfo.
	 FALSE-else.
*/
static gboolean ui_editor_get_mimeinfo(EditorInfo * editor_info, HeaderInfo * msgHeader)
{
	gint mode = NEW_MAIL;
	gint errorNum = -1;
	gchar *emailid = NULL;
	gchar *sendersid = NULL;
	gchar *password = NULL;
	guint certid = 0;
	gint cert_state = 0;
	gboolean html_flag = FALSE;

	mode = editor_info->composeType;
	if (mode == OUTBOX_MAIL || mode == DRAFT_MAIL) {
		html_flag = TRUE;
	}
	if (editor_info->accountSettings != NULL) {
		emailid = editor_info->accountSettings->user_settings.email;
	} else {
		emailid = NULL;
	}
	if (msgHeader != NULL && msgHeader->mimeInfo != NULL) {
		/*editor is opened from a email attachment */
		editor_info->mimeInfo = msgHeader->mimeInfo;
		return (get_body_file(editor_info, msgHeader, mode));
	} else if (mode != NEW_MAIL && mode != SEND_AS) {
		sendersid = ui_parse_from_address(msgHeader->from);
		if (msgHeader->smime_type != SMIME_NONE
		    && msgHeader->smime_type != SMIME_INTERNAL
		    && msgHeader->smime_type != SMIME_SIGN) {
			certid = get_private_certificate(emailid, &cert_state);
			if (certid == 0) {
				get_user_confirmation(_("mcen_in_smime_cannot_decrypt"), NULL, NULL,
						      INFO_BANNER);
				return FALSE;
			}
			password = get_certificate_password(certid);
			if (password == NULL) {
				g_free(sendersid);
				return FALSE;
			}
		}
		editor_info->mimeInfo =
		    (MimeInfo *) ui_dbus_recv_msg_body(editor_info->msgid, TRUE,
						       msgHeader->download_attachments,
						       sendersid, emailid,
						       msgHeader->smime_type, TRUE, certid,
						       password, TRUE, html_flag);

		g_free(sendersid);
		g_free(password);
		if ((editor_info->mimeInfo) == NULL) {
			errorNum = ui_dbus_get_engine_errno();
			ui_set_msg_cb(errorNum);
			ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
			return FALSE;
		}
	}
    /**Finding whether it is a html/plain mail,when the editor is opened other then
       * new_mail mode*/
	ui_editor_set_mail_type(editor_info);
	return TRUE;

}

/**
 This is a helper function that checks for attachments .
 
 @param editor_info is the EditorInfo structure pointer.
	 This paramter is not freed in this function.
 @param msgHeader is a  pointer to the HeaderInfo structure.
 	This paramter is not freed in this function.
 @param attachmentList is a GSList containing the attachment names(OUT)param.
 
 @return FALSE if unable to allocate memory or on error.
         True,else.
*/
static gboolean
ui_editor_check_attachments(EditorInfo * editor_info,
			    const HeaderInfo * msgHeader, GSList ** attachmentList)
{
	AttachmentStatus attachmentStatus = NO_ATTACHMENTS;
	gint mode = NEW_MAIL;
	MimeInfo *mimeInfo = NULL;
	MsgInfo *msgInfo = NULL;
	MsgFields *msgFields = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgInfo == NULL, "Input param failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgFields == NULL, "Input param failed\n", return FALSE);

	mode = editor_info->composeType;
	mimeInfo = editor_info->mimeInfo;
	msgInfo = editor_info->msgInfo;
	msgFields = editor_info->msgFields;
	if (mode == NEW_MAIL) {
		return TRUE;
	}
	CHECK_PARAM(msgHeader == NULL, "Input param failed\n", return FALSE);

	if ((mode == FORWARD || mode == FORWARD_FROM_FOLDER)
	    && editor_info->emailType == HTML_EDITOR) {
		ULOG_DEBUG_F("In forwarding mail is added as attachment.");
		return TRUE;
	}
	attachmentStatus =
	    ui_viewer_check_for_attachments(editor_info->mimeInfo,
					    editor_info->msgid,
					    msgFields->liststore, msgInfo,
					    mode, (msgHeader->download_attachments),
					    attachmentList);
	if (attachmentStatus == MEM_ALLOCATION_FAIL) {
		ULOG_DEBUG("Input param check failed");
		hildon_app_set_appview(app, appview);
		return FALSE;
	} else if (attachmentStatus == NO_ATTACHMENTS) {
		show_attachment_field(msgFields, FALSE);
	} else if (mode == FORWARD || mode == FORWARD_FROM_FOLDER
		   || mode == OUTBOX_MAIL || mode == DRAFT_MAIL) {
		stop_editor_from_recieving_events(editor_info, TRUE);
	}
	if (mode == SEND_AS) {
		if (msgHeader->msgInfo->attach_list != NULL) {
			if (ui_editor_attachments_in_sendas_mode(msgInfo, msgHeader->msgInfo) ==
			    FALSE) {
				hildon_app_set_appview(app, appview);
				return FALSE;
			}
			ui_editor_show_attachments(msgFields->liststore, msgInfo->attach_list);
			set_attachment_view_size(msgInfo->attach_list, msgFields);
			show_attachment_field(msgFields, TRUE);
		}
	}

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return TRUE;
}

/**
 This is a helper fuction that gets the account settings of a given msgid &

 @param msgid is the gchar representing the msgid of the mail.
 
 @return TRUE if -original is reply is set
        FALSE-else
*/
gboolean ui_get_original_in_reply_setting(const gchar * msgid)
{
	gchar *temp_acc = NULL;
	gchar *current_acc = NULL;
	PrefsAccount *accountSettings = NULL;
	gboolean ori_in_reply = TRUE;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (msgid == NULL) {
		return ori_in_reply;
	}
	temp_acc = strchr(msgid, MSGID_SEPARATOR);
	if (temp_acc != NULL) {
		current_acc = g_strndup(msgid, strlen(msgid) - strlen(temp_acc));
		if (current_acc != NULL
		    && g_ascii_strncasecmp(current_acc, DEFAULT,
					   MAX_STR_LEN(current_acc, DEFAULT)) != 0) {
			accountSettings = ui_get_account_detail(current_acc);
			if (accountSettings != NULL) {
				ori_in_reply = accountSettings->advanced_settings.org_in_reply;
				ui_free_prefs_account_structure(accountSettings);
			}
		}
		g_free(current_acc);
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return ori_in_reply;
}


/**
 This is a helper function to get the current account & the send option.
 
 @param headerInfo is a HeaderInfo pointer.
	 This paramter is not freed in this function.
 @param send_option is the OUT param.
 @param editor_info is the EditorInfo structure pointer.
	 This paramter is not freed in this function.
 
 @result gchar the current account name.
*/
static gchar *ui_editor_get_current_account_settings(const HeaderInfo * headerInfo,
						     gint * send_option, EditorInfo * editor_info)
{
	gchar *temp_acc = NULL;
	gchar *current_acc = NULL;
	gchar *send_pref = NULL;
	ULOG_DEBUG("[%s]:[%d] :starts here\n", __FUNCTION__, __LINE__);

	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return NULL);
	CHECK_PARAM(headerInfo == NULL, "Input param failed\n", return NULL);
	CHECK_PARAM(headerInfo->msgid == NULL, "Input param failed\n", return NULL);

	temp_acc = strchr(headerInfo->msgid, MSGID_SEPARATOR);
	if (temp_acc == NULL) {
		return NULL;
	}
	current_acc = g_strndup(headerInfo->msgid, strlen(headerInfo->msgid) - strlen(temp_acc));
	/*Checking for the dummy account */
	if (g_ascii_strcasecmp(current_acc, DEFAULT) != 0) {
		if (editor_info->accountSettings != NULL) {
			ui_free_prefs_account_structure(editor_info->accountSettings);
			editor_info->accountSettings = NULL;
		}
		editor_info->accountSettings = ui_get_account_detail(current_acc);
	}
	send_pref = (gchar *) strrchr(headerInfo->msgid, MSGID_SEPARATOR);
	if (send_pref == NULL) {
		return NULL;
	}
	send_pref = g_strndup(send_pref + 1, 1);
	*send_option = atoi(send_pref);
	g_free(send_pref);

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return current_acc;
}

/**
 Function to prepare the formatted date string.
 
 @param recvDate is the time_t value.
 
 @return formatted gchar* date string.
*/
static gchar *prepare_time_string(time_t recvDate)
{
	struct tm *time_ptr = NULL;
	gint retval = 0;
	gchar *sentTime = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	time_ptr = localtime((const time_t *) &(recvDate));
	sentTime = g_new0(gchar, TIME_STRING_SPACE);
	retval = strftime(sentTime, TIME_STRING_SPACE, "%c", time_ptr);
	if (retval == 0) {
		g_free(sentTime);
		return NULL;
	}

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return (sentTime);
}

/**
 This function reads the contents of a file & put them in the
 body parameter whish is a OUT param.
 
 @param fileName is the file name of the file.
 @param body is the OUT parm in which the content of the body is present.
 
 @return void.
*/
void read_file(const gchar * fileName, gchar ** body)
{

	GError *error = NULL;
	gchar *message_content = NULL;

	if (body == NULL)
		return;


	if (g_file_get_contents(fileName, &message_content, NULL, &error) == FALSE) {
		ULOG_CRIT_F("error reading the file contents. error is %s", error->message);
		g_error_free(error);
		return;
	}
	*body = message_content;
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 THis is a helper function for creating the email header while replying or
 forwarding a mail.
 
 @param is the headerInfo of the message.
 This paramter is not freed in this function.

 @return is the formatted email header string.
*/
static gchar *ui_prepare_message_header(const HeaderInfo * headerInfo)
{
	GString *headerStr = NULL;
	gchar *sentDate = NULL;
	gchar *headers = NULL;

	ULOG_DEBUG("[%s]:[%d] :starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(headerInfo == NULL, "Input param failed", return NULL);

	headerStr = g_string_new(NULL);
	if (headerStr == NULL) {
		ULOG_CRIT_F("unable to allocate memory\n");
		return NULL;
	}
	if (headerInfo->from != NULL) {
		g_string_append_printf(headerStr, "%s %s\n", _("mail_va_from"), headerInfo->from);
	}
	if (headerInfo->to != NULL) {
		g_string_append_printf(headerStr, "%s %s\n", _("mail_va_to"), headerInfo->to);
	}
	if (headerInfo->cc != NULL && strlen(headerInfo->cc) > 0) {
		g_string_append_printf(headerStr, "%s %s\n", _("mail_va_cc"), headerInfo->cc);
	}
	if (headerInfo->sent_date != 0) {
		sentDate = prepare_time_string(headerInfo->sent_date);
		if (sentDate != NULL) {
			g_string_append_printf(headerStr, "%s: %s\n", _("mail_ln_sent"), sentDate);
			g_free(sentDate);
		}
	}
	if (headerInfo->subject != NULL) {
		g_string_append_printf(headerStr, "%s %s\n", _("mail_va_subject"),
				       headerInfo->subject);
	}
	if (headerInfo->priority != NORMAL) {
		g_string_append(headerStr, _("emev_fi_edi_smtp_sendopt_priority"));
		if (!g_str_has_suffix(_("emev_fi_edi_smtp_sendopt_priority"), SUBJECT_SEPERATOR)) {
			g_string_append(headerStr, SUBJECT_SEPERATOR);
		}
		g_string_append(headerStr, " ");
		if (headerInfo->priority == LOW) {
			g_string_append(headerStr, _("emev_va_smtp_sendopt_prilow"));
		} else {
			g_string_append(headerStr, _("emev_va_smtp_sendopt_prihigh"));
		}
		g_string_append(headerStr, CARRIAGE_RETURN);
	}
	headers = g_string_free(headerStr, FALSE);
	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	return (headers);
}

/**
 This function is used to set the msg header either in reply/forward
 mail
 @param headerInfo is a pointer to the HeaderInfo structure.
 	This paramter is not freed in this function.
 @param buffer_body is a pointer to the GtkTextBuffer for the body.
 
 @return
*/
static void ui_set_message_header(const HeaderInfo * headerInfo, GtkTextBuffer * buffer_body)
{
	gchar *headerStr = NULL;
	GtkTextIter iter;

	headerStr = ui_prepare_message_header(headerInfo);
	if (headerStr != NULL) {
		gtk_text_buffer_get_end_iter((GtkTextBuffer *) buffer_body, &iter);
		gtk_text_buffer_insert(GTK_TEXT_BUFFER(buffer_body), &iter, headerStr, -1);
		g_free(headerStr);
	}
	return;
}

/**
 This is ahelper function to add the html mail as attachment when forwarding.
 
 @param editor_info is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
 @param msgHeader is a  pointer to the HeaderInfo structure.
	 This paramter is not freed in this function.
 @param body is the body of the editor.
 
 @result void.
*/
static void
ui_editor_in_reply_mode(EditorInfo * editor_info, const HeaderInfo * headerInfo, const gchar * body)
{
	GtkTextIter iter;
	gint mode = NEW_MAIL;
	MsgFields *msgFields = NULL;
	GtkTextBuffer *buffer_body = NULL;
	MimeInfo *mimeInfo = NULL;
	gboolean plainpart = FALSE;
	gchar *converted_file = NULL, *charset = NULL;


	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);
	CHECK_PARAM(editor_info->msgFields == NULL, "Input param failed\n", return);

	mode = editor_info->composeType;
	if (mode != SEND_AS && mode != NEW_MAIL) {
		CHECK_PARAM(editor_info->mimeInfo == NULL, "Input param failed\n", return);
	}
	msgFields = editor_info->msgFields;

/*if editor opened from email attachment*/
	if (headerInfo != NULL && headerInfo->mimeInfo != NULL)
		mimeInfo = editor_info->mimeInfo->main;
	else
		mimeInfo = editor_info->mimeInfo;

	buffer_body = gtk_text_view_get_buffer((GtkTextView *) msgFields->body);
	if (mode == REPLY || mode == REPLY_ALL || mode == REPLY_ALL_FROM_FOLDER
	    || mode == REPLY_FROM_FOLDER || mode == SEND_AS
	    || ((mode == FORWARD || mode == FORWARD_FROM_FOLDER) &&
		editor_info->emailType == PLAIN_EDITOR)) {
		if (mode != SEND_AS) {
			ui_set_message_header(headerInfo, buffer_body);
		}
		if (body != NULL && strlen(body) > 1) {
			if (mode == SEND_AS)
				gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(buffer_body), &iter);
			else
				gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(buffer_body), &iter);
			gtk_text_buffer_insert(GTK_TEXT_BUFFER(buffer_body), &iter, body, -1);
		}
		/*for editor opened thr email attachment */
		else if (headerInfo != NULL && headerInfo->mimeInfo != NULL) {
			ui_read_bodyFile(mimeInfo->filename, buffer_body);
			return;
		}

		else if (mimeInfo != NULL && mimeInfo->filename != NULL) {
			plainpart = check_for_plain_html(mimeInfo);
			if (plainpart == FALSE) {
				charset = UTF8;
				ui_dbus_html_to_text_converter(mimeInfo->filename, charset,
							       &converted_file);
			} else {
				converted_file = g_strdup(mimeInfo->filename);
			}
			if (converted_file != NULL) {
				ui_read_bodyFile(converted_file, buffer_body);
				if (plainpart == FALSE) {
					gnome_vfs_unlink(converted_file);
				}
				g_free(converted_file);
			}
		}
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}


/**
 helper function to create the attachInfo while forwarding html mail from mainui.

 @param headerInfo is a pointer to the HeaderInfo structure.
  This paramter is not freed in this function.
 @param editor_info is the EditorInfo structure pointer.
  This paramter is not freed in this function.
 @result FALSE-if param check fails.
       TRUE-else.
*/
static gboolean ui_editor_create_attachinfo(HeaderInfo * headerInfo, EditorInfo * editor_info)
{
	AttachInfo *attachSend = NULL;
	MsgFields *msgFields = NULL;
	gchar *size = NULL;

	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return FALSE);
	CHECK_PARAM(headerInfo == NULL, "Input param failed\n", return FALSE);

	msgFields = editor_info->msgFields;
	attachSend = g_new0(AttachInfo, 1);

	attachSend->filepath = NULL;
	if (headerInfo->subject != NULL) {
		attachSend->attachmentname =
		    g_strndup(headerInfo->subject, strlen(headerInfo->subject));
	} else {
		attachSend->attachmentname = g_strdup(_("mail_va_no_subject"));
	}

	attachSend->type = EMAIL;
	attachSend->msgid = g_strdup(headerInfo->msgid);

	/*We get the path of the mail from msgid */
	if (headerInfo->email_filename == NULL)
		headerInfo->email_filename = ui_get_message_file_path(headerInfo->msgid,
								      headerInfo->folderNum);
	/*copy email file path to attachments location */
	attachSend->email_path =
	    ui_create_filepath_link(headerInfo->email_filename, &(attachSend->attachmentname),
				    FALSE, FALSE);
	size = (gchar *) ui_file_size(attachSend->email_path);

	editor_info->msgInfo->attach_list =
	    g_slist_append(editor_info->msgInfo->attach_list, attachSend);
	ui_show_attachments_in_buffer(msgFields->liststore, attachSend->attachmentname,
				      attachSend->email_path, size, FALSE);
	set_attachment_view_size(editor_info->msgInfo->attach_list, msgFields);
	show_attachment_field(msgFields, TRUE);
	g_free(size);
	return TRUE;

}


/**
 This function is called when the editor is opened from the mail folders directly.
 
 @param editor_info is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
 @param headerInfo is a  pointer to the HeaderInfo structure.
	 This paramter is not freed in this function.
 @param body is the body of the editor.
	 This paramter is not freed in this function.
	 
 @result gboolean FALSE,if input validation/memory allocation fails
                  TRUE,else.
*/
static gboolean
ui_editor_from_folder(EditorInfo * editor_info, const HeaderInfo * headerInfo, const gchar * body)
{
	gint mode = NEW_MAIL;
	MsgFields *msgFields = NULL;
	GtkTextBuffer *buffer_body = NULL;
	MimeInfo *mimeInfo = NULL;
	gboolean mem_allocated = TRUE;
	gchar *html_filename = NULL;
	gchar *message_content = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Input param failed", return FALSE);
	CHECK_PARAM(editor_info->mimeInfo == NULL,
		    "probably a new mail or editor in send as mode.", return TRUE);

	mode = editor_info->composeType;
	msgFields = editor_info->msgFields;
	CHECK_PARAM(mode == NEW_MAIL, "No content to be set in the body", return TRUE);

	if (headerInfo->mimeInfo != NULL)
		mimeInfo = editor_info->mimeInfo->main;
	else
		mimeInfo = editor_info->mimeInfo;

	buffer_body = gtk_text_view_get_buffer((GtkTextView *) msgFields->body);

	html_filename = g_strdup(mimeInfo->filename);
	if (((mode == DRAFT_MAIL || mode == OUTBOX_MAIL)
	     && editor_info->emailType == HTML_EDITOR)) {
		read_file(html_filename, &message_content);
		osso_email_convert_html_to_gtk(editor_info, message_content);
		g_free(message_content);
	} else if (mode == DRAFT_MAIL || mode == OUTBOX_MAIL) {
		ui_read_bodyFile(html_filename, buffer_body);
	}
	if (((mode == FORWARD_FROM_FOLDER || mode == FORWARD)
	     && editor_info->emailType == HTML_EDITOR)) {
		/*Preparing the html mail as attachment while forwarding */
		mem_allocated = ui_editor_create_attachinfo((HeaderInfo *) headerInfo, editor_info);
	}
	g_free(html_filename);
	return mem_allocated;
}

/*helper function to unlink the attachment paths.
 * @param attach_info is the pointer to the AttachInfo structure pointer.
 * @return void.
 */
static void unlink_paths(AttachInfo * attach_info)
{
	if (attach_info->filepath != NULL)
		gnome_vfs_unlink(attach_info->filepath);
	if (attach_info->email_path != NULL)
		gnome_vfs_unlink(attach_info->email_path);
}

/**
 Function to free the undoinfo .
 
 @param editorinfo is the EditorInfo structure pointer.
 
 @return void.
*/
static void ui_free_undoinfo(EditorInfo * editorinfo)
{
	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (editorinfo != NULL) {
		undo_destroy(editorinfo);
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 function to free the memory allocated to the editorInfo structure.
 
 @param editorInfo is the EditorInfo structure pointer.
 
 @return void.
*/
void ui_free_editorinfo(EditorInfo * editorInfo)
{
	ULOG_DEBUG("[%s]:[%d] :starts here\n", __FUNCTION__, __LINE__);
	if (editorInfo == NULL) {
		return;
	}
	if (editorInfo->editor_appview != NULL) {
		gtk_widget_destroy((GtkWidget *)
				   hildon_appview_get_menu((HildonAppView *)
							   editorInfo->editor_appview));
		/*This call  unregisters the view */
		hildon_app_unregister_view(app, editorInfo->editor_appview);
		gtk_widget_destroy(editorInfo->editor_appview);
	}
	g_free(editorInfo->menu);
	g_free(editorInfo->msgFields);
	g_free(editorInfo->toolbar);
	if (editorInfo->mimeInfo != NULL && editorInfo->mimeInfo->filename != NULL) {
		gnome_vfs_unlink(editorInfo->mimeInfo->filename);
	}
	ui_mimeinfo_free(editorInfo->mimeInfo);
	ui_free_undoinfo(editorInfo);

	if (editorInfo->editorModified == FALSE)
		g_slist_foreach(editorInfo->msgInfo->attach_list, (GFunc) unlink_paths, NULL);

	ui_freeMem(editorInfo->msgInfo, NULL);
	/*deleting the msg from the open messages list once the editor is closed. */
	if (editorInfo->composeType == OUTBOX_MAIL) {
		ui_dbus_editor_open_mail(editorInfo->msgid, EDITOR_OPEN_MSG_DEL);
	}
	g_free(editorInfo->msgid);
	g_free(editorInfo->old_msgid);
	if (editorInfo->zoominfo != NULL) {
		g_free(editorInfo->zoominfo->msgid);
		g_free(editorInfo->zoominfo);
	}
	/*The contents of list contain the textbuffers which get unrefed once
	   the widget is destroyed */
	g_slist_free(editorInfo->addressBuffers);
	ui_free_prefs_account_structure(editorInfo->accountSettings);
	if (editorInfo->attr_list != NULL) {
		pango_attr_list_unref(editorInfo->attr_list);
	}
	g_free(editorInfo->fieldinfo_to);
	g_free(editorInfo->fieldinfo_cc);
	g_free(editorInfo->fieldinfo_bcc);
	g_free(editorInfo);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This function is used to fill the email specific settings of msginfo structure.
 
 @param editor_info is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
 @param msgHeader is a  pointer to the HeaderInfo structure.
 	This paramter is not freed in this function.
 @param mail is the gint representing the mode in which editor is opened.
 
 @return void.
*/
static void
ui_editor_initialise_mailsettings(EditorInfo * editorInfo, const HeaderInfo * msgHeader, gint mail)
{
	EmailSpecificSettings *settings = NULL;
	gchar *current_acc = NULL;
	gchar *defAccName = NULL;
	gint send_option = SEND_IMMEDIATE;
	MsgInfo *msginfo = NULL;
	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	if (editorInfo == NULL || (editorInfo != NULL && editorInfo->msgInfo == NULL)) {
		return;
	}
	msginfo = editorInfo->msgInfo;
	msginfo->settings = g_new0(EmailSpecificSettings, 1);
	settings = msginfo->settings;
	if (email_main_view->account_manager->default_account != NULL) {
		defAccName =
		    g_strndup(email_main_view->account_manager->default_account,
			      strlen(email_main_view->account_manager->default_account));
	}

	if (defAccName != NULL && g_ascii_strncasecmp(defAccName, DEFAULT,
						      MAX_STR_LEN(defAccName, DEFAULT)) != 0) {
		editorInfo->accountSettings = ui_get_account_detail(defAccName);
	}
	current_acc = ui_editor_get_current_account_settings(msgHeader, &send_option, editorInfo);

	if ((mail == DRAFT_MAIL || mail == OUTBOX_MAIL) && msgHeader != NULL) {
		settings->priority = msgHeader->priority;
		settings->read_receipt_request = msgHeader->read_receipt;
		settings->copy_to_own_address = msgHeader->copy_to_own_addr;
		settings->options = send_option;
		settings->smime_type = msgHeader->smime_type;
	} else {
		settings->priority = NORMAL;
	}
	if (editorInfo->emailType == HTML_EDITOR) {
		settings->type = EMAIL_HTML;
	} else {
		settings->type = EMAIL_TEXT;
	}
	if ((editorInfo->accountSettings != NULL) && (mail != DRAFT_MAIL)
	    && (mail != OUTBOX_MAIL)) {
		settings->read_receipt_request =
		    editorInfo->accountSettings->advanced_settings.request_read_report;
		settings->copy_to_own_address =
		    editorInfo->accountSettings->advanced_settings.copy_to_own_address;
		settings->options = editorInfo->accountSettings->advanced_settings.email_sending;
		/*get the account specific security options for the mail */
		settings->smime_type =
		    editorInfo->accountSettings->advanced_settings.smime_security;
		if (mail == NEW_MAIL) {
			settings->type = editorInfo->accountSettings->advanced_settings.email_type;
		}
	}
	if (mail == OUTBOX_MAIL) {
		settings->smime_type = msgHeader->smime_type;
	}
	if (current_acc == NULL && email_main_view->account_manager->default_account != NULL) {
		settings->account = g_strdup(email_main_view->account_manager->default_account);
	} else if (current_acc != NULL) {
		settings->account = g_strdup(current_acc);
	}
	g_free(defAccName);
	g_free(current_acc);
	return;

}

/**
 This function is used to initilase the undo info structure
 
 @param editorinfo is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
	
 @return True-successful memory allocation & initilaisation.
	 False-else.
*/
static gboolean ui_editor_initialise_undoinfo(EditorInfo * editorinfo)
{
	gboolean initialised = TRUE;
	editorinfo->undo_queue = g_queue_new();
	editorinfo->prevcolor = g_new0(GdkColor, 1);
	editorinfo->bgcolor = g_new0(GdkColor, 1);
	editorinfo->bgcolor->pixel = 0;
	editorinfo->bgcolor->red = 65535;
	editorinfo->bgcolor->green = 65535;
	editorinfo->bgcolor->blue = 65535;
	/*editorinfo->prevcolor = gdk_color_copy(editorinfo->bgcolor); */
	editorinfo->prevcolor->pixel = 0;
	editorinfo->prevcolor->red = 65535;
	editorinfo->prevcolor->green = 65535;
	editorinfo->prevcolor->blue = 65535;
	return initialised;
}

/**
 Fuction to allocate memory for the editorInfo structure & its sub fields
 
 @return newly allocated EditorInfo structure pointer.
*/
static EditorInfo *ui_create_editor_info()
{
	EditorInfo *editorInfo = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	editorInfo = g_new0(EditorInfo, 1);
	editorInfo->menu = g_new0(EditorMenu, 1);
	editorInfo->msgFields = g_new0(MsgFields, 1);
	editorInfo->toolbar = g_new0(EditorToolbar, 1);
	editorInfo->msgInfo = g_new0(MsgInfo, 1);
	editorInfo->autosaved = FALSE;
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return editorInfo;
}

/*get the font scaling factor.
 @return void
*/
static void get_font_scaling_factor()
{
	GConfClient *client = NULL;
	GConfSchema *schema = NULL;
	GError *error = NULL;
	GConfValue *gvalue = NULL;

	client = gconf_client_get_default();
	schema = gconf_client_get_schema(client, SETTINGS_KEY_FONT_SCALING_FACTOR, &error);
	if (error == NULL) {
		gvalue = gconf_schema_get_default_value(schema);
		if (gvalue->type == GCONF_VALUE_FLOAT) {
			font_scaling_factor = gconf_value_get_float(gvalue);
		}
		gconf_value_free(gvalue);
	} else {
		font_scaling_factor = 1.5;
	}
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	g_object_unref(client);
	return;
}

/**
 This function initialises the editorinfo
 
 @param editor_info is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
 @param msgHeader is a  pointer to the HeaderInfo structure.
	 This paramter is not freed in this function.
 @param mail is the gint representing the mode in which editor is opened.
 
 @return True-successful memory allocation & initilaisation.
	 False-else.
*/
static gboolean
ui_editor_initialise_editorinfo(EditorInfo * editor_info, const HeaderInfo * msgHeader, gint mail)
{
	gboolean initialised = TRUE;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (editor_info != NULL && msgHeader != NULL) {
		editor_info->folderNum = msgHeader->folderNum;
		if (msgHeader->msgid != NULL) {
			editor_info->msgid = g_strdup(msgHeader->msgid);
		}
	}
	editor_info->editorModified = FALSE;
	if (mail == FAILED_MAIL) {
		mail = OUTBOX_MAIL;
	}
	editor_info->composeType = mail;
	editor_info->emailType = HTML_EDITOR;
	editor_info->toolbarhidden_normalview = ui_get_editor_toolbar_hidden_status();
	editor_info->toolbarhidden_fullscreen = ui_get_editor_fullscreen_toolbar_hidden_status();
	initialised = ui_editor_initialise_undoinfo(editor_info);
	if (initialised == TRUE) {
		ui_editor_initialise_mailsettings(editor_info, msgHeader, mail);
	}
	/*get the font scaling factor */
	get_font_scaling_factor();
	editor_info->insert_mark = NULL;

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return initialised;
}


/**
 This function is used to show different infonote for paste button
  is clicked when insensitive.
  
 @param insensitive_widget is the GtkWidget for which the insensitive_press
  signal is fired.
  
 @return void.
*/
static void paste_dimming_note(GtkWidget * paste_menu)
{
	GtkWidget *view = NULL;

	view = gtk_window_get_focus((GtkWindow *) app);
	if (GTK_IS_TREE_VIEW(view)) {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_unable_to_paste"));
	} else {
		gtk_infoprint((GtkWindow *) app, _("mcen_ib_nothing_to_paste"));
	}
	return;
}

/**
 This function is used to check/uncheck the fullscreen menu item.
  
 @param win is the Gtkwidget pointer to the view menu.
 @param editor_info is the EditorInfo structure  pointer of the current editor.
  
 @return void.
*/
static void ui_view_menu_activate_cb(GtkWidget * win, EditorInfo * editor_info)
{
	gboolean full_screen_state = TRUE;

	full_screen_state = email_main_view->tags->full_screen_mode;
	g_signal_handlers_block_by_func(G_OBJECT(editor_info->menu->fullscreen),
					(gpointer) ui_editor_toggle_to_fullscreen, editor_info);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
				       (editor_info->menu->fullscreen), full_screen_state);
	g_signal_handlers_unblock_by_func(G_OBJECT(editor_info->menu->fullscreen),
					  (gpointer) ui_editor_toggle_to_fullscreen, editor_info);
}

/*Function to handle the removal of attachment through hardward.
 * backspace/delete keys.
 @param attachment_view is the attachment_view widget.
 @param event is the pointer to the GdkEvent union.
 @param editor_info is the pointer to the EditorInfo staructure pointer.
 @return FALSE to propagate the event furture.
**/

static gboolean
ui_key_press_on_attachment(GtkWidget * attachment_view, GdkEventKey * event,
			   EditorInfo * editor_info)
{
	if (event->keyval == GDK_BackSpace || event->keyval == GDK_Delete)
		ui_editor_remove_attachment_cb(attachment_view, editor_info);

	return FALSE;
}


/**
 Creates the editor menu
 
 @param editor_info is the EditorInfo structure pointer.
	 This paramter is not freed in this function.
 
 @return void.
*/
static void ui_editor_create_menubar(EditorInfo * editor_info)
{
	GtkWidget *menubar = NULL;
	GtkWidget *message_menuitem = NULL;
	GtkWidget *message_menu = NULL;
	GtkWidget *msg_separator = NULL;

	GtkWidget *edit_menuitem = NULL;
	GtkWidget *edit_menu = NULL;
	GtkWidget *undo_separator = NULL;
	GtkWidget *separator = NULL;
	GtkWidget *newmail_menu = NULL;
	GtkWidget *attachments_menuitem = NULL;
	GtkWidget *attachments_menu = NULL;

	GtkWidget *view_menuitem = NULL;
	GtkWidget *view_menu = NULL;
	GtkWidget *show_toolbar = NULL;
	GtkWidget *show_toolbar_menu = NULL;
	GtkWidget *zoom = NULL;
	GtkWidget *zoom_menu = NULL;
	GSList *zoomlevels = NULL;
	GtkWidget *allignment_menuitem = NULL;
	GtkWidget *allignment_menu = NULL;

	GtkWidget *format_menu = NULL;
	GSList *left_group = NULL;
	GtkWidget *tools_menuitem = NULL;
	GtkWidget *tools_menu = NULL;
	GtkWidget *sending_options_menuitem = NULL;
	GtkWidget *help_menuitem = NULL;
#if SMIME
	GSList *group = NULL;
	GtkWidget *encryption_menuitem = NULL;
	GtkWidget *encryption_menu = NULL;
#endif
	GtkWidget *close_menu = NULL;
	GtkWidget *close = NULL;
	GtkWidget *close_window = NULL;
	EditorMenu *menu = NULL;
	gchar zoomstr[BUFFER_SIZE];

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);
	CHECK_PARAM(editor_info->menu == NULL, "Input param failed\n", return);
	menu = editor_info->menu;

	/*Creating the menu bar & menuitems for editor. */
	menubar =
	    (GtkWidget *) hildon_appview_get_menu((HildonAppView *) editor_info->editor_appview);
	message_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_message"));
	gtk_container_add(GTK_CONTAINER(menubar), message_menuitem);

	message_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(message_menuitem), message_menu);

#if RETAINED_EMAIL
	/*Send menu */
	menu->send = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_send"));
	gtk_widget_show(menu->send);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->send);
	g_signal_connect(G_OBJECT(menu->send), "activate",
			 G_CALLBACK(ui_editor_send_msg_cb), editor_info);

	msg_separator = gtk_menu_item_new();
	gtk_widget_show(msg_separator);
	gtk_container_add(GTK_CONTAINER(message_menu), msg_separator);
	gtk_widget_set_sensitive(msg_separator, FALSE);

	/*Newmail menu */
	newmail_menu = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_newemail"));
	gtk_widget_show(newmail_menu);
	gtk_container_add(GTK_CONTAINER(message_menu), newmail_menu);
	g_signal_connect(G_OBJECT(newmail_menu), "activate",
			 G_CALLBACK(ui_on_new_email_activate), NULL);

	msg_separator = gtk_menu_item_new();
	gtk_widget_show(msg_separator);
	gtk_container_add(GTK_CONTAINER(message_menu), msg_separator);
	gtk_widget_set_sensitive(msg_separator, FALSE);

	/*Delete menu */
	menu->delete_msg = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_deletemessage"));
	gtk_widget_show(menu->delete_msg);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->delete_msg);
	g_signal_connect(G_OBJECT(menu->delete_msg), "activate",
			 G_CALLBACK(ui_editor_delete_message_cb), editor_info);


	msg_separator = gtk_menu_item_new();
	gtk_widget_show(msg_separator);
	gtk_container_add(GTK_CONTAINER(message_menu), msg_separator);
	gtk_widget_set_sensitive(msg_separator, FALSE);

	/*Select recipients menu */
	menu->select_recipients =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_selectrecipients"));
	gtk_widget_show(menu->select_recipients);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->select_recipients);
	g_signal_connect(G_OBJECT(menu->select_recipients), "activate",
			 G_CALLBACK(ui_editor_select_recipients_cb), editor_info->msgFields);

	/*Check names menu */
	menu->check_names = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_checknames"));
	gtk_widget_show(menu->check_names);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->check_names);
	gtk_widget_set_sensitive(menu->check_names, TRUE);


	edit_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_edit"));
	gtk_container_add((GtkContainer *) menubar, edit_menuitem);
	g_signal_connect_after(G_OBJECT(edit_menuitem), "activate",
			       G_CALLBACK(handle_ccp_menus), editor_info);

	edit_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit_menuitem), edit_menu);

	/*Undo menu */
	menu->undo = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_undo"));
	gtk_widget_show(menu->undo);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->undo);
	gtk_widget_set_sensitive(menu->undo, FALSE);
	g_signal_connect(G_OBJECT(menu->undo), "insensitive_press",
			 G_CALLBACK(show_dimming_note), DGETTEXT_HCS("ecoc_ib_nothing_to_undo"));

	undo_separator = gtk_menu_item_new();
	gtk_widget_show(undo_separator);
	gtk_container_add(GTK_CONTAINER(edit_menu), undo_separator);
	gtk_widget_set_sensitive(undo_separator, FALSE);

	/*Cut menu */
	menu->cut = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_cut"));
	gtk_widget_show(menu->cut);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->cut);
	g_signal_connect(G_OBJECT(menu->cut), "activate",
			 G_CALLBACK(ui_msg_cut_cb), editor_info->msgFields);
	g_signal_connect(G_OBJECT(menu->cut), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_unable_to_cut"));

	/*Copy menu */
	menu->copy = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_copy"));
	gtk_widget_show(menu->copy);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->copy);
	g_signal_connect(G_OBJECT(menu->copy), "activate",
			 G_CALLBACK(ui_msg_copy_cb), editor_info->msgFields);
	g_signal_connect(G_OBJECT(menu->copy), "insensitive_press",
			 G_CALLBACK(ui_msg_copy_cb), editor_info->msgFields);

	/*paste menu */
	menu->paste = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_paste"));
	gtk_widget_show(menu->paste);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->paste);
	gtk_widget_set_sensitive(menu->paste, FALSE);
	g_signal_connect(G_OBJECT(menu->paste), "activate", G_CALLBACK(ui_msg_paste_cb),
			 editor_info);
	g_signal_connect(G_OBJECT(menu->paste), "insensitive_press", G_CALLBACK(paste_dimming_note),
			 NULL);

	/*Select all menu */
	menu->select_all = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_selectall"));
	gtk_widget_show(menu->select_all);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->select_all);
	g_signal_connect(G_OBJECT(menu->select_all), "activate",
			 G_CALLBACK(ui_msg_select_all_cb), editor_info->msgFields);
	g_signal_connect(G_OBJECT(menu->select_all), "activate",
			 G_CALLBACK(call_attribute_refresh_func), editor_info);

	/*Attachment menuitem */
	attachments_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_attachments"));
	gtk_container_add((GtkContainer *) menubar, attachments_menuitem);
	g_signal_connect(G_OBJECT(attachments_menuitem), "activate",
			 G_CALLBACK(update_attach_menu_status), editor_info);

	attachments_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(attachments_menuitem), attachments_menu);
	/*attachfile menu */
	menu->attach_file = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_attachfile"));
	gtk_widget_show(menu->attach_file);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->attach_file);
	g_signal_connect(G_OBJECT(menu->attach_file), "activate",
			 G_CALLBACK(ui_editor_attach_file_cb), editor_info);
	g_signal_connect(G_OBJECT(menu->attach_file), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_move_cursor_to_message"));

	/*attach image menu */
	menu->attach_image =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_attach_inlineimage"));
	gtk_widget_show(menu->attach_image);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->attach_image);
	g_signal_connect(G_OBJECT(menu->attach_image), "activate",
			 G_CALLBACK(ui_editor_attach_image_cb), editor_info);
	gtk_widget_set_sensitive(GTK_WIDGET(menu->attach_image), FALSE);
	g_signal_connect(G_OBJECT(menu->attach_image), "insensitive_press",
			 G_CALLBACK(format_dimming_note), editor_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(attachments_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*remove attachmnet menu */
	menu->remove_attachment = gtk_menu_item_new_with_mnemonic
	    (_("mcen_me_editor_remove_attachments"));
	gtk_widget_show(menu->remove_attachment);
	gtk_widget_set_sensitive(GTK_WIDGET(menu->remove_attachment), FALSE);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->remove_attachment);
	g_signal_connect(G_OBJECT(menu->remove_attachment), "activate",
			 G_CALLBACK(ui_editor_remove_attachment_cb), editor_info);
	g_signal_connect(G_OBJECT(menu->remove_attachment), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_no_attachment_selected"));

	view_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_view"));
	gtk_container_add((GtkContainer *) menubar, view_menuitem);

	view_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(view_menuitem), view_menu);
	g_signal_connect(G_OBJECT(view_menuitem), "activate",
			 G_CALLBACK(ui_view_menu_activate_cb), editor_info);

	/*Zoom menuitem */
	zoom = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_zoom"));
	gtk_widget_set_sensitive(zoom, TRUE);
	gtk_widget_show(zoom);
	gtk_container_add(GTK_CONTAINER(view_menu), zoom);
	zoom_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(zoom), zoom_menu);

	/*50% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_50"));
	menu->zoom_in_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_in_level));
	gtk_widget_show(menu->zoom_in_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_in_level);
	memset(zoomstr, 0, BUFFER_SIZE);
#if 0
	/*80% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_80"));
	menu->zoom_80 = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_80));
	gtk_widget_show(menu->zoom_80);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_80);
	memset(zoomstr, 0, BUFFER_SIZE);
#endif

	/*100% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_100"));
	menu->current_zoom_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->current_zoom_level));
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu->current_zoom_level), TRUE);
	gtk_widget_show(menu->current_zoom_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->current_zoom_level);
	memset(zoomstr, 0, BUFFER_SIZE);

#if 0
	/*120% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_120"));
	menu->zoom_120 = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_120));
	gtk_widget_show(menu->zoom_120);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_120);
	memset(zoomstr, 0, BUFFER_SIZE);
#endif

	/*150% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_150"));
	menu->zoom_out_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_out_level));
	gtk_widget_show(menu->zoom_out_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_out_level);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*200% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_200"));
	menu->zoom_out_max_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_out_max_level));
	gtk_widget_show(menu->zoom_out_max_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_out_max_level);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*show toolbar menu */
	show_toolbar = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_toolbar"));
	gtk_widget_show(show_toolbar);
	gtk_container_add(GTK_CONTAINER(view_menu), show_toolbar);

	show_toolbar_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(show_toolbar), show_toolbar_menu);

	/*normal view menu */
	menu->toolbar_normalview =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_normalview"));
	gtk_widget_show(menu->toolbar_normalview);
	gtk_container_add(GTK_CONTAINER(show_toolbar_menu), menu->toolbar_normalview);
	g_signal_connect(G_OBJECT(menu->toolbar_normalview), "activate",
			 G_CALLBACK(ui_toggle_editor_toolbar_cb), editor_info);

	/*full screen view menu */
	menu->toolbar_fullscreen =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_optimizedview"));
	gtk_widget_show(menu->toolbar_fullscreen);
	gtk_container_add(GTK_CONTAINER(show_toolbar_menu), menu->toolbar_fullscreen);
	g_signal_connect(G_OBJECT(menu->toolbar_fullscreen), "activate",
			 G_CALLBACK(ui_toggle_editor_fullscreen_toolbar_cb), editor_info);

	/*full screen menu */
	menu->fullscreen = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_fullscreen"));
	gtk_widget_show(menu->fullscreen);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->fullscreen);

	g_signal_connect(G_OBJECT(menu->fullscreen), "activate",
			 G_CALLBACK(ui_editor_toggle_to_fullscreen), editor_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(view_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*show cc menu */
	menu->show_cc = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_showcc"));
	gtk_widget_show(menu->show_cc);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->show_cc);
	g_signal_connect(G_OBJECT(menu->show_cc), "activate",
			 G_CALLBACK(ui_msg_showCc_cb), editor_info);
	/*show bcc menu */
	menu->show_bcc = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_showbcc"));
	gtk_widget_show(menu->show_bcc);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->show_bcc);
	g_signal_connect(G_OBJECT(menu->show_bcc), "activate",
			 G_CALLBACK(ui_msg_showBcc_cb), editor_info);

	/*format menu */
	menu->format_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_format"));
	gtk_container_add((GtkContainer *) menubar, menu->format_menuitem);
	gtk_widget_set_sensitive(menu->format_menuitem, FALSE);

	format_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu->format_menuitem), format_menu);
	g_signal_connect(G_OBJECT(menu->format_menuitem), "insensitive_press",
			 G_CALLBACK(format_dimming_note), editor_info);
	/*font menu */
	menu->font = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_font"));
	gtk_widget_show(menu->font);
	gtk_container_add(GTK_CONTAINER(format_menu), menu->font);

	/*bullents menu */
	menu->bullets = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_bullets"));
	gtk_widget_show(menu->bullets);
	gtk_container_add(GTK_CONTAINER(format_menu), menu->bullets);
	g_signal_connect(G_OBJECT(menu->bullets), "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

	/*numbering menu */
	menu->numbering = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_numbers"));
	gtk_widget_show(menu->numbering);
	gtk_container_add(GTK_CONTAINER(format_menu), menu->numbering);
	g_signal_connect(G_OBJECT(menu->numbering), "activate",
			 G_CALLBACK(on_format_button_click), editor_info);
	/*alingment menu */
	allignment_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_align"));
	gtk_widget_show(allignment_menuitem);
	gtk_container_add(GTK_CONTAINER(format_menu), allignment_menuitem);

	allignment_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(allignment_menuitem), allignment_menu);
/*left alingment menu*/
	menu->left = gtk_radio_menu_item_new_with_mnemonic(NULL, _("mcen_me_editor_align_left"));
	left_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->left));
	gtk_widget_show(menu->left);
	gtk_container_add(GTK_CONTAINER(allignment_menu), menu->left);
	g_signal_connect((gpointer) menu->left, "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

/*center alingment menu*/
	menu->centred = gtk_radio_menu_item_new_with_mnemonic(left_group,
							      _("mcen_me_editor_align_centred"));
	gtk_widget_show(menu->centred);
	gtk_container_add(GTK_CONTAINER(allignment_menu), menu->centred);
	left_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->centred));
	g_signal_connect((gpointer) menu->centred, "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

/*right alingment menu*/
	menu->right = gtk_radio_menu_item_new_with_mnemonic(left_group,
							    _("mcen_me_editor_align_right"));
	gtk_widget_show(menu->right);
	left_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->right));
	gtk_container_add(GTK_CONTAINER(allignment_menu), menu->right);
	g_signal_connect((gpointer) menu->right, "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

	menu->none = gtk_radio_menu_item_new_with_mnemonic(left_group, "");
	left_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->none));
	gtk_container_add(GTK_CONTAINER(allignment_menu), menu->none);
	gtk_widget_hide(menu->none);

	/*bg color menu */
	menu->bgcolor = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_bgcolor"));
	gtk_widget_show(menu->bgcolor);
	gtk_container_add(GTK_CONTAINER(format_menu), menu->bgcolor);
	g_signal_connect(G_OBJECT(menu->bgcolor), "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

	tools_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_tools"));
	gtk_container_add((GtkContainer *) menubar, tools_menuitem);

	tools_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(tools_menuitem), tools_menu);

	/*sending opt menu */
	sending_options_menuitem =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_send_options"));
	gtk_widget_show(sending_options_menuitem);
	gtk_container_add(GTK_CONTAINER(tools_menu), sending_options_menuitem);
	g_signal_connect(G_OBJECT(sending_options_menuitem), "activate",
			 G_CALLBACK(ui_editor_sendingOption_cb), editor_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*find menu */
	menu->find = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_find"));
	gtk_widget_show(menu->find);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->find);
	g_signal_connect(G_OBJECT(menu->find), "activate",
			 G_CALLBACK(show_find_toolbar_cb), editor_info->msgFields);

	/*search emails menu */
	menu->searchemails = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_searchemails"));
	gtk_widget_show(menu->searchemails);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->searchemails);
	g_signal_connect((gpointer) menu->searchemails, "activate",
			 G_CALLBACK(ui_viewer_searchemails_cb), NULL);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*help menu */
	help_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_help"));
	gtk_widget_show(help_menuitem);
	gtk_container_add(GTK_CONTAINER(tools_menu), help_menuitem);
	g_signal_connect(help_menuitem, "activate",
			 G_CALLBACK(ui_on_view_help_activate), HELP_ID_EDITOR);

#if SMIME
	encryption_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_security"));
	gtk_widget_show(encryption_menuitem);
	gtk_container_add(GTK_CONTAINER(menubar), encryption_menuitem);

	encryption_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(encryption_menuitem), encryption_menu);
	/*no encryption menu */
	menu->no_encrypt = gtk_radio_menu_item_new_with_mnemonic(NULL, _("mcen_me_editor_none"));
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->no_encrypt));
	gtk_widget_show(menu->no_encrypt);
	gtk_container_add(GTK_CONTAINER(encryption_menu), menu->no_encrypt);
	g_signal_connect((gpointer) menu->no_encrypt, "activate",
			 G_CALLBACK(ui_editor_set_encryptmail_cb), editor_info);

	/* encryption menu */
	menu->encrypt = gtk_radio_menu_item_new_with_mnemonic(group, _("mcen_me_editor_encrypt"));
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->encrypt));
	gtk_widget_show(menu->encrypt);
	gtk_container_add(GTK_CONTAINER(encryption_menu), menu->encrypt);
	g_signal_connect((gpointer) menu->encrypt, "activate",
			 G_CALLBACK(ui_editor_set_encryptmail_cb), editor_info);
	/*sign  menu */
	menu->sign = gtk_radio_menu_item_new_with_mnemonic(group, _("mcen_me_editor_sign"));
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->sign));
	gtk_widget_show(menu->sign);
	gtk_container_add(GTK_CONTAINER(encryption_menu), menu->sign);
	g_signal_connect((gpointer) menu->sign, "activate",
			 G_CALLBACK(ui_editor_set_encryptmail_cb), editor_info);

	/*sign & encryption menu */
	menu->sign_encrypt =
	    gtk_radio_menu_item_new_with_mnemonic(group, _("mcen_me_editor_encrypt_sign"));
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->sign_encrypt));
	gtk_widget_show(menu->sign_encrypt);

	gtk_container_add(GTK_CONTAINER(encryption_menu), menu->sign_encrypt);
	g_signal_connect((gpointer) menu->sign_encrypt, "activate",
			 G_CALLBACK(ui_editor_set_encryptmail_cb), editor_info);
#endif
#else
	/*Send menu */
	menu->send = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_send"));
	gtk_widget_show(menu->send);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->send);
	g_signal_connect(G_OBJECT(menu->send), "activate",
			 G_CALLBACK(ui_editor_send_msg_cb), editor_info);

	msg_separator = gtk_menu_item_new();
	gtk_widget_show(msg_separator);
	gtk_container_add(GTK_CONTAINER(message_menu), msg_separator);
	gtk_widget_set_sensitive(msg_separator, FALSE);

	/*Select recipients menu */
	menu->select_recipients =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_selectrecipients"));
	gtk_widget_show(menu->select_recipients);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->select_recipients);
	g_signal_connect(G_OBJECT(menu->select_recipients), "activate",
			 G_CALLBACK(ui_editor_select_recipients_cb), editor_info->msgFields);

	/*Check names menu */
	menu->check_names = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_checknames"));
	gtk_widget_show(menu->check_names);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->check_names);
	gtk_widget_set_sensitive(menu->check_names, TRUE);

	msg_separator = gtk_menu_item_new();
	gtk_widget_show(msg_separator);
	gtk_container_add(GTK_CONTAINER(message_menu), msg_separator);
	gtk_widget_set_sensitive(msg_separator, FALSE);

	/*Newmail menu */
	newmail_menu = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_newemail"));
	gtk_widget_show(newmail_menu);
	gtk_container_add(GTK_CONTAINER(message_menu), newmail_menu);
	g_signal_connect(G_OBJECT(newmail_menu), "activate",
			 G_CALLBACK(ui_on_new_email_activate), NULL);

	msg_separator = gtk_menu_item_new();
	gtk_widget_show(msg_separator);
	gtk_container_add(GTK_CONTAINER(message_menu), msg_separator);
	gtk_widget_set_sensitive(msg_separator, FALSE);

	/*Delete menu */
	menu->delete_msg = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_deletemessage"));
	gtk_widget_show(menu->delete_msg);
	gtk_container_add(GTK_CONTAINER(message_menu), menu->delete_msg);
	g_signal_connect(G_OBJECT(menu->delete_msg), "activate",
			 G_CALLBACK(ui_editor_delete_message_cb), editor_info);

	edit_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_edit"));
	gtk_container_add((GtkContainer *) menubar, edit_menuitem);
	g_signal_connect_after(G_OBJECT(edit_menuitem), "activate",
			       G_CALLBACK(handle_ccp_menus), editor_info);

	edit_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit_menuitem), edit_menu);

	/*Undo menu */
	menu->undo = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_undo"));
	gtk_widget_show(menu->undo);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->undo);
	gtk_widget_set_sensitive(menu->undo, FALSE);
	g_signal_connect(G_OBJECT(menu->undo), "insensitive_press",
			 G_CALLBACK(show_dimming_note), DGETTEXT_HCS("ecoc_ib_nothing_to_undo"));

	undo_separator = gtk_menu_item_new();
	gtk_widget_show(undo_separator);
	gtk_container_add(GTK_CONTAINER(edit_menu), undo_separator);
	gtk_widget_set_sensitive(undo_separator, FALSE);

	/*Cut menu */
	menu->cut = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_cut"));
	gtk_widget_show(menu->cut);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->cut);
	g_signal_connect(G_OBJECT(menu->cut), "activate",
			 G_CALLBACK(ui_msg_cut_cb), editor_info->msgFields);
	g_signal_connect(G_OBJECT(menu->cut), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_unable_to_cut"));

	/*Copy menu */
	menu->copy = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_copy"));
	gtk_widget_show(menu->copy);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->copy);
	g_signal_connect(G_OBJECT(menu->copy), "activate",
			 G_CALLBACK(ui_msg_copy_cb), editor_info->msgFields);
	g_signal_connect(G_OBJECT(menu->copy), "insensitive_press",
			 G_CALLBACK(ui_msg_copy_cb), editor_info->msgFields);

	/*paste menu */
	menu->paste = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_paste"));
	gtk_widget_show(menu->paste);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->paste);
	gtk_widget_set_sensitive(menu->paste, FALSE);
	g_signal_connect(G_OBJECT(menu->paste), "activate", G_CALLBACK(ui_msg_paste_cb),
			 editor_info);
	g_signal_connect(G_OBJECT(menu->paste), "insensitive_press", G_CALLBACK(paste_dimming_note),
			 NULL);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(edit_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*Select all menu */
	menu->select_all = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_selectall"));
	gtk_widget_show(menu->select_all);
	gtk_container_add(GTK_CONTAINER(edit_menu), menu->select_all);
	g_signal_connect(G_OBJECT(menu->select_all), "activate",
			 G_CALLBACK(ui_msg_select_all_cb), editor_info->msgFields);
	g_signal_connect(G_OBJECT(menu->select_all), "activate",
			 G_CALLBACK(call_attribute_refresh_func), editor_info);

	/*view menu items */
	view_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_view"));
	gtk_container_add((GtkContainer *) menubar, view_menuitem);

	view_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(view_menuitem), view_menu);
	g_signal_connect(G_OBJECT(view_menuitem), "activate",
			 G_CALLBACK(ui_view_menu_activate_cb), editor_info);

	/*Zoom menuitem */
	zoom = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_zoom"));
	gtk_widget_set_sensitive(zoom, TRUE);
	gtk_widget_show(zoom);
	gtk_container_add(GTK_CONTAINER(view_menu), zoom);
	zoom_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(zoom), zoom_menu);

	/*50% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_50"));
	menu->zoom_in_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_in_level));
	gtk_widget_show(menu->zoom_in_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_in_level);
	memset(zoomstr, 0, BUFFER_SIZE);

#if 0
	/*80% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_80"));
	menu->zoom_80 = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_80));
	gtk_widget_show(menu->zoom_80);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_80);
	memset(zoomstr, 0, BUFFER_SIZE);
#endif

	/*100% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_100"));
	menu->current_zoom_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->current_zoom_level));
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu->current_zoom_level), TRUE);
	gtk_widget_show(menu->current_zoom_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->current_zoom_level);
	memset(zoomstr, 0, BUFFER_SIZE);

#if 0
	/*120% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_120"));
	menu->zoom_120 = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_120));
	gtk_widget_show(menu->zoom_120);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_120);
	memset(zoomstr, 0, BUFFER_SIZE);
#endif

	/*150% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_150"));
	menu->zoom_out_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_out_level));
	gtk_widget_show(menu->zoom_out_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_out_level);
	memset(zoomstr, 0, BUFFER_SIZE);

	/*200% zoom menu */
	sprintf(zoomstr, _("mail_me_editor_200"));
	menu->zoom_out_max_level = gtk_radio_menu_item_new_with_label(zoomlevels, zoomstr);
	zoomlevels = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->zoom_out_max_level));
	gtk_widget_show(menu->zoom_out_max_level);
	gtk_container_add(GTK_CONTAINER(zoom_menu), menu->zoom_out_max_level);
	memset(zoomstr, 0, BUFFER_SIZE);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(view_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*full screen menu */
	menu->fullscreen = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_fullscreen"));
	gtk_widget_show(menu->fullscreen);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->fullscreen);

	g_signal_connect(G_OBJECT(menu->fullscreen), "activate",
			 G_CALLBACK(ui_editor_toggle_to_fullscreen), editor_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(view_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*show toolbar menu */
	show_toolbar = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_toolbar"));
	gtk_widget_show(show_toolbar);
	gtk_container_add(GTK_CONTAINER(view_menu), show_toolbar);

	show_toolbar_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(show_toolbar), show_toolbar_menu);

	/*normal view menu */
	menu->toolbar_normalview =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_normalview"));
	gtk_widget_show(menu->toolbar_normalview);
	gtk_container_add(GTK_CONTAINER(show_toolbar_menu), menu->toolbar_normalview);
	g_signal_connect(G_OBJECT(menu->toolbar_normalview), "activate",
			 G_CALLBACK(ui_toggle_editor_toolbar_cb), editor_info);

	/*full screen(show toolbar) view menu */
	menu->toolbar_fullscreen =
	    gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_optimizedview"));
	gtk_widget_show(menu->toolbar_fullscreen);
	gtk_container_add(GTK_CONTAINER(show_toolbar_menu), menu->toolbar_fullscreen);
	g_signal_connect(G_OBJECT(menu->toolbar_fullscreen), "activate",
			 G_CALLBACK(ui_toggle_editor_fullscreen_toolbar_cb), editor_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(view_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*show cc menu */
	menu->show_cc = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_showcc"));
	gtk_widget_show(menu->show_cc);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->show_cc);
	g_signal_connect(G_OBJECT(menu->show_cc), "activate",
			 G_CALLBACK(ui_msg_showCc_cb), editor_info);
	/*show bcc menu */
	menu->show_bcc = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_showbcc"));
	gtk_widget_show(menu->show_bcc);
	gtk_container_add(GTK_CONTAINER(view_menu), menu->show_bcc);
	g_signal_connect(G_OBJECT(menu->show_bcc), "activate",
			 G_CALLBACK(ui_msg_showBcc_cb), editor_info);

	/*format menu */
	menu->format_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_format"));
	gtk_container_add((GtkContainer *) menubar, menu->format_menuitem);
	gtk_widget_set_sensitive(menu->format_menuitem, FALSE);

	format_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu->format_menuitem), format_menu);
	g_signal_connect(G_OBJECT(menu->format_menuitem), "insensitive_press",
			 G_CALLBACK(format_dimming_note), editor_info);
	/*font menu */
	menu->font = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_font"));
	gtk_widget_show(menu->font);
	gtk_container_add(GTK_CONTAINER(format_menu), menu->font);

	/*alingment menu */
	allignment_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_align"));
	gtk_widget_show(allignment_menuitem);
	gtk_container_add(GTK_CONTAINER(format_menu), allignment_menuitem);

	allignment_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(allignment_menuitem), allignment_menu);
/*left alingment menu*/
	menu->left = gtk_radio_menu_item_new_with_mnemonic(NULL, _("mcen_me_editor_align_left"));
	left_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->left));
	gtk_widget_show(menu->left);
	gtk_container_add(GTK_CONTAINER(allignment_menu), menu->left);
	g_signal_connect((gpointer) menu->left, "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

/*center alingment menu*/
	menu->centred = gtk_radio_menu_item_new_with_mnemonic(left_group,
							      _("mcen_me_editor_align_centred"));
	gtk_widget_show(menu->centred);
	gtk_container_add(GTK_CONTAINER(allignment_menu), menu->centred);
	left_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->centred));
	g_signal_connect((gpointer) menu->centred, "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

/*right alingment menu*/
	menu->right = gtk_radio_menu_item_new_with_mnemonic(left_group,
							    _("mcen_me_editor_align_right"));
	gtk_widget_show(menu->right);
	left_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->right));
	gtk_container_add(GTK_CONTAINER(allignment_menu), menu->right);
	g_signal_connect((gpointer) menu->right, "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

	menu->none = gtk_radio_menu_item_new_with_mnemonic(left_group, "");
	left_group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->none));
	gtk_container_add(GTK_CONTAINER(allignment_menu), menu->none);
	gtk_widget_hide(menu->none);

	/*bullents menu */
	menu->bullets = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_bullets"));
	gtk_widget_show(menu->bullets);
	gtk_container_add(GTK_CONTAINER(format_menu), menu->bullets);
	g_signal_connect(G_OBJECT(menu->bullets), "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

	/*numbering menu */
	menu->numbering = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_numbers"));
	gtk_widget_show(menu->numbering);
	gtk_container_add(GTK_CONTAINER(format_menu), menu->numbering);
	g_signal_connect(G_OBJECT(menu->numbering), "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(format_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*bg color menu */
	menu->bgcolor = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_bgcolor"));
	gtk_widget_show(menu->bgcolor);
	gtk_container_add(GTK_CONTAINER(format_menu), menu->bgcolor);
	g_signal_connect(G_OBJECT(menu->bgcolor), "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

	/*Attachment menuitem */
	attachments_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_attachments"));
	gtk_container_add((GtkContainer *) menubar, attachments_menuitem);
	g_signal_connect(G_OBJECT(attachments_menuitem), "activate",
			 G_CALLBACK(update_attach_menu_status), editor_info);

	attachments_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(attachments_menuitem), attachments_menu);
	/*attachfile menu */
	menu->attach_file = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_attachfile"));
	gtk_widget_show(menu->attach_file);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->attach_file);
	g_signal_connect(G_OBJECT(menu->attach_file), "activate",
			 G_CALLBACK(ui_editor_attach_file_cb), editor_info);
	g_signal_connect(G_OBJECT(menu->attach_file), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_move_cursor_to_message"));

	/*attach image menu */
	menu->attach_image =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_attach_inlineimage"));
	gtk_widget_show(menu->attach_image);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->attach_image);
	g_signal_connect(G_OBJECT(menu->attach_image), "activate",
			 G_CALLBACK(ui_editor_attach_image_cb), editor_info);
	gtk_widget_set_sensitive(GTK_WIDGET(menu->attach_image), FALSE);
	g_signal_connect(G_OBJECT(menu->attach_image), "insensitive_press",
			 G_CALLBACK(format_dimming_note), editor_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(attachments_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*remove attachmnet menu */
	menu->remove_attachment = gtk_menu_item_new_with_mnemonic
	    (_("mcen_me_editor_remove_attachments"));
	gtk_widget_show(menu->remove_attachment);
	gtk_widget_set_sensitive(GTK_WIDGET(menu->remove_attachment), FALSE);
	gtk_container_add(GTK_CONTAINER(attachments_menu), menu->remove_attachment);
	g_signal_connect(G_OBJECT(menu->remove_attachment), "activate",
			 G_CALLBACK(ui_editor_remove_attachment_cb), editor_info);
	g_signal_connect(G_OBJECT(menu->remove_attachment), "insensitive_press",
			 G_CALLBACK(show_dimming_note), _("mcen_ib_no_attachment_selected"));

	/*tools menu items */
	tools_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_tools"));
	gtk_container_add((GtkContainer *) menubar, tools_menuitem);

	tools_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(tools_menuitem), tools_menu);

	/*sending opt menu */
	sending_options_menuitem =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_send_options"));
	gtk_widget_show(sending_options_menuitem);
	gtk_container_add(GTK_CONTAINER(tools_menu), sending_options_menuitem);
	g_signal_connect(G_OBJECT(sending_options_menuitem), "activate",
			 G_CALLBACK(ui_editor_sendingOption_cb), editor_info);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

#if SMIME
	encryption_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_security"));
	gtk_widget_show(encryption_menuitem);
	gtk_container_add(GTK_CONTAINER(tools_menu), encryption_menuitem);

	encryption_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(encryption_menuitem), encryption_menu);
	/*no encryption menu */
	menu->no_encrypt = gtk_radio_menu_item_new_with_mnemonic(NULL, _("mcen_me_editor_none"));
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->no_encrypt));
	gtk_widget_show(menu->no_encrypt);
	gtk_container_add(GTK_CONTAINER(encryption_menu), menu->no_encrypt);
	g_signal_connect((gpointer) menu->no_encrypt, "activate",
			 G_CALLBACK(ui_editor_set_encryptmail_cb), editor_info);

	/* encryption menu */
	menu->encrypt = gtk_radio_menu_item_new_with_mnemonic(group, _("mcen_me_editor_encrypt"));
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->encrypt));
	gtk_widget_show(menu->encrypt);
	gtk_container_add(GTK_CONTAINER(encryption_menu), menu->encrypt);
	g_signal_connect((gpointer) menu->encrypt, "activate",
			 G_CALLBACK(ui_editor_set_encryptmail_cb), editor_info);
	/*sign  menu */
	menu->sign = gtk_radio_menu_item_new_with_mnemonic(group, _("mcen_me_editor_sign"));
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->sign));
	gtk_widget_show(menu->sign);
	gtk_container_add(GTK_CONTAINER(encryption_menu), menu->sign);
	g_signal_connect((gpointer) menu->sign, "activate",
			 G_CALLBACK(ui_editor_set_encryptmail_cb), editor_info);

	/*sign & encryption menu */
	menu->sign_encrypt =
	    gtk_radio_menu_item_new_with_mnemonic(group, _("mcen_me_editor_encrypt_sign"));
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menu->sign_encrypt));
	gtk_widget_show(menu->sign_encrypt);

	gtk_container_add(GTK_CONTAINER(encryption_menu), menu->sign_encrypt);
	g_signal_connect((gpointer) menu->sign_encrypt, "activate",
			 G_CALLBACK(ui_editor_set_encryptmail_cb), editor_info);
#endif
	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*find menu */
	menu->find = gtk_check_menu_item_new_with_mnemonic(_("mcen_me_editor_find"));
	gtk_widget_show(menu->find);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->find);
	g_signal_connect(G_OBJECT(menu->find), "activate",
			 G_CALLBACK(show_find_toolbar_cb), editor_info->msgFields);

	/*search emails menu */
	menu->searchemails = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_searchemails"));
	gtk_widget_show(menu->searchemails);
	gtk_container_add(GTK_CONTAINER(tools_menu), menu->searchemails);
	g_signal_connect((gpointer) menu->searchemails, "activate",
			 G_CALLBACK(ui_viewer_searchemails_cb), NULL);

	separator = gtk_menu_item_new();
	gtk_widget_show(separator);
	gtk_container_add(GTK_CONTAINER(tools_menu), separator);
	gtk_widget_set_sensitive(separator, FALSE);

	/*help menu */
	help_menuitem = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_help"));
	gtk_widget_show(help_menuitem);
	gtk_container_add(GTK_CONTAINER(tools_menu), help_menuitem);
	g_signal_connect(help_menuitem, "activate",
			 G_CALLBACK(ui_on_view_help_activate), HELP_ID_EDITOR);
#endif
	/*close menu */
	close = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_close"));
	gtk_container_add((GtkContainer *) menubar, close);

	close_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(close), close_menu);

	close_window = gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_close_window"));
	gtk_widget_show(close_window);
	gtk_container_add(GTK_CONTAINER(close_menu), close_window);
	g_signal_connect(G_OBJECT(close_window), "activate",
			 G_CALLBACK(ui_editor_close_cb), editor_info);
	/*close all menu */
	menu->close_all_windows =
	    gtk_menu_item_new_with_mnemonic(_("mcen_me_editor_close_windows"));
	gtk_widget_show(menu->close_all_windows);
	gtk_container_add(GTK_CONTAINER(close_menu), menu->close_all_windows);
	g_signal_connect(G_OBJECT(menu->close_all_windows), "activate",
			 G_CALLBACK(ui_on_close_all_windows_activate), NULL);
	gtk_widget_show_all((GtkWidget *) menubar);
	gtk_widget_hide(menu->none);

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 * This function is used tos et the persistent toolbar state.
 * @param editor_info is the EditorInfo structure pointer.
 * @return void.
 */
static void set_toolbar_states(EditorInfo * editor_info)
{

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (ui_get_editor_toolbar_hidden_status() == TRUE) {
		if (!email_main_view->tags->full_screen_mode) {
			ui_toggle_toolbar(FALSE, (HildonAppView *) editor_info->editor_appview,
					  (editor_info->editor_toolbar));
		}
		g_signal_handlers_block_by_func(editor_info->menu->toolbar_normalview,
						ui_toggle_editor_toolbar_cb, editor_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (editor_info->menu->toolbar_normalview), FALSE);
		g_signal_handlers_unblock_by_func(editor_info->menu->toolbar_normalview,
						  ui_toggle_editor_toolbar_cb, editor_info);
	} else {
		g_signal_handlers_block_by_func(editor_info->menu->toolbar_normalview,
						ui_toggle_editor_toolbar_cb, editor_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (editor_info->menu->toolbar_normalview), TRUE);
		g_signal_handlers_unblock_by_func(editor_info->menu->toolbar_normalview,
						  ui_toggle_editor_toolbar_cb, editor_info);
	}
	if (ui_get_editor_fullscreen_toolbar_hidden_status() == TRUE) {
		if (email_main_view->tags->full_screen_mode) {
			ui_toggle_toolbar(FALSE,
					  (HildonAppView *) editor_info->
					  editor_fullscreen_toolbar, (editor_info->editor_toolbar));
		}
		g_signal_handlers_block_by_func(editor_info->menu->toolbar_fullscreen,
						ui_toggle_editor_fullscreen_toolbar_cb,
						editor_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (editor_info->menu->toolbar_fullscreen), FALSE);
		g_signal_handlers_unblock_by_func(editor_info->menu->toolbar_fullscreen,
						  ui_toggle_editor_fullscreen_toolbar_cb,
						  editor_info);
	} else {
		g_signal_handlers_block_by_func(editor_info->menu->toolbar_fullscreen,
						ui_toggle_editor_fullscreen_toolbar_cb,
						editor_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
					       (editor_info->menu->toolbar_fullscreen), TRUE);
		g_signal_handlers_unblock_by_func(editor_info->menu->toolbar_fullscreen,
						  ui_toggle_editor_fullscreen_toolbar_cb,
						  editor_info);
	}


	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
This function is called for signal registery of components of combobox.
 @param editorinfo is the EditorInfo structure pointer.
 @return void.
 */
static void insensitive_click_cb(GtkWidget * widget, EditorInfo * editorinfo)
{
	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editorinfo == NULL, "Input param failed\n", return);

	GTK_WIDGET_UNSET_FLAGS(widget, GTK_CAN_FOCUS);
	g_signal_connect(G_OBJECT(widget), "insensitive_press",
			 G_CALLBACK(format_dimming_note), editorinfo);

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This function is used to create the toolbar.
 @param editor_info is the EditorInfo structure pointer.
 This paramter is not freed in this function.
 @return void.
 */
static void ui_editor_create_toolbar(EditorInfo * editor_info)
{

	GtkWidget *image_send = NULL;
	GtkWidget *image_recipients = NULL;
	GtkWidget *image_attach = NULL;
	GtkWidget *image_bold = NULL;
	GtkWidget *image_italic = NULL;
	GtkWidget *image_bullets = NULL;
	GtkComboBox *combobox_fontname_selector = NULL;
	GtkToolItem *item_size = NULL, *item_name = NULL;
	GtkToolbar *EditView_Toolbar = NULL;
	EditorToolbar *toolbar = NULL;
	gint i = 0;
	static const gchar *font_sizes[] = { "6", "8", "10", "12",
		"16", "24", "32"
	};

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);
	CHECK_PARAM(editor_info->toolbar == NULL, "Input param failed\n", return);

	toolbar = editor_info->toolbar;
	EditView_Toolbar = (GtkToolbar *) gtk_toolbar_new();
	gtk_box_pack_start(GTK_BOX(HILDON_APPVIEW
				   (editor_info->editor_appview)->vbox),
			   (GtkWidget *) EditView_Toolbar, FALSE, FALSE, ZERO_PADDING);
	gtk_widget_show(GTK_WIDGET(EditView_Toolbar));

	/*send tool item */
	image_send = gtk_image_new_from_icon_name("qgn_toolb_messagin_send", GTK_ICON_SIZE_BUTTON);
	toolbar->send = (GtkWidget *) gtk_tool_button_new(image_send, NULL);
	gtk_toolbar_insert(EditView_Toolbar, GTK_TOOL_ITEM(toolbar->send), -1);
	gtk_widget_show_all(toolbar->send);
	g_signal_connect(G_OBJECT(toolbar->send), "clicked",
			 G_CALLBACK(ui_editor_send_msg_cb), editor_info);

	/*attachment tool item */
	image_attach = gtk_image_new_from_icon_name("qgn_list_gene_attacpap", GTK_ICON_SIZE_BUTTON);
	toolbar->attachment = (GtkWidget *) gtk_tool_button_new(image_attach, NULL);
	gtk_widget_show(toolbar->attachment);
	gtk_toolbar_insert(EditView_Toolbar, GTK_TOOL_ITEM(toolbar->attachment), -1);
	g_signal_connect(G_OBJECT(toolbar->attachment), "clicked",
			 G_CALLBACK(ui_editor_attach_file_cb), editor_info);

	g_signal_connect(G_OBJECT
			 (gtk_bin_get_child((GtkBin *) toolbar->attachment)),
			 "insensitive_press", G_CALLBACK(show_dimming_note),
			 _("mcen_ib_move_cursor_to_message"));


	/*select recipients tool item */
	image_recipients =
	    gtk_image_new_from_icon_name("qgn_list_gene_contacts", GTK_ICON_SIZE_BUTTON);
	toolbar->recipients = (GtkWidget *) gtk_tool_button_new(image_recipients, NULL);
	gtk_widget_show(toolbar->recipients);
	gtk_toolbar_insert(EditView_Toolbar, GTK_TOOL_ITEM(toolbar->recipients), -1);
	g_signal_connect(G_OBJECT(toolbar->recipients), "clicked",
			 G_CALLBACK(ui_editor_select_recipients_cb), editor_info->msgFields);


	/*font name tool item */
	toolbar->fontname = gtk_combo_box_new_text();
	item_name = gtk_tool_item_new();
	gtk_tool_item_set_expand(GTK_TOOL_ITEM(item_name), TRUE);
	gtk_container_add((GtkContainer *) item_name, toolbar->fontname);
	gtk_toolbar_insert(EditView_Toolbar, GTK_TOOL_ITEM(item_name), -1);
	gtk_widget_show_all((GtkWidget *) item_name);

	/*to take the extra space in the toolbar the size of fontname is increase by 5 */
	gtk_widget_set_size_request(toolbar->fontname, 5 * TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request((GtkWidget *) item_name, 5 * TOOLBAR_BUTTON_WIDTH,
				    TOOLBAR_BUTTON_HEIGHT);
	for (i = 0; i < sizeof(font_families) / sizeof(gchar *); i++) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(toolbar->fontname), font_families[i]);
	}

	combobox_fontname_selector = GTK_COMBO_BOX(toolbar->fontname);
	gtk_widget_show((GtkWidget *) combobox_fontname_selector);

	/*font size tool item */
	toolbar->fontsize = gtk_combo_box_new_text();
	gtk_widget_show(toolbar->fontsize);
	gtk_tool_item_set_expand(GTK_TOOL_ITEM(item_name), TRUE);
	item_size = gtk_tool_item_new();
	gtk_container_add((GtkContainer *) item_size, toolbar->fontsize);
	gtk_toolbar_insert(EditView_Toolbar, GTK_TOOL_ITEM(item_size), -1);
	gtk_widget_show_all((GtkWidget *) item_size);
	for (i = 0; i < sizeof(font_sizes) / sizeof(gchar *); i++) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(toolbar->fontsize), font_sizes[i]);
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(toolbar->fontsize), DEFAULT_FONT_POS);
	gtk_widget_show(toolbar->fontsize);
	gtk_widget_set_size_request(toolbar->fontsize, TOOLBAR_BUTTON_WIDTH + 30,
				    TOOLBAR_BUTTON_HEIGHT);
	gtk_widget_set_size_request((GtkWidget *) item_size,
				    TOOLBAR_BUTTON_WIDTH + 30, TOOLBAR_BUTTON_HEIGHT);
	g_signal_connect(G_OBJECT(toolbar->fontsize), "changed",
			 G_CALLBACK(on_format_button_click), editor_info);

	/*bold tool item */
	image_bold = gtk_image_new_from_icon_name("qgn_list_gene_bold", GTK_ICON_SIZE_BUTTON);
	toolbar->bold = (GtkWidget *) gtk_toggle_tool_button_new();
	gtk_toolbar_insert(EditView_Toolbar, GTK_TOOL_ITEM(toolbar->bold), -1);
	gtk_widget_show(toolbar->bold);
	gtk_tool_button_set_icon_widget((GtkToolButton *) toolbar->bold, image_bold);
	g_signal_connect(G_OBJECT(toolbar->bold), "toggled",
			 G_CALLBACK(on_format_button_click), editor_info);
	g_signal_connect(G_OBJECT(gtk_bin_get_child((GtkBin *) toolbar->bold)),
			 "insensitive_press", G_CALLBACK(format_dimming_note), editor_info);

	/*italic tool item */
	image_italic = gtk_image_new_from_icon_name("qgn_list_gene_italic", GTK_ICON_SIZE_BUTTON);
	toolbar->italic = (GtkWidget *) gtk_toggle_tool_button_new();
	gtk_widget_show(toolbar->italic);
	gtk_toolbar_insert(EditView_Toolbar, GTK_TOOL_ITEM(toolbar->italic), -1);
	gtk_tool_button_set_icon_widget((GtkToolButton *) toolbar->italic, image_italic);
	g_signal_connect(G_OBJECT(toolbar->italic), "toggled",
			 G_CALLBACK(on_format_button_click), editor_info);
	g_signal_connect(G_OBJECT(gtk_bin_get_child((GtkBin *) toolbar->italic)),
			 "insensitive_press", G_CALLBACK(format_dimming_note), editor_info);
	/*bgcolor tool item */
	toolbar->fgcolor = (GtkWidget *) gtk_tool_item_new();
	toolbar->color_btn = hildon_color_button_new();
	gtk_container_add(GTK_CONTAINER(toolbar->fgcolor), GTK_WIDGET(toolbar->color_btn));
	gtk_widget_show(GTK_WIDGET(toolbar->color_btn));
	gtk_toolbar_insert(EditView_Toolbar, GTK_TOOL_ITEM(toolbar->fgcolor), -1);
	gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->color_btn), FALSE);
	g_signal_connect(G_OBJECT(toolbar->color_btn), "clicked",
			 G_CALLBACK(on_format_button_click), editor_info);
	g_signal_connect(G_OBJECT(toolbar->color_btn), "insensitive_press",
			 G_CALLBACK(format_dimming_note), editor_info);

	/*bullents tool item */
	image_bullets = gtk_image_new_from_icon_name("qgn_toolb_messagin_bullets",
						     GTK_ICON_SIZE_BUTTON);
	toolbar->bullets = (GtkWidget *) gtk_toggle_tool_button_new();
	gtk_widget_show(toolbar->bullets);
	gtk_toolbar_insert(EditView_Toolbar, GTK_TOOL_ITEM(toolbar->bullets), -1);
	gtk_tool_button_set_icon_widget((GtkToolButton *) toolbar->bullets, image_bullets);
	g_signal_connect(G_OBJECT(toolbar->bullets), "clicked",
			 G_CALLBACK(on_format_button_click), editor_info);
	g_signal_connect(G_OBJECT(gtk_bin_get_child((GtkBin *) toolbar->bullets)),
			 "insensitive_press", G_CALLBACK(format_dimming_note), editor_info);

	gtk_widget_show_all(GTK_WIDGET(EditView_Toolbar));
	editor_info->editor_toolbar = (GtkWidget *) EditView_Toolbar;

	set_toolbar_states(editor_info);

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}


/**
 This function is used to launch the add to contacts dialog
 when select key is pressed  on the address field.
 @param address_field is the textview widget that registered the callback.
 @param event is the button press event .
 @param address_buffer is the GtkTextBuffer pointer to the address field.
 @return False to propagate the event.
*/
static gboolean
ui_open_address_book_on_select(GtkWidget * address_field,
			       GdkEventKey * event, GtkTextBuffer * address_buffer)
{

	if (event->keyval == GDK_Return) {
		ui_editor_openAddrBook_cb(address_field, NULL, address_buffer);
		return TRUE;
	}
	return FALSE;
}


/**
 This function is used to create the to,cc,bcc address fields
  the same function is used to create the to/cc/bcc fields.so the address pointers
  are passed.
  
 @param address_field is the GtkWidget for the address(from/to/cc)field.
 @param table is the container in which the address field packed.
 @param address_image is the image for the address book.
 @param editor_info is the pointer to the EditorInf structure of the current editor.
	 This paramter is not freed in this function.
 @param field is a enum specifing the diff address fields.
 
 @return void.
*/
static void
ui_editor_create_address_fields(GtkWidget ** address_field,
				GtkWidget * table, GtkWidget ** address_image,
				GtkWidget ** address_hbox, EditorInfo * editor_info, gint field)
{
	GtkWidget *image_event_box = NULL;
	GtkWidget *image_securityevent_box = NULL;

	GtkWidget *address_scrolledwindow = NULL;
	GtkTextBuffer *address_buffer = NULL;
	AddressFieldInfo *fieldinfo = NULL;
	GtkWidget *alignment = NULL;
	GtkWidget *hbox = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	if (field == TO) {
		editor_info->fieldinfo_to = g_new0(AddressFieldInfo, 1);
		fieldinfo = editor_info->fieldinfo_to;
	} else if (field == CC) {
		editor_info->fieldinfo_cc = g_new0(AddressFieldInfo, 1);
		fieldinfo = editor_info->fieldinfo_cc;
	} else if (field == BCC) {
		editor_info->fieldinfo_bcc = g_new0(AddressFieldInfo, 1);
		fieldinfo = editor_info->fieldinfo_bcc;
	}
	alignment = gtk_alignment_new(0, 0, 0, 0);
	gtk_widget_show(alignment);

	*address_hbox = gtk_hbox_new(FALSE, ZERO_SPACING);
	gtk_widget_show(*address_hbox);

	image_event_box = gtk_event_box_new();
	*address_image = gtk_image_new_from_icon_name("qgn_list_gene_contacts",
						      GTK_ICON_SIZE_LARGE_TOOLBAR);
	gtk_container_add(GTK_CONTAINER(image_event_box), *address_image);
	gtk_container_add(GTK_CONTAINER(alignment), GTK_WIDGET(image_event_box));
	gtk_widget_show_all(image_event_box);

	hbox = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
	gtk_widget_show_all(hbox);

	address_scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show(address_scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(address_scrolledwindow),
				       GTK_POLICY_NEVER, GTK_POLICY_NEVER);
	gtk_box_pack_start(GTK_BOX(*address_hbox), address_scrolledwindow, TRUE, TRUE,
			   ZERO_PADDING);
	gtk_scrolled_window_set_shadow_type((GtkScrolledWindow *)
					    address_scrolledwindow, GTK_SHADOW_ETCHED_IN);
	gtk_widget_show_all(*address_hbox);
	image_securityevent_box = gtk_event_box_new();
	gtk_box_pack_start(GTK_BOX(*address_hbox), image_securityevent_box, FALSE, FALSE,
			   ZERO_PADDING);

	*address_field = gtk_text_view_new();
	gtk_container_add(GTK_CONTAINER(address_scrolledwindow), *address_field);
	gtk_widget_set_size_request(*address_field, BODY_WIDTH, BODY_HEIGHT);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(*address_field), GTK_WRAP_WORD_CHAR);
	gtk_text_view_set_accepts_tab(GTK_TEXT_VIEW(*address_field), FALSE);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(*address_field), TRUE);
	address_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(*address_field));
	/*fix for bug# 19432--> autocap is set to false */
	/*setting the auto caps false */
	hildon_gtk_text_view_set_input_mode(GTK_TEXT_VIEW(*address_field),
					    HILDON_GTK_INPUT_MODE_FULL);


	fieldinfo->field_sw = address_scrolledwindow;
	fieldinfo->field_hbox = *address_hbox;
	fieldinfo->field = *address_field;

	g_signal_connect_after(G_OBJECT(address_buffer), "insert_text",
			       G_CALLBACK(resize_addr_field), (gpointer) fieldinfo);
	g_signal_connect_after(G_OBJECT(address_buffer), "delete_range",
			       G_CALLBACK(decrease_address_field), (gpointer) fieldinfo);

	g_signal_connect(G_OBJECT(image_event_box), "button_press_event",
			 G_CALLBACK(ui_editor_openAddrBook_cb), (gpointer) address_buffer);
	g_signal_connect(G_OBJECT(address_buffer), "insert-text",
			 G_CALLBACK(ui_editor_check_modified_on_insert),
			 &(editor_info->editorModified));
	g_signal_connect(G_OBJECT(address_buffer), "delete-range",
			 G_CALLBACK(ui_editor_check_modified_on_delete),
			 &(editor_info->editorModified));
	g_signal_connect_after(G_OBJECT(*address_field), "focus-in-event",
			       G_CALLBACK(ch_toolbar_insens), editor_info);
	g_signal_connect(G_OBJECT(*address_field), "button_release_event",
			 G_CALLBACK(ui_editor_select_emailid), NULL);
	g_signal_connect(G_OBJECT(*address_field), "key_press_event",
			 G_CALLBACK(ui_open_address_book_on_select), address_buffer);

	g_signal_connect_after(G_OBJECT(*address_field), "paste-clipboard",
			       G_CALLBACK(ui_msg_paste_cb), editor_info);

	if (field == TO) {
		editor_info->msgFields->to_lable_hbox = hbox;
		editor_info->msgFields->to_lable = gtk_label_new(_("mail_va_to"));
		gtk_misc_set_alignment(GTK_MISC(editor_info->msgFields->to_lable), X_RIGHT,
				       Y_BOTTOM);
		gtk_box_pack_start(GTK_BOX(hbox), editor_info->msgFields->to_lable, TRUE, TRUE,
				   ZERO_PADDING);
		gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, ZERO_PADDING);

		gtk_table_attach((GtkTable *) table, hbox, 0,
				 2, 0, 1, (GtkAttachOptions) (GTK_FILL),
				 (GtkAttachOptions) (GTK_FILL), 0, 0);
		gtk_table_attach_defaults((GtkTable *) table, *address_hbox, 2, 4, 0, 1);
		editor_info->msgFields->sign_image =
		    gtk_image_new_from_icon_name("qgn_viewer_messagin_signed",
						 GTK_ICON_SIZE_BUTTON);
		gtk_container_add(GTK_CONTAINER(image_securityevent_box),
				  editor_info->msgFields->sign_image);

	} else if (field == CC) {
		editor_info->msgFields->cc_lable_hbox = hbox;
		editor_info->msgFields->cc_lable = gtk_label_new(_("mail_va_cc"));
		gtk_misc_set_alignment(GTK_MISC(editor_info->msgFields->cc_lable), X_RIGHT,
				       Y_BOTTOM);
		gtk_box_pack_start(GTK_BOX(hbox), editor_info->msgFields->cc_lable, TRUE, TRUE,
				   ZERO_PADDING);
		gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, ZERO_PADDING);

		gtk_table_attach((GtkTable *) table, hbox, 0,
				 2, 1, 2, (GtkAttachOptions) (GTK_FILL),
				 (GtkAttachOptions) (GTK_FILL), 0, 0);
		gtk_table_attach_defaults((GtkTable *) table, *address_hbox, 2, 4, 1, 2);
		editor_info->msgFields->ccencrypt_image =
		    gtk_image_new_from_icon_name("qgn_viewer_messagin_encrypted",
						 GTK_ICON_SIZE_BUTTON);
		gtk_container_add(GTK_CONTAINER(image_securityevent_box),
				  editor_info->msgFields->ccencrypt_image);

		if (ui_get_cc_field_status() == TRUE) {
			g_signal_handlers_block_by_func(editor_info->menu->show_cc,
							ui_msg_showCc_cb, editor_info);
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
						       (editor_info->menu->show_cc), TRUE);
			show_cc_field(editor_info->msgFields, TRUE);
			g_signal_handlers_unblock_by_func(editor_info->menu->show_cc,
							  ui_msg_showCc_cb, editor_info);
		} else {
			g_signal_handlers_block_by_func(editor_info->menu->show_cc,
							ui_msg_showCc_cb, editor_info);
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
						       (editor_info->menu->show_cc), FALSE);
			show_cc_field(editor_info->msgFields, FALSE);
			g_signal_handlers_unblock_by_func(editor_info->menu->show_cc,
							  ui_msg_showCc_cb, editor_info);
		}
	} else if (field == BCC) {
		editor_info->msgFields->bcc_lable_hbox = hbox;
		editor_info->msgFields->bcc_lable = gtk_label_new(_("mail_va_hotfix1"));
		gtk_misc_set_alignment(GTK_MISC(editor_info->msgFields->bcc_lable), X_RIGHT,
				       Y_BOTTOM);
		gtk_box_pack_start(GTK_BOX(hbox), editor_info->msgFields->bcc_lable, TRUE, TRUE,
				   ZERO_PADDING);
		gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, ZERO_PADDING);
		gtk_table_attach((GtkTable *) table, hbox,
				 0, 2, 2, 3, (GtkAttachOptions) (GTK_FILL),
				 (GtkAttachOptions) (GTK_FILL), 0, 0);
		gtk_table_attach_defaults((GtkTable *) table, *address_hbox, 2, 4, 2, 3);
		editor_info->msgFields->bccencrypt_image =
		    gtk_image_new_from_icon_name("qgn_viewer_messagin_encrypted",
						 GTK_ICON_SIZE_BUTTON);
		gtk_container_add(GTK_CONTAINER(image_securityevent_box),
				  editor_info->msgFields->bccencrypt_image);

		if (ui_get_bcc_field_status() == TRUE) {
			g_signal_handlers_block_by_func(editor_info->menu->show_bcc,
							ui_msg_showBcc_cb, editor_info);
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
						       (editor_info->menu->show_bcc), TRUE);
			g_signal_handlers_unblock_by_func(editor_info->menu->show_bcc,
							  ui_msg_showBcc_cb, editor_info);
			show_bcc_field(editor_info->msgFields, TRUE);
		} else {
			g_signal_handlers_block_by_func(editor_info->menu->show_bcc,
							ui_msg_showBcc_cb, editor_info);
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
						       (editor_info->menu->show_bcc), FALSE);
			g_signal_handlers_unblock_by_func(editor_info->menu->show_bcc,
							  ui_msg_showBcc_cb, editor_info);
			show_bcc_field(editor_info->msgFields, FALSE);
		}
	}
	gtk_widget_set_size_request(*address_hbox, BODY_WIDTH, DEFAULT_ADDR_FIELD_SZ);

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This function creates the editor subject field  and connects the various signals
 	associated with  the field
 
 @param table is the container in which the subject field packed.
 @param editor_info is the pointer to the EditorInfo structure of the current editor.
	 This paramter is not freed in this function.
	 
 @return void.
*/
static void
ui_editor_create_subject_field(EditorInfo * editor_info, GtkWidget * table,
			       GtkSizeGroup * size_group)
{
	MsgFields *msgFields = NULL;
	GtkWidget *sub_label = NULL;
	GtkWidget *hbox = NULL;
	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);

	CHECK_PARAM(editor_info == NULL, "input param check failed", return);
	CHECK_PARAM(editor_info->msgFields == NULL, "input param check failed", return);

	hbox = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
	msgFields = editor_info->msgFields;
	msgFields->priority_image =
	    gtk_image_new_from_icon_name("qgn_list_messaging_high", GTK_ICON_SIZE_BUTTON);

	sub_label = gtk_label_new(_("mail_va_subject"));
	gtk_box_pack_start(GTK_BOX(hbox), msgFields->priority_image, FALSE, FALSE, ZERO_PADDING);
	gtk_box_pack_start(GTK_BOX(hbox), sub_label, TRUE, TRUE, ZERO_PADDING);
	gtk_widget_show_all(hbox);
	gtk_size_group_add_widget(size_group, hbox);
	gtk_misc_set_alignment(GTK_MISC(sub_label), X_RIGHT, Y_BOTTOM);
	gtk_table_attach((GtkTable *) table, hbox, 0, 2, 3, 4, (GtkAttachOptions) (GTK_FILL), 0, 0,
			 0);
	msgFields->subject = gtk_entry_new();
	gtk_entry_set_width_chars(GTK_ENTRY(msgFields->subject), SUBJECT_WIDTH_CHARS);
	gtk_entry_set_max_length(GTK_ENTRY(msgFields->subject), SUBJECT_MAX_WIDTH);
	gtk_table_attach_defaults((GtkTable *) table, msgFields->subject, 2, 3, 3, 4);

	msgFields->encrypt_image =
	    gtk_image_new_from_icon_name("qgn_viewer_messagin_encrypted.png", GTK_ICON_SIZE_BUTTON);
	gtk_table_attach((GtkTable *) table, msgFields->encrypt_image, 3, 4, 3, 4,
			 (GtkAttachOptions) (GTK_FILL), 0, 0, 0);
	gtk_widget_show(msgFields->subject);

	g_signal_connect_after(G_OBJECT(msgFields->subject), "insert-text",
			       G_CALLBACK(ui_editor_subject_insert_cb),
			       &(editor_info->editorModified));
	g_signal_connect(G_OBJECT(msgFields->subject), "delete-text",
			 G_CALLBACK(ui_editor_subjectChanged_cb), &(editor_info->editorModified));
	g_signal_connect(G_OBJECT(msgFields->subject), "changed",
			 G_CALLBACK(ui_editor_change_title), editor_info->editor_appview);
	g_signal_connect_after(G_OBJECT(msgFields->subject), "focus-in-event",
			       G_CALLBACK(ch_toolbar_insens), editor_info);
	g_signal_connect(G_OBJECT(msgFields->subject), "key-press-event",
			 G_CALLBACK(ui_navigate_to_bcc), msgFields->bcc);
	g_signal_connect(G_OBJECT(msgFields->subject), "key-press-event",
			 G_CALLBACK(ui_navigate_to_cc), msgFields->cc);
	g_signal_connect(G_OBJECT(msgFields->subject), "key-press-event",
			 G_CALLBACK(ui_navigate_to_prev), msgFields->to);
	g_signal_connect_after(G_OBJECT(msgFields->subject), "paste-clipboard",
			       G_CALLBACK(ui_msg_paste_cb), editor_info);


	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 This fuction is used to create the body & associate the necessary signals to it.
 
 @param editor_info is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
 @param vbox_editor is the container in which the body widget is packed.
 @param scrolledwindow_body is the scrolledwindow  in which the body widget is packed.
 
 @return void.
*/
static void
ui_create_body_field(EditorInfo * editor_info, GtkWidget * vbox_editor,
		     GtkWidget * scrolledwindow_body)
{
	GtkWidget *frame = NULL;
	GtkTextBuffer *buffer_body = NULL;
	MsgFields *msgFields = NULL;
	GtkTextIter start, end;
	GtkStyle *style = NULL;
	GtkAdjustment *vadj = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);
	msgFields = editor_info->msgFields;

	frame = gtk_frame_new(NULL);
	msgFields->body = (GtkWidget *) gtk_text_view_new();
	gtk_text_view_set_editable((GtkTextView *) msgFields->body, TRUE);
	gtk_widget_set_size_request(msgFields->body, BODY_WIDTH, BODY_HEIGHT);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(msgFields->body), GTK_WRAP_WORD_CHAR);
	msgFields->scroll_area = hildon_scroll_area_new(scrolledwindow_body, msgFields->body);
	gtk_container_add((GtkContainer *) frame, msgFields->scroll_area);
	vadj = gtk_scrolled_window_get_vadjustment((GtkScrolledWindow *)
						   scrolledwindow_body);
	gtk_container_set_focus_vadjustment(GTK_CONTAINER(msgFields->scroll_area), vadj);
	gtk_box_pack_start(GTK_BOX(vbox_editor), frame, TRUE, TRUE, ZERO_PADDING);

	buffer_body = gtk_text_view_get_buffer(GTK_TEXT_VIEW(msgFields->body));
	gtk_widget_show_all(frame);
	gtk_text_buffer_get_bounds(buffer_body, &start, &end);

	g_signal_connect(G_OBJECT(buffer_body), "delete-range",
			 G_CALLBACK(ui_editor_check_modified_on_delete),
			 &(editor_info->editorModified));
	g_signal_connect(G_OBJECT(buffer_body), "insert-text",
			 G_CALLBACK(ui_editor_check_modified_on_insert),
			 &(editor_info->editorModified));

	g_signal_connect(G_OBJECT(buffer_body), "apply-tag",
			 G_CALLBACK(updatemodified), &(editor_info->editorModified));
	g_signal_connect(G_OBJECT(buffer_body), "remove-tag",
			 G_CALLBACK(updatemodified), &(editor_info->editorModified));

	g_signal_connect_after((gpointer) buffer_body, "apply-tag",
			       G_CALLBACK(on_text_buffer_apply_tag), editor_info);

	g_signal_connect_after((gpointer) buffer_body, "remove-tag",
			       G_CALLBACK(on_text_buffer_remove_tag), editor_info);
	//g_signal_connect ((gpointer) buffer_body, "delete-range",
	//                  G_CALLBACK (bullet_num_deletion), editor_info);

	g_signal_connect_after((gpointer) buffer_body, "delete-range",
			       G_CALLBACK(on_text_buffer_delete_range), editor_info);
	g_signal_connect_after((gpointer) buffer_body, "delete-range",
			       G_CALLBACK(on_body_after_text_delete), editor_info);

	g_signal_connect_after((gpointer) buffer_body, "insert-text",
			       G_CALLBACK(on_text_buffer_insert_text), editor_info);


	g_signal_connect_after(G_OBJECT(msgFields->body), "focus-in-event",
			       G_CALLBACK(ch_toolbar), editor_info);
	g_signal_connect(G_OBJECT(msgFields->body), "button-release-event",
			 G_CALLBACK(on_body_button_release_event), editor_info);
	g_signal_connect(G_OBJECT(msgFields->body), "key-release-event",
			 G_CALLBACK(on_body_key_release_event), editor_info);

	g_signal_connect(G_OBJECT(msgFields->body), "key-press-event",
			 G_CALLBACK(ui_navigate_to_prev), msgFields->subject);


	gtk_text_buffer_get_bounds(buffer_body, &start, &end);
	gtk_text_buffer_place_cursor(buffer_body, &start);
	/*back ground color initialized */
	style = gtk_widget_get_style(msgFields->body);
	editor_info->menu->bgcolorvalue = style->white;
	editor_info->body_style = style;
	/*events_blocked boolean for button-release/ key-release initialized */
	editor_info->events_blocked = FALSE;
	/*preselection of formattings boolean initialized */
	editor_info->formating_preselected = TRUE;
	editor_info->ignore_underline = TRUE;
	editor_info->ignore_strikethrough = TRUE;

	g_signal_connect(G_OBJECT(buffer_body), "insert-text",
			 G_CALLBACK(on_body_before_text_insert), editor_info);
	g_signal_connect_after(G_OBJECT(buffer_body), "insert-text",
			       G_CALLBACK(on_body_after_text_insert), editor_info);
	g_signal_connect(G_OBJECT(buffer_body), "insert-text", G_CALLBACK(undoins), editor_info);

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;

}

/**
 this function is used to create the editor layout
  the  to,cc,bcc subject,attachment,body fields are created.
  
 @param editor_info is the editorInfo structure pointer.
 	This paramter is not freed in this function.
 @return void.
*/
static void ui_editor_create_layout(EditorInfo * editor_info)
{
	GtkWidget *vbox_editor = NULL;
	MsgFields *msgFields = NULL;
	GtkWidget *table = NULL;
	GtkAdjustment *vadj = NULL;
	GtkSizeGroup *size_group = NULL;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);
	CHECK_PARAM(editor_info->msgFields == NULL, "Input param failed\n", return);

	msgFields = editor_info->msgFields;
/*scrolled window*/
	msgFields->scrollwindow_body = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_show_all(msgFields->scrollwindow_body);
	gtk_container_add(GTK_CONTAINER(editor_info->editor_appview), msgFields->scrollwindow_body);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				       (msgFields->scrollwindow_body),
				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_widget_set_size_request(msgFields->scrollwindow_body, BODY_WIDTH, BODY_HEIGHT);
	gtk_container_set_border_width(GTK_CONTAINER(msgFields->scrollwindow_body),
				       HILDON_MARGIN_DEFAULT);

	/*vbox */
	vbox_editor = gtk_vbox_new(FALSE, HILDON_MARGIN_DOUBLE);
	gtk_widget_show_all(vbox_editor);
	gtk_scrolled_window_add_with_viewport((GtkScrolledWindow *)
					      msgFields->scrollwindow_body, vbox_editor);
	gtk_viewport_set_shadow_type((GtkViewport *)
				     ((GtkBin *) (msgFields->scrollwindow_body))->
				     child, GTK_SHADOW_NONE);
	vadj =
	    gtk_scrolled_window_get_vadjustment((GtkScrolledWindow *) msgFields->scrollwindow_body);
	gtk_container_set_focus_vadjustment(GTK_CONTAINER(msgFields->scrollwindow_body), vadj);

	/*All the header fields are inserted in a table of 5x4-to,cc,bcc,subject
	   & attachments   are the 5 rows */
	table = gtk_table_new(MAX_ROWS, MAX_COLS, FALSE);
	gtk_table_set_col_spacing((GtkTable *) table, 1, HILDON_MARGIN_DEFAULT);

	gtk_widget_show(table);
	gtk_box_pack_start(GTK_BOX(vbox_editor), table, FALSE, FALSE, ZERO_PADDING);
	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	/*Creating the To, address field  & their corressponding signals are connected */
	ui_editor_create_address_fields(&(msgFields->to), table,
					&(msgFields->to_addrimage),
					&(msgFields->to_hbox), editor_info, TO);
	gtk_size_group_add_widget(size_group, msgFields->to_lable_hbox);
	/*Creating the cc, address field  & their corressponding signals are connected */
	ui_editor_create_address_fields(&(msgFields->cc), table,
					&(msgFields->cc_addrimage),
					&(msgFields->cc_hbox), editor_info, CC);
	gtk_size_group_add_widget(size_group, msgFields->cc_lable_hbox);

	/*Creating the bcc, address field  & their corressponding signals are connected */
	ui_editor_create_address_fields(&(msgFields->bcc), table,
					&(msgFields->bcc_addrimage),
					&(msgFields->bcc_hbox), editor_info, BCC);
	gtk_size_group_add_widget(size_group, msgFields->bcc_lable_hbox);

	/*Creating the Subject field */
	ui_editor_create_subject_field(editor_info, table, size_group);
	gtk_widget_hide(msgFields->priority_image);

	g_signal_connect(G_OBJECT(msgFields->to), "key-press-event",
			 G_CALLBACK(navigate_to_cc), msgFields->cc);
	g_signal_connect(G_OBJECT(msgFields->to), "key-press-event",
			 G_CALLBACK(navigate_to_bcc), msgFields->bcc);
	g_signal_connect(G_OBJECT(msgFields->to), "key-press-event",
			 G_CALLBACK(ui_navigate_to_next), msgFields->subject);

	g_signal_connect(G_OBJECT(msgFields->cc), "key-press-event",
			 G_CALLBACK(ui_navigate_to_prev), msgFields->to);
	g_signal_connect(G_OBJECT(msgFields->cc), "key-press-event",
			 G_CALLBACK(ui_navigate_to_bcc), msgFields->bcc);
	g_signal_connect(G_OBJECT(msgFields->cc), "key-press-event",
			 G_CALLBACK(ui_navigate_to_next), msgFields->subject);

	g_signal_connect(G_OBJECT(msgFields->bcc), "key-press-event",
			 G_CALLBACK(ui_navigate_to_next), msgFields->subject);
	g_signal_connect(G_OBJECT(msgFields->bcc), "key-press-event",
			 G_CALLBACK(ui_navigate_to_cc), msgFields->cc);
	g_signal_connect(G_OBJECT(msgFields->bcc), "key-press-event",
			 G_CALLBACK(ui_navigate_to_prev), msgFields->to);
	/*create the attachment field */
	ui_create_attachment_list_view(msgFields, table, FALSE);

	g_signal_connect(G_OBJECT
			 (gtk_tree_view_get_selection
			  ((GtkTreeView *) msgFields->view)), "changed",
			 G_CALLBACK(ui_attachment_selection), editor_info);
	g_signal_connect(G_OBJECT
			 ((GtkTreeView *) msgFields->view), "key-press-event",
			 G_CALLBACK(ui_key_press_on_attachment), editor_info);
	g_signal_connect_after(G_OBJECT(msgFields->view), "focus-in-event",
			       G_CALLBACK(ch_toolbar_insens), editor_info);

  /**Create the body field & all the signal handlers associated with it*/
	ui_create_body_field(editor_info, vbox_editor, msgFields->scrollwindow_body);
	editor_info->editorModified = FALSE;
	g_signal_connect(G_OBJECT(msgFields->subject), "key-press-event",
			 G_CALLBACK(ui_navigate_to_next), msgFields->body);

	if (ui_get_cc_field_status() == FALSE) {
		g_signal_handlers_block_by_func(editor_info->menu->show_cc, ui_msg_showCc_cb,
						editor_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(editor_info->menu->show_cc),
					       FALSE);
		show_cc_field(msgFields, FALSE);
		g_signal_handlers_unblock_by_func(editor_info->menu->show_cc,
						  ui_msg_showCc_cb, editor_info);
	}
	if (ui_get_bcc_field_status() == FALSE) {
		g_signal_handlers_block_by_func(editor_info->menu->show_bcc,
						ui_msg_showBcc_cb, editor_info);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(editor_info->menu->show_bcc),
					       FALSE);
		show_bcc_field(msgFields, FALSE);
		g_signal_handlers_unblock_by_func(editor_info->menu->show_bcc,
						  ui_msg_showBcc_cb, editor_info);
	}
	show_attachment_field(msgFields, FALSE);
	gtk_widget_hide(msgFields->priority_image);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}

/**
 this function sets the body contents such as the signature & the body
 for the a mail other then newmail
 
 @param editor_info is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
 @param headerInfo is a  pointer to the HeaderInfo structure.
	 This paramter is not freed in this function.
 @param body is the gchar representing the body.
	 This paramter is not freed in this function.
	 
 @return True-successful memory allocation & initilaisation.
	 False-else.
*/
static gboolean
ui_editor_set_body_content(EditorInfo * editor_info,
			   const HeaderInfo * headerInfo, const gchar * body)
{
	gint mode = NEW_MAIL;
	MsgFields *msgFields = NULL;
	gchar *signature = NULL;
	GtkTextBuffer *body_buffer = NULL;
	gboolean mem_allocated = TRUE;

	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return FALSE);
	CHECK_PARAM(editor_info->msgFields == NULL, "Input param failed\n", return FALSE);

	msgFields = editor_info->msgFields;
	mode = editor_info->composeType;

	body_buffer = gtk_text_view_get_buffer((GtkTextView *) msgFields->body);
	if (email_main_view->account_manager->default_account != NULL &&
	    editor_info->accountSettings != NULL) {
		signature =
		    ui_editor_check_for_siganuters(editor_info->accountSettings->
						   user_settings.account_name);
		ui_editor_embed_siganture(signature, mode, body_buffer);
	}
	g_free(signature);

	if (headerInfo != NULL) {
		ui_editor_insert_orgstring(editor_info, body_buffer);
	}

	/*handling opening of mail in all replying modes both plain & html
	   & also opening plain mails while forwarding */
	ui_editor_in_reply_mode(editor_info, headerInfo, body);

	/*hadling opening of mail in FORWARDing(only html) both from folder & 
	   viewer /DRAFT/OUTBOX plain &html */
	mem_allocated = ui_editor_from_folder(editor_info, headerInfo, body);
	if (mem_allocated == FALSE) {
		return mem_allocated;
	}
	gtk_text_buffer_set_modified(body_buffer, FALSE);
	focus_to_start(body_buffer, msgFields->body);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return mem_allocated;
}


/**
 This function opens the editor only with the header info.
    if original_in_reply is false or download attachments is false. 
 @param editor_info is the EditorIfo structure pointer.
 @param  msgHeader is the HeaderInfo structure pointer.
 @param buffer_body is teh GtkTextBuffer of the body field.
 
 @return TRUE-if editor is opened.
         FALSE-else.
*/
static gboolean
editor_with_headersonly(EditorInfo * editor_info, HeaderInfo * msgHeader,
			GtkTextBuffer * buffer_body)
{
	gint composeMode = NEW_MAIL;
	gchar *signature = NULL;

	composeMode = editor_info->composeType;

	if (((composeMode == REPLY_FROM_FOLDER || composeMode == REPLY_ALL_FROM_FOLDER
	      || composeMode == REPLY || composeMode == REPLY_ALL)
	     && msgHeader->original_in_reply == FALSE)) {
		if (email_main_view->account_manager->default_account != NULL &&
		    editor_info->accountSettings != NULL) {
			signature =
			    ui_editor_check_for_siganuters(editor_info->accountSettings->
							   user_settings.account_name);
		}
		if (signature != NULL) {
			ui_editor_embed_siganture(signature, composeMode, buffer_body);
		}
		editor_info->headersonly = TRUE;
		focus_to_start(buffer_body, editor_info->msgFields->body);
		ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
		return TRUE;
	}
	return FALSE;
}

/**
 this function is used to set the editor contents i.e to,cc,bcc,subject..etc
 if it is not a new mail
 
 @param editor_info is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
 @param headerInfo is a  pointer to the HeaderInfo structure.
	 This paramter is not freed in this function.
 @param body is the gchar representing the body.
	 This paramter is not freed in this function.
 @return True-successful memory allocation & initilaisation.
	 False-if memory allocation fails.
*/
static gboolean
ui_editor_set_contents(EditorInfo * editor_info, HeaderInfo * msgHeader, const gchar * body)
{
	GtkTextBuffer *buffer_to = NULL;
	GtkTextBuffer *buffer_cc = NULL;
	GtkTextBuffer *buffer_bcc = NULL;
	GtkTextBuffer *buffer_body = NULL;
	MsgFields *msgFields = NULL;
	gint composeMode = NEW_MAIL;
	gchar *from = NULL;
	gchar *replyto = NULL;
	gchar *to = NULL;
	gchar *cc = NULL;
	gchar *bcc = NULL;
	gchar *subjectString = NULL;
	gchar *subject = NULL;
	gboolean mem_allocated = TRUE;
	GSList *attachmentList = NULL;


	ULOG_DEBUG("[%s]:[%d] : starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Param NULL ", return TRUE);
	CHECK_PARAM(editor_info->msgFields == NULL, "Param NULL ", return TRUE);

	msgFields = editor_info->msgFields;
	composeMode = editor_info->composeType;
	buffer_to = gtk_text_view_get_buffer((GtkTextView *) msgFields->to);
	buffer_cc = gtk_text_view_get_buffer((GtkTextView *) msgFields->cc);
	buffer_bcc = gtk_text_view_get_buffer((GtkTextView *) msgFields->bcc);
	buffer_body = gtk_text_view_get_buffer((GtkTextView *) msgFields->body);

	if (composeMode == SEND_AS) {
		if (msgHeader != NULL && msgHeader->msgInfo != NULL
		    && msgHeader->msgInfo->header != NULL) {
			from = msgHeader->msgInfo->header->from;
			to = msgHeader->msgInfo->header->to;
			cc = msgHeader->msgInfo->header->cc;
			bcc = msgHeader->msgInfo->header->bcc;
			replyto = msgHeader->msgInfo->header->replyto;
			if (msgHeader->msgInfo->header->subject != NULL) {
				subjectString = g_strdup(msgHeader->msgInfo->header->subject);
			}
		}
	} else if (msgHeader != NULL) {
		from = msgHeader->from;
		replyto = msgHeader->reply_to;
		to = msgHeader->to;
		cc = msgHeader->cc;
		bcc = msgHeader->bcc;
		subject = msgHeader->subject;
	}
	if (editor_info->accountSettings != NULL) {
		if (g_ascii_strcasecmp(editor_info->msgInfo->settings->account,
				       editor_info->accountSettings->user_settings.account_name) ==
		    0) {
			ui_editor_fill_to_address(composeMode, buffer_to, from, to, replyto,
						  editor_info->accountSettings->user_settings.
						  email);
			ui_editor_fill_cc_address(composeMode, buffer_cc, from, cc, editor_info,
						  editor_info->accountSettings->user_settings.
						  email);
		} else {
			ui_editor_fill_to_address(composeMode, buffer_to, from, to, replyto, NULL);
			ui_editor_fill_cc_address(composeMode, buffer_cc, from, cc, editor_info,
						  NULL);
		}

	} else {
		ui_editor_fill_to_address(composeMode, buffer_to, from, to, replyto, DEFAULT);
		ui_editor_fill_cc_address(composeMode, buffer_cc, from, cc, editor_info, DEFAULT);
	}
	ui_editor_fill_bcc_address(composeMode, buffer_bcc, bcc, editor_info);
	/*sets the cursor back to the start position */
	focus_to_start(buffer_to, msgFields->to);
	focus_to_start(buffer_cc, msgFields->cc);
	focus_to_start(buffer_bcc, msgFields->bcc);
	if (msgHeader != NULL && (composeMode == NEW_MAIL ||
				  composeMode == DRAFT_MAIL ||
				  composeMode == OUTBOX_MAIL || composeMode == SEND_AS)) {
		ui_set_priority_icon(msgHeader->priority, (GtkImage *) msgFields->priority_image);
	}
	/*Checking for additional "Re:" is the subject field & removing them.
	 * It will return a newly allocated subject string */
	ui_editor_subject_in_reply_mode(composeMode, subject, &subjectString);

	/*Checking for additional "Fw:" is the subject field & removing them.
	 * It will return a newly allocated subject string */
	ui_editor_subject_in_forward_mode(composeMode, subject, &subjectString);

	/*This function is used to set the subject of the editor */
	ui_editor_set_subject(composeMode, msgHeader, msgFields->subject,
			      subjectString, editor_info->editor_appview);

	/*To include only headers when include original in reply is false
	   & forwarding a partially downloaded mail */
	if (editor_with_headersonly(editor_info, msgHeader, buffer_body) == TRUE) {
		return TRUE;
	}
	if (!ui_editor_get_mimeinfo(editor_info, msgHeader) && !msgHeader->email_attachment) {
		return FALSE;
	}

	mem_allocated = ui_editor_check_attachments(editor_info, msgHeader, &attachmentList);
	if (mem_allocated == FALSE) {
		return mem_allocated;
	}

	mem_allocated = ui_editor_set_body_content(editor_info, msgHeader, body);
	if (mem_allocated == TRUE) {
		ui_editor_insert_attachments_in_reply(editor_info, attachmentList);
		g_slist_foreach(attachmentList, (GFunc) g_free, NULL);
		g_slist_free(attachmentList);
	}
	if (msgHeader != NULL) {
		g_free(msgHeader->subject);
		msgHeader->subject = g_strdup(subjectString);
	}
	gtk_widget_set_sensitive(editor_info->menu->undo, FALSE);
	g_free(subjectString);
	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return mem_allocated;

}

/**
 This function is used to activate the toolbar buttons based on HTML/Plain mail.
 
 @param editor_info is the EditorInfo structure pointer.
	 This paramter is not freed in this function.
 
 @return void.
*/
static void ui_editor_activate_toolbuttons(EditorInfo * editor_info)
{
	gboolean html_mail = FALSE;
	gint mode = NEW_MAIL;
	EditorMenu *menu = NULL;
	EditorToolbar *toolbar = NULL;

	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);
	CHECK_PARAM(editor_info->toolbar == NULL, "Input param failed\n", return);
	CHECK_PARAM(editor_info->menu == NULL, "Input param failed\n", return);
	CHECK_PARAM(editor_info->accountSettings == NULL, "Input param failed\n", return);

	mode = editor_info->composeType;
	toolbar = editor_info->toolbar;
	menu = editor_info->menu;

	if ((mode == DRAFT_MAIL || mode == OUTBOX_MAIL) && editor_info->emailType == HTML_EDITOR) {
		editor_info->msgInfo->settings->type = EMAIL_HTML;
		html_mail = TRUE;
	} else if ((mode == DRAFT_MAIL || mode == OUTBOX_MAIL)
		   && editor_info->emailType == PLAIN_EDITOR) {
		editor_info->msgInfo->settings->type = EMAIL_TEXT;
	}
	if ((editor_info->accountSettings->advanced_settings.email_type == EMAIL_TEXT
	     && html_mail == FALSE)) {
		if (toolbar != NULL) {
			gtk_widget_set_sensitive(toolbar->bold, FALSE);
			gtk_widget_set_sensitive(toolbar->italic, FALSE);
			gtk_widget_set_sensitive(toolbar->fontname, FALSE);
			gtk_widget_set_sensitive(toolbar->fontsize, FALSE);
			gtk_widget_set_sensitive(toolbar->fontsize, FALSE);
			gtk_widget_set_sensitive(toolbar->fgcolor, FALSE);
			gtk_widget_set_sensitive(toolbar->bullets, FALSE);
		}
		gtk_widget_set_sensitive(menu->format_menuitem, FALSE);
		editor_info->emailType = PLAIN_EDITOR;
		editor_info->msgInfo->settings->type = EMAIL_TEXT;
	} else if (gtk_widget_is_focus(editor_info->msgFields->body) == FALSE) {
		if ((ui_get_editor_toolbar_hidden_status() == FALSE &&
		     email_main_view->tags->full_screen_mode == FALSE) ||
		    (ui_get_editor_fullscreen_toolbar_hidden_status() == FALSE &&
		     email_main_view->tags->full_screen_mode == TRUE)) {
			gtk_widget_set_sensitive(toolbar->bold, FALSE);
			gtk_widget_set_sensitive(toolbar->italic, FALSE);
			gtk_widget_set_sensitive(toolbar->fontname, FALSE);
			gtk_widget_set_sensitive(toolbar->fontsize, FALSE);
			gtk_widget_set_sensitive(toolbar->fontsize, FALSE);
			gtk_widget_set_sensitive(toolbar->fgcolor, FALSE);
			gtk_widget_set_sensitive(toolbar->bullets, FALSE);
		}
		gtk_widget_set_sensitive(menu->format_menuitem, FALSE);
		gtk_widget_set_sensitive(menu->attach_image, FALSE);

	}
	return;

}

/**
 This function is used to update the editor info into the global open windows
  structure.
  
 @param editor_info is the EditorInfo structure pointer.
 	This paramter is not freed in this function.
 @param headerInfo is a  pointer to the HeaderInfo structure.
	 This paramter is not freed in this function.
 
 @return void.
*/
static void ui_editor_register_in_openwindows(EditorInfo * editor_info, HeaderInfo * header_info)
{
	gint mode = NEW_MAIL;
	OpenedWindowData *open_window = NULL;

	ULOG_DEBUG("[%s]:[%d] :starts here\n", __FUNCTION__, __LINE__);
	CHECK_PARAM(editor_info == NULL, "Input param failed\n", return);
	mode = editor_info->composeType;

	/*Store window data in global list */
	open_window = g_new0(OpenedWindowData, 1);
	/*Set the message as READ if this is not a new message */
	if (mode != NEW_MAIL) {
		if (editor_info->headersonly == FALSE && header_info->email_attachment == FALSE) {
			ui_update_row_tag_in_folder(header_info->folderNum,
						    header_info->msgid, READ_STATUS, FALSE);
		}
	}
	if ((mode == DRAFT_MAIL || mode == OUTBOX_MAIL) && header_info != NULL &&
	    header_info->email_attachment == FALSE) {
		ui_update_row_tag_in_folder(header_info->folderNum, header_info->msgid,
					    IN_USE_STATUS, TRUE);
	}
	open_window->msgid = NULL;
	open_window->oldmsgid = NULL;
	/*removed GSting usage,fixed leak */
	if (mode != DRAFT_MAIL && mode != OUTBOX_MAIL) {
		open_window->msgid = NULL;
		open_window->oldmsgid = NULL;
	} else if (header_info != NULL) {
		open_window->msgid = g_strdup(header_info->msgid);
		open_window->oldmsgid = g_strdup(header_info->msgid);
	}
	open_window->msgNum = email_main_view->tags->window_count;
	editor_info->msgnum = open_window->msgNum;
	open_window->window_ptr = (gpointer) editor_info->editor_appview;
	open_window->iseditor = TRUE;
	open_window->zoom_percentage = editor_info->zoominfo->previous_level;
	email_main_view->open_windows =
	    g_slist_insert(email_main_view->open_windows, open_window, 0);

	/*Update Task Navigator */
	ui_editor_update_tasknavigator(open_window, header_info, editor_info);

	ULOG_DEBUG("[%s]:[%d] : ends here\n", __FUNCTION__, __LINE__);
	return;
}


/**
 This function is used to highlight the invalid email address bcause of which the send
 operation failed.

 @param address_buffers is the list of GTKTextBUffer pointers of the to/cc/bcc fields.
 @param header is the HeaderInfo structure pointer.

 @return void.
*/
static void highlight_invalid_address(const GSList * address_buffers, const HeaderInfo * header)
{
	GSList *buffer_list = NULL;
	gint i = 0;
	gchar *recepients = NULL, *field = NULL;
	gboolean status = FALSE;
	buffer_list = (GSList *) address_buffers;
	CHECK_PARAM(header == NULL, "input param failed", return);
	CHECK_PARAM(address_buffers == NULL, "input param failed", return);
	for (i = 0; i < 3; i++) {
		switch (i) {
		case 0:
			field = header->to;
			break;
		case 1:
			field = header->cc;
			break;
		case 2:
			field = header->bcc;
			break;
		}
		status = ui_editor_validate_addresses(field, &recepients, (GtkTextBuffer *)
						      g_slist_nth_data(buffer_list, i));
		if (status) {
			g_free(recepients);
			recepients = NULL;
			continue;
		} else
			return;
	}

	return;
}

/**
 This function is called higher layers to create the UI for the Edtior window
 for e-mail.

 @param headerInfo is a pointer to the _headerInfo structure.
 	This paramter is not freed in this function.
 @param body is a pointer to the buffer containing the body.
	 This paramter is not freed in this function.
 @param msgType is a enum containing the different ways of invoking the editor

 @return pointer to the window widget created.
*/

HildonAppView *ui_editor_create(HeaderInfo * msgHeader, const gchar * body, gint msgType)
{
	GtkWidget *editorview = NULL;
	gboolean mem_allocated = TRUE;
	EditorInfo *editor_info = NULL;
	EditorToolbar *toolbar = NULL;
	EditorMenu *menu = NULL;
	GtkTextBuffer *bufferTo = NULL;
	GtkTextBuffer *bufferCc = NULL;
	GtkTextBuffer *bufferBcc = NULL;
	GtkTextBuffer *buffer_body = NULL;

	ULOG_DEBUG("[%s][%s] : starts here :\n", __FILE__, __FUNCTION__);

	(email_main_view->tags->window_count)++;
	editor_info = ui_create_editor_info();
	ui_make_temp_dir();

       /**Creating the appview for the editor*/
	editorview = hildon_appview_new(NULL);

	/*Set full screen short cut key */
	hildon_appview_set_fullscreen_key_allowed((HildonAppView *) editorview, TRUE);
	gtk_widget_show_all(editorview);
	gtk_widget_set_size_request(editorview, BODY_WIDTH, -1);
	editor_info->editor_appview = editorview;

	mem_allocated = ui_editor_initialise_editorinfo(editor_info, msgHeader, msgType);
	if (mem_allocated == FALSE) {
		ULOG_CRIT_F("memory allocation failed\n");
		ui_free_editorinfo(editor_info);
		return NULL;
	}

	/*Creation of the menuitems for the editor */
	ui_editor_create_menubar(editor_info);


	/*Creation of the internal layout for editor */
	ui_editor_create_layout(editor_info);

	/*Create find toolbar */
	create_find_toolbar(editor_info->msgFields,
			    HILDON_APPVIEW(editorview)->vbox, editor_info->menu->find);

	/*Creation of the toolbar for the editor */
	ui_editor_create_toolbar(editor_info);

	/*Enabling or disabling of the toolbuttons based on the html/plain editor */
	ui_editor_activate_toolbuttons(editor_info);

	create_supported_tags((GtkTextView *) editor_info->msgFields->body);
	/*set the security icon */
	ui_editor_fill_smime_type(editor_info->msgInfo->settings,
				  editor_info->msgInfo->settings->smime_type,
				  editor_info->msgFields, editor_info->menu);

	/*Initialize zoominfo  */
	menu = editor_info->menu;
	editor_info->zoominfo =
	    set_initial_zoom_level((GtkTextView *) editor_info->msgFields->body,
				   NULL, editor_info->msgid, DEFAULT_ZOOM);
	set_zoominfo_menus(editor_info->zoominfo, menu);

	/*initialise the font attributes */
	on_body_button_release_event(NULL, NULL, editor_info);

	/*Set the contents of editor if it is not new mail */
	mem_allocated = ui_editor_set_contents(editor_info, msgHeader, body);
	if (mem_allocated == FALSE) {
		/*if editor opened from email attachment the mimeinfo should not be freed
		   The viewer frees the mimeinfo */
		if (msgHeader->mimeInfo)
			editor_info->mimeInfo = NULL;

		ui_free_editorinfo(editor_info);
		return NULL;
	}
	if (msgType == NEW_MAIL) {
		hildon_appview_set_title((HildonAppView *) editor_info->editor_appview,
					 _("mcen_me_editor_newemail"));
	}

	if (msgHeader != NULL) {
		update_fullydownloaded_flag(msgHeader, editor_info->mimeInfo);
	}

	/*Set the focus for the editor */
	hildon_app_set_appview((HildonApp *) app, (HildonAppView *) editorview);

	/*restore if state save exists */
	if (msgHeader != NULL && msgHeader->set_prev_settings == TRUE) {
		restore_editor_saved_state(editor_info);
	}

	toolbar = editor_info->toolbar;
	gtk_container_forall(GTK_CONTAINER(toolbar->fontname),
			     (GtkCallback) insensitive_click_cb, editor_info);
	gtk_container_forall(GTK_CONTAINER(toolbar->fontsize),
			     (GtkCallback) insensitive_click_cb, editor_info);

	/*initialise the font attributes */
	//on_body_button_release_event(NULL, NULL, editor_info);
	/*set the default font for editor */
	set_default_font(editor_info, editor_info->msgInfo->settings);

	/*set initial zoom */
	if (msgType != NEW_MAIL)
		set_inherited_zoom(msgType, editor_info->zoominfo, msgHeader->zoomlevel);

	ui_editor_set_focus(editor_info);

	bufferTo = gtk_text_view_get_buffer((GtkTextView *) editor_info->msgFields->to);
	bufferCc = gtk_text_view_get_buffer((GtkTextView *) editor_info->msgFields->cc);
	bufferBcc = gtk_text_view_get_buffer((GtkTextView *) editor_info->msgFields->bcc);
	buffer_body = gtk_text_view_get_buffer((GtkTextView *) editor_info->msgFields->body);

	editor_info->addressBuffers = g_slist_append(editor_info->addressBuffers, bufferTo);
	editor_info->addressBuffers = g_slist_append(editor_info->addressBuffers, bufferCc);
	editor_info->addressBuffers = g_slist_append(editor_info->addressBuffers, bufferBcc);


	if (msgType == REPLY || msgType == REPLY_ALL
	    || msgType == REPLY_FROM_FOLDER || msgType == REPLY_ALL_FROM_FOLDER) {
		editor_tag_reply_ids(editor_info->addressBuffers);
	}
	if (msgType == FAILED_MAIL) {
		highlight_invalid_address(editor_info->addressBuffers, msgHeader);
	}

	/*fontname signal registry done here to avoid the callback getting fired
	 * with set_default_font ()*/
	g_signal_connect(G_OBJECT(editor_info->toolbar->fontname),
			 "changed", G_CALLBACK(on_format_button_click), editor_info);
	g_signal_connect(G_OBJECT(editorview), "delete-event",
			 G_CALLBACK(ui_editor_close_cb), editor_info);
	g_signal_connect(G_OBJECT(editorview), "destroy-event",
			 G_CALLBACK(ui_editor_close_cb), editor_info);

	g_signal_connect(G_OBJECT(editor_info->menu->zoom_out_level), "activate",
			 G_CALLBACK(on_zoom_activate), editor_info->zoominfo);
	g_signal_connect(G_OBJECT(editor_info->menu->zoom_out_max_level),
			 "activate", G_CALLBACK(set_format_presel_tag), editor_info);

	g_signal_connect(G_OBJECT(editor_info->menu->zoom_out_max_level),
			 "activate", G_CALLBACK(on_zoom_activate), editor_info->zoominfo);
	g_signal_connect(G_OBJECT(editor_info->menu->zoom_out_max_level),
			 "activate", G_CALLBACK(set_format_presel_tag), editor_info);

	g_signal_connect(G_OBJECT(editor_info->menu->current_zoom_level),
			 "activate", G_CALLBACK(on_zoom_activate), editor_info->zoominfo);
	g_signal_connect(G_OBJECT(editor_info->menu->current_zoom_level),
			 "activate", G_CALLBACK(set_format_presel_tag), editor_info);

	g_signal_connect(G_OBJECT(editor_info->menu->zoom_in_level), "activate",
			 G_CALLBACK(on_zoom_activate), editor_info->zoominfo);
	g_signal_connect(G_OBJECT(editor_info->menu->zoom_in_level), "activate",
			 G_CALLBACK(set_format_presel_tag), editor_info);
#if 0
	g_signal_connect(G_OBJECT(editor_info->menu->zoom_80), "activate",
			 G_CALLBACK(on_zoom_activate), editor_info->zoominfo);
	g_signal_connect(G_OBJECT(editor_info->menu->zoom_80), "activate",
			 G_CALLBACK(set_format_presel_tag), editor_info);

	g_signal_connect(G_OBJECT(editor_info->menu->zoom_120), "activate",
			 G_CALLBACK(on_zoom_activate), editor_info->zoominfo);
	g_signal_connect(G_OBJECT(editor_info->menu->zoom_120), "activate",
			 G_CALLBACK(set_format_presel_tag), editor_info);

#endif
	g_signal_connect(G_OBJECT(editor_info->msgFields->to), "key-press-event",
			 G_CALLBACK(ui_check_recp_name), (editor_info->addressBuffers));
	g_signal_connect(G_OBJECT(editor_info->msgFields->to),
			 "key-press-event", G_CALLBACK(backspace_handling), NULL);
	g_signal_connect(G_OBJECT(editor_info->msgFields->to), "key-press-event",
			 G_CALLBACK(skip_resolved_emailid), NULL);
	g_signal_connect(G_OBJECT(editor_info->msgFields->to), "key-release-event",
			 G_CALLBACK(skip_emailid_on_key_release), NULL);
	g_signal_connect(G_OBJECT(editor_info->msgFields->cc), "key-press-event",
			 G_CALLBACK(ui_check_recp_name), (editor_info->addressBuffers));
	g_signal_connect(G_OBJECT(editor_info->msgFields->cc),
			 "key-press-event", G_CALLBACK(backspace_handling), NULL);
	g_signal_connect(G_OBJECT(editor_info->msgFields->cc), "key-press-event",
			 G_CALLBACK(skip_resolved_emailid), NULL);
	g_signal_connect(G_OBJECT(editor_info->msgFields->cc), "key-release-event",
			 G_CALLBACK(skip_emailid_on_key_release), NULL);
	g_signal_connect(G_OBJECT(editor_info->msgFields->bcc), "key-press-event",
			 G_CALLBACK(ui_check_recp_name), (editor_info->addressBuffers));
	g_signal_connect(G_OBJECT(editor_info->msgFields->bcc),
			 "key-press-event", G_CALLBACK(backspace_handling), NULL);
	g_signal_connect(G_OBJECT(editor_info->msgFields->bcc), "key-press-event",
			 G_CALLBACK(skip_resolved_emailid), NULL);
	g_signal_connect(G_OBJECT(editor_info->msgFields->bcc), "key-release-event",
			 G_CALLBACK(skip_emailid_on_key_release), NULL);
	g_signal_connect(G_OBJECT(menu->check_names), "activate",
			 G_CALLBACK(ui_editor_do_check_names), editor_info->addressBuffers);
	g_signal_connect(G_OBJECT(menu->check_names), "activate",
			 G_CALLBACK(recipient_field_empty_check), editor_info);
	g_signal_connect(G_OBJECT(menu->undo), "activate", G_CALLBACK(undo_do_undo), editor_info);
	g_signal_connect(G_OBJECT(menu->font), "activate",
			 G_CALLBACK(on_format_button_click), editor_info);

	/*creation of new signals */
	if (editor_signals_created == FALSE) {
		if (common_signals_created == FALSE) {
			g_signal_new("close", G_TYPE_FROM_INSTANCE(editorview),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE,
				     0, NULL, NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
			g_signal_new("toggle_fullscreen", G_TYPE_FROM_INSTANCE(editorview),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
			g_signal_new("increase_zoom", G_TYPE_FROM_INSTANCE(editorview),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
			g_signal_new("decrease_zoom", G_TYPE_FROM_INSTANCE(editorview),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
			g_signal_new("get_state_values", G_TYPE_FROM_INSTANCE(editorview),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
			g_signal_new("update_msginfo", G_TYPE_FROM_INSTANCE(editorview),
				     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE, 0, NULL,
				     NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
			/*To avoid registering same signals again & again removing runtime warnings */
			common_signals_created = TRUE;
		}
		g_signal_new("get_new_msgid", G_TYPE_FROM_INSTANCE(editorview),
			     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE,
			     0, NULL, NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
		g_signal_new("send", G_TYPE_FROM_INSTANCE(editorview),
			     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE,
			     0, NULL, NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
		g_signal_new("save_all", G_TYPE_FROM_INSTANCE(editorview),
			     G_SIGNAL_RUN_CLEANUP | G_SIGNAL_NO_RECURSE,
			     0, NULL, NULL, gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
		editor_signals_created = TRUE;
	}

	g_signal_connect(G_OBJECT(editorview), "close", G_CALLBACK(ui_editor_close_cb),
			 editor_info);

	g_signal_connect(G_OBJECT(editorview), "toggle_fullscreen",
			 G_CALLBACK(ui_editor_toggle_to_fullscreen), editor_info);

	g_signal_connect(G_OBJECT(editorview), "increase_zoom",
			 G_CALLBACK(increase_zoom_view), editor_info->zoominfo);

	g_signal_connect(G_OBJECT(editorview), "decrease_zoom",
			 G_CALLBACK(decrease_zoom_view), editor_info->zoominfo);

	g_signal_connect(G_OBJECT(editorview), "send", G_CALLBACK(ui_editor_send_msg_cb),
			 editor_info);

	g_signal_connect(G_OBJECT(editorview), "get_new_msgid",
			 G_CALLBACK(ui_editor_get_new_msgid_cb), editor_info);

	g_signal_connect(G_OBJECT(editorview), "get_state_values",
			 G_CALLBACK(ui_editor_get_state_values_cb), editor_info);

	g_signal_connect(G_OBJECT(editorview), "update_msginfo",
			 G_CALLBACK(ui_editor_msginfo_thr_cb), editor_info);

	/*g_signal_connect(G_OBJECT(buffer_body), "insert-text",
	   G_CALLBACK(on_body_before_text_insert), editor_info);
	   g_signal_connect_after(G_OBJECT(buffer_body), "insert-text",
	   G_CALLBACK(on_body_after_text_insert), editor_info); */
	g_signal_connect(G_OBJECT(editor_info->msgFields->body), "paste-clipboard",
			 G_CALLBACK(handle_paste_with_im), editor_info);
	g_signal_connect_after(G_OBJECT(editor_info->msgFields->body), "paste-clipboard",
			       G_CALLBACK(handle_paste_completion_with_im), editor_info);
	g_signal_connect(G_OBJECT(bufferTo), "insert-text", G_CALLBACK(undoins), editor_info);
	g_signal_connect(G_OBJECT(bufferCc), "insert-text", G_CALLBACK(undoins), editor_info);
	g_signal_connect(G_OBJECT(bufferBcc), "insert-text", G_CALLBACK(undoins), editor_info);
	g_signal_connect(G_OBJECT(bufferTo), "delete-range", G_CALLBACK(undodel), editor_info);
	g_signal_connect(G_OBJECT(bufferCc), "delete-range", G_CALLBACK(undodel), editor_info);
	g_signal_connect(G_OBJECT(bufferBcc), "delete-range", G_CALLBACK(undodel), editor_info);
	//g_signal_connect(G_OBJECT(buffer_body), "insert-text", G_CALLBACK(undoins), editor_info);
	g_signal_connect(G_OBJECT(buffer_body), "delete-range", G_CALLBACK(undodel), (editor_info));
	g_signal_connect((gpointer) buffer_body, "delete-range",
			 G_CALLBACK(bullet_num_deletion), editor_info);
	g_signal_connect(G_OBJECT(editor_info->msgFields->subject),
			 "insert-text", G_CALLBACK(undoinssub), editor_info);
	g_signal_connect(G_OBJECT(editor_info->msgFields->subject),
			 "delete-text", G_CALLBACK(undodelsub), editor_info);

	if (msgType != NEW_MAIL) {
		ui_dbus_autosave_mail(editor_info->msgid, 1);

		/*for editor opened from email attachment , the mimeinfo will
		 * be freed when the email viewer is closed.need not free hear*/
		if (msgHeader->mimeInfo != NULL) {
			if (msgType == REPLY || msgType == REPLY_ALL
			    || editor_info->emailType == PLAIN_EDITOR) {
				gnome_vfs_unlink(editor_info->mimeInfo->main->filename);
				g_free(editor_info->mimeInfo->main->filename);
				editor_info->mimeInfo->main->filename = NULL;
			}
			editor_info->mimeInfo = NULL;
		}

	}
	ui_editor_register_in_openwindows(editor_info, msgHeader);

	/*Connect switched_to signal for appview */
	g_signal_connect(G_OBJECT(editorview), "switched_to", G_CALLBACK(ui_view_topped_cb), NULL);

	if (msgType == OUTBOX_MAIL) {
		ui_dbus_editor_open_mail(editor_info->msgid, EDITOR_OPEN_MSG_ADD);
	}

	editor_info->editorModified = FALSE;
	if (msgType == SEND_AS)
		editor_info->editorModified = TRUE;

	/*set the fullscreen mode */
	hildon_appview_set_fullscreen((HildonAppView *) editorview,
				      email_main_view->tags->full_screen_mode);

	/*update the seen flag for a unread imap mail */
        if (msgHeader && msgHeader->folderNum == INBOX_MODEL_NUM  &&
                msgHeader->read_status && msgHeader->protocol == PROTO_IMAP4)
                ui_dbus_update_seen_flag(editor_info->msgid);

	ULOG_DEBUG("[%s][%s] : ends	 here :\n", __FILE__, __FUNCTION__);
	return ((HildonAppView *) editorview);
}
